@@ -40,6 +40,8 @@
 #include "gretl_xml.h"
 #include "gretl_panel.h"
 #include "usermat.h"
+#include "gretl_string_table.h"
+#include "dbread.h"
 
 #ifdef WIN32
 # include <windows.h>
@@ -75,7 +77,7 @@ int data_status;
 char linebak[MAXLINE];      /* for storing comments */
 char *line_read;
 
-static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo);
+static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo);
 static int push_input_file (FILE *fp);
 static FILE *pop_input_file (void);
 static int saved_object_action (const char *line, double ***pZ,
@@ -100,54 +102,37 @@ static void usage(void)
     exit(EXIT_SUCCESS);
 }
 
-#ifndef WIN32
-
-int make_userdir (void) 
+static int cli_workdir_callback (const char *s)
 {
-    const char *userdir = gretl_user_dir();
-    DIR *dir = NULL;
-    int err = 0;
-    
-    if ((dir = opendir(userdir)) == NULL) {
-	err = mkdir(userdir, 0755);
-	if (err) {
-	    fprintf(stderr, _("Couldn't create user directory %s\n"), 
-		    userdir);
-	} else {
-	    fprintf(stderr, _("Created user directory %s\n"), userdir);
-	}
-    } else {
-	closedir(dir);
-    }
-
-    return err;
+    return set_gretl_work_dir(s, &paths);
 }
 
-#endif /* WIN32 */
-
-void gretl_abort (char *line)
+static void gretl_abort (char *line)
 {
     fprintf(stderr, _("\ngretlcli: error executing script: halting\n"));
     fprintf(stderr, "> %s\n", line);
     exit(EXIT_FAILURE);
 }
 
-void noalloc (const char *str)
+static void noalloc (void)
 {
-    fprintf(stderr, _("Couldn't allocate memory for %s\n"), str);
+    fputs(_("Out of memory\n"), stderr);
     exit(EXIT_FAILURE);
 }
 
-void file_get_line (char *line, CMD *cmd)
+static int file_get_line (char *line, CMD *cmd)
 {
     clear(line, MAXLINE);
-    fgets(line, MAXLINE - 1, fb);
+    fgets(line, MAXLINE, fb);
 
     if (*line == '\0') {
 	strcpy(line, "quit");
+    } else if (line[strlen(line)-1] != '\n') {
+	return E_TOOLONG;
     } else {
 	*linebak = 0;
-	strncat(linebak, line, MAXLINE - 1);
+	strncat(linebak, line, MAXLINE-1);
+	tailstrip(linebak);
     }
 
     if (!strncmp(line, "noecho", 6)) {
@@ -158,11 +143,13 @@ void file_get_line (char *line, CMD *cmd)
 	printf("%s", line);
 	*linebak = 0;
     }
+
+    return 0;
 }
 
 #ifdef ENABLE_NLS
 
-void nls_init (void)
+static void nls_init (void)
 {
 # ifdef WIN32
     char gretldir[MAXLEN], LOCALEDIR[MAXLEN];
@@ -208,18 +195,15 @@ static void force_language (int f)
 
 #endif /* ENABLE_NLS */
 
-static int clear_data (CMD *cmd, double ***pZ, DATAINFO **ppdinfo,
-		       MODEL **models)
+static int cli_clear_data (CMD *cmd, double ***pZ, DATAINFO *pdinfo,
+			   MODEL **models)
 {
-    DATAINFO *pdinfo;
     int err = 0;
 
     *paths.datfile = 0;
 
-    err = restore_full_sample(pZ, ppdinfo, NULL); 
-    pdinfo = *ppdinfo;
-
     if (pZ != NULL && *pZ != NULL) {
+	err = restore_full_sample(pZ, pdinfo, NULL); 
 	free_Z(*pZ, pdinfo); 
 	*pZ = NULL;
     }
@@ -232,7 +216,13 @@ static int clear_data (CMD *cmd, double ***pZ, DATAINFO **ppdinfo,
     clear_model(models[1]);
 
     free_modelspec();
-    libgretl_session_cleanup();
+
+    if (cmd->opt & OPT_P) {
+	libgretl_session_cleanup(SESSION_PRESERVE_MATRICES);
+    } else {
+	libgretl_session_cleanup(SESSION_CLEAR_FULL);
+    }
+
     reset_model_count();
     gretl_cmd_destroy_context(cmd);
 
@@ -247,7 +237,7 @@ static void get_a_filename (char *fname)
 }
 
 static int 
-get_an_input_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
+cli_get_input_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 {
     int coding = gretl_compiling_function() || 
 	gretl_compiling_loop();
@@ -255,15 +245,19 @@ get_an_input_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
     if (runit || batch) {
 	/* reading from script file */
-	file_get_line(s->line, s->cmd);
+	err = file_get_line(s->line, s->cmd);
     } else {
 	/* interactive use */
 #ifdef HAVE_READLINE
 	rl_gets(&line_read, (coding)? "> " : "? ");
 	if (line_read == NULL) {
 	    strcpy(s->line, "quit");
+	} else if (strlen(line_read) > MAXLINE - 2) {
+	    err = E_TOOLONG;
 	} else {
-	    strcpy(s->line, line_read);
+	    *s->line = '\0';
+	    strncat(s->line, line_read, MAXLINE - 2);
+	    strcat(s->line, "\n");
 	}
 #else
 	printf("%s", (coding)? "> " : "? ");
@@ -275,19 +269,21 @@ get_an_input_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
     return err;
 }
 
-/* allow for backslash continuation of lines */
+/* allow for continuation of lines */
 
 static int maybe_get_input_line_continuation (char *line)
 {
     char tmp[MAXLINE];
-    int err = 0;
+    int contd, err = 0;
 
     if (!strncmp(line, "quit", 4)) {
 	return 0;
     }
 
-    while (top_n_tail(line)) {
-	tmp[0] = '\0';
+    contd = top_n_tail(line, &err);
+
+    while (contd && !err) {
+	*tmp = '\0';
 
 	if (batch || runit) {
 	    fgets(tmp, MAXLINE - 1, fb);
@@ -302,15 +298,14 @@ static int maybe_get_input_line_continuation (char *line)
 
 	if (*tmp != '\0') {
 	    if (strlen(line) + strlen(tmp) > MAXLINE - 1) {
-		fprintf(stderr, _("Maximum length of command line "
-				  "(%d bytes) exceeded\n"), MAXLINE);
-		err = 1;
+		err = E_TOOLONG;
 		break;
 	    } else {
 		strcat(line, tmp);
 		compress_spaces(line);
 	    }
 	}
+	contd = top_n_tail(line, &err);
     }
 
     return err;
@@ -373,7 +368,7 @@ int main (int argc, char *argv[])
 
     datainfo = datainfo_new();
     if (datainfo == NULL) {
-	noalloc(_("data information"));
+	noalloc();
     }
     
     if (argc > 1) {
@@ -430,27 +425,32 @@ int main (int argc, char *argv[])
     logo(); 
     session_time(NULL);
 
-    prn = gretl_print_new(GRETL_PRINT_STDOUT);
+    prn = gretl_print_new(GRETL_PRINT_STDOUT, &err);
+    if (err) {
+	noalloc();
+    }
 
     line = malloc(MAXLINE);
     if (line == NULL) {
-	noalloc(_("command line"));
+	noalloc();
     } 
 
     gretl_set_paths(&paths, OPT_D); /* defaults, not gui */
+
 #ifdef WIN32
     cli_read_registry(argv[0], &paths);
-    gretl_set_paths(&paths, OPT_NONE); /* not defaults; use registry info */
 #else
-    make_userdir();
+    cli_read_rc(&paths);
 #endif /* WIN32 */
 
+    set_workdir_callback(cli_workdir_callback);
+
     if (!batch) {
-	strcpy(cmdfile, gretl_user_dir());
+	strcpy(cmdfile, gretl_work_dir());
 	strcat(cmdfile, "session.inp");
-	cmdprn = gretl_print_new_with_filename(cmdfile);
-	if (cmdprn == NULL) {
-	    printf(_("Can't open file to save commands\n"));
+	cmdprn = gretl_print_new_with_filename(cmdfile, &err);
+	if (err) {
+	    errmsg(err, prn);
 	    return EXIT_FAILURE;
 	}
     }
@@ -471,24 +471,28 @@ int main (int argc, char *argv[])
 	    exit(EXIT_FAILURE);
 	    break;
 	case GRETL_NATIVE_DATA:
-	    err = gretl_get_data(&Z, &datainfo, paths.datfile, &paths, 
-				 prn);
-	    break;
-	case GRETL_XML_DATA:
-	    err = gretl_read_gdt(&Z, &datainfo, paths.datfile, &paths, 
+	    err = gretl_get_data(paths.datfile, &paths, &Z, datainfo,
 				 OPT_NONE, prn);
 	    break;
-	case GRETL_CSV_DATA:
-	    err = import_csv(&Z, &datainfo, paths.datfile, OPT_NONE, prn);
+	case GRETL_XML_DATA:
+	    err = gretl_read_gdt(paths.datfile, &paths, &Z, datainfo, 
+				 OPT_NONE, prn);
 	    break;
-	case GRETL_OCTAVE:
-	    err = import_octave(&Z, &datainfo, paths.datfile, prn);
+	case GRETL_CSV:
+	    err = import_csv(paths.datfile, &Z, datainfo, OPT_NONE, prn);
 	    break;
+	case GRETL_XLS:
 	case GRETL_GNUMERIC:
-	case GRETL_EXCEL:
-	case GRETL_WF1:
+	case GRETL_ODS:
+	    err = import_spreadsheet(paths.datfile, ftype, NULL, NULL,
+				     &Z, datainfo, OPT_NONE, prn);
+	    break;
 	case GRETL_DTA:
-	    err = import_other(&Z, &datainfo, ftype, paths.datfile, prn);
+	case GRETL_JMULTI:
+	case GRETL_OCTAVE:
+	case GRETL_WF1:
+	    err = import_other(paths.datfile, ftype, &Z, datainfo, 
+			       OPT_NONE, prn);
 	    break;
 	case GRETL_SCRIPT:
 	    runit = 1;
@@ -518,7 +522,7 @@ int main (int argc, char *argv[])
     /* allocate memory for models */
     models = allocate_working_models(2);
     if (models == NULL) {
-	noalloc("models"); 
+	noalloc(); 
     }
 
     gretl_cmd_init(&cmd);
@@ -559,8 +563,8 @@ int main (int argc, char *argv[])
     if (batch || runit) {
 	/* re-initialize: will be incremented by "run" cmd */
 	runit = 0;
-	sprintf(line, "run %s", runfile);
-	exec_line(&state, &Z, &datainfo);
+	sprintf(line, "run %s\n", runfile);
+	exec_line(&state, &Z, datainfo);
     }
 
     /* should we stop immediately on error, in batch mode? */
@@ -569,34 +573,44 @@ int main (int argc, char *argv[])
     /* main command loop */
     while (cmd.ci != QUIT && fb != NULL && !xout) {
 	char linecopy[MAXLINE];
-	int overflow;
 
 	if (err && errfatal) {
 	    gretl_abort(linecopy);
 	}
 
 	if (gretl_execute_loop()) { 
-	    if (gretl_loop_exec(&state, &Z, &datainfo)) {
+	    if (gretl_loop_exec(&state, &Z, datainfo)) {
 		return 1;
 	    }
 	    set_errfatal(ERRFATAL_AUTO);
 	} else {
-	    err = get_an_input_line(&state, &Z, datainfo);
+	    err = cli_get_input_line(&state, &Z, datainfo);
 	    if (err) {
 		errmsg(err, prn);
-		continue;
+		break;
 	    }
 	}
 
-	overflow = maybe_get_input_line_continuation(line); 
-	if (overflow) {
-	    break;
-	} else {
-	    strcpy(linecopy, line);
-	    err = exec_line(&state, &Z, &datainfo);
-	}
+	if (!state.in_comment) {
+	    err = maybe_get_input_line_continuation(line); 
+	    if (err) {
+		errmsg(err, prn);
+		break;
+	    }
+	} 
+
+	strcpy(linecopy, line);
+	tailstrip(linecopy);
+	err = exec_line(&state, &Z, datainfo);
     } /* end of get commands loop */
 
+    if (!err) {
+	err = gretl_if_state_check(0);
+	if (err) {
+	    errmsg(err, prn);
+	}
+    }
+
     /* leak check -- try explicitly freeing all memory allocated */
 
     destroy_working_models(models, 2);
@@ -616,19 +630,8 @@ int main (int argc, char *argv[])
     return 0;
 }
 
-static void printf_strip (char *s)
+static void printline (const char *s)
 {
-    int i, n;
-
-    while (isspace((unsigned char) *s)) s++;
-
-    n = strlen(s);
-
-    for (i=n-1; i>0; i--) {
-	if (isspace(s[i]) || s[i] == '\r') s[i] = '\0';
-	else break;
-    }
-
     if (gretl_compiling_loop()) {
 	printf("> %s\n", s);
     } else {
@@ -636,66 +639,47 @@ static void printf_strip (char *s)
     }
 }
 
-static void
-cli_do_autofit_plot (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
-		     PRN *prn)
-{
-    int plotlist[3];
-    int err = 0;
-
-    plotlist[0] = 2;
-    plotlist[1] = gretl_model_get_depvar(pmod);
-    plotlist[2] = varindex(pdinfo, "autofit");
-
-    err = gnuplot(plotlist, NULL, (const double **) *pZ, 
-		  pdinfo, OPT_B | OPT_O | OPT_T);
-
-    if (err) {
-	pputs(prn, _("gnuplot command failed\n"));
-    }
-}
-
 static void cli_exec_callback (ExecState *s, double ***pZ,
 			       DATAINFO *pdinfo)
 {
     int ci = s->cmd->ci;
 
-    if (ci == FIT && dataset_is_time_series(pdinfo)) {
-	cli_do_autofit_plot(s->models[0], pZ, pdinfo, s->prn);
-    } else if (ci == VAR || ci == VECM) {
+    if (ci == VAR || ci == VECM) {
 	maybe_stack_var(s->var, s->cmd);
     } else if (ci == END && !strcmp(s->cmd->param, "restrict")) {
 	maybe_stack_var(s->var, s->cmd);
-    }
+    } 
 }
 
 static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
-			    DATAINFO **ppdinfo, MODEL **models,
+			    DATAINFO *pdinfo, MODEL **models,
 			    PRN *prn)
 {
-    DATAINFO *pdinfo = *ppdinfo;
     char datfile[MAXLEN] = {0};
     char response[3];
-    int k, dbdata = 0;
+    int ftype, dbdata = 0;
     int err = 0;
 
-    err = getopenfile(line, datfile, &paths, (cmd->opt & OPT_W)?
-		      OPT_W : OPT_NONE);
-    if (err) {
-	errmsg(err, prn);
-	return err;
+    if (!(cmd->opt & OPT_O)) {
+	err = getopenfile(line, datfile, &paths, (cmd->opt & OPT_W)?
+			  OPT_W : OPT_NONE);
+	if (err) {
+	    errmsg(err, prn);
+	    return err;
+	}
     }
 
     if (cmd->opt & OPT_W) {
-	k = GRETL_NATIVE_DB_WWW;
-     } else if (cmd->opt & OPT_O) {
-	k = GRETL_CSV_DATA;
+	ftype = GRETL_NATIVE_DB_WWW;
+    } else if (cmd->opt & OPT_O) {
+	ftype = GRETL_ODBC;
     } else {
-	k = detect_filetype(datfile, &paths, prn);
+	ftype = detect_filetype(datfile, &paths, prn);
     }
 
-    dbdata = (k == GRETL_NATIVE_DB || k == GRETL_NATIVE_DB_WWW ||
-	      k == GRETL_RATS_DB || k == GRETL_PCGIVE_DB);
+    dbdata = (ftype == GRETL_NATIVE_DB || ftype == GRETL_NATIVE_DB_WWW ||
+	      ftype == GRETL_RATS_DB || ftype == GRETL_PCGIVE_DB ||
+	      ftype == GRETL_ODBC);
 
     if (data_status && !batch && !dbdata && cmd->ci != APPEND &&
 	strcmp(datfile, paths.datfile)) {
@@ -708,31 +692,29 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
 	}
     }
 
-    if (data_status) {
-	if (!dbdata && cmd->ci != APPEND) {
-	    clear_data(cmd, pZ, ppdinfo, models);
-	    pdinfo = *ppdinfo;
-	}
+    if (!dbdata && cmd->ci != APPEND) {
+	cli_clear_data(cmd, pZ, pdinfo, models);
     } 
 
-    if (k == GRETL_CSV_DATA) {
-	err = import_csv(pZ, ppdinfo, datfile, OPT_NONE, prn);
-    } else if (k == GRETL_OCTAVE) {
-	err = import_octave(pZ, ppdinfo, datfile, prn);
-    } else if (WORKSHEET_IMPORT(k)) {
-	err = import_other(pZ, ppdinfo, k, datfile, prn);
-    } else if (k == GRETL_XML_DATA) {
-	err = gretl_read_gdt(pZ, ppdinfo, datfile, &paths, 
-			     OPT_NONE, prn);
+    if (ftype == GRETL_CSV) {
+	err = import_csv(datfile, pZ, pdinfo, cmd->opt, prn);
+    } else if (SPREADSHEET_IMPORT(ftype)) {
+	err = import_spreadsheet(datfile, ftype, cmd->list, cmd->extra,
+				 pZ, pdinfo, cmd->opt, prn);
+    } else if (OTHER_IMPORT(ftype)) {
+	err = import_other(datfile, ftype, pZ, pdinfo, cmd->opt, prn);
+    } else if (ftype == GRETL_XML_DATA) {
+	err = gretl_read_gdt(datfile, &paths, pZ, pdinfo, 
+			     cmd->opt, prn);
+    } else if (ftype == GRETL_ODBC) {
+	err = set_odbc_dsn(line, prn);
     } else if (dbdata) {
-	err = set_db_name(datfile, k, &paths, prn);
+	err = set_db_name(datfile, ftype, &paths, prn);
     } else {
-	err = gretl_get_data(pZ, ppdinfo, datfile, &paths, 
-			     prn);
+	err = gretl_get_data(datfile, &paths, pZ, pdinfo,  
+			     cmd->opt, prn);
     }
 
-    pdinfo = *ppdinfo;
-
     if (err) {
 	errmsg(err, prn);
 	return err;
@@ -761,9 +743,8 @@ static gretlopt plot_opt (gretlopt opt, int batch)
 
 #define ENDRUN (NC + 1)
 
-static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
+static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 {
-    DATAINFO *pdinfo = *ppdinfo;
     char *line = s->line;
     CMD *cmd = s->cmd;
     PRN *prn = s->prn;
@@ -787,11 +768,11 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 	return err;
     } 
 
-    if (!s->in_comment) {
+    if (!s->in_comment && !cmd->context) {
 	/* catch requests relating to saved objects, which are not
 	   really "commands" as such */
 	k = saved_object_action(line, pZ, pdinfo, models, prn);
-	if (k == 1) return 0;   /* action was OK */
+	if (k == 1) return 0;   /* action was OK, or ignored */
 	if (k == -1) return 1;  /* action was faulty */
 
 	/* are we ready for this? */
@@ -812,6 +793,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
     }
 
     if (err) {
+	gretl_exec_state_uncomment(s);
 	errmsg(err, prn);
 	return err;
     }
@@ -821,7 +803,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 
     /* if in batch mode, echo comments from input */
     if (batch && cmd->ci == CMD_COMMENT && gretl_echo_on()) {
-	printf_strip(linebak);
+	printline(linebak);
     }
 
     if (cmd->ci < 0) {
@@ -891,16 +873,31 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 		errmsg(err, prn);
 	    } 
 	    break;
-	}	    
-	err = dataset_drop_listed_variables(cmd->list, pZ, pdinfo, &k);
+	}
+	if (get_string_by_name(cmd->param)) {
+	    err = delete_saved_string(cmd->param, prn);
+	    if (err) {
+		errmsg(err, prn);
+	    } 
+	    break;
+	}
+	if (get_list_by_name(cmd->extra)) {
+	    err = delete_list_by_name(cmd->extra);
+	    if (err) {
+		errmsg(err, prn);
+	    } 
+	    break;
+	}
+	err = dataset_drop_listed_variables(cmd->list, pZ, pdinfo, 
+					    &k, prn);
 	if (err) {
-	    pputs(prn, _("Failed to shrink the data set"));
+	    errmsg(err, prn);
 	} else {
 	    if (k) {
 		pputs(prn, _("Take note: variables have been renumbered"));
 		pputc(prn, '\n');
+		maybe_list_vars(pdinfo, prn);
 	    }
-	    maybe_list_vars(pdinfo, prn);
 	}
 	break;
 
@@ -914,9 +911,14 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 			     "which is a dummy variable\n(with values 1 or 0)\n"));
 		break;
 	    }
-	    
-	    err = gnuplot(cmd->list, cmd->param, (const double **) *pZ, 
-			  pdinfo, plot_opt(cmd->opt, batch));
+	    if (cmd->opt & OPT_C) {
+		err = xy_plot_with_control(cmd->list, cmd->param, 
+					   (const double **) *pZ, pdinfo, 
+					   plot_opt(cmd->opt, batch));
+	    } else {
+		err = gnuplot(cmd->list, cmd->param, (const double **) *pZ, 
+			      pdinfo, plot_opt(cmd->opt, batch));
+	    }
 	} else {
 	    err = multi_scatters(cmd->list, (const double **) *pZ, pdinfo, 
 				 plot_opt(cmd->opt, batch));
@@ -930,13 +932,12 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 	break;
 
     case HELP:
-	cli_help(cmd->param, paths.helpfile, 1, prn);
+	cli_help(cmd->param, &paths, cmd->opt, prn);
 	break;
 
     case OPEN:
     case APPEND:
-	err = cli_open_append(cmd, line, pZ, ppdinfo, models, prn);
-	pdinfo = *ppdinfo;
+	err = cli_open_append(cmd, line, pZ, pdinfo, models, prn);
 	break;
 
     case NULLDATA:
@@ -949,10 +950,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 	    pputs(prn, _("Data series length count missing or invalid\n"));
 	    break;
 	}
-	if (data_status) {
-	    clear_data(cmd, pZ, ppdinfo, models);
-	    pdinfo = *ppdinfo;
-	}	
+	cli_clear_data(cmd, pZ, pdinfo, models);
 	err = open_nulldata(pZ, pdinfo, data_status, k, prn);
 	if (err) { 
 	    pputs(prn, _("Failed to create empty data set\n"));
@@ -983,11 +981,11 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 
 	printf(_("type a filename to store output (enter to quit): "));
 	get_a_filename(outfile);
-	top_n_tail(outfile);
+	top_n_tail(outfile, NULL);
 
 	if (*outfile != 0 && *outfile != '\n' && *outfile != '\r' 
 	    && strcmp(outfile, "q")) {
-	    const char *udir = gretl_user_dir();
+	    const char *udir = gretl_work_dir();
 
 	    printf(_("writing session output to %s%s\n"), udir, outfile);
 #ifdef WIN32
@@ -1011,6 +1009,9 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 	    pputs(prn, _("Command is malformed\n"));
 	    break;
 	}
+	if (gretl_messages_on()) {
+	    pprintf(prn, " %s\n", runfile);
+	}
 	if (cmd->ci == INCLUDE && gretl_is_xml_file(runfile)) {
 	    err = load_user_XML_file(runfile);
 	    if (err) {
@@ -1043,9 +1044,16 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 	}
 	break;
 
+    case DATAMOD:
+	if (cmd->aux == DS_CLEAR) {
+	    err = cli_clear_data(cmd, pZ, pdinfo, models);
+	    pputs(prn, _("Dataset cleared\n"));
+	    break;
+	}
+	/* else fall through */
+
     default:
-	err = gretl_cmd_exec(s, pZ, ppdinfo);
-	pdinfo = *ppdinfo;
+	err = gretl_cmd_exec(s, pZ, pdinfo);
 	break;
     }
 
@@ -1073,6 +1081,10 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 	echo_cmd(cmd, pdinfo, line, CMD_RECORDING, cmdprn);
     }
 
+    if (err) {
+	gretl_exec_state_uncomment(s);
+    }
+
     return err;
 }
 
