@@ -652,25 +652,29 @@ make_area (PLOTGROUP *grp)
     return grp->window;
 }
 
-static double median (double *x, const int n)
+#define NEWQUANT 1
+
+#if NEWQUANT
+
+static void quartiles (double *x, const int n, BOXPLOT *box)
 {
-    int n2;
-    double xx;
+    double p[3] = {0.25, 0.5, 0.75};
 
-    qsort(x, n, sizeof *x, gretl_compare_doubles);
+    gretl_array_quantiles(x, n, p, 3);
 
-    n2 = n/2;
-    xx = (n % 2)? x[n2] : 0.5 * (x[n2 - 1] + x[n2]);
-    return xx;
+    box->lq = p[0];
+    box->median = p[1];
+    box->uq = p[2];
 }
 
+#else
+
 static double 
 quartiles (const double *x, const int n, BOXPLOT *box)
 {
-    int n2;
+    int n2 = n / 2;
     double xx;
 
-    n2 = n/2;
     xx = (n % 2)? x[n2] : 0.5 * (x[n2 - 1] + x[n2]);
 
     if (box != NULL) {
@@ -683,9 +687,12 @@ quartiles (const double *x, const int n, BOXPLOT *box)
 	    box->uq = quartiles(x + n2, n2, NULL);
 	}
     }
+
     return xx;
 }
 
+#endif
+
 static int 
 add_outliers (const double *x, const int n, BOXPLOT *box)
 {
@@ -694,15 +701,18 @@ add_outliers (const double *x, const int n, BOXPLOT *box)
     
     box->outliers = NULL;
 
-    for (i=0; i<n; i++) 
-	if (x[i] < (box->lq - iqr) || x[i] > (box->uq + iqr)) 
+    for (i=0; i<n; i++) {
+	if (x[i] < (box->lq - iqr) || x[i] > (box->uq + iqr)) {
 	    nout++;
+	}
+    }
  
     if (nout > 0) {
 	int nlo = 0, nhi = 0;
 
 	box->outliers = malloc(sizeof *box->outliers);
 	if (box->outliers == NULL) return 1;
+
 	box->outliers->n = nout;
 	box->outliers->vals = malloc (nout * sizeof(double));
 	if (box->outliers->vals == NULL) {
@@ -710,6 +720,7 @@ add_outliers (const double *x, const int n, BOXPLOT *box)
 	    box->outliers = NULL;
 	    return 1;
 	}
+
 	nout = 0;
 	for (i=0; i<n; i++) {
 	    if (x[i] < (box->lq - iqr)) {
@@ -721,28 +732,33 @@ add_outliers (const double *x, const int n, BOXPLOT *box)
 		nhi++;
 	    }
 	}
+
 	box->outliers->rmin = (nlo)? x[nlo] : box->min;
 	box->outliers->rmax = (nhi)? x[n-nhi-1] : box->max;
     }
+
     return 0;
 }
 
 #define ITERS 560
-#define CONFIDENCE 90
+#define CONFIDENCE 0.90
+
+/* obtain bootstrap estimate of 90% confidence interval
+   for the sample median of data series x; return low and
+   high values in 'low' and 'high' */
 
 static int 
 median_interval (double *x, int n, double *low, double *high)
-     /* obtain bootstrap estimate of 90% confidence interval
-	for the sample median of data series x; return low and
-	high values in 'low' and 'high' */
 {
     double *medians, *samp;
+    double p[2];
     int i, j, t;
+    int err;
 
-    medians = malloc (ITERS * sizeof *medians);
+    medians = malloc(ITERS * sizeof *medians);
     if (medians == NULL) return 1;
 
-    samp = malloc (n * sizeof *samp);
+    samp = malloc(n * sizeof *samp);
     if (samp == NULL) {
 	free(medians);
 	return 1;
@@ -755,21 +771,21 @@ median_interval (double *x, int n, double *low, double *high)
 	    samp[j] = x[t];
 	}
 	/* find the median of the sample */
-	medians[i] = median(samp, n);
+	medians[i] = gretl_array_quantile(samp, n, 0.5);
     }
 
-    /* sort the sample medians */
-    qsort(medians, ITERS, sizeof *medians, gretl_compare_doubles);
-    
-    /* return the right values */
-    j = 100 / ((100 - CONFIDENCE) / 2);
-    *low = medians[ITERS / j];
-    *high = medians[ITERS - ITERS/j];
+    p[0] = (1.0 - CONFIDENCE) / 2.0;
+    p[1] = 1.0 - p[0];
+
+    err = gretl_array_quantiles(medians, ITERS, p, 2);
+
+    *low = p[0];
+    *high = p[1];
 
     free(samp);
     free(medians);
 
-    return 0;
+    return err;
 }
 
 int ps_print_plots (const char *fname, int flag, gpointer data) 
@@ -965,6 +981,7 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
     }
 
     plotgrp->saved = 0;
+    plotgrp->show_outliers = 0;
 
     plotgrp->nplots = list[0];
     plotgrp->plots = mymalloc(plotgrp->nplots * sizeof *plotgrp->plots);
@@ -993,7 +1010,7 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
 	}
 
 	plotgrp->plots[i].outliers = NULL;
-	plotgrp->plots[i].mean = gretl_mean(0, n, x);
+	plotgrp->plots[i].mean = gretl_mean(0, n-1, x);
 	qsort(x, n, sizeof *x, gretl_compare_doubles);
 	plotgrp->plots[i].min = x[0];
 	plotgrp->plots[i].max = x[n-1];
@@ -1004,7 +1021,7 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
 	    /* notched boxplots wanted */
 	    if (median_interval(x, n, &plotgrp->plots[i].conf[0],
 				&plotgrp->plots[i].conf[1])) {
-		errbox (_("Couldn't obtain confidence interval"));
+		errbox(_("Couldn't obtain confidence interval"));
 		plotgrp->plots[i].conf[0] = 
 		    plotgrp->plots[i].conf[1] = NADBL;
 	    }
@@ -1025,7 +1042,6 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
     plotgrp->width = width;
     plotgrp->numbers = NULL;
 
-    plotgrp->show_outliers = 0;
     read_boxrc(plotgrp);
 
     if (plotgrp->show_outliers) {
@@ -1269,7 +1285,7 @@ static FILE *get_custom_file (void)
     fp = fopen(".boxplotrc", "r");
 
     if (fp == NULL) {
-	sprintf(boxrc, "%s.boxplotrc", paths.userdir);
+	sprintf(boxrc, "%s.boxplotrc", paths.workdir);
 	fp = gretl_fopen(boxrc, "r");
     }
 
@@ -1278,7 +1294,7 @@ static FILE *get_custom_file (void)
     }
 
     if (fp == NULL) {
-	sprintf(boxrc, "%splotconfig.txt", paths.userdir);
+	sprintf(boxrc, "%splotconfig.txt", paths.workdir);
 	fp = gretl_fopen(boxrc, "r");
     }    
 
@@ -1360,7 +1376,7 @@ static int dump_boxplot (PLOTGROUP *grp)
     int i;
     BOXPLOT *plt;
 
-    build_path(boxplottmp, paths.userdir, "boxdump.tmp", NULL);
+    build_path(boxplottmp, paths.dotdir, "boxdump.tmp", NULL);
 
     fp = gretl_fopen(boxplottmp, "w");
     if (fp == NULL) {
