@@ -21,10 +21,10 @@
 
 #define NTESTS 6
 #define NPTESTS 2
-#define NPVAL 7
-#define NDISTS 7
-#define NGRAPHS 6
-#define NRAND 8
+#define NPVAL 8
+#define NDISTS 8
+#define NGRAPHS 7
+#define NRAND 9
 #define NTESTENTRY 7
 #define NDISTENTRY 4
 
@@ -74,7 +74,9 @@ enum {
     GAMMA_DIST,
     BINOMIAL_DIST,
     POISSON_DIST,
-    DW_DIST
+    WEIBULL_DIST,
+    DW_DIST,
+    MAX_DIST
 };  
 
 enum {
@@ -91,14 +93,6 @@ enum {
     NP_RUNS
 };
 
-enum {
-    C_INT,
-    C_POS_INT,
-    C_DBL,
-    C_POS_DBL,
-    C_FRAC
-};
-
 /* functions relating to distribution graphics */
 
 static int get_dist_and_params (const char *s, int *d, double *x)
@@ -121,6 +115,8 @@ static int get_dist_and_params (const char *s, int *d, double *x)
 	*d = BINOMIAL_DIST;
     } else if (sscanf(s, "# Poisson(%lf)", &x[0]) == 1) {
 	*d = POISSON_DIST;
+    } else if (sscanf(s, "# Weibull(%lf,%lf)", &x[0], &x[1]) == 2) {
+	*d = WEIBULL_DIST;
     } else {
 	ret = 0;
     }
@@ -150,7 +146,9 @@ static int current_graph_dist (png_plot *plot)
 	    d = BINOMIAL_DIST;
 	} else if (!strncmp(s, "# Poisson(", 10)) {
 	    d = POISSON_DIST;
-	} 
+	} else if (!strncmp(s, "# Weibull(", 10)) {
+	    d = WEIBULL_DIST;
+	}
     }
 
     return d;
@@ -193,6 +191,8 @@ dist_graph_title (int dist, double *parms)
 	s = g_strdup_printf(G_("Binomial(%d, %g)"), (int) parms[1], parms[0]);
     } else if (dist == POISSON_DIST) {
 	s = g_strdup_printf(G_("Poisson(%g)"), parms[0]);
+    } else if (dist == WEIBULL_DIST) {
+	s = g_strdup_printf(G_("Weibull(%g, %g)"), parms[0], parms[1]);
     }
 
     return s;
@@ -214,6 +214,8 @@ static gchar *dist_comment_line (int dist, double *parms)
 	s = g_strdup_printf("# Binomial(%d,%g)", (int) parms[1], parms[0]);
     } else if (dist == POISSON_DIST) {
 	s = g_strdup_printf("# Poisson(%g)", parms[0]);
+    } else if (dist == WEIBULL_DIST) {
+	s = g_strdup_printf("# Weibull(%g,%g)", parms[0], parms[1]);
     }
 
     return s;
@@ -226,6 +228,8 @@ enum {
     ID_L,
     ID_M,
     ID_S,
+    ID_SHP,
+    ID_SCL,
     ID_MAX
 };
 
@@ -240,6 +244,7 @@ enum {
 #define F_COMB   "comb(n,k)=n!/(k!*(n-k)!)"
 #define F_BINOM  "binom(k,n,p)=comb(int(n),int(k))*p**k*(1-p)**(n-k)"
 #define F_POIS   "poisson(z,k)=exp(-z)*(z**k)/(int(k))!"
+#define F_WEIB   "weibull(x,shp,scl)=(shp/scl)*(x/scl)**(shp-1.0)*exp(-(x/scl)**shp)"
 
 static void 
 dist_xmin_xmax (int d, double *parms, double *xmin, double *xmax)
@@ -292,6 +297,11 @@ dist_xmin_xmax (int d, double *parms, double *xmin, double *xmax)
 	st = 'P';
 	*xmin = 0;
 	x[1] = 0.0015;
+    } else if (d == WEIBULL_DIST) {
+	st = 'W';
+	*xmin = 0;
+	x[1] = parms[1];
+	x[2] = 0.0004;
     }
 
     if (st) {
@@ -329,6 +339,11 @@ static double dist_xmax (int d, double *parms)
 	st = 'P';
 	x[1] = 0.0015;
 	break;
+    case WEIBULL_DIST:
+	st = 'W';
+	x[1] = parms[1];
+	x[2] = 0.0004;
+	break;
     }
 
     return gretl_get_critval(st, x);
@@ -343,6 +358,7 @@ static int n_literal_lines (int d, int alt, int ptype)
 	n = (ptype == PLOT_H_TEST)? 2 : 4; 
 	break;
     case T_DIST:
+    case WEIBULL_DIST:
 	n = 4;
 	break;
     case CHISQ_DIST:	
@@ -483,6 +499,11 @@ make_plot_line (char *targ, int d, int alt, const int *ids)
 	k = ids[ID_L] + 1;
 	sprintf(targ, "int(t),poisson(lambda%d,t)", k);
 	break;
+    case WEIBULL_DIST:
+	k = ids[ID_SHP] + 1;
+	j = ids[ID_SCL] + 1;
+	sprintf(targ, "t,weibull(t,shp%d,scl%d)", k, j);
+	break;
     } 
 }
 
@@ -640,6 +661,11 @@ static void dist_graph (int d, double *parms)
 	fprintf(fp, "lambda1=%g\n", parms[0]);
 	fprintf(fp, "%s\n", F_POIS);
 	break;
+    case WEIBULL_DIST:
+	fprintf(fp, "shp1=%g\n", parms[0]);
+	fprintf(fp, "scl1=%g\n", parms[1]);
+	fprintf(fp, "%s\n", F_WEIB);
+	break;
     }
 
     fprintf(fp, "plot \\\n");
@@ -666,7 +692,7 @@ static void dist_graph (int d, double *parms)
     }
 }
 
-#define ALT_CHI NGRAPHS
+#define ALT_CHI MAX_DIST
 
 static void revise_distribution_plot (png_plot *plot, int d, double *parms)
 {
@@ -675,7 +701,7 @@ static void revise_distribution_plot (png_plot *plot, int d, double *parms)
     const char *f1 = NULL, *f2 = NULL;
     char v1[32] = {0};
     char v2[32] = {0};
-    char got[NGRAPHS+1] = {0};
+    char got[MAX_DIST+1] = {0};
     int ids[ID_MAX] = {0};
     double x[2] = {0};
     int i, k, alt = 0;
@@ -727,7 +753,15 @@ static void revise_distribution_plot (png_plot *plot, int d, double *parms)
 	    if (id > ids[ID_S]) {
 		ids[ID_S] = id;
 	    }
-	}	    
+	} else if (sscanf(s, "shp%d=", &id) == 1) {
+	    if (id > ids[ID_SHP]) {
+		ids[ID_SHP] = id;
+	    }
+	} else if (sscanf(s, "scl%d=", &id) == 1) {
+	    if (id > ids[ID_SCL]) {
+		ids[ID_SCL] = id;
+	    }
+	}
     }
 
     /* alternate forms for some plots */
@@ -778,6 +812,12 @@ static void revise_distribution_plot (png_plot *plot, int d, double *parms)
 	k = ids[ID_L] + 1;
 	sprintf(v1, "lambda%d=%g", k, parms[0]);
 	break;
+    case WEIBULL_DIST:
+	k = ids[ID_SHP] + 1;
+	sprintf(v1, "shp%d=%g", k, parms[0]);
+	k = ids[ID_SCL] + 1;
+	sprintf(v2, "scl%d=%g", k, parms[1]);
+	break;
     }  
 
     if (*v1) {
@@ -828,6 +868,9 @@ static void revise_distribution_plot (png_plot *plot, int d, double *parms)
     case POISSON_DIST:
 	if (!got[POISSON_DIST]) f1 = F_POIS;
 	break;
+    case WEIBULL_DIST:
+	if (!got[WEIBULL_DIST]) f1 = F_WEIB;
+	break;
     }
 
     if (f1 != NULL) {
@@ -872,7 +915,20 @@ static void revise_distribution_plot (png_plot *plot, int d, double *parms)
 
 /* end of graphics functions */
 
-/* getval: get a numerical value from a text entry box
+static double get_real_const (const char *s, EntryValType t)
+{
+    if (t == C_DBL || t == C_POS_DBL) {
+	if (!strcmp(s, "e")) {
+	    return 2.71828182845904523536;
+	} else if (!strcmp(s, "pi")) {
+	    return M_PI;
+	}
+    }
+
+    return NADBL;
+}
+
+/* entry_get_numeric_value: get a numerical value from a text entry box
 
    t == C_DBL: parse the entry as a double
    t == C_POS_DBL: parse as a positive double
@@ -888,7 +944,7 @@ static void revise_distribution_plot (png_plot *plot, int d, double *parms)
    C_INT and C_POS_INT, we flag an error by returning -1.
 */
 
-static double getval (GtkWidget *w, int t)
+double entry_get_numeric_value (GtkWidget *w, EntryValType t)
 {
     const gchar *text = gtk_entry_get_text(GTK_ENTRY(w));
     char s[32];
@@ -904,7 +960,15 @@ static double getval (GtkWidget *w, int t)
     }
 
     *s = '\0';
+
+    while (isspace(*text)) text++;
     strncat(s, text, 31);
+    tailstrip(s);
+
+    x = get_real_const(s, t);
+    if (!na(x)) {
+	return x;
+    }
 
     if (get_local_decpoint() != '.') {
 	gretl_push_c_numeric_locale();
@@ -959,22 +1023,30 @@ static double getval (GtkWidget *w, int t)
     return x;
 }
 
-/* call plugin function to look up part of the table of
-   critical values for the Durbin-Watson statistic
+#define getval(w,t) entry_get_numeric_value(w,t)
+
+/* call plugin function to look up the table of critical values for
+   the Durbin-Watson statistic
 */
 
-static void dw_lookup (dist_t *tab)
+static void dw_lookup_call (dist_t *tab)
 {
     void *handle = NULL;
-    void (*dw)(int, PRN *) = NULL;
+    int (*dw)(int, int, gretl_matrix **) = NULL;
+    gretl_vector *v = NULL;
     PRN *prn;
-    int n;
+    int n, k, err = 0;
 
     n = getval(tab->entry[0], C_POS_INT);
     if (n < 0) {
 	return;
     }
 
+    k = getval(tab->entry[1], C_POS_INT);
+    if (k < 0) {
+	return;
+    }    
+
     dw = gui_get_plugin_function("dw_lookup", &handle);
     if (dw == NULL) {
 	return;
@@ -985,11 +1057,23 @@ static void dw_lookup (dist_t *tab)
 	return;
     }  
 
-    (*dw)(n, prn);
+    err = (*dw)(n, k, &v);
     close_plugin(handle);
 
-    view_buffer(prn, 77, 300, _("gretl: critical values"), 
-		STAT_TABLE, NULL);
+    if (!err) {
+	pprintf(prn, "%s, n = %d, k = %d\n\n",
+		/* xgettext:no-c-format */
+		_("5% critical values for Durbin-Watson statistic"), 
+		(int) gretl_vector_get(v, 2),
+		(int) gretl_vector_get(v, 3));
+	pprintf(prn, "  dL = %6.4f\n", gretl_vector_get(v, 0));
+	pprintf(prn, "  dU = %6.4f\n", gretl_vector_get(v, 1));
+	gretl_vector_free(v);
+	view_buffer(prn, 77, 300, _("gretl: critical values"), 
+		    STAT_TABLE, NULL);
+    } else {
+	gui_errmsg(err);
+    }
 }
 
 static int page_from_dist (int code, int dist)
@@ -1012,8 +1096,10 @@ static int page_from_dist (int code, int dist)
 	    return 4;
 	case POISSON_DIST:
 	    return 5;
-	case DW_DIST:
+	case WEIBULL_DIST:
 	    return 6;
+	case DW_DIST:
+	    return 7;
 	}
     }
 
@@ -1041,6 +1127,8 @@ static int dist_from_page (int code, int page)
 	case 5:
 	    return POISSON_DIST;
 	case 6:
+	    return WEIBULL_DIST;
+	case 7:
 	    return DW_DIST;
 	}
     }
@@ -1050,9 +1138,9 @@ static int dist_from_page (int code, int page)
 
 static char dist_to_char (int d)
 {
-    const char *dchars = "uztXFGBP";
+    const char *dchars = "uztXFGBPW";
 
-    return (d >= 0 && d < 8)? dchars[d] : 0;
+    return (d >= 0 && d < 10)? dchars[d] : 0;
 }
 
 static int 
@@ -1086,6 +1174,7 @@ get_dist_entry_vector (int code, dist_t *tab, int d, double *x,
 	if (x[j++] < 0) return 1;
 	break;
     case GAMMA_DIST: 
+    case WEIBULL_DIST:
 	x[j] = getval(tab->entry[j], C_POS_DBL); /* shape */
 	if (na(x[j++])) return 1;
 	x[j] = getval(tab->entry[j], C_POS_DBL); /* scale */
@@ -1148,7 +1237,7 @@ static void get_critical (GtkWidget *w, CalcChild *child)
     
     if (d == DW_DIST) {
 	/* special: just a table look-up */
-	dw_lookup(tabs[i]);
+	dw_lookup_call(tabs[i]);
 	return;
     }
 
@@ -1215,6 +1304,7 @@ static void get_pvalue (GtkWidget *w, CalcChild *child)
     case CHISQ_DIST:
     case F_DIST:
     case GAMMA_DIST:
+    case WEIBULL_DIST:
 	x[j] = getval(tabs[i]->entry[j], C_POS_DBL);
 	if (na(x[j])) return;
 	break;
@@ -1318,9 +1408,13 @@ static void get_random (GtkWidget *w, CalcChild *child)
 	gretl_command_sprintf("genr %s = randgen(G,%g,%g)", vname, 
 			      x[0], x[1]);
 	break;
+    case WEIBULL_DIST:
+	gretl_command_sprintf("genr %s = randgen(W,%g,%g)", vname, 
+			      x[0], x[1]);
+	break;
     case BINOMIAL_DIST: 
 	gretl_command_sprintf("genr %s = randgen(B,%g,%g)", vname, 
-			      x[1], x[0]);
+			      x[0], x[1]);
 	break;
     case POISSON_DIST: 
 	/* FIXME allow variable as param? */
@@ -1395,6 +1489,10 @@ static void np_test (GtkWidget *w, test_t *test)
 	    GTK_TOGGLE_BUTTON(test->extra)->active) {
 	    opt |= OPT_D;
 	}
+	if (test->check != NULL &&
+	    GTK_TOGGLE_BUTTON(test->check)->active) {
+	    opt |= OPT_E;
+	}	
 	err = runs_test(v1, (const double **) Z, datainfo, 
 			opt, prn);
     }	
@@ -1443,7 +1541,7 @@ static void do_h_test (test_t *test, double *x, int n1, int n2)
 	} else {
 	    pprintf(prn, _("Test statistic: t(%d) = (%g - %g)/%g = %g\n"), n1-1,
 		    x[0], x[2], se, ts);
-	    pv = student_pvalue_2(ts, n1 - 1);
+	    pv = student_pvalue_2(n1 - 1, ts);
 	    print_pv(prn, pv, 0.5 * pv);
 	    if (grf) {
 		gparm[0] = n1 - 1;
@@ -1462,9 +1560,9 @@ static void do_h_test (test_t *test, double *x, int n1, int n2)
 		n1-1, n1-1, x[0], x[1], ts);
 
 	if (x[0] > x[1]) {
-	    pv = chisq_cdf_comp(ts, n1 - 1);
+	    pv = chisq_cdf_comp(n1 - 1, ts);
 	} else {
-	    pv = chisq_cdf(ts, n1 - 1);
+	    pv = chisq_cdf(n1 - 1, ts);
 	}
 	print_pv(prn, 2.0 * pv, pv);
 	if (grf) {
@@ -1543,9 +1641,9 @@ static void do_h_test (test_t *test, double *x, int n1, int n2)
 	    pprintf(prn, _("Test statistic: t(%d) = (%g - %g)/%g = %g\n"),
 		    n1 + n2 - 2, x[0], x[2], se, ts);
 	    if (ts > 0) {
-		pv = student_pvalue_2(ts, n1 + n2 - 2);
+		pv = student_pvalue_2(n1 + n2 - 2, ts);
 	    } else {
-		pv = student_pvalue_2(-ts, n1 + n2 - 2);
+		pv = student_pvalue_2(n1 + n2 - 2, -ts);
 	    }
 	    print_pv(prn, pv, 0.5 * pv);
 	    if (grf) {
@@ -1589,12 +1687,12 @@ static void do_h_test (test_t *test, double *x, int n1, int n2)
 	    ts = x[0] / x[1];
 	    pprintf(prn, _("Test statistic: F(%d, %d) = %g\n"), 
 		    n1 - 1, n2 - 1, ts);
-	    pv = snedecor_cdf_comp(ts, n1 - 1, n2 - 1);
+	    pv = snedecor_cdf_comp(n1 - 1, n2 - 1, ts);
 	} else {
 	    ts = x[1] / x[0];
 	    pprintf(prn, _("Test statistic: F(%d, %d) = %g\n"), 
 		    n2 - 1, n1 - 1, ts);
-	    pv = snedecor_cdf_comp(ts, n2 - 1, n1 - 1);
+	    pv = snedecor_cdf_comp(n2 - 1, n1 - 1, ts);
 	}
 
 	print_pv(prn, 2.0 * pv, pv);
@@ -1829,6 +1927,7 @@ static void make_dist_tab (CalcChild *child, int i)
 	N_("gamma"),	    
 	N_("binomial"),
 	N_("poisson"),
+	N_("weibull"),
 	N_(" DW "),
     };
     int d = dist_from_page(child->code, i);
@@ -1871,6 +1970,7 @@ static void make_dist_tab (CalcChild *child, int i)
 	calc_entry(tbl, &rows, N_("dfd"), child, i);
 	break;
     case GAMMA_DIST:
+    case WEIBULL_DIST:
 	calc_entry(tbl, &rows, N_("shape"), child, i);
 	calc_entry(tbl, &rows, N_("scale"), child, i);
 	break;
@@ -1882,7 +1982,9 @@ static void make_dist_tab (CalcChild *child, int i)
 	calc_entry(tbl, &rows, N_("mean"), child, i);
 	break;
     case DW_DIST:
-	calc_entry(tbl, &rows, N_("n"), child, i);
+	calc_entry(tbl, &rows, N_("sample size, n"), child, i);
+	calc_entry(tbl, &rows, N_("number of regressors\n(excluding the constant)"), 
+		   child, i);
 	break;
     default:
 	break;
@@ -2458,15 +2560,26 @@ static void make_nptest_tab (CalcChild *child, int idx)
 			      rows - 1, rows);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), FALSE);
 
+    gtk_widget_show(tmp);
+    test->extra = tmp; 
+
+    if (idx == NP_RUNS) {
+	rows += 1;
+	gtk_table_resize(GTK_TABLE(tbl), rows, 2);
+	tmp = gtk_check_button_new_with_label(_("Assume positive and negative are equiprobable"));
+	gtk_table_attach_defaults(GTK_TABLE(tbl), tmp, 0, 2, 
+				  rows - 1, rows);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), FALSE);
+	gtk_widget_show(tmp);
+	test->check = tmp; 
+    }
+
     if (idx == NP_DIFF) {
 	gtk_widget_set_sensitive(tmp, FALSE);
 	g_signal_connect(G_OBJECT(test->radio[0]), "toggled",
 			 G_CALLBACK(toggle_verbose_state),
 			 tmp);
     }
-
-    gtk_widget_show(tmp);
-    test->extra = tmp; 
 }
 
 /* make tab (notebook page) for hypothesis test */
