@@ -163,7 +163,9 @@ void open_data (gpointer data, guint code, GtkWidget *widget)
 	break;
     case OPEN_CSV:
     case APPEND_CSV:
-	delimiter_dialog(NULL);
+	if (delimiter_dialog(NULL)) {
+	    return;
+	}
 	file_selector(_("Open CSV file"), code, FSEL_DATA_NONE, NULL);
 	break;
     case OPEN_OCTAVE:
@@ -174,8 +176,8 @@ void open_data (gpointer data, guint code, GtkWidget *widget)
     case APPEND_GNUMERIC:
 	file_selector(_("Open Gnumeric file"), code, FSEL_DATA_NONE, NULL);
 	break;
-    case OPEN_EXCEL:
-    case APPEND_EXCEL:
+    case OPEN_XLS:
+    case APPEND_XLS:
 	file_selector(_("Open Excel file"), code, FSEL_DATA_NONE, NULL);
 	break;
     case OPEN_WF1:
@@ -216,9 +218,8 @@ void open_script (gpointer data, guint action, GtkWidget *widget)
     }
 }
 
-void file_save (gpointer data, guint file_code, GtkWidget *widget)
+void file_save (windata_t *vwin, guint file_code, GtkWidget *widget)
 {
-    windata_t *vwin = (windata_t *) data;
     gretlopt opt = OPT_NONE;
     gpointer p = NULL;
 
@@ -239,7 +240,9 @@ void file_save (gpointer data, guint file_code, GtkWidget *widget)
 	data_save_selection_wrapper(file_code, NULL);
 	break;
     case EXPORT_CSV:
-	delimiter_dialog(&opt);
+	if (delimiter_dialog(&opt)) {
+	    return;
+	}
 	p = GINT_TO_POINTER(opt);
     case EXPORT_R:
     case EXPORT_OCTAVE:
@@ -250,9 +253,15 @@ void file_save (gpointer data, guint file_code, GtkWidget *widget)
     case SAVE_TEX:
 	file_selector(_("Save LaTeX file"), file_code, FSEL_DATA_MISC, vwin->data);
 	break;
+    case SAVE_TEXT:
+	file_selector(_("Save text"), file_code, FSEL_DATA_MISC, vwin->data);
+	break;
     case SAVE_GP_CMDS:
 	file_selector(_("Save gnuplot commands"), file_code, FSEL_DATA_MISC, vwin);
 	break;
+    case SAVE_R_CMDS:
+	file_selector(_("Save R commands"), file_code, FSEL_DATA_MISC, vwin);
+	break;
     default:
 	dummy_call();
     }
@@ -293,20 +302,10 @@ void fit_resid_callback (gpointer data, guint code, GtkWidget *widget)
     add_fit_resid(pmod, code, 0);
 }
 
-void VAR_resid_callback (gpointer data, guint eqnum, GtkWidget *widget)
-{
-    add_system_resid(data, eqnum, VAR);
-}
-
-void SYS_resid_callback (gpointer data, guint eqnum, GtkWidget *widget)
-{
-    add_system_resid(data, eqnum, SYSTEM);
-}
-
 void model_stat_callback (gpointer data, guint which, GtkWidget *widget)
 {
-    windata_t *mydata = (windata_t *) data; 
-    MODEL *pmod = mydata->data;
+    windata_t *vwin = (windata_t *) data; 
+    MODEL *pmod = vwin->data;
 
     add_model_stat(pmod, which);
 }
@@ -326,12 +325,13 @@ void model_callback (gpointer data, guint model_code, GtkWidget *widget)
 
 void model_genr_callback (gpointer data, guint u, GtkWidget *widget)
 {
-    windata_t *mydata = (windata_t *) data;
+    windata_t *vwin = (windata_t *) data;
+    int cancel = 0;
 
     edit_dialog(_("gretl: add var"), 
 		_("Enter formula for new variable:"),
-		"", do_model_genr, mydata, 
-		MODEL_GENR, VARCLICK_INSERT_NAME, NULL);   
+		"", do_model_genr, vwin, 
+		MODEL_GENR, VARCLICK_INSERT_NAME, &cancel);   
 }
 
 void selector_callback (gpointer data, guint action, GtkWidget *widget)
@@ -354,7 +354,8 @@ void selector_callback (gpointer data, guint action, GtkWidget *widget)
     } else if (action == VLAGSEL) {
 	selection_dialog(_("gretl: VAR lag selection"), do_vector_model, action, 0);
     } else if (action == GR_XY || action == GR_IMP || action == GR_DUMMY
-	       || action == SCATTERS || action == GR_3D) {
+	       || action == SCATTERS || action == GR_3D
+	       || action == GR_XYZ) {
 	int (*selfunc)() = NULL;
 
 	switch (action) {
@@ -368,6 +369,9 @@ void selector_callback (gpointer data, guint action, GtkWidget *widget)
 	case GR_DUMMY:
 	    selfunc = do_dummy_graph;
 	    break;
+	case GR_XYZ:
+	    selfunc = do_xyz_graph;
+	    break;
 	case SCATTERS:
 	    selfunc = do_scatters;
 	    break;
@@ -402,6 +406,7 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
     const char *defstr = NULL;
     void (*okfunc)() = NULL;
     guint varclick = VARCLICK_NONE;
+    int cancel = 0;
 
     switch (action) {
     case SMPLBOOL:
@@ -475,18 +480,12 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
     }
 
     edit_dialog(_(title), _(query), defstr, okfunc, data, 
-		action, varclick, NULL);   
+		action, varclick, &cancel);   
 }
 
-void gp_send_callback (GtkWidget *w, gpointer data)
-{
-    gp_to_gnuplot(data, 0, w);
-}
-
-void file_save_callback (GtkWidget *w, gpointer data)
+void file_save_callback (GtkWidget *w, windata_t *vwin)
 {
     guint u = 0;
-    windata_t *vwin = (windata_t *) data;
 
     if (g_object_get_data(G_OBJECT(vwin->dialog), "text_out")) {
 	const char *opts[] = {
@@ -513,16 +512,22 @@ void file_save_callback (GtkWidget *w, gpointer data)
 	case VIEW_FUNC_CODE:
 	    u = SAVE_SCRIPT;
 	    break;
-	case GR_PLOT:
+	case EDIT_GP:
 	    u = SAVE_GP_CMDS;
 	    break;
+	case EDIT_R:
+	    u = SAVE_R_CMDS;
+	    break;
+	case VIEW_FILE:
+	    u = SAVE_TEXT;
+	    break;
 	default:
 	    errbox(_("Sorry, not yet implemented"));
 	    return;
 	}
     }
 
-    file_save(data, u, w);
+    file_save(vwin, u, w);
 }
 
 void newdata_callback (gpointer data, guint pd_code, GtkWidget *widget) 
@@ -567,7 +572,8 @@ void do_nistcheck (gpointer p, guint v, GtkWidget *w)
 {
     void *handle;
     int (*run_nist_tests)(const char *, const char *, int);
-    gchar *fname;
+    gchar *datadir = NULL;
+    gchar *fname = NULL;
     
     run_nist_tests = gui_get_plugin_function("run_nist_tests", 
 					     &handle);
@@ -575,13 +581,16 @@ void do_nistcheck (gpointer p, guint v, GtkWidget *w)
 	return;
     }
 
-    fname = g_strdup_printf("%snist.out", paths.userdir);
+    datadir = g_strdup_printf("%sdata%s", paths.gretldir, SLASHSTR);
+    fname = g_strdup_printf("%snist.out", paths.dotdir);
 
-    (*run_nist_tests)(paths.datadir, fname, (int) v);
+    (*run_nist_tests)(datadir, fname, (int) v);
 
     close_plugin(handle);
 
     view_file(fname, 0, 1, 78, 400, VIEW_CODEBOOK);
+
+    g_free(datadir);
     g_free(fname);
 }
 
@@ -597,7 +606,7 @@ void send_file (char *fullname)
         return;
     }
     
-    email_file(fullname, paths.userdir);
+    email_file(fullname, paths.dotdir);
     close_plugin(handle);
 }
 
