@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "gretl.h"
@@ -41,7 +41,7 @@ static int n_full = sizeof full_targets / sizeof full_targets[0];
 
 static void gretl_clipboard_set (int copycode);
 
-#undef CLIPDEBUG
+#define CLIPDEBUG 0
 
 static void gretl_clipboard_free (void)
 {
@@ -76,7 +76,7 @@ static void gretl_clipboard_get (GtkClipboard *clip,
     gchar *str;
     gint length;
 
-#ifdef CLIPDEBUG
+#if CLIPDEBUG
     fprintf(stderr, "info = %d\n", (int) info);
     if (info == TARGET_STRING) {
 	fprintf(stderr, " = TARGET_STRING\n");
@@ -162,6 +162,7 @@ int prn_to_clipboard (PRN *prn, int fmt)
 
     gretl_clipboard_free();
 
+#ifdef ENABLE_NLS
     if (fmt == GRETL_FORMAT_TXT || fmt == GRETL_FORMAT_RTF_TXT) { 
 	/* need to convert from utf8 */
 	gchar *trbuf = my_locale_from_utf8(buf);
@@ -187,6 +188,15 @@ int prn_to_clipboard (PRN *prn, int fmt)
 	clipboard_buf = gretl_strdup(buf);
 	err = (clipboard_buf == NULL);
     }
+#else
+    if (fmt == GRETL_FORMAT_RTF_TXT) { 
+	clipboard_buf = dosify_buffer(buf, fmt);
+	err = (clipboard_buf == NULL);
+    } else { 
+	clipboard_buf = gretl_strdup(buf);
+	err = (clipboard_buf == NULL);
+    }
+#endif
 
     if (!err) {
 	gretl_clipboard_set(fmt);
