@@ -22,12 +22,6 @@
 
 void show_gretl_console (void);
 
-gint console_key_handler (GtkWidget *w, GdkEventKey *key, 
-			      gpointer p);
-
-gint console_mouse_handler (GtkWidget *w, GdkEventButton *event,
-			    gpointer p);
-
 void console_record_sample (const DATAINFO *pdinfo);
 
 int console_sample_changed (const DATAINFO *pdinfo);
