@@ -427,7 +427,7 @@ add_dbdata (windata_t *vwin, double **dbZ, DATAINFO *dbinfo,
     }
 
     if (!err) {
-	register_data(NULL, NULL, 0);
+	register_data(DATA_APPENDED);
     }
 }
 
@@ -978,42 +978,29 @@ void open_bn7_window (char *fname)
 
 static int check_serinfo (char *str, char *sername, int *nobs)
 {
-    char pdc;
     char stobs[OBSLEN], endobs[OBSLEN];
+    char pdc = 0;
     int err = 0;
 
-    if (!isalpha((unsigned char) sername[0]) || 
+    *stobs = *endobs = '\0';
+    *nobs = 0;
+
+    if (!isalpha((unsigned char) *sername) || 
 	sscanf(str, "%c %10s - %10s %*s = %d", 
 	       &pdc, stobs, endobs, nobs) != 4 || 
-	!isdigit((unsigned char) stobs[0]) || 
-	!isdigit((unsigned char) endobs[0]) ||
+	!isdigit((unsigned char) *stobs) || 
+	!isdigit((unsigned char) *endobs) ||
 	(pdc != 'M' && pdc != 'A' && pdc != 'Q' && pdc != 'U' &&
 	 pdc != 'D' && pdc != 'B')) {
-	gchar *msg = g_strdup_printf(_("Database parse error at variable '%s'"), 
-				     sername);
-	errbox(msg);
-	g_free(msg);
+	errbox(_("Database parse error at variable '%s'"), sername);
+	fprintf(stderr, "%s: stobs='%s', endobs='%s', pdc='%c', nobs = %d\n",
+		sername, stobs, endobs, pdc, *nobs);
 	err = 1;
     }
 
     return err;
 }
 
-static char *end_trim (char *s)
-{
-    size_t i, n = strlen(s);
-
-    for (i=n-1; i>0; i--) {
-	if (s[i] == ' ' || s[i] == '\n' || s[i] == '\r') {
-	    s[i] = '\0';
-	} else {
-	    break;
-	}
-    }
-
-    return s;
-}
-
 static char *start_trim (char *s)
 {
     while (*s == ' ') s++;
@@ -1021,42 +1008,44 @@ static char *start_trim (char *s)
     return s;
 }
 
-static void db_drag_series (GtkWidget *w, GdkDragContext *context,
-			    GtkSelectionData *sel, guint info, guint t,
-			    windata_t *vwin)
+static void do_db_drag (GtkWidget *w, GdkDragContext *context,
+			GtkSelectionData *sel, guint info, guint t,
+			windata_t *vwin)
 {
     gtk_selection_data_set(sel, GDK_SELECTION_TYPE_INTEGER, 8, 
 			   (const guchar *) &vwin, sizeof vwin);
 }
 
-static void db_drag_connect (windata_t *vwin)
+static void db_drag_connect (windata_t *vwin, int i)
 {
     gtk_drag_source_set(vwin->listbox, GDK_BUTTON1_MASK,
-			&gretl_drag_targets[GRETL_POINTER],
+			&gretl_drag_targets[i],
 			1, GDK_ACTION_COPY);
 
     g_signal_connect(G_OBJECT(vwin->listbox), "drag_data_get",
-		     G_CALLBACK(db_drag_series),
+		     G_CALLBACK(do_db_drag),
 		     vwin);
 }
 
+#define db_drag_series_connect(v) db_drag_connect(v, GRETL_DBSERIES_PTR)
+#define db_drag_db_connect(v) db_drag_connect(v, GRETL_REMOTE_DB_PTR)
+#define funcpkg_drag_connect(v) db_drag_connect(v, GRETL_REMOTE_FNPKG_PTR)
+
+#define DB_LINELEN 512
+
 static int add_local_db_series_list (windata_t *vwin)
 {
     GtkListStore *store;
     GtkTreeIter iter; 
     gchar *row[3];
     char sername[VNAMELEN];
-    char line1[256], line2[72], dbidx[MAXLEN];
+    char line1[DB_LINELEN], line2[128];
+    char dbidx[MAXLEN];
     FILE *fp;
     size_t n;
     int offset = 0;
     int err = 0;
 
-#if 0
-    fprintf(stderr, "make_series_list: vwin->fname = '%s', vwin->data = %p\n",
-	    vwin->fname, vwin->data);
-#endif
-
     strcpy(dbidx, vwin->fname);
     strcat(dbidx, ".idx");
     fp = gretl_fopen(dbidx, "r");
@@ -1072,15 +1061,22 @@ static int add_local_db_series_list (windata_t *vwin)
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
     while (fgets(line1, sizeof line1, fp) != NULL && !err) {
-	int nobs = 0;
+	int len, nobs = 0;
 
 	if (*line1 == '#') {
 	    continue;
 	}
 
+	len = strlen(line1);
+	if (line1[len-1] != '\n') {
+	    errbox("Database index line is too long: max is %d characters",
+		   DB_LINELEN - 1);
+	    break;
+	}
+
 	err = utf8_correct(line1);
 
-	end_trim(line1);
+	tailstrip(line1);
 	charsub(line1, '\t', ' ');
 
 	if (sscanf(line1, "%15s", sername) != 1) {
@@ -1095,7 +1091,7 @@ static int add_local_db_series_list (windata_t *vwin)
 	    break;
 	}
 
-	end_trim(line2);
+	tailstrip(line2);
 	row[2] = line2;
 
 	if (!err) {
@@ -1111,7 +1107,7 @@ static int add_local_db_series_list (windata_t *vwin)
     }
 
     fclose(fp);
-    db_drag_connect(vwin);
+    db_drag_series_connect(vwin);
 
     return 0;
 }
@@ -1140,7 +1136,7 @@ static int add_remote_db_series_list (windata_t *vwin, char *buf)
 	    continue;
 	}
 
-	end_trim(line1);
+	tailstrip(line1);
 	charsub(line1, '\t', ' ');
 
 	err = utf8_correct(line1);
@@ -1157,7 +1153,8 @@ static int add_remote_db_series_list (windata_t *vwin, char *buf)
 	    break;
 	}
 
-	row[2] = line2;
+	row[2] = tailstrip(line2);
+
 	if (!err) {
 	    err = check_serinfo(line2, sername, &nobs);
 	}
@@ -1171,7 +1168,7 @@ static int add_remote_db_series_list (windata_t *vwin, char *buf)
     }
 
     bufgets_finalize(buf);
-    db_drag_connect(vwin);
+    db_drag_series_connect(vwin);
 
     return 0;
 }
@@ -1288,7 +1285,7 @@ static int add_rats_db_series_list (windata_t *vwin)
 
     insert_and_free_dbwrapper(dw, vwin->listbox);
     vwin->active_var = 0;
-    db_drag_connect(vwin);
+    db_drag_series_connect(vwin);
 
     return 0;
 }
@@ -1320,7 +1317,7 @@ static int add_pcgive_db_series_list (windata_t *vwin)
 
     insert_and_free_dbwrapper(dw, vwin->listbox);
     vwin->active_var = 0;
-    db_drag_connect(vwin);
+    db_drag_series_connect(vwin);
 
     return 0;
 }
@@ -1802,17 +1799,84 @@ static int ggz_extract (char *ggzname)
     return err;
 }
 
+static void offer_db_open (char *target)
+{
+    int resp = yes_no_dialog ("gretl",                      
+			      _("Database installed.\n"
+				"Open it now?"), 0);
+
+    if (resp == GRETL_YES) { 
+	char dbpath[MAXLEN];
+	    
+	strcpy(dbpath, target);
+	strcpy(strrchr(dbpath, '.'), ".bin");
+	open_named_db_index(dbpath);
+    }
+}
+
 enum {
     REAL_INSTALL,
     TMP_INSTALL
 };
 
+static char *get_writable_target (int code, int op, char *objname)
+{
+    FILE *fp;
+    char *targ;
+    int err = 0;
+
+    targ = mymalloc(MAXLEN);
+    if (targ == NULL) {
+	return NULL;
+    }
+
+    if (code == REMOTE_DB) {
+	build_path(targ, paths.binbase, objname, ".ggz");
+    } else {
+	if (op == TMP_INSTALL) {
+	    build_path(targ, paths.dotdir, "dltmp", NULL);
+	    err = gretl_tempname(targ);
+	} else {
+	    char fndir[MAXLEN];
+
+	    get_default_dir(fndir, SAVE_FUNCTIONS);
+	    build_path(targ, fndir, objname, ".gfn");
+	}
+    } 
+
+    if (err) {
+	free(targ);
+	targ = NULL;
+    } else {
+	errno = 0;
+	fp = gretl_fopen(targ, "w");
+
+	if (fp == NULL) {
+	    if (errno == EACCES && code != REMOTE_FUNC_FILES) { 
+		/* write to user dir instead? */
+		build_path(targ, paths.workdir, objname, ".ggz");
+	    } else {
+		file_write_errbox(targ);
+		free(targ);
+		targ = NULL;
+	    }
+	} else {
+	    fclose(fp);
+	}
+    }
+
+    return targ;
+}
+
+/* note : 'vwin' here is the source viewer window displaying the
+   remote file (database or function package) that is being installed
+   onto the local machine.
+*/
+
 static int real_install_file_from_server (windata_t *vwin, int op)
 {
-    gchar *objname;
-    char fndir[MAXLEN];
-    char *target;
-    FILE *fp;
+    gchar *objname = NULL;
+    char *target = NULL;
     int err = 0;
 
     if (vwin->role == REMOTE_DB) {
@@ -1821,57 +1885,25 @@ static int real_install_file_from_server (windata_t *vwin, int op)
 	GtkTreeIter iter;
 
 	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(vwin->listbox));
-	if (!gtk_tree_selection_get_selected(sel, &model, &iter)) {
-	    return 1;
-	}
-
-	gtk_tree_model_get(model, &iter, 0, &objname, -1);
-	if (objname == NULL || *objname == '\0') {
-	    g_free(objname);
-	    return 1;
+	if (gtk_tree_selection_get_selected(sel, &model, &iter)) {
+	    gtk_tree_model_get(model, &iter, 0, &objname, -1);
 	}
     } else {
+	/* remote function packages */
 	tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
 			     vwin->active_var, 0, &objname);
+	
     }
-    
-    target = mymalloc(MAXLEN);
-    if (target == NULL) {
+
+    if (objname == NULL || *objname == '\0') {
+	free(objname);
 	return 1;
-    }
+    }    
 
-    if (vwin->role == REMOTE_FUNC_FILES) {
-	if (op == TMP_INSTALL) {
-	    build_path(target, paths.userdir, "dltmp", NULL);
-	    err = gretl_tempname(target);
-	} else {
-	    get_default_dir(fndir, SAVE_FUNCTIONS);
-	    build_path(target, fndir, objname, ".gfn");
-	}
-    } else {
-	build_path(target, paths.binbase, objname, ".ggz");
-    }
-
-    if (err) {
-	return err;
-    }
-
-    /* try test write to target file */
-
-    errno = 0;
-    fp = gretl_fopen(target, "w");
-
-    if (fp == NULL) {
-	if (errno == EACCES && vwin->role != REMOTE_FUNC_FILES) { 
-	    /* write to user dir instead? */
-	    build_path(target, paths.userdir, objname, ".ggz");
-	} else {
-	    file_write_errbox(target);
-	    free(target);
-	    return 1;
-	}
-    } else {
-	fclose(fp);
+    target = get_writable_target(vwin->role, op, objname);
+    if (target == NULL) {
+	free(objname);
+	return 1;
     }
 
     if (vwin->role == REMOTE_FUNC_FILES) {
@@ -1886,44 +1918,43 @@ static int real_install_file_from_server (windata_t *vwin, int op)
 
     if (err) {
 	show_network_error(NULL);
-	free(objname);
-	free(target);
-	return err;
-    } 
-
-    if (vwin->role == REMOTE_FUNC_FILES) {
-	if (op == REAL_INSTALL) {
-	    infobox(_("Function package installed"));
-	    populate_filelist(vwin, NULL);
-	} else {
-	    gui_show_function_info(target, VIEW_FUNC_INFO);
-	}
     } else {
-	err = ggz_extract(target);
-	if (err) {
-	    if (err != E_FOPEN) {
-		errbox(_("Error unzipping compressed data"));
+	windata_t *local = get_local_viewer(vwin->role);
+
+	if (vwin->role == REMOTE_FUNC_FILES) {
+	    if (op == REAL_INSTALL) {
+		list_store_set_string(GTK_TREE_VIEW(vwin->listbox),
+				      vwin->active_var, 3,
+				      _("Up to date"));
+		if (local != NULL) {
+		    populate_filelist(local, NULL);
+		} 
+	    } else {
+		gui_show_function_info(target, VIEW_FUNC_INFO);
 	    }
 	} else {
-	    int resp = yes_no_dialog ("gretl",                      
-				      _("Database installed.\n"
-					"Open it now?"), 0);
-
-	    if (resp == GRETL_YES) { 
-		char dbpath[MAXLEN];
-	    
-		strcpy(dbpath, target);
-		strcpy(strrchr(dbpath, '.'), ".bin");
-		open_named_db_index(dbpath);
+	    err = ggz_extract(target);
+	    if (err) {
+		if (err != E_FOPEN) {
+		    errbox(_("Error unzipping compressed data"));
+		}
+	    } else {
+		tree_store_set_string(GTK_TREE_VIEW(vwin->listbox),
+				      vwin->active_var, 2,
+				      _("Up to date"));
+		if (local != NULL) {
+		    populate_filelist(local, NULL);
+		} else {
+		    offer_db_open(target);
+		}
 	    }
-	    populate_filelist(vwin, NULL);
 	}
     }
 
     free(objname);
     free(target);
 
-    return 0;
+    return err;
 }
 
 void install_file_from_server (GtkWidget *w, gpointer data)
@@ -2091,13 +2122,27 @@ static void get_local_object_status (char *fname, int role, char *status,
 #else
 	    /* try user dir too, if not on Windows */
 	    if (role == REMOTE_DB) {
-		build_path(fullname, paths.userdir, fname, NULL);
+		build_path(fullname, paths.workdir, fname, NULL);
 	    } else {
-		build_path(fndir, paths.userdir, "functions", NULL);
+		build_path(fndir, paths.workdir, "functions", NULL);
 		build_path(fullname, fndir, fname, NULL);
 	    }
-	    if ((err = stat(fullname, &fbuf)) == -1) {
-		strcpy(status, _("Not installed"));
+	    err = stat(fullname, &fbuf);
+	    if (err == -1) {
+		if (role == REMOTE_FUNC_FILES) {
+		    /* try default working dir */
+		    char *tmp = gretl_default_workdir(&paths);
+
+		    if (tmp != NULL) {
+			build_path(fndir, tmp, "functions", NULL);
+			build_path(fullname, fndir, fname, NULL);
+			err = stat(fullname, &fbuf);
+			free(tmp);
+		    }
+		}
+		if (err == -1) {
+		    strcpy(status, _("Not installed"));
+		}
 	    } 
 #endif
 	} else {
@@ -2315,6 +2360,7 @@ gint populate_remote_db_list (windata_t *vwin)
 	row[2] = status;
 
 	if (bufgets(line, sizeof line, getbuf)) {
+	    tailstrip(line);
 	    utf8_correct(line);
 	    row[1] = line + 2;
 	    ndb = get_ndbs(i);
@@ -2345,7 +2391,7 @@ gint populate_remote_db_list (windata_t *vwin)
 	    /* insert at top level */
 	    gtk_tree_store_append(store, &iter, NULL);
 	    gtk_tree_store_set(store, &iter, 0, row[0], 1, row[1],
-			   2, row[2], -1);
+			       2, row[2], -1);
 	}	    
 
 	i++;
@@ -2355,6 +2401,10 @@ gint populate_remote_db_list (windata_t *vwin)
     free(getbuf);
     free_src_info();
 
+    if (!err) {
+	db_drag_db_connect(vwin);
+    }
+
     return err;
 }
 
@@ -2402,11 +2452,13 @@ gint populate_remote_func_list (windata_t *vwin)
 	basename = strip_extension(fname);
 
 	if (bufgets(line, sizeof line, getbuf)) {
+	    tailstrip(line);
 	    utf8_correct(line);
 	    descrip = gretl_strdup(line + 2);
 	} 
 
 	if (bufgets(line, sizeof line, getbuf)) {
+	    tailstrip(line);
 	    version = line + 2;
 	} 
 
@@ -2435,17 +2487,17 @@ gint populate_remote_func_list (windata_t *vwin)
 	err = 1;
     }
 
+    if (!err) {
+	funcpkg_drag_connect(vwin);
+    }
+
     return err;
 }
 
-gint populate_dbfilelist (windata_t *vwin)
+static DIR *open_gretl_db_dir (char *dbdir)
 {
-    GtkListStore *store;
-    GtkTreeIter iter;
-    gchar *dbdir = paths.binbase;
     DIR *dir = NULL;
     int tries = 0;
-    int ndb = 0;
 
     while (dir == NULL) {
 #ifdef G_OS_WIN32 
@@ -2455,28 +2507,43 @@ gint populate_dbfilelist (windata_t *vwin)
 #endif
 	if (dir == NULL) {
 	    errbox(_("Can't open folder %s"), dbdir);
-	    if (++tries == 2 || options_dialog(1)) {
+	    if (++tries == 2 || options_dialog(TAB_DBS, "binbase")) {
 		/* canceled */
-		return 1;
+		return NULL;
 	    }
 	}
     }
 
+    return dir;
+}
+
+gint populate_dbfilelist (windata_t *vwin)
+{
+    GtkListStore *store;
+    GtkTreeIter iter;
+    char *dbdir;
+    DIR *dir = NULL;
+    int ndb = 0;
+
     store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox)));
+    gtk_list_store_clear(store);
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
-    ndb = read_db_files_in_dir(dir, vwin->role, dbdir, store, &iter);
-
-    closedir(dir);
+    /* pick up any databases in the shared gretl dir */
+    dbdir = paths.binbase;
+    dir = open_gretl_db_dir(dbdir);
+    if (dir != NULL) {
+	ndb += read_db_files_in_dir(dir, vwin->role, dbdir, store, &iter);
+	closedir(dir);
+    }
 
     /* pick up any databases in the user's personal dir */
-    dbdir = paths.userdir;
+    dbdir = paths.workdir;
 #ifdef G_OS_WIN32 
     dir = win32_opendir(dbdir);
 #else
     dir = opendir(dbdir);
 #endif
-
     if (dir != NULL) {
 	ndb += read_db_files_in_dir(dir, vwin->role, dbdir, store, &iter);
 	closedir(dir);
