@@ -85,8 +85,26 @@ enum {
     COLL_PS
 };
 
+struct fpkg_response {
+    int col1_width;
+    int try_server;
+};
+
 #define REMOTE_ACTION(c) (c == REMOTE_DB || c == REMOTE_FUNC_FILES)
 
+static void 
+fpkg_response_init (struct fpkg_response *f, GtkWidget *w)
+{
+    f->col1_width = 0;
+
+    if (w != NULL) {
+	/* called from main menu */
+	f->try_server = 0;
+    } else {
+	f->try_server = -1;
+    }
+}
+
 static char *full_path (char *s1, const char *s2)
 {
     static char fpath[FILENAME_MAX];
@@ -111,21 +129,6 @@ static char *full_path (char *s1, const char *s2)
     return fpath;
 }
 
-static char *unslash (const char *s)
-{
-    char *dest = g_strdup(s);
-
-    if (dest != NULL) {
-	size_t n = strlen(dest);
-
-	if (dest[n-1] == '\\' || dest[n-1] == '/') {
-	    dest[n-1] = '\0';
-	}
-    }
-
-    return dest;
-}
-
 static int recognized_collection (file_collection *coll)
 {
     const file_collection recognized_data[] = {
@@ -431,13 +434,26 @@ static int dont_go_there (const char *s)
     return ret;
 }
 
-static int seek_file_collections (const char *topdir)
+static int seek_file_collections (int location)
 {
+    char *tmp = NULL;
     DIR *dir, *try;  
     struct dirent *dirent;
     char *subdir;
-    int err = 0;
-    char *tmp = unslash(topdir);
+    int i = 0, err = 0;
+
+    if (location == DATA_SEARCH) {
+	tmp = g_strdup_printf("%sdata", paths.gretldir);
+    } else if (location == SCRIPT_SEARCH) {
+	tmp = g_strdup_printf("%sscripts", paths.gretldir);
+    } else if (location == USER_SEARCH) {
+	tmp = g_strdup(paths.workdir);
+	trim_slash(tmp);
+    } else {
+	return 1;
+    }
+
+ user_search_2:
 
     dir = opendir(tmp);
     if (dir == NULL) {
@@ -445,7 +461,7 @@ static int seek_file_collections (const char *topdir)
     }
 
 #if COLL_DEBUG
-    fprintf(stderr, "seeking file collections in '%s'\n", topdir);
+    fprintf(stderr, "seeking file collections in '%s'\n", tmp);
 #endif
 
     while (!err && (dirent = readdir(dir))) {
@@ -470,8 +486,14 @@ static int seek_file_collections (const char *topdir)
     }
 
     closedir(dir);
+    g_free(tmp);
 
-    free(tmp);
+    if (location == USER_SEARCH && i++ == 0) {
+	tmp = gretl_default_workdir(&paths);
+	if (tmp != NULL) {
+	    goto user_search_2;
+	}
+    }
 
     return err;
 }
@@ -516,12 +538,12 @@ static int build_file_collections (void)
     static int err;
 
     if (!built) {
-	err = seek_file_collections(paths.datadir);
+	err = seek_file_collections(DATA_SEARCH);
 	if (!err) {
-	    err = seek_file_collections(paths.scriptdir);
+	    err = seek_file_collections(SCRIPT_SEARCH);
 	}
 	if (!err) {
-	    err = seek_file_collections(paths.userdir);
+	    err = seek_file_collections(USER_SEARCH);
 	}
 	if (!err) {
 	    sort_data_stack();
@@ -901,6 +923,24 @@ static int browser_busy (guint code)
     return ret;
 }
 
+windata_t *get_local_viewer (int remote_role)
+{
+    windata_t *vwin = NULL;
+    GtkWidget *w = NULL;
+
+    if (remote_role == REMOTE_DB) {
+	w = get_browser(NATIVE_DB);
+    } else if (remote_role == REMOTE_FUNC_FILES) {
+	w = get_browser(FUNC_FILES);
+    }
+
+    if (w != NULL) {
+	vwin = g_object_get_data(G_OBJECT(w), "vwin");
+    }
+
+    return vwin;
+}
+
 static void new_package_callback (GtkWidget *w , gpointer p)
 {
     file_save(NULL, SAVE_FUNCTIONS, NULL);
@@ -963,6 +1003,52 @@ static void build_funcfiles_popup (windata_t *vwin)
     }
 }
 
+static void build_db_popup (windata_t *vwin)
+{
+    if (vwin->popup == NULL) {
+	vwin->popup = gtk_menu_new();
+
+	if (vwin->role == NATIVE_DB) {
+	    add_popup_item(_("List series"), vwin->popup, 
+			   G_CALLBACK(open_db_index), 
+			   vwin);
+	    add_popup_item(_("Find..."), vwin->popup, 
+			   G_CALLBACK(datafile_find), 
+			   vwin);
+	} else {
+	    add_popup_item(_("List series"), vwin->popup, 
+			   G_CALLBACK(open_remote_db_index), 
+			   vwin);
+	    add_popup_item(_("Install"), vwin->popup, 
+			   G_CALLBACK(install_file_from_server), 
+			   vwin);
+	    add_popup_item(_("Find..."), vwin->popup, 
+			   G_CALLBACK(datafile_find), 
+			   vwin);
+	}
+    }
+}
+
+static void show_server_dbs (GtkWidget *w, gpointer p)
+{
+    display_files(NULL, REMOTE_DB, NULL);
+}
+
+static void show_local_dbs (GtkWidget *w, gpointer p)
+{
+    display_files(NULL, NATIVE_DB, NULL);
+}
+
+static void show_server_funcs (GtkWidget *w, gpointer p)
+{
+    display_files(NULL, REMOTE_FUNC_FILES, NULL);
+}
+
+static void show_local_funcs (GtkWidget *w, gpointer p)
+{
+    display_files(NULL, FUNC_FILES, NULL);
+}
+
 enum {
     BTN_EDIT = 1,
     BTN_INFO,
@@ -971,6 +1057,8 @@ enum {
     BTN_INST,
     BTN_EXEC,
     BTN_DEL,
+    BTN_WWW,
+    BTN_HOME,
     BTN_NEW,
     BTN_FIND,
     BTN_OPEN,
@@ -984,17 +1072,19 @@ struct files_item {
 };
 
 static struct files_item files_items[] = {
-    { N_("Open"),      BTN_OPEN,  GTK_STOCK_OK },
-    { N_("Edit"),      BTN_EDIT,  GTK_STOCK_EDIT },
-    { N_("Info"),      BTN_INFO,  GTK_STOCK_INFO },
-    { N_("View code"), BTN_CODE,  GTK_STOCK_PROPERTIES },
-    { N_("Index"),     BTN_INDX,  GTK_STOCK_INDEX },
-    { N_("Install"),   BTN_INST,  GTK_STOCK_SAVE },
-    { N_("Execute"),   BTN_EXEC,  GTK_STOCK_EXECUTE },
-    { N_("Delete"),    BTN_DEL,   GTK_STOCK_DELETE },
-    { N_("New"),       BTN_NEW,   GTK_STOCK_NEW },
-    { N_("Find"),      BTN_FIND,  GTK_STOCK_FIND },
-    { N_("Close"),     BTN_CLOSE, GTK_STOCK_CLOSE },
+    { N_("Open"),           BTN_OPEN,  GTK_STOCK_OK },
+    { N_("Edit"),           BTN_EDIT,  GTK_STOCK_EDIT },
+    { N_("Info"),           BTN_INFO,  GTK_STOCK_INFO },
+    { N_("View code"),      BTN_CODE,  GTK_STOCK_PROPERTIES },
+    { N_("List series"),    BTN_INDX,  GTK_STOCK_INDEX },
+    { N_("Install"),        BTN_INST,  GTK_STOCK_SAVE },
+    { N_("Execute"),        BTN_EXEC,  GTK_STOCK_EXECUTE },
+    { N_("Delete"),         BTN_DEL,   GTK_STOCK_DELETE },
+    { N_("Look on server"), BTN_WWW,   GTK_STOCK_NETWORK },
+    { N_("Local machine"),  BTN_HOME,  GTK_STOCK_HOME },
+    { N_("New"),            BTN_NEW,   GTK_STOCK_NEW },
+    { N_("Find..."),        BTN_FIND,  GTK_STOCK_FIND },
+    { N_("Close"),          BTN_CLOSE, GTK_STOCK_CLOSE },
     { NULL, 0, NULL }
 };
 
@@ -1029,7 +1119,9 @@ static void make_filesbar (windata_t *vwin)
 		toolfunc = browser_del_func;
 	    } else if (files_items[i].action == BTN_NEW) {
 		toolfunc = new_package_callback;
-	    } 
+	    } else if (files_items[i].action == BTN_WWW) {
+		toolfunc = show_server_funcs;
+	    }
 	} else if (vwin->role == REMOTE_FUNC_FILES) {
 	    if (files_items[i].action == BTN_INFO) {
 		toolfunc = file_info_from_server;
@@ -1037,16 +1129,22 @@ static void make_filesbar (windata_t *vwin)
 		toolfunc = install_file_from_server;
 	    } else if (files_items[i].action == BTN_EXEC) {
 		toolfunc = browser_call_func;
-	    } 
+	    } else if (files_items[i].action == BTN_HOME) {
+		toolfunc = show_local_funcs;
+	    }
 	} else if (vwin->role == NATIVE_DB) {
 	    if (files_items[i].action == BTN_INDX) {
 		toolfunc = open_db_index;
+	    } else if (files_items[i].action == BTN_WWW) {
+		toolfunc = show_server_dbs;
 	    }
 	} else if (vwin->role == REMOTE_DB) {
 	    if (files_items[i].action == BTN_INDX) {
 		toolfunc = open_remote_db_index;
 	    } else if (files_items[i].action == BTN_INST) {
 		toolfunc = install_file_from_server;
+	    } else if (files_items[i].action == BTN_HOME) {
+		toolfunc = show_local_dbs;
 	    }
 	} else if (vwin->role == TEXTBOOK_DATA) {
 	    if (files_items[i].action == BTN_OPEN) {
@@ -1076,14 +1174,106 @@ static void make_filesbar (windata_t *vwin)
     gtk_widget_show(hbox);
 }
 
+static gchar *files_title (int code)
+{
+    static char hname[48];
+    gchar *ret = NULL;
+
+    if (*hname == '\0') {
+#if GTK_MINOR_VERSION >= 8
+	const gchar *s = g_get_host_name();
+
+	if (s != NULL && strlen(s) < 48) {
+	    strcpy(hname, s);
+	} else {
+	    strcpy(hname, _("local machine"));
+	}
+#else
+	strcpy(hname, _("local machine"));
+#endif
+    }
+
+    if (code == NATIVE_DB) {
+	ret = g_strdup_printf(_("gretl: databases on %s"), hname);
+    } else {
+	ret = g_strdup_printf(_("gretl: function packages on %s"), hname);
+    }
+
+    return ret;
+}
+
+static void  
+db_window_handle_drag  (GtkWidget *widget,
+			GdkDragContext *context,
+			gint x,
+			gint y,
+			GtkSelectionData *data,
+			guint info,
+			guint time,
+			gpointer p)
+{
+    /* handle drag of pointer from remote database window */
+    if (info == GRETL_REMOTE_DB_PTR && data != NULL && 
+	data->type == GDK_SELECTION_TYPE_INTEGER) {
+	install_file_from_server(NULL, *(void **) data->data);
+    }
+}
+
+static void  
+pkg_window_handle_drag  (GtkWidget *widget,
+			GdkDragContext *context,
+			gint x,
+			gint y,
+			GtkSelectionData *data,
+			guint info,
+			guint time,
+			gpointer p)
+{
+    /* handle drag of pointer from remote function package window */
+    if (info == GRETL_REMOTE_FNPKG_PTR && data != NULL && 
+	data->type == GDK_SELECTION_TYPE_INTEGER) {
+	install_file_from_server(NULL, *(void **) data->data);
+    }
+}
+
+static void set_up_viewer_drag_target (windata_t *vwin)
+{
+    GCallback callback;
+    int i;
+
+    if (vwin->role == NATIVE_DB) {
+	i = GRETL_REMOTE_DB_PTR;
+	callback = G_CALLBACK(db_window_handle_drag);
+    } else if (vwin->role == FUNC_FILES) {
+	i = GRETL_REMOTE_FNPKG_PTR;
+	callback = G_CALLBACK(pkg_window_handle_drag);
+    } else {
+	return;
+    }
+
+    gtk_drag_dest_set(vwin->listbox,
+		      GTK_DEST_DEFAULT_ALL,
+		      &gretl_drag_targets[i], 1,
+		      GDK_ACTION_COPY);
+
+    g_signal_connect(G_OBJECT(vwin->listbox), "drag_data_received",
+		     callback, NULL);
+}
+
+/* make a browser window to display a set of files: textbook
+   data files, practice scripts, databases...  
+*/
+
 void display_files (gpointer p, guint code, GtkWidget *w)
 {
     GtkWidget *filebox;
     windata_t *vwin;
-    int col1w = 0;
+    struct fpkg_response fresp;
+    gchar *title = NULL;
     int err = 0;
 
     if (browser_busy(code)) {
+	/* an appropriate window is already open */
 	return;
     }
 
@@ -1093,6 +1283,7 @@ void display_files (gpointer p, guint code, GtkWidget *w)
     }
 
     windata_init(vwin);
+    fpkg_response_init(&fresp, w);
 
     vwin->role = code;
     vwin->w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -1103,22 +1294,24 @@ void display_files (gpointer p, guint code, GtkWidget *w)
 
     set_browser_status(vwin, BROWSER_BUSY);
 
+    if (code == FUNC_FILES || code == NATIVE_DB) {
+	title = files_title(code);
+    }
+
     switch (code) {
     case PS_FILES:
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
 			     _("gretl: practice files"));
 	break;
     case FUNC_FILES:
-	gtk_window_set_title(GTK_WINDOW(vwin->w), 
-			     _("gretl: function packages"));
+	gtk_window_set_title(GTK_WINDOW(vwin->w), title);
 	break;
     case TEXTBOOK_DATA:
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
 			     _("gretl: data files"));
 	break;
     case NATIVE_DB:
-	gtk_window_set_title(GTK_WINDOW(vwin->w), 
-			     _("gretl: database files"));
+	gtk_window_set_title(GTK_WINDOW(vwin->w), title);
 	break;
     case REMOTE_DB:
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
@@ -1131,6 +1324,10 @@ void display_files (gpointer p, guint code, GtkWidget *w)
 	break;
     }
 
+    if (title != NULL) {
+	g_free(title);
+    }
+
     /* set up grids */
     vwin->vbox = gtk_vbox_new(FALSE, 1);
     gtk_box_set_spacing(GTK_BOX(vwin->vbox), 4);
@@ -1140,6 +1337,7 @@ void display_files (gpointer p, guint code, GtkWidget *w)
     make_filesbar(vwin);
 
     if (code == TEXTBOOK_DATA || code == PS_FILES) {
+	/* we'll need more than one tab */
 	filebox = files_notebook(vwin, code);
     } else {
 	filebox = files_window(vwin);
@@ -1152,9 +1350,7 @@ void display_files (gpointer p, guint code, GtkWidget *w)
     if (code == TEXTBOOK_DATA) { 
 	file_collection *coll;
 
-	/* create popup menu */
 	build_datafiles_popup(vwin);
-
 	while ((coll = pop_data_collection())) {
 	    g_signal_connect(G_OBJECT(coll->page), "button_press_event",
 			     G_CALLBACK(popup_menu_handler), 
@@ -1166,7 +1362,12 @@ void display_files (gpointer p, guint code, GtkWidget *w)
 	g_signal_connect(G_OBJECT(vwin->listbox), "button_press_event",
 			 G_CALLBACK(popup_menu_handler), 
 			 vwin->popup);
-    }
+    } else if (code == NATIVE_DB || code == REMOTE_DB) {
+	build_db_popup(vwin);
+	g_signal_connect(G_OBJECT(vwin->listbox), "button_press_event",
+			 G_CALLBACK(popup_menu_handler), 
+			 vwin->popup);
+    }	
 
     if (REMOTE_ACTION(code)) {
 	GtkWidget *hbox;
@@ -1182,8 +1383,8 @@ void display_files (gpointer p, guint code, GtkWidget *w)
     if (code == TEXTBOOK_DATA || code == PS_FILES) {
 	err = populate_notebook_filelists(vwin, filebox, code);
     } else if (code == FUNC_FILES) {
-	err = populate_filelist(vwin, &col1w);
-	if (col1w > 15) {
+	err = populate_filelist(vwin, &fresp);
+	if (fresp.col1_width > 15) {
 	    /* widen the file box if need be */
 	    gint w, h;
 
@@ -1200,6 +1401,14 @@ void display_files (gpointer p, guint code, GtkWidget *w)
     } else {
 	gtk_widget_show_all(vwin->w); 
 	gtk_widget_grab_focus(vwin->listbox);
+	if (code == NATIVE_DB || code == FUNC_FILES) {
+	    set_up_viewer_drag_target(vwin);
+	}
+    }
+
+    if (err && code == FUNC_FILES && fresp.try_server == 1) {
+	/* no function packages on local machine */
+	display_files(p, REMOTE_FUNC_FILES, w);
     }
 }
 
@@ -1270,16 +1479,51 @@ read_fn_files_in_dir (int role, DIR *dir, const char *fndir,
     return nfn;
 }
 
-gint populate_func_list (windata_t *vwin, int *wid)
+static gint 
+compare_pkgnames (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
+		  gpointer p)
+{
+    gchar *t1, *t2;
+    gint ret;
+
+    gtk_tree_model_get(model, a, 0, &t1, -1);
+    gtk_tree_model_get(model, b, 0, &t2, -1);
+
+    lower(t1);
+    lower(t2);
+
+    ret = strcmp(t1, t2);
+
+    g_free(t1);
+    g_free(t2);
+
+    return ret;    
+}
+
+static void sort_pkglist (windata_t *vwin)
+{
+    GtkTreeModel *model;
+
+    model = gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox));
+
+    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), 
+					 0, GTK_SORT_ASCENDING);
+    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), 0,
+				    compare_pkgnames, NULL, NULL);
+}
+
+gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
 {
     GtkListStore *store;
     GtkTreeIter iter;
     char fndir[FILENAME_MAX];
+    char *tmp;
     DIR *dir;
     int maxlen = 0;
     int nfn = 0;
 
     store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox)));
+    gtk_list_store_clear(store);
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
     /* pick up any function files in system dir */
@@ -1291,8 +1535,8 @@ gint populate_func_list (windata_t *vwin, int *wid)
 	closedir(dir);
     }
 
-    /* pick up any function files in the user's personal dir */
-    build_path(fndir, paths.userdir, "functions", NULL);
+    /* plus any function files in the user's working dir */
+    build_path(fndir, paths.workdir, "functions", NULL);
     dir = opendir(fndir);
     if (dir != NULL) {
 	nfn += read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
@@ -1300,14 +1544,45 @@ gint populate_func_list (windata_t *vwin, int *wid)
 	closedir(dir);
     }
 
-    if (nfn == 0) {
-	errbox(_("No function files found"));
-	/* FIXME don't leak list store? */
-	return 1;
+    /* plus any function files in the user's personal data dir */
+    build_path(fndir, paths.dotdir, "functions", NULL);
+    dir = opendir(fndir);
+    if (dir != NULL) {
+	nfn += read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
+				    &maxlen);
+	closedir(dir);
     } 
 
-    if (wid != NULL) {
-	*wid = maxlen;
+    /* plus any in the default working dir, if not already searched */
+    tmp = gretl_default_workdir(&paths);
+    if (tmp != NULL) {
+	build_path(fndir, tmp, "functions", NULL);
+	dir = opendir(fndir);
+	if (dir != NULL) {
+	    nfn += read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
+				    &maxlen);
+	    closedir(dir);
+	}
+	free(tmp);
+    }
+
+    if (nfn == 0) {
+	if (fresp->try_server == 0) {
+	    int resp;
+
+	    resp = yes_no_dialog(_("gretl: function packages"),
+				 _("No gretl function packages were found on this computer.\n"
+				   "Do you want to take a look on the gretl server?"),
+				 0);
+	    if (resp == GRETL_YES) {
+		fresp->try_server = 1;
+	    }
+	} else {
+	    warnbox(_("No gretl function packages were found on this computer."));
+	}
+	return 1;
+    } else {
+	sort_pkglist(vwin);
     }
 
     return 0;
