@@ -22,10 +22,11 @@
 
 #include <stdarg.h>
 
+#define GRETL_CANCEL (-1)
+
 enum {
     GRETL_YES,
     GRETL_NO,
-    GRETL_CANCEL,
     HELP_BUTTON
 } buttons;
 
@@ -55,7 +56,7 @@ void menu_exit_check (void);
 
 double gui_double_from_string (const char *str, int *err);
 
-void delimiter_dialog (gretlopt *optp);
+int delimiter_dialog (gretlopt *optp);
 
 void varinfo_dialog (int varnum, int full);
 
@@ -90,6 +91,10 @@ int radio_dialog_with_spinner (const char *title, const char **opts,
 			       int *spinvar, const char *spintxt,
 			       int spinmin, int spinmax);
 
+int radio_dialog_with_check (const char *title, const char *label, 
+			     const char **opts, int nopts, int deflt, int hcode,
+			     int *checkvar, const char *checktxt);
+
 int checks_dialog (const char *title, const char *blurb,
 		   const char **opts, int nopts, 
 		   int *active, int nradios, int *rvar,
@@ -110,11 +115,12 @@ int add_obs_dialog (const char *blurb, int addmin);
 int forecast_dialog (int t1min, int t1max, int *t1, 
 		     int t2min, int t2max, int *t2,
 		     int pmin, int pmax, int *p,
-		     int dyn, MODEL *pmod);
+		     int dyn, gretlopt *optp,
+		     MODEL *pmod);
 
 int freq_dialog (const char *title, const char *blurb,
 		 int *nbins, int nbmax, double *f0, double *fwid,
-		 double xmin, double xmax);
+		 double xmin, double xmax, int *dist, int plot);
 
 void bootstrap_dialog (windata_t *vwin, int *pp, int *pB,
 		       gretlopt *popt, int *cancelled);
