@@ -26,6 +26,35 @@
 
 #include "system.h"
 
+GtkWidget *windata_get_toplevel (windata_t *vwin)
+{
+    GtkWidget *top = NULL;
+
+    if (vwin->w != NULL) {
+	top = gtk_widget_get_toplevel(vwin->w);
+    } else if (vwin->dialog != NULL) {
+	top = gtk_widget_get_toplevel(vwin->dialog);
+    }
+
+    return top;
+}
+
+void set_window_busy (windata_t *vwin)
+{
+    vwin->flags |= VWIN_BUSY;
+    if (vwin->mbar != NULL) {
+	gtk_widget_set_sensitive(vwin->mbar, FALSE);
+    }
+}
+
+void unset_window_busy (windata_t *vwin)
+{
+    vwin->flags &= ~VWIN_BUSY;
+    if (vwin->mbar != NULL) {
+	gtk_widget_set_sensitive(vwin->mbar, TRUE);
+    }
+}
+
 dialog_opts *dialog_opts_new (int n, int type, 
 			      gretlopt *optp,
 			      const gretlopt *vals,
@@ -84,37 +113,49 @@ GtkWidget *context_help_button (GtkWidget *hbox, int cmdcode)
 
 static void set_canceled (GtkWidget *w, int *c)
 {
-    if (c != NULL) *c = 1;
+    if (c != NULL) {
+	*c = 1;
+    }
+}
+
+static void maybe_set_canceled (GtkDialog *d, int resp, int *c)
+{
+    if (resp == GTK_RESPONSE_DELETE_EVENT) {
+	*c = 1;
+    }
 }
 
 GtkWidget *cancel_delete_button (GtkWidget *hbox, GtkWidget *targ,
 				 int *canceled)
 {
-    GtkWidget *w;
+    GtkWidget *button;
 
-    w = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-    GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
+    button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
     if (canceled != NULL) {
-	g_signal_connect(G_OBJECT(w), "clicked", 
+	g_signal_connect(G_OBJECT(button), "clicked", 
 			 G_CALLBACK(set_canceled), 
 			 canceled);
+	g_signal_connect(GTK_DIALOG(targ), "response", 
+			 G_CALLBACK(maybe_set_canceled), 
+			 canceled);
     }
-    g_signal_connect(G_OBJECT(w), "clicked", 
+    g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(delete_widget), 
 		     targ);
 	
-    gtk_widget_show(w);
+    gtk_widget_show(button);
 
-    return w;
+    return button;
 }
 
-static void opt_invalid (GtkWidget *w, int *opt)
+static void invalidate_opt (GtkWidget *w, int *opt)
 {
     *opt = -1;
 }
 
-static void maybe_opt_invalid (GtkDialog *d, int resp, int *opt)
+static void maybe_invalidate_opt (GtkDialog *d, int resp, int *opt)
 {
     if (resp == GTK_RESPONSE_NONE || resp == GTK_RESPONSE_DELETE_EVENT) {
 	*opt = -1;
@@ -124,26 +165,25 @@ static void maybe_opt_invalid (GtkDialog *d, int resp, int *opt)
 GtkWidget *cancel_options_button (GtkWidget *hbox, GtkWidget *targ,
 				  int *opt)
 {
-    GtkWidget *w = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+    GtkWidget *button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
 
-    GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
+    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
 
     if (opt != NULL) {
-	g_signal_connect(G_OBJECT(w), "clicked", 
-			 G_CALLBACK(opt_invalid), 
+	g_signal_connect(G_OBJECT(button), "clicked", 
+			 G_CALLBACK(invalidate_opt), 
 			 opt);
 	g_signal_connect(GTK_DIALOG(targ), "response", 
-			 G_CALLBACK(maybe_opt_invalid), 
+			 G_CALLBACK(maybe_invalidate_opt), 
 			 opt);
     }
-
-    g_signal_connect(G_OBJECT(w), "clicked", 
+    g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(delete_widget), 
 		     targ);
-    gtk_widget_show(w);
+    gtk_widget_show(button);
 
-    return w;
+    return button;
 }
 
 GtkWidget *ok_button (GtkWidget *hbox)
@@ -224,7 +264,7 @@ int maybe_raise_dialog (void)
     int ret = 0;
 
     if (current_dialog != NULL) {
-	gdk_window_raise(current_dialog->window);
+	gtk_window_present(GTK_WINDOW(current_dialog));
 	ret = 1;
     }
 
@@ -411,10 +451,10 @@ static int dlg_text_set_gmm_skel (dialog_t *d)
 {
     const char *skel = 
 	"# initializations go here\n\n\n"
-	"gmm\n\n"
-	"orthog\n"
-	"weights\n"
-	"params\n"
+	"gmm\n"
+	"  orthog\n"
+	"  weights\n"
+	"  params\n"
 	"end gmm\n";
 
     if (d->code == GMM && edit_save_buf == NULL) {
@@ -461,7 +501,7 @@ const gchar *edit_dialog_get_text (dialog_t *dlg)
     buf = gtk_entry_get_text(GTK_ENTRY(dlg->edit));
 
     if (buf == NULL || *buf == '\0') {
-	if (dlg->code != CORRGM && dlg->code != CREATE_DATASET) {
+	if (dlg->code != CORRGM) {
 	    gtk_widget_destroy(dlg->dialog);
 	}
 	return NULL;
@@ -700,24 +740,52 @@ static void sample_replace_buttons (GtkWidget *box, gpointer data)
     gtk_widget_show(tmp);
 }
 
-static void free_sys_strings (GtkWidget *w, char **strs)
+static void set_sys_method (GtkComboBox *box, dialog_t *d)
 {
-    free_strings_array(strs,  SYS_METHOD_MAX - 1); 
+    gchar *str = gtk_combo_box_get_active_text(box);
+    char *s, mstr[8] = {0};
+
+    s = strrchr(str, '(');
+
+    if (s != NULL) {
+	GtkWidget *bt, *bv;
+
+	sscanf(s + 1, "%7[^)]", mstr);
+	d->opt = system_method_from_string(mstr);
+
+	bt = g_object_get_data(G_OBJECT(box), "bt");
+	bv = g_object_get_data(G_OBJECT(box), "bv");
+
+	if (d->opt == 0 || d->opt == 1 || d->opt == 6) {
+	    /* SUR, 3SLS, WLS */
+	    gtk_widget_set_sensitive(bt, TRUE);
+	    gtk_widget_set_sensitive(bv, TRUE);
+	} else if (d->opt == 2) {
+	    /* FIML */
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bt), TRUE);
+	    gtk_widget_set_sensitive(bt, FALSE);
+	    gtk_widget_set_sensitive(bv, TRUE);
+	} else {
+	    /* LIML, OLS, TSLS */
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bt), FALSE);
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bt), FALSE);
+	    gtk_widget_set_sensitive(bt, FALSE);
+	    gtk_widget_set_sensitive(bv, FALSE);
+	} 
+    }
+
+    g_free(str);
 }
 
-static void set_sys_method (GtkEditable *entry, dialog_t *d)
+static gboolean opt_t_callback (GtkWidget *w, dialog_t *dlg)
 {
-    const gchar *s = gtk_entry_get_text(GTK_ENTRY(entry));
-    
-    if (*s != '\0') {
-	char mstr[8] = {0};
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
+	dlg->opt |= OPT_T;
+    } else {
+	dlg->opt &= ~OPT_T;
+    }
 
-	s = strrchr(s, '(');
-	if (s != NULL) {
-	    sscanf(s + 1, "%7[^)]", mstr);
-	    d->opt = system_method_from_string(mstr);
-	}
-    } 
+    return FALSE;
 }
 
 static gboolean opt_v_callback (GtkWidget *w, dialog_t *dlg)
@@ -764,12 +832,16 @@ static gboolean opt_f_callback (GtkWidget *w, dialog_t *dlg)
     return FALSE;
 }
 
-static void dialog_option_switch (GtkWidget *vbox, dialog_t *dlg,
-				  gretlopt opt)
+static GtkWidget *dialog_option_switch (GtkWidget *vbox, dialog_t *dlg,
+					gretlopt opt)
 {
-    GtkWidget *b, *hbox;
+    GtkWidget *b = NULL;
 
-    if (opt == OPT_V) {
+    if (opt == OPT_T) {
+	b = gtk_check_button_new_with_label(_("Iterated estimation"));
+	g_signal_connect(G_OBJECT(b), "toggled", 
+			 G_CALLBACK(opt_t_callback), dlg);
+    } else if (opt == OPT_V) {
 	b = gtk_check_button_new_with_label(_("Show details of iterations"));
 	g_signal_connect(G_OBJECT(b), "toggled", 
 			 G_CALLBACK(opt_v_callback), dlg);
@@ -785,62 +857,64 @@ static void dialog_option_switch (GtkWidget *vbox, dialog_t *dlg,
 	b = gtk_check_button_new_with_label(_("Show full restricted estimates"));
 	g_signal_connect(G_OBJECT(b), "toggled", 
 			 G_CALLBACK(opt_f_callback), dlg);
-    } else {
-	return;
+    } 
+
+    if (b != NULL) {
+	GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
+
+	gtk_box_pack_start(GTK_BOX(hbox), b, TRUE, TRUE, 5);
+	gtk_widget_show(b);
+
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
     }
 
-    hbox = gtk_hbox_new(FALSE, 5);
-    gtk_box_pack_start(GTK_BOX(hbox), b, TRUE, TRUE, 5);
-    gtk_widget_show(b);
-
-    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-    gtk_widget_show(hbox);
+    return b;
 }
 
-static void combo_opt_changed (GtkWidget *w, combo_opts *opts)
+static void combo_opt_changed (GtkComboBox *box, combo_opts *opts)
 {
-    const char *s = gtk_entry_get_text(GTK_ENTRY(w));
+    gchar *s = gtk_combo_box_get_active_text(box);
     int i;
 
-    if (s != NULL && *s != '\0') {
-        for (i=0; opts->strs[i] != NULL; i++) {
-            if (!strcmp(s, _(opts->strs[i]))) {
-                *opts->optp |= opts->vals[i];
-            } else {
-		*opts->optp &= ~opts->vals[i];
-	    }
-        }
+    for (i=0; opts->strs[i] != NULL; i++) {
+	if (!strcmp(s, _(opts->strs[i]))) {
+	    *opts->optp |= opts->vals[i];
+	} else {
+	    *opts->optp &= ~opts->vals[i];
+	}
     }
+
+    g_free(s);
+}
+
+GtkWidget *gretl_opts_combo_full (combo_opts *opts, int deflt, 
+				  GCallback callback,
+				  gpointer calldata)
+{
+    GtkWidget *combo;
+    int i;
+
+    *opts->optp |= opts->vals[deflt];
+
+    combo = gtk_combo_box_new_text();
+    for (i=0; opts->strs[i] != NULL; i++) {
+	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _(opts->strs[i]));
+    }
+    gtk_combo_box_set_active(GTK_COMBO_BOX(combo), deflt);
+    g_signal_connect(G_OBJECT(combo), "changed",
+		     G_CALLBACK(combo_opt_changed), opts);
+    if (callback != NULL) {
+	g_signal_connect(G_OBJECT(combo), "changed",
+			 callback, calldata);
+    }
+
+    return combo;
 }
 
 GtkWidget *gretl_opts_combo (combo_opts *opts, int deflt)
 {
-    GtkWidget *combo;
-    GList *optlist = NULL;
-    int i, ni, n = 0;
-
-    for (i=0; opts->strs[i] != NULL; i++) {
-	optlist = g_list_append(optlist, _(opts->strs[i]));
-	ni = strlen(_(opts->strs[i]));
-	if (ni > n) {
-	    n = ni;
-	}
-    } 
-
-    *opts->optp |= opts->vals[deflt];
-
-    combo = gtk_combo_new();
-    gtk_combo_set_popdown_strings(GTK_COMBO(combo), optlist); 
-    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), 
-		       _(opts->strs[deflt]));
-    gtk_entry_set_width_chars(GTK_ENTRY(GTK_COMBO(combo)->entry), n);
-    gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(combo)->entry), 
-			      FALSE);
-    g_signal_connect(G_OBJECT(GTK_COMBO(combo)->entry), "changed",
-		     G_CALLBACK(combo_opt_changed), opts);
-    g_list_free(optlist);
-
-    return combo;
+    return gretl_opts_combo_full(opts, deflt, NULL, NULL);
 }
 
 static void build_gmm_combo (GtkWidget *vbox, dialog_t *d)
@@ -874,30 +948,19 @@ static void build_gmm_combo (GtkWidget *vbox, dialog_t *d)
     gtk_widget_show(hbox);      
 }
 
-static void system_estimator_list (GtkWidget *vbox, dialog_t *d)
+static void system_estimator_list (GtkWidget *vbox, dialog_t *d,
+				   GtkWidget *bt, GtkWidget *bv)
 {
     equation_system *sys = NULL;
-    GList *items = NULL;
     GtkWidget *w, *hbox;
-    gchar **strs;
-    int method = -1;
+    gchar *str;
+    int method = 0;
     int i;
 
     if (d->data != NULL) {
 	sys = (equation_system *) d->data;
     }
 
-    strs = strings_array_new(SYS_METHOD_MAX);
-
-    for (i=SYS_METHOD_SUR; i<SYS_METHOD_MAX; i++) {
-	strs[i] = g_strdup_printf("%s (%s)", _(system_method_full_string(i)),
-				  system_method_short_string(i));
-	items = g_list_append(items, strs[i]);
-	if (sys != NULL && sys->method == i) {
-	    method = i;
-	}
-    }
-
     hbox = gtk_hbox_new(FALSE, 5);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
     gtk_widget_show(hbox);
@@ -906,17 +969,25 @@ static void system_estimator_list (GtkWidget *vbox, dialog_t *d)
     gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
     gtk_widget_show(w);
 
-    w = gtk_combo_new();
-    gtk_combo_set_popdown_strings(GTK_COMBO(w), items); 
-    gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(w)->entry), FALSE);
-    g_signal_connect(G_OBJECT(w), "destroy", G_CALLBACK(free_sys_strings), strs);
-    g_signal_connect(G_OBJECT(GTK_COMBO(w)->entry), "changed",
-		     G_CALLBACK(set_sys_method), d);
+    w = gtk_combo_box_new_text();
 
-    if (method >= 0) {
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(w)->entry), strs[method]);
+    for (i=SYS_METHOD_SUR; i<SYS_METHOD_MAX; i++) {
+	str = g_strdup_printf("%s (%s)", _(system_method_full_string(i)),
+			      system_method_short_string(i));
+	gtk_combo_box_append_text(GTK_COMBO_BOX(w), str);
+	g_free(str);
+	if (sys != NULL && sys->method == i) {
+	    method = i;
+	}
     }
 
+    gtk_combo_box_set_active(GTK_COMBO_BOX(w), method);
+
+    g_object_set_data(G_OBJECT(w), "bt", bt);
+    g_object_set_data(G_OBJECT(w), "bv", bv);
+    g_signal_connect(G_OBJECT(w), "changed",
+		     G_CALLBACK(set_sys_method), d);
+
     gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);
     gtk_widget_show(w);  
 }
@@ -954,8 +1025,7 @@ static int edit_dialog_help_code (int ci, void *p)
 
     if (ci == SYSTEM && p != NULL) {
 	hc = 0;
-    } else if (ci == PRINT || ci == CREATE_USERDIR || 
-	       ci == CREATE_DATASET || ci == MINIBUF) {
+    } else if (ci == PRINT || ci == CREATE_DATASET || ci == MINIBUF) {
 	hc = 0;
     } else if (ci == RESTRICT) {
 	windata_t *vwin = (windata_t *) p;
@@ -983,6 +1053,15 @@ static void edit_dialog_ok (GtkWidget *w, dialog_t *d)
     gtk_widget_destroy(d->dialog);
 }
 
+static gboolean cancel_vwin_edit (GtkWidget *w, gpointer p)
+{
+    if (open_edit_dialog != NULL) {
+	gtk_widget_destroy(open_edit_dialog);
+    }
+
+    return FALSE;
+}
+
 static int ols_model_window (windata_t *vwin)
 {
     if (vwin->role == VIEW_MODEL) {
@@ -1013,7 +1092,7 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
     int clear = 0;
 
     if (open_edit_dialog != NULL && cmdcode != MINIBUF) {
-	gdk_window_raise(open_edit_dialog->window);
+	gtk_window_present(GTK_WINDOW(open_edit_dialog));
 	return;
     }
 
@@ -1094,7 +1173,11 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
     if (cmdcode == SMPLBOOL && dataset_is_restricted()) {
 	sample_replace_buttons(top_vbox, d);
     } else if (cmdcode == SYSTEM) {
-	system_estimator_list(top_vbox, d);
+	GtkWidget *bt, *bv;
+
+	bt = dialog_option_switch(top_vbox, d, OPT_T);
+	bv = dialog_option_switch(top_vbox, d, OPT_V);
+	system_estimator_list(top_vbox, d, bt, bv);
     } else if (cmdcode == NLS || cmdcode == MLE) {
 	dialog_option_switch(top_vbox, d, OPT_V);
 	dialog_option_switch(top_vbox, d, OPT_R);
@@ -1129,7 +1212,7 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
     }    
 
     /* "Cancel" button? */
-    if (cmdcode != CREATE_USERDIR) {
+    if (canceled != NULL) {
 	cancel_delete_button(button_box, d->dialog, canceled);
     }
 
@@ -1141,7 +1224,8 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
     } else {
 	g_signal_connect(G_OBJECT(w), "clicked", 
 			 G_CALLBACK(edit_dialog_ok), d);
-    }	
+    }
+	
     gtk_widget_grab_default(w);
     gtk_widget_show(w);    
 
@@ -1152,6 +1236,17 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 	modal = 0;
     }
 
+    /* schedule destruction */
+    if (cmdcode == MODEL_GENR || cmdcode == RESTRICT) {
+	GtkWidget *w = windata_get_toplevel(okptr);
+
+	if (w != NULL) {
+	    g_signal_connect(G_OBJECT(w), "destroy",
+			     G_CALLBACK(cancel_vwin_edit), 
+			     NULL);
+	}
+    }
+
     gtk_window_set_destroy_with_parent(GTK_WINDOW(d->dialog), TRUE);
 
     gtk_widget_show(d->dialog); 
