@@ -39,6 +39,12 @@ struct combo_opts_ {
 
 typedef struct combo_opts_ combo_opts;
 
+GtkWidget *windata_get_toplevel (windata_t *vwin);
+
+void set_window_busy (windata_t *vwin);
+
+void unset_window_busy (windata_t *vwin);
+
 GtkWidget *gretl_dialog_new (const char *title, GtkWidget *parent,
 			     unsigned char flags);
 
@@ -85,6 +91,10 @@ char *entry_box_get_trimmed_text (GtkWidget *w);
 
 GtkWidget *gretl_opts_combo (combo_opts *opts, int deflt);
 
+GtkWidget *gretl_opts_combo_full (combo_opts *opts, int deflt, 
+				  GCallback callback,
+				  gpointer calldata);
+
 dialog_opts *dialog_opts_new (int n, int type, 
 			      gretlopt *optp,
 			      const gretlopt *vals,
