@@ -41,7 +41,7 @@
                               i == OPEN_ASCII || \
                               i == OPEN_OCTAVE || \
                               i == OPEN_GNUMERIC || \
-	                      i == OPEN_EXCEL || \
+	                      i == OPEN_XLS || \
                               i == OPEN_WF1 || \
                               i == OPEN_DTA || \
                               i == OPEN_JMULTI || \
@@ -51,7 +51,7 @@
                                i == APPEND_CSV || \
                                i == APPEND_OCTAVE || \
                                i == APPEND_GNUMERIC || \
-                               i == APPEND_EXCEL || \
+                               i == APPEND_XLS || \
                                i == APPEND_ASCII || \
                                i == APPEND_WF1 || \
                                i == APPEND_DTA || \
@@ -75,6 +75,8 @@
                        i == OPEN_DATA || \
                        i == APPEND_DATA)
 
+#define SET_DIR_ACTION(i) (i == SET_DIR || i == SET_WDIR)
+
 struct extmap {
     int action;
     char *ext;
@@ -83,9 +85,11 @@ struct extmap {
 static struct extmap action_map[] = {
     { SAVE_DBDATA,       ".bin" },
     { SAVE_SCRIPT,       ".inp" },
+    { SAVE_FUNCTIONS_AS, ".inp" },
     { SAVE_CONSOLE,      ".inp" },
     { SAVE_SESSION,      ".gretl" },
     { SAVE_GP_CMDS,      ".plt" },
+    { SAVE_R_CMDS,       ".R" },
     { SAVE_BOXPLOT_EPS,  ".eps" },
     { SAVE_BOXPLOT_PS,   ".ps" },
     { SAVE_BOXPLOT_XPM,  ".xpm" },
@@ -99,6 +103,7 @@ static struct extmap action_map[] = {
     { SAVE_OUTPUT,       ".txt" },
     { SAVE_TEX,          ".tex" },
     { SAVE_RTF,          ".rtf" },
+    { SAVE_TEXT,         ".txt" },
     { OPEN_SCRIPT,       ".inp" },
     { OPEN_SESSION,      ".gretl" },
     { OPEN_CSV,          ".csv" },
@@ -107,8 +112,8 @@ static struct extmap action_map[] = {
     { APPEND_ASCII,      ".txt" },
     { OPEN_GNUMERIC,     ".gnumeric" },
     { APPEND_GNUMERIC,   ".gnumeric" },
-    { OPEN_EXCEL,        ".xls" },
-    { APPEND_EXCEL,      ".xls" },
+    { OPEN_XLS,          ".xls" },
+    { APPEND_XLS,        ".xls" },
     { OPEN_WF1,          ".wf1" },
     { APPEND_WF1,        ".wf1" },
     { OPEN_DTA,          ".dta" },
@@ -207,7 +212,6 @@ static const char *get_ext (int action, gpointer data)
 
 static int check_maybe_add_ext (char *fname, int action, gpointer data)
 {
-    FILE *fp;
     const char *ext = NULL;
 
     if (fname == NULL) {
@@ -216,16 +220,9 @@ static int check_maybe_add_ext (char *fname, int action, gpointer data)
 
     /* don't mess if the fname is really a dir */
     if (isdir(fname)) {
-	return (action != SET_DIR);
+	return !SET_DIR_ACTION(action);
     }
 
-    /* don't mess with the name of a previously existing file */
-    fp = gretl_fopen(fname, "r");
-    if (fp != NULL && fgetc(fp) != EOF) {
-	fclose(fp);
-	return 0;
-    }    
-
     /* don't mess with a filename that already has an extension */
     if (dotpos(fname) != strlen(fname)) {
 	return 0;
@@ -314,7 +311,9 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
 	strcpy(scriptfile, fname);
 	mkfilelist(FILE_LIST_SCRIPT, scriptfile);
 	script_window_update(vwin, fname);
-    }
+    } else if (action == SAVE_GP_CMDS || action == SAVE_R_CMDS) {
+	script_window_update(vwin, fname);
+    } 
 }
 
 static void set_startdir (char *startdir, int action)
@@ -344,32 +343,36 @@ static void filesel_save_prn_buffer (PRN *prn, const char *fname)
 
 static void filesel_open_script (const char *fname)
 {
-    int spos;
-
-    strcpy(tryfile, fname);
-
-    if (view_file(tryfile, 1, 0, 78, 370, EDIT_SCRIPT) != NULL) {
-	strcpy(scriptfile, tryfile);
-	mkfilelist(FILE_LIST_SCRIPT, scriptfile);
-	spos = slashpos(scriptfile);
-	if (spos) {
-	    paths.currdir[0] = 0;
-	    strncat(paths.currdir, scriptfile, spos + 1);
+    if (has_suffix(fname, ".R")) {
+	view_file(fname, 1, 0, 78, 370, EDIT_R);
+    } else if (has_suffix(fname, ".plt")) {
+	view_file(fname, 1, 0, 78, 370, EDIT_GP);	
+    } else {
+	strcpy(tryfile, fname);
+	if (view_file(tryfile, 1, 0, 78, 370, EDIT_SCRIPT) != NULL) {
+	    strcpy(scriptfile, tryfile);
+	    mkfilelist(FILE_LIST_SCRIPT, scriptfile);
+	    set_currdir_from_filename(scriptfile);
 	}
     }
 }
 
 static void filesel_open_session (const char *fname)
 {
+    strcpy(tryfile, fname);
+
     if (is_session_file(fname)) {
-	strcpy(tryfile, fname);
 	verify_open_session();
     } else {
-	int pub;
+	windata_t *vwin;
 
-	strcpy(tryfile, fname);
-	pub = !strncmp(tryfile, paths.scriptdir, strlen(paths.scriptdir));
-	if (view_file(tryfile, 1, 0, 78, 370, (pub)? VIEW_SCRIPT : EDIT_SCRIPT)) {
+	if (has_system_prefix(tryfile, &paths, SCRIPT_SEARCH)) {
+	    vwin = view_file(tryfile, 0, 0, 78, 370, VIEW_SCRIPT);
+	} else {
+	    vwin = view_file(tryfile, 1, 0, 78, 370, EDIT_SCRIPT);
+	}
+
+	if (vwin != NULL) {
 	    strcpy(scriptfile, tryfile);
 	}
     }
@@ -524,12 +527,14 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 	save_session(fname);
     } else if (action == SAVE_FUNCTIONS) {
 	save_user_functions(fname, data);
+    } else if (action == SAVE_FUNCTIONS_AS) {
+	save_user_functions_as_script(fname, data);
     } else if (action == SAVE_BOOT_DATA) {
 	bootstrap_save_callback(fname);
     } else if (action == SET_PROG || action == SET_DIR) {
-	char *strvar = (char *) data;
-
-	filesel_set_path_callback(fname, strvar);
+	set_path_callback(data, fname);
+    } else if (action == SET_WDIR) {
+	set_working_dir_callback(data, fname);
     } else {
 	windata_t *vwin = (windata_t *) data;
 
@@ -593,46 +598,49 @@ static struct winfilter get_gp_filter (int ttype)
 static struct winfilter get_filter (int action, gpointer data)
 {
     static struct win32_filtermap map[] = {
-	{ SAVE_DATA,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
-	{ SAVE_DBDATA,  { N_("gretl database files (*.bin)"), "*.bin" }},
-	{ SAVE_SCRIPT,  { N_("gretl script files (*.inp)"), "*.inp" }},
-	{ SAVE_CONSOLE, { N_("gretl command files (*.inp)"), "*.inp" }},
-	{ SAVE_SESSION, { N_("session files (*.gretl)"), "*.gretl" }},
+	{ SAVE_DATA,        { N_("gretl data files (*.gdt)"), "*.gdt" }},
+	{ SAVE_DBDATA,      { N_("gretl database files (*.bin)"), "*.bin" }},
+	{ SAVE_SCRIPT,      { N_("gretl script files (*.inp)"), "*.inp" }},
+	{ SAVE_FUNCTIONS_AS,{ N_("gretl script files (*.inp)"), "*.inp" }},
+	{ SAVE_CONSOLE,     { N_("gretl command files (*.inp)"), "*.inp" }},
+	{ SAVE_SESSION,     { N_("session files (*.gretl)"), "*.gretl" }},
 	{ SAVE_BOXPLOT_EPS, { N_("postscript files (*.eps)"), "*.eps" }},
 	{ SAVE_BOXPLOT_PS,  { N_("postscript files (*.ps)"), "*.ps" }},
-	{ SAVE_GP_CMDS, { N_("gnuplot files (*.plt)"), "*.plt" }},
+	{ SAVE_GP_CMDS,     { N_("gnuplot files (*.plt)"), "*.plt" }},
+	{ SAVE_R_CMDS,      { N_("GNU R files (*.R)"), "*.R" }},
 	{ SAVE_FUNCTIONS,   { N_("gretl function files (*.gfn)"), "*.gfn" }},
-	{ EXPORT_CSV,   { N_("CSV files (*.csv)"), "*.csv" }},
-	{ EXPORT_R,     { N_("GNU R files (*.R)"), "*.R" }},
-	{ EXPORT_OCTAVE, { N_("GNU Octave files (*.m)"), "*.m" }},
-	{ OPEN_OCTAVE,  { N_("GNU Octave files (*.m)"), "*.m" }},
-	{ APPEND_OCTAVE, { N_("GNU Octave files (*.m)"), "*.m" }},
-	{ EXPORT_DAT,   { N_("PcGive files (*.dat)"), "*.dat" }},
-	{ EXPORT_JM,    { N_("JMulti files (*.dat)"), "*.dat" }},
-	{ SAVE_OUTPUT,  { N_("text files (*.txt)"), "*.txt" }},
-	{ SAVE_TEX,     { N_("TeX files (*.tex)"), "*.tex" }},
-	{ SAVE_RTF,     { N_("RTF files (*.rtf)"), "*.rtf" }},
-	{ OPEN_SCRIPT,  { N_("gretl script files (*.inp)"), "*.inp" }},
-	{ OPEN_SESSION, { N_("session files (*.gretl)"), "*.gretl" }},
-	{ OPEN_CSV,     { N_("CSV files (*.csv)"), "*.csv" }},
-	{ APPEND_CSV,   { N_("CSV files (*.csv)"), "*.csv" }},
-	{ OPEN_ASCII,   { N_("ASCII files (*.txt)"), "*.txt" }},
-	{ APPEND_ASCII, { N_("ASCII files (*.txt)"), "*.txt" }},
-	{ OPEN_GNUMERIC,   { N_("Gnumeric files (*.gnumeric)"), "*.gnumeric" }},
-	{ APPEND_GNUMERIC, { N_("Gnumeric files (*.gnumeric)"), "*.gnumeric" }},
-	{ OPEN_EXCEL,   { N_("Excel files (*.xls)"), "*.xls" }},
-	{ APPEND_EXCEL, { N_("Excel files (*.xls)"), "*.xls" }},
-	{ OPEN_WF1,     { N_("Eviews workfiles (*.wf1)"), "*.wf1" }},
-	{ APPEND_WF1,   { N_("Eviews workfiles (*.wf1)"), "*.wf1" }},
-	{ OPEN_DTA,     { N_("Stata files (*.dta)"), "*.dta" }},
-	{ APPEND_DTA,   { N_("Stata files (*.dta)"), "*.dta" }},
-	{ OPEN_JMULTI,  { N_("JMulTi files (*.dat)"), "*.dat" }},
-	{ APPEND_JMULTI,{ N_("JMulTi files (*.dat)"), "*.dat" }},
-	{ OPEN_ODS,     { N_("Open Document Spreadsheet (*.ods)"), "*.ods" }},
-	{ APPEND_ODS,   { N_("Open Document Spreadsheet (*.ods)"), "*.ods" }},
-	{ OPEN_RATS_DB, { N_("RATS databases (*.rat)"), "*.rat" }},
-	{ OPEN_PCGIVE_DB, { N_("PcGive data files (*.bn7)"), "*.bn7" }},
-	{ SET_PROG,     { N_("program files (*.exe)"), "*.exe" }}
+	{ EXPORT_CSV,       { N_("CSV files (*.csv)"), "*.csv" }},
+	{ EXPORT_R,         { N_("GNU R files (*.R)"), "*.R" }},
+	{ EXPORT_OCTAVE,    { N_("GNU Octave files (*.m)"), "*.m" }},
+	{ OPEN_OCTAVE,      { N_("GNU Octave files (*.m)"), "*.m" }},
+	{ APPEND_OCTAVE,    { N_("GNU Octave files (*.m)"), "*.m" }},
+	{ EXPORT_DAT,       { N_("PcGive files (*.dat)"), "*.dat" }},
+	{ EXPORT_JM,        { N_("JMulti files (*.dat)"), "*.dat" }},
+	{ SAVE_OUTPUT,      { N_("text files (*.txt)"), "*.txt" }},
+	{ SAVE_TEX,         { N_("TeX files (*.tex)"), "*.tex" }},
+	{ SAVE_RTF,         { N_("RTF files (*.rtf)"), "*.rtf" }},
+	{ SAVE_TEXT,        { N_("ASCII files (*.txt)"), "*.txt" }},
+	{ OPEN_SCRIPT,      { N_("gretl script files (*.inp)"), "*.inp" }},
+	{ OPEN_SESSION,     { N_("session files (*.gretl)"), "*.gretl" }},
+	{ OPEN_CSV,         { N_("CSV files (*.csv)"), "*.csv" }},
+	{ APPEND_CSV,       { N_("CSV files (*.csv)"), "*.csv" }},
+	{ OPEN_ASCII,       { N_("ASCII files (*.txt)"), "*.txt" }},
+	{ APPEND_ASCII,     { N_("ASCII files (*.txt)"), "*.txt" }},
+	{ OPEN_GNUMERIC,    { N_("Gnumeric files (*.gnumeric)"), "*.gnumeric" }},
+	{ APPEND_GNUMERIC,  { N_("Gnumeric files (*.gnumeric)"), "*.gnumeric" }},
+	{ OPEN_XLS,         { N_("Excel files (*.xls)"), "*.xls" }},
+	{ APPEND_XLS,       { N_("Excel files (*.xls)"), "*.xls" }},
+	{ OPEN_WF1,         { N_("Eviews workfiles (*.wf1)"), "*.wf1" }},
+	{ APPEND_WF1,       { N_("Eviews workfiles (*.wf1)"), "*.wf1" }},
+	{ OPEN_DTA,         { N_("Stata files (*.dta)"), "*.dta" }},
+	{ APPEND_DTA,       { N_("Stata files (*.dta)"), "*.dta" }},
+	{ OPEN_JMULTI,      { N_("JMulTi files (*.dat)"), "*.dat" }},
+	{ APPEND_JMULTI,    { N_("JMulTi files (*.dat)"), "*.dat" }},
+	{ OPEN_ODS,         { N_("Open Document Spreadsheet (*.ods)"), "*.ods" }},
+	{ APPEND_ODS,       { N_("Open Document Spreadsheet (*.ods)"), "*.ods" }},
+	{ OPEN_RATS_DB,     { N_("RATS databases (*.rat)"), "*.rat" }},
+	{ OPEN_PCGIVE_DB,   { N_("PcGive data files (*.bn7)"), "*.bn7" }},
+	{ SET_PROG,         { N_("program files (*.exe)"), "*.exe" }}
     };
     static struct winfilter default_filter = {
 	N_("all files (*.*)"), "*.*" 
@@ -663,7 +671,7 @@ static char *make_winfilter (int action, gpointer data)
     struct winfilter filter;
     char *ret, *p;
 
-    ret = calloc(128, 1);
+    ret = calloc(256, 1);
     if (ret == NULL) {
 	return NULL;
     }
@@ -680,6 +688,17 @@ static char *make_winfilter (int action, gpointer data)
     p += strlen(p) + 1;
     strcpy(p, filter.pat);
 
+    if (action == OPEN_SCRIPT) {
+	p += strlen(p) + 1;
+	strcpy(p, I_("gnuplot files (*.plt)"));
+	p += strlen(p) + 1;
+	strcpy(p, "*.plt");
+	p += strlen(p) + 1;
+	strcpy(p, I_("GNU R files (*.*)"));
+	p += strlen(p) + 1;
+	strcpy(p, "*.R");
+    }	
+
     if (strncmp(filter.descrip, "all", 3)) {
 	p += strlen(p) + 1;
 	strcpy(p, I_("all files (*.*)"));
@@ -703,7 +722,7 @@ static int select_dirname (char *fname, char *trmsg)
     bi.pidlRoot = NULL; /* FIXME? */
     bi.pszDisplayName = dirname;
     bi.lpszTitle = trmsg;
-    bi.ulFlags = 0;
+    bi.ulFlags = BIF_USENEWUI;
     bi.lpfn = NULL;
     bi.lParam = 0;
     bi.iImage = 0;   
@@ -719,7 +738,8 @@ static int select_dirname (char *fname, char *trmsg)
     return ret;
 }
 
-void file_selector (const char *msg, int action, FselDataSrc src, gpointer data) 
+static void win32_file_selector (const char *msg, int action, FselDataSrc src, 
+				 gpointer data) 
 {
     OPENFILENAME of;
     int retval;
@@ -755,8 +775,8 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 		strcpy(fname, strvar + slashpos(strvar) + 1);
 	    } 
 	}
-    } else if (action == SAVE_FUNCTIONS) {
-	get_default_package_name(fname, data);
+    } else if (action == SAVE_FUNCTIONS || action == SAVE_FUNCTIONS_AS) {
+	get_default_package_name(fname, data, action);
     }
 
     if (doing_nls()) {
@@ -765,7 +785,7 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 	trmsg = g_strdup(msg);
     }
 
-    if (action == SET_DIR) {
+    if (SET_DIR_ACTION(action)) {
 	retval = select_dirname(fname, trmsg);
     } else {
 	/* initialize file dialog info struct */
@@ -838,9 +858,9 @@ static GtkFileFilter *get_file_filter (int action, gpointer data)
     return filter;
 }
 
-void file_selector (const char *msg, int action, FselDataSrc src, gpointer data) 
+static void gtk_file_selector (const char *msg, int action, FselDataSrc src, 
+			       gpointer data, GtkWidget *parent) 
 {
-    GtkWindow *parent = GTK_WINDOW(mdata->w);
     GtkWidget *filesel;
     char startdir[MAXLEN];
     GtkFileFilter *filter;
@@ -849,7 +869,7 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 
     set_startdir(startdir, action);
 
-    if (action == SET_DIR) {
+    if (SET_DIR_ACTION(action)) {
 	fa = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
 	okstr = GTK_STOCK_OK;
     } else if (action == SET_PROG) {
@@ -868,6 +888,10 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
        in particular when action == SAVE_GNUPLOT
     */
 
+    if (parent == NULL) {
+	parent = mdata->w;
+    }
+
     filesel = gtk_file_chooser_dialog_new(msg, GTK_WINDOW(parent), fa,
 					  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					  okstr, GTK_RESPONSE_ACCEPT,
@@ -876,10 +900,14 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
     gtk_dialog_set_default_response(GTK_DIALOG(filesel), GTK_RESPONSE_ACCEPT);
 
     filter = get_file_filter(action, data);
+
     if (action == OPEN_ASCII || action == APPEND_ASCII) {
 	gtk_file_filter_set_name(filter, _("ASCII files (*.txt)"));
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
-    }
+    } else if (action == OPEN_SCRIPT) {
+	gtk_file_filter_set_name(filter, _("gretl script files (*.inp)"));
+	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
+    }	
 
     gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(filesel), filter);
 
@@ -888,7 +916,16 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 	gtk_file_filter_set_name(filter, _("all files (*.*)"));
 	gtk_file_filter_add_pattern(filter, "*.*");
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
-    }
+    } else if (action == OPEN_SCRIPT) {
+	filter = gtk_file_filter_new();
+	gtk_file_filter_set_name(filter, _("GNU R files (*.R)"));
+	gtk_file_filter_add_pattern(filter, "*.R");
+	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
+	filter = gtk_file_filter_new();
+	gtk_file_filter_set_name(filter, _("gnuplot files (*.plt)"));
+	gtk_file_filter_add_pattern(filter, "*.plt");
+	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
+    }    
 
     /* FIXME session dir */
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), startdir);
@@ -924,14 +961,14 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 	    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), 
 					  strvar);
 	} 
-    } else if (action == SAVE_FUNCTIONS) {
+    } else if (action == SAVE_FUNCTIONS || action == SAVE_FUNCTIONS_AS) {
 	char fname[MAXLEN];
 
 	*fname = '\0';
-	get_default_package_name(fname, data);
+	get_default_package_name(fname, data, action);
 	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
 					  fname);
-    }	
+    } 
 
     if (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_ACCEPT) {
 	char *fname;
@@ -946,3 +983,22 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 }
 
 #endif /* end of non-MS Windows code */
+
+void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
+{
+#ifdef G_OS_WIN32
+    win32_file_selector(msg, action, src, data);
+#else
+    gtk_file_selector(msg, action, src, data, NULL);
+#endif
+}
+
+void file_selector_with_parent (const char *msg, int action, FselDataSrc src, 
+				gpointer data, GtkWidget *w)
+{
+#ifdef G_OS_WIN32
+    win32_file_selector(msg, action, src, data);
+#else
+    gtk_file_selector(msg, action, src, data, w);
+#endif
+}
