@@ -577,32 +577,26 @@ static void function_call_dialog (call_info *cinfo)
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cinfo->dlg)->vbox),
 			   tbl, FALSE, FALSE, 5);
 
-	label = gtk_label_new(_("name"));
-	gtk_table_attach(GTK_TABLE(tbl), label, 0, 1, 0, 1,
-			 GTK_EXPAND, GTK_FILL, 5, 5);
-	gtk_widget_show(label);
-
-	label = gtk_label_new(_("type"));
-	gtk_table_attach(GTK_TABLE(tbl), label, 1, 2, 0, 1,
-			 GTK_EXPAND, GTK_FILL, 5, 5);
-	gtk_widget_show(label);
-
 	label = gtk_label_new(_("selection"));
 	gtk_table_attach(GTK_TABLE(tbl), label, 2, 3, 0, 1,
 			 GTK_EXPAND, GTK_FILL, 5, 5);
 	gtk_widget_show(label);
 
 	for (i=0; i<cinfo->n_params; i++) {
-	    const char *pname = fn_param_name(cinfo->func, i);
+	    const char *desc = fn_param_descrip(cinfo->func, i);
 	    int ptype = fn_param_type(cinfo->func, i);
-	    
-	    label = gtk_label_new(pname);
-	    gtk_table_attach(GTK_TABLE(tbl), label, 0, 1, i+1, i+2,
-			     GTK_EXPAND, GTK_FILL, 5, 5);
-	    gtk_widget_show(label);
 
-	    label = gtk_label_new(arg_type_string(ptype));
-	    gtk_table_attach(GTK_TABLE(tbl), label, 1, 2, i+1, i+2,
+	    if (desc != NULL) {
+		label = gtk_label_new(desc);
+	    } else {
+		gchar *tmp = g_strdup_printf("%s (%s)",
+					     fn_param_name(cinfo->func, i), 
+					     arg_type_string(ptype));
+		label = gtk_label_new(tmp);
+		g_free(tmp);			     
+	    }
+
+	    gtk_table_attach(GTK_TABLE(tbl), label, 0, 2, i+1, i+2,
 			     GTK_EXPAND, GTK_FILL, 5, 5);
 	    gtk_widget_show(label);
 
@@ -756,7 +750,7 @@ static int temp_install_remote_fnpkg (const char *fname, char *target)
 {
     int err = 0;
 
-    build_path(target, paths.userdir, "dltmp", NULL);
+    build_path(target, paths.dotdir, "dltmp", NULL);
     err = gretl_tempname(target);
     if (err) {
 	return err;
@@ -840,7 +834,7 @@ static int needs_quoting (call_info *cinfo, int i)
     char *s = cinfo->args[i];
 
     return (t == GRETL_TYPE_STRING && 
-	    get_named_string(s) == NULL &&
+	    get_string_by_name(s) == NULL &&
 	    *s != '"');
 }
 
@@ -1015,8 +1009,8 @@ void call_function_package (const char *fname, GtkWidget *w,
     gretl_exec_state_init(&state, SCRIPT_EXEC, fnline, get_lib_cmd(),
 			  models, prn);
 
-    err = gui_exec_line(&state, &Z, &datainfo);
-    view_buffer(prn, 80, 400, fnname, SCRIPT_OUT, NULL);
+    err = gui_exec_line(&state, &Z, datainfo);
+    view_buffer(prn, 80, 400, fnname, PRINT, NULL);
 
     if (datainfo->v > orig_v) {
 	mark_dataset_as_modified();
