@@ -171,7 +171,7 @@ static void login_finalize (GtkWidget *w, login_info *linfo)
 
 /* for use in File, Save dialog */
 
-void get_default_package_name (char *fname, gpointer p)
+void get_default_package_name (char *fname, gpointer p, int mode)
 {
     function_info *finfo = (function_info *) p;
     const char *pubname;
@@ -181,7 +181,11 @@ void get_default_package_name (char *fname, gpointer p)
 
     if (pubname != NULL) {
 	strcpy(fname, pubname);
-	strcat(fname, ".gfn");	
+	if (mode == SAVE_FUNCTIONS_AS) {
+	    strcat(fname, ".inp");
+	} else {
+	    strcat(fname, ".gfn");
+	}	
     }
 }
 
@@ -353,13 +357,34 @@ static void edit_code_callback (GtkWidget *w, function_info *finfo)
 		       EDIT_FUNC_CODE, GINT_TO_POINTER(q));
 
     if (vwin != NULL) {
-	build_path(vwin->fname, paths.userdir, "pkgedit", NULL);
+	build_path(vwin->fname, paths.dotdir, "pkgedit", NULL);
 	gretl_tempname(vwin->fname);
 	g_object_set_data(G_OBJECT(vwin->w), "iface", 
 			  GINT_TO_POINTER(finfo->iface));
     }
 }
 
+static void gfn_to_script_callback (GtkWidget *w, function_info *finfo)
+{
+    int n = 0;
+
+    if (finfo->pub >= 0) {
+	n = 1;
+    }
+
+    if (finfo->privlist != NULL) {
+	n += finfo->privlist[0];
+    }
+
+    if (n == 0) {
+	warnbox("No code to save");
+	return;
+    }
+
+    file_selector(_("Save function package"), SAVE_FUNCTIONS_AS, 
+		  FSEL_DATA_MISC, finfo);
+}
+
 static GtkWidget *label_hbox (GtkWidget *w, const char *txt)
 {
     GtkWidget *hbox, *label;
@@ -687,6 +712,11 @@ static void finfo_dialog (function_info *finfo)
 	finfo->iface = finfo->pub;
     }
 
+    button = gtk_button_new_with_label(_("Save as script"));
+    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 5);
+    g_signal_connect(G_OBJECT(button), "clicked", 
+		     G_CALLBACK(gfn_to_script_callback), finfo);
+
     gtk_widget_show_all(hbox);
 
     /* check box for upload option */
@@ -873,6 +903,7 @@ static void do_upload (const char *fname)
     GdkCursor *cursor;
     GdkWindow *w1;
     gint x, y;
+    int error_printed = 0;
     int err = 0;
 
     login_dialog(&linfo);
@@ -884,17 +915,19 @@ static void do_upload (const char *fname)
 
     /* set waiting cursor */
     disp = gdk_display_get_default();
-    cursor = gdk_cursor_new(GDK_WATCH);
     w1 = gdk_display_get_window_at_pointer(disp, &x, &y);
-    gdk_window_set_cursor(w1, cursor);
-    gdk_display_sync(disp);
-    gdk_cursor_unref(cursor);
+    if (w1 != NULL) {
+	cursor = gdk_cursor_new(GDK_WATCH);
+	gdk_window_set_cursor(w1, cursor);
+	gdk_display_sync(disp);
+	gdk_cursor_unref(cursor);
+    }
 
-    g_file_get_contents(fname, &buf, NULL, NULL);
+    err = gretl_file_get_contents(fname, &buf);
 
-    if (buf == NULL) {
-	err = E_ALLOC;
-    } else {  
+    if (err) {
+	error_printed = 1;
+    } else {
 	ubuf = url_encode_string(buf);
 	ulogin = url_encode_string(linfo.login);
 	upass = url_encode_string(linfo.pass);
@@ -911,10 +944,14 @@ static void do_upload (const char *fname)
     }
 
     /* reset default cursor */
-    gdk_window_set_cursor(w1, NULL);
+    if (w1 != NULL) {
+	gdk_window_set_cursor(w1, NULL);
+    }
 
     if (err) {
-	gui_errmsg(err);
+	if (!error_printed) {
+	    gui_errmsg(err);
+	}
     } else if (retbuf != NULL && *retbuf != '\0') {
 	infobox(retbuf);
     }
@@ -1027,6 +1064,39 @@ int save_user_functions (const char *fname, gpointer p)
     return err;
 }
 
+int save_user_functions_as_script (const char *fname, gpointer p)
+{
+    function_info *finfo = p;
+    PRN *prn;
+    int i, err = 0;
+
+    prn = gretl_print_new_with_filename(fname, &err);
+    if (err) {
+	file_write_errbox(fname);
+	return err;
+    }
+
+    pprintf(prn, "# author='%s'\n", finfo->author);
+    pprintf(prn, "# version='%s'\n", finfo->version);
+    pprintf(prn, "# date='%s'\n", finfo->date);
+
+    if (finfo->privlist != NULL) {
+	for (i=1; i<=finfo->privlist[0]; i++) {
+	    pputc(prn, '\n');
+	    gretl_function_print_code(finfo->privlist[i], prn);
+	}
+    }
+
+    if (finfo->pub >= 0) {
+	pputc(prn, '\n');
+	gretl_function_print_code(finfo->pub, prn);
+    }
+
+    gretl_print_destroy(prn);
+
+    return 0;
+}
+
 /* called from function selection dialog: a set of functions has been
    selected and now we need to add info on author, version, etc.
 */
@@ -1126,3 +1196,23 @@ void edit_function_package (const char *fname, int *loaderr)
 
     finfo_dialog(finfo);
 }
+
+int no_user_functions_check (void)
+{
+    int err = 0;
+
+    if (n_free_functions() == 0) {
+	int resp;
+
+	err = 1;
+	resp = yes_no_dialog(_("gretl: function packages"),
+			     _("No functions are available for packaging at present.\n"
+			       "Do you want to write a function now?"),
+			     0);
+	if (resp == GRETL_YES) {
+	    do_new_script(NULL, FUNC, NULL);
+	}
+    } 
+
+    return err;
+}
