@@ -28,7 +28,7 @@ int remove_png_term_from_plotfile_by_name (const char *fname);
 
 void display_session_graph_png (const char *pltname);
 
-void gnuplot_show_png_by_name (const char *fname);
+int gnuplot_show_png_by_name (const char *fname);
 
 void plot_label_position_click (GtkWidget *w, png_plot *plot);
 
@@ -42,6 +42,10 @@ int plot_is_mouseable (const png_plot *plot);
 
 GtkWidget *plot_get_shell (png_plot *plot);
 
+double plot_get_xmin (png_plot *plot);
+
+double plot_get_ymin (png_plot *plot);
+
 GPT_SPEC *plot_get_spec (png_plot *plot);
 
 void revise_distribution_plotspec (png_plot *plot, int d, int df1, int df2);
@@ -50,4 +54,12 @@ int gp_term_code (gpointer p);
 
 void save_graph_to_file (gpointer p, const char *fname);
 
+void filter_gnuplot_file (int ttype, int latin, int mono, int recolor, 
+			  FILE *fpin, FILE *fpout);
+
+void run_gp_script (gchar *buf);
+
+int dump_plot_buffer (const char *buf, const char *fname,
+		      int addpause);
+
 #endif /* GPT_CONTROL_H */
