@@ -25,6 +25,7 @@
 #include "session.h"
 #include "dlgutils.h"
 #include "fileselect.h"
+#include "calculator.h"
 
 #include "../pixmaps/mouse.xpm"
 
@@ -39,6 +40,7 @@ struct gpt_range_t {
     GtkWidget *isauto;
     GtkWidget *min;
     GtkWidget *max;
+    GtkWidget *lbase;
 };
 
 static GtkWidget **linetitle;
@@ -54,14 +56,13 @@ static GtkWidget *labelpos[MAX_PLOT_LABELS];
 static GtkWidget *gpt_control;
 static GtkWidget *keycombo;
 static GtkWidget *fitcombo;
-static GtkWidget *termcombo;
 static GtkWidget *border_check;
 static GtkWidget *markers_check;
 static GtkWidget *y2_check;
 static GtkWidget *ttfcombo;
 static GtkWidget *ttfspin;
 
-GtkWidget *filesavebutton;
+static int gui_nlines;
 
 #define MAX_AXES 3
 
@@ -110,47 +111,41 @@ static void close_plot_controller (GtkWidget *widget, gpointer data)
 
 static void flip_manual_range (GtkWidget *widget, gpointer data)
 {
-    gint axis = GPOINTER_TO_INT(data);
+    gint i = GPOINTER_TO_INT(data);
+    gboolean s = GTK_TOGGLE_BUTTON(axis_range[i].isauto)->active;
 
-    if (GTK_TOGGLE_BUTTON (axis_range[axis].isauto)->active) {
-	gtk_widget_set_sensitive(GTK_WIDGET(axis_range[axis].min), FALSE);
-	gtk_widget_set_sensitive(GTK_WIDGET(axis_range[axis].max), FALSE);
-    } else {
-	gtk_widget_set_sensitive(GTK_WIDGET(axis_range[axis].min), TRUE);
-	gtk_widget_set_sensitive(GTK_WIDGET(axis_range[axis].max), TRUE);
-    }
+    gtk_widget_set_sensitive(GTK_WIDGET(axis_range[i].min), !s);
+    gtk_widget_set_sensitive(GTK_WIDGET(axis_range[i].max), !s);
 }
 
-/* Take text from a gtkentry and write to gnuplot spec string.
-   Under gtk2, the entries will be in utf-8, and have to be converted
-   to the locale for use with gnuplot.
-*/
+static void disable_lbase (GtkWidget *b, GtkWidget *entry)
+{
+    gboolean s = GTK_TOGGLE_BUTTON(b)->active;
+
+    gtk_widget_set_sensitive(entry, !s);
+}
+
+static void enable_lbase (GtkWidget *b, GtkWidget *entry)
+{
+    gboolean s = GTK_TOGGLE_BUTTON(b)->active;
+
+    gtk_widget_set_sensitive(entry, s);
+}
+
+/* Take text from a gtkentry and write to gnuplot spec string */
 
 static void entry_to_gp_string (GtkWidget *w, char *targ, size_t n)
 {
-    const gchar *wstr;
-    
+    const gchar *str;
+
     *targ = '\0';
 
     g_return_if_fail(GTK_IS_ENTRY(w));
-    wstr = gtk_entry_get_text(GTK_ENTRY(w));
 
-#ifdef ENABLE_NLS
-    if (wstr != NULL && *wstr != '\0') {
-	gchar *trstr = gp_locale_from_utf8(wstr);
-
-	if (trstr != NULL) {
-	    strncat(targ, trstr, n-1);
-	    g_free(trstr);
-	} else {
-	    strncat(targ, wstr, n-1);
-	}
+    str = gtk_entry_get_text(GTK_ENTRY(w));
+    if (str != NULL || *str != '\0') {
+	strncat(targ, str, n-1);
     }
-#else
-    if (wstr != NULL && *wstr != '\0') {
-	strncat(targ, wstr, n-1);
-    }
-#endif
 }
 
 static FitType fit_type_from_string (const char *s)
@@ -170,14 +165,13 @@ static FitType fit_type_from_string (const char *s)
     return f;
 }
 
-static void fittype_from_combo (GtkWidget *w, GPT_SPEC *spec)
+static void fittype_from_combo (GtkComboBox *box, GPT_SPEC *spec)
 {
-    FitType f = PLOT_FIT_NONE;
-    const char *s;
+    gchar *s = gtk_combo_box_get_active_text(box);
+    FitType f;
 
-    g_return_if_fail(GTK_IS_ENTRY(w));
-    s = gtk_entry_get_text(GTK_ENTRY(w));
     f = fit_type_from_string(s);
+    g_free(s);
 
     if (f == PLOT_FIT_OLS || f == PLOT_FIT_QUADRATIC || 
 	f == PLOT_FIT_INVERSE || f == PLOT_FIT_LOESS) {
@@ -191,37 +185,39 @@ static void fittype_from_combo (GtkWidget *w, GPT_SPEC *spec)
     }
 }
 
-static gboolean fit_type_changed (GtkEditable *entry, GPT_SPEC *spec)
+static gboolean fit_type_changed (GtkComboBox *box, GPT_SPEC *spec)
 {
-    const gchar *s = gtk_entry_get_text(GTK_ENTRY(entry));
     const char *s1 = spec->yvarname;
     const char *s2 = spec->xvarname;
-    char title[128];
-    FitType f;
+    gchar *s, *title = NULL;
+    FitType f = PLOT_FIT_NONE;
 
     if (*s1 == '\0' || *s2 == '\0') {
 	return FALSE;
     }
 
+    s = gtk_combo_box_get_active_text(box);
     f = fit_type_from_string(s);
-
-    *title = '\0';
+    g_free(s);
 
     if (f == PLOT_FIT_OLS) {
-	sprintf(title, _("%s versus %s (with least squares fit)"),
+	title = g_strdup_printf(_("%s versus %s (with least squares fit)"),
 		s1, s2);
     } else if (f == PLOT_FIT_QUADRATIC) {
-	sprintf(title, _("%s versus %s (with quadratic fit)"),
+	title = g_strdup_printf(_("%s versus %s (with quadratic fit)"),
 		s1, s2);
     } else if (f == PLOT_FIT_INVERSE) {
-	sprintf(title, _("%s versus %s (with inverse fit)"),
+	title = g_strdup_printf(_("%s versus %s (with inverse fit)"),
 		s1, s2);
     } else if (f == PLOT_FIT_LOESS) {
-	sprintf(title, _("%s versus %s (with loess fit)"),
+	title = g_strdup_printf(_("%s versus %s (with loess fit)"),
 		s1, s2);
     }
 
-    gtk_entry_set_text(GTK_ENTRY(gpt_titles[0].widget), title);
+    if (title != NULL) {
+	gtk_entry_set_text(GTK_ENTRY(gpt_titles[0].widget), title);
+	g_free(title);
+    }
     
     return FALSE;
 }
@@ -253,68 +249,36 @@ static double entry_to_gp_double (GtkWidget *w)
     if (w != NULL && GTK_IS_ENTRY(w)) {
 	const gchar *s = gtk_entry_get_text(GTK_ENTRY(w));
 
-	if (s != NULL && *s != '\0') {
 #ifdef ENABLE_NLS
+	if (s != NULL && *s != '\0') {
 	    gchar *tmp = g_strdup(s);
 
 	    charsub(tmp, ',', '.');
-	    gretl_push_c_numeric_locale();
-#else
-	    const gchar *tmp = s;
-#endif
+	    gretl_push_c_numeric_locale();	    
 	    if (check_atof(tmp)) {
 		errbox(gretl_errmsg_get());
 	    } else {
 		ret = atof(tmp);
 	    }
-#ifdef ENABLE_NLS
 	    gretl_pop_c_numeric_locale();
 	    g_free(tmp);
-#endif
 	}
+#else
+	if (s != NULL && *s != '\0') {
+	    if (check_atof(s)) {
+		errbox(gretl_errmsg_get());
+	    } else {
+		ret = atof(s);
+	    }
+	}
+#endif
     }
 
     return ret;
 }
 
-/* Take text from a gnuplot spec string and put it into a gtkentry.
-   We have to ensure that the text is put into utf-8.
-*/
-
-static void gp_string_to_entry (GtkWidget *w, const char *str)
-{
-#ifdef ENABLE_NLS
-    int lv = iso_latin_version();
-    int pngterm = gnuplot_png_terminal();
-#endif
-    gchar *trstr = NULL;
-
-    if (*str == '\0') {
-	gtk_entry_set_text(GTK_ENTRY(w), str);
-	return;
-    }
-
-
-#ifdef ENABLE_NLS
-    if (lv == 2 && pngterm < GP_PNG_CAIRO) {
-	char lstr[MAXTITLE];
-	
-	sprint_html_to_l2(lstr, str);
-	trstr = latin2_to_utf8(lstr);
-    } else if (!g_utf8_validate(str, -1, NULL)) {
-	trstr = latin1_to_utf8(str);
-    } else {
-	trstr = g_strdup(str);
-    }
-#else
-    trstr = g_strdup(str);
-#endif
-
-    if (trstr != NULL) {
-	gtk_entry_set_text(GTK_ENTRY(w), trstr);
-	g_free(trstr);
-    }    
-}
+#define gp_string_to_entry(w,s) do { \
+	gtk_entry_set_text(GTK_ENTRY(w),s); } while (0)
 
 static int
 get_label_pos_from_entry (GtkWidget *w, double *pos)
@@ -354,24 +318,35 @@ static int validate_range (double *r)
     return err;
 }
 
-static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec) 
+static int 
+set_logscale_from_entry (GPT_SPEC *spec, int i, GtkWidget *entry)
 {
-    const gchar *yaxis;
-    int supress_y2 = 0;
-    int i, k, fsave = 0;
+    double base;
     int err = 0;
 
-    /* entry_to_gp_string translates from utf-8 to the locale, if
-       using NLS */
-
-    if (widget == filesavebutton) {
-	entry_to_gp_string(GTK_COMBO(termcombo)->entry, spec->termtype, 
-			   sizeof spec->termtype);
-	if (strcmp(spec->termtype, "screen")) {
-	    fsave = 1;
-	}
+    base = entry_get_numeric_value(entry, C_POS_DBL);
+    if (na(base)) {
+	err = 1;
+    } else if (!na(base) && base < 1.1) {
+	err = 1;
+	errbox("bad base");
+    } else {
+	spec->logbase[i] = base;
     }
-   
+
+    return err;
+}
+
+enum {
+    ERRORBARS = 1,
+    FILLEDCURVE
+};
+
+static void apply_gpt_changes (GtkWidget *w, GPT_SPEC *spec) 
+{
+    int supress_y2 = 0;
+    int i, k, err = 0;
+
     for (i=0; i<NTITLES; i++) {
 	if (gpt_titles[i].widget != NULL) {
 	    entry_to_gp_string(gpt_titles[i].widget, spec->titles[i], 
@@ -379,29 +354,32 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 	}
     }
 
-    entry_to_gp_string(GTK_COMBO(keycombo)->entry, spec->keyspec, 
-		       sizeof spec->keyspec);
+    if (keycombo != NULL) {
+	gchar *s = gtk_combo_box_get_active_text(GTK_COMBO_BOX(keycombo));
+
+	strcpy(spec->keyspec, s);
+	g_free(s);
+    }
 
     spec->flags &= ~GPT_Y2AXIS;
 
-    if (y2_check != NULL) {
-	if (GTK_TOGGLE_BUTTON(y2_check)->active) {
-	    supress_y2 = 1;
-	} 
+    if (y2_check != NULL && GTK_TOGGLE_BUTTON(y2_check)->active) {
+	supress_y2 = 1;
     } 
 
-    for (i=0; i<spec->n_lines; i++) {
+    for (i=0; i<gui_nlines; i++) {
 	spec->lines[i].yaxis = 1;
-	if (supress_y2 || yaxiscombo[i] == NULL) {
-	    continue;
-	}
-	yaxis = 
-	    gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(yaxiscombo[i])->entry));
-	if (yaxis != NULL && *yaxis && !strcmp(yaxis, "right")) {
-	    spec->lines[i].yaxis = 2;	
-	}
-	if (spec->lines[i].yaxis == 2) {
-	    spec->flags |= GPT_Y2AXIS;
+	if (!supress_y2 && yaxiscombo[i] != NULL) {
+	    gchar *s = 
+		gtk_combo_box_get_active_text(GTK_COMBO_BOX(yaxiscombo[i]));
+
+	    if (!strcmp(s, "right")) {
+		spec->lines[i].yaxis = 2;	
+	    }
+	    if (spec->lines[i].yaxis == 2) {
+		spec->flags |= GPT_Y2AXIS;
+	    }
+	    g_free(s);
 	}
     }
 
@@ -418,15 +396,38 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 		    err = validate_range(spec->range[i]);
 		}
 	    }
+	    if (axis_range[i].lbase != NULL) {
+		if (GTK_WIDGET_SENSITIVE(axis_range[i].lbase)) {
+		    err = set_logscale_from_entry(spec, i, axis_range[i].lbase);
+		} else {
+		    spec->logbase[i] = 0.0;
+		}
+	    }
 	}
     }
 
     if (!err) {   
-	for (i=0; i<spec->n_lines; i++) {
+	for (i=0; i<gui_nlines; i++) {
 	    if (stylecombo[i] != NULL) {
+		int oldalt = 0;
+
+		if (!strncmp(spec->lines[i].style, "filled", 6)) {
+		    oldalt = FILLEDCURVE;
+		} else if (!strncmp(spec->lines[i].style, "error", 5)) {
+		    oldalt = ERRORBARS;
+		}
 		entry_to_gp_string(GTK_COMBO(stylecombo[i])->entry, 
 				   spec->lines[i].style, 
 				   sizeof spec->lines[0].style);
+		if (oldalt == FILLEDCURVE &&
+		    !strncmp(spec->lines[i].style, "error", 5)) {
+		    spec->flags |= GPT_ERR_SWITCH;
+		    spec->flags &= ~GPT_FILL_SWITCH;
+		} else if (oldalt == ERRORBARS &&
+			   !strncmp(spec->lines[i].style, "filled", 6)) {
+		    spec->flags |= GPT_FILL_SWITCH;
+		    spec->flags &= ~GPT_ERR_SWITCH;
+		}
 	    }
 	    if (linetitle[i] != NULL) {
 		entry_to_gp_string(linetitle[i], 
@@ -494,23 +495,19 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 	}
     }
 
-    if (fitcombo != NULL) {
-	fittype_from_combo(GTK_COMBO(fitcombo)->entry, spec);
+    if (!err && fitcombo != NULL) {
+	fittype_from_combo(GTK_COMBO_BOX(fitcombo), spec);
     }
 
     if (!err) {
-	if (fsave) { 
-	    /* saving to file */
-	    file_selector(_("Save gnuplot graph"), SAVE_GNUPLOT, 
-			  FSEL_DATA_MISC, spec);
-	} else { 
-	    png_plot *plot = (png_plot *) spec->ptr;
+	png_plot *plot = (png_plot *) spec->ptr;
 
-	    set_plot_has_y2_axis(plot, spec->flags & GPT_Y2AXIS);
-	    redisplay_edited_plot(plot);
-	}
+	set_plot_has_y2_axis(plot, spec->flags & GPT_Y2AXIS);
+	redisplay_edited_plot(plot);
 	mark_session_changed();
     }
+
+    spec->flags &= ~(GPT_FILL_SWITCH | GPT_ERR_SWITCH);
 }
 
 static void set_keyspec_sensitivity (GPT_SPEC *spec)
@@ -519,7 +516,7 @@ static void set_keyspec_sensitivity (GPT_SPEC *spec)
     const char *p;
     int i;
 
-    for (i=0; i<spec->n_lines; i++) {
+    for (i=0; i<gui_nlines; i++) {
 	p = gtk_entry_get_text(GTK_ENTRY(linetitle[i]));
 	if (p != NULL && *p != 0) {
 	    state = TRUE;
@@ -567,7 +564,7 @@ static const char *get_font_filename (const char *showname)
 static int font_is_ok (const char *fname)
 {
     static int pngterm;
-    char cmd[64];
+    gchar *cmd;
     int err;
 
     if (pngterm == 0) {
@@ -575,12 +572,13 @@ static int font_is_ok (const char *fname)
     }
 
     if (pngterm == GP_PNG_CAIRO) {
-	sprintf(cmd, "set term pngcairo font \"%s,10\"", fname);
+	cmd = g_strdup_printf("set term pngcairo font \"%s,10\"", fname);
     } else {
-	sprintf(cmd, "set term png font %s 10", fname);
+	cmd = g_strdup_printf("set term png font %s 10", fname);
     }
 
     err = gnuplot_test_command(cmd);
+    g_free(cmd);
 
     return err == 0;
 }
@@ -656,7 +654,7 @@ static void toggle_axis_selection (GtkWidget *w, GPT_SPEC *spec)
     int no_y2 = GTK_TOGGLE_BUTTON(w)->active;
     int i;
 
-    for (i=0; i<spec->n_lines; i++) {
+    for (i=0; i<gui_nlines; i++) {
 	if (yaxiscombo[i] != NULL) {
 	    gtk_widget_set_sensitive(yaxiscombo[i], !no_y2);
 	}
@@ -752,14 +750,6 @@ static void add_color_selector (int i, GtkWidget *tbl, int *rows,
     }
 }
 
-#if 0
-static void gp_commands_window (GtkWidget *w, GPT_SPEC *spec)
-{
-    remove_png_term_from_plotfile_by_name(spec->fname);
-    view_file(spec->fname, 1, 0, 78, 400, GR_PLOT);
-}
-#endif
-
 /* PNG anti-aliasing switch */
 
 static void set_aa_status (GtkWidget *w, int *ok)
@@ -769,13 +759,51 @@ static void set_aa_status (GtkWidget *w, int *ok)
     gnuplot_png_set_use_aa(*ok);
 }
 
+static GtkWidget *gp_dialog_table (int rows, int cols, 
+				   GtkWidget *vbox)
+{
+    GtkWidget *tbl;
+
+    tbl = gtk_table_new(rows, cols, FALSE);
+    gtk_table_set_row_spacings(GTK_TABLE(tbl), 5);
+    gtk_table_set_col_spacings(GTK_TABLE(tbl), 5);
+    gtk_box_pack_start(GTK_BOX(vbox), tbl, FALSE, FALSE, 0);
+
+    return tbl;
+}
+
+static GtkWidget *gp_dialog_vbox (void)
+{
+    GtkWidget *vbox;
+
+    vbox = gtk_vbox_new(FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
+
+    return vbox;
+}
+
+static GtkWidget *gp_page_vbox (GtkWidget *notebook, char *str)
+{
+    GtkWidget *vbox;
+    GtkWidget *label;
+
+    vbox = gtk_vbox_new(FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
+    gtk_widget_show(vbox);
+    label = gtk_label_new(str);
+    gtk_widget_show(label);
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label); 
+
+    return vbox;
+}
+
 static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec) 
 {
     static int aa_ok = 1;
     static int show_aa_check = -1;
     GtkWidget *label, *vbox, *tbl;
     int i, rows = 1;
-    GList *keypos_list = NULL;
+    int kactive = 0;
     gchar *keypos[] = {
 	"left top",
 	"right top",
@@ -786,22 +814,9 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 	NULL
     };
 
-    for (i=0; keypos[i] != NULL; i++) {
-	keypos_list = g_list_append(keypos_list, keypos[i]);
-    }
-   
-    vbox = gtk_vbox_new(FALSE, 0);
-    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
-    gtk_widget_show(vbox);
-    
-    label = gtk_label_new(_("Main"));
-    gtk_widget_show(label);
-    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);   
+    vbox = gp_page_vbox(notebook, _("Main"));
 
-    tbl = gtk_table_new(rows, TAB_MAIN_COLS, FALSE);
-    gtk_table_set_row_spacings(GTK_TABLE(tbl), 5);
-    gtk_table_set_col_spacings(GTK_TABLE(tbl), 5);
-    gtk_box_pack_start(GTK_BOX(vbox), tbl, FALSE, FALSE, 0);
+    tbl = gp_dialog_table(rows, TAB_MAIN_COLS, vbox);
     gtk_widget_show(tbl);
    
     for (i=0; i<NTITLES; i++) {
@@ -842,39 +857,37 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 			      label, 0, 1, rows-1, rows);
     gtk_widget_show(label);
 
-    keycombo = gtk_combo_new();
+    keycombo = gtk_combo_box_new_text();
     gtk_table_attach_defaults(GTK_TABLE(tbl), 
 			      keycombo, 1, TAB_MAIN_COLS, rows-1, rows);
-    gtk_combo_set_popdown_strings(GTK_COMBO(keycombo), keypos_list); 
-    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(keycombo)->entry), spec->keyspec);
+    for (i=0; keypos[i] != NULL; i++) {
+	gtk_combo_box_append_text(GTK_COMBO_BOX(keycombo), keypos[i]);
+	if (!strcmp(keypos[i], spec->keyspec)) {
+	    kactive = i;
+	}
+    }
+    gtk_combo_box_set_active(GTK_COMBO_BOX(keycombo), kactive);
     gtk_widget_show(keycombo);
-    g_list_free(keypos_list);
 
     /* choice of fitted line type, if appropriate */
     if (spec->fit != PLOT_FIT_NA) {
-	GList *fitlist = NULL;
-
-	for (i=0; fittype_strings[i] != NULL; i++) {
-	    fitlist = g_list_append(fitlist, _(fittype_strings[i]));
-	}
-
 	table_add_row(tbl, &rows, TAB_MAIN_COLS);
+
 	label = gtk_label_new(_("fitted line"));
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  label, 0, 1, rows-1, rows);
 	gtk_widget_show(label);
 
-	fitcombo = gtk_combo_new();
+	fitcombo = gtk_combo_box_new_text();
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  fitcombo, 1, TAB_MAIN_COLS, rows-1, rows);
-	gtk_combo_set_popdown_strings(GTK_COMBO(fitcombo), fitlist); 
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(fitcombo)->entry), 
-			   _(fittype_strings[spec->fit]));
-	gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(fitcombo)->entry), FALSE);
-	g_signal_connect(G_OBJECT(GTK_COMBO(fitcombo)->entry), "changed",
+	for (i=0; fittype_strings[i] != NULL; i++) {
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(fitcombo), _(fittype_strings[i]));
+	}
+	gtk_combo_box_set_active(GTK_COMBO_BOX(fitcombo), spec->fit);
+	g_signal_connect(G_OBJECT(fitcombo), "changed",
 			 G_CALLBACK(fit_type_changed), spec);
 	gtk_widget_show(fitcombo);
-	g_list_free(fitlist);
     } else {
 	fitcombo = NULL;
     }
@@ -942,7 +955,7 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 	gtk_widget_show(aa_check);
     }	
 
-    /* set TT font (if gnuplot uses libgd and freetype) */
+    /* set TT font, if gnuplot uses freetype */
     if (gnuplot_has_ttf(0)) {
 	GtkWidget *ebox, *hsep;
 	GList *fontnames = NULL;
@@ -1022,95 +1035,6 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 	    }
 	}
     }
-
-#if 0 /* this is not thought out properly */
-    if (1) {
-	GtkWidget *hsep = gtk_hseparator_new();
-	GtkWidget *hbox, *button;
-
-	table_add_row(tbl, &rows, TAB_MAIN_COLS);
-	gtk_table_attach_defaults(GTK_TABLE(tbl), hsep, 0, TAB_MAIN_COLS, 
-				  rows - 1, rows);  
-	gtk_widget_show(hsep);
-
-	table_add_row(tbl, &rows, TAB_MAIN_COLS);
-	hbox = gtk_hbox_new(FALSE, 2);
-	button = gtk_button_new_with_label(_("Edit plot commands"));
-	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 10);
-	g_signal_connect(G_OBJECT(button), "clicked", 
-			 G_CALLBACK(gp_commands_window), 
-			 spec);
-	gtk_table_attach(GTK_TABLE(tbl), hbox, 0, 2, rows - 1, rows,
-			 GTK_FILL, 0, 0, 5);
-	gtk_widget_show(button);
-	gtk_widget_show(hbox);
-    }
-#endif
-}
-
-static void gpt_tab_output (GtkWidget *notebook, GPT_SPEC *spec) 
-{
-    GtkWidget *label, *vbox, *tbl;
-    int i, tbl_len;
-    GList *termlist = NULL;
-    gchar *termtypes[] = {
-	"postscript",
-	"postscript color",
-	"PDF",
-	"fig",
-	"latex",
-	"png",
-	"plot commands",
-	NULL
-    }; 
-    int pdf_ok = gnuplot_pdf_terminal();
-
-    for (i=0; termtypes[i] != NULL; i++) {
-	if (!pdf_ok && !strcmp(termtypes[i], "PDF")) {
-	    continue;
-	}
-	termlist = g_list_append(termlist, termtypes[i]);
-    }
-   
-    vbox = gtk_vbox_new(FALSE, 0);
-    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
-    gtk_widget_show(vbox);
-    
-    label = gtk_label_new (_("Output to file"));
-    gtk_widget_show(label);
-    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);   
-
-    tbl_len = 1;
-    tbl = gtk_table_new(tbl_len, 2, FALSE);
-    gtk_table_set_row_spacings(GTK_TABLE(tbl), 5);
-    gtk_table_set_col_spacings(GTK_TABLE(tbl), 5);
-    gtk_box_pack_start(GTK_BOX(vbox), tbl, FALSE, FALSE, 0);
-    gtk_widget_show(tbl);
-   
-    tbl_len++;
-    label = gtk_label_new(_("output format"));
-    gtk_table_attach_defaults(GTK_TABLE(tbl), 
-			      label, 0, 1, tbl_len-1, tbl_len);
-    gtk_widget_show(label);
-
-    termcombo = gtk_combo_new();
-    gtk_table_attach_defaults(GTK_TABLE(tbl), termcombo, 1, 2, 
-			      tbl_len-1, tbl_len);
-    gtk_combo_set_popdown_strings(GTK_COMBO(termcombo), termlist); 
-    gtk_widget_show(termcombo);
-    g_list_free(termlist); 
-
-    /* button to generate output to file */
-    filesavebutton = gtk_button_new_from_stock(GTK_STOCK_SAVE_AS);
-    GTK_WIDGET_SET_FLAGS(filesavebutton, GTK_CAN_DEFAULT);
-    tbl_len++;
-    gtk_table_attach_defaults(GTK_TABLE(tbl), 
-			      filesavebutton, 1, 2, tbl_len-1, tbl_len);
-    g_signal_connect (G_OBJECT(filesavebutton), "clicked", 
-		      G_CALLBACK(apply_gpt_changes), 
-		      spec);
-    gtk_widget_grab_default(filesavebutton);
-    gtk_widget_show(filesavebutton);  
 }
 
 static void linetitle_callback (GtkWidget *w, GPT_SPEC *spec)
@@ -1125,7 +1049,6 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
     int i, tbl_len, tbl_num, tbl_col;
     char label_text[32];
     GList *stylist = NULL;
-    GList *yaxis_loc = NULL;
     int do_scale_axis = 0;
 
     if (spec->code == PLOT_REGULAR && (spec->flags & GPT_TS)) {
@@ -1149,19 +1072,13 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
     stylist = g_list_append(stylist, "dots");
     stylist = g_list_append(stylist, "steps");
 
-    if (do_scale_axis) {
-	yaxis_loc = g_list_append(yaxis_loc, "left");
-	yaxis_loc = g_list_append(yaxis_loc, "right");
-    }
-
-    vbox = gtk_vbox_new(FALSE, 0);
-    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
+    vbox = gp_dialog_vbox();
     gtk_widget_show(vbox);
 
     label = gtk_label_new(_("Lines"));
     gtk_widget_show(label);
 
-    if (spec->n_lines > 4) {
+    if (gui_nlines > 4) {
 	GtkWidget *scroller;
 	
 	scroller = gtk_scrolled_window_new(NULL, NULL);
@@ -1177,16 +1094,12 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
     }  
 
     tbl_len = 1;
-    tbl = gtk_table_new(tbl_len, 3, FALSE);
-    gtk_table_set_row_spacings(GTK_TABLE(tbl), 5);
-    gtk_table_set_col_spacings(GTK_TABLE(tbl), 5);
-
-    gtk_box_pack_start(GTK_BOX(vbox), tbl, FALSE, FALSE, 0);
+    tbl = gp_dialog_table(tbl_len, 3, vbox);
     gtk_widget_show(tbl);
    
     tbl_num = tbl_col = 0;
 
-    for (i=0; i<spec->n_lines; i++) {
+    for (i=0; i<gui_nlines; i++) {
 	/* identifier and key or legend text */
 	tbl_len++;
 	gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
@@ -1230,16 +1143,30 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
 	stylecombo[i] = gtk_combo_new();
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  stylecombo[i], 2, 3, tbl_len-1, tbl_len);
-	/* errorbars style is not exchangeable with the others */
-	if (!strcmp(spec->lines[i].style, "errorbars")) {
+
+	/* the errorbars and filledcurves styles are not exchangeable
+	   with the others */
+	if (!strcmp(spec->lines[i].style, "errorbars") &&
+	    !gnuplot_has_style_fill()) {
 	    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(stylecombo[i])->entry), 
 			       spec->lines[i].style); 
 	    gtk_widget_set_sensitive(stylecombo[i], FALSE);
+	} else if (!strcmp(spec->lines[i].style, "errorbars") ||
+		   !strcmp(spec->lines[i].style, "filledcurve")) {
+	    GList *altsty = NULL;
+
+	    altsty = g_list_append(altsty, "errorbars"); 
+	    altsty = g_list_append(altsty, "filledcurve");
+	    gtk_combo_set_popdown_strings(GTK_COMBO(stylecombo[i]), altsty); 
+	    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(stylecombo[i])->entry), 
+			       spec->lines[i].style); 
+	    g_list_free(altsty);
 	} else {
 	    gtk_combo_set_popdown_strings(GTK_COMBO(stylecombo[i]), stylist); 
 	    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(stylecombo[i])->entry), 
 			       spec->lines[i].style); 
 	} 
+
 	gtk_widget_show(stylecombo[i]);	
 
 	if (!do_scale_axis) {
@@ -1275,12 +1202,13 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
 				      label, 1, 2, tbl_len-1, tbl_len);
 	    gtk_widget_show(label);
 
-	    yaxiscombo[i] = gtk_combo_new();
+	    yaxiscombo[i] = gtk_combo_box_new_text();
 	    gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				      yaxiscombo[i], 2, 3, tbl_len-1, tbl_len);
-	    gtk_combo_set_popdown_strings(GTK_COMBO(yaxiscombo[i]), yaxis_loc); 
-	    gtk_entry_set_text (GTK_ENTRY(GTK_COMBO(yaxiscombo[i])->entry), 
-				(spec->lines[i].yaxis == 1)? "left" : "right");  
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(yaxiscombo[i]), "left");
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(yaxiscombo[i]), "right");
+	    gtk_combo_box_set_active(GTK_COMBO_BOX(yaxiscombo[i]), 
+				     (spec->lines[i].yaxis == 1)? 0 : 1);
 	    gtk_widget_show(yaxiscombo[i]);
 	}
 
@@ -1310,10 +1238,6 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
     }
 
     g_list_free(stylist);
-
-    if (yaxis_loc != NULL) {    
-	g_list_free(yaxis_loc);
-    }
 }
 
 static void label_pos_to_entry (double *pos, GtkWidget *w)
@@ -1335,20 +1259,10 @@ static void gpt_tab_labels (GtkWidget *notebook, GPT_SPEC *spec)
     char label_text[32];
     png_plot *plot = (png_plot *) spec->ptr;
 
-    vbox = gtk_vbox_new(FALSE, 0);
-    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
-    gtk_widget_show(vbox);
-
-    label = gtk_label_new(_("Labels"));
-
-    gtk_widget_show(label);
-    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);   
-
+    vbox = gp_page_vbox(notebook, _("Labels"));
+ 
     tbl_len = 1;
-    tbl = gtk_table_new(tbl_len, 3, FALSE);
-    gtk_table_set_row_spacings(GTK_TABLE(tbl), 5);
-    gtk_table_set_col_spacings(GTK_TABLE(tbl), 5);
-    gtk_box_pack_start(GTK_BOX(vbox), tbl, FALSE, FALSE, 0);
+    tbl = gp_dialog_table(tbl_len, 3, vbox);
     gtk_widget_show(tbl);
    
     tbl_num = tbl_col = 0;
@@ -1454,32 +1368,26 @@ static void gpt_tab_labels (GtkWidget *notebook, GPT_SPEC *spec)
 
 static void gpt_tab_XY (GtkWidget *notebook, GPT_SPEC *spec, gint axis) 
 {
-    GtkWidget *vbox, *manual, *tbl;
+    png_plot *plot = (png_plot *) spec->ptr;
+    GtkWidget *b1, *b2, *entry, *vbox, *tbl;
     GtkWidget *label = NULL;
+    char *labelstr = NULL;
     int i, tbl_len;
    
-    vbox = gtk_vbox_new(FALSE, 0);
-    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
-    gtk_widget_show(vbox);
-
     if (axis == 0) {
-	label = gtk_label_new(_("X-axis"));
+	labelstr = _("X-axis");
     } else if (axis == 1) {
-	label = gtk_label_new(_("Y-axis"));
+	labelstr = _("Y-axis");
     } else if (axis == 2) {
-	label = gtk_label_new(_("Y2-axis"));
+	labelstr = _("Y2-axis");
     } else {
 	return;
     }
 
-    gtk_widget_show(label);
-    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);   
+    vbox = gp_page_vbox(notebook, labelstr);
 
     tbl_len = 1;
-    tbl = gtk_table_new(tbl_len, 2, FALSE);
-    gtk_table_set_row_spacings(GTK_TABLE(tbl), 5);
-    gtk_table_set_col_spacings(GTK_TABLE(tbl), 5);
-    gtk_box_pack_start(GTK_BOX(vbox), tbl, FALSE, FALSE, 0);
+    tbl = gp_dialog_table(tbl_len, 2, vbox);
     gtk_widget_show(tbl);
    
     for (i=0; i<NTITLES; i++) {
@@ -1512,8 +1420,9 @@ static void gpt_tab_XY (GtkWidget *notebook, GPT_SPEC *spec, gint axis)
 
     if (spec->code != PLOT_REGULAR) return;
 
-    /* axis range: auto versus manual buttons */
     axis_range[axis].ID = axis;
+
+    /* axis range: "auto" button */
     tbl_len += 3;
     gtk_table_resize(GTK_TABLE(tbl), tbl_len, 2);
 
@@ -1521,32 +1430,26 @@ static void gpt_tab_XY (GtkWidget *notebook, GPT_SPEC *spec, gint axis)
     gtk_table_attach_defaults(GTK_TABLE(tbl), label, 0, 1, 
 			      tbl_len-3, tbl_len-2);
     gtk_widget_show(label);
-
-    axis_range[axis].isauto = 
-	gtk_radio_button_new_with_label(NULL, _("auto axis range"));
-
-    g_signal_connect(G_OBJECT(axis_range[axis].isauto), "clicked",
+    b1 = gtk_radio_button_new_with_label(NULL, _("auto axis range"));
+    g_signal_connect(G_OBJECT(b1), "clicked",
 		     G_CALLBACK(flip_manual_range), 
-		     GINT_TO_POINTER(axis_range[axis].ID));
+		     GINT_TO_POINTER(axis));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b1), TRUE);
+    gtk_table_attach_defaults(GTK_TABLE(tbl), b1, 0, 1, 
+			      tbl_len-2, tbl_len-1);
+    gtk_widget_show(b1);
+    axis_range[axis].isauto = b1;
 
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
-				 (axis_range[axis].isauto), TRUE);
-    gtk_table_attach_defaults(GTK_TABLE(tbl), axis_range[axis].isauto, 
-			      0, 1, tbl_len-2, tbl_len-1);
-    gtk_widget_show(axis_range[axis].isauto);
-
-    manual = 
-	gtk_radio_button_new_with_label(gtk_radio_button_get_group 
-					(GTK_RADIO_BUTTON 
-					 (axis_range[axis].isauto)),
-					_("manual range:")); 
-    g_signal_connect(G_OBJECT(manual), "clicked",
+    /* axis range: manual range button */
+    b2 = gtk_radio_button_new_with_label(gtk_radio_button_get_group 
+					 (GTK_RADIO_BUTTON(b1)),
+					 _("manual range:")); 
+    g_signal_connect(G_OBJECT(b2), "clicked",
 		     G_CALLBACK(flip_manual_range), 
-		     GINT_TO_POINTER(axis_range[axis].ID));
-
-    gtk_table_attach_defaults(GTK_TABLE(tbl), manual, 0, 1, 
+		     GINT_TO_POINTER(axis));
+    gtk_table_attach_defaults(GTK_TABLE(tbl), b2, 0, 1, 
 			      tbl_len-1, tbl_len);
-    gtk_widget_show(manual);
+    gtk_widget_show(b2);
 
     /* axis range min. entry */
     tbl_len++;
@@ -1555,16 +1458,15 @@ static void gpt_tab_XY (GtkWidget *notebook, GPT_SPEC *spec, gint axis)
 			      label, 0, 1, tbl_len-1, tbl_len);
     gtk_widget_show(label);
     gtk_table_resize(GTK_TABLE(tbl), tbl_len, 2);
-    axis_range[axis].min = gtk_entry_new();
-    gtk_table_attach_defaults(GTK_TABLE(tbl), axis_range[axis].min, 
-			      1, 2, tbl_len-1, tbl_len);
-    gtk_entry_set_text(GTK_ENTRY(axis_range[axis].min), "");
-
-    g_signal_connect(G_OBJECT(axis_range[axis].min), "activate", 
+    entry = gtk_entry_new();
+    gtk_table_attach_defaults(GTK_TABLE(tbl), entry, 1, 2, 
+			      tbl_len-1, tbl_len);
+    gtk_entry_set_text(GTK_ENTRY(entry), "");
+    g_signal_connect(G_OBJECT(entry), "activate", 
 		     G_CALLBACK(apply_gpt_changes), 
 		     spec);
-
-    gtk_widget_show(axis_range[axis].min);
+    gtk_widget_show(entry);
+    axis_range[axis].min = entry;
 
     /* axis range max. entry */
     tbl_len++;
@@ -1573,26 +1475,83 @@ static void gpt_tab_XY (GtkWidget *notebook, GPT_SPEC *spec, gint axis)
 			      tbl_len-1, tbl_len);
     gtk_widget_show(label);
     gtk_table_resize(GTK_TABLE(tbl), tbl_len, 2);
-    axis_range[axis].max = gtk_entry_new();
-    gtk_table_attach_defaults(GTK_TABLE(tbl), axis_range[axis].max, 
-			      1, 2, tbl_len-1, tbl_len);
-    gtk_entry_set_text(GTK_ENTRY(axis_range[axis].max), "");
-
-    g_signal_connect(G_OBJECT(axis_range[axis].max), "activate", 
+    entry = gtk_entry_new();
+    gtk_table_attach_defaults(GTK_TABLE(tbl), entry, 1, 2, 
+			      tbl_len-1, tbl_len);
+    gtk_entry_set_text(GTK_ENTRY(entry), "");
+    g_signal_connect(G_OBJECT(entry), "activate", 
 		     G_CALLBACK(apply_gpt_changes), 
 		     spec);
-
-    gtk_widget_show(axis_range[axis].max);
+    gtk_widget_show(entry);
+    axis_range[axis].max = entry;
 
     if (na(spec->range[axis][0])) {
-	flip_manual_range(NULL, GINT_TO_POINTER(axis_range[axis].ID));
+	flip_manual_range(NULL, GINT_TO_POINTER(axis));
     } else {
 	double_to_gp_entry(spec->range[axis][0], axis_range[axis].min);
 	double_to_gp_entry(spec->range[axis][1], axis_range[axis].max);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(axis_range[axis].isauto), 
 				     FALSE);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(manual), 
-				     TRUE);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b2), TRUE);
+    }
+
+    /* axis scale: linear vs log? */
+
+    if ((axis == 0 && plot_get_xmin(plot) >= 0.0) ||
+	(axis == 1 && plot_get_ymin(plot) >= 0.0)) {
+	GtkWidget *combo;
+	GList *strs = NULL;
+
+	strs = g_list_append(strs, "e");
+	strs = g_list_append(strs, "2");
+	strs = g_list_append(strs, "10");
+
+	tbl_len += 3;
+	gtk_table_resize(GTK_TABLE(tbl), tbl_len, 2);
+
+	label = gtk_label_new("");
+	gtk_table_attach_defaults(GTK_TABLE(tbl), label, 0, 1, 
+				  tbl_len-3, tbl_len-2);
+	gtk_widget_show(label);
+	b1 = gtk_radio_button_new_with_label(NULL, _("linear scale"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b1), TRUE);
+	gtk_table_attach_defaults(GTK_TABLE(tbl), b1, 0, 1, 
+				  tbl_len-2, tbl_len-1);
+	gtk_widget_show(b1);
+
+	b2 = gtk_radio_button_new_with_label(gtk_radio_button_get_group 
+					     (GTK_RADIO_BUTTON(b1)),
+					     _("logarithmic scale, base:")); 
+	gtk_table_attach_defaults(GTK_TABLE(tbl), b2, 0, 1, 
+				  tbl_len-1, tbl_len);
+	gtk_widget_show(b2);    
+
+	combo = gtk_combo_new();
+	gtk_combo_set_popdown_strings(GTK_COMBO(combo), strs); 
+	g_list_free(strs);
+	gtk_table_attach_defaults(GTK_TABLE(tbl), combo, 1, 2, 
+				  tbl_len-1, tbl_len);
+	entry = GTK_COMBO(combo)->entry;
+	g_signal_connect(G_OBJECT(entry), "activate", 
+			 G_CALLBACK(apply_gpt_changes), 
+			 spec);
+	gtk_widget_show(combo);
+	axis_range[axis].lbase = entry;
+
+	if (spec->logbase[axis] > 1.1) {
+	    gchar *txt = g_strdup_printf("%g", spec->logbase[axis]);
+
+	    gtk_entry_set_text(GTK_ENTRY(entry), txt);
+	    g_free(txt);
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b2), TRUE);
+	} else {
+	    gtk_widget_set_sensitive(entry, FALSE);
+	}
+
+	g_signal_connect(G_OBJECT(b1), "clicked", G_CALLBACK(disable_lbase), 
+			 entry);
+	g_signal_connect(G_OBJECT(b2), "clicked", G_CALLBACK(enable_lbase), 
+			 entry);
     }
 }
 
@@ -1641,6 +1600,8 @@ static int gpt_allocate_widgets (int n)
 	linewidth[i] = NULL;
     }
 
+    gui_nlines = n;
+
     return 0;
 }
 
@@ -1664,6 +1625,7 @@ int show_gnuplot_dialog (GPT_SPEC *spec)
 
     for (i=0; i<MAX_AXES; i++) {
 	axis_range[i].isauto = NULL;
+	axis_range[i].lbase = NULL;
     }
 
     for (i=0; i<NTITLES; i++) {
@@ -1699,8 +1661,8 @@ int show_gnuplot_dialog (GPT_SPEC *spec)
     if (spec->lines != NULL) {
 	gpt_tab_lines(notebook, spec);
     }
+
     gpt_tab_labels(notebook, spec); 
-    gpt_tab_output(notebook, spec);
 
     hbox = GTK_DIALOG(gpt_control)->action_area;
 
