@@ -23,6 +23,7 @@
 #include "version.h"
 #include "gretl_func.h"
 #include "gretl_xml.h"
+#include "libset.h"
 
 #include "treeutils.h"
 #include "ssheet.h"
@@ -37,6 +38,7 @@
 #include "fileselect.h"
 #include "filters.h"
 #include "calculator.h"
+#include "fnsave.h"
 
 #include <dirent.h>
 
@@ -60,14 +62,18 @@ static GtkWidget *make_main_window (void);
 static gboolean main_popup_handler (GtkWidget *w, GdkEventButton *event,
 				    gpointer data);
 static void set_up_main_menu (void);
-static void startRcallback (gpointer p, guint opt, GtkWidget *w);
+static void start_R_callback (gpointer p, guint opt, GtkWidget *w);
 static void auto_store (void);
 static void restore_sample_callback (gpointer p, int verbose, GtkWidget *w);
 static void mdata_select_all (void);
 
 GtkTargetEntry gretl_drag_targets[] = {
-    { "text/uri-list", 0, GRETL_FILENAME },
-    { "db_pointer", GTK_TARGET_SAME_APP, GRETL_POINTER }    
+    { "text/uri-list",  0, GRETL_FILENAME },
+    { "db_series_ptr",  GTK_TARGET_SAME_APP, GRETL_DBSERIES_PTR },
+    { "model_ptr",      GTK_TARGET_SAME_APP, GRETL_MODEL_PTR },
+    { "remote_db_ptr",  GTK_TARGET_SAME_APP, GRETL_REMOTE_DB_PTR },
+    { "remote_pkg_ptr", GTK_TARGET_SAME_APP, GRETL_REMOTE_FNPKG_PTR },
+    { "graph_file",     GTK_TARGET_SAME_APP, GRETL_GRAPH_FILE }
 };
 
 static void  
@@ -116,7 +122,6 @@ int data_status, orig_vars;
 float gui_scale;
 
 /* defaults for some options */
-int expert = FALSE; 
 int updater = FALSE;
 int want_toolbar = TRUE;
 int winsize = FALSE;
@@ -126,23 +131,23 @@ int mainwin_width = 520;
 int mainwin_height = 420;
 
 #if defined(G_OS_WIN32)
-char Rcommand[MAXSTR] = "RGui.exe";
 char calculator[MAXSTR] = "calc.exe";
 char latex[MAXSTR] = "pdflatex.exe";
 char viewdvi[MAXSTR] = "windvi.exe";
+char Rcommand[MAXSTR] = "RGui.exe";
 #elif defined(OSX_BUILD)
 char calculator[MAXSTR] = "xcalc";
 char latex[MAXSTR] = "pdflatex";
 char viewdvi[MAXSTR] = "xdvi";
 char Rcommand[MAXSTR] = "xterm -e R";
 #else
+char Browser[MAXSTR] = "mozilla";
 char calculator[MAXSTR] = "xcalc";
 char latex[MAXSTR] = "latex";
 char viewdvi[MAXSTR] = "xdvi";
-char viewps[MAXSTR] = "gv";
 char viewpdf[MAXSTR] = "acroread";
+char viewps[MAXSTR] = "gv";
 char Rcommand[MAXSTR] = "xterm -e R";
-char Browser[MAXSTR] = "mozilla";
 #endif
 
 static void spreadsheet_edit (gpointer p, guint u, GtkWidget *w) 
@@ -160,13 +165,27 @@ static void manual_update_query (gpointer p, guint u, GtkWidget *w)
     update_query();
 }
 
-extern void find_var (gpointer p, guint u, GtkWidget *w); /* gui_utils.c */
+extern void find_var (gpointer p, guint u, GtkWidget *w); /* helpfiles.c */
 
 static void varinfo_callback (gpointer p, guint u, GtkWidget *w)
 {
     varinfo_dialog(mdata->active_var, 1);
 }
 
+static void options_dialog_callback (gpointer p, guint u, GtkWidget *w)
+{
+    options_dialog(0, NULL);
+}
+
+static void new_function_pkg_callback (gpointer p, guint u, GtkWidget *w)
+{
+    if (no_user_functions_check()) {
+	return;
+    } else {
+	file_save(p, SAVE_FUNCTIONS, w);
+    }
+}
+
 #ifdef ENABLE_MAILER
 static void email_data (gpointer p, guint u, GtkWidget *w)
 {
@@ -191,7 +210,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Open data/Import/_ASCII..."), NULL, open_data, OPEN_ASCII, NULL, GNULL },
     { N_("/File/Open data/Import/_Octave..."), NULL, open_data, OPEN_OCTAVE, NULL, GNULL },
     { N_("/File/Open data/Import/_Gnumeric..."), NULL, open_data, OPEN_GNUMERIC, NULL, GNULL },
-    { N_("/File/Open data/Import/_Excel..."), NULL, open_data, OPEN_EXCEL, NULL, GNULL },
+    { N_("/File/Open data/Import/_Excel..."), NULL, open_data, OPEN_XLS, NULL, GNULL },
     { N_("/File/Open data/Import/_Open Document..."), NULL, open_data, OPEN_ODS, NULL, GNULL },
     { N_("/File/Open data/Import/_Eviews..."), NULL, open_data, OPEN_WF1, NULL, GNULL },
     { N_("/File/Open data/Import/_Stata..."), NULL, open_data, OPEN_DTA, NULL, GNULL },
@@ -204,7 +223,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Append data/_ASCII..."), NULL, open_data, APPEND_ASCII, NULL, GNULL },
     { N_("/File/Append data/_Octave..."), NULL, open_data, APPEND_OCTAVE, NULL, GNULL },
     { N_("/File/Append data/_Gnumeric..."), NULL, open_data, APPEND_GNUMERIC, NULL, GNULL },
-    { N_("/File/Append data/_Excel..."), NULL, open_data, APPEND_EXCEL, NULL, GNULL },
+    { N_("/File/Append data/_Excel..."), NULL, open_data, APPEND_XLS, NULL, GNULL },
     { N_("/File/Append data/_Open Document..."), NULL, open_data, OPEN_ODS, NULL, GNULL },
     { N_("/File/Append data/_Eviews..."), NULL, open_data, APPEND_WF1, NULL, GNULL },
     { N_("/File/Append data/_Stata..."), NULL, open_data, APPEND_DTA, NULL, GNULL },
@@ -234,13 +253,24 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/C_lear data set"), NULL, verify_clear_data, 0, "<StockItem>", GTK_STOCK_CLEAR },
     { "/File/sep1", NULL, NULL, 0, "<Separator>", GNULL },
 
+    /* File, working dir */
+    { N_("/File/_Working directory..."), NULL, working_dir_dialog, 0, NULL, GNULL },
+    { "/File/sep2", NULL, NULL, 0, "<Separator>", GNULL },
+
     /* File, script files */
     { N_("/File/_Script files"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/File/Script files/_User file..."), "", open_script, OPEN_SCRIPT, 
       "<StockItem>", GTK_STOCK_OPEN },
     { N_("/File/Script files/_Practice file..."), "", display_files, PS_FILES, 
       "<StockItem>", GTK_STOCK_OPEN },
+#if 0
     { N_("/File/Script files/_New script"), "", do_new_script, 0, "<StockItem>", GTK_STOCK_NEW },
+#else
+    { N_("/File/Script files/_New script"), NULL, NULL, 0, "<Branch>", GNULL },
+    { N_("/File/Script files/_New script/gretl script"), "", do_new_script, EDIT_SCRIPT, NULL, GNULL },
+    { N_("/File/Script files/_New script/gnuplot script"), "", do_new_script, EDIT_GP, NULL, GNULL },
+    { N_("/File/Script files/_New script/R script"), "", do_new_script, EDIT_R, NULL, GNULL },
+#endif
 
     /* File, session files */
     { N_("/File/_Session files"), NULL, NULL, 0, "<Branch>", GNULL },
@@ -268,9 +298,10 @@ GtkItemFactoryEntry data_items[] = {
       "<StockItem>", GTK_STOCK_OPEN },
     { N_("/File/Function files/On _server..."), "", display_files, 
       REMOTE_FUNC_FILES, "<StockItem>", GTK_STOCK_NETWORK },
-    { N_("/File/Function files/_New package"), "", file_save, 
-      SAVE_FUNCTIONS, "<StockItem>", GTK_STOCK_NEW },
-    { "/File/sep5", NULL, NULL, 0, "<Separator>", GNULL },
+    { N_("/File/Function files/_New package"), "", new_function_pkg_callback, 0,
+      "<StockItem>", GTK_STOCK_NEW },
+
+    { "/File/sep3", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/File/E_xit"), "<control>X", menu_exit_check, 0, "<StockItem>", GTK_STOCK_QUIT },
 
     /* Tools menu */
@@ -288,7 +319,7 @@ GtkItemFactoryEntry data_items[] = {
     { "/Tools/sep1", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Tools/_Command log"), NULL, view_command_log, 0, NULL, GNULL },
     { N_("/Tools/_Gretl console"), NULL, show_gretl_console, 0, NULL, GNULL },
-    { N_("/Tools/Start GNU _R"), NULL, startRcallback, 0, NULL, GNULL },
+    { N_("/Tools/Start GNU _R"), NULL, start_R_callback, 0, NULL, GNULL },
     { "/Tools/sep2", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Tools/_Sort variables"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Tools/Sort variables/By _ID number"), NULL, sort_varlist, 0, NULL, GNULL },
@@ -346,6 +377,8 @@ GtkItemFactoryEntry data_items[] = {
       NULL, selector_callback, GR_IMP, NULL, GNULL },
     { N_("/View/Graph specified vars/X-Y with _factor separation..."), 
       NULL, selector_callback, GR_DUMMY, NULL, GNULL },
+    { N_("/View/Graph specified vars/X-Y with _control..."), 
+      NULL, selector_callback, GR_XYZ, NULL, GNULL },
     { N_("/View/Graph specified vars/_Boxplots..."), 
       NULL, gretl_callback, GR_BOX, NULL, GNULL },
     { N_("/View/Graph specified vars/_Notched boxplots..."), 
@@ -411,6 +444,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Sample/_Restrict, based on criterion..."), NULL, gretl_callback, 
       SMPLBOOL, NULL, GNULL },
     { N_("/Sample/R_andom sub-sample..."), NULL, sample_range_dialog, SMPLRAND, NULL, GNULL },
+    { N_("/Sample/_Resample with replacement..."), NULL, gui_resample_data, 0, NULL, GNULL },
     { "/Sample/sep3", NULL, NULL, 0, "<Separator>", GNULL },  
     { N_("/Sample/Drop all obs with _missing values"), NULL, drop_all_missing, 
       0, NULL, GNULL },
@@ -422,16 +456,9 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/_Variable"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Variable/_Find..."), NULL, find_var, 0, "<StockItem>", GTK_STOCK_FIND },
     { N_("/Variable/_Display values"), NULL, display_var, 0, NULL, GNULL },
-    { N_("/Variable/_Summary statistics"), NULL, do_menu_op, 
-      VAR_SUMMARY, NULL, GNULL },
-    { N_("/Variable/_Frequency distribution"), NULL, do_menu_op, 
-      FREQ, NULL, GNULL },
-    { N_("/Variable/Frequency _plot"), NULL, NULL, 0, "<Branch>", GNULL },
-    { N_("/Variable/Frequency plot/_Simple"), NULL, do_freqplot, 0, NULL, GNULL },
-    { N_("/Variable/Frequency plot/Against _Normal"), NULL, do_freqplot, 
-      D_NORMAL, NULL, GNULL },
-    { N_("/Variable/Frequency plot/Against _Gamma"), NULL, do_freqplot, 
-      D_GAMMA, NULL, GNULL },
+    { N_("/Variable/_Summary statistics"), NULL, do_menu_op, VAR_SUMMARY, NULL, GNULL },
+    { N_("/Variable/_Frequency distribution"), NULL, do_freq_dist, 0, NULL, GNULL },
+    { N_("/Variable/Frequency _plot"), NULL, do_freq_dist, 1, NULL, GNULL },
     { N_("/Variable/Estimated _density plot..."), NULL, do_kernel, 0, NULL, GNULL },
     { N_("/Variable/_Gini coefficient"), NULL, do_gini, 0, NULL, GNULL },
     { N_("/Variable/_Range-mean graph"), NULL, do_range_mean, 0, NULL, GNULL }, 
@@ -442,6 +469,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Variable/Spectrum/Sample _periodogram"), NULL, do_pergm, 0, NULL, GNULL }, 
     { N_("/Variable/Spectrum/_Bartlett lag window"), NULL, do_pergm, 1, NULL, GNULL }, 
     { N_("/Variable/_Augmented Dickey-Fuller test"), NULL, unit_root_test, ADF, NULL, GNULL },
+    { N_("/Variable/ADF-GLS test"), NULL, unit_root_test, DFGLS, NULL, GNULL },
     { N_("/Variable/_KPSS test"), NULL, unit_root_test, KPSS, NULL, GNULL },
     { N_("/Variable/_Filter"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/Variable/Filter/_Simple moving average"), NULL, filter_callback, 
@@ -494,6 +522,8 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Model/_Robust estimation"), NULL, NULL, 0, "<Branch>", GNULL }, 
     { N_("/Model/Robust estimation/Least _Absolute Deviation..."), NULL, 
       model_callback, LAD, NULL, GNULL },
+    { N_("/Model/Robust estimation/_Quantile regression..."), NULL, 
+      model_callback, QUANTREG, NULL, GNULL },
     { N_("/Model/Robust estimation/_Rank correlation..."), NULL, 
       selector_callback, SPEARMAN, NULL, GNULL },
     { N_("/Model/_Maximum likelihood..."), NULL, gretl_callback, MLE, NULL, GNULL },
@@ -507,6 +537,7 @@ GtkItemFactoryEntry data_items[] = {
       GRETL_STOCK_BOOK },
     { N_("/Help/Command reference/_PDF"), NULL, display_pdf_help, 0, "<StockItem>", 
       GRETL_STOCK_PDF },
+    { N_("/Help/Function reference"), NULL, genr_funcs_ref, 0, NULL, GNULL },
     { N_("/Help/_User's guide"), NULL, display_pdf_help, 1, "<StockItem>", 
       GRETL_STOCK_PDF },
     { "/Help/sep1", NULL, NULL, 0, "<Separator>", GNULL },
@@ -546,8 +577,6 @@ static void noalloc (const char *str)
 
 static void get_runfile (char *fname)
 {
-    int i;
-
     *tryfile = '\0';
 
 #ifdef G_OS_WIN32
@@ -561,12 +590,7 @@ static void get_runfile (char *fname)
 	exit(EXIT_FAILURE);
     } else {
 	fprintf(stderr, I_("%s found\n"), tryfile);
-	i = slashpos(tryfile);
-	if (i) {
-	    paths.currdir[0] = '\0';
-	    strncat(paths.currdir, tryfile, i);
-	}
-	strcat(paths.currdir, SLASHSTR);
+	set_currdir_from_filename(tryfile);
     }
 }
 
@@ -740,7 +764,6 @@ int main (int argc, char *argv[])
 #endif  
 
 #ifdef G_OS_WIN32
-    putenv("PANGO_WIN32_NO_UNISCRIBE=a");
 # if GTK_MINOR_VERSION < 6
     putenv("G_FILENAME_ENCODING=@locale");
 # endif
@@ -766,7 +789,6 @@ int main (int argc, char *argv[])
 
     libgretl_init();
     gretl_set_paths(&paths, OPT_D | OPT_X); /* defaults, gui */
-    set_program_startdir(argv[0]);
 
 #ifdef G_OS_WIN32
     gretl_win32_init(argv[0]);
@@ -774,6 +796,8 @@ int main (int argc, char *argv[])
     gretl_config_init();
 #endif
 
+    set_workdir_callback(gui_set_working_dir);
+
     if (argc > 1) {
 	int force_lang = 0;
 
@@ -858,8 +882,8 @@ int main (int argc, char *argv[])
     if (open_datafile) {
 	PRN *prn; 
 
-	prn = gretl_print_new(GRETL_PRINT_STDERR);
-	if (prn == NULL) exit(EXIT_FAILURE);
+	prn = gretl_print_new(GRETL_PRINT_STDERR, &err);
+	if (err) exit(EXIT_FAILURE);
 
 	*paths.datfile = '\0';
 
@@ -878,25 +902,25 @@ int main (int argc, char *argv[])
 
 	switch (ftype) {
 	case GRETL_NATIVE_DATA:
-	    err = gretl_get_data(&Z, &datainfo, paths.datfile, &paths, prn);
+	    err = gretl_get_data(paths.datfile, &paths, &Z, datainfo, 
+				 OPT_NONE, prn);
 	    break;
 	case GRETL_XML_DATA:
-	    err = gretl_read_gdt(&Z, &datainfo, paths.datfile, &paths, OPT_NONE, 
-				 prn);
+	    err = gretl_read_gdt(paths.datfile, &paths, &Z, datainfo, 
+				 OPT_NONE, prn);
 	    break;
-	case GRETL_CSV_DATA:
-	    err = import_csv(&Z, &datainfo, paths.datfile, OPT_NONE, prn);
+	case GRETL_CSV:
+	    err = import_csv(paths.datfile, &Z, datainfo, 
+			     OPT_NONE, prn);
 	    break;
-	case GRETL_OCTAVE:
-	    err = import_octave(&Z, &datainfo, paths.datfile, prn);
-	    break;
-	case GRETL_EXCEL:
+	case GRETL_XLS:
 	case GRETL_GNUMERIC:
-	case GRETL_WF1:
+	case GRETL_ODS:
 	case GRETL_DTA:
 	case GRETL_JMULTI:
-	case GRETL_ODS:
-	    err = get_worksheet_data(paths.datfile, ftype, 0);
+	case GRETL_OCTAVE:
+	case GRETL_WF1:
+	    err = get_imported_data(paths.datfile, ftype, 0);
 	    break;
 	case GRETL_SCRIPT:
 	case GRETL_SESSION:
@@ -948,11 +972,6 @@ int main (int argc, char *argv[])
 	set_sample_label(datainfo);
     }
 
-#ifndef G_OS_WIN32
-    /* Let a first-time user set the working dir */
-    first_time_set_user_dir(); 
-#endif
-
     add_files_to_menus();
 
     session_menu_state(FALSE);
@@ -962,7 +981,7 @@ int main (int argc, char *argv[])
     /* run init script, if found? */
 
     if (have_data()) {
-	register_data(paths.datfile, tryfile, 1);
+	register_startup_data(tryfile);
 	maybe_display_string_table();
 	*tryfile = 0;
     }
@@ -975,7 +994,7 @@ int main (int argc, char *argv[])
 	if (ftype == GRETL_SESSION) {
 	    do_open_session();
 	} else {
-	    do_open_script();
+	    do_open_script(EDIT_SCRIPT);
 	}
     }
 
@@ -993,7 +1012,12 @@ int main (int argc, char *argv[])
 
 #ifdef G_OS_WIN32
     if (debug) {
+	char *loc;
+
 	gretl_win32_debug();
+	loc = setlocale(LC_ALL, "");
+	fprintf(stderr, "setlocale gives '%s'\n", 
+		(loc != NULL)? loc : "NULL");
     }
 #endif
 
@@ -1579,15 +1603,17 @@ static void set_up_main_menu (void)
     mdata->mbar = gtk_item_factory_get_widget(mdata->ifac, "<main>");
 }
 
-int gui_restore_sample (double ***pZ, DATAINFO **ppdinfo)
+int gui_restore_sample (double ***pZ, DATAINFO *pdinfo)
 {
-    int err;
+    int err = 0;
 
-    err = restore_full_sample(pZ, ppdinfo, NULL);
-    if (err) {
-	gui_errmsg(err);
-    } else {
-	restore_sample_state(FALSE);
+    if (pZ != NULL && *pZ != NULL) {
+	err = restore_full_sample(pZ, pdinfo, NULL);
+	if (err) {
+	    gui_errmsg(err);
+	} else {
+	    restore_sample_state(FALSE);
+	}
     }
 
     return err;
@@ -1595,7 +1621,7 @@ int gui_restore_sample (double ***pZ, DATAINFO **ppdinfo)
 
 static void restore_sample_callback (gpointer p, int verbose, GtkWidget *w)
 {
-    int err = gui_restore_sample(&Z, &datainfo); 
+    int err = gui_restore_sample(&Z, datainfo); 
 
     if (verbose && !err) {
 	set_sample_label(datainfo);    
@@ -1604,17 +1630,44 @@ static void restore_sample_callback (gpointer p, int verbose, GtkWidget *w)
     }
 }
 
-static void startRcallback (gpointer p, guint opt, GtkWidget *w)
+static void start_R_callback (gpointer p, guint opt, GtkWidget *w)
 {
-    startR(Rcommand);
+    start_R(NULL, 1, 1);
 }
 
 #ifndef G_OS_WIN32
 
-int gretl_fork (const char *prog, const char *arg)
+int gretl_fork (const char *progvar, const char *arg)
 {
+    const char *prog = NULL;
     gchar *argv[3];
+    GError *err = NULL;
     gboolean run;
+
+#ifdef OSX_BUILD
+    if (!strcmp(progvar, "calculator")) {
+	prog = calculator;
+    } else if (!strcmp(progvar, "viewdvi")) {
+	prog = viewdvi;
+    } 
+#else
+    if (!strcmp(progvar, "Browser")) {
+	prog = Browser;
+    } else if (!strcmp(progvar, "calculator")) {
+	prog = calculator;
+    } else if (!strcmp(progvar, "viewdvi")) {
+	prog = viewdvi;
+    } else if (!strcmp(progvar, "viewpdf")) {
+	prog = viewpdf;
+    } else if (!strcmp(progvar, "viewps")) {
+	prog = viewps;
+    }    
+#endif
+
+    if (prog == NULL) {
+	errbox("Internal error: variable %s is undefined", progvar);
+	return 1;
+    }
     
     argv[0] = g_strdup(prog);
     if (arg != NULL) {
@@ -1625,10 +1678,15 @@ int gretl_fork (const char *prog, const char *arg)
     }
     
     run = g_spawn_async(NULL, argv, NULL, G_SPAWN_SEARCH_PATH, 
-			NULL, NULL, NULL, NULL);
+			NULL, NULL, NULL, &err);
 
-    if (!run) {
-	errbox("%s: %s", _("Command failed"), prog);
+    if (err != NULL) {
+	errbox(err->message);
+	if (err->domain == G_SPAWN_ERROR &&
+	    err->code == G_SPAWN_ERROR_NOENT) {
+	    options_dialog(TAB_PROGS, progvar);
+	}
+	g_error_free(err);
     }
 
     g_free(argv[0]);
@@ -1670,12 +1728,16 @@ mdata_handle_drag  (GtkWidget *widget,
     int pos, skip = 5;
 
     /* handle drag of pointer from database window */
-    if (info == GRETL_POINTER && data != NULL && 
+    if (info == GRETL_DBSERIES_PTR && data != NULL && 
 	data->type == GDK_SELECTION_TYPE_INTEGER) {
 	import_db_series(*(void **) data->data);
 	return;
     }
 
+    if (info != GRETL_FILENAME) {
+	return;
+    }
+
     /* ignore the wrong sort of data */
     if (data == NULL || (dfname = (gchar *) data->data) == NULL || 
 	strlen(dfname) <= 5 || strncmp(dfname, "file:", 5)) {
@@ -1707,8 +1769,12 @@ mdata_handle_drag  (GtkWidget *widget,
 
     if (gretl_is_pkzip_file(tmp)) {
 	verify_open_session();
-    } else if (probably_script_file(tmp)) {
-	do_open_script();
+    } else if (has_suffix(tmp, ".inp")) {
+	do_open_script(EDIT_SCRIPT);
+    } else if (has_suffix(tmp, ".R")) {
+	do_open_script(EDIT_R);
+    } else if (has_suffix(tmp, ".plt")) {
+	do_open_script(EDIT_GP);
     } else {
 	verify_open_data(NULL, 0);
     }
@@ -1726,8 +1792,7 @@ static void auto_store (void)
 	oflag = OPT_NONE;
     }
 
-    if ((data_status & USER_DATA) && 
-	probably_native_datafile(paths.datfile)) {
+    if ((data_status & USER_DATA) && has_suffix(paths.datfile, ".gdt")) {
 	do_store(paths.datfile, oflag);
     } else {
 	file_selector(_("Save data file"), SAVE_DATA, FSEL_DATA_NONE, NULL);
