@@ -33,7 +33,10 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+
 #include "libgretl.h"
+#include "dbread.h"
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -89,7 +92,6 @@ extern char sessionfile[MAXLEN];
 extern char tryfile[MAXLEN];
 
 /* global option-related vars */
-extern int expert;
 extern int updater;
 extern int winsize;
 extern int main_x;
@@ -106,6 +108,7 @@ extern char Browser[MAXSTR];
 extern char calculator[MAXSTR];
 extern char latex[MAXSTR];
 extern char viewdvi[MAXSTR];
+extern char Rcommand[MAXSTR];
 
 #if defined(HAVE_AUDIO) && !defined(G_OS_WIN32)
 extern char midiplayer[MAXSTR];
@@ -121,7 +124,7 @@ extern PangoFontDescription *fixed_font;
 /* functions follow */
 
 #ifndef WIN32
-int gretl_fork (const char *prog, const char *arg);
+int gretl_fork (const char *progvar, const char *arg);
 void set_wm_icon (GtkWidget *w, gpointer data);
 #endif
 
@@ -130,7 +133,7 @@ int mdata_active_var (void);
 void populate_varlist (void);
 void clear_varlist (GtkWidget *widget);
 void mdata_select_last_var (void);
-int gui_restore_sample (double ***pZ, DATAINFO **ppdinfo);
+int gui_restore_sample (double ***pZ, DATAINFO *pdinfo);
 
 /* functions defined in files other than gretl.c */
 void about_dialog (gpointer data);
