@@ -23,6 +23,7 @@ enum extra_cmds {
     GR_XY,
     GR_IMP,
     GR_DUMMY,
+    GR_XYZ,
     GR_BOX,
     GR_NBOX,
     GR_3D,
@@ -52,22 +53,27 @@ enum extra_cmds {
     EDIT_SCRIPT,
     EDIT_NOTES,
     EDIT_FUNC_CODE,
+    EDIT_GP,
+    EDIT_R,
+    EDIT_BOX,
     CLI_HELP,
     GUI_HELP,
     CLI_HELP_EN,
     GUI_HELP_EN,
+    FUNCS_HELP,
     MODELTABLE,
     GRAPHPAGE,
-    CREATE_USERDIR,
     LMTEST_LOGS,
     LMTEST_SQUARES,
     LMTEST_WHITE,
+    LMTEST_WHITE_NOX,
+    LMTEST_BP,
+    LMTEST_BPK,
     LMTEST_BG,
     LMTEST_GROUPWISE,
     KERNEL_DENSITY,
     CREATE_DATASET,
     HCCME,
-    VAR_VCV,
     VAR_IRF,
     VAR_DECOMP,
     IRF_BOOT,
@@ -94,6 +100,12 @@ enum extra_cmds {
     BOOTSTRAP,
     TRANSPOS,
     DATASORT,
+    WORKDIR,
+    DFGLS,
+    CORC,
+    HILU,
+    PWE,
+    AFR, /* actual, fitted, residual */
     GUI_CMD_MAX
 };
 
@@ -111,8 +123,8 @@ enum file_ops {
     APPEND_OCTAVE,
     OPEN_GNUMERIC,
     APPEND_GNUMERIC,
-    OPEN_EXCEL,
-    APPEND_EXCEL,
+    OPEN_XLS,
+    APPEND_XLS,
     OPEN_WF1,
     APPEND_WF1,
     OPEN_DTA,
@@ -136,6 +148,7 @@ enum file_ops {
     END_SAVE_DATA,  /* marker for end of data-saving section */
     SAVE_TEX,
     SAVE_RTF,
+    SAVE_TEXT,
     SAVE_SCRIPT,
     SAVE_OUTPUT,
     SAVE_SESSION,
@@ -144,11 +157,14 @@ enum file_ops {
     SAVE_BOXPLOT_XPM,
     SAVE_GNUPLOT,
     SAVE_GP_CMDS,
+    SAVE_R_CMDS,
     SAVE_CONSOLE,
     SAVE_FUNCTIONS,
+    SAVE_FUNCTIONS_AS,
     SAVE_BOOT_DATA,
     SET_PROG,
     SET_DIR,
+    SET_WDIR,
     FILE_OP_MAX
 };
 
@@ -168,6 +184,16 @@ enum browser_codes {
     MAINWIN
 };
 
+enum pref_tabs {
+    TAB_NONE = 0,
+    TAB_MAIN,
+    TAB_DBS,
+    TAB_PROGS,
+    TAB_VCV,
+    TAB_MAN,
+    TAB_MAX
+};
+
 enum stat_codes {
     ESS = 1,
     R2,
@@ -197,16 +223,28 @@ enum data_status_flags {
     GZIPPED_DATA  = 1 << 6
 };
 
+enum register_data_flags {
+    DATAFILE_OPENED = 1,
+    OPENED_VIA_CLI,
+    OPENED_VIA_SESSION,
+    DATA_APPENDED,
+    NULLDATA_STARTED
+};
+
 enum drag_types {
     GRETL_FILENAME,
-    GRETL_POINTER,
-    GRETL_MODEL_POINTER
+    GRETL_DBSERIES_PTR,
+    GRETL_MODEL_PTR,
+    GRETL_REMOTE_DB_PTR,
+    GRETL_REMOTE_FNPKG_PTR,
+    GRETL_GRAPH_FILE
 };
 
 enum file_lists {
     FILE_LIST_DATA,
     FILE_LIST_SESSION,
     FILE_LIST_SCRIPT,
+    FILE_LIST_WDIR,
 };
 
 enum varclick_actions {
@@ -245,8 +283,8 @@ enum dynamic_forecast_status {
 };
 
 #define MULTI_FORMAT_ENABLED(c) (c == SUMMARY || c == VAR_SUMMARY || \
-                                 c == ALL_SUMMARY || \
-	                         c == CORR || c == ALL_CORR || c == FCASTERR || \
+                                 c == ALL_SUMMARY || c == AFR || \
+	                         c == CORR || c == ALL_CORR || \
 	                         c == FCAST || c == COEFFINT || \
 	                         c == COVAR || c == VIEW_MODEL || \
                                  c == VIEW_MODELTABLE || c == VAR || c == VECM || \
