@@ -140,120 +140,53 @@ int create_child_process (char *prog)
     return real_create_child_process(prog, 1);
 }
 
-void startR (char *Rcommand)
+/* try registry for path to Rgui.exe */
+
+static int Rgui_path_from_registry (void)
+{
+    char tmp[MAX_PATH] = {0}; 
+    int err;
+
+    err = read_reg_val(HKEY_LOCAL_MACHINE, "R-core\\R", "InstallPath", tmp);
+
+    if (err) {
+	err = read_reg_val(HKEY_LOCAL_MACHINE, "R", "InstallPath", tmp);
+    }
+
+    if (!err) {
+	strcat(tmp, "\\bin\\");
+	strcat(tmp, "Rgui.exe");
+	*Rcommand = '\0';
+	strncat(Rcommand, tmp, MAXSTR - 1);
+    }
+
+    return err;
+}
+
+/* start R in asynchronous (interactive) mode */
+
+void win32_start_R_async (void)
 {
-    char Rprofile[MAXLEN], Rdata[MAXLEN], Rline[MAXLEN];
     const char *supp1 = "--no-init-file";
     const char *supp2 = "--no-restore-data";
-    int *list;
-    FILE *fp;
-    int err = 0;
-
-    if (!data_status) {
-	warnbox(_("Please open a data file first"));
-	return;
-    }
-
-    build_path(Rprofile, paths.userdir, "gretl.Rprofile", NULL);
-    fp = gretl_fopen(Rprofile, "w");
-    if (fp == NULL) {
-	errbox(_("Couldn't write R startup file"));
-	return;
-    }
-
-    err = ! SetEnvironmentVariable("R_PROFILE", Rprofile);
-    if (err) {
-	errbox(_("Couldn't set R_PROFILE environment variable"));
-	fclose(fp);
-	return;
-    } 	
-
-    build_path(Rdata, paths.userdir, "Rdata.tmp", NULL);
-
-    sprintf(Rline, "store \"%s\" -r", Rdata);
-    list = command_list_from_string(Rline);
-
-    if (list == NULL ||
-	write_data(Rdata, list, (const double **) Z, datainfo, 
-		   OPT_R, NULL)) {
-	errbox(_("Write of R data file failed"));
-	fclose(fp);
-	return; 
-    }
-
-    free(list);
-
-    if (dataset_is_time_series(datainfo)) {
-	fputs("# load data from gretl\n", fp);
-	fputs("vnum <- as.double(R.version$major) + (as.double(R.version$minor) / 10.0)\n", fp);
-	fputs("if (vnum > 1.89) library(stats) else library(ts)\n", fp);
-	fprintf(fp, "source(\"%s\", echo=TRUE)\n", 
-		slash_convert(Rdata, FROM_BACKSLASH));
-    } else {
-	char Rtmp[MAXLEN];
-	FILE *fq;
-
-	build_path(Rtmp, paths.userdir, "Rtmp", NULL);
-	fq = gretl_fopen(Rtmp, "w");
-	if (fq != NULL) {
-	    fputs("# load data from gretl\n", fq);
-	    fputs("library(stats)\n", fq);
-	    fputs("vnum <- as.double(R.version$major) + (as.double(R.version$minor) / 10.0)\n", fq);
-	    fputs("if (vnum > 2.41) library(utils)\n", fq);
-	    fprintf(fq, "gretldata <- read.table(\"%s\", header=TRUE)\n", 
-		    slash_convert(Rdata, FROM_BACKSLASH));
-	    fprintf(fq, "attach(gretldata)\n");
-	    fclose(fq);
-	}
-	fprintf(fp, "source(\"%s\", echo=TRUE)\n", 
-		slash_convert(Rtmp, FROM_BACKSLASH));
-    }
-
-    fclose(fp);
-
-    sprintf(Rline, "\"%s\" %s %s", Rcommand, supp1, supp2);
+    gchar *Rline = NULL;
+    int err;
+    
+    Rline = g_strdup_printf("\"%s\" %s %s", Rcommand, supp1, supp2);
     err = real_create_child_process(Rline, 0);
 
     if (err) {
-	/* failed: try registry for Rgui.exe path? */
-	char tmp[MAX_PATH] = {0}; 
-
-	err = read_reg_val(HKEY_LOCAL_MACHINE, "R-core\\R", "InstallPath", tmp);
-	if (err) {
-	    err = read_reg_val(HKEY_LOCAL_MACHINE, "R", "InstallPath", tmp);
-	}
+	err = Rgui_path_from_registry();
 	if (!err) {
-	    strcat(tmp, "\\bin\\Rgui.exe");
-	    sprintf(Rline, "\"%s\" %s %s", tmp, supp1, supp2);
-	    err = real_create_child_process(Rline, 1);
-	    if (!err) {
-		/* got a good R path, so record it */
-		*Rcommand = '\0';
-		strncat(Rcommand, tmp, MAXSTR - 1);
-	    }
+	    g_free(Rline);
+	    Rline = g_strdup_printf("\"%s\" %s %s", Rcommand, supp1, supp2);
+	    real_create_child_process(Rline, 1);
+	} else {
+	    gui_errmsg(E_EXTERNAL);
 	}
     }
-}
 
-char *slash_convert (char *str, int which)
-{
-    char *p;
-
-    if (str == NULL) {
-	return NULL;
-    }
-
-    p = str;
-    while (*p) {
-	if (which == FROM_BACKSLASH) {
-	    if (*p == '\\') *p = '/';
-	} else if (which == TO_BACKSLASH) {
-	    if (*p == '/') *p = '\\';
-	}
-	p++;
-    }
-
-    return str;
+    g_free(Rline);
 }
 
 static char *substr (char *targ, const char *src, const char *p)
@@ -480,6 +413,7 @@ static int set_network_cfg_filename (const char *prog)
     return 0;
 }
 
+#if 0
 static int set_gd_fontpath (void)
 {
     char fpath[MAX_PATH];
@@ -505,6 +439,7 @@ static int set_gd_fontpath (void)
 
     return 0;        
 }
+#endif
 
 void gretl_win32_init (const char *progname)
 {
@@ -512,7 +447,10 @@ void gretl_win32_init (const char *progname)
 
     wimp_init();
     read_rc(); /* get config info from registry */
+    set_gretl_startdir();
+#if 0
     set_gd_fontpath();
+#endif
     hush_warnings();
     ws_startup(); 
 }
@@ -626,42 +564,6 @@ int win_buf_to_clipboard (const char *buf)
     return 0;
 }
 
-int fnamecmp_win32 (const char *f1, const char *f2)
-{
-    GError *err = NULL;
-    gchar *u1 = NULL, *u2 = NULL;
-    gchar *c1 = NULL, *c2 = NULL;
-    gsize bytes;
-    int ret = 0;
-
-    u1 = g_locale_to_utf8(f1, -1, NULL, &bytes, &err);
-    if (err != NULL) {
-	errbox(err->message);
-	g_error_free(err);
-	return 0;
-    }
-
-    u2 = g_locale_to_utf8(f2, -1, NULL, &bytes, &err);
-    if (err != NULL) {
-	errbox(err->message);
-	g_error_free(err);
-	g_free(u1);
-	return 0;
-    }
-
-    c1 = g_utf8_casefold(u1, -1);
-    c2 = g_utf8_casefold(u2, -1);
-
-    ret = strcmp(c1, c2);
-
-    g_free(u1);
-    g_free(u2);
-    g_free(c1);
-    g_free(c2);
-
-    return ret;
-}
-
 static char *fname_from_fullname (char *fullname)
 {
     char *fname = fullname;
@@ -679,6 +581,51 @@ static char *fname_from_fullname (char *fullname)
     return fname;
 }
 
+static int real_send_file (char *fullname, LPMAPISENDMAIL send_mail)
+{
+    char *fname = fname_from_fullname(fullname);
+    gchar *note = NULL;
+    gchar *tmp = NULL;
+    MapiFileDesc mfd;
+    MapiMessage msg;
+    ULONG sd;
+    int err = 0;
+
+    memset(&mfd, 0, sizeof mfd);
+    memset(&msg, 0, sizeof msg);
+
+    mfd.lpszPathName = fullname;
+    mfd.lpszFileName = fname;
+    mfd.nPosition = 1; /* ? */
+
+    if (strstr(fname, ".gdt") != NULL) {
+	tmp = g_strdup_printf(_("Please find the gretl data file %s attached."), 
+			      fname);
+	msg.lpszSubject  = "dataset";
+    } else {
+	tmp = g_strdup_printf(_("Please find the gretl script %s attached."), 
+			      fname);
+	msg.lpszSubject  = "script";
+    }
+
+    note = g_strdup_printf("%s\n", tmp);
+    g_free(tmp);
+
+    msg.lpszNoteText = note;
+    msg.nFileCount = 1;
+    msg.lpFiles = &mfd;
+
+    sd = send_mail(0L, 0, &msg, MAPI_DIALOG, 0L);
+
+    if (sd != SUCCESS_SUCCESS && sd != MAPI_E_USER_ABORT) {
+	err = 1;
+    }
+
+    g_free(note);
+
+    return err;
+}
+
 int send_file (char *fullname)
 {
     HINSTANCE mapilib = NULL;
@@ -695,49 +642,14 @@ int send_file (char *fullname)
 	} 
     }
 
-    if (err) {
-	errbox("Couldn't access Windows MAPI system");
-    } else {
-	char *fname = fname_from_fullname(fullname);
-	gchar *note = NULL;
-	gchar *tmp = NULL;
-	MapiFileDesc mfd;
-	MapiMessage msg;
-	ULONG sd;
-
-	memset(&mfd, 0, sizeof mfd);
-	memset(&msg, 0, sizeof msg);
-
-	mfd.lpszPathName = fullname;
-	mfd.lpszFileName = fname;
-	mfd.nPosition = 1; /* ? */
-
-	if (strstr(fname, ".gdt") != NULL) {
-	    tmp = g_strdup_printf(_("Please find the gretl data file %s attached."), 
-				  fname);
-	    msg.lpszSubject  = "dataset";
-	} else {
-	    tmp = g_strdup_printf(_("Please find the gretl script %s attached."), 
-				  fname);
-	    msg.lpszSubject  = "script";
-	}
-
-	note = g_strdup_printf("%s\n", tmp);
-	g_free(tmp);
-
-	msg.lpszNoteText = note;
-	msg.nFileCount = 1;
-	msg.lpFiles = &mfd;
-
-	sd = send_mail(0L, 0, &msg, MAPI_DIALOG, 0L);
-
-	if (sd != SUCCESS_SUCCESS && sd != MAPI_E_USER_ABORT) {
-	    errbox("MAPI error sending message");
-	}
-
-	g_free(note);
+    if (!err) {
+	err = real_send_file(fullname, send_mail);
     }
 
+    if (err) {
+	win_show_last_error();
+    } 
+
     if (mapilib != NULL) {
 	FreeLibrary(mapilib);
     }
@@ -748,16 +660,9 @@ int send_file (char *fullname)
 /* copy plot to clipboard by generating an EMF file (enhanced
    metafile), reading it into a buffer, and putting it on the
    clipboard.
-
-   Weirdness: when an emf is put on the clipboard as below, Word 2000
-   behaves thus: a straight "Paste" puts in a version of the graph
-   with squashed up numbers on the axes and no legend text; but a
-   "Paste special" (where one accepts the default of pasting it as an
-   enhanced metafile) puts in an accurate version with correct text.
-   Go figure.  (This is on win98)
 */
 
-static int emf_to_clip (char *emfname)
+int emf_to_clipboard (char *emfname)
 {
     HWND mainw;
     HENHMETAFILE hemf, hemfclip;
@@ -795,78 +700,11 @@ static int emf_to_clip (char *emfname)
     }  	
 
     CloseClipboard();
-
     DeleteEnhMetaFile(hemf);
 
     return 0;
 }
 
-void win32_process_graph (GPT_SPEC *spec, int color, int dest)
-{
-    FILE *fq;
-    PRN *prn;
-    char plottmp[MAXLEN], plotline[MAXLEN];
-    gchar *plotcmd = NULL;
-    gchar *emfname = NULL;
-    int err;
-
-    /* create temporary file to hold the special gnuplot commands */
-    if (user_fopen("gptout.tmp", plottmp, &prn)) {
-	return;
-    }
-
-    /* open the gnuplot source file for the graph */
-    fq = gretl_fopen(spec->fname, "r");
-    if (fq == NULL) {
-	errbox(_("Couldn't access graph info"));
-	gretl_print_destroy(prn);
-	return;
-    }
-
-    /* generate gnuplot source file to make emf */
-    pprintf(prn, "%s\n", get_gretl_emf_term_line(spec->code, color));
-    emfname = g_strdup_printf("%sgpttmp.emf", paths.userdir);
-    pprintf(prn, "set output '%s'\n", emfname);
-
-    while (fgets(plotline, MAXLEN-1, fq)) {
-	if (!strncmp(plotline, "set term", 8) ||
-	    !strncmp(plotline, "set output", 10)) {
-	    continue;
-	}
-	if (!color && strstr(plotline, "set style fill solid")) {
-	    pputs(prn, "set style fill solid 0.3\n");
-	} else if (html_encoded(plotline)) {
-	    pprint_as_latin(prn, plotline, 1);
-	} else {
-	    pputs(prn, plotline);
-	}
-    }
-
-    gretl_print_destroy(prn);
-    fclose(fq);
-
-    /* get gnuplot to create the emf file */
-    plotcmd = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot, 
-			      plottmp);
-    err = winfork(plotcmd, NULL, SW_SHOWMINIMIZED, 0);
-    g_free(plotcmd);
-    remove(plottmp);
-    
-    if (err) {
-        errbox(_("Gnuplot error creating graph"));
-    } else if (dest == WIN32_TO_CLIPBOARD) {
-	err = emf_to_clip(emfname);
-	if (!err) {
-	    infobox(_("To paste, use Edit/Paste special.../Enhanced metafile"));
-	}
-    } else if (dest == WIN32_TO_PRINTER) {
-	err = winprint_graph(emfname);
-    }
-
-    remove(emfname);
-    g_free(emfname);
-}
-
 static long GetRegKey (HKEY key, char *subkey, char *retdata)
 {
     long err;
@@ -951,4 +789,16 @@ int win32_open_file (const char *fname)
     return err;
 }
 
+void win32_raise_window (GtkWidget *w)
+{
+    GtkWidget *top = gtk_widget_get_toplevel(w);
+
+    if (top != NULL && top->window != NULL) {
+	HWND h = GDK_WINDOW_HWND(top->window);
+
+	SetWindowPos(h, HWND_TOP, 0, 0, 0, 0, /* was HWND_TOPMOST */
+		     SWP_NOMOVE | SWP_NOSIZE);
+	gtk_widget_grab_focus(w);
+    }
+}
 
