@@ -25,11 +25,6 @@
 
 #include "plotspec.h"
 
-enum {
-    TO_BACKSLASH,
-    FROM_BACKSLASH
-};
-
 enum {
     WIN32_TO_CLIPBOARD,
     WIN32_TO_PRINTER
@@ -39,9 +34,7 @@ extern int use_wimp;
 
 int create_child_process (char *prog);
 
-void startR (char *Rcommand);
-
-char *slash_convert (char *str, int which);
+void win32_start_R_async (void);
 
 int unmangle (const char *dosname, char *longname);
 
@@ -61,14 +54,14 @@ int prn_to_clipboard (PRN *prn, int copycode);
 
 int win_buf_to_clipboard (const char *buf);
 
-int fnamecmp_win32 (const char *f1, const char *f2);
-
 int send_file (char *fullname);
 
-void win32_process_graph (GPT_SPEC *spec, int color, int dest);
+int emf_to_clipboard (char *emfname);
 
 int browser_open (const char *url);
 
 int win32_open_file (const char *fname);
 
+void win32_raise_window (GtkWidget *w);
+
 #endif /* GRETLWIN32_H */
