@@ -26,12 +26,6 @@ void gretl_stock_icons_init (void);
 gchar *menu_translate (const gchar *path, gpointer p);
 #endif
 
-int probably_native_datafile (const char *fname);
-
-int probably_script_file (const char *fname);
-
-int probably_session_file (const char *fname);
-
 int copyfile (const char *src, const char *dest);
 
 int isdir (const char *path);
@@ -50,8 +44,9 @@ void nomem (void);
 
 void mark_dataset_as_modified (void);
 
-void register_data (char *fname, const char *user_fname,
-		    int record);
+void register_data (int flag);
+
+void register_startup_data (const char *fname);
 
 void do_open_data (GtkWidget *w, gpointer data, int code);
 
@@ -79,6 +74,8 @@ GtkWidget *match_window_by_data (const gpointer p);
 
 GtkWidget *match_window_by_filename (const char *fname);
 
+GtkWidget *build_text_popup (windata_t *vwin);
+
 void mark_content_saved (windata_t *vwin);
 
 windata_t *view_buffer (PRN *prn, int hsize, int vsize, 
@@ -118,12 +115,17 @@ void get_stats_table (void);
 void *gui_get_plugin_function (const char *funcname, 
 			       void **phandle);
 
-int get_worksheet_data (char *fname, int datatype, int append);
+int get_imported_data (char *fname, int ftype, int append);
 
 char *double_underscores (char *targ, const char *src);
 
+void verbose_gerror_report (GError *gerr, const char *src);
+
+int gretl_file_get_contents (const gchar *fname, gchar **contents);
+
+void start_R (const char *buf, int send_data, int interactive);
+
 #ifndef G_OS_WIN32
-void startR (const char *Rcommand);
 int browser_open (const char *url);
 #endif
 
