@@ -38,6 +38,8 @@
 # endif
 #endif
 
+#define PAGE_LINES 47
+
 #ifdef NATIVE_PRINTING
 
 static gchar *user_string (void)
@@ -87,7 +89,7 @@ static char *header_string (void)
 
 #endif /* NATIVE_PRINTING */
 
-/* Windows only: print using Windows spooler */
+/* win32: print using Windows spooler */
 
 #if defined(G_OS_WIN32)
 
@@ -101,7 +103,7 @@ void winprint (char *fullbuf, char *selbuf)
     DOCINFO di;
     TEXTMETRIC lptm;
     BYTE charset;
-    int px, x, y, incr, page_lines = 47;
+    int px, x, y, incr;
     gchar *printbuf = NULL;
     gchar *hdrstart, hdr[90];
     size_t len;
@@ -201,7 +203,7 @@ void winprint (char *fullbuf, char *selbuf)
 	TextOut(dc, x, px / 8, hdr, strlen(hdr));
 	line = 0;
 	y = px/2;
-	while (*printbuf && line < page_lines) { /* lines loop */
+	while (*printbuf && line < PAGE_LINES) { /* lines loop */
 	    len = strcspn(printbuf, "\n");
 	    TextOut(dc, x, y, printbuf, len);
 	    printbuf += len + 1;
@@ -344,7 +346,7 @@ int winprint_graph (char *emfname)
 #include <fcntl.h>
 
 #define GRETL_PRINT_CONFIG_FILE "gretl-print-config"
-#define GRETL_PBM_TMP           "gretltmp.pbm"
+#define GRETL_PBM_TMP "gretltmp.pbm"
 #define GRETL_PNG_TMP "gretltmp.png"
 
 static GdkPixbuf *png_mono_pixbuf (const char *fname);
@@ -352,16 +354,16 @@ static GdkPixbuf *png_mono_pixbuf (const char *fname);
 static GnomePrintConfig *load_gretl_print_config_from_file (void)
 {
     gchar *file_name;
-    gboolean res;
     gchar *contents;
     GnomePrintConfig *gretl_print_config;
+    int err;
 	
     file_name = gnome_util_home_file(GRETL_PRINT_CONFIG_FILE);
 
-    res = g_file_get_contents(file_name, &contents, NULL, NULL);
+    err = gretl_file_get_contents(file_name, &contents);
     g_free(file_name);
 
-    if (res) {
+    if (!err) {
 	gretl_print_config = gnome_print_config_from_string(contents, 0);
 	g_free(contents);
     } else {
@@ -409,6 +411,8 @@ save_gretl_print_config_to_file (GnomePrintConfig *gretl_print_config)
 
 #define GUCAST (const guchar *)
 
+/* still under gnomeprint conditional */
+
 void winprint (char *fullbuf, char *selbuf)
 {
     GnomePrintJob *job;
@@ -420,7 +424,6 @@ void winprint (char *fullbuf, char *selbuf)
     GnomeFont *font = NULL;
     gchar *hdrstart;
     char *p, linebuf[90], hdr[90];
-    int page_lines = 47;
     int x, y, line, page;
     size_t len;
 
@@ -471,7 +474,7 @@ void winprint (char *fullbuf, char *selbuf)
 	gnome_print_moveto(gpc, x, y);
 	gnome_print_show(gpc, GUCAST hdr);
 	y = 720;
-	while (*p && line < page_lines) { /* lines loop */
+	while (*p && line < PAGE_LINES) { /* lines loop */
 	    len = strcspn(p, "\n");
 	    *linebuf = '\0';
 	    strncat(linebuf, p, len);
@@ -614,9 +617,7 @@ void gtk_print_graph (const char *fname)
     gtk_widget_destroy(dialog);
 }
 
-#endif /* G_OS_WIN32, USE_GNOMEPRINT */
-
-#ifdef USE_GNOMEPRINT
+/* still under gnomeprint conditional */
 
 static GdkPixbuf *png_mono_pixbuf (const char *fname)
 {
@@ -624,7 +625,7 @@ static GdkPixbuf *png_mono_pixbuf (const char *fname)
     char cmd[MAXLEN], temp[MAXLEN], fline[MAXLEN];
     GdkPixbuf *pbuf = NULL;
 
-    sprintf(temp, "%sgpttmp", paths.userdir);
+    sprintf(temp, "%sgpttmp", paths.dotdir);
 
     ftmp = gretl_tempfile_open(temp);
     if (ftmp == NULL) {
@@ -640,7 +641,7 @@ static GdkPixbuf *png_mono_pixbuf (const char *fname)
 
     fprintf(ftmp, "set term pbm mono\n"
 	    "set output '%s%s'\n", 
-	    paths.userdir, GRETL_PBM_TMP);
+	    paths.dotdir, GRETL_PBM_TMP);
 
     while (fgets(fline, MAXLEN-1, fsrc)) {
 	if (strncmp(fline, "set term", 8) && 
@@ -661,7 +662,7 @@ static GdkPixbuf *png_mono_pixbuf (const char *fname)
 
     remove(temp);
 
-    build_path(temp, paths.userdir, GRETL_PBM_TMP, NULL);
+    build_path(temp, paths.dotdir, GRETL_PBM_TMP, NULL);
     pbuf = gdk_pixbuf_new_from_file(temp, NULL);
     remove(temp);
 
@@ -672,6 +673,8 @@ static GdkPixbuf *png_mono_pixbuf (const char *fname)
 
 #ifdef GTK_PRINTING
 
+/* native GTK printing with recent GTK+ */
+
 #define GRETL_PNG_TMP "gretltmp.png"
 
 struct print_info {
@@ -692,10 +695,6 @@ static void begin_text_print (GtkPrintOperation *op,
     PangoFontDescription *desc;
     GtkPageSetup *setup;
     gdouble x, y;
-    int lines = 0;
-    const char *p;
-
-    pinfo->pagelines = 54; /* FIXME? */
  
     setup = gtk_print_context_get_page_setup(context);
 
@@ -735,18 +734,6 @@ static void begin_text_print (GtkPrintOperation *op,
     pango_font_description_free(desc);
     pango_layout_set_width(pinfo->layout, -1);
     pango_layout_set_alignment(pinfo->layout, PANGO_ALIGN_LEFT);
-
-    p = pinfo->buf;
-    while (*p) {
-	if (*p == '\n') {
-	    lines++;
-	}
-	p++;
-    }
-
-    pinfo->n_pages = lines / pinfo->pagelines + 
-	(lines % pinfo->pagelines != 0);
-    gtk_print_operation_set_n_pages(op, pinfo->n_pages);
 }
 
 static void
@@ -791,6 +778,23 @@ draw_text_page (GtkPrintOperation *op, GtkPrintContext *context,
     pango_cairo_show_layout(pinfo->cr, pinfo->layout);
 }
 
+static void job_set_n_pages (GtkPrintOperation *op, struct print_info *pinfo)
+{
+    const char *s = pinfo->buf;
+    int lines = 0;
+
+    while (*s) {
+	if (*s == '\n') {
+	    lines++;
+	}
+	s++;
+    }
+
+    pinfo->n_pages = lines / pinfo->pagelines + 
+	(lines % pinfo->pagelines != 0);
+    gtk_print_operation_set_n_pages(op, pinfo->n_pages);
+}
+
 static GtkPrintSettings *settings = NULL;
 
 void winprint (char *fullbuf, char *selbuf)
@@ -808,11 +812,15 @@ void winprint (char *fullbuf, char *selbuf)
 
     gtk_print_operation_set_use_full_page(op, FALSE);
     gtk_print_operation_set_unit(op, GTK_UNIT_POINTS);
+    gtk_print_operation_set_n_pages(op, 1); /* FIXME */
 
     pinfo.buf = (selbuf != NULL)? selbuf : fullbuf;
     pinfo.p = pinfo.buf;
     pinfo.hdr = header_string();
     pinfo.layout = NULL;
+    pinfo.pagelines = 54; /* FIXME */
+
+    job_set_n_pages(op, &pinfo); 
 
     g_signal_connect(op, "begin_print", G_CALLBACK(begin_text_print), &pinfo);
     g_signal_connect(op, "draw_page", G_CALLBACK(draw_text_page), &pinfo);
@@ -847,7 +855,7 @@ static int make_png_file (const char *fname,
     FILE *fsrc, *ftmp;
     char cmd[MAXLEN], temp[MAXLEN], fline[MAXLEN];
 
-    sprintf(temp, "%sgpttmp", paths.userdir);
+    sprintf(temp, "%sgpttmp", paths.dotdir);
 
     ftmp = gretl_tempfile_open(temp);
     if (ftmp == NULL) {
@@ -861,7 +869,7 @@ static int make_png_file (const char *fname,
 	return 1;
     }
 
-    build_path(pngname, paths.userdir, GRETL_PNG_TMP, NULL);
+    build_path(pngname, paths.dotdir, GRETL_PNG_TMP, NULL);
 
     while (fgets(fline, MAXLEN-1, fsrc)) {
 	if (!strncmp(fline, "set output", 10)) {
@@ -1686,7 +1694,7 @@ static void texprint_fcast_with_errs (const FITRESID *fr,
     int t;
 
     pputs(prn, "\\begin{center}\n");
-    if (fr->model_ci == ARMA) {
+    if (fr->asymp) {
 	pprintf(prn, _("For 95\\%% confidence intervals, $z(.025) = %.2f$\n\n"), 
 		1.96);
     } else {
@@ -1791,7 +1799,7 @@ static void rtfprint_fcast_with_errs (const FITRESID *fr,
     double maxerr;
     char tmp[128];
 
-    if (fr->model_ci == ARMA) {
+    if (fr->asymp) {
 	sprintf(tmp, I_("For 95%% confidence intervals, z(.025) = %.2f"), 
 		1.96);
     } else {
@@ -1883,9 +1891,11 @@ texprint_coeff_interval (const CoeffIntervals *cf, int i, PRN *prn)
 static void texprint_confints (const CoeffIntervals *cf, PRN *prn)
 {
     char pt = get_local_decpoint();
+    double tail = cf->alpha / 2;
+    gchar *cstr;
     int i;
 
-    pprintf(prn, "$t(%d, .025) = %.3f$\n\n", cf->df, tcrit95(cf->df));
+    pprintf(prn, "$t(%d, %g) = %.3f$\n\n", cf->df, tail, cf->t);
 
     pputs(prn, "%% The table below needs the \"dcolumn\" package\n\n");
 
@@ -1893,12 +1903,15 @@ static void texprint_confints (const CoeffIntervals *cf, PRN *prn)
 	    "\\begin{tabular}{rD{%c}{%c}{-1}D{%c}{%c}{-1}D{%c}{%c}{-1}}\n",
 	    pt, pt, pt, pt, pt, pt);
 
+    cstr = g_strdup_printf(I_("%g\\%% confidence interval"), 100 * (1 - cf->alpha));
+
     pprintf(prn, " %s%%\n"
 	    " & \\multicolumn{1}{c}{%s}%%\n"
 	    "  & \\multicolumn{2}{c}{%s}\\\\\n",
 	    I_("Variable"), I_("Coefficient"),
-	    /* xgettext:no-c-format */
-	    I_("95\\% confidence interval"));
+	    cstr);
+
+    g_free(cstr);
 
     pprintf(prn, " & & \\multicolumn{1}{c}{%s}%%\n"
 	    "  & \\multicolumn{1}{c}{%s}\\\\\n",
@@ -1934,10 +1947,14 @@ rtfprint_coeff_interval (const CoeffIntervals *cf, int i, PRN *prn)
 
 static void rtfprint_confints (const CoeffIntervals *cf, PRN *prn)
 {
+    double tail = cf->alpha / 2;
+    gchar *cstr;
     int i;
 
-    pprintf(prn, "{\\rtf1\\par\n\\qc t(%d, .025) = %.3f\\par\n\\par\n", 
-	    cf->df, tcrit95(cf->df));
+    pprintf(prn, "{\\rtf1\\par\n\\qc t(%d, %g) = %.3f\\par\n\\par\n", 
+	    cf->df, tail, cf->t);
+
+    cstr = g_strdup_printf(I_("%g\\%% confidence interval"), 100 * (1 - cf->alpha));
 
     pputs(prn, "{" CF_ROW "\\intbl ");
     pprintf(prn, 
@@ -1946,8 +1963,9 @@ static void rtfprint_confints (const CoeffIntervals *cf, PRN *prn)
 	    " \\qc %s\\cell"
 	    " \\intbl \\row\n", 
 	    I_("Variable"), I_("Coefficient"), 
-	    /* xgettext:no-c-format */
-	    I_("95% confidence interval"));
+	    cstr);
+
+    g_free(cstr);
 
     for (i=0; i<cf->ncoeff; i++) {
 	rtfprint_coeff_interval(cf, i, prn);
@@ -2066,7 +2084,10 @@ int csv_to_clipboard (void)
     int *list = NULL;
     int err = 0;
 
-    delimiter_dialog(&opt);
+    if (delimiter_dialog(&opt)) {
+	return 0;
+    }
+
     data_save_selection_wrapper(COPY_CSV, GINT_TO_POINTER(opt));
 
     if (storelist != NULL && *storelist != 0) {
@@ -2088,7 +2109,9 @@ int csv_selected_to_clipboard (void)
     int err = 0;
 
     if (list != NULL) {
-	delimiter_dialog(NULL);
+	if (delimiter_dialog(NULL)) {
+	    return 0;
+	}
 	err = real_csv_to_clipboard(list);
 	free(list);
     }
