@@ -37,6 +37,12 @@
 
 #define HDEBUG 0
 
+enum {
+    HELP_CLI =   1 << 0,
+    HELP_EN  =   1 << 1,
+    HELP_FUNCS = 1 << 2
+};
+
 static int translated_helpfile = -1;
 static char *en_gui_helpfile;
 static char *en_cli_helpfile;
@@ -55,14 +61,14 @@ struct help_head_t {
 static help_head **cli_heads, **gui_heads;
 static help_head **en_cli_heads, **en_gui_heads;
 
-static windata_t *helpwin (int cli, int en);
-static void real_do_help (int hcode, int pos, int cli, int en);
+static windata_t *make_helpwin (int flags);
+static void real_do_help (int hcode, int pos, int flags);
 static void en_text_cmdref (gpointer p, guint u, GtkWidget *w);
 
 /* searching stuff */
 static void find_in_text (GtkWidget *widget, gpointer data);
 static void find_in_listbox (GtkWidget *widget, gpointer data);
-static void find_string_dialog (void (*findfunc)(), gpointer data);
+static void find_string_dialog (void (*findfunc)(), windata_t *vwin);
 
 static GtkWidget *find_window = NULL;
 static GtkWidget *find_entry;
@@ -85,6 +91,7 @@ static struct gui_help_item gui_help_items[] = {
     { GR_PLOT,        "graphing" },
     { GR_XY,          "graphing" },
     { GR_DUMMY,       "factorized" },
+    { GR_XYZ,         "controlled" },
     { BXPLOT,         "boxplots" },
     { GR_BOX,         "boxplots" },
     { GR_NBOX,        "boxplots" },
@@ -122,7 +129,25 @@ static struct gui_help_item gui_help_items[] = {
     { BOOTSTRAP,      "bootstrap" },
     { TRANSPOS,       "transpos" },
     { DATASORT,       "datasort" },
-    { -1,          NULL },
+    { WORKDIR,        "working-dir" },
+    { DFGLS,          "dfgls" },
+    { -1,             NULL },
+};
+
+enum {
+    COMPAT_CORC = GUI_CMD_MAX + 1,
+    COMPAT_FCASTERR,
+    COMPAT_HILU,
+    COMPAT_PWE
+};
+
+static struct gui_help_item compat_help_items[] = {
+    { GUI_CMD_MAX,     "nothing" },
+    { COMPAT_CORC,     "corc" },
+    { COMPAT_FCASTERR, "fcasterr" },
+    { COMPAT_HILU,     "hilu" },
+    { COMPAT_PWE,      "pwe" },
+    { -1,              NULL }
 };
 
 /* state the topic headings from the help files so they 
@@ -153,6 +178,19 @@ static int extra_command_number (const char *s)
     return -1;
 }
 
+static int compat_command_number (const char *s)
+{
+    int i;
+
+    for (i=1; compat_help_items[i].code > 0; i++) {
+	if (!strcmp(s, compat_help_items[i].string)) {
+	    return compat_help_items[i].code;
+	}
+    }
+
+    return -1;
+}
+
 static char *help_string_from_cmd (int cmd)
 {
     int i;
@@ -163,6 +201,13 @@ static char *help_string_from_cmd (int cmd)
 	}
     }
 
+    for (i=1; compat_help_items[i].code > 0; i++) {
+	if (cmd == compat_help_items[i].code) {
+	    return compat_help_items[i].string;
+	}
+    }
+
+
     return NULL;    
 }
 
@@ -273,6 +318,10 @@ static int help_topic_number (const char *word, int gui)
 	hnum = extra_command_number(word);
     }
 
+    if (hnum <= 0) {
+	hnum = compat_command_number(word);
+    }
+
     return hnum;
 }
 
@@ -672,32 +721,154 @@ void helpfile_init (void)
     }
 }
 
+/* apparatus for genr functions help */
+
+typedef struct func_finder_ func_finder;
+
+struct func_finder_ {
+    int pos;
+    char word[12];
+};
+
+static func_finder *ffinder;
+static int n_help_funcs;
+
+/* read the functions helpfile and build a mapping from
+   function name to offset in file */
+
+static int make_func_help_mapping (void)
+{
+    int err = 0;
+
+    if (ffinder == NULL) {
+	gchar *fname = NULL;
+	gchar *s, *buf = NULL;
+	char word[12];
+	int i = 0, pos = 0;
+	int err;
+
+	fname = g_strdup_printf("%s%s", paths.gretldir, _("genrgui.hlp"));
+
+	err = gretl_file_get_contents(fname, &buf);
+	if (err) {
+	    g_free(fname);
+	    return err;
+	}
+
+	s = buf;
+	while (*s) {
+	    if (*s == '\n' && *(s+1) == '#' && *(s+2) != '#') {
+		n_help_funcs++;
+	    }
+	    s++;
+	}
+
+	ffinder = mymalloc(n_help_funcs * sizeof *ffinder);
+
+	if (ffinder != NULL) {
+	    s = buf;
+	    while (*s) {
+		if (*s == '\n' && *(s+1) == '#' && 
+		    *(s+2) != '#' && *(s+2) != '\0') {
+		    if (sscanf(s+2, "%10s", word)) {
+			ffinder[i].pos = pos + 1;
+			strcpy(ffinder[i].word, word);
+			i++;
+		    }
+		}
+		s++;
+		pos++;
+	    }
+	} else {
+	    err = 1;
+	    n_help_funcs = 0;
+	}
+
+	g_free(buf);
+	g_free(fname);
+    }
+
+    return err;
+}
+
+int function_help_index_from_word (const char *s)
+{
+    int i, idx = 0;
+
+    if (n_help_funcs == 0) {
+	make_func_help_mapping();
+    }
+
+    for (i=0; i<n_help_funcs; i++) {
+	if (!strcmp(ffinder[i].word, s)) {
+	    idx = i+1;
+	    break;
+	}
+    }
+
+    return idx;
+}
+
+static int function_help_pos_from_word (const char *s)
+{
+    int i, pos = 0;
+
+    if (n_help_funcs == 0) {
+	make_func_help_mapping();
+    }
+
+    for (i=0; i<n_help_funcs; i++) {
+	if (!strcmp(ffinder[i].word, s)) {
+	    pos = ffinder[i].pos;
+	}
+    }
+
+    return pos;
+}
+
+static int help_pos_from_function_index (int fnum)
+{
+    int pos = 0;
+
+    if (n_help_funcs == 0) {
+	make_func_help_mapping();
+    }
+
+    if (fnum > 0 && fnum <= n_help_funcs) {
+	pos = ffinder[fnum-1].pos;
+    } 
+
+    return pos;
+}
+
+/* end apparatus for genr functions help */
+
 static void do_gui_help (gpointer p, guint pos, GtkWidget *w) 
 {
     int hcode = GPOINTER_TO_INT(p);
 
-    real_do_help(hcode, pos, 0, 0);
+    real_do_help(hcode, pos, 0);
 }
 
 static void do_en_gui_help (gpointer p, guint pos, GtkWidget *w) 
 {
     int hcode = GPOINTER_TO_INT(p);
 
-    real_do_help(hcode, pos, 0, 1);
+    real_do_help(hcode, pos, HELP_EN);
 }
 
 static void do_cli_help (gpointer p, guint pos, GtkWidget *w) 
 {
     int hcode = GPOINTER_TO_INT(p);
 
-    real_do_help(hcode, pos, 1, 0);
+    real_do_help(hcode, pos, HELP_CLI);
 }
 
 static void do_en_cli_help (gpointer p, guint pos, GtkWidget *w) 
 {
     int hcode = GPOINTER_TO_INT(p);
 
-    real_do_help(hcode, pos, 1, 1);
+    real_do_help(hcode, pos, HELP_CLI | HELP_EN);
 }
 
 static char *get_gui_help_string (int pos)
@@ -715,6 +886,22 @@ static char *get_gui_help_string (int pos)
     return NULL;
 }
 
+static char *get_compat_help_string (int pos)
+{
+    int i, j;
+
+    for (i=0; cli_heads[i] != NULL; i++) { 
+	for (j=0; j<cli_heads[i]->ntopics; j++) {
+	    if (pos == cli_heads[i]->pos[j]) {
+		return help_string_from_cmd(cli_heads[i]->topics[j]);
+	    }
+	}
+    }
+
+    return NULL;
+}
+
+
 static int gui_pos_from_cmd (int cmd, int en)
 {
     help_head **heads;
@@ -780,8 +967,10 @@ static void en_help_callback (gpointer p, int cli, GtkWidget *w)
 {
     windata_t *hwin = (windata_t *) p;
     int pos, hc = hwin->active_var;
+    int flags = HELP_EN;
 
     if (cli) {
+	flags |= HELP_CLI;
 	pos = cli_pos_from_cmd(hc, 1);
 	if (pos < 0) {
 	    pos = 0;
@@ -790,7 +979,7 @@ static void en_help_callback (gpointer p, int cli, GtkWidget *w)
 	pos = gui_pos_from_cmd(hc, 1);
     }
 
-    real_do_help(hc, pos, cli, 1);
+    real_do_help(hc, pos, flags);
 }
 
 static void add_en_help_item (windata_t *hwin, int cli)
@@ -813,38 +1002,45 @@ static void add_en_help_item (windata_t *hwin, int cli)
     g_free(item.path);
 }
 
-static void add_help_topics (windata_t *hwin, int cli, int en)
+static void add_help_topics (windata_t *hwin, int flags)
 {
     GtkItemFactoryEntry hitem;
     const gchar *mpath = N_("/_Topics");
-    help_head **hds;
+    help_head **hds = NULL;
+    int en = (flags & HELP_EN);
+    int cli = (flags & HELP_CLI);
+    int funcs = (flags & HELP_FUNCS);
     int i, j;
 
     if (en) {
-	hds = (cli)? en_cli_heads : en_gui_heads;
+	hds = (cli)? en_cli_heads : (funcs)? NULL : en_gui_heads;
     } else {
-	hds = (cli)? cli_heads : gui_heads;
-    }
-
-    /* Are there any topics to add? */
-    if (hds == NULL) {
-	return;
+	hds = (cli)? cli_heads : (funcs)? NULL : gui_heads;
     }
 
     hitem.accelerator = NULL;
 
-    if (cli) {
-	/* Add general index as "topic" */
+    if (cli || funcs) {
+	/* Add general index as (first) "topic" */
 	hitem.callback_action = 0; 
 	hitem.item_type = NULL;
 	hitem.path = g_strdup_printf("%s/%s", mpath, (en)? "Index" : _("Index"));
-	hitem.callback = (en)? en_text_cmdref : plain_text_cmdref;
+	if (funcs) {
+	    hitem.callback = genr_funcs_ref;
+	} else {
+	    hitem.callback = (en)? en_text_cmdref : plain_text_cmdref;
+	}
 	gtk_item_factory_create_item(hwin->ifac, &hitem, 
 				     GINT_TO_POINTER(0), 
 				     1);
 	g_free(hitem.path);
     }
 
+    /* Are there any actual topics to add? */
+    if (hds == NULL) {
+	return;
+    }
+
     /* put the topics under the menu heading */
     for (i=0; hds[i] != NULL; i++) {
 	const char *headname = (en)? hds[i]->name : _(hds[i]->name);
@@ -910,7 +1106,15 @@ static void add_help_topics (windata_t *hwin, int cli, int en)
 			topic_ok = 0;
 		    }
 		} else {
-		    topic_ok = 0;
+		    char *cstr = get_compat_help_string(hds[i]->pos[j]);
+
+		    if (cstr != NULL) {
+			hitem.path = 
+			    g_strdup_printf("%s/%s/%s", 
+					    mpath, headname, cstr);
+		    } else {		    
+			topic_ok = 0;
+		    }
 		}
 	    }
 
@@ -929,13 +1133,30 @@ static void add_help_topics (windata_t *hwin, int cli, int en)
     }
 }
 
-static windata_t *helpwin (int cli, int en) 
+static char *funcs_helpfile (void)
+{
+    static char fname[MAXLEN];
+
+    if (*fname == '\0') {
+	sprintf(fname, "%s%s", paths.gretldir, _("genrgui.hlp"));
+    }
+
+    return fname;
+}
+
+static windata_t *make_helpwin (int flags) 
 {
     windata_t *vwin = NULL;
     char *helpfile = NULL;
+    int en = (flags & HELP_EN);
+    int cli = (flags & HELP_CLI);
+    int funcs = (flags & HELP_FUNCS);
     int role;
 
-    if (cli) {
+    if (funcs) {
+	helpfile = funcs_helpfile();
+	role = FUNCS_HELP;
+    } else if (cli) {
 	helpfile = (en)? en_cli_helpfile : paths.cmd_helpfile;
 	role = (en)? CLI_HELP_EN : CLI_HELP;
     } else {
@@ -943,15 +1164,16 @@ static windata_t *helpwin (int cli, int en)
 	role = (en)? GUI_HELP_EN : GUI_HELP;
     }
 
-    if (helpfile == NULL) return NULL;
+    if (helpfile != NULL) {
+	vwin = view_help_file(helpfile, role, help_menu_items);
+    }
 
-    vwin = view_help_file(helpfile, role, help_menu_items);
-
-    add_help_topics(vwin, cli, en);
-
-    if (translated_helpfile && !en) {
-	add_en_help_item(vwin, cli);
-    }    
+    if (vwin != NULL) {
+	add_help_topics(vwin, flags);
+	if (!funcs && translated_helpfile && !en) {
+	    add_en_help_item(vwin, cli);
+	} 
+    }   
 
     return vwin;
 }
@@ -959,21 +1181,21 @@ static windata_t *helpwin (int cli, int en)
 void context_help (GtkWidget *widget, gpointer data)
 {
     int pos, hcode = GPOINTER_TO_INT(data);
-    int en = 0;
+    int flags = 0;
 
     pos = gui_pos_from_cmd(hcode, 0);
 
     if (pos < 0 && translated_helpfile) {
-	en = 1;
-	pos = gui_pos_from_cmd(hcode, en);
+	flags = HELP_EN;
+	pos = gui_pos_from_cmd(hcode, 1);
     }
 
 #if HDEBUG
     fprintf(stderr, "context_help: hcode=%d, pos=%d, en=%d\n", 
-	    hcode, pos, en);
+	    hcode, pos, (flags & HELP_EN)? 1 : 0);
 #endif
 
-    real_do_help(hcode, pos, 0, en);
+    real_do_help(hcode, pos, flags);
 }
 
 static gboolean nullify_hwin (GtkWidget *w, windata_t **phwin)
@@ -982,14 +1204,18 @@ static gboolean nullify_hwin (GtkWidget *w, windata_t **phwin)
     return FALSE;
 }
 
-static void real_do_help (int hcode, int pos, int cli, int en)
+static void real_do_help (int hcode, int pos, int flags)
 {
     static windata_t *gui_hwin;
     static windata_t *cli_hwin;
     static windata_t *en_gui_hwin;
     static windata_t *en_cli_hwin;
+    static windata_t *funcs_hwin;
 
     windata_t *hwin;
+    int cli = (flags & HELP_CLI);
+    int en = (flags & HELP_EN);
+    int funcs = (flags & HELP_FUNCS);
 
     if (pos < 0) {
 	dummy_call();
@@ -997,22 +1223,23 @@ static void real_do_help (int hcode, int pos, int cli, int en)
     }
 
 #if HDEBUG
-    fprintf(stderr, "real_do_help: hcode=%d, pos=%d, cli=%d, en=%d\n",
-	    hcode, pos, cli, en);
+    fprintf(stderr, "real_do_help: hcode=%d, pos=%d, cli=%d, en=%d, funcs=%d\n",
+	    hcode, pos, cli, en, funcs);
     fprintf(stderr, "gui_hwin = %p\n", (void *) gui_hwin);
     fprintf(stderr, "cli_hwin = %p\n", (void *) cli_hwin);
     fprintf(stderr, "en_gui_hwin = %p\n", (void *) en_gui_hwin);
     fprintf(stderr, "en_cli_hwin = %p\n", (void *) en_cli_hwin);
+    fprintf(stderr, "funcs_hwin = %p\n", (void *) funcs_hwin);
 #endif
 
     if (en) {
 	hwin = (cli)? en_cli_hwin : en_gui_hwin;
     } else {
-	hwin = (cli)? cli_hwin : gui_hwin;
+	hwin = (funcs)? funcs_hwin : (cli)? cli_hwin : gui_hwin;
     }
 
     if (hwin == NULL) {
-	hwin = helpwin(cli, en);
+	hwin = make_helpwin(flags);
 	if (hwin != NULL) {
 	    windata_t **phwin;
 
@@ -1025,7 +1252,10 @@ static void real_do_help (int hcode, int pos, int cli, int en)
 		    phwin = &en_gui_hwin;
 		}
 	    } else {
-		if (cli) {
+		if (funcs) {
+		    funcs_hwin = hwin;
+		    phwin = &funcs_hwin;
+		} else if (cli) {
 		    cli_hwin = hwin;
 		    phwin = &cli_hwin;
 		} else {
@@ -1037,8 +1267,7 @@ static void real_do_help (int hcode, int pos, int cli, int en)
 			     G_CALLBACK(nullify_hwin), phwin);
 	}
     } else {
-	gdk_window_show(hwin->w->parent->window);
-	gdk_window_raise(hwin->w->parent->window);
+	gtk_window_present(GTK_WINDOW(gtk_widget_get_toplevel(hwin->w)));
     }
 
 #if HDEBUG
@@ -1047,19 +1276,21 @@ static void real_do_help (int hcode, int pos, int cli, int en)
 	    (void *) hwin, hcode, pos, en);
 #endif
 
-    set_help_topic_buffer(hwin, hcode, pos, en);
+    if (hwin != NULL) {
+	set_help_topic_buffer(hwin, hcode, pos, en);
+    }
 }
 
 /* called from main menu in gretl.c; also used as callback from
-   command ref index in textbuf.c (gtk2 version only)
+   command ref index in textbuf.c
 */
 
 void plain_text_cmdref (gpointer p, guint cmdnum, GtkWidget *w)
 {
     /* pos = 0 gives index of commands */
     int pos = 0;
+    int flags = HELP_CLI;
     int en = 0;
-    int cli = 1;
 
     if (w == NULL && p != NULL) {
 	en = GPOINTER_TO_INT(p);
@@ -1075,20 +1306,65 @@ void plain_text_cmdref (gpointer p, guint cmdnum, GtkWidget *w)
 	if (pos < 0 && !en && translated_helpfile == 1) {
 	    /* no translated entry: fall back on English */
 	    en = 1;
-	    pos = cli_pos_from_cmd(cmdnum, en);
+	    pos = cli_pos_from_cmd(cmdnum, 1);
 	}
     }
 
-    real_do_help(cmdnum, pos, cli, en);
+    if (en) {
+	flags |= HELP_EN;
+    }
+
+    real_do_help(cmdnum, pos, flags);
 } 
 
+/* called from main menu in gretl.c; also used as callback from
+   command ref index in textbuf.c
+*/
+
+void genr_funcs_ref (gpointer p, guint fnum, GtkWidget *w)
+{
+    int pos = help_pos_from_function_index(fnum);
+
+    real_do_help(fnum, pos, HELP_FUNCS);    
+}
+
 static void en_text_cmdref (gpointer p, guint u, GtkWidget *w)
 {
-    real_do_help(0, 0, 1, 1);
+    real_do_help(0, 0, HELP_CLI | HELP_EN);
 } 
 
-gint edit_script_help (GtkWidget *widget, GdkEventButton *b,
-		       windata_t *vwin)
+static int help_pos_from_string (const char *s, int *en, 
+				 int *hcode, int *flags)
+{
+    char word[12];
+    int pos;
+
+    *word = '\0';
+    strncat(word, s, 11);
+
+    *hcode = gretl_command_number(word);
+    pos = cli_pos_from_cmd(*hcode, *en);
+
+    if (pos < 0 && translated_helpfile) {
+	pos = cli_pos_from_cmd(*hcode, 1);
+	if (pos >= 0) {
+	    *en = 1;
+	}
+    }
+
+    if (pos < 0) {
+	/* try function instead of command */
+	pos = function_help_pos_from_word(word);
+	if (pos > 0) {
+	    *flags |= HELP_FUNCS;
+	}
+    }
+
+    return pos;
+}
+
+gint interactive_script_help (GtkWidget *widget, GdkEventButton *b,
+			      windata_t *vwin)
 {
     if (!window_help_is_active(vwin)) { 
 	/* command help not activated */
@@ -1097,6 +1373,7 @@ gint edit_script_help (GtkWidget *widget, GdkEventButton *b,
 	gchar *text = NULL;
 	int pos = -1;
 	int hcode = 0;
+	int flags = HELP_CLI;
 	int en = 0;
 	GtkTextBuffer *buf;
 	GtkTextIter iter;
@@ -1121,6 +1398,24 @@ gint edit_script_help (GtkWidget *widget, GdkEventButton *b,
 
 	    text = gtk_text_buffer_get_text(buf, &w_start, &w_end, FALSE);
 
+	    /* dollar accessors! */
+	    if (text != NULL) {
+		GtkTextIter dstart = w_start;
+		gchar *dtest = NULL;
+
+		if (gtk_text_iter_backward_char(&dstart)) {
+		    dtest = gtk_text_buffer_get_text(buf, &dstart, 
+						     &w_start, FALSE);
+		    if (*dtest == '$') {
+			gchar *s = g_strdup_printf("$%s", text);
+			
+			g_free(text);
+			text = s;
+		    }
+		    g_free(dtest);
+		}
+	    }
+
 	    /* "coint2" command! */
 	    if (text != NULL && !strcmp(text, "coint") && 
 		gtk_text_iter_forward_char(&w_end)) {
@@ -1138,23 +1433,21 @@ gint edit_script_help (GtkWidget *widget, GdkEventButton *b,
 	} 
 
 	if (text != NULL && *text != '\0') {
-	    char word[9];
-
-	    *word = '\0';
-	    strncat(word, text, 8);
-	    hcode = gretl_command_number(word);
-	    pos = cli_pos_from_cmd(hcode, en);
-	    if (pos < 0 && translated_helpfile) {
-		en = 1;
-		pos = cli_pos_from_cmd(hcode, en);
-	    }
+	    pos = help_pos_from_string(text, &en, &hcode, &flags);
 	} 
 
 	g_free(text);
 	unset_window_help_active(vwin);
 	text_set_cursor(vwin->w, 0);
 
-	real_do_help(hcode, pos, 1, en);
+	if (pos <= 0) {
+	    warnbox(_("Sorry, help not found"));
+	} else {
+	    if (en) {
+		flags |= HELP_EN;
+	    }
+	    real_do_help(hcode, pos, flags);
+	}
     }
 
     return FALSE;
@@ -1410,23 +1703,22 @@ static void parent_find (GtkWidget *finder, windata_t *caller)
     }
 }
 
-static void find_string_dialog (void (*findfunc)(), gpointer p)
+static void find_string_dialog (void (*findfunc)(), windata_t *vwin)
 {
-    windata_t *mydat = (windata_t *) p;
     GtkWidget *label;
     GtkWidget *button;
     GtkWidget *hbox;
 
     if (find_window != NULL) {
-	g_object_set_data(G_OBJECT(find_window), "windat", mydat);
-	parent_find(find_window, mydat);
+	g_object_set_data(G_OBJECT(find_window), "windat", vwin);
+	parent_find(find_window, vwin);
 	gdk_window_raise(find_window->window);
 	return;
     }
 
     find_window = gretl_dialog_new(_("gretl: find"), NULL, 0);
-    g_object_set_data(G_OBJECT(find_window), "windat", mydat);
-    parent_find(find_window, mydat);
+    g_object_set_data(G_OBJECT(find_window), "windat", vwin);
+    parent_find(find_window, vwin);
 
     g_signal_connect(G_OBJECT(find_window), "destroy",
 		     G_CALLBACK(close_find_dialog),
@@ -1475,9 +1767,9 @@ static void find_string_dialog (void (*findfunc)(), gpointer p)
     gtk_widget_show(find_window);
 }
 
-void text_find_callback (GtkWidget *w, gpointer data)
+void text_find_callback (GtkWidget *w, windata_t *vwin)
 {
-    find_string_dialog(find_in_text, data);
+    find_string_dialog(find_in_text, vwin);
 }
 
 static GtkTooltips *gretl_tips;
@@ -1524,9 +1816,9 @@ static int get_writable_path (char *path, const char *fname)
 
     if (!sysdoc_writable && userdoc_writable < 0) {
 	userdoc_writable = 0;
-	sprintf(path, "%sdoc", paths.userdir);
+	sprintf(path, "%sdoc", paths.dotdir);
 	if (gretl_mkdir(path) == 0) {
-	    sprintf(path, "%sdoc%c%s", paths.userdir, SLASH, fname);
+	    sprintf(path, "%sdoc%c%s", paths.dotdir, SLASH, fname);
 	    fp = gretl_fopen(path, "w");
 	    if (fp != NULL) {
 		userdoc_writable = 1;
@@ -1576,7 +1868,7 @@ static int find_or_download_pdf (int uguide, int i, char *fullpath)
     }
 
     /* or maybe in user dir? */
-    sprintf(fullpath, "%sdoc%c%s", paths.userdir, SLASH, fname);
+    sprintf(fullpath, "%sdoc%c%s", paths.dotdir, SLASH, fname);
     fp = gretl_fopen(fullpath, "r");
     if (fp != NULL) {
 	fclose(fp);
@@ -1621,6 +1913,6 @@ void display_pdf_help (gpointer p, guint uguide, GtkWidget *w)
 #elif defined(OSX_BUILD)
     osx_open_file(fname);
 #else
-    gretl_fork(viewpdf, fname);
+    gretl_fork("viewpdf", fname);
 #endif
 }
