@@ -26,8 +26,10 @@ void context_help (GtkWidget *widget, gpointer data);
 
 void plain_text_cmdref (gpointer p, guint cmdnum, GtkWidget *w);
 
-gint edit_script_help (GtkWidget *widget, GdkEventButton *b,
-		       windata_t *vwin);
+void genr_funcs_ref (gpointer p, guint fnum, GtkWidget *w);
+
+gint interactive_script_help (GtkWidget *widget, GdkEventButton *b,
+			      windata_t *vwin);
 
 void display_pdf_help (gpointer p, guint uguide, GtkWidget *w);
 
@@ -35,7 +37,7 @@ void datafile_find (GtkWidget *widget, gpointer data);
 
 void menu_find (gpointer data, guint dbfind, GtkWidget *widget);
 
-void text_find_callback (GtkWidget *w, gpointer data);
+void text_find_callback (GtkWidget *w, windata_t *vwin);
 
 char *quoted_help_string (const char *s);
 
@@ -43,4 +45,6 @@ void gretl_tooltips_init (void);
 
 void gretl_tooltips_add (GtkWidget *w, const gchar *str);
 
+int function_help_index_from_word (const char *s);
+
 #endif /* HELPFILES_H */
