@@ -55,7 +55,7 @@ int gretl_command_sprintf (const char *template, ...);
 
 int gretl_command_strcpy (const char *s);
 
-int *command_list_from_string (char *s);
+int gretl_command_strcat (const char *s);
 
 int user_fopen (const char *fname, char *fullname, PRN **pprn);
 
@@ -63,7 +63,9 @@ gint bufopen (PRN **pprn);
 
 void do_menu_op (gpointer data, guint action, GtkWidget *w);
 
-void do_run_script (GtkWidget *w, gpointer data);
+void set_currdir_from_filename (const char *fname);
+
+void do_run_script (GtkWidget *w, windata_t *vwin);
 
 void run_script_fragment (windata_t *vwin, gchar *buf);
 
@@ -95,6 +97,8 @@ void gui_transpose_data (gpointer p, guint u, GtkWidget *w);
 
 void gui_sort_data (gpointer p, guint u, GtkWidget *w);
 
+void gui_resample_data (gpointer p, guint u, GtkWidget *w);
+
 int dataset_is_subsampled (void);
 
 void set_original_n (int n);
@@ -105,7 +109,7 @@ int get_original_n (void);
 
 int do_coint (selector *sr);
 
-void do_forecast (gpointer p, guint u, GtkWidget *w);
+void gui_do_forecast (gpointer p, guint u, GtkWidget *w);
 
 void do_bootstrap (gpointer p, guint u, GtkWidget *w) ;
 
@@ -185,7 +189,7 @@ int record_varlabel_change (int v);
 
 void do_resid_freq (gpointer p, guint action, GtkWidget *w);
 
-void do_freqplot (gpointer p, guint gamma, GtkWidget *w);
+void do_freq_dist (gpointer p, guint plot, GtkWidget *w);
 
 void do_corrgm (gpointer p, guint u, GtkWidget *w);
 
@@ -217,12 +221,10 @@ void do_remove_obs (gpointer p, guint u, GtkWidget *w);
 
 void add_logs_etc (gpointer p, guint action, GtkWidget *w);
 
-int add_system_resid (gpointer p, int eqnum, int ci);
+void add_system_resid (gpointer p, int eqnum, GtkWidget *w);
 
 int add_fit_resid (MODEL *pmod, int code, int undo);
 
-int add_system_resid (gpointer p, int eqnum, int ci);
-
 void add_model_stat (MODEL *pmod, int which);
 
 void resid_plot (gpointer p, guint xvar, GtkWidget *w);
@@ -251,6 +253,8 @@ void do_box_graph (GtkWidget *w, dialog_t *dlg);
 
 int do_dummy_graph (selector *sr);
 
+int do_xyz_graph (selector *sr);
+
 int do_xcorrgm (selector *sr);
 
 void delete_selected_vars (void);
@@ -263,13 +267,13 @@ void display_var (void);
 
 /* script- and file-related functions */
 
-void do_open_script (void);
+void do_open_script (int action);
 
 void open_info (gpointer p, guint edit, GtkWidget *w);
 
 void do_new_script (gpointer p, guint action, GtkWidget *w);
 
-void do_open_csv_octave (char *fname, int code, int append);
+void do_open_csv_octave (char *fname, int ftype, int append);
 
 int do_store (char *mydatfile, gretlopt oflag);
 
@@ -279,7 +283,7 @@ void save_latex (PRN *prn, const char *fname);
 
 void do_save_text (char *fname, MODEL *pmod);
 
-int gui_exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo);
+int gui_exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo);
 
 int check_and_record_command (void);
 
