@@ -23,11 +23,13 @@
 #include "console.h"
 #include "guiprint.h"
 #include "ssheet.h"
+#include "menustate.h"
 
 void refresh_data (void)
 {
     if (data_status) {
 	populate_varlist();
+	set_sample_label(datainfo);
     }
 }
 
@@ -188,6 +190,7 @@ void time_series_menu_state (gboolean s)
     flip(mdata->ifac, "/Variable/Correlogram", s);
     flip(mdata->ifac, "/Variable/Spectrum", s);
     flip(mdata->ifac, "/Variable/Augmented Dickey-Fuller test", s);
+    flip(mdata->ifac, "/Variable/ADF-GLS test", s);
     flip(mdata->ifac, "/Variable/KPSS test", s);
     flip(mdata->ifac, "/Variable/Filter", s);
 #ifdef HAVE_X12A
@@ -320,9 +323,9 @@ static gint var_popup_click (GtkWidget *w, gpointer p)
     else if (!strcmp(item, _("Time series plot"))) 
 	do_graph_var(v);
     else if (!strcmp(item, _("Frequency distribution"))) 
-	do_menu_op(NULL, FREQ, NULL);
+	do_freq_dist(NULL, 0, NULL);
     else if (!strcmp(item, _("Frequency plot"))) 
-	do_freqplot(NULL, 0, NULL);
+	do_freq_dist(NULL, 1, NULL);
     else if (!strcmp(item, _("Boxplot")))
 	do_boxplot_var(v);
     else if (!strcmp(item, _("Gini coefficient")))
