@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* objectsave.c for gretl: save models estimated via commands */
@@ -136,7 +136,7 @@ int maybe_save_var (const CMD *cmd, GRETL_VAR **pvar, PRN *prn)
     return err;
 }
 
-int maybe_save_system (const CMD *cmd, gretl_equation_system *sys, PRN *prn)
+int maybe_save_system (const CMD *cmd, equation_system *sys, PRN *prn)
 {
     char name[MAXSAVENAME];
     int err = 0;
@@ -275,7 +275,8 @@ static int session_VAR_omit (GRETL_VAR *orig, char *cmdstr, PRN *prn)
 	return err;
     }
 
-    var = gretl_VAR_omit_test(mycmd.list, orig, &Z, datainfo, prn, &err);
+    var = gretl_VAR_omit_test(mycmd.list, orig, (const double **) Z, 
+			      datainfo, prn, &err);
     gretl_VAR_free(var);
 
     if (err) {
