@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* series_view.c for gretl */
@@ -89,11 +89,12 @@ void free_multi_series_view (gpointer p)
 
 static int series_view_allocate (series_view *sview)
 {
+    int err = 0;
+
     if (sview->npoints != 0) {
 	/* already allocated */
 	return 0;
     } else if (var_is_scalar(datainfo, sview->varnum)) {
-	sview->npoints = 1;
 	return 0;
     } else {
 	int t, tp, T = datainfo->t2 - datainfo->t1 + 1;
@@ -101,24 +102,23 @@ static int series_view_allocate (series_view *sview)
 
 	sview->points = malloc(T * sizeof *sview->points);
 	if (sview->points == NULL) {
-	    return 1;
-	} 
-
-	sview->npoints = T;
-
-	/* populate from data set */
-	for (t=datainfo->t1; t<=datainfo->t2; t++) {
-	    tp = t - datainfo->t1; 
-	    sview->points[tp].val = Z[v][t];
-	    if (datainfo->markers) {
-		strcpy(sview->points[tp].label, datainfo->S[t]);
-	    } else {
-		ntodate(sview->points[tp].label, t, datainfo);
+	    err = E_ALLOC;
+	} else {
+	    sview->npoints = T;
+	    /* populate from data set */
+	    for (t=datainfo->t1; t<=datainfo->t2; t++) {
+		tp = t - datainfo->t1; 
+		sview->points[tp].val = Z[v][t];
+		if (datainfo->markers) {
+		    strcpy(sview->points[tp].label, datainfo->S[t]);
+		} else {
+		    ntodate(sview->points[tp].label, t, datainfo);
+		}
 	    }
 	}
     }
 
-    return 0;
+    return err;
 }
 
 static int multi_series_view_allocate (multi_series_view *mview)
@@ -195,6 +195,7 @@ static void series_view_print (windata_t *vwin)
     }
 	
     /* print formatted data to buffer */
+
     if (var_is_series(datainfo, sview->varnum)) {
 	pprintf(prn, "\n%*s ", obslen, _("Obs"));
 	pprintf(prn, "%13s\n\n", datainfo->varname[sview->varnum]);
