@@ -44,6 +44,7 @@
 #include "gretl_xml.h"
 #include "gretl_func.h"
 #include "modelspec.h"
+#include "matrix_extra.h"
 #include "cmd_private.h"
 
 #include <sys/stat.h>
@@ -129,7 +130,8 @@ struct sample_info {
     int t1;
     int t2;
     char *mask;
-    int mode;
+    unsigned int seed;
+    int resample_n;
 };
 
 enum {
@@ -143,11 +145,6 @@ enum {
     SAVEFILE_ERROR
 } savefile_returns;
 
-static GtkTargetEntry session_drag_targets[] = {
-    { "model_pointer", GTK_TARGET_SAME_APP, GRETL_MODEL_POINTER },
-    { "text/uri-list", 0, GRETL_FILENAME }
-};
-
 static char *global_items[] = {
     N_("Save session"),
     N_("Arrange icons"),
@@ -711,7 +708,7 @@ static int session_dir_ok (void)
 	errno = 0;
 
 	sprintf(session.dirname, ".gretl-%d", (int) p); /* added dot! */
-	if (chdir(paths.userdir)) {
+	if (chdir(paths.dotdir)) {
 	    perror("moving to user directory");
 	    ret = 0;
 	} else if (gretl_mkdir(session.dirname)) {
@@ -736,7 +733,7 @@ int add_graph_to_session (char *fname, char *fullname)
 	    gui_errmsg(err);
 	    return 1;
 	}
-	register_data(NULL, NULL, 0);
+	register_data(NULLDATA_STARTED);
     }
     
     errno = 0;
@@ -746,7 +743,7 @@ int add_graph_to_session (char *fname, char *fullname)
 	return 1;
     }
 
-    chdir(paths.userdir);
+    chdir(paths.dotdir);
 
     sprintf(shortname, "graph.%d", session_graph_count + 1);
     session_file_make_path(fullname, shortname);
@@ -781,7 +778,7 @@ void add_boxplot_to_session (const char *boxtmp)
 	return;
     }
 
-    chdir(paths.userdir);
+    chdir(paths.dotdir);
 
     sprintf(fname, "plot.%d", session_bplot_count + 1);
     session_file_make_path(grpath, fname);
@@ -807,7 +804,7 @@ int cli_add_graph_to_session (const char *fname, const char *gname,
 	return ADD_OBJECT_FAIL;
     }
 
-    chdir(paths.userdir);
+    chdir(paths.dotdir);
 
     strcpy(name, gname);
     space_to_score(name);
@@ -885,11 +882,23 @@ int maybe_add_model_to_session (void *ptr, GretlObjType type)
     return real_add_model_to_session(ptr, name, type);
 }
 
-void model_add_as_icon (gpointer p, guint type, GtkWidget *w)
+static int model_type_from_vwin (windata_t *vwin)
+{
+    if (vwin->role == SYSTEM) {
+	return GRETL_OBJ_SYS;
+    } else if (vwin->role == VAR || vwin->role == VECM) {
+	return GRETL_OBJ_VAR;
+    } else {
+	return GRETL_OBJ_EQN;
+    }
+}
+
+void model_add_as_icon (gpointer p, guint u, GtkWidget *w)
 {
     windata_t *vwin = (windata_t *) p;
     void *ptr = vwin->data;
     const char *name;
+    int type;
 
     if (ptr == NULL) {
 	return;
@@ -903,7 +912,8 @@ void model_add_as_icon (gpointer p, guint type, GtkWidget *w)
 	infobox(_("Model is already saved"));
 	return;
     }
-
+    
+    type = model_type_from_vwin(vwin);
     name = gretl_object_get_name(ptr, type);
 
     if (real_add_model_to_session(ptr, name, type)) {
@@ -913,11 +923,11 @@ void model_add_as_icon (gpointer p, guint type, GtkWidget *w)
     mark_session_changed();
 }
 
-void model_add_as_icon_and_close (gpointer p, guint type, GtkWidget *w)
+void model_add_as_icon_and_close (gpointer p, guint u, GtkWidget *w)
 {
     windata_t *vwin = (windata_t *) p;
 
-    model_add_as_icon(p, type, w);
+    model_add_as_icon(p, u, w);
 
     if (!window_is_busy(vwin)) {
 	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(vwin->w)));
@@ -959,7 +969,7 @@ static int unzip_session_file (const char *fname)
     }
     fclose(fp);
 
-    chdir(paths.userdir);
+    chdir(paths.dotdir);
 
     gretl_unzip_file = gui_get_plugin_function("gretl_unzip_file", 
 					       &handle);
@@ -990,7 +1000,8 @@ static void sinfo_init (struct sample_info *sinfo)
     sinfo->t1 = 0;
     sinfo->t2 = 0;
     sinfo->mask = NULL;
-    sinfo->mode = SUBSAMPLE_NONE;
+    sinfo->seed = 0;
+    sinfo->resample_n = 0;
 }
 
 static int set_session_dirname (const char *sname)
@@ -1041,12 +1052,12 @@ void do_open_session (void)
     }
 
     /* close existing session, if any, and initialize */
-    close_session(NULL, &Z, &datainfo);
+    close_session(NULL, &Z, datainfo, OPT_NONE);
 
     strcpy(sessionfile, tryfile);
     fprintf(stderr, I_("\nReading session file %s\n"), sessionfile);
 
-    chdir(paths.userdir);
+    chdir(paths.dotdir);
     status = unzip_session_file(sessionfile);
 
     if (status == SAVEFILE_ERROR) {
@@ -1056,7 +1067,7 @@ void do_open_session (void)
     } else if (status == SAVEFILE_SCRIPT) {
 	strcpy(scriptfile, sessionfile);
 	*sessionfile = '\0';
-	do_open_script();
+	do_open_script(EDIT_SCRIPT);
 	return;
     }
 
@@ -1081,8 +1092,8 @@ void do_open_session (void)
     }
 
     session_file_make_path(paths.datfile, sinfo.datafile);
-    err = gretl_read_gdt(&Z, &datainfo, paths.datfile, &paths, OPT_P, 
-			 NULL);
+    err = gretl_read_gdt(paths.datfile, &paths, &Z, datainfo, 
+			 OPT_P, NULL);
     if (err) {
 	/* FIXME more explicit error message */
 	file_read_errbox(sinfo.datafile);
@@ -1100,9 +1111,11 @@ void do_open_session (void)
     session_file_make_path(fname, "lists.xml");
     err = maybe_read_lists_file(fname);
 
-    if (sinfo.mask != NULL) {
-	err = restrict_sample_from_mask(sinfo.mask, sinfo.mode, &Z, &datainfo,
-					NULL);
+    if (sinfo.resample_n > 0) {
+	err = dataset_resample(sinfo.resample_n, sinfo.seed, &Z, 
+			       datainfo);
+    } else if (sinfo.mask != NULL) {
+	err = restrict_sample_from_mask(sinfo.mask, &Z, datainfo);
     }
 
     if (err) {
@@ -1113,7 +1126,7 @@ void do_open_session (void)
     datainfo->t1 = sinfo.t1;
     datainfo->t2 = sinfo.t2;
 
-    register_data(paths.datfile, NULL, 0);
+    register_data(OPENED_VIA_SESSION);
 
     if (sinfo.mask != NULL) {
 	set_sample_label(datainfo);
@@ -1154,29 +1167,27 @@ void verify_clear_data (void)
 	return;
     }
 
-    if (!expert) {
-        if (yes_no_dialog ("gretl",                      
-			   _("Clearing the data set will end\n"
-			     "your current session.  Continue?"), 
-			   0) != GRETL_YES) {
-            return;
-	}
+    if (yes_no_dialog ("gretl",                      
+		       _("Clearing the data set will end\n"
+			 "your current session.  Continue?"), 
+		       0) != GRETL_YES) {
+	return;
     }
 
-    close_session(NULL, &Z, &datainfo);
+    close_session(NULL, &Z, datainfo, OPT_NONE); /* FIXME opt */
 }
 
 static void remove_session_dir (void)
 {
 #ifdef G_OS_WIN32
-    char *fullpath = g_strdup_printf("%s%s", paths.userdir,
+    char *fullpath = g_strdup_printf("%s%s", paths.dotdir,
 				     session.dirname);
 
-    chdir(paths.userdir);
+    chdir(paths.dotdir);
     win32_delete_dir(fullpath);
     g_free(fullpath);
 #else
-    chdir(paths.userdir);
+    chdir(paths.dotdir);
     gretl_deltree(session.dirname);
 #endif
 }
@@ -1249,37 +1260,35 @@ void free_session (void)
 
 int highest_numbered_variable_in_session (void)
 {
-    MODEL *pmod;
-    GRETL_VAR *var;
-    equation_system *sys;
+    GretlObjType type;
+    void *ptr;
     int i, mvm, vmax = 0;
 
-    if (session.models) {
-	for (i=0; i<session.nmodels; i++) {
-	    if (session.models[i]->type == GRETL_OBJ_EQN) {
-		pmod = session.models[i]->ptr;
-		if (pmod != NULL) {
-		    mvm = highest_numbered_var_in_model(pmod, datainfo);
-		    if (mvm > vmax) {
-			vmax = mvm;
-		    }
-		}
-	    } else if (session.models[i]->type == GRETL_OBJ_VAR) {
-		var = session.models[i]->ptr;
-		if (var != NULL) {
-		    mvm = gretl_VAR_get_highest_variable(var);
-		    if (mvm > vmax) {
-			vmax = mvm;
-		    }
-		}		
-	    } else if (session.models[i]->type == GRETL_OBJ_SYS) {
-		sys = session.models[i]->ptr;
-		if (sys != NULL) {
-		    mvm = highest_numbered_var_in_system(sys, datainfo);
-		    if (mvm > vmax) {
-			vmax = mvm;
-		    }
-		}
+    if (session.models == NULL) {
+	return 0;
+    }
+
+    for (i=0; i<session.nmodels; i++) {
+	ptr = session.models[i]->ptr;
+	if (ptr == NULL) {
+	    continue;
+	}
+	type = session.models[i]->type;
+	if (type == GRETL_OBJ_EQN) {
+	    mvm = highest_numbered_var_in_model((MODEL *) ptr, datainfo);
+	    if (mvm > vmax) {
+		vmax = mvm;
+	    }
+	} else if (type == GRETL_OBJ_VAR) {
+	    mvm = gretl_VAR_get_highest_variable((GRETL_VAR *) ptr);
+	    if (mvm > vmax) {
+		vmax = mvm;
+	    }
+	} else if (type == GRETL_OBJ_SYS) {
+	    mvm = highest_numbered_var_in_system((equation_system *) ptr, 
+						 datainfo);
+	    if (mvm > vmax) {
+		vmax = mvm;
 	    }
 	}
     }
@@ -1312,9 +1321,9 @@ void gui_clear_dataset (void)
 }
 
 static void 
-session_clear_data (double ***pZ, DATAINFO **ppdinfo, int realclean)
+session_clear_data (double ***pZ, DATAINFO *pdinfo, int realclean)
 {
-    gui_restore_sample(pZ, ppdinfo);
+    gui_restore_sample(pZ, pdinfo);
     gui_clear_dataset();
 
     /* clear protected models */
@@ -1332,7 +1341,8 @@ session_clear_data (double ***pZ, DATAINFO **ppdinfo, int realclean)
     lib_cmd_destroy_context();
 }
 
-void close_session (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
+void close_session (ExecState *s, double ***pZ, DATAINFO *pdinfo,
+		    gretlopt opt)
 {
     int realclean = 1;
 
@@ -1345,7 +1355,7 @@ void close_session (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
 #if SESSION_DEBUG
     fprintf(stderr, "close_session: starting cleanup\n");
 #endif
-    session_clear_data(pZ, ppdinfo, realclean); 
+    session_clear_data(pZ, pdinfo, realclean); 
 
     if (realclean) {
 	free_session();
@@ -1370,7 +1380,11 @@ void close_session (ExecState *s, double ***pZ, DATAINFO **ppdinfo)
     edit_dialog_special_get_text(NULL);
 
     if (realclean) {
-	libgretl_session_cleanup();
+	if (opt & OPT_P) {
+	    libgretl_session_cleanup(SESSION_PRESERVE_MATRICES);
+	} else {
+	    libgretl_session_cleanup(SESSION_CLEAR_FULL);
+	}
     }
 
     session_graph_count = 0;
@@ -1510,7 +1524,7 @@ int save_session (char *fname)
     }
 
     /* paths below are relative to this */
-    chdir(paths.userdir);
+    chdir(paths.dotdir);
 
     if (strcmp(dirname, session.dirname)) {
 	/* rename session directory */
@@ -1676,7 +1690,6 @@ static int maybe_raise_object_window (gpointer p)
 static int display_session_model (SESSION_MODEL *sm)
 { 
     PRN *prn;
-    int err = 0;
 
     if (maybe_raise_object_window(sm->ptr)) {
 	return 0;
@@ -1698,12 +1711,13 @@ static int display_session_model (SESSION_MODEL *sm)
 	view_buffer(prn, 78, 450, sm->name, var->ci, var);
     } else if (sm->type == GRETL_OBJ_SYS) {
 	equation_system *sys = (equation_system *) sm->ptr;
+	int cancel = 0;
 
 	edit_dialog(sm->name, NULL, NULL, do_saved_eqn_system, sys, 
-		    SYSTEM, VARCLICK_NONE, NULL); 
+		    SYSTEM, VARCLICK_NONE, &cancel); 
     }
 
-    return err;
+    return 0;
 }
 
 /* callback used in objectsave.c */
@@ -1826,7 +1840,7 @@ static void remove_session_graph_file (const char *gfname)
 {
     char fname[MAXLEN];
 
-    chdir(paths.userdir);
+    chdir(paths.dotdir);
     session_file_make_path(fname, gfname);
     remove(fname);
 }
@@ -2554,11 +2568,11 @@ static void object_popup_activated (GtkWidget *widget, gpointer data)
 	    SESSION_GRAPH *graph = (SESSION_GRAPH *) obj->data;
 	    char fullname[MAXLEN];
 
-	    chdir(paths.userdir);
+	    chdir(paths.dotdir);
 	    session_file_make_path(fullname, graph->fname);
 	    remove_png_term_from_plotfile_by_name(fullname);
 	    view_file(fullname, 1, 0, 78, 400, 
-		      (obj->sort == GRETL_OBJ_GRAPH)? GR_PLOT : GR_BOX);
+		      (obj->sort == GRETL_OBJ_GRAPH)? EDIT_GP : EDIT_BOX);
 	}
     } else if (!strcmp(item, _("Rename"))) {
 	rename_object_dialog(obj);
@@ -2623,13 +2637,13 @@ session_data_received (GtkWidget *widget,
 		       guint time,
 		       gpointer p)
 {
-    if (info == GRETL_MODEL_POINTER && data != NULL) {
+    if (info == GRETL_MODEL_PTR && data != NULL) {
 	MODEL **ppmod = (MODEL **) data->data;
 
 	if (ppmod != NULL) {
 	    add_to_model_table(*ppmod, MODEL_ADD_BY_DRAG, NULL);
 	}
-    } else if (info == GRETL_FILENAME && data != NULL) {
+    } else if (info == GRETL_GRAPH_FILE && data != NULL) {
 	gchar *fname = (gchar *) data->data;
 
 	if (fname != NULL) {
@@ -2644,9 +2658,9 @@ static void session_drag_setup (gui_obj *obj)
     GtkTargetEntry *targ;
     
     if (obj->sort == GRETL_OBJ_MODTAB) {
-	targ = &session_drag_targets[0];
+	targ = &gretl_drag_targets[GRETL_MODEL_PTR];
     } else {
-	targ = &session_drag_targets[1];
+	targ = &gretl_drag_targets[GRETL_GRAPH_FILE];
     }
 
     gtk_drag_dest_set (w,
@@ -2671,7 +2685,7 @@ static void drag_graph (GtkWidget *w, GdkDragContext *context,
 static void graph_drag_connect (GtkWidget *w, SESSION_GRAPH *graph)
 {
     gtk_drag_source_set(w, GDK_BUTTON1_MASK,
-                        &session_drag_targets[1],
+                        &gretl_drag_targets[GRETL_GRAPH_FILE],
                         1, GDK_ACTION_COPY);
     g_signal_connect(G_OBJECT(w), "drag_data_get",
                      G_CALLBACK(drag_graph), graph);
@@ -2689,7 +2703,7 @@ static void drag_model (GtkWidget *w, GdkDragContext *context,
 static void model_drag_connect (GtkWidget *w, SESSION_MODEL *mod)
 {
     gtk_drag_source_set(w, GDK_BUTTON1_MASK,
-                        &session_drag_targets[0],
+                        &gretl_drag_targets[GRETL_MODEL_PTR],
                         1, GDK_ACTION_COPY);
     g_signal_connect(G_OBJECT(w), "drag_data_get",
                      G_CALLBACK(drag_model), mod);
@@ -3092,164 +3106,6 @@ static gui_obj *gui_object_new (gchar *name, int sort, gpointer data)
     return obj;
 } 
 
-static char *dump_current_plot (windata_t *vwin)
-{
-    FILE *fp;
-    char *tmpfile = NULL;
-    int gotpause = 0;
-    char bufline[1024];
-    char *buf;
-# ifdef ENABLE_NLS
-    gchar *trbuf;
-# endif
-
-    buf = textview_get_text(vwin->w);
-    if (buf == NULL) return NULL;
-
-    tmpfile = g_strdup_printf("%showtmp.gp", paths.userdir);
-
-    fp = gretl_fopen(tmpfile, "w");
-    if (fp == NULL) {
-	g_free(tmpfile);
-	g_free(buf);
-	file_write_errbox(tmpfile);
-	return NULL;
-    }
-
-    bufgets_init(buf);
-
-    while (bufgets(bufline, sizeof bufline, buf)) {
-# ifdef ENABLE_NLS
-	trbuf = gp_locale_from_utf8(bufline);
-	if (trbuf != NULL) {
-	    fputs(trbuf, fp);
-	    g_free(trbuf);
-	} else {
-	    fputs(bufline, fp);
-	}
-# else
-	fputs(bufline, fp);
-# endif
-	fputc('\n', fp);
-	if (strstr(bufline, "pause -1")) {
-	    gotpause = 1;
-	}
-    }
-
-    bufgets_finalize(buf);
-    g_free(buf);
-
-#ifdef G_OS_WIN32
-    if (!gotpause) {
-	fputs("pause -1\n", fp);
-    }
-#endif
-
-    fclose(fp);
-
-    return tmpfile;
-}
-
-#ifdef G_OS_WIN32
-
-void gp_to_gnuplot (gpointer data, guint i, GtkWidget *w)
-{
-    windata_t *vwin = (windata_t *) data;
-    gchar *tmpfile;
-    int err = 0;
-
-    tmpfile = dump_current_plot(vwin);
-
-    if (tmpfile != NULL) {
-	gchar *buf = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot, tmpfile);
-
-	err = (WinExec(buf, SW_SHOWNORMAL) < 32);
-	remove(tmpfile); /* is this OK? */
-	g_free(tmpfile);
-	g_free(buf);
-    } else {
-	err = 1;
-    }
-
-    if (err) {
-	gui_errmsg(err);
-    }
-}
-
-#else /* !G_OS_WIN32 */
-
-#include <sys/types.h>
-#include <sys/wait.h>
-
-void gp_to_gnuplot (gpointer data, guint i, GtkWidget *w)
-{
-    windata_t *vwin = (windata_t *) data;
-    gchar *tmpfile;
-    GError *error = NULL;
-    gchar *argv[4];
-    GPid pid = 0;
-    gint fd = -1;
-    gboolean run;
-
-    tmpfile = dump_current_plot(vwin);
-    if (tmpfile == NULL) {
-	return;
-    }
-
-    argv[0] = g_strdup(paths.gnuplot);
-    argv[1] = g_strdup("-persist");
-    argv[2] = g_strdup(tmpfile);
-    argv[3] = NULL;
-
-    run = g_spawn_async_with_pipes(NULL, argv, NULL, 
-				   G_SPAWN_SEARCH_PATH | 
-				   G_SPAWN_DO_NOT_REAP_CHILD,
-				   NULL, NULL, &pid, NULL, NULL,
-				   &fd, &error);
-
-    if (error != NULL) {
-	errbox(error->message);
-	g_error_free(error);
-    } else if (!run) {
-	errbox(_("gnuplot command failed"));
-    } else if (pid > 0) {
-	int status = 0, err = 0;
-
-	/* bodge below: try to give gnuplot time to bomb
-	   out, if it's going to -- but we don't want to
-	   hold things up if we're doing OK */
-	
-	sleep(1);
-	waitpid(pid, &status, WNOHANG);
-	if (WIFEXITED(status)) {
-	    err = WEXITSTATUS(status);
-	} 
-
-	if (err && fd > 0) {
-	    char buf[128] = {0};
-
-	    if (read(fd, buf, 127) > 0) {
-		errbox(buf);
-	    }
-	}
-    }
-
-    if (fd > 0) {
-	close(fd);
-    }
-
-    g_spawn_close_pid(pid);
-
-    g_free(argv[0]);
-    g_free(argv[1]);
-    g_free(argv[2]);
-
-    remove(tmpfile);
-    g_free(tmpfile);
-}
-
-#endif /* ? G_OS_WIN32 */
-
 static void open_gui_graph (gui_obj *obj)
 {
     SESSION_GRAPH *graph = (SESSION_GRAPH *) obj->data;
