@@ -60,9 +60,9 @@ void add_boxplot_to_session (const char *boxtmp);
 int cli_add_graph_to_session (const char *fname, const char *gname,
 			      GretlObjType type);
 
-void model_add_as_icon (gpointer p, guint type, GtkWidget *w);
+void model_add_as_icon (gpointer p, guint u, GtkWidget *w);
 
-void model_add_as_icon_and_close (gpointer p, guint type, GtkWidget *w);
+void model_add_as_icon_and_close (gpointer p, guint u, GtkWidget *w);
 
 int maybe_add_model_to_session (void *ptr, GretlObjType type);
 
@@ -86,7 +86,8 @@ void gui_clear_dataset (void);
 
 void verify_clear_data (void);
 
-void close_session (ExecState *s, double ***pZ, DATAINFO **ppdinfo);
+void close_session (ExecState *s, double ***pZ, DATAINFO *pdinfo,
+		    gretlopt opt);
 
 void free_session (void);
 
@@ -104,8 +105,6 @@ int session_file_is_open (void);
 
 int clear_or_save_model (MODEL **ppmod, DATAINFO *pdinfo, int rebuild);
 
-void gp_to_gnuplot (gpointer data, guint i, GtkWidget *w);
-
 void disable_graph_page (void);
 
 void display_session_graph_by_data (void *p);
