@@ -27,6 +27,7 @@
 #include "fileselect.h"
 #include "menustate.h"
 #include "session.h"
+#include "textbuf.h"
 
 #include "libset.h"
 #include "version.h"
@@ -36,11 +37,6 @@
 #include <sys/types.h>
 #include <dirent.h>
 
-#ifdef USE_GNOME
-# define USE_GCONF
-# include <gconf/gconf-client.h>
-#endif
-
 #ifdef G_OS_WIN32
 # include <windows.h>
 # include "gretlwin32.h"
@@ -51,12 +47,11 @@
 # include "gtkfontselhack.h"
 #endif
 
-#if !defined(G_OS_WIN32) && !defined(USE_GNOME)
+#if !defined(G_OS_WIN32)
 char rcfile[MAXLEN];
 #endif
 
 extern int want_toolbar;
-extern char Rcommand[MAXSTR];
 
 char dbproxy[21];
 int use_proxy;
@@ -113,22 +108,12 @@ extern int use_wimp;
 #endif
 
 #if defined(HAVE_AUDIO) && !defined(G_OS_WIN32)
-char midiplayer[MAXSTR];
+char midiplayer[MAXSTR] = "timidity -ig";
 #endif
 
-enum {
-    TAB_NONE = 0,
-    TAB_MAIN,
-    TAB_DBS,
-    TAB_PROGS,
-    TAB_SAVE,
-    TAB_VCV,
-    TAB_MAN
-};
-
 typedef enum {
     ROOTSET  = 1 << 0,
-    USERSET  = 1 << 1,
+    USERSET  = 1 << 1, 
     BOOLSET  = 1 << 2,
     INTSET   = 1 << 3,
     LISTSET  = 1 << 4,
@@ -151,7 +136,7 @@ typedef struct {
                           INTSET integer (user)
 			  LISTSET user string, from fixed menu
                           RADIOSET user int, from fixed menu
-			  INVISET "invisible" (user) string 
+			  INVISET not visible in Prefs dialog
 		       */
     int len;           /* storage size for string variable (also see Note) */
     short tab;         /* which tab (if any) does the item fall under? */
@@ -168,10 +153,8 @@ typedef struct {
 RCVAR rc_vars[] = {
     { "gretldir", N_("Main gretl directory"), NULL, paths.gretldir, 
       MACHSET | BROWSER, MAXLEN, TAB_MAIN, NULL },
-    { "userdir", N_("User's gretl directory"), NULL, paths.userdir, 
-      USERSET | BROWSER, MAXLEN, TAB_MAIN, NULL },
-    { "expert", N_("Expert mode (no warnings)"), NULL, &expert, 
-      BOOLSET, 0, TAB_MAIN, NULL },
+    { "userdir", N_("User's gretl directory"), NULL, paths.workdir, 
+      INVISET, MAXLEN, TAB_MAIN, NULL },
     { "updater", N_("Tell me about gretl updates"), NULL, &updater, 
       BOOLSET, 0, TAB_MAIN, NULL },
     { "toolbar", N_("Show gretl toolbar"), NULL, &want_toolbar, 
@@ -194,6 +177,8 @@ RCVAR rc_vars[] = {
 #endif
     { "shellok", N_("Allow shell commands"), NULL, &shellok, 
       BOOLSET, 0, TAB_MAIN, NULL },
+    { "usecwd", N_("Set working directory from shell"), NULL, &usecwd, 
+      INVISET | BOOLSET, 0, TAB_NONE, NULL },
     { "gnuplot", N_("Command to launch gnuplot"), NULL, paths.gnuplot, 
       MACHSET | BROWSER, MAXLEN, TAB_PROGS, NULL },
     { "Rcommand", N_("Command to launch GNU R"), NULL, Rcommand, 
@@ -232,9 +217,6 @@ RCVAR rc_vars[] = {
       USERSET, 21, TAB_DBS, NULL },
     { "useproxy", N_("Use HTTP proxy"), NULL, &use_proxy, 
       BOOLSET, 1, TAB_DBS, NULL },
-    { "usecwd", N_("Use current working directory as default"), 
-      N_("Use gretl user directory as default"), &usecwd, 
-      BOOLSET, 0, TAB_SAVE, NULL },
     { "useqr", N_("Use QR decomposition"), N_("Use Cholesky decomposition"), &useqr, 
       BOOLSET, 0, TAB_MAIN, NULL },
     { "Fixed_font", N_("Fixed font"), NULL, fixedfontname, 
@@ -251,6 +233,10 @@ RCVAR rc_vars[] = {
       INVISET, 32, TAB_NONE, NULL },
     { "Gp_colors", N_("Gnuplot colors"), NULL, gpcolors, 
       INVISET, sizeof gpcolors, TAB_NONE, NULL },
+    { "tabwidth", "spaces per tab", NULL, &tabwidth, 
+      INVISET | INTSET, 0, TAB_NONE, NULL },
+    { "smarttab", "\"Smart\" Tab and Enter", NULL, &smarttab, 
+      INVISET | BOOLSET, 0, TAB_NONE, NULL },
     { "main_width", "main window width", NULL, &mainwin_width, 
       INVISET | INTSET, 0, TAB_NONE, NULL },
     { "main_height", "main window height", NULL, &mainwin_height, 
@@ -361,7 +347,7 @@ static void record_shell_opt (void)
 {
     char shellstamp[FILENAME_MAX];
 
-    sprintf(shellstamp, "%s.gretl_shell_stamp", paths.userdir);
+    sprintf(shellstamp, "%s.gretl_shell_stamp", paths.dotdir);
 
     if (shellok) {
 	FILE *fp = fopen(shellstamp, "w");
@@ -449,7 +435,7 @@ static void get_functions_dir (char *dirname)
     
     if (ok) return;
 
-    sprintf(dirname, "%sfunctions", paths.userdir);
+    sprintf(dirname, "%sfunctions", paths.dotdir); /* ?? */
     err = gretl_mkdir(dirname);
     if (!err) {
 	target = g_strdup_printf("%s%c%s", dirname, SLASH, "wtest");
@@ -471,18 +457,38 @@ static void get_functions_dir (char *dirname)
 
 static char startdir[MAXLEN];
 
-void set_program_startdir (const char *callname)
+void set_gretl_startdir (void)
 {
-    char *test = getcwd(startdir, MAXLEN);
+    char *test = getenv("GRETL_STARTDIR");
 
-    if (test == NULL) {
+    if (test != NULL) {
 	*startdir = '\0';
+	strncat(startdir, test, MAXLEN - 1);
+    } else {
+	test = getcwd(startdir, MAXLEN);
+	if (test == NULL) {
+	    *startdir = '\0';
+	}
     }
 
 #if 0
-    fprintf(stderr, "Starting in '%s'\n", startdir);
-    fprintf(stderr, "Called as '%s'\n", callname);
+    fprintf(stderr, "startdir = '%s'\n", startdir);
 #endif
+
+    if (usecwd && *startdir != '\0') {
+	int err = set_gretl_work_dir(startdir, &paths);
+
+	if (err) {
+	    fprintf(stderr, "%s\n", gretl_errmsg_get());
+	} else {
+	    fprintf(stderr, "working dir = '%s'\n", paths.workdir);
+	}
+    }
+}
+
+const char *get_gretl_startdir (void)
+{
+    return startdir;
 }
 
 void get_default_dir (char *s, int action)
@@ -491,28 +497,10 @@ void get_default_dir (char *s, int action)
 
     if (action == SAVE_FUNCTIONS) {
 	get_functions_dir(s);
-	if (*s != '\0') {
-	    slash_terminate(s);
-	    return;
-	}
     } else if (action == OPEN_RATS_DB) {
 	strcpy(s, paths.ratsbase);
-	return;
-    }
-
-    if (usecwd && action != SAVE_DBDATA) {
-	if (*startdir != '\0') {
-	    strcpy(s, startdir);
-	} else {
-	    const char *sdir = get_session_dirname();
-	    char *test = getcwd(s, MAXLEN);
-
-	    if (test == NULL || (*sdir != '\0' && strstr(s, sdir))) {
-		strcpy(s, paths.userdir);
-	    }
-	} 
     } else {
-	strcpy(s, paths.userdir);   
+	strcpy(s, paths.workdir);
     }
 
     slash_terminate(s);
@@ -750,36 +738,47 @@ static void root_check (void)
 
 void gretl_config_init (void)
 {
-#ifndef USE_GCONF
     sprintf(rcfile, "%s/.gretl2rc", getenv("HOME"));
-#endif
-
     read_rc();
+    set_gretl_startdir();
     set_gd_fontpath();
-
-    if (!expert) {
-	root_check();
-    }
+    root_check();
 }
 
 #endif /* *nix versus Windows */
 
+static void highlight_options_entry (const char *vname)
+{
+    GtkWidget *w;
+    int i;
+
+    for (i=0; rc_vars[i].key != NULL; i++) {
+	if (!strcmp(vname, rc_vars[i].key)) {
+	    w = rc_vars[i].widget;
+	    if (w != NULL && GTK_IS_ENTRY(w)) {
+		gtk_editable_select_region(GTK_EDITABLE(w), 0, -1);
+		gtk_widget_grab_focus(w);
+	    }
+	    break;
+	}
+    }
+}
+
 static void option_dialog_canceled (GtkWidget *w, int *c)
 {
     *c = 1;
 }
 
-int options_dialog (int page) 
+int options_dialog (int page, const char *varname) 
 {
     static GtkWidget *dialog;
-
     GtkWidget *notebook;
     GtkWidget *button;
     GtkWidget *hbox;
     int canceled = 0;
 
     if (dialog != NULL) {
-	gdk_window_raise(dialog->window);
+	gtk_window_present(GTK_WINDOW(dialog));
 	return 0;
     }
 
@@ -799,7 +798,6 @@ int options_dialog (int page)
     make_prefs_tab(notebook, TAB_MAIN);
     make_prefs_tab(notebook, TAB_DBS);
     make_prefs_tab(notebook, TAB_PROGS);
-    make_prefs_tab(notebook, TAB_SAVE);
     make_prefs_tab(notebook, TAB_VCV);
     make_prefs_tab(notebook, TAB_MAN);
 
@@ -831,8 +829,12 @@ int options_dialog (int page)
 		     dialog);
     gtk_widget_show(button);
 
-    if (page > 0) {
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), page);
+    if (page > 1 && page < TAB_MAX) {
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), page - 1);
+    }
+
+    if (varname != NULL) {
+	highlight_options_entry(varname);
     }
 
     gtk_widget_show(dialog);
@@ -840,11 +842,6 @@ int options_dialog (int page)
     return canceled;
 }
 
-void options_dialog_callback (gpointer p, guint u, GtkWidget *w)
-{
-    options_dialog(u);
-}
-
 static void flip_sensitive (GtkWidget *w, gpointer data)
 {
     GtkWidget *entry = GTK_WIDGET(data);
@@ -852,15 +849,17 @@ static void flip_sensitive (GtkWidget *w, gpointer data)
     gtk_widget_set_sensitive(entry, GTK_TOGGLE_BUTTON(w)->active);
 }
 
-void filesel_set_path_callback (const char *setting, char *strvar)
+void set_path_callback (char *setvar, char *setting)
 {
     int i = 0;
 
     while (rc_vars[i].key != NULL) {
-	if (rc_vars[i].var == (void *) strvar) {
-	    /* FIXME: utf-8 issues here?? */
-	    gtk_entry_set_text(GTK_ENTRY(rc_vars[i].widget), 
-			       setting);
+	if (rc_vars[i].var == (void *) setvar) {
+	    /* FIXME: utf-8 issues here? */
+	    if (rc_vars[i].widget != NULL) {
+		gtk_entry_set_text(GTK_ENTRY(rc_vars[i].widget), 
+				   setting);
+	    }
 	    break;
 	}
 	i++;
@@ -897,57 +896,57 @@ static gboolean takes_effect_on_restart (void)
 
 #define HIDE_SPANISH_MANUAL 1
 
-static GList *get_settings_list (void *var, int *nopt)
+static const char **get_radio_setting_strings (void *var, int *n)
 {
-    char *hc_strs[] = {
-	"HC0", "HC1", "HC2", "HC3", "HC3a", "HAC"
-    };
-    char *hc_panel_strs[] = {
-	"Arellano", "PCSE"
-    };
-    char *garch_strs[] = {
-	"QML", "BW"
-    };
-    const char *man_strs[] = {
+    static const char *man_strs[] = {
         N_("English (US letter paper)"),
         N_("English (A4 paper)"),
         N_("Italian"),
 	N_("Spanish")
     };
-    GList *list = NULL;
-    int i, n = 0;
+    const char **strs = NULL;
+
+    *n = 0;
+
+    if (var == &manpref) {
+	strs = man_strs;
+	*n = sizeof man_strs / sizeof man_strs[0];
+#if HIDE_SPANISH_MANUAL
+	*n -= 1;
+#endif
+    }
+
+    return strs;
+}
+
+static const char **get_list_setting_strings (void *var, int *n)
+{
+    static const char *hc_strs[] = {
+	"HC0", "HC1", "HC2", "HC3", "HC3a", "HAC"
+    };
+    static const char *hc_panel_strs[] = {
+	"Arellano", "PCSE"
+    };
+    static const char *garch_strs[] = {
+	"QML", "BW"
+    };
+    const char **strs = NULL;
+
+    *n = 0;
 
     if (var == hc_xsect || var == hc_tseri) {
-	n = sizeof hc_strs / sizeof hc_strs[0];
-	if (var == hc_xsect) n--;
-	for (i=0; i<n; i++) {
-	    list = g_list_append(list, hc_strs[i]);
-	}
+	strs = hc_strs;
+	*n = sizeof hc_strs / sizeof hc_strs[0];
+	if (var == hc_xsect) *n -= 1;
     } else if (var == hc_panel) {
-	n = sizeof hc_panel_strs / sizeof hc_panel_strs[0];
-	for (i=0; i<n; i++) {
-	    list = g_list_append(list, hc_panel_strs[i]);
-	}
+	strs = hc_panel_strs;
+	*n = sizeof hc_panel_strs / sizeof hc_panel_strs[0];
     } else if (var == hc_garch) {
-	n = sizeof garch_strs / sizeof garch_strs[0];
-	for (i=0; i<n; i++) {
-	    list = g_list_append(list, garch_strs[i]);
-	}
-    } else if (var == &manpref) {
-	n = sizeof man_strs / sizeof man_strs[0];
-#if HIDE_SPANISH_MANUAL
-	n--;
-#endif
-	for (i=0; i<n; i++) {
-	    list = g_list_append(list, _(man_strs[i]));
-	}
-    }
+	strs = garch_strs;
+	*n = sizeof garch_strs / sizeof garch_strs[0];
+    } 
 
-    if (nopt != NULL) {
-	*nopt = n;
-    }
-
-    return list;
+    return strs;
 }
 
 static void 
@@ -956,21 +955,18 @@ get_table_sizes (int page, int *n_str, int *n_bool, int *n_browse)
     int i;
 
     for (i=0; rc_vars[i].key != NULL; i++) {
-	if (rc_vars[i].tab != page) {
-	    continue;
+	if (rc_vars[i].tab == page) {
+	    if (rc_vars[i].flags & BROWSER) {
+		*n_browse += 1;
+	    }
+	    if (rc_vars[i].flags & BOOLSET) {
+		*n_bool += 1;
+	    } else if (rc_vars[i].flags & RADIOSET) {
+		*n_bool += 1;
+	    } else if (!(rc_vars[i].flags & INVISET)) {
+		*n_str += 1;
+	    }
 	}
-
-	if (rc_vars[i].flags & BROWSER) {
-	    *n_browse += 1;
-	}
-
-	if (rc_vars[i].flags & BOOLSET) {
-	    *n_bool += 1;
-	} else if (rc_vars[i].flags & RADIOSET) {
-	    *n_bool += 1;
-	} else if (!(rc_vars[i].flags & INVISET)) {
-	    *n_str += 1;
-	} 
     }
 }
 
@@ -1004,8 +1000,6 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	w = gtk_label_new(_("Databases"));
     } else if (tab == TAB_PROGS) {
 	w = gtk_label_new(_("Programs"));
-    } else if (tab == TAB_SAVE) {
-	w = gtk_label_new(_("File Open/Save"));
     } else if (tab == TAB_VCV) {
 	w = gtk_label_new(_("HCCME"));
     } else if (tab == TAB_MAN) {
@@ -1057,8 +1051,10 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rc->widget), FALSE);
 	    }
 
-	    /* special case: warning */
-	    if (!strcmp(rc->key, "lcnumeric") || !strcmp(rc->key, "wimp")) {
+	    /* special case: deferred processing */
+	    if (!strcmp(rc->key, "lcnumeric") ||
+		!strcmp(rc->key, "usecwd") ||
+		!strcmp(rc->key, "wimp")) {
 		g_signal_connect(G_OBJECT(rc->widget), "toggled",
 				 G_CALLBACK(takes_effect_on_restart), 
 				 NULL);
@@ -1145,7 +1141,8 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	    }
 	} else if (rc->flags & LISTSET) {
 	    char *strvar = (char *) rc->var;
-	    GList *list;
+	    const char **strs;
+	    int j, nopt, active = 0;
 
 	    s_len++;
 
@@ -1156,24 +1153,25 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 				      w, 0, 1, s_len - 1, s_len);
 	    gtk_widget_show(w);
 
-	    rc->widget = gtk_combo_new();
+	    rc->widget = gtk_combo_box_new_text();
 	    gtk_table_attach(GTK_TABLE(s_table), rc->widget, 
 			     1, 2, s_len-1, s_len,
 			     0, 0, 0, 0);
-
-	    list = get_settings_list(rc->var, NULL);
-	    gtk_combo_set_popdown_strings(GTK_COMBO(rc->widget), list);
-	    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(rc->widget)->entry), strvar);
-	    gtk_entry_set_width_chars(GTK_ENTRY(GTK_COMBO(rc->widget)->entry), 8);
-	    gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(rc->widget)->entry), 
-				      FALSE);
+	    strs = get_list_setting_strings(rc->var, &nopt);
+	    for (j=0; j<nopt; j++) {
+		gtk_combo_box_append_text(GTK_COMBO_BOX(rc->widget), 
+					  strs[j]);
+		if (!strcmp(strs[j], strvar)) {
+		    active = j;
+		}
+	    }
+	    gtk_combo_box_set_active(GTK_COMBO_BOX(rc->widget), active);
 	    gtk_widget_show(rc->widget);
 	} else if (rc->flags & RADIOSET) {
-	    int i, rcval = *(int *) (rc->var);
-	    int nopt = 0;
+	    int nopt, j, rcval = *(int *) (rc->var);
 	    GtkWidget *b;
 	    GSList *group = NULL;
-	    GList *list, *mylist;
+	    const char **strs;
 
 	    b_len++;
 	    b = gtk_label_new(_(rc->description));
@@ -1182,26 +1180,24 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 				      b_len - 1, b_len);
 	    gtk_widget_show(b);
 
-	    mylist = list = get_settings_list(rc->var, &nopt);
+	    strs = get_radio_setting_strings(rc->var, &nopt);
 
-	    for (i=0; i<nopt; i++) {
+	    for (j=0; j<nopt; j++) {
 		b_len++;
 		gtk_table_resize(GTK_TABLE(b_table), b_len, 2);
-		b = gtk_radio_button_new_with_label(group, mylist->data);
+		b = gtk_radio_button_new_with_label(group, _(strs[j]));
 		gtk_table_attach_defaults(GTK_TABLE(b_table), b, 
 					  b_col, b_col + 1, 
 					  b_len - 1, b_len);
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), i == rcval);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), j == rcval);
 		g_object_set_data(G_OBJECT(b), "action", 
-				  GINT_TO_POINTER(i));
+				  GINT_TO_POINTER(j));
 		g_signal_connect(G_OBJECT(b), "clicked",
 				 G_CALLBACK(radio_change_value),
 				 rc->var);
 		gtk_widget_show(b);
 		group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(b));
-		mylist = g_list_next(mylist);
 	    }
-	    g_list_free(list);
 	} else if (!(rc->flags & INVISET)) { 
 	    /* visible string variable */
 	    char *strvar = (char *) rc->var;
@@ -1221,8 +1217,8 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	    gtk_entry_set_text(GTK_ENTRY(rc->widget), strvar);
 	    gtk_widget_show(rc->widget);
 
-	    /* path browse button */
 	    if (rc->flags & BROWSER) {
+		/* add path browse button */
 		w = make_path_browse_button(rc);
 		gtk_table_attach_defaults(GTK_TABLE(s_table), 
 					  w, 2, 3, s_len-1, s_len);
@@ -1405,14 +1401,14 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 		    strncat(strvar, str, rc_vars[i].len - 1);
 		}
 	    } else if (rc_vars[i].flags & LISTSET) {
-		GtkWidget *entry = GTK_COMBO(rc_vars[i].widget)->entry;
+		GtkComboBox *box = GTK_COMBO_BOX(rc_vars[i].widget);
+		gchar *boxval;
 
-		str = gtk_entry_get_text(GTK_ENTRY(entry));
-		if (str != NULL && *str != '\0') { 
-		    strvar = (char *) rc_vars[i].var;
-		    *strvar = '\0';
-		    strncat(strvar, str, rc_vars[i].len - 1);
-		}
+		boxval = gtk_combo_box_get_active_text(box);
+		strvar = (char *) rc_vars[i].var;
+		*strvar = '\0';
+		strcat(strvar, boxval);
+		g_free(boxval);
 	    }
 	}
     }
@@ -1427,7 +1423,6 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 
     /* register these for session using libset apparatus */
     libset_set_bool(USE_QR, useqr);
-    libset_set_bool(USE_CWD, usecwd);
     libset_set_bool(SHELL_OK, shellok);
     set_xsect_hccme(hc_xsect);
     set_tseries_hccme(hc_tseri);
@@ -1437,9 +1432,55 @@ static void apply_changes (GtkWidget *widget, gpointer data)
     gretl_www_init(paths.dbhost, dbproxy, use_proxy);
 }
 
-#ifndef USE_GCONF
+static void boolvar_to_str (void *b, char *s)
+{
+    if (*(int *) b) {
+	strcpy(s, "true");
+    } else {
+	strcpy(s, "false");
+    }
+}
 
-static void str_to_boolvar (char *s, void *b)
+#ifndef G_OS_WIN32
+
+static int write_plain_text_rc (void) 
+{
+    FILE *rc;
+    char val[6];
+    char *strvar;
+    int i;
+
+    rc = gretl_fopen(rcfile, "w");
+    if (rc == NULL) {
+	file_write_errbox(rcfile);
+	return E_FOPEN;
+    }
+
+    fprintf(rc, "# gretl config file\n");
+
+    for (i=0; rc_vars[i].var != NULL; i++) {
+	fprintf(rc, "# %s\n", rc_vars[i].description);
+	if (rc_vars[i].flags & BOOLSET) {
+	    boolvar_to_str(rc_vars[i].var, val);
+	    fprintf(rc, "%s = %s\n", rc_vars[i].key, val);
+	} else if (rc_vars[i].flags & INTSET) {
+	    fprintf(rc, "%s = %d\n", rc_vars[i].key, *(int *) rc_vars[i].var);
+	} else {
+	    strvar = (char *) rc_vars[i].var;
+	    fprintf(rc, "%s = %s\n", rc_vars[i].key, strvar);
+	}
+    }
+
+    rc_save_file_lists(rc);
+
+    fclose(rc);
+
+    return 0;
+}
+
+#endif /* !G_OS_WIN32 */
+
+static void str_to_boolvar (const char *s, void *b)
 {
     int *bvar = (int *) b;
 
@@ -1452,7 +1493,7 @@ static void str_to_boolvar (char *s, void *b)
     }	
 }
 
-static void str_to_int (char *s, void *b)
+static void str_to_int (const char *s, void *b)
 {
     int *ivar = (int *) b;
 
@@ -1463,17 +1504,6 @@ static void str_to_int (char *s, void *b)
     }
 }
 
-#endif
-
-static void boolvar_to_str (void *b, char *s)
-{
-    if (*(int *) b) {
-	strcpy(s, "true");
-    } else {
-	strcpy(s, "false");
-    }
-}
-
 static int dir_exists (const char *dname, FILE *fp)
 {
     DIR *test;
@@ -1498,17 +1528,53 @@ static int dir_exists (const char *dname, FILE *fp)
     return ok;
 }
 
+#if !defined(G_OS_WIN32) && !defined(OSX_BUILD)
+
+static void maybe_fix_viewpdf (void)
+{
+    gchar *prog = NULL;
+    const gchar *viewers[] = {
+	"xpdf",
+	"acroread",
+	"evince",
+	"kpdf",
+	"gpdf",
+	NULL
+    };
+    int i;
+
+    prog = g_find_program_in_path(viewpdf);
+
+    if (prog != NULL) {
+	g_free(prog);
+    } else {
+	for (i=0; viewers[i] != NULL; i++) {
+	    if (strcmp(viewers[i], viewpdf)) {
+		prog = g_find_program_in_path(viewers[i]);
+		if (prog != NULL) {
+		    strcpy(viewpdf, viewers[i]);
+		    g_free(prog);
+		    break;
+		}
+	    }
+	}
+    }
+}
+
+#endif
+
 static int common_read_rc_setup (void)
 {
     int err = 0;
 
     libset_set_bool(USE_QR, useqr);
-    libset_set_bool(USE_CWD, usecwd);
     libset_set_bool(SHELL_OK, shellok);
+    libset_set_bool(USE_CWD, usecwd);
     set_gp_colors();
-    
+
     set_xsect_hccme(hc_xsect);
     set_tseries_hccme(hc_tseri);
+    set_panel_hccme(hc_panel);
     set_garch_robust_vcv(hc_garch);
 
     err = gretl_set_paths(&paths, set_paths_opt);
@@ -1516,6 +1582,10 @@ static int common_read_rc_setup (void)
     gretl_www_init(paths.dbhost, dbproxy, use_proxy);
     set_tex_use_pdf(latex);
 
+#if !defined(G_OS_WIN32) && !defined(OSX_BUILD)
+    maybe_fix_viewpdf();
+#endif
+
 # if defined(HAVE_TRAMO) || defined(HAVE_X12A)
     set_tramo_x12a_status();
 # endif
@@ -1527,98 +1597,7 @@ static int common_read_rc_setup (void)
     return err;
 }
 
-/* next section: variant versions of write_rc and read_rc, depending
-   on both gconf presence and platform
-*/
-
-#ifdef USE_GCONF 
-
-void write_rc (void) 
-{
-    GConfClient *client;   
-    char key[MAXSTR];
-    gboolean bval;
-    int ival;
-    char *strvar;
-    int i;
-
-    client = gconf_client_get_default();
-
-    for (i=0; rc_vars[i].key != NULL; i++) {
-	sprintf(key, "/apps/gretl/%s", rc_vars[i].key);
-	if (rc_vars[i].flags & BOOLSET) {
-	    bval = *(gboolean *) rc_vars[i].var;
-	    gconf_client_set_bool(client, key, bval, NULL);
-	} else if (rc_vars[i].flags & INTSET) {
-	    ival = *(int *) rc_vars[i].var;
-	    gconf_client_set_int(client, key, ival, NULL);
-	} else {
-	    strvar = (char *) rc_vars[i].var;
-	    gconf_client_set_string(client, key, strvar, NULL);
-	}
-    }
-
-    save_file_lists(client);
-    g_object_unref(G_OBJECT(client));
-    gretl_set_paths(&paths, set_paths_opt);
-    record_shell_opt();
-}
-
-static void read_rc (void) 
-{
-    GConfClient *client;
-    GError *error = NULL;
-    gboolean bval;
-    int ival;
-    gchar *strval;
-    char key[MAXSTR];
-    int i;
-
-    client = gconf_client_get_default();
-
-    for (i=0; rc_vars[i].key != NULL; i++) {
-	sprintf(key, "/apps/gretl/%s", rc_vars[i].key);
-	if (rc_vars[i].flags & BOOLSET) {
-	    bval = gconf_client_get_bool(client, key, &error);
-	    if (error) {
-		fprintf(stderr, "Error reading %s\n", rc_vars[i].key);
-		g_clear_error(&error);
-	    } else {
-		*(int *) rc_vars[i].var = (int) bval;
-	    }
-	} else if (rc_vars[i].flags & INTSET) {
-	    ival = gconf_client_get_int(client, key, &error);
-	    if (error) {
-		fprintf(stderr, "Error reading %s\n", rc_vars[i].key);
-		g_clear_error(&error);
-	    } else {
-		*(int *) rc_vars[i].var = ival;
-	    }	    
-	} else {
-	    strval = gconf_client_get_string(client, key, &error);
-	    if (error) {
-		fprintf(stderr, "Error reading %s\n", rc_vars[i].key);
-		g_clear_error(&error);
-	    } else if (strval != NULL) {
-		if (*strval != '\0') {
-		    char *strvar = (char *) rc_vars[i].var;
-
-		    *strvar = '\0';
-		    strncat(strvar, strval, rc_vars[i].len - 1);
-		}
-		g_free(strval);
-	    }
-	}
-    }
-
-    read_file_lists(client);
-    g_object_unref(G_OBJECT(client));
-    common_read_rc_setup();
-}
-
-/* end of gconf version, now win32 */
-
-#elif defined(G_OS_WIN32)
+#ifdef G_OS_WIN32
 
 void write_rc (void) 
 {
@@ -1661,12 +1640,6 @@ void write_rc (void)
 	}
     }
 
-#if 0
-    if (err) {
-	win_show_last_error();
-    }
-#endif
-
     save_file_lists();
     gretl_set_paths(&paths, set_paths_opt);
 }
@@ -1732,8 +1705,8 @@ void read_rc (void)
     char *strvar;
     int i;
 
-    if (get_network_settings() && *paths.userdir != '\0') {
-	int err = set_gretl_user_dir(paths.userdir, &paths);
+    if (get_network_settings() && *paths.workdir != '\0') {
+	int err = set_gretl_work_dir(paths.workdir, &paths);
 
 	if (err) {
 	    gui_errmsg(err);
@@ -1792,48 +1765,45 @@ void read_rc (void)
     set_app_font(NULL);
 }
 
-#else /* end of gconf and win32 versions, now plain GTK */
+#else /* end of win32 version, now plain GTK */
 
-void write_rc (void) 
+void write_rc (void)
 {
-    FILE *rc;
-    char val[6];
-    char *strvar;
-    int i;
+    int err;
 
-    rc = fopen(rcfile, "w");
-    if (rc == NULL) {
-	file_write_errbox(rcfile);
-	return;
+    err = write_plain_text_rc();
+    if (!err) {
+	gretl_set_paths(&paths, set_paths_opt);
+	record_shell_opt();
     }
+}
 
-    fprintf(rc, "# gretl config file (note: not used by gnome version)\n");
+static void find_and_write_var (const char *key, const char *val)
+{
+    char *strvar;
+    int j;
 
-    for (i=0; rc_vars[i].var != NULL; i++) {
-	fprintf(rc, "# %s\n", rc_vars[i].description);
-	if (rc_vars[i].flags & BOOLSET) {
-	    boolvar_to_str(rc_vars[i].var, val);
-	    fprintf(rc, "%s = %s\n", rc_vars[i].key, val);
-	} else if (rc_vars[i].flags & INTSET) {
-	    fprintf(rc, "%s = %d\n", rc_vars[i].key, *(int *) rc_vars[i].var);
-	} else {
-	    strvar = (char *) rc_vars[i].var;
-	    fprintf(rc, "%s = %s\n", rc_vars[i].key, strvar);
+    for (j=0; rc_vars[j].key != NULL; j++) {
+	if (!strcmp(key, rc_vars[j].key)) {
+	    if (rc_vars[j].flags & BOOLSET) {
+		str_to_boolvar(val, rc_vars[j].var);
+	    } else if (rc_vars[j].flags & INTSET) {
+		str_to_int(val, rc_vars[j].var);
+	    } else {
+		strvar = (char *) rc_vars[j].var;
+		*strvar = '\0';
+		strncat(strvar, val, rc_vars[j].len - 1);
+	    }
+	    break;
 	}
     }
-
-    save_file_lists(rc);
-    fclose(rc);
-    gretl_set_paths(&paths, set_paths_opt);
-    record_shell_opt();
 }
 
 static void read_rc (void) 
 {
     FILE *fp;
     char line[MAXLEN], key[32], linevar[MAXLEN];
-    char *strvar;
-    int i, j;
+    int i;
 
     fp = fopen(rcfile, "r");
     if (fp == NULL) {
@@ -1855,19 +1825,8 @@ static void read_rc (void)
 	if (sscanf(line, "%s", key) == 1) {
 	    strcpy(linevar, line + strlen(key) + 3); 
 	    chopstr(linevar); 
-	    for (j=0; rc_vars[j].key != NULL; j++) {
-		if (!strcmp(key, rc_vars[j].key)) {
-		    if (rc_vars[j].flags & BOOLSET) {
-			str_to_boolvar(linevar, rc_vars[j].var);
-		    } else if (rc_vars[j].flags & INTSET) {
-			str_to_int(linevar, rc_vars[j].var);
-		    } else {
-			strvar = (char *) rc_vars[j].var;
-			*strvar = '\0';
-			strncat(strvar, linevar, rc_vars[j].len - 1);
-		    }
-		    break;
-		}
+	    if (*linevar) {
+		find_and_write_var(key, linevar);
 	    }
 	}
 	i++;
@@ -1878,7 +1837,7 @@ static void read_rc (void)
     common_read_rc_setup();
 }
 
-#endif /* end of "plain gtk" versions of read_rc, write_rc */
+#endif /* end of non-Windows versions of read_rc, write_rc */
 
 /* font selection: non-Windows, gtk-2.0 version first */
 
@@ -2158,8 +2117,8 @@ static void color_select_callback (GtkWidget *button, GtkWidget *w)
     gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(csel), &gcolor);
 
     rgb.r = (unsigned char) (scale_round(gcolor.red));
-    rgb.g = (unsigned char) (scale_round(gcolor.red));
-    rgb.b = (unsigned char) (scale_round(gcolor.red));
+    rgb.g = (unsigned char) (scale_round(gcolor.green));
+    rgb.b = (unsigned char) (scale_round(gcolor.blue));
 
     i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "colnum"));
 
@@ -2259,52 +2218,6 @@ void graph_color_selector (GtkWidget *w, gpointer p)
 
 /* end graph color selection apparatus */
 
-#ifndef G_OS_WIN32
-
-static void real_set_userdir (GtkWidget *widget, dialog_t *dlg)
-{
-    const gchar *dirname;
-    int err;
-
-    dirname = edit_dialog_get_text(dlg);
-    err = set_gretl_user_dir(dirname, &paths);
-
-    if (err) {
-	gui_errmsg(err);
-	return;
-    } else {
-#if defined(HAVE_TRAMO) || defined(HAVE_X12A)
-	set_tramo_x12a_status();
-#endif
-	close_dialog(dlg);
-    }
-}
-
-void first_time_set_user_dir (void)
-{
-    DIR *test;
-
-    /* see if the already-specified userdir exists */
-    if (*paths.userdir != '\0') {
-	test = opendir(paths.userdir);
-	if (test != NULL) {
-	    closedir(test);
-	    return;
-	}
-    }
-	
-    /* user dir is not specified, or doesn't exist */
-    edit_dialog (_("gretl: working directory"), 
-                 _("You seem to be using gretl for the first time.\n"
-		   "Please enter a directory for gretl user files."),
-                 paths.userdir, 
-                 real_set_userdir, NULL, 
-                 CREATE_USERDIR, VARCLICK_NONE,
-		 NULL);
-}
-
-#endif /* G_OS_WIN32 */
-
 void dump_rc (void) 
 {
     char dumper[MAXLEN];
@@ -2313,7 +2226,7 @@ void dump_rc (void)
     char val[6];
     int i;
 
-    sprintf(dumper, "%sconfig-dump.txt", paths.userdir);
+    sprintf(dumper, "%sconfig-dump.txt", paths.workdir);
 
     fp = gretl_fopen(dumper, "w");
     if (fp == NULL) {
@@ -2351,3 +2264,172 @@ void dump_rc (void)
     fclose(fp);
 }
 
+int gui_set_working_dir (char *dirname)
+{
+    int err = set_gretl_work_dir(dirname, &paths);
+
+    if (err) {
+	gui_errmsg(err);
+	delete_from_filelist(FILE_LIST_WDIR, dirname);
+    } else {
+	mkfilelist(FILE_LIST_WDIR, dirname);
+    }
+
+    return err;
+}
+
+struct wdir_setter {
+    GtkWidget *dialog;
+    GtkWidget *wdir_entry;
+    GtkWidget *r1, *r2;
+};
+
+/* callback from the file selector */
+
+void set_working_dir_callback (GtkWidget *w, char *path)
+{
+    GtkWidget *entry = GTK_COMBO(w)->entry;
+
+    gtk_entry_set_text(GTK_ENTRY(entry), path);
+}
+
+static void wdir_browse_callback (GtkWidget *w, struct wdir_setter *wset)
+{
+    GtkWidget *entry = wset->wdir_entry;
+    const char *s = N_("gretl working directory");
+
+    file_selector_with_parent(_(s), SET_WDIR, FSEL_DATA_MISC, entry, 
+			      wset->dialog);
+}
+
+static void free_fname (gchar *s, gpointer p)
+{
+    g_free(s);
+}
+
+static void 
+add_wdir_content (GtkWidget *dialog, struct wdir_setter *wset) 
+{
+    GtkWidget *hbox, *vbox, *w = NULL;
+    GSList *group = NULL;
+    GList *list = NULL;
+    char tmp[MAXLEN];
+
+    vbox = GTK_DIALOG(dialog)->vbox;
+    list = get_working_dir_list();
+
+    hbox = gtk_hbox_new(FALSE, 5);
+    w = gtk_label_new(_("Working directory:"));
+    gtk_box_pack_start(GTK_BOX(hbox), w, 0, 0, 5); 
+
+    strcpy(tmp, paths.workdir); 
+    trim_slash(tmp);
+ 
+    /* combo + browse button for current working dir */
+    w = gtk_combo_new();
+    gtk_container_add(GTK_CONTAINER(hbox), w);
+    gtk_combo_set_popdown_strings(GTK_COMBO(w), list);
+    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(w)->entry), tmp);
+    gtk_entry_set_width_chars(GTK_ENTRY(GTK_COMBO(w)->entry), 32);
+    gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(w)->entry), 
+			      TRUE);
+    wset->wdir_entry = w;
+    w = gtk_button_new_with_label(_("Browse..."));
+    g_signal_connect(G_OBJECT(w), "clicked",
+		     G_CALLBACK(wdir_browse_callback), wset);
+    gtk_box_pack_start(GTK_BOX(hbox), w, 0, 0, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, 0, 0, 5); 
+
+    vbox_add_hsep(vbox);
+
+    hbox = gtk_hbox_new(FALSE, 5);
+    w = gtk_label_new(_("On start-up, gretl should use:"));
+    gtk_box_pack_start(GTK_BOX(hbox), w, 0, 0, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, 0, 0, 5);
+
+    /* radio 1 for "next time" */
+    hbox = gtk_hbox_new(FALSE, 5);
+    w = gtk_radio_button_new_with_label(group, 
+					_("the directory selected above"));
+    group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(w));
+    gtk_box_pack_start(GTK_BOX(hbox), w, 0, 0, 5);
+    gtk_container_add(GTK_CONTAINER(vbox), hbox);
+    wset->r1 = w;
+
+    /* radio 2 for "next time" */
+    hbox = gtk_hbox_new(FALSE, 5);
+    w = gtk_radio_button_new_with_label(group, _("the current directory "
+						 "as determined via the shell"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), usecwd);
+    gtk_box_pack_start(GTK_BOX(hbox), w, 0, 0, 5);
+    gtk_container_add(GTK_CONTAINER(vbox), hbox);
+    wset->r2 = w;
+
+    g_list_foreach(list, (GFunc) free_fname, NULL);
+    g_list_free(list);
+}
+
+static void 
+apply_wdir_changes (GtkWidget *w, struct wdir_setter *wset)
+{ 
+    GtkWidget *entry;
+    char tmp[MAXLEN];
+    const char *str;
+    int err;
+
+    entry = GTK_COMBO(wset->wdir_entry)->entry;
+    str = gtk_entry_get_text(GTK_ENTRY(entry));
+    *tmp = '\0';
+    strncat(tmp, str, MAXLEN - 2);
+
+    err = set_gretl_work_dir(tmp, &paths);
+    if (err) {
+	gui_errmsg(err);
+	delete_from_filelist(FILE_LIST_WDIR, tmp);
+    } else {
+	mkfilelist(FILE_LIST_WDIR, tmp);
+    }
+
+    usecwd = GTK_TOGGLE_BUTTON(wset->r2)->active;
+}
+
+void working_dir_dialog (void) 
+{
+    static GtkWidget *dialog;
+    struct wdir_setter wset;
+    GtkWidget *button;
+    GtkWidget *hbox;
+
+    if (dialog != NULL) {
+	gtk_window_present(GTK_WINDOW(dialog));
+	return;
+    }
+
+    dialog = gretl_dialog_new(_("gretl: working directory"), 
+			      mdata->w, GRETL_DLG_BLOCK);
+    gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), 5);
+    g_signal_connect(G_OBJECT(dialog), "destroy", 
+		     G_CALLBACK(gtk_widget_destroyed), 
+		     &dialog);
+
+    wset.dialog = dialog;
+    add_wdir_content(dialog, &wset);
+
+    hbox = GTK_DIALOG(dialog)->action_area;
+
+    button = cancel_button(hbox);
+    g_signal_connect(G_OBJECT(button), "clicked", 
+		     G_CALLBACK(delete_widget), 
+		     dialog);
+
+    button = ok_button(hbox);
+    g_signal_connect(G_OBJECT(button), "clicked", 
+		     G_CALLBACK(apply_wdir_changes), &wset);
+    g_signal_connect(G_OBJECT(button), "clicked", 
+		     G_CALLBACK(delete_widget), 
+		     dialog);
+
+    context_help_button(hbox, WORKDIR);
+
+    gtk_widget_show_all(dialog);
+}
