@@ -7,7 +7,7 @@ void read_rc (void);
 void gretl_config_init (void);
 #endif
 
-void set_program_startdir (const char *callname);
+void set_gretl_startdir (void);
 
 int using_hc_by_default (void);
 
@@ -19,9 +19,7 @@ void dump_rc (void);
 
 void force_english_help (void);
 
-int options_dialog (int page);
-
-void options_dialog_callback (gpointer p, guint u, GtkWidget *w);
+int options_dialog (int page, const char *varname);
 
 void font_selector (gpointer data, guint which, GtkWidget *widget);
 
@@ -43,7 +41,13 @@ void color_patch_button_reset (GtkWidget *button, int cnum);
 
 void get_default_dir (char *s, int action);
 
-void filesel_set_path_callback (const char *setting, char *strvar);
+int gui_set_working_dir (char *dirname);
+
+void set_working_dir_callback (GtkWidget *w, char *path);
+
+void working_dir_dialog (void);
+
+void set_path_callback (char *setvar, char *setting);
 
 void set_datapage (const char *str);
 
@@ -53,10 +57,6 @@ const char *get_datapage (void);
 
 const char *get_scriptpage (void);
 
-#ifndef G_OS_WIN32
-void first_time_set_user_dir (void);
-#endif
-
 int check_for_prog (const char *prog);
 
 #endif /* SETTINGS_H */
