@@ -25,10 +25,15 @@
                       r == CLI_HELP_EN || \
                       r == GUI_HELP_EN)
 
+extern int tabwidth;
+extern int smarttab;
+
 void text_set_cursor (GtkWidget *w, GdkCursorType cspec);
 
 void cursor_to_top (windata_t *vwin);
 
+void cursor_to_mark (windata_t *vwin, GtkTextMark *mark);
+
 gint get_char_width (GtkWidget *widget);
 
 GtkTextBuffer *gretl_text_buf_new (void);
@@ -50,9 +55,12 @@ void text_paste (windata_t *vwin, guint u, GtkWidget *widget);
 
 void text_undo (windata_t *vwin, guint u, GtkWidget *widget);
 
+int text_can_undo (windata_t *vwin);
+
 void textview_set_text_colorized (GtkWidget *view, const char *buf);
 
-void textview_append_text_colorized (GtkWidget *view, const char *buf);
+void textview_append_text_colorized (GtkWidget *view, const char *buf,
+				     int trim);
 
 void textview_insert_file (windata_t *vwin, const char *fname);
 
@@ -70,10 +78,6 @@ void set_help_topic_buffer (windata_t *hwin, int hcode, int pos, int en);
 gboolean help_popup_handler (GtkWidget *w, GdkEventButton *event, 
 			     gpointer p);
 
-gboolean 
-script_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer p);
-
-
 void create_source (windata_t *vwin, int hsize, int vsize, 
 		    gboolean editable);
 
@@ -81,4 +85,6 @@ void sourceview_insert_file (windata_t *vwin, const char *fname);
 
 void sourceview_insert_buffer (windata_t *vwin, const char *buf);
 
+void script_tabs_dialog (GtkWidget *w, windata_t *vwin);
+
 #endif /* TEXTBUF_H */
