@@ -516,6 +516,34 @@ void tree_view_get_string (GtkTreeView *view, int row, int col, gchar **val)
     g_free(path);
 }
 
+static void tree_view_set_string (GtkTreeView *view, int row, int col, 
+				  const gchar *val, int tstore)
+{
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+    gchar *path;
+
+    model = gtk_tree_view_get_model(view);
+    path = g_strdup_printf("%d", row);
+    gtk_tree_model_get_iter_from_string(model, &iter, path);
+    if (tstore == 1) {
+	gtk_tree_store_set(GTK_TREE_STORE(model), &iter, col, val, -1);
+    } else {
+	gtk_list_store_set(GTK_LIST_STORE(model), &iter, col, val, -1);
+    }
+    g_free(path);
+}
+
+void list_store_set_string (GtkTreeView *view, int row, int col, const gchar *val)
+{
+    tree_view_set_string(view, row, col, val, 0);
+}
+
+void tree_store_set_string (GtkTreeView *view, int row, int col, const gchar *val)
+{
+    tree_view_set_string(view, row, col, val, 1);
+}
+
 void tree_view_get_int (GtkTreeView *view, int row, int col, int *val)
 {
     GtkTreeModel *model;
