@@ -28,9 +28,75 @@ enum {
     ADF_EG_RESIDS = 1 << 1
 } adf_flags;
 
+/* replace y with demeaned or detrended y */
+
+static int GLS_demean_detrend (double *y, int T, int test)
+{
+    gretl_matrix *ya = NULL;
+    gretl_matrix *Za = NULL;
+    gretl_matrix *b = NULL;
+    double c;
+    int t, zcols;
+    int err = 0;
+
+    zcols = (test == UR_TREND)? 2 : 1;
+
+    if (T - zcols <= 0) {
+	return E_DF;
+    }
+
+    ya = gretl_column_vector_alloc(T);
+    Za = gretl_matrix_alloc(T, zcols);
+    b = gretl_column_vector_alloc(zcols);
+
+    if (ya == NULL || Za == NULL || b == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    c = (test == UR_CONST)? (1.0 - 7.0/T) : (1.0 - 13.5/T);
+
+    ya->val[0] = y[0];
+    for (t=1; t<T; t++) {
+	ya->val[t] = y[t] - y[t-1] * c;
+    }
+
+    gretl_matrix_set(Za, 0, 0, 1);
+    if (zcols == 2) {
+	gretl_matrix_set(Za, 0, 1, 1);
+    }
+
+    for (t=1; t<T; t++) {
+	gretl_matrix_set(Za, t, 0, 1 - c);
+	if (zcols == 2) {
+	    gretl_matrix_set(Za, t, 1, t+1 - t*c);
+	}
+    }
+
+    err = gretl_matrix_ols(ya, Za, b, NULL, NULL, NULL);
+
+    if (!err) {
+	for (t=0; t<T; t++) {
+	    y[t] -= b->val[0];
+	    if (zcols == 2) {
+		y[t] -= b->val[1] * (t+1);
+	    }
+	}
+    }    
+
+ bailout:
+
+    gretl_matrix_free(ya);
+    gretl_matrix_free(Za);
+    gretl_matrix_free(b);
+    
+    return err;
+}
+
 static int *
 adf_prepare_vars (int order, int varno, int nseas, int *d0,
-		  double ***pZ, DATAINFO *pdinfo)
+		  double ***pZ, DATAINFO *pdinfo,
+		  gretlopt opt)
 {
     int nl = 5 + nseas;
     int i, j, orig_t1 = pdinfo->t1;
@@ -46,6 +112,32 @@ adf_prepare_vars (int order, int varno, int nseas, int *d0,
 	return NULL;
     }
 
+    if (opt & OPT_G) {
+	/* GLS adjustment wanted */
+	int test = (opt & OPT_T)? UR_TREND : UR_CONST;
+	int t, v = pdinfo->v;
+
+	err = dataset_add_series(1, pZ, pdinfo);
+	if (!err) {
+	    int T, offset = 0;
+
+	    for (t=0; t<=pdinfo->t2; t++) {
+		(*pZ)[v][t] = (*pZ)[varno][t];
+		if (na((*pZ)[v][t])) {
+		    offset = t+1;
+		}
+	    }
+	    T = pdinfo->t2 - offset + 1;
+	    err = GLS_demean_detrend((*pZ)[v] + offset, T, test);
+	}
+	if (err) {
+	    free(list);
+	    return NULL;
+	}
+	strcpy(pdinfo->varname[v], "yd");
+	varno = v;
+    }
+
     /* temporararily reset sample */
     pdinfo->t1 = 0;
 
@@ -101,10 +193,34 @@ adf_prepare_vars (int order, int varno, int nseas, int *d0,
     return list;
 }
 
+static double *df_gls_ct_cval (int T)
+{
+    /* Elliott, Rothenberg and Stock (1996), Table 1 */
+    static double df_gls_ct_cvals[4][4] = {
+	/* 10%     5%    2.5%    1% */
+	{ -2.89, -3.19, -3.46, -3.77 }, /* T = 50  */
+	{ -2.74, -3.03, -3.29, -3.58 }, /* T = 100 */
+	{ -2.64, -2.93, -3.18, -3.46 }, /* T = 200 */
+	{ -2.57, -2.89, -3.15, -3.48 }  /* \infty  */
+    };
+    int i = 3;
+
+    if (T <= 50){
+	i = 0;
+    } else if (T <= 100){
+	i = 1;
+    } else if (T <= 200){
+	i = 2;
+    } 
+
+    return df_gls_ct_cvals[i];
+}
+
 static void 
-print_adf_results (int order, int auto_order, double DFt, double pv, const MODEL *dfmod,
-		   int dfnum, const char *vname, int *blurb_done,
-		   unsigned char flags, int i, int niv, int nseas, PRN *prn)
+print_adf_results (int order, int auto_order, double DFt, double pv, 
+		   const MODEL *dfmod, int dfnum, const char *vname, 
+		   int *blurb_done, unsigned char flags, int i, 
+		   int niv, int nseas, gretlopt opt, PRN *prn)
 {
     const char *models[] = {
 	"(1 - L)y = (a-1)*y(-1) + e",
@@ -127,16 +243,21 @@ print_adf_results (int order, int auto_order, double DFt, double pv, const MODEL
     const char *urcstrs[] = {
 	"nc", "c", "ct", "ctt"
     };
-
     char pvstr[48];
+    char taustr[16];
 
     if (prn == NULL) return;
 
+    /* convert test-type to 0-base */
+    i--;
+
     if (na(pv)) {
 	sprintf(pvstr, "%s %s", _("p-value"), _("unknown"));
     } else {
+	int asy = (order > 0 || (opt & OPT_G));
+
 	sprintf(pvstr, "%s %.4g", 
-		(order > 0)? _("asymptotic p-value") : _("p-value"), 
+		(asy)? _("asymptotic p-value") : _("p-value"), 
 		pv);
     } 
 
@@ -167,6 +288,9 @@ print_adf_results (int order, int auto_order, double DFt, double pv, const MODEL
 
     if (!(flags & ADF_EG_RESIDS)) {
 	pprintf(prn, "   %s ", _(teststrs[i]));
+	if (opt & OPT_G) {
+	    pputs(prn, "(GLS) ");
+	}
 	if (nseas > 0 && i > 0) {
 	    pputs(prn, _("plus seasonal dummies"));
 	}
@@ -185,12 +309,27 @@ print_adf_results (int order, int auto_order, double DFt, double pv, const MODEL
 	}
     }
 
+    if (opt & OPT_G) {
+	strcpy(taustr, "tau");
+    } else {
+	sprintf(taustr, "tau_%s(%d)", urcstrs[i], niv);
+    }
+
     pprintf(prn, "   %s: %g\n"
-	    "   %s: tau_%s(%d) = %g\n"
-	    "   %s\n",
+	    "   %s: %s = %g\n",
 	    _("estimated value of (a - 1)"), dfmod->coeff[dfnum],
-	    _("test statistic"), urcstrs[i], niv, DFt,
-	    pvstr);	
+	    _("test statistic"), taustr, DFt);
+
+    if ((opt & OPT_G) && i+1 == UR_TREND) {
+	const double *c = df_gls_ct_cval(dfmod->nobs);
+
+	pprintf(prn, "\n   %*s    ", TRANSLATED_WIDTH(_("Critical values")), " ");
+	pprintf(prn, "%g%%     %g%%     %g%%     %g%%\n", 10.0, 5.0, 2.5, 1.0);
+	pprintf(prn, "   %s: %.2f   %.2f   %.2f   %.2f\n", 
+		_("Critical values"), c[0], c[1], c[2], c[3]);
+    } else {
+	pprintf(prn, "   %s\n", pvstr);
+    } 
 }
 
 static int auto_adjust_order (int *list, int order_max,
@@ -250,7 +389,8 @@ static void copy_list_values (int *targ, const int *src)
     }
 }
 
-static double df_pvalue_from_plugin (double tau, int n, int niv, int itv)
+static double df_pvalue_from_plugin (double tau, int n, int niv, int itv,
+				     gretlopt opt)
 {
     char datapath[FILENAME_MAX];
     void *handle;
@@ -273,6 +413,10 @@ static double df_pvalue_from_plugin (double tau, int n, int niv, int itv)
     append_dir(datapath, "plugins");
 #endif
 
+    if ((opt & OPT_G) && itv == UR_CONST) {
+	itv = UR_NO_CONST;
+    }
+
     pval = (*mackinnon_pvalue)(tau, n, niv, itv, datapath);
 
 #if ADF_DEBUG
@@ -337,7 +481,6 @@ static int real_adf_test (int varno, int order, int niv,
 			  PRN *prn)
 {
     MODEL dfmod;
-
     gretlopt eg_opt = OPT_NONE;
     int orig_nvars = pdinfo->v;
     int blurb_done = 0;
@@ -392,7 +535,16 @@ static int real_adf_test (int varno, int order, int niv,
 	nseas = pdinfo->pd - 1;
     }
 
-    list = adf_prepare_vars(order, varno, nseas, &d0, pZ, pdinfo);
+    if (test_opt_not_set(opt)) {
+	/* default model(s) */
+	if (opt & OPT_G) {
+	    opt |= OPT_C;
+	} else {
+	    opt |= (OPT_C | OPT_T | OPT_R);
+	}
+    }
+
+    list = adf_prepare_vars(order, varno, nseas, &d0, pZ, pdinfo, opt);
     if (list == NULL) {
 	return E_ALLOC;
     }
@@ -408,11 +560,6 @@ static int real_adf_test (int varno, int order, int niv,
 
     gretl_model_init(&dfmod);
 
-    if (test_opt_not_set(opt)) {
-	/* do the full default set of models */
-	opt |= (OPT_C | OPT_T | OPT_R);
-    }
-
     for (i=UR_NO_CONST; i<UR_MAX; i++) {
 	int j, k, dfnum = (i > UR_NO_CONST);
 	int itv = i;
@@ -427,6 +574,13 @@ static int real_adf_test (int varno, int order, int niv,
 	    copy_list_values(list, biglist);
 	}
 
+	if (opt & OPT_G) {
+	    /* DF-GLS: skip const, trend */
+	    list[0] = 2 + order;
+	    dfnum--;
+	    goto skipdet;
+	}
+
 	list[0] = 1 + order + i;
 
 	/* list[1] and list[2], plus the "order" lags, are in common
@@ -461,6 +615,8 @@ static int real_adf_test (int varno, int order, int niv,
 	    }	    
 	} 
 
+    skipdet:
+
 	if (auto_order) {
 	    order = auto_adjust_order(list, order_max, pZ, pdinfo, prn);
 	    if (order < 0) {
@@ -470,6 +626,10 @@ static int real_adf_test (int varno, int order, int niv,
 	    }
 	}
 
+#if ADF_DEBUG
+	printlist(list, "final ADF regression list");
+#endif
+
 	dfmod = lsq(list, pZ, pdinfo, OLS, OPT_A);
 	if (dfmod.errcode) {
 	    fprintf(stderr, "adf_test: dfmod.errcode = %d\n", 
@@ -483,17 +643,29 @@ static int real_adf_test (int varno, int order, int niv,
 
 	if (flags & ADF_EG_RESIDS) {
 	    itv = engle_granger_itv(eg_opt);
-	}
+	} 
 
-	pv = df_pvalue_from_plugin(DFt, 
-				   /* use asymptotic p-value for augmented case */
-				   (order > 0)? 0 : dfmod.nobs, 
-				   niv, itv);
+	if (getenv("DFGLS_NO_PVALUE")) {
+	    /* to speed up monte carlo stuff */
+	    pv = NADBL;
+	} else if ((opt & OPT_G) && itv == UR_TREND) {
+	    /* DF-GLS with trend: MacKinnon p-values won't work */
+	    pv = NADBL;
+	} else {
+	    /* Use asymp. p-value in augmented case; also the
+	       finite-sample MacKinnon p-values are not correct
+	       in the GLS case.
+	    */
+	    int asymp = (order > 0 || (opt & OPT_G));
+
+	    pv = df_pvalue_from_plugin(DFt, (asymp)? 0 : dfmod.nobs, 
+				       niv, itv, opt);
+	}
 
 	if (!(opt & OPT_Q)) {
 	    print_adf_results(order, auto_order, DFt, pv, &dfmod, dfnum, 
-			      pdinfo->varname[varno], &blurb_done, flags, 
-			      itv - 1, niv, nseas, prn);
+			      pdinfo->varname[varno], &blurb_done, flags,
+			      itv, niv, nseas, opt, prn);
 	}
 
 	if (opt & OPT_V) {
@@ -548,11 +720,32 @@ static int real_adf_test (int varno, int order, int niv,
 int adf_test (int order, const int *list, double ***pZ,
 	      DATAINFO *pdinfo, gretlopt opt, PRN *prn)
 {
-    int i, err = 0;
+    int t1 = pdinfo->t1;
+    int t2 = pdinfo->t2;
+    int v, vlist[2] = {1,0};
+    int i, err;
+
+    /* GLS incompatible with no const, quadratic trend or seasonals */
+    err = incompatible_options(opt, OPT_N | OPT_R | OPT_G);
+    if (!err) {
+	err = incompatible_options(opt, OPT_D | OPT_G);
+    }
+
+    if (!err && (opt & OPT_G)) {
+	/* under GLS, have to choose between cases */
+	err = incompatible_options(opt, OPT_C | OPT_T);
+    }
 
     for (i=1; i<=list[0] && !err; i++) {
-	err = real_adf_test(list[i], order, 1, pZ, pdinfo, opt, 
-			    0, prn);
+	v = list[i];
+	vlist[1] = v;
+	err = list_adjust_t1t2(vlist, (const double **) *pZ, pdinfo);
+	if (!err) {
+	    err = real_adf_test(v, order, 1, pZ, pdinfo, opt, 
+				0, prn);
+	}
+	pdinfo->t1 = t1;
+	pdinfo->t2 = t2;
     }
 
     return err;
