@@ -545,7 +545,7 @@ int bhhh_max (LL_FUNC loglik,
     }
 
     /* create temp dataset for OPG regression: k vars plus constant */    
-    tinfo = create_new_dataset(&tZ, k + 1, minfo->n, 0);
+    tinfo = create_auxiliary_dataset(&tZ, k + 1, minfo->n);
     if (tinfo == NULL) {
 	free(delta);
 	free(ctemp);
@@ -594,6 +594,12 @@ int bhhh_max (LL_FUNC loglik,
 	if (bmod->errcode) {
 	    fprintf(stderr, "BHHH model error code = %d\n", bmod->errcode);
 	    err = E_NOCONV;
+	} else if (bmod->list[0] < blist[0]) {
+	    /* failed to get the full complement of coefficients */
+	    err = E_DATA;
+	}
+
+	if (err) {
 	    break;
 	}
 
