@@ -1,19 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License along
- *   with this program; if not, write to the Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -193,7 +194,7 @@ static int make_bs_flags (gretlopt opt)
 int maybe_adjust_B (int B, double a, int flags)
 {
     if (B <= 0) {
-	B = get_bootstrap_replications();
+	B = libset_get_int(BOOTREP);
     }
 
     if (flags & BOOT_CI) {
@@ -275,7 +276,7 @@ static void make_normal_y (boot *bs)
     int i, t;
 
     /* generate scaled normal errors */
-    gretl_normal_dist(bs->y->val, 0, bs->T - 1);
+    gretl_rand_normal(bs->y->val, 0, bs->T - 1);
     for (t=0; t<bs->T; t++) {
 	bs->y->val[t] *= bs->SE;
     }
@@ -300,7 +301,7 @@ resample_vector (const gretl_matrix *u0, gretl_matrix *u,
     int i, t;
 
     /* generate uniform random series */
-    gretl_uniform_dist(z, 0, T - 1);
+    gretl_rand_uniform(z, 0, T - 1);
 
     /* sample from source vector based on indices */
     for (t=0; t<T; t++) {
