@@ -9,12 +9,15 @@
 typedef struct Laginfo_ Laginfo;
 
 enum {
-    CMD_NOLIST = 1 << 0,
-    CMD_IGNORE = 1 << 1
+    CMD_NOLIST  = 1 << 0, /* command doesn't have a list of variables */
+    CMD_IGNORE  = 1 << 1, /* line should be ignored */
+    CMD_NULLIST = 1 << 2, /* command has been given a null list on input */
+    CMD_SUBST   = 1 << 3  /* string substitution has been done on command */
 };
 
-#define cmd_nolist(c) (c->flags & CMD_NOLIST)
-#define cmd_ignore(c) (c->flags & CMD_IGNORE)
+#define cmd_nolist(c)    (c->flags & CMD_NOLIST)
+#define cmd_ignore(c)    (c->flags & CMD_IGNORE)
+#define cmd_subst(c)     (c->flags & CMD_SUBST)
 
 struct CMD_ {
     char word[FN_NAMELEN];      /* command word */
@@ -45,7 +48,9 @@ struct ExecState_ {
     equation_system *sys;
     gretl_restriction *rset;
     GRETL_VAR *var;
-    DATAINFO *subinfo; /* record of incoming sub-sample for functions */
+    void *prev_model;
+    GretlObjType prev_type;
+    char *submask; /* record of incoming sub-sample for functions */
     int alt_model;
     int in_comment;
     int funcerr;
@@ -61,9 +66,17 @@ void gretl_exec_state_init (ExecState *s,
 
 void gretl_exec_state_clear (ExecState *s);
 
-int gretl_exec_state_finalize (void);
+void gretl_exec_state_uncomment (ExecState *s);
 
-int maybe_exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo);
+void gretl_if_state_clear (void);
+
+int gretl_if_state_finalize (void);
+
+int gretl_if_state_record (void);
+
+int gretl_if_state_check (int indent0);
+
+int maybe_exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo);
 
 int plausible_genr_start (const char *s, const DATAINFO *pdinfo);
 
