@@ -25,6 +25,7 @@
 #include "libset.h"
 #include "gretl_panel.h"
 #include "var.h"
+#include "system.h"
 #include "missing_private.h"
 #include "matrix_extra.h"
 
@@ -167,17 +168,11 @@ wald_test (const int *list, MODEL *pmod, double *chisq, double *F)
     mask = mask_from_test_list(list, pmod, &err);
 
     if (!err) {
-	C = gretl_vcv_matrix_from_model(pmod, mask);
-	if (C == NULL) {
-	    err = E_ALLOC;
-	} 
+	C = gretl_vcv_matrix_from_model(pmod, mask, &err);
     }
 
     if (!err) {
-	b = gretl_coeff_vector_from_model(pmod, mask);
-	if (b == NULL) {
-	    err = E_ALLOC;
-	} 
+	b = gretl_coeff_vector_from_model(pmod, mask, &err);
     }  
 
     if (!err) {
@@ -273,10 +268,10 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 {
     ModelTest *test = NULL;
     double pval = NADBL;
-    int verbosity = 2;
+    int asy, verbosity = 2;
     int stat_ok, i;
 
-    if (cmp->err || cmp->ci == LAD) {
+    if (cmp->err) {
 	return;
     }
 
@@ -310,15 +305,16 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 	}
     }
 
+    asy = !LIMDEP(cmp->ci); /* ?? */
+
     if (!na(cmp->chisq)) {
-	pval = chisq_cdf_comp(cmp->chisq, cmp->dfn);
+	pval = chisq_cdf_comp(cmp->dfn, cmp->chisq);
 	if (verbosity > 0) {
 	    pprintf(prn, "\n  %s:%s%s(%d) = %g, ",  
-		    (LIMDEP(cmp->ci))? _("Test statistic") : 
-		    _("Asymptotic test statistic"),
-		    (LIMDEP(cmp->ci))? " " : "\n    ",
+		    (asy)? _("Asymptotic test statistic") : _("Test statistic"),
+		    (asy)? "\n    " : " ",
 		    _("Chi-square"), cmp->dfn, cmp->chisq);
-	    pprintf(prn, _("with p-value = %g\n\n"), pval);
+	    pprintf(prn, _("with p-value = %g\n"), pval);
 	}
 
 	record_test_result(cmp->chisq, pval, (cmp->cmd == OMIT)? _("omit") : _("add"));
@@ -333,13 +329,14 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 
 	if (verbosity > 0 && !na(cmp->F)) {
 	    /* alternate form */
-	    pval = snedecor_cdf_comp(cmp->F, cmp->dfn, cmp->dfd);
-	    pprintf(prn, "  %s: F(%d, %d) = %g, ", _("F-form"), 
+	    pval = snedecor_cdf_comp(cmp->dfn, cmp->dfd, cmp->F);
+	    pputs(prn, (asy)? "    " : "  ");
+	    pprintf(prn, "%s: F(%d, %d) = %g, ", _("F-form"), 
 		    cmp->dfn, cmp->dfd, cmp->F);
 	    pprintf(prn, _("with p-value = %g\n"), pval);
 	}	    
     } else if (!na(cmp->F)) {
-	pval = snedecor_cdf_comp(cmp->F, cmp->dfn, cmp->dfd);
+	pval = snedecor_cdf_comp(cmp->dfn, cmp->dfd, cmp->F);
 	if (verbosity > 0) {
 	    pprintf(prn, "\n  %s: %s(%d, %d) = %g, ", _("Test statistic"), 
 		    (cmp->robust)? _("Robust F") : "F",
@@ -364,6 +361,7 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
     }
 
     if (verbosity > 1 && cmp->score >= 0) {
+	pputc(prn, '\n');
 	pprintf(prn, _("  Of the %d model selection statistics, %d "), 
 		C_MAX, cmp->score);
 	if (cmp->score == 1) {
@@ -527,7 +525,7 @@ static int obs_diff_ok (const MODEL *old, const MODEL *new)
 {
     int tdiff, ndiff = new->nobs - old->nobs;
 
-    if (old->ci == CORC || old->ci == HILU || old->ci == PWE) {
+    if (old->ci == AR1) {
 	return 0;
     }
 
@@ -551,6 +549,7 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 {
     MODEL rep;
     const char *param = NULL;
+    char altparm[32] = {0};
     double rho = 0.0;
     int *list = *plist;
     int mc = get_model_count();
@@ -562,14 +561,16 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 
     /* recreate options and auxiliary vars, if required */
 
-    if (orig->ci == CORC || orig->ci == HILU || orig->ci == PWE) {
-	gretlopt hlopt = OPT_NONE;
-
-	if (orig->ci == HILU && gretl_model_get_int(orig, "no-corc")) {
-	    hlopt = OPT_B;
+    if (orig->ci == AR1) {
+	if (gretl_model_get_int(orig, "hilu")) {
+	    myopt = OPT_H;
+	    if (gretl_model_get_int(orig, "no-corc")) {
+		myopt |= OPT_B;
+	    }
+	} else if (gretl_model_get_int(orig, "pwe")) {
+	    myopt = OPT_P;
 	}
-	rho = estimate_rho(list, pZ, pdinfo, orig->ci, 
-			   &rep.errcode, hlopt, prn);
+	rho = estimate_rho(list, pZ, pdinfo, myopt, prn, &rep.errcode);
     } else if (orig->ci == WLS || orig->ci == AR || 
 	       (orig->ci == POISSON && gretl_model_get_int(orig, "offset_var"))) {
 	int *full_list = full_model_list(orig, list);
@@ -602,6 +603,19 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 		myopt |= OPT_T;
 	    }
 	}
+    } else if (orig->ci == LAD && gretl_model_get_int(orig, "rq")) {
+	double x;
+
+	x = gretl_model_get_double(orig, "tau");
+	sprintf(altparm, "%g", x);
+	if (gretl_model_get_int(orig, "rq_nid")) {
+	    myopt |= OPT_R;
+	}
+	x = gretl_model_get_double(orig, "rq_alpha");
+	if (!na(x)) {
+	    myopt |= OPT_I;
+	    set_optval_double(QUANTREG, OPT_I, x);
+	}
     }
 
     if (rep.errcode) {
@@ -630,7 +644,11 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 	rep = tobit_model(list, pZ, pdinfo, NULL);
 	break;
     case LAD:
-	rep = lad(list, pZ, pdinfo);
+	if (gretl_model_get_int(orig, "rq")) {
+	    rep = quantreg(altparm, list, pZ, pdinfo, myopt, NULL);
+	} else {
+	    rep = lad(list, pZ, pdinfo);
+	}
 	break;
     case POISSON:
 	rep = poisson_model(list, pZ, pdinfo, NULL);
@@ -655,7 +673,7 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 	rep = panel_model(list, pZ, pdinfo, myopt, prn);
 	break;
     default:
-	/* handles OLS, WLS, HSK, HCCM, etc. */
+	/* handles OLS, AR1, WLS, HSK, HCCM, etc. */
 	if (gretl_model_get_int(orig, "robust")) {
 	    myopt |= OPT_R;
 	}
@@ -740,7 +758,7 @@ real_nonlinearity_test (MODEL *pmod, int *list,
     } else {
 	double trsq = aux.rsq * aux.nobs;
 	int df = list[0] - pmod->list[0];
-	double pval = chisq_cdf_comp(trsq, df);
+	double pval = chisq_cdf_comp(df, trsq);
 
 	aux.aux = aux_code;
 
@@ -760,10 +778,10 @@ real_nonlinearity_test (MODEL *pmod, int *list,
 	    test = model_test_new((aux_code == AUX_SQ)?
 				  GRETL_TEST_SQUARES : GRETL_TEST_LOGS);
 	    if (test != NULL) {
-		model_test_set_teststat(test, GRETL_STAT_TR2);
+		model_test_set_teststat(test, GRETL_STAT_LM);
 		model_test_set_dfn(test, df);
 		model_test_set_value(test, trsq);
-		model_test_set_pvalue(test, chisq_cdf_comp(trsq, df));
+		model_test_set_pvalue(test, chisq_cdf_comp(df, trsq));
 		maybe_add_test_to_model(pmod, test);
 	    }
 	}
@@ -915,6 +933,10 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
 	return E_NOTIMP;
     }
 
+    if (exact_fit_check(orig, prn)) {
+	return 0;
+    }
+
     /* check for changes in original list members */
     err = list_members_replaced(orig->list, pdinfo, orig->ID);
     if (err) {
@@ -1025,10 +1047,9 @@ static int wald_omit_test (const int *list, MODEL *pmod,
 */
 
 static int auto_drop_var (const MODEL *pmod, int *list,
-			  DATAINFO *pdinfo, int d0, 
-			  PRN *prn, int *err)
+			  DATAINFO *pdinfo, double alpha_max,
+			  int d0, PRN *prn, int *err)
 {
-    double alpha_max = .10; /* FIXME make this configurable? */
     double tstat, pv = 0.0, tmin = 4.0;
     int i, k = -1;
     int ret = 0;
@@ -1088,6 +1109,7 @@ static int auto_omit (MODEL *orig, MODEL *new,
 		      gretlopt est_opt, gretlopt opt,
 		      PRN *prn)
 {
+    double amax;
     int *tmplist = NULL;
     int err = 0;
 
@@ -1096,7 +1118,12 @@ static int auto_omit (MODEL *orig, MODEL *new,
 	return E_ALLOC;
     }
 
-    if (!auto_drop_var(orig, tmplist, pdinfo, 1, prn, &err)) {
+    amax = get_optval_double(OMIT, OPT_A);
+    if (na(amax)) {
+	amax = 0.10;
+    }
+
+    if (!auto_drop_var(orig, tmplist, pdinfo, amax, 1, prn, &err)) {
 	free(tmplist);
 	return (err)? err : E_NOOMIT;
     }    
@@ -1108,7 +1135,7 @@ static int auto_omit (MODEL *orig, MODEL *new,
 	    err = new->errcode;
 	} else {
 	    list_copy_values(tmplist, new->list);
-	    if (auto_drop_var(new, tmplist, pdinfo, 0, prn, &err)) {
+	    if (auto_drop_var(new, tmplist, pdinfo, amax, 0, prn, &err)) {
 		model_count_minus();
 		clear_model(new);
 	    } else {
@@ -1171,6 +1198,11 @@ static int omit_options_inconsistent (gretlopt opt)
 	}
     }
 
+    if ((opt & OPT_A) && (opt & OPT_W)) {
+	/* auto and Wald options incompatible */
+	return 1;
+    }
+
     return 0;
 }
 
@@ -1294,26 +1326,30 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
     return err;
 }
 
-static int 
-ljung_box (int m, int t1, int t2, const double *y, double *pLB)
+double ljung_box (int m, int t1, int t2, const double *y, int *err)
 {
     double acf, LB = 0.0;
     int k, n = t2 - t1 + 1;
 
+    *err = 0;
+
     /* calculate acf up to lag m, cumulating LB */
     for (k=1; k<=m; k++) {
 	acf = gretl_acf(k, t1, t2, y);
 	if (na(acf)) {
-	    return E_MISSDATA;
+	    *err = E_MISSDATA;
+	    break;
 	}
 	LB += acf * acf / (n - k);
     }
 
-    LB *= n * (n + 2.0);
+    if (*err) {
+	LB = NADBL;
+    } else {
+	LB *= n * (n + 2.0);
+    }
 
-    *pLB = LB;
-
-    return 0;
+    return LB;
 }
 
 /**
@@ -1321,7 +1357,9 @@ ljung_box (int m, int t1, int t2, const double *y, double *pLB)
  * @pmod: pointer to model to be tested.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
- * @opt: if contains %OPT_S, save test results to model.
+ * @opt: if contains %OPT_S, save test results to model. %OPT_Q
+ * suppresses the printout of the auxiliary regression. %OPT_R and
+ * %OPT_C stand for "squares only" and "cubes only", respectively.
  * @prn: gretl printing struct.
  *
  * Carries out Ramsey's RESET test for model specification.
@@ -1334,43 +1372,81 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 {
     int *newlist;
     MODEL aux;
-    int i, t, v = pdinfo->v; 
     double RF;
+    int i, t, v = pdinfo->v; 
+    int addcols;
+    const char *mode;
     int err = 0;
 
-    if (pmod->ci != OLS) return E_OLSONLY;
+    if (pmod->ci != OLS) {
+	return E_OLSONLY;
+    }
+
+    err = incompatible_options(opt, OPT_C | OPT_R);
+
+    if (err) {
+	return err;
+    }
+
+    if (exact_fit_check(pmod, prn)) {
+	return 0;
+    }
 
     gretl_model_init(&aux);
 
-    if (pmod->ncoeff + 2 >= pdinfo->t2 - pdinfo->t1) {
+    if (opt & OPT_C) {
+	addcols = 1;
+	mode = N_("squares only");
+    } else if (opt & OPT_R) {
+	addcols = 1;
+	mode = N_("cubes only");
+    } else {
+	addcols = 2;
+	mode = N_("squares and cubes");
+    }
+
+    if (pmod->ncoeff + addcols >= pdinfo->t2 - pdinfo->t1) {
 	return E_DF;
     }
 
-    newlist = malloc((pmod->list[0] + 3) * sizeof *newlist);
+    newlist = malloc((pmod->list[0] + addcols + 1) * sizeof *newlist);
     if (newlist == NULL) {
 	err = E_ALLOC;
     } else {
-	newlist[0] = pmod->list[0] + 2;
+	newlist[0] = pmod->list[0] + addcols;
 	for (i=1; i<=pmod->list[0]; i++) {
 	    newlist[i] = pmod->list[i];
 	}
-	if (dataset_add_series(2, pZ, pdinfo)) {
+	if (dataset_add_series(addcols, pZ, pdinfo)) {
 	    err = E_ALLOC;
 	}
     }
 
     if (!err) {
-	/* add yhat^2, yhat^3 to data set */
+	/* add yhat^2 and/or yhat^3 to data set */
+	int sqcol = v;
+	int cubecol = (opt & OPT_C) ? v : v + 1;
+
 	for (t = pmod->t1; t<=pmod->t2; t++) {
 	    double xx = pmod->yhat[t];
 
-	    (*pZ)[v][t] = xx * xx;
-	    (*pZ)[v+1][t] = xx * xx * xx;
+	    if (!(opt & OPT_C)) {
+		(*pZ)[sqcol][t] = xx * xx;
+	    }
+	    if (!(opt & OPT_R)) {
+		(*pZ)[cubecol][t] = xx * xx * xx;
+	    }
+	}
+
+	if (!(opt & OPT_C)) {
+	    strcpy(pdinfo->varname[sqcol], "yhat^2");
+	    newlist[pmod->list[0] + 1] = sqcol;
+	}
+
+	if (!(opt & OPT_R)) {
+	    strcpy(pdinfo->varname[cubecol], "yhat^3");
+	    newlist[newlist[0]] = cubecol;
 	}
-	strcpy(pdinfo->varname[v], "yhat^2");
-	strcpy(pdinfo->varname[v+1], "yhat^3");
-	newlist[pmod->list[0] + 1] = v;
-	newlist[pmod->list[0] + 2] = v + 1;
     }
 
     if (!err) {
@@ -1385,20 +1461,31 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	double pval;
 
 	aux.aux = AUX_RESET;
-	printmodel(&aux, pdinfo, OPT_NONE, prn);
-	RF = ((pmod->ess - aux.ess) / 2) / (aux.ess / aux.dfd);
-	pval = snedecor_cdf_comp(RF, 2, aux.dfd);
 
-	pprintf(prn, "\n%s: F = %f,\n", _("Test statistic"), RF);
+	if (!(opt & OPT_Q)) {
+	    printmodel(&aux, pdinfo, OPT_NONE, prn);
+	} else {
+	    if (!(opt & OPT_G)) {
+		pputc(prn, '\n');
+	    }
+	    pputs(prn, _("RESET specification test"));
+	    pprintf(prn, " (%s)\n", _(mode));
+	}
+
+	RF = ((pmod->ess - aux.ess) / addcols) / (aux.ess / aux.dfd);
+	pval = snedecor_cdf_comp(addcols, aux.dfd, RF);
+
+	pprintf(prn, "%s: F = %f,\n", _("Test statistic"), RF);
 	pprintf(prn, "%s = P(F(%d,%d) > %g) = %.3g\n", _("with p-value"), 
-		2, aux.dfd, RF, pval);
+		addcols, aux.dfd, RF, pval);
+	pputc(prn, '\n');
 
 	if (opt & OPT_S) {
 	    ModelTest *test = model_test_new(GRETL_TEST_RESET);
 
 	    if (test != NULL) {
 		model_test_set_teststat(test, GRETL_STAT_RESET);
-		model_test_set_dfn(test, 2);
+		model_test_set_dfn(test, addcols);
 		model_test_set_dfd(test, aux.dfd);
 		model_test_set_value(test, RF);
 		model_test_set_pvalue(test, pval);
@@ -1410,7 +1497,7 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     }
 
     free(newlist);
-    dataset_drop_last_variables(2, pZ, pdinfo); 
+    dataset_drop_last_variables(addcols, pZ, pdinfo); 
     clear_model(&aux); 
 
     return err;
@@ -1572,7 +1659,7 @@ static int tsls_autocorr_test (MODEL *pmod, int order,
 
 	aux.aux = AUX_AR;
 	gretl_model_set_int(&aux, "BG_order", order);
-	pval = snedecor_cdf_comp(x, order, aux.nobs - pmod->ncoeff - order);
+	pval = snedecor_cdf_comp(order, aux.nobs - pmod->ncoeff - order, x);
 
 	if (opt & OPT_Q) {
 	    bg_test_header(order, prn, 1);
@@ -1639,6 +1726,7 @@ int autocorr_test (MODEL *pmod, int order,
     int smpl_t2 = pdinfo->t2;
     int *newlist = NULL;
     MODEL aux;
+    double RSSxe, RSSx = pmod->ess;
     int i, t, n = pdinfo->n, v = pdinfo->v; 
     double trsq, LMF, lb, pval = 1.0;
     int err = 0;
@@ -1656,12 +1744,13 @@ int autocorr_test (MODEL *pmod, int order,
     }
 
     if (dataset_is_panel(pdinfo)) {
+#if 1 /* FIXME */
+	return E_NOTIMP;
+#else
 	return panel_autocorr_test(pmod, order, *pZ, pdinfo, opt, prn);
+#endif
     }
 
-    /* impose original sample range */
-    impose_model_smpl(pmod, pdinfo);
-
     gretl_model_init(&aux);
 
     if (order <= 0) {
@@ -1672,7 +1761,7 @@ int autocorr_test (MODEL *pmod, int order,
 	return E_DF;
     }
 
-    newlist = malloc((pmod->list[0] + order + 1) * sizeof *newlist);
+    newlist = gretl_list_new(pmod->list[0] + order);
 
     if (newlist == NULL) {
 	err = E_ALLOC;
@@ -1681,49 +1770,68 @@ int autocorr_test (MODEL *pmod, int order,
 	for (i=2; i<=pmod->list[0]; i++) {
 	    newlist[i] = pmod->list[i];
 	}
-	if (dataset_add_series(1, pZ, pdinfo)) {
+	if (dataset_add_series(1 + order, pZ, pdinfo)) {
 	    err = E_ALLOC;
 	}
     }
 
     if (!err) {
-	/* add uhat to data set */
+	/* add uhat to data set: substitute zeros for 
+	   pre-sample values */
 	for (t=0; t<n; t++) {
-	    (*pZ)[v][t] = pmod->uhat[t];
+	    if (t < pmod->t1) {
+		(*pZ)[v][t] = 0.0;
+	    } else {
+		(*pZ)[v][t] = pmod->uhat[t];
+	    }
 	}
 	strcpy(pdinfo->varname[v], "uhat");
 	strcpy(VARLABEL(pdinfo, v), _("residual"));
-	/* then lags of same */
+	/* then order lags of same */
 	for (i=1; i<=order; i++) {
-	    int lnum;
+	    int s, lv = v + i;
+	    double ul;
 
-	    lnum = laggenr(v, i, pZ, pdinfo);
-	    if (lnum < 0) {
-		sprintf(gretl_errmsg, _("lagging uhat failed"));
-		err = E_LAGS;
-	    } else {
-		newlist[pmod->list[0] + i] = lnum;
+	    sprintf(pdinfo->varname[lv], "uhat_%d", i);
+	    newlist[pmod->list[0] + i] = lv;
+	    for (t=0; t<pdinfo->n; t++) {
+		s = t - i;
+		if (s < 0) {
+		    (*pZ)[lv][t] = 0.0;
+		} else {
+		    ul = (*pZ)[v][s];
+		    (*pZ)[lv][t] = (na(ul))? 0.0 : ul;
+		}
 	    }
+	} 
+    }
+
+    /* LMF apparatus: see Kiviet, Review of Economic Studies,
+       53/2, 1986, equation (5), p. 245.
+    */
+
+    if (!err) {
+	/* regression on [X~E], using original sample */
+	impose_model_smpl(pmod, pdinfo);
+	newlist[1] = v;
+	aux = lsq(newlist, pZ, pdinfo, OLS, OPT_A);
+	err = aux.errcode;
+	if (err) {
+	   errmsg(err, prn);
+	} else { 
+	    RSSxe = aux.ess;
 	}
     }
 
     if (!err) {
-	newlist[1] = v;
-	/* printlist(newlist); */
-	aux = lsq(newlist, pZ, pdinfo, OLS, OPT_A);
-	err = aux.errcode;
-	if (err) {
-	    errmsg(aux.errcode, prn);
-	}
-    } 
+	int dfd = aux.nobs - pmod->ncoeff - order;
+	int lberr;
 
-    if (!err) {
 	aux.aux = AUX_AR;
 	gretl_model_set_int(&aux, "BG_order", order);
 	trsq = aux.rsq * aux.nobs;
-	LMF = (aux.rsq/(1.0 - aux.rsq)) * 
-	    (aux.nobs - pmod->ncoeff - order)/order; 
-	pval = snedecor_cdf_comp(LMF, order, aux.nobs - pmod->ncoeff - order);
+	LMF = ((RSSx - RSSxe) / RSSxe) * dfd / order;
+	pval = snedecor_cdf_comp(order, dfd, LMF);
 
 	if (pmod->aux != AUX_VAR) {
 	    if (opt & OPT_Q) {
@@ -1734,15 +1842,19 @@ int autocorr_test (MODEL *pmod, int order,
 	    pprintf(prn, "%s: LMF = %f,\n", _("Test statistic"), LMF);
 	    pprintf(prn, "%s = P(F(%d,%d) > %g) = %.3g\n", _("with p-value"), 
 		    order, aux.nobs - pmod->ncoeff - order, LMF, pval);
+
 	    pprintf(prn, "\n%s: TR^2 = %f,\n", 
 		    _("Alternative statistic"), trsq);
-	    pprintf(prn, "%s = P(%s(%d) > %g) = %.3g\n\n", 	_("with p-value"), 
-		    _("Chi-square"), order, trsq, chisq_cdf_comp(trsq, order));
-	    if (ljung_box(order, pmod->t1, pmod->t2, (*pZ)[v], &lb) == 0) {
+	    pprintf(prn, "%s = P(%s(%d) > %g) = %.3g\n\n", _("with p-value"), 
+		    _("Chi-square"), order, trsq, chisq_cdf_comp(order, trsq));
+
+	    lb = ljung_box(order, pmod->t1, pmod->t2, (*pZ)[v], &lberr);
+	    if (!na(lb)) {
 		pprintf(prn, "Ljung-Box Q' = %g %s = P(%s(%d) > %g) = %.3g\n", 
 			lb, _("with p-value"), _("Chi-square"), order,
-			lb, chisq_cdf_comp(lb, order));
+			lb, chisq_cdf_comp(order, lb));
 	    }
+
 	    pputc(prn, '\n');
 	    record_test_result(LMF, pval, _("autocorrelation"));
 	}
@@ -1835,8 +1947,35 @@ make_chow_list (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     return chowlist;
 }
 
+static double QLR_get_critval (double Fmax, int dfn, int *a, int *approx)
+{
+    double crit = 0.0;
+    int i, j = dfn - 1;
+
+    *approx = 0;
+
+    if (j >= QLR_QMAX) {
+	j = QLR_QMAX - 1;
+	*approx = 1;
+    } 
+
+    for (i=2; i>=0; i--) {
+	if (Fmax > QLR_critvals[j][i]) {
+	    *a = (i == 2)? 1 : (i == 1)? 5 : 10;
+	    crit = QLR_critvals[j][i];
+	    break;
+	}
+    }
+
+    if (crit == 0.0) {
+	crit = QLR_critvals[j][0];
+    }
+
+    return crit;
+}
+
 static int QLR_graph (const double *Ft, int t1, int t2, 
-		      const DATAINFO *pdinfo)
+		      int tmax, int dfn, const DATAINFO *pdinfo)
 {
     const double *x = gretl_plotx(pdinfo);
     FILE *fp = NULL;
@@ -1846,6 +1985,8 @@ static int QLR_graph (const double *Ft, int t1, int t2,
 	return E_FOPEN;
     }
 
+    fputs("set key top left\n", fp);
+
     gretl_push_c_numeric_locale();
 
     fprintf(fp, "plot \\\n"
@@ -1863,7 +2004,7 @@ static int QLR_graph (const double *Ft, int t1, int t2,
     return gnuplot_make_graph();
 }
 
-static void save_QLR_test (MODEL *pmod, char *datestr,
+static void save_QLR_test (MODEL *pmod, const char *datestr,
 			   double Fmax, double crit, double alpha,
 			   int dfn, int dfd)
 {
@@ -1888,35 +2029,17 @@ static void QLR_print_result (MODEL *pmod,
     char datestr[OBSLEN];
     double crit = 0.0;
     int a = 0, approx = 0;
-    int i, j;
 
     ntodate(datestr, tmax, pdinfo);
 
     pputs(prn, _("Quandt likelihood ratio test for structural break at an "
-	  "unknown point,\nwith 15 percent trimming"));
+		 "unknown point,\nwith 15 percent trimming"));
     pputs(prn, ":\n\n");
     pprintf(prn, _("The maximum F(%d, %d) = %g occurs "
-	    "at observation %s"), dfn, dfd, Fmax, datestr);
+		   "at observation %s"), dfn, dfd, Fmax, datestr);
     pputc(prn, '\n');
 
-    j = dfn - 1;
-
-    if (j >= QLR_QMAX) {
-	j = QLR_QMAX - 1;
-	approx = 1;
-    } 
-
-    for (i=2; i>=0; i--) {
-	if (Fmax > QLR_critvals[j][i]) {
-	    a = (i == 2)? 1 : (i == 1)? 5 : 10;
-	    crit = QLR_critvals[j][i];
-	    break;
-	}
-    }
-
-    if (crit == 0.0) {
-	crit = QLR_critvals[j][0];
-    }
+    crit = QLR_get_critval(Fmax, dfn, &a, &approx);
 
     if (a > 0) {
 	pprintf(prn, _("Significant at the %d percent level "), a);
@@ -1995,6 +2118,10 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 	return E_OLSONLY;
     }
 
+    if (exact_fit_check(pmod, prn)) {
+	return 0;
+    }
+
     /* temporarily impose the sample that was in force when the
        original model was estimated */
     impose_model_smpl(pmod, pdinfo);
@@ -2064,7 +2191,7 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 	    QLR_print_result(pmod, Fmax, tmax, dfn, dfd, pdinfo, opt, prn);
 	    record_test_result(Fmax, NADBL, "QLR");
 	    if (Ft != NULL) {
-		QLR_graph(Ft, split, smax, pdinfo);
+		QLR_graph(Ft, split, smax, tmax, dfn, pdinfo);
 	    }
 	}
 
@@ -2087,7 +2214,7 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 	    printmodel(&chow_mod, pdinfo, OPT_NONE, prn);
 	    F = (pmod->ess - chow_mod.ess) * chow_mod.dfd / 
 		(chow_mod.ess * dfn);
-	    pval = snedecor_cdf_comp(F, dfn, chow_mod.dfd);
+	    pval = snedecor_cdf_comp(dfn, chow_mod.dfd, F);
 	    pprintf(prn, _("\nChow test for structural break at observation %s:\n"
 		    "  F(%d, %d) = %f with p-value %f\n\n"), chowdate,
 		    dfn, chow_mod.dfd, F, pval);
@@ -2187,6 +2314,8 @@ static int cusum_compute (MODEL *pmod, double *cresid, int T, int k,
     return err;
 }
 
+#define okfreq(p) (p == 1 || p == 4 || p == 12 || p == 24 || p == 52)
+
 static int cusum_do_graph (double a, double b, const double *W, 
 			   int t1, int k, int m, 
 			   DATAINFO *pdinfo, gretlopt opt)
@@ -2196,23 +2325,19 @@ static int cusum_do_graph (double a, double b, const double *W,
     double x0 = 0.0;
     int j, t;
     int err = 0;
-    double frac = 1;
-    double slope = b;
-
-    if( pdinfo->pd==4 || pdinfo->pd==12 ) {
-	slope *= pdinfo->pd;
-	frac = 1.0/pdinfo->pd;
-    }
+    double frac = 1.0;
 
     err = gnuplot_init(PLOT_CUSUM, &fq);
     if (err) {
 	return err;
     }
 
-    if (dataset_is_time_series(pdinfo)) { 
+    if (dataset_is_time_series(pdinfo) && okfreq(pdinfo->pd)) {
+	b *= pdinfo->pd;
+	frac /= pdinfo->pd;
         obs = gretl_plotx(pdinfo);
 	if (obs != NULL) {
-	    x0 = obs[t1+k];
+	    x0 = obs[t1 + k];
 	}
     }
 
@@ -2225,19 +2350,20 @@ static int cusum_do_graph (double a, double b, const double *W,
 	fprintf(fq, "set title '%s'\n",
 		/* xgettext:no-c-format */
 		G_("CUSUMSQ plot with 95% confidence band"));
-	fprintf(fq, "plot \\\n%g*(x-%g) title '' w dots lt 2, \\\n", slope, x0 - frac);
-	fprintf(fq, "%g+%g*(x-%g) title '' w lines lt 2, \\\n", -a, slope, x0 - frac);
-	fprintf(fq, "%g+%g*(x-%g) title '' w lines lt 2, \\\n", a, slope, x0 - frac);
+	fprintf(fq, "plot \\\n%g*(x-%g) title '' w dots lt 2, \\\n", b, x0 - frac);
+	fprintf(fq, "%g+%g*(x-%g) title '' w lines lt 2, \\\n", -a, b, x0 - frac);
+	fprintf(fq, "%g+%g*(x-%g) title '' w lines lt 2, \\\n", a, b, x0 - frac);
     } else {
 	fputs("set xzeroaxis\n", fq);
 	fprintf(fq, "set title '%s'\n",
 		/* xgettext:no-c-format */
 		G_("CUSUM plot with 95% confidence band"));
-	fprintf(fq, "plot \\\n%g+%g*(x-%g) title '' w lines lt 2, \\\n", a, slope, x0);
-	fprintf(fq, "%g-%g*(x-%g) title '' w lines lt 2, \\\n", -a, slope, x0);
+	fprintf(fq, "plot \\\n%g+%g*(x-%g) title '' w lines lt 2, \\\n", a, b, x0);
+	fprintf(fq, "%g-%g*(x-%g) title '' w lines lt 2, \\\n", -a, b, x0);
     }	
 
     fputs("'-' using 1:2 w linespoints lt 1\n", fq);
+
     for (j=0; j<m; j++) { 
 	t = t1 + k + j;
 	if (obs != NULL) {
@@ -2246,6 +2372,7 @@ static int cusum_do_graph (double a, double b, const double *W,
 	    fprintf(fq, "%d %g\n", t, W[j]);
 	}
     }
+
     fputs("e\n", fq);
 
     gretl_pop_c_numeric_locale();
@@ -2264,7 +2391,7 @@ static void cusum_harvey_collier (double wbar, double sigma, int m,
     double hct, pval;
 
     hct = (sqrt((double) m) * wbar) / sigma;
-    pval = student_pvalue_2(hct, m - 1);
+    pval = student_pvalue_2(m - 1, hct);
     pprintf(prn, _("\nHarvey-Collier t(%d) = %g with p-value %.4g\n\n"), 
 	    m - 1, hct, pval);
 
@@ -2292,7 +2419,8 @@ static void cusum_harvey_collier (double wbar, double sigma, int m,
  * @prn: gretl printing struct.
  *
  * Tests the given model for parameter stability via the CUSUM test,
- * or if @opt includes %OPT_R, via the CUSUMSQ test.
+ * or if @opt includes %OPT_R, via the CUSUMSQ test; %OPT_Q makes
+ * the test quiet.
  * 
  * Returns: 0 on successful completion, error code on error.
  */
@@ -2308,12 +2436,17 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     char cumdate[OBSLEN];
     double wbar = 0.0;
     double *cresid = NULL, *W = NULL;
+    int quiet = opt & OPT_Q;
     int err = 0;
 
     if (pmod->ci != OLS) {
 	return E_OLSONLY;
     }
 
+    if (exact_fit_check(pmod, prn)) {
+	return 0;
+    }
+
     if (has_missing_obs(pmod)) {
 	return E_DATA;
     }
@@ -2363,8 +2496,9 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	    a = a1 + a2 + a3;
 	    /* slope of expectation wrt time */
 	    b = 1.0 / m;
-
-	    pputs(prn, _("Cumulated sum of squared residuals"));
+	    if (!quiet) {
+		pputs(prn, _("Cumulated sum of squared residuals"));
+	    }
 	} else {
 	    wbar /= T - k;
 	    pprintf(prn, "\n%s\n\n", _("CUSUM test for stability of parameters"));
@@ -2381,8 +2515,9 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	    a = 0.948 * sqrt((double) m);
 	    /* slope of confidence band limit wrt time */
 	    b = 2.0 * a / m;
-
-	    pputs(prn, _("Cumulated sum of scaled residuals"));
+	    if (!quiet) {
+		pputs(prn, _("Cumulated sum of scaled residuals"));
+	    }
 	}
 
 	pputc(prn, '\n');
@@ -2404,8 +2539,10 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 		W[j] /= sigma;
 		sig = fabs(W[j]) > a + j * b;
 	    }
-	    ntodate(cumdate, pmod->t1 + k + j, pdinfo);
-	    pprintf(prn, " %s %9.3f %s\n", cumdate, W[j], sig? "*" : "");
+	    if (!quiet) {
+		ntodate(cumdate, pmod->t1 + k + j, pdinfo);
+		pprintf(prn, " %s %9.3f %s\n", cumdate, W[j], sig? "*" : "");
+	    }
 	}
 
 	if (!(opt & OPT_R)) {
@@ -2677,7 +2814,7 @@ int lmtest_driver (const char *param,
     }
 
     err = incompatible_options(opt, OPT_A | OPT_H | OPT_L | OPT_S |
-			       OPT_P | OPT_W);
+			       OPT_P | OPT_W | OPT_X);
     if (err) {
 	return err;
     }
@@ -2687,8 +2824,15 @@ int lmtest_driver (const char *param,
 	return E_DATA;
     }
 
+    if (type == GRETL_OBJ_EQN && exact_fit_check(ptr, prn)) {
+	return 0;
+    }
+
     if (opt & (OPT_A | OPT_H)) {
 	k = atoi(param);
+	if (k == 0) {
+	    k = pdinfo->pd;
+	}
     }
 
     testopt = (opt & OPT_Q)? OPT_Q : OPT_NONE;
@@ -2713,9 +2857,18 @@ int lmtest_driver (const char *param,
 	}
     }
 
-    /* heteroskedasticity (White) */
-    if (!err && (opt & OPT_W)) {
+    /* heteroskedasticity (White or Breusch-Pagan) */
+    if (!err && (opt & (OPT_W | OPT_X | OPT_B))) {
 	if (type == GRETL_OBJ_EQN) {
+	    if (opt & OPT_B) {
+		testopt |= OPT_B;
+		if (opt & OPT_R) {
+		    testopt |= OPT_R;
+		}
+	    }
+	    if (opt & OPT_X) {
+		testopt |= OPT_X;
+	    }
 	    err = whites_test(ptr, pZ, pdinfo, testopt, prn);
 	} else {
 	    err = E_NOTIMP;
@@ -2728,6 +2881,8 @@ int lmtest_driver (const char *param,
 	    err = autocorr_test(ptr, k, pZ, pdinfo, testopt, prn);
 	} else if (type == GRETL_OBJ_VAR) {
 	    err = gretl_VAR_autocorrelation_test(ptr, k, pZ, pdinfo, prn);
+	} else if (type == GRETL_OBJ_SYS) {
+	    err = system_autocorrelation_test(ptr, k, prn);
 	} else {
 	    err = E_NOTIMP;
 	}
@@ -2739,6 +2894,8 @@ int lmtest_driver (const char *param,
 	    err = arch_test(ptr, k, pdinfo, testopt, prn);
 	} else if (type == GRETL_OBJ_VAR) {
 	    err = gretl_VAR_arch_test(ptr, k, pdinfo, prn);
+	} else if (type == GRETL_OBJ_SYS) {
+	    err = system_arch_test(ptr, k, prn);
 	} else {
 	    err = E_NOTIMP;
 	}
