@@ -54,6 +54,7 @@ typedef enum {
     AUX_JOHANSEN,  /* Johansen cointegration test */
     AUX_GROUPWISE, /* testing for groupwise heteroskedasticity */
     AUX_HET_1, /* aux. regression for Pesaran-Taylor HET_1 test */
+    AUX_BP,    /* aux. regression for Breusch-Pagan heterosked. test */
     AUX_AUX    /* auxiliary regression not otherwise specified */
 } ModelAuxCode;
 
@@ -79,6 +80,8 @@ int autocorr_test (MODEL *pmod, int order,
 		   double ***pZ, DATAINFO *pdinfo, 
 		   gretlopt opt, PRN *prn);
 
+double ljung_box (int m, int t1, int t2, const double *y, int *err);
+
 int chow_test (const char *line, MODEL *pmod, 
 	       double ***pZ, DATAINFO *pdinfo, 
 	       gretlopt opt, PRN *prn);
