@@ -715,77 +715,3 @@ int rhodiff (char *param, const int *list, double ***pZ, DATAINFO *pdinfo)
 
     return 0;
 }
-
-int gretl_multiply (char *s, int *list, char *sfx, double ***pZ,
-		    DATAINFO *pdinfo)
-{
-    int i, t, v = 0, nv, n = pdinfo->n, lv, l0 = list[0];
-    int slen;
-    double m = 0;
-    char tmp[VNAMELEN];
-    
-    if (s == NULL || sfx == NULL) {
-	return E_ALLOC;
-    }
-
-    /* parse s */
-    if (isdigit((unsigned char) *s)) {
-	m = dot_atof(s);
-    } else {
-	v = varindex(pdinfo, s);
-	if (v == pdinfo->v) {
-	    return E_UNKVAR; 
-	}
-    }
-
-    if (dataset_add_series(l0, pZ, pdinfo)) {
-	return E_ALLOC;
-    }
-
-    slen = strlen(sfx);
-
-    /* fill out values */
-    for (i=1; i<=l0; i++) {
-	nv = pdinfo->v - l0 - 1 + i;
-	lv = list[i];
-
-	for (t=0; t<n; t++) {
-	    (*pZ)[nv][t] = NADBL;
-	}
-
-	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	    if (na((*pZ)[lv][t])) {
-		(*pZ)[nv][t] = NADBL;
-		continue;
-	    }
-	    if (v) {
-		double yy = (var_is_series(pdinfo, v))? 
-		    (*pZ)[v][t] : (*pZ)[v][0];
-
-		if (na(yy)) {
-		    (*pZ)[nv][t] = NADBL;
-		} else {
-		    (*pZ)[nv][t] = yy * (*pZ)[lv][t];
-		}
-	    } else {
-		(*pZ)[nv][t] = m * (*pZ)[lv][t];
-	    }
-	}
-
-	/* do names and labels */
-	strcpy(tmp, pdinfo->varname[lv]);
-	gretl_trunc(tmp, 8 - slen);
-	strcat(tmp, sfx);
-	strcpy(pdinfo->varname[nv], tmp);
-	if (v) {
-	    sprintf(VARLABEL(pdinfo, nv), "%s = %s * %s",
-		    pdinfo->varname[nv], pdinfo->varname[v], 
-		    pdinfo->varname[lv]); 
-	} else {
-	    sprintf(VARLABEL(pdinfo, nv), "%s = %g * %s",
-		    pdinfo->varname[nv], m, pdinfo->varname[lv]); 
-	}
-    }
-
-    return 0;
-}
