@@ -84,6 +84,10 @@ time_series_label_check (DATAINFO *pdinfo, char *skipstr, PRN *prn);
 
 static void csvdata_free (csvdata *c)
 {
+    if (c == NULL) {
+	return;
+    }
+
     if (c->descrip != NULL) {
 	free(c->descrip);
     }
@@ -263,7 +267,7 @@ static int csv_weekly_data (double ***pZ, DATAINFO *pdinfo)
 	    fprintf(stderr, "check: Tc = %d, missing = %d\n", Tc, altmiss);
 	    if (altmiss != misscount) {
 		ret = 0;
-	    } else {
+	    } else if (pZ != NULL) {
 		int err;
 
 		fprintf(stderr, "OK, consistent\n");
@@ -548,10 +552,6 @@ csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, char *skipstr,
     char *lbl1 = pdinfo->S[0];
     char *lbl2 = pdinfo->S[pdinfo->n - 1];
 
-    if (pZ == NULL) {
-	return E_DATA;
-    }
-
     if (sscanf(lbl1, "%d%1[/-]%d%1[/-]%d", &d1[0], sep1, &d1[1], sep2, &d1[2]) == 5 &&
 	sscanf(lbl2, "%d%1[/-]%d%1[/-]%d", &d2[0], sep1, &d2[1], sep2, &d2[2]) == 5) {
 	int yr1, mon1, day1;
@@ -596,7 +596,7 @@ csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, char *skipstr,
 		    return -1;
 		}
 	    }
-	    pprintf(prn, "? %s - %s\n", lbl1, lbl2);
+	    pprintf(prn, "Could be %s - %s\n", lbl1, lbl2);
 	    ret = check_daily_dates(pdinfo, &pd, prn);
 	    if (ret >= 0 && pd > 0) {
 		if (pd == 52) {
@@ -722,9 +722,11 @@ time_series_label_check (DATAINFO *pdinfo, char *skipstr, PRN *prn)
     return pd;
 }
 
-/* attempt to parse csv row labels as dates.  Return -1 if this
+/* Attempt to parse csv row labels as dates.  Return -1 if this
    doesn't work out, or 0 if the labels seem to be just integer
-   observation numbers, else return the inferred data frequency 
+   observation numbers, else return the inferred data frequency.  The
+   pZ argument may be NULL; in that case we will not attempt to pad
+   weekly data.
 */
 
 int test_markers_for_dates (double ***pZ, DATAINFO *pdinfo, 
@@ -922,10 +924,24 @@ static void compress_csv_line (char *line, csvdata *c)
 
 int import_obs_label (const char *s)
 {
-    char tmp[32];
+    char tmp[OBSLEN];
+
+    if (s == NULL) {
+	return 1;
+    }
+
+    if (*s == '"' || *s == '\'') s++;
+
+    if (*s == '\0') {
+	return 1;
+    }
+
+    if (strlen(s) > OBSLEN - 1) {
+	return 0;
+    }
 
     *tmp = '\0';
-    strncat(tmp, s, 31);
+    strncat(tmp, s, OBSLEN - 1);
     lower(tmp);
 
     return (!strcmp(tmp, "obs") ||
@@ -1048,7 +1064,7 @@ static int non_numeric_check (csvdata *c, PRN *prn)
 		    }
 		}
 
-		nnfrac = (nok == 0)? 1.0 : (double) nnon / nok;
+		nnfrac = (nok == 0)? 1.0 : (double) nnon / (nnon + nok);
 		pprintf(prn, "variable %d (%s): non-numeric values = %d "
 			"(%.2f percent)\n", v, c->dinfo->varname[v], 
 			nnon, 100 * nnfrac);
@@ -1232,6 +1248,19 @@ static int csv_fields_check (char *line, int maxlen, FILE *fp,
     return err;
 }
 
+static void strip_illegals (char *s)
+{
+    int i;
+
+    if (*s == '\0') return;
+
+    for (i=1; s[i]!='\0'; i++) {
+	if (!isalnum(s[i])) {
+	    s[i] = '_';
+	}
+    }
+}
+
 static int 
 csv_reconfigure_for_markers (double ***pZ, DATAINFO *pdinfo)
 {
@@ -1304,7 +1333,9 @@ static int csv_varname_scan (csvdata *c, char *line, int maxlen, FILE *fp,
 		    numcount++;
 		} else {
 		    iso_to_ascii(c->dinfo->varname[nv]);
+		    strip_illegals(c->dinfo->varname[nv]);
 		    if (check_varname(c->dinfo->varname[nv])) {
+			fprintf(stderr, "bombing here\n");
 			errmsg(1, prn);
 			err = E_DATA;
 		    }
@@ -1450,12 +1481,13 @@ static void csv_parsing_header (const char *fname, PRN *prn)
 
 /**
  * import_csv:
- * @pZ: pointer to data set.
- * @ppdinfo: pointer to data information struct.
  * @fname: name of CSV file.
+ * @pZ: pointer to data set.
+ * @pdinfo: pointer to data information struct.
  * @opt: use %OPT_C to force interpretation of data colums containing
- * strings as coded values and not errors.
- * @prn: gretl printing struct (can be NULL).
+ * strings as coded values and not errors; for use of %OPT_T see
+ * the help for "append".
+ * @prn: gretl printing struct (or %NULL).
  * 
  * Open a Comma-Separated Values data file and read the data into
  * the current work space.
@@ -1463,8 +1495,8 @@ static void csv_parsing_header (const char *fname, PRN *prn)
  * Returns: 0 on successful completion, non-zero otherwise.
  */
 
-int import_csv (double ***pZ, DATAINFO **ppdinfo, 
-		const char *fname, gretlopt opt, PRN *prn)
+int import_csv (const char *fname, double ***pZ, DATAINFO *pdinfo, 
+		gretlopt opt, PRN *prn)
 {
     csvdata *c = NULL;
     int popit = 0;
@@ -1493,7 +1525,7 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
 	goto csv_bailout;
     }
 
-    c = csvdata_new(*pZ, *ppdinfo, opt);
+    c = csvdata_new(*pZ, pdinfo, opt);
     if (c == NULL) {
 	err = E_ALLOC;
 	goto csv_bailout;
@@ -1536,7 +1568,7 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
     rewind(fp);
 
     /* read lines, check for consistency in number of fields */
-    err = csv_fields_check(line, maxlen, fp, c, prn);
+    err = csv_fields_check(line, maxlen, fp, c, mprn);
     if (err) {
 	goto csv_bailout;
     }
@@ -1575,7 +1607,7 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
 	goto csv_bailout;
     }
 
-    if (*ppdinfo != NULL && (*ppdinfo)->decpoint != ',') {
+    if (pdinfo != NULL && pdinfo->decpoint != ',') {
 	gretl_push_c_numeric_locale();
 	popit = 1;
     }
@@ -1649,10 +1681,10 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
 	pputs(prn, M_("warning: some variable names were duplicated\n"));
     }
 
-    err = merge_or_replace_data(pZ, ppdinfo, &c->Z, &c->dinfo, prn);
+    err = merge_or_replace_data(pZ, pdinfo, &c->Z, &c->dinfo, opt, prn);
 
     if (!err && newdata && c->descrip != NULL) {
-	(*ppdinfo)->descrip = c->descrip;
+	pdinfo->descrip = c->descrip;
 	c->descrip = NULL;
     }
 
