@@ -24,6 +24,7 @@
 #include "gretl_xml.h"
 #include "gretl_panel.h"
 #include "csvdata.h"
+#include "usermat.h"
 
 #include <ctype.h>
 #include <time.h>
@@ -31,6 +32,19 @@
 
 #include <glib.h>
 
+typedef enum {
+    GRETL_FMT_FLOAT = 1, /* single-precision binary data */
+    GRETL_FMT_DOUBLE,    /* double-precision binary data */
+    GRETL_FMT_OCTAVE,    /* data in Gnu Octave format */
+    GRETL_FMT_CSV,       /* data in Comma Separated Values format */
+    GRETL_FMT_R,         /* data in Gnu R format */
+    GRETL_FMT_GZIPPED,   /* gzipped data */
+    GRETL_FMT_TRAD,      /* traditional (ESL-style) data */
+    GRETL_FMT_DAT,       /* data in PcGive format */
+    GRETL_FMT_DB,        /* gretl native database format */
+    GRETL_FMT_JM         /* JMulti ascii data */
+} GretlDataFormat;
+
 #define IS_DATE_SEP(c) (c == '.' || c == ':' || c == ',')
 
 static int writelbl (const char *lblfile, const int *list, 
@@ -858,11 +872,11 @@ real_ntodate (char *datestr, int t, const DATAINFO *pdinfo, int full)
 /**
  * ntodate:
  * @datestr: string to which date is to be printed.
- * @nt: an observation number (zero-based).
+ * @t: an observation number (zero-based).
  * @pdinfo: data information struct.
  * 
  * print to @datestr the calendar representation of observation
- * number nt.
+ * number @t.
  * 
  * Returns: the observation string.
  */
@@ -1016,9 +1030,9 @@ static int writehdr (const char *hdrfile, const int *list,
     char startdate[OBSLEN], enddate[OBSLEN];
     int i, binary = 0;
 
-    if (opt == GRETL_DATA_FLOAT) {
+    if (opt == GRETL_FMT_FLOAT) {
 	binary = 1;
-    } else if (opt == GRETL_DATA_DOUBLE) {
+    } else if (opt == GRETL_FMT_DOUBLE) {
 	binary = 2;
     }
 
@@ -1131,6 +1145,23 @@ int get_precision (const double *x, int n, int placemax)
     return pmax;
 }
 
+gretlopt data_save_opt_from_suffix (const char *fname)
+{
+    gretlopt opt = OPT_NONE;
+
+    if (has_suffix(fname, ".R")) {
+	opt = OPT_R;
+    } else if (has_suffix(fname, ".m")) {
+	opt = OPT_M;
+    } else if (has_suffix(fname, ".csv") ||
+	       has_suffix(fname, ".txt") ||
+	       has_suffix(fname, ".asc")) {
+	opt = OPT_C;
+    } 
+
+    return opt;
+}
+
 static GretlDataFormat 
 format_from_opt_or_name (gretlopt opt, const char *fname,
 			 char *delim)
@@ -1138,33 +1169,33 @@ format_from_opt_or_name (gretlopt opt, const char *fname,
     GretlDataFormat fmt = 0;
     
     if (opt & OPT_T) {
-	fmt = GRETL_DATA_TRAD;
+	fmt = GRETL_FMT_TRAD;
     } else if (opt & OPT_M) {
-	fmt = GRETL_DATA_OCTAVE;
+	fmt = GRETL_FMT_OCTAVE;
     } else if (opt & OPT_R) {
-	fmt = GRETL_DATA_R;
+	fmt = GRETL_FMT_R;
     } else if (opt & OPT_C) {
-	fmt = GRETL_DATA_CSV;
+	fmt = GRETL_FMT_CSV;
     } else if (opt & OPT_Z) {
-	fmt = GRETL_DATA_GZIPPED;
+	fmt = GRETL_FMT_GZIPPED;
     } else if (opt & OPT_D) {
-	fmt = GRETL_DATA_DB;
+	fmt = GRETL_FMT_DB;
     } else if (opt & OPT_G) {
-	fmt = GRETL_DATA_DAT;
+	fmt = GRETL_FMT_DAT;
     } else if (opt & OPT_J) {
-	fmt = GRETL_DATA_JM;
+	fmt = GRETL_FMT_JM;
     }
 
     if (fmt == 0) {
 	if (has_suffix(fname, ".R")) {
-	    fmt = GRETL_DATA_R;
+	    fmt = GRETL_FMT_R;
 	} else if (has_suffix(fname, ".csv")) {
-	    fmt = GRETL_DATA_CSV;
+	    fmt = GRETL_FMT_CSV;
 	} else if (has_suffix(fname, ".m")) {
-	    fmt = GRETL_DATA_OCTAVE;
+	    fmt = GRETL_FMT_OCTAVE;
 	} else if (has_suffix(fname, ".txt") ||
 		   has_suffix(fname, ".asc")) {
-	    fmt = GRETL_DATA_CSV;
+	    fmt = GRETL_FMT_CSV;
 	    *delim = ' ';
 	} 
     }
@@ -1193,7 +1224,7 @@ static void date_maj_min (int t, const DATAINFO *pdinfo, int *maj, int *min)
 /**
  * write_data:
  * @fname: name of file to write.
- * @list: list of variables to write.
+ * @list: list of variables to write (or %NULL to write all series).
  * @Z: data matrix.
  * @pdinfo: data information struct.
  * @opt: option flag indicating format in which to write the data.
@@ -1206,7 +1237,7 @@ static void date_maj_min (int t, const DATAINFO *pdinfo, int *maj, int *min)
  * Returns: 0 on successful completion, non-zero on error.
  */
 
-int write_data (const char *fname, const int *list, 
+int write_data (const char *fname, int *list, 
 		const double **Z, const DATAINFO *pdinfo, 
 		gretlopt opt, PATHS *ppaths)
 {
@@ -1218,45 +1249,59 @@ int write_data (const char *fname, const int *list,
     char delim = 0;
     FILE *fp = NULL;
     int *pmax = NULL;
+    int freelist = 0;
     double xx;
+    int err = 0;
 
     gretl_error_clear();
 
-    if (list == NULL || list[0] == 0) {
-	return 1;
+    if (list != NULL && list[0] == 0) {
+	return E_ARGS;
+    }
+
+    if (list == NULL) {
+	list = full_var_list(pdinfo, &l0);
+	if (l0 == 0) {
+	    return E_ARGS;
+	} else if (list == NULL) {
+	    return E_ALLOC;
+	} else {
+	    freelist = 1;
+	}
     }
 
     l0 = list[0];
 
     fmt = format_from_opt_or_name(opt, fname, &delim);
 
-    gretl_maybe_switch_dir(fname);
+    fname = gretl_maybe_switch_dir(fname);
 
-    if (fmt == 0 || fmt == GRETL_DATA_GZIPPED) {
-	return gretl_write_gdt(fname, list, Z, pdinfo, fmt, ppaths);
+    if (fmt == 0 || fmt == GRETL_FMT_GZIPPED) {
+	err = gretl_write_gdt(fname, list, Z, pdinfo, 
+			      (fmt == GRETL_FMT_GZIPPED)? OPT_Z : OPT_NONE,
+			      ppaths);
+	goto write_exit;
     }
 
-    if (fmt == GRETL_DATA_DB) {
-	return write_db_data(fname, list, opt, Z, pdinfo);
+    if (fmt == GRETL_FMT_DB) {
+	err = write_db_data(fname, list, opt, Z, pdinfo);
+	goto write_exit;
     }
 
-    if (fmt == GRETL_DATA_CSV && get_csv_delim(pdinfo) == ',' && 
+    if (fmt == GRETL_FMT_CSV && get_csv_delim(pdinfo) == ',' && 
 	',' == pdinfo->decpoint) {
 	sprintf(gretl_errmsg, _("You can't use the same character for "
 				"the column delimiter and the decimal point"));
-	return 1;
+	err = E_DATA;
+	goto write_exit;
     }
 
     strcpy(datfile, fname);
 
-    if (fmt == GRETL_DATA_R && dataset_is_time_series(pdinfo)) {
-	fmt = GRETL_DATA_R_TS;
-    }
-
     /* write header and label files if not exporting to other formats */
-    if (fmt != GRETL_DATA_R && fmt != GRETL_DATA_R_TS && 
-	fmt != GRETL_DATA_CSV && fmt != GRETL_DATA_OCTAVE &&
-	fmt != GRETL_DATA_DAT && fmt != GRETL_DATA_JM) {
+    if (fmt != GRETL_FMT_R && fmt != GRETL_FMT_CSV && 
+	fmt != GRETL_FMT_OCTAVE && fmt != GRETL_FMT_DAT && 
+	fmt != GRETL_FMT_JM) {
 	if (!has_suffix(datfile, ".gz")) {
 	    switch_ext(hdrfile, datfile, "hdr");
 	    switch_ext(lblfile, datfile, "lbl");
@@ -1266,28 +1311,32 @@ int write_data (const char *fname, const int *list,
 	}
 	if (writehdr(hdrfile, list, pdinfo, fmt)) {
 	    fprintf(stderr, I_("Write of header file failed"));
-	    return 1;
+	    err = E_FOPEN;
+	    goto write_exit;
 	}
 	if (writelbl(lblfile, list, pdinfo)) {
 	    fprintf(stderr, I_("Write of labels file failed"));
-	    return 1;
+	    err = E_FOPEN;
+	    goto write_exit;
 	}
     }
 
     /* open file for output */
     fp = gretl_fopen(datfile, "w");
     if (fp == NULL) {
-	return E_FOPEN;
+	err = E_FOPEN;
+	goto write_exit;
     }
 
-    if (fmt == GRETL_DATA_CSV || fmt == GRETL_DATA_OCTAVE || 
-	GRETL_DATA_R || fmt == GRETL_DATA_TRAD || 
-	fmt == GRETL_DATA_DAT || fmt == GRETL_DATA_JM) { 
+    if (fmt == GRETL_FMT_CSV || fmt == GRETL_FMT_OCTAVE || 
+	GRETL_FMT_R || fmt == GRETL_FMT_TRAD || 
+	fmt == GRETL_FMT_DAT || fmt == GRETL_FMT_JM) { 
 	/* an ASCII variant of some sort */
 	pmax = malloc(l0 * sizeof *pmax);
 	if (pmax == NULL) {
 	    fclose(fp);
-	    return 1;
+	    err = E_ALLOC;
+	    goto write_exit;
 	}
 	for (i=1; i<=l0; i++) {
 	    v = list[i];
@@ -1299,11 +1348,11 @@ int write_data (const char *fname, const int *list,
 	}	
     }
 
-    if (fmt != GRETL_DATA_CSV || pdinfo->decpoint != ',') {
+    if (fmt != GRETL_FMT_CSV || pdinfo->decpoint != ',') {
 	gretl_push_c_numeric_locale();
     }
 
-    if (fmt == GRETL_DATA_TRAD) { 
+    if (fmt == GRETL_FMT_TRAD) { 
 	/* plain ASCII */
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	    if (pdinfo->markers && pdinfo->S != NULL) {
@@ -1325,11 +1374,11 @@ int write_data (const char *fname, const int *list,
 	    }
 	    fputc('\n', fp);
 	}
-    } else if (fmt == GRETL_DATA_CSV || fmt == GRETL_DATA_R) { 
+    } else if (fmt == GRETL_FMT_CSV || fmt == GRETL_FMT_R) { 
 	/* export CSV or GNU R (dataframe) */
 	int print_obs = 0;
 
-	if (fmt == GRETL_DATA_CSV) {
+	if (fmt == GRETL_FMT_CSV) {
 	    if ((pdinfo->structure == TIME_SERIES || pdinfo->S != NULL)
 		&& !(opt & OPT_X)) {
 		print_obs = 1;
@@ -1342,8 +1391,16 @@ int write_data (const char *fname, const int *list,
 	    delim = ' ';
 	}
 
+	if (fmt == GRETL_FMT_R && dataset_is_time_series(pdinfo)) {
+	    char datestr[OBSLEN];
+
+	    ntodate_full(datestr, pdinfo->t1, pdinfo);
+	    fprintf(fp, "# time-series data: start = %s, frequency = %d\n",
+		    datestr, pdinfo->pd);
+	}
+
 	/* variable names */
-	if (fmt == GRETL_DATA_CSV && print_obs && 
+	if (fmt == GRETL_FMT_CSV && print_obs && 
 	    (pdinfo->S != NULL || pdinfo->structure != CROSS_SECTION)) {
 	    fprintf(fp, "obs%c", delim);
 	}
@@ -1383,51 +1440,7 @@ int write_data (const char *fname, const int *list,
 		}
 	    }
 	}
-    } else if (fmt == GRETL_DATA_R_TS) {
-	char *p, datestr[OBSLEN];
-	int subper = 1;
-
-	fprintf(fp, "\"%s\" <- ts (t (matrix (data = c(\n", "gretldata");
-
-	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	    for (i=1; i<=l0; i++) {
-		v = list[i];
-		xx = (var_is_series(pdinfo, v))? Z[v][t] : Z[v][0];
-		if (na(xx)) {
-		    fputs("NA", fp);
-		} else {
-		    fprintf(fp, "%g", xx);
-		}
-		if (i == l0) {
-		    if (t == pdinfo->t2) {
-			fputs("),\n", fp);
-		    } else {
-			fputs(" ,\n", fp);
-		    }
-		} else {
-		    fputs(" , ", fp);
-		}
-	    }
-	}
-
-	ntodate_full(datestr, pdinfo->t1, pdinfo);
-	p = strchr(datestr, ':');
-	if (p != NULL) {
-	    subper = atoi(p + 1);
-	}
-	fprintf(fp, "nrow = %d, ncol = %d)), start = c(%d,%d), frequency = %d)\n",
-		l0, pdinfo->t2 - pdinfo->t1 + 1, 
-		atoi(datestr), subper, pdinfo->pd);
-	fprintf(fp, "colnames(%s) <- c(", "gretldata");
-	for (i=1; i<=l0; i++) {
-	    fprintf(fp, "\"%s\"", pdinfo->varname[list[i]]);
-	    if (i < l0) {
-		fputs(", ", fp);
-	    } else {
-		fputs(")\n", fp);
-	    }
-	}
-    } else if (fmt == GRETL_DATA_OCTAVE) { 
+    } else if (fmt == GRETL_FMT_OCTAVE) { 
 	/* GNU Octave: write out data as a matrix */
 	fprintf(fp, "# name: X\n# type: matrix\n# rows: %d\n# columns: %d\n", 
 		n, list[0]);
@@ -1444,7 +1457,7 @@ int write_data (const char *fname, const int *list,
 	    }
 	    fputc('\n', fp);
 	}
-    } else if (fmt == GRETL_DATA_DAT) { 
+    } else if (fmt == GRETL_FMT_DAT) { 
 	/* PcGive: data file with load info */
 	int pd = pdinfo->pd;
 
@@ -1478,7 +1491,7 @@ int write_data (const char *fname, const int *list,
 	    }
 	    fputc('\n', fp);
 	}
-    } else if (fmt == GRETL_DATA_JM) { 
+    } else if (fmt == GRETL_FMT_JM) { 
 	/* JMulti: ascii with comments and date info */
 	int maj, min;
 
@@ -1516,7 +1529,7 @@ int write_data (const char *fname, const int *list,
 	}
     }
 
-    if (fmt != GRETL_DATA_CSV || pdinfo->decpoint != ','){
+    if (fmt != GRETL_FMT_CSV || pdinfo->decpoint != ',') {
 	gretl_pop_c_numeric_locale();
     }
 
@@ -1528,7 +1541,13 @@ int write_data (const char *fname, const int *list,
 	fclose(fp);
     }
 
-    return 0;
+ write_exit:
+
+    if (freelist) {
+	free(list);
+    }
+
+    return err;
 }
 
 static void dataset_type_string (char *str, const DATAINFO *pdinfo)
@@ -1628,38 +1647,35 @@ int data_report (const DATAINFO *pdinfo, PATHS *ppaths, PRN *prn)
 static int readlbl (const char *lblfile, DATAINFO *pdinfo)
 {
     FILE * fp;
-    char line[MAXLEN], *label, varname[VNAMELEN];
+    char line[MAXLEN], varname[VNAMELEN];
+    char *p;
     int v;
     
     gretl_error_clear();
 
     fp = gretl_fopen(lblfile, "r");
-    if (fp == NULL) return 0;
+    if (fp == NULL) {
+	return 0;
+    }
 
-    while (1) {
-        if (fgets(line, MAXLEN-1, fp) == NULL) {
-            fclose(fp);
-            return 0;
-        }
+    while (fgets(line, MAXLEN, fp)) {
+	tailstrip(line);
         if (sscanf(line, "%s", varname) != 1) {
-            fclose(fp);
 	    sprintf(gretl_errmsg, _("Bad data label in %s"), lblfile); 
-            return 0;
-        }
-        label = line + strlen(varname);
-        if (top_n_tail(label) == E_ALLOC) {
-            fclose(fp);
-            return E_ALLOC;
+            break;
         }
 	v = varindex(pdinfo, varname);
 	if (v < pdinfo->v) {
-	    strcpy(VARLABEL(pdinfo, v), label);
+	    p = line + strlen(varname);
+	    p += strspn(p, " \t");
+	    VARLABEL(pdinfo, v)[0] = '\0';
+	    strncat(VARLABEL(pdinfo, v), p, MAXLABEL - 1);
 	} else {
 	    fprintf(stderr, I_("extraneous label for var '%s'\n"), varname);
 	}
     }
 
-    if (fp != NULL) fclose(fp);
+    fclose(fp);
 
     return 0;
 }
@@ -1779,11 +1795,11 @@ static void try_gdt (char *fname)
 
 /**
  * gretl_get_data:
- * @pZ: pointer to data set.
- * @ppdinfo: pointer to data information struct.
  * @datfile: name of file to try.
  * @ppaths: path information struct.
- * @ocode: %DATA_NONE, %DATA_CLEAR or %DATA_APPEND.
+ * @pZ: pointer to data set.
+ * @pdinfo: pointer to data information struct.
+ * @opt: for use with "append".
  * @prn: where messages should be written.
  * 
  * Read data from file into gretl's work space, allocating space as
@@ -1792,8 +1808,9 @@ static void try_gdt (char *fname)
  * Returns: 0 on successful completion, non-zero otherwise.
  */
 
-int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, char *datfile, PATHS *ppaths, 
-		    PRN *prn) 
+int gretl_get_data (char *datfile, PATHS *ppaths,
+		    double ***pZ, DATAINFO *pdinfo, 
+		    gretlopt opt, PRN *prn) 
 {
     DATAINFO *tmpdinfo = NULL;
     double **tmpZ = NULL;
@@ -1848,8 +1865,7 @@ int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, char *datfile, PATHS *ppat
 
     /* catch XML files that have strayed in here? */
     if (gdtsuff && gretl_is_xml_file(datfile)) {
-	return gretl_read_gdt(pZ, ppdinfo, datfile, ppaths, 
-			      OPT_NONE, prn);
+	return gretl_read_gdt(datfile, ppaths, pZ, pdinfo, OPT_NONE, prn);
     }
 
     tmpdinfo = datainfo_new();
@@ -1869,7 +1885,7 @@ int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, char *datfile, PATHS *ppat
     err = readhdr(hdrfile, tmpdinfo, &binary, &old_byvar);
     if (err == E_FOPEN) {
 	/* no header file, so maybe it's just an ascii datafile */
-	return import_csv(pZ, ppdinfo, datfile, OPT_NONE, prn);
+	return import_csv(datfile, pZ, pdinfo, OPT_NONE, prn);
     } else if (err) {
 	return err;
     } else { 
@@ -1949,7 +1965,7 @@ int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, char *datfile, PATHS *ppat
     err = readlbl(lblfile, tmpdinfo);
     if (err) goto bailout;
 
-    err = merge_or_replace_data(pZ, ppdinfo, &tmpZ, &tmpdinfo, prn);
+    err = merge_or_replace_data(pZ, pdinfo, &tmpZ, &tmpdinfo, opt, prn);
 
     if (!err && newdata && ppaths != NULL && datfile != ppaths->datfile) {
 	strcpy(ppaths->datfile, datfile);
@@ -2043,49 +2059,63 @@ static int extend_markers (DATAINFO *pdinfo, int old_n, int new_n)
     return err;
 }
 
-static int count_add_vars (const DATAINFO *pdinfo, const DATAINFO *addinfo)
+static void merge_error (char *msg, PRN *prn)
 {
+    pputs(prn, msg);
+    strcpy(gretl_errmsg, msg);
+}
+
+static int count_new_vars (const DATAINFO *pdinfo, const DATAINFO *addinfo,
+			   PRN *prn)
+{
+    const char *newname;
+    /* default to all new, and subtract */
     int addvars = addinfo->v - 1;
     int i, j;
 
-    for (i=1; i<addinfo->v; i++) {
-	for (j=1; j<pdinfo->v; j++) {
-	    if (!strcmp(addinfo->varname[i], pdinfo->varname[j])) {
-		if (var_is_scalar(pdinfo, j)) {
-		    addvars = -1;
-		} else {
-		    addvars--;
+    for (i=1; i<addinfo->v && addvars >= 0; i++) {
+	newname = addinfo->varname[i];
+	if (get_matrix_by_name(newname)) {
+	    merge_error("can't replace matrix with series\n", prn);
+	    addvars = -1;
+	} else if (get_string_by_name(newname)) {
+	    merge_error("can't replace string with series\n", prn);
+	    addvars = -1;
+	} else {
+	    for (j=1; j<pdinfo->v; j++) {
+		if (!strcmp(newname, pdinfo->varname[j])) {
+		    if (var_is_scalar(pdinfo, j)) {
+			addvars = -1;
+			merge_error("can't replace scalar with series\n", prn);
+		    } else {
+			addvars--;
+		    }
+		    break;
 		}
-		break;
 	    }
 	}
-	if (addvars < 0) {
-	    fprintf(stderr, "%s: can't replace scalar with vector\n",
-		    addinfo->varname[i]);
-	    break;
-	}
     }
 
     return addvars;
 }
 
-static int compare_ranges (const DATAINFO *pdinfo,
-			   const DATAINFO *addinfo,
+static int compare_ranges (const DATAINFO *targ,
+			   const DATAINFO *src,
 			   int *offset)
 {
     int ed0, sd1, ed1;
     int addobs = -1;
 
-    ed0 = dateton(pdinfo->endobs, pdinfo);
-    sd1 = merge_dateton(addinfo->stobs, pdinfo);
-    ed1 = merge_dateton(addinfo->endobs, pdinfo);
+    ed0 = dateton(targ->endobs, targ);
+    sd1 = merge_dateton(src->stobs, targ);
+    ed1 = merge_dateton(src->endobs, targ);
 
-#if 1
+#if 0
     fprintf(stderr, "compare_ranges:\n"
-	    " pdinfo->n = %d, addinfo->n = %d\n"
-	    " pdinfo->stobs = '%s', addinfo->stobs = '%s'\n" 
+	    " targ->n = %d, src->n = %d\n"
+	    " targ->stobs = '%s', src->stobs = '%s'\n" 
 	    " sd1 = %d, ed1 = %d\n",
-	    pdinfo->n, addinfo->n, pdinfo->stobs, addinfo->stobs,
+	    targ->n, src->n, targ->stobs, src->stobs,
 	    sd1, ed1);
 #endif
 
@@ -2112,7 +2142,7 @@ static int compare_ranges (const DATAINFO *pdinfo,
     } else if (sd1 == ed0 + 1) {
 	/* case: new data start right after end of old */
 	*offset = sd1;
-	addobs = addinfo->n;
+	addobs = src->n;
     } else if (sd1 > 0) {
 	/* case: new data start later than old */
 	if (sd1 <= ed0) {
@@ -2133,18 +2163,135 @@ static int compare_ranges (const DATAINFO *pdinfo,
     return addobs;
 }
 
-static void merge_error (char *msg, PRN *prn)
+static int panel_expand_ok (DATAINFO *pdinfo, DATAINFO *addinfo,
+			    gretlopt opt)
 {
-    pputs(prn, msg);
-    strcpy(gretl_errmsg, msg);
+    int n = pdinfo->paninfo->nunits;
+    int T = pdinfo->paninfo->Tmax;
+    int ok = 0;
+
+    if (addinfo->n == T) {
+	ok = 1;
+    } else if (!(opt & OPT_T) &&
+	       addinfo->n == n && 
+	       addinfo->pd == 1) {
+	ok = 1;
+    }
+
+    return ok;
 }
 
+static int panel_append_special (int addvars, 
+				 double ***pZ, 
+				 DATAINFO *pdinfo, 
+				 double **addZ, 
+				 DATAINFO *addinfo,
+				 gretlopt opt,
+				 PRN *prn)
+{
+    int n = pdinfo->paninfo->nunits;
+    int T = pdinfo->paninfo->Tmax;
+    int k = pdinfo->v;
+    int tsdata;
+    int i, j, s, p, t;
+    int err = 0;
+
+    if (addvars > 0 && dataset_add_series(addvars, pZ, pdinfo)) {
+	merge_error(_("Out of memory adding data\n"), prn);
+	err = 1;
+    }
+
+    tsdata = ((opt & OPT_T) || addinfo->n != n);
+
+    for (i=1; i<addinfo->v && !err; i++) {
+	int v = varindex(pdinfo, addinfo->varname[i]);
+
+	if (v >= k) {
+	    /* a new variable */
+	    v = k++;
+	    strcpy(pdinfo->varname[v], addinfo->varname[i]);
+	    copy_varinfo(pdinfo->varinfo[v], addinfo->varinfo[i]);
+	} 
+
+	s = 0;
+	for (j=0; j<n; j++) {
+	    /* loop across units */
+	    for (t=0; t<T; t++) {
+		/* loop across periods */
+		p = (tsdata)? t : j;
+		(*pZ)[v][s++] = addZ[i][p]; 
+	    }
+	}
+    }
+
+    return err;
+}
+
+static int 
+just_append_rows (const DATAINFO *targ, const DATAINFO *src,
+		  int *offset)
+{
+    if (targ->structure == CROSS_SECTION &&
+	src->structure == CROSS_SECTION &&
+	targ->markers == 0 && src->markers == 0 &&
+	targ->sd0 == 1 && src->sd0 == 1) {
+	*offset = targ->n;
+	return src->n;
+    } else {
+	return 0;
+    }
+}
+
+static int simple_range_match (const DATAINFO *targ, const DATAINFO *src,
+			       int *offset)
+{
+    int ret = 0;
+
+    if (src->pd == 1 && src->structure == CROSS_SECTION) {
+	if (src->n == targ->n) {
+	    ret = 1;
+	} else if (src->n == targ->t2 - targ->t1 + 1) {
+	    ret = 1;
+	    *offset = targ->t1;
+	}
+    }
+
+    return ret;
+}
+
+#if 0
+static int markers_are_ints (const DATAINFO *pdinfo)
+{
+    char *test;
+    int i;
+
+    errno = 0;
+
+    for (i=0; i<pdinfo->n; i++) {
+	strtol(pdinfo->S[i], &test, 10);
+	if (*test || errno) {
+	    errno = 0;
+	    return 0;
+	}
+    }
+
+    return 1;
+}
+#endif
+
+#define simple_structure(p) (p->structure == TIME_SERIES ||		\
+			     p->structure == SPECIAL_TIME_SERIES ||	\
+			     (p->structure == CROSS_SECTION &&		\
+			      p->S == NULL))
+
 /**
  * merge_data:
  * @pZ: pointer to data set.
  * @pdinfo: data information struct.
  * @addZ: new data set to be merged in.
  * @addinfo: data information associated with @addZ.
+ * @opt: may include %OPT_T to force a time-series interpretation
+ * when appending to a panel dataset.
  * @prn: print struct to accept messages.
  * 
  * Attempt to merge the content of a newly opened data file into
@@ -2155,31 +2302,43 @@ static void merge_error (char *msg, PRN *prn)
 
 static int merge_data (double ***pZ, DATAINFO *pdinfo,
 		       double **addZ, DATAINFO *addinfo,
-		       PRN *prn)
+		       gretlopt opt, PRN *prn)
 {
     int addsimple = 0;
+    int addpanel = 0;
     int addvars = 0;
     int addobs = 0;
     int offset = 0;
     int err = 0;
 
-    /* first check for conformability */
+    /* first see how many new vars we have */
+    addvars = count_new_vars(pdinfo, addinfo, prn);
+    if (addvars < 0) {
+	return 1;
+    }
 
-    if (addinfo->pd == 1 && addinfo->structure == CROSS_SECTION &&
-	addinfo->n == pdinfo->n) {
-	/* we'll allow undated data to be merged with dated, provided
-	   the number of observations matches OK */
+    if (simple_structure(pdinfo) && simple_range_match(pdinfo, addinfo, &offset)) {
+	/* we'll allow undated data to be merged with x-sectional or
+	   dated, sideways, provided the number of observations
+	   matches OK */
 	addsimple = 1;
+    } else if (dataset_is_panel(pdinfo) && 
+	       panel_expand_ok(pdinfo, addinfo, opt)) {
+	/* allow appending to panel when the number of obs matches
+	   either the cross-section size or the time-series length */
+	addpanel = 1;
     } else if (pdinfo->pd != addinfo->pd) {
 	merge_error(_("Data frequency does not match\n"), prn);
 	err = 1;
     }
 
     if (!err) {
-	if (!addsimple) {
+	if (!addsimple && !addpanel) {
 	    addobs = compare_ranges(pdinfo, addinfo, &offset);
 	}
-	addvars = count_add_vars(pdinfo, addinfo);
+	if (addobs <= 0 && addvars == 0) {
+	    addobs = just_append_rows(pdinfo, addinfo, &offset);
+	}
     }
 
     if (!err && (addobs < 0 || addvars < 0)) {
@@ -2187,7 +2346,7 @@ static int merge_data (double ***pZ, DATAINFO *pdinfo,
 	err = 1;
     }
 
-    if (!err && pdinfo->markers != addinfo->markers) {
+    if (!err && !addpanel && pdinfo->markers != addinfo->markers) {
 	if (addinfo->n != pdinfo->n) {
 	    merge_error(_("Inconsistency in observation markers\n"), prn);
 	    err = 1;
@@ -2195,7 +2354,12 @@ static int merge_data (double ***pZ, DATAINFO *pdinfo,
 	    dataset_destroy_obs_markers(addinfo);
 	}
     }
-	
+
+#if 0
+    fprintf(stderr, "merge_data: addvars = %d, addobs = %d\n",
+	    addvars, addobs);
+#endif
+
     /* if checks are passed, try merging the data */
 
     if (!err && addobs > 0) { 
@@ -2242,7 +2406,10 @@ static int merge_data (double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    if (!err) { 
+    if (!err && addpanel) {
+	err = panel_append_special(addvars, pZ, pdinfo, addZ, addinfo, 
+				   opt, prn);
+    } else if (!err) { 
 	int k = pdinfo->v;
 	int i, t;
 
@@ -2283,9 +2450,11 @@ static int merge_data (double ***pZ, DATAINFO *pdinfo,
 /**
  * merge_or_replace_data:
  * @pZ0: pointer to original data set.
- * @ppdinfo0: pointer to original data information struct.
+ * @pdinfo0: original dataset information struct.
  * @pZ1: new data set.
- * @ppdinfo1: pointer data information associated with @pZ1.
+ * @ppdinfo1: pointer to dataset information associated with @pZ1.
+ * @opt: may include %OPT_T when appending to a panel dataset,
+ * to force a time-series interpretation of the added data.
  * @prn: print struct to accept messages.
  *
  * Given a newly-created dataset, pointed to by @pZ1 and
@@ -2298,19 +2467,19 @@ static int merge_data (double ***pZ, DATAINFO *pdinfo,
  * Returns: 0 on successful completion, non-zero otherwise.
  */
 
-int merge_or_replace_data (double ***pZ0, DATAINFO **ppdinfo0,
+int merge_or_replace_data (double ***pZ0, DATAINFO *pdinfo0,
 			   double ***pZ1, DATAINFO **ppdinfo1,
-			   PRN *prn)
+			   gretlopt opt, PRN *prn)
 {
     int err = 0;
 
     if (*pZ0 != NULL) {
-	err = merge_data(pZ0, *ppdinfo0, *pZ1, *ppdinfo1, prn);
+	err = merge_data(pZ0, pdinfo0, *pZ1, *ppdinfo1, opt, prn);
 	destroy_dataset(*pZ1, *ppdinfo1);
     } else {
-	free(*ppdinfo0);
+	*pdinfo0 = **ppdinfo1;
+	free(*ppdinfo1);
 	*pZ0 = *pZ1;
-	*ppdinfo0 = *ppdinfo1;
     }
 
     *pZ1 = NULL;
@@ -2475,57 +2644,31 @@ static int get_max_line_length (FILE *fp, PRN *prn)
     return maxlen;
 }
 
-/**
- * import_octave:
- * @pZ: pointer to data set.
- * @ppdinfo: pointer to data information struct.
- * @fname: name of GNU octave ascii data file.
- * @prn: gretl printing struct (can be NULL).
- * 
- * Open a GNU octave ascii data file (matrix type) and read the data into
- * the current work space.
- * 
- * Returns: 0 on successful completion, non-zero otherwise.
- *
- */
-
-int import_octave (double ***pZ, DATAINFO **ppdinfo, 
-		   const char *fname, PRN *prn)
+static int 
+import_octave (const char *fname, double ***pZ, DATAINFO *pdinfo, 
+	       gretlopt opt, PRN *prn)
 {
     DATAINFO *octinfo = NULL;
     double **octZ = NULL;
-    
     FILE *fp = NULL;
     char *line = NULL;
-
     char tmp[8], name[32];
     int nrows = 0, ncols = 0, nblocks = 0;
     int brows = 0, bcols = 0, oldbcols = 0;
     int maxlen, got_type = 0, got_name = 0;
-    int err = 0;
-
-    int i, t;
-
-#ifdef ENABLE_NLS
-    if (prn != NULL) {
-	check_for_console(prn);
-    }
-#endif
-
-    fp = gretl_fopen(fname, "r");
-    if (fp == NULL) {
-	goto oct_bailout;
-    }   
+    int i, t, err = 0;
 
     pprintf(prn, "%s %s...\n", M_("parsing"), fname);
 
     maxlen = get_max_line_length(fp, prn);
     if (maxlen <= 0) {
+	err = E_DATA;
 	goto oct_bailout;
     }
  
     line = malloc(maxlen);
     if (line == NULL) {
+	err = E_ALLOC;
 	goto oct_bailout;
     }
 
@@ -2588,6 +2731,7 @@ int import_octave (double ***pZ, DATAINFO **ppdinfo,
 
     if (err || nrows == 0 || ncols == 0) {
 	pputs(prn, M_("Invalid data file\n"));
+	err = E_DATA;
 	goto oct_bailout;
     } 
 
@@ -2595,8 +2739,8 @@ int import_octave (double ***pZ, DATAINFO **ppdinfo,
 
     octinfo = datainfo_new();
     if (octinfo == NULL) {
-	fclose(fp);
 	pputs(prn, M_("Out of memory\n"));
+	err = E_ALLOC;
 	goto oct_bailout;
     }
 
@@ -2605,6 +2749,7 @@ int import_octave (double ***pZ, DATAINFO **ppdinfo,
 
     if (start_new_Z(&octZ, octinfo, 0)) {
 	pputs(prn, M_("Out of memory\n"));
+	err = E_ALLOC;
 	goto oct_bailout;
     }  
 
@@ -2664,28 +2809,11 @@ int import_octave (double ***pZ, DATAINFO **ppdinfo,
 
     if (err) {
 	pputs(prn, M_("Invalid data file\n"));
+	err = E_DATA;
 	goto oct_bailout;
     } 
-    
-    if (*pZ == NULL) {
-	/* no dataset currently in place */
-	*pZ = octZ;
-	if (*ppdinfo != NULL) {
-	    free(*ppdinfo);
-	}
-	*ppdinfo = octinfo;
-    } else if (merge_data(pZ, *ppdinfo, octZ, octinfo, prn)) {
-	goto oct_bailout;
-    }
 
-    fclose(fp); 
-    free(line);
-
-#ifdef ENABLE_NLS
-    console_off();
-#endif
-
-    return 0;
+    err = merge_or_replace_data(pZ, pdinfo, &octZ, &octinfo, opt, prn);
 
  oct_bailout:
 
@@ -2705,15 +2833,16 @@ int import_octave (double ***pZ, DATAINFO **ppdinfo,
     console_off();
 #endif
 
-    return 1;
+    return err;
 }
 
 /**
  * import_other:
- * @pZ: pointer to data set.
- * @ppdinfo: pointer to data information struct.
- * @ftype: type of data file.
  * @fname: name of file.
+ * @ftype: type of data file.
+ * @pZ: pointer to data set.
+ * @pdinfo: pointer to data information struct.
+ * @opt: for use with "append".
  * @prn: gretl printing struct.
  * 
  * Open a data file of a type that requires a special plugin.
@@ -2721,12 +2850,89 @@ int import_octave (double ***pZ, DATAINFO **ppdinfo,
  * Returns: 0 on successful completion, non-zero otherwise.
  */
 
-int import_other (double ***pZ, DATAINFO **ppdinfo, 
-		  int ftype, const char *fname, PRN *prn)
+int import_other (const char *fname, int ftype,
+		  double ***pZ, DATAINFO *pdinfo, 
+		  gretlopt opt, PRN *prn)
 {
     void *handle;
     FILE *fp;
-    int (*sheet_get_data)(const char*, double ***, DATAINFO **, PRN *);
+    int (*importer) (const char *, 
+		     double ***, DATAINFO *, 
+		     gretlopt, PRN *);
+    int err = 0;
+
+#ifdef ENABLE_NLS
+    check_for_console(prn);
+#endif
+
+    fp = gretl_fopen(fname, "r");
+    if (fp == NULL) {
+	pprintf(prn, M_("Couldn't open %s\n"), fname);
+	err = E_FOPEN;
+	goto bailout;
+    }
+
+    fclose(fp);
+
+    if (ftype == GRETL_OCTAVE) {
+	/* plugin not needed */
+	return import_octave(fname, pZ, pdinfo, opt, prn);
+    }
+
+    if (ftype == GRETL_WF1) {
+	importer = get_plugin_function("wf1_get_data", &handle);
+    } else if (ftype == GRETL_DTA) {
+	importer = get_plugin_function("dta_get_data", &handle);
+    } else if (ftype == GRETL_JMULTI) {
+	importer = get_plugin_function("jmulti_get_data", &handle);
+    } else {
+	pprintf(prn, M_("Unrecognized data type"));
+	pputc(prn, '\n');
+	return E_DATA;
+    }
+
+    if (importer == NULL) {
+        err = 1;
+    } else {
+	err = (*importer)(fname, pZ, pdinfo, opt, prn);
+	close_plugin(handle);
+    }
+
+ bailout:
+
+#ifdef ENABLE_NLS
+    console_off();
+#endif
+
+    return err;
+}
+
+/**
+ * import_spreadsheet:
+ * @fname: name of file.
+ * @ftype: type of data file.
+ * @list: list of parameters for spreadsheet import, or %NULL.
+ * @sheetname: name of worksheet, or %NULL.
+ * @pZ: pointer to data set.
+ * @pdinfo: pointer to data information struct.
+ * @opt: fir use with "append".
+ * @prn: gretl printing struct.
+ * 
+ * Open a data file of a type that requires a special plugin.
+ * 
+ * Returns: 0 on successful completion, non-zero otherwise.
+ */
+
+int import_spreadsheet (const char *fname, int ftype, 
+			int *list, char *sheetname,
+			double ***pZ, DATAINFO *pdinfo, 
+			gretlopt opt, PRN *prn)
+{
+    void *handle;
+    FILE *fp;
+    int (*importer) (const char*, int *, char *,
+		     double ***, DATAINFO *, 
+		     gretlopt, PRN *);
     int err = 0;
 
 #ifdef ENABLE_NLS
@@ -2743,27 +2949,21 @@ int import_other (double ***pZ, DATAINFO **ppdinfo,
     fclose(fp);
 
     if (ftype == GRETL_GNUMERIC) {
-	sheet_get_data = get_plugin_function("cli_get_gnumeric", &handle);
-    } else if (ftype == GRETL_EXCEL) {
-	sheet_get_data = get_plugin_function("cli_get_xls", &handle);
+	importer = get_plugin_function("gnumeric_get_data", &handle);
+    } else if (ftype == GRETL_XLS) {
+	importer = get_plugin_function("xls_get_data", &handle);
     } else if (ftype == GRETL_ODS) {
-	sheet_get_data = get_plugin_function("cli_get_ods", &handle);
-    } else if (ftype == GRETL_WF1) {
-	sheet_get_data = get_plugin_function("wf1_get_data", &handle);
-    } else if (ftype == GRETL_DTA) {
-	sheet_get_data = get_plugin_function("dta_get_data", &handle);
-    } else if (ftype == GRETL_JMULTI) {
-	sheet_get_data = get_plugin_function("jmulti_get_data", &handle);
+	importer = get_plugin_function("ods_get_data", &handle);
     } else {
 	pprintf(prn, M_("Unrecognized data type"));
 	pputc(prn, '\n');
 	return E_DATA;
     }
 
-    if (sheet_get_data == NULL) {
+    if (importer == NULL) {
         err = 1;
     } else {
-	err = (*sheet_get_data)(fname, pZ, ppdinfo, prn);
+	err = (*importer)(fname, list, sheetname, pZ, pdinfo, opt, prn);
 	close_plugin(handle);
     }
 
@@ -2865,7 +3065,7 @@ GretlFileType detect_filetype (char *fname, PATHS *ppaths, PRN *prn)
     if (has_suffix(fname, ".gnumeric"))
 	return GRETL_GNUMERIC;
     if (has_suffix(fname, ".xls"))
-	return GRETL_EXCEL;
+	return GRETL_XLS;
     if (has_suffix(fname, ".ods"))
 	return GRETL_ODS;
     if (has_suffix(fname, ".wf1"))
@@ -2877,9 +3077,9 @@ GretlFileType detect_filetype (char *fname, PATHS *ppaths, PRN *prn)
     if (has_suffix(fname, ".rat"))
 	return GRETL_RATS_DB;
     if (has_suffix(fname, ".csv"))
-	return GRETL_CSV_DATA;
+	return GRETL_CSV;
     if (has_suffix(fname, ".txt"))
-	return GRETL_CSV_DATA;
+	return GRETL_CSV;
     if (has_suffix(fname, ".m"))
 	return GRETL_OCTAVE;
     if (has_suffix(fname, ".bn7"))
