@@ -23,27 +23,13 @@
 #include <stdio.h>
 #include <string.h>
 
-typedef enum {
-    GRETL_DATA_FLOAT = 1, /* single-precision binary data */
-    GRETL_DATA_DOUBLE,    /* double-precision binary data */
-    GRETL_DATA_OCTAVE,    /* data in Gnu Octave format */
-    GRETL_DATA_CSV,       /* data in Comma Separated Values format */
-    GRETL_DATA_R,         /* data in Gnu R format */
-    GRETL_DATA_R_TS,      /* data in Gnu R format (time series) */
-    GRETL_DATA_GZIPPED,   /* gzipped data */
-    GRETL_DATA_TRAD,      /* traditional (ESL-style) data */
-    GRETL_DATA_DAT,       /* data in PcGive format */
-    GRETL_DATA_DB,        /* gretl native database format */
-    GRETL_DATA_JM         /* JMulti ascii data */
-} GretlDataFormat;
-
 typedef enum {
     GRETL_NATIVE_DATA,    /* gretl native format data file */
     GRETL_XML_DATA,       /* gretl xml format data file */
-    GRETL_CSV_DATA,       /* comma-separated data file */
+    GRETL_CSV,            /* comma-separated data file */
     GRETL_OCTAVE,         /* GNU octave ascii data file */
     GRETL_GNUMERIC,       /* gnumeric workbook data */
-    GRETL_EXCEL,          /* MS Excel spreadsheet data */
+    GRETL_XLS,            /* MS Excel spreadsheet data */
     GRETL_ODS,            /* Open Document Spreadsheet data */
     GRETL_WF1,            /* Eviews workfile data */
     GRETL_DTA,            /* Stata .dta data */
@@ -54,6 +40,7 @@ typedef enum {
     GRETL_RATS_DB,        /* RATS 4.0 database */
     GRETL_PCGIVE_DB,      /* PcGive bn7/in7 pair */
     GRETL_JMULTI,         /* JMulTi data file */
+    GRETL_ODBC,           /* Open DataBase Connectivity */
     GRETL_UNRECOGNIZED    /* none of the above */
 } GretlFileType;
 
@@ -68,11 +55,14 @@ typedef enum {
     VARNAME_BADCHAR       /* illegal character in second or subsequent place */
 } GretlVarnameError;
 
-#define WORKSHEET_IMPORT(f) (f == GRETL_EXCEL || \
-			     f == GRETL_GNUMERIC || \
-			     f == GRETL_ODS || \
-			     f == GRETL_DTA || \
-			     f == GRETL_WF1)
+#define SPREADSHEET_IMPORT(f) (f == GRETL_XLS || \
+			       f == GRETL_GNUMERIC || \
+			       f == GRETL_ODS)
+
+#define OTHER_IMPORT(f) (f == GRETL_DTA || \
+                         f == GRETL_JMULTI || \
+                         f == GRETL_OCTAVE || \
+			 f == GRETL_WF1)
 
 #define free_datainfo(p) do { if (p != NULL) { clear_datainfo(p, 0); free(p); } \
                             } while (0);
@@ -85,7 +75,7 @@ typedef enum {
 
 int dateton (const char *date, const DATAINFO *pdinfo);
 
-char *ntodate (char *datestr, int nt, const DATAINFO *pdinfo);
+char *ntodate (char *datestr, int t, const DATAINFO *pdinfo);
 
 char *ntodate_full (char *datestr, int t, const DATAINFO *pdinfo);
 
@@ -97,7 +87,7 @@ int get_precision (const double *x, int n, int placemax);
 
 double get_date_x (int pd, const char *obs);
 
-int write_data (const char *fname, const int *list, 
+int write_data (const char *fname, int *list, 
 		const double **Z, const DATAINFO *pdinfo, 
 	        gretlopt opt, PATHS *ppaths);
 
@@ -109,31 +99,36 @@ int gretl_is_pkzip_file (const char *fname);
 
 void gz_switch_ext (char *targ, char *src, char *ext);
 
-int merge_or_replace_data (double ***pZ0, DATAINFO **ppdinfo0,
+int merge_or_replace_data (double ***pZ0, DATAINFO *pdinfo0,
 			   double ***pZ1, DATAINFO **ppdinfo1,
-			   PRN *prn);
+			   gretlopt opt, PRN *prn);
 
-int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, 
-		    char *datfile, PATHS *ppaths, 
-		    PRN *prn);
+int gretl_get_data (char *datfile, PATHS *ppaths,
+		    double ***pZ, DATAINFO *pdinfo, 
+		    gretlopt opt, PRN *prn);
 
 int open_nulldata (double ***pZ, DATAINFO *pdinfo, 
 		   int data_status, int length,
 		   PRN *prn);
 
-int import_csv (double ***pZ, DATAINFO **ppdinfo, 
-                const char *fname, gretlopt opt, PRN *prn);
+int import_csv (const char *fname, double ***pZ, DATAINFO *pdinfo, 
+		gretlopt opt, PRN *prn);
 
-int import_octave (double ***pZ, DATAINFO **ppdinfo, 
-		   const char *fname, PRN *prn);
+int import_spreadsheet (const char *fname, int ftype,
+			int *list, char *sheetname,
+			double ***pZ, DATAINFO *pdinfo, 
+			gretlopt opt, PRN *prn);
 
-int import_other (double ***pZ, DATAINFO **ppdinfo, 
-		  int ftype, const char *fname, PRN *prn);
+int import_other (const char *fname, int ftype,
+		  double ***pZ, DATAINFO *pdinfo, 
+		  gretlopt opt, PRN *prn);
 
 int add_obs_markers_from_file (DATAINFO *pdinfo, const char *fname);
 
 GretlFileType detect_filetype (char *fname, PATHS *ppaths, PRN *prn);
 
+gretlopt data_save_opt_from_suffix (const char *fname);
+
 int check_varname (const char *varname);
 
 int check_atof (const char *numstr);
