@@ -19,6 +19,8 @@
 
 #include "libgretl.h"
 #include "gretl_func.h"
+#include "libset.h"
+#include "dbread.h"
 
 #define DDEBUG 0
 
@@ -124,12 +126,10 @@ void clear_datainfo (DATAINFO *pdinfo, int code)
     } 
 
     if (pdinfo->submask != NULL) {
-	free(pdinfo->submask);
+	free_subsample_mask(pdinfo->submask);
 	pdinfo->submask = NULL;
     }
 
-    pdinfo->submode = 0;
-
     if (pdinfo->paninfo != NULL) {
 	dataset_destroy_panel_info(pdinfo);
     }    
@@ -157,7 +157,6 @@ void clear_datainfo (DATAINFO *pdinfo, int code)
 
 	maybe_free_full_dataset(pdinfo);
 
-	/* added Sat Dec  4 12:19:26 EST 2004 */
 	pdinfo->v = pdinfo->n = 0;
     } 
 }
@@ -257,7 +256,7 @@ int dataset_allocate_obs_markers (DATAINFO *pdinfo)
  * @pdinfo: dataset information struct
  *
  * Allocates space in @pdinfo for two indices representing
- * the unit or group and time-period, respectively,  of each 
+ * the unit or group and time-period, respectively, of each 
  * observation in a panel data set.
  *
  * Returns: 0 on success, %E_ALLOC on error.
@@ -477,8 +476,10 @@ static void gretl_varinfo_init (VARINFO *vinfo)
 {
     vinfo->label[0] = '\0';
     vinfo->display_name[0] = '\0';
-    vinfo->gp_display_name[0] = '\0';
+    vinfo->parent[0] = '\0';
     vinfo->flags = 0;
+    vinfo->transform = 0;
+    vinfo->lag = 0;
     vinfo->compact_method = COMPACT_NONE;
     vinfo->stack_level = 0;
     vinfo->line_width = 1;
@@ -502,8 +503,10 @@ void copy_varinfo (VARINFO *targ, const VARINFO *src)
 
     strcpy(targ->label, src->label);
     strcpy(targ->display_name, src->display_name);
-    strcpy(targ->gp_display_name, src->gp_display_name);
+    strcpy(targ->parent, src->parent);
     targ->flags = src->flags;
+    targ->transform = src->transform;
+    targ->lag = src->lag;
     targ->compact_method = src->compact_method;
     targ->stack_level = src->stack_level;
     targ->line_width = src->line_width;
@@ -599,9 +602,6 @@ DATAINFO *datainfo_new (void)
     dinfo->S = NULL;
     dinfo->descrip = NULL;
     dinfo->submask = NULL;
-    dinfo->submode = 0;
-
-    dinfo->data = NULL;
 
     dinfo->structure = CROSS_SECTION;
 
@@ -653,6 +653,20 @@ create_new_dataset (double ***pZ, int nvar, int nobs, int markers)
     return pdinfo;
 }
 
+DATAINFO *
+create_auxiliary_dataset (double ***pZ, int nvar, int nobs)
+{
+    DATAINFO *pdinfo;
+
+    pdinfo = create_new_dataset(pZ, nvar, nobs, 0);
+
+    if (pdinfo != NULL) {
+	pdinfo->descrip = gretl_strdup("auxiliary");
+    }
+
+    return pdinfo;
+}
+
 /**
  * allocate_Z:
  * @pZ: pointer to data array.
@@ -728,10 +742,8 @@ int start_new_Z (double ***pZ, DATAINFO *pdinfo, int resample)
     pdinfo->markers = NO_MARKERS;
     pdinfo->delim = ',';
     pdinfo->descrip = NULL;
-    pdinfo->data = NULL;
     pdinfo->paninfo = NULL;
     pdinfo->submask = NULL;
-    pdinfo->submode = 0;
     
     return 0;
 }
@@ -1341,6 +1353,16 @@ int dataset_add_scalars (int n, double ***pZ, DATAINFO *pdinfo)
     int v = pdinfo->v;
     int i, err = 0;
 
+    if (v == 0) {
+	pdinfo->n = pdinfo->v = 1;
+	err = start_new_Z(pZ, pdinfo, 0);
+	if (err) {
+	    pdinfo->n = pdinfo->v = 0;
+	    return err;
+	}
+	v = 1;
+    }
+
     newZ = realloc(*pZ, (v + n) * sizeof *newZ);  
 
     if (newZ == NULL) {
@@ -1412,11 +1434,6 @@ int dataset_add_scalar_as (double x, const char *newname,
 {
     int v, err = 0;
 
-    if (pdinfo->varinfo == NULL) {
-	strcpy(gretl_errmsg, _("Please open a data file first"));
-	return 1;
-    }
-
     err = dataset_add_scalar(pZ, pdinfo);
 
     if (!err) {
@@ -1595,25 +1612,39 @@ int overwrite_err (const DATAINFO *pdinfo, int v)
     return E_DATA;
 }
 
-static int real_drop_listed_vars (const int *list, double ***pZ, 
+static int real_drop_listed_vars (int *list, double ***pZ, 
 				  DATAINFO *pdinfo, int *renumber,
-				  int drop)
+				  int drop, PRN *prn)
 {
     double **Z = *pZ;
     int oldv = pdinfo->v, vmax = pdinfo->v;
+    char vname[VNAMELEN] = {0};
+    int d0, d1;
     int delmin = oldv;
     int i, v, ndel = 0; 
-    int err;
+    int err = 0;
 
     if (renumber != NULL) {
 	*renumber = 0;
     }
 
-    if (list == NULL) {
+    if (list == NULL || list[0] == 0) {
 	/* no-op */
 	return 0;
     }
 
+    d0 = list[0];
+
+    check_variable_deletion_list(list, pdinfo);
+    d1 = list[0];
+    if (prn != NULL && d1 == 1) {
+	strcpy(vname, pdinfo->varname[list[1]]);
+    }
+
+    if (d1 == 0) {
+	goto finish;
+    }
+
 #if DDEBUG
     fprintf(stderr, "real_drop_listed_variables: dropping %d vars:\n",
 	    list[0]);
@@ -1693,6 +1724,33 @@ static int real_drop_listed_vars (const int *list, double ***pZ,
 
     err = shrink_dataset_to_size(pZ, pdinfo, oldv - ndel, drop);
 
+ finish:
+
+    /* report results, if appropriate */
+
+    if (!err && prn != NULL) {
+	if (d0 == d1) {
+	    if (gretl_messages_on()) {
+		if (*vname != '\0') {
+		    pprintf(prn, "Deleted %s", vname);
+		} else {
+		    pprintf(prn, "Deleted %d variables", d1);
+		}
+		pputc(prn, '\n');
+	    }
+	} else {
+	    if (d1 == 0) {
+		pputs(prn, "No variables deleted");
+	    } else if (*vname != '\0') {
+		pprintf(prn, "Deleted %s", vname);
+	    } else {
+		pprintf(prn, "Deleted %d variables", d1);
+	    }
+	    pputs(prn, " (some data were in use)\n");
+	    
+	}
+    }
+
     return err;
 }
 
@@ -1722,6 +1780,7 @@ static int *make_dollar_list (DATAINFO *pdinfo, int *err)
  * @renumber: location for return of information on whether
  * remaining variables have been renumbered as a result, or
  * %NULL.
+ * @prn: pointer to printing struct.
  *
  * Deletes the variables given in @list from the dataset.  Remaining
  * variables may have their ID numbers changed as a consequence. If
@@ -1733,13 +1792,19 @@ static int *make_dollar_list (DATAINFO *pdinfo, int *err)
 
 int dataset_drop_listed_variables (int *list, double ***pZ, 
 				   DATAINFO *pdinfo, 
-				   int *renumber)
+				   int *renumber,
+				   PRN *prn)
 {
     int *dlist = NULL;
     int free_dlist = 0;
     int lastvar[2];
     int err = 0;
 
+    if (pdinfo->n == 0) {
+	strcpy(gretl_errmsg, _("No dataset is in place"));
+	return E_DATA;
+    }
+
     if (list == NULL) {
 	/* signal to drop internal "$" variables */
 	dlist = make_dollar_list(pdinfo, &err);
@@ -1760,19 +1825,21 @@ int dataset_drop_listed_variables (int *list, double ***pZ,
     }
 
     err = real_drop_listed_vars(dlist, pZ, pdinfo, renumber,
-				DROP_NORMAL);
+				DROP_NORMAL, prn);
 
-    if (!err) {
-	err = gretl_lists_revise(dlist, 0);
-    }
+    if (dlist[0] > 0) {
+	if (!err) {
+	    err = gretl_lists_revise(dlist, 0);
+	}
 
-    if (!err && complex_subsampled()) {
-	double ***fZ = fetch_full_Z();
-	DATAINFO *fdinfo = fetch_full_datainfo();
+	if (!err && complex_subsampled()) {
+	    double ***fZ = fetch_full_Z();
+	    DATAINFO *fdinfo = fetch_full_datainfo();
 
-	err = real_drop_listed_vars(dlist, fZ, fdinfo, NULL,
-				    DROP_SPECIAL);
-	reset_full_Z(fZ);
+	    err = real_drop_listed_vars(dlist, fZ, fdinfo, NULL,
+					DROP_SPECIAL, NULL);
+	    reset_full_Z(fZ);
+	}
     }
 
     if (free_dlist) {
@@ -1801,7 +1868,7 @@ int dataset_drop_variable (int v, double ***pZ, DATAINFO *pdinfo)
 	return E_DATA;
     }
 
-    return dataset_drop_listed_variables(list, pZ, pdinfo, NULL);
+    return dataset_drop_listed_variables(list, pZ, pdinfo, NULL, NULL);
 }
 
 /**
@@ -1846,7 +1913,8 @@ int dataset_destroy_hidden_variables (double ***pZ, DATAINFO *pdinfo,
 		    list[j++] = i;
 		}
 	    }	    
-	    err = dataset_drop_listed_variables(list, pZ, pdinfo, NULL);
+	    err = dataset_drop_listed_variables(list, pZ, pdinfo, 
+						NULL, NULL);
 	    free(list);
 	}
     }
@@ -2121,7 +2189,7 @@ static int get_optional_offset (const char *s, const double **Z,
 
     if (p != NULL) {
 	if (strncmp(p, "--offset=", 9)) {
-	    *err = E_SYNTAX;
+	    *err = E_PARSE;
 	} else {
 	    off = get_stack_param_val(p + 9, Z, pdinfo);
 	    if (off < 0 || off > pdinfo->n - 1) {
@@ -2141,7 +2209,7 @@ static int get_optional_length (const char *s, const double **Z,
 
     if (p != NULL) {
 	if (strncmp(p, "--length=", 9)) {
-	    *err = E_SYNTAX;
+	    *err = E_PARSE;
 	} else {
 	    len = get_stack_param_val(p + 9, Z, pdinfo);
 	    if (len < 0 || len > pdinfo->n) {
@@ -2208,7 +2276,7 @@ int dataset_stack_variables (const char *vname, const char *line,
     line += 6; /* skip "stack(" */
     if (*line == ',') {
 	free(scpy);
-	return E_SYNTAX;
+	return E_PARSE;
     }
 
     /* copy active portion of line (we use strtok below) */
@@ -2220,7 +2288,7 @@ int dataset_stack_variables (const char *vname, const char *line,
 
     p = strrchr(s, ')');
     if (p == NULL) {
-	err = E_SYNTAX;
+	err = E_PARSE;
 	goto bailout;
     }
 
@@ -2256,7 +2324,7 @@ int dataset_stack_variables (const char *vname, const char *line,
 	nv++;
 
 	if (nv < 2) {
-	    return E_SYNTAX;
+	    return E_PARSE;
 	}
 
 	vnum = malloc(nv * sizeof *vnum);
@@ -2298,6 +2366,10 @@ int dataset_stack_variables (const char *vname, const char *line,
 	goto bailout;
     }
 
+#if PDEBUG
+    fprintf(stderr, "offset = %d, maxok = %d\n", offset, maxok);
+#endif
+
     if (offset + maxok > pdinfo->n) {
 	err = E_DATA;
 	goto bailout;
@@ -2337,6 +2409,10 @@ int dataset_stack_variables (const char *vname, const char *line,
 	}
     }
 
+#if PDEBUG
+    fprintf(stderr, "bign = %d, allocating bigx (oldn = %d)\n", bign, pdinfo->n);
+#endif
+
     /* allocate stacked series */
     bigx = malloc(bign * sizeof *bigx);
     if (bigx == NULL) {
@@ -2569,22 +2645,6 @@ int var_set_display_name (DATAINFO *pdinfo, int i,
     *targ = 0;
     strncat(targ, s, MAXDISP - 1);
 
-    targ = pdinfo->varinfo[i]->gp_display_name;
-    *targ = 0;
-
-#ifdef ENABLE_NLS
-    if (*s != '\0') {
-	char *trs = utf8_to_latin(s);
-
-	if (trs != NULL) {
-	    strncat(targ, trs, MAXDISP - 1);
-	    free(trs);
-	}
-    }
-#else
-    strncat(targ, s, MAXDISP - 1);
-#endif
-
     return 0;
 }
 
@@ -2593,7 +2653,7 @@ const char *var_get_graph_name (const DATAINFO *pdinfo, int i)
     const char *ret = pdinfo->varname[i];
 
     if (pdinfo->varinfo != NULL && pdinfo->varinfo[i] != NULL) {
-	ret = pdinfo->varinfo[i]->gp_display_name;
+	ret = pdinfo->varinfo[i]->display_name;
 	if (ret[0] == '\0') {
 	    ret = pdinfo->varname[i];
 	}
@@ -2644,15 +2704,20 @@ int dataset_op_from_string (const char *s)
 	op = DS_DSORTBY;
     } else if (!strcmp(s, "resample")) {
 	op = DS_RESAMPLE;
+    } else if (!strcmp(s, "restore")) {
+	op = DS_RESTORE;
+    } else if (!strcmp(s, "clear")) {
+	op = DS_CLEAR;
     }
 
     return op;
 }
 
-static int dataset_int_param (const char *s, int op, 
+static int dataset_int_param (const char **ps, int op, 
 			      double **Z, DATAINFO *pdinfo, 
 			      int *err)
 {
+    const char *s = *ps;
     char test[32];
     int k = 0;
 
@@ -2664,6 +2729,7 @@ static int dataset_int_param (const char *s, int op,
 
     *test = '\0';
     sscanf(s, "%31s", test);
+    *ps += strlen(test);
 
     k = gretl_int_from_string(test, (const double **) Z, pdinfo, err);
     if (*err) {
@@ -2712,22 +2778,37 @@ static int compact_data_set_wrapper (const char *s, double ***pZ,
 {
     CompactMethod method = COMPACT_AVG;
 
-    if (strstr(s, "sum")) {
+    s += strspn(s, " ");
+
+    if (!strcmp(s, "sum")) {
 	method = COMPACT_SUM;
-    } else if (strstr(s, "first")) {
+    } else if (!strcmp(s, "first") || !strcmp(s, "sop")) {
 	method = COMPACT_SOP;
-    } else if (strstr(s, "last")) {
+    } else if (!strcmp(s, "last") || !strcmp(s, "eop")) {
 	method = COMPACT_EOP;
+    } else if (!strcmp(s, "avg") || !strcmp(s, "average")) {
+	method = COMPACT_AVG;
+    } else if (*s != '\0') {
+	return E_PARSE;
     }
 
     return compact_data_set(pZ, pdinfo, k, method, 0, 0);
 }
 
+static unsigned int resample_seed;
+
+unsigned int get_resampling_seed (void)
+{
+    return resample_seed;
+}
+
 /* resample the dataset by observation, with replacement */
 
-static int dataset_resample (int n, double ***pZ, DATAINFO *pdinfo)
+int dataset_resample (int n, unsigned int seed,
+		      double ***pZ, DATAINFO *pdinfo)
 {
     double **RZ = NULL;
+    DATAINFO *rinfo = NULL;
     char **S = NULL;
     int T = pdinfo->t2 - pdinfo->t1 + 1;
     int v = pdinfo->v;
@@ -2738,8 +2819,14 @@ static int dataset_resample (int n, double ***pZ, DATAINFO *pdinfo)
 	return E_DATA;
     }
 
+    rinfo = datainfo_new();
+    if (rinfo == NULL) {
+	return E_ALLOC;
+    }
+
     RZ = malloc(v * sizeof *RZ);
     if (RZ == NULL) {
+	free(rinfo);
 	return E_ALLOC;
     }
 
@@ -2747,6 +2834,8 @@ static int dataset_resample (int n, double ***pZ, DATAINFO *pdinfo)
 	RZ[i] = NULL;
     }
 
+    rinfo->v = v;
+
     j = 0;
     for (i=0; i<pdinfo->v && !err; i++) {
 	if (i > 0 && var_is_scalar(pdinfo, i)) {
@@ -2769,17 +2858,20 @@ static int dataset_resample (int n, double ***pZ, DATAINFO *pdinfo)
     }
 
     if (err) {
-	for (i=0; i<v; i++) {
-	    free(RZ[i]);
-	}
-	free(RZ);
-	return E_ALLOC;
+	goto bailout;
     }
 
     if (pdinfo->markers == REGULAR_MARKERS) {
 	S = strings_array_new_with_length(n, OBSLEN);
     }
 
+    if (seed > 0) {
+	resample_seed = seed;
+	gretl_rand_set_seed(seed);
+    } else {
+	resample_seed = gretl_rand_get_seed();
+    }
+
     for (t=0; t<n; t++) {
 	s = gretl_rand_int_max(T) + pdinfo->t1;
 	j = 1;
@@ -2794,22 +2886,34 @@ static int dataset_resample (int n, double ***pZ, DATAINFO *pdinfo)
 	}
     }
 
-    free_Z(*pZ, pdinfo);
-    *pZ = RZ;
-
     if (S != NULL) {
-	dataset_destroy_obs_markers(pdinfo);
-	pdinfo->S = S;
-	pdinfo->markers = REGULAR_MARKERS;
-    } else if (pdinfo->S != NULL) {
-	dataset_destroy_obs_markers(pdinfo);
-    }
+	rinfo->S = S;
+	rinfo->markers = REGULAR_MARKERS;
+    } 
 
-    pdinfo->v = v;
-    pdinfo->n = n;
-    pdinfo->t1 = 0;
-    pdinfo->t2 = n - 1;
-    dataset_obs_info_default(pdinfo);
+    rinfo->varname = pdinfo->varname;
+    rinfo->varinfo = pdinfo->varinfo;
+    rinfo->descrip = pdinfo->descrip;
+
+    rinfo->n = n;
+    rinfo->t1 = 0;
+    rinfo->t2 = n - 1;
+    dataset_obs_info_default(rinfo);
+
+    rinfo->submask = RESAMPLED;
+
+ bailout:
+
+    if (err) {
+	free_Z(RZ, rinfo);
+	clear_datainfo(rinfo, CLEAR_SUBSAMPLE);
+	free(rinfo);
+    } else {
+	backup_full_dataset(*pZ, pdinfo);
+	*pZ = RZ;
+	*pdinfo = *rinfo;
+	free(rinfo);
+    }
 
     return err;
 }
@@ -2818,16 +2922,42 @@ int modify_dataset (int op, const int *list, const char *s,
 		    double ***pZ, DATAINFO *pdinfo, 
 		    PRN *prn)
 {
+    static int resampled;
     int k = 0, err = 0;
 
-    if (complex_subsampled()) {
+    if (op == DS_CLEAR) {
+	/* must be handled by the calling program */
+	return E_NOTIMP;
+    }
+
+    if (gretl_function_depth() > 0) {
+	strcpy(gretl_errmsg, _("The 'dataset' command is not available within functions"));
+	return 1;
+    }
+
+    if (op != DS_RESAMPLE && op != DS_RESTORE && gretl_looping()) {
+	pputs(prn, _("Sorry, this command is not available in loop mode\n"));
+	return 1;
+    }
+
+    if (op == DS_RESAMPLE && resampled) {
+	/* repeated "resample": implicitly restore first */
+	err = restore_full_sample(pZ, pdinfo, NULL);
+	if (err) {
+	    return err;
+	} else {
+	    resampled = 0;
+	}
+    }
+
+    if (op != DS_RESTORE && complex_subsampled()) {
 	strcpy(gretl_errmsg, _("The data set is currently sub-sampled"));
 	return 1;
     }
 
     if (op == DS_ADDOBS || op == DS_COMPACT || 
 	op == DS_EXPAND || op == DS_RESAMPLE) {
-	k = dataset_int_param(s, op, *pZ, pdinfo, &err);
+	k = dataset_int_param(&s, op, *pZ, pdinfo, &err);
 	if (err) {
 	    return err;
 	}
@@ -2846,7 +2976,18 @@ int modify_dataset (int op, const int *list, const char *s,
     } else if (op == DS_DSORTBY) {
 	err = dataset_sort(s, *pZ, pdinfo, OPT_D);
     } else if (op == DS_RESAMPLE) {
-	err = dataset_resample(k, pZ, pdinfo);
+	err = dataset_resample(k, 0, pZ, pdinfo);
+	if (!err) {
+	    resampled = 1;
+	}
+    } else if (op == DS_RESTORE) {
+	if (resampled) {
+	    err = restore_full_sample(pZ, pdinfo, NULL);
+	    resampled = 0;
+	} else {
+	    pprintf(prn, _("dataset restore: dataset is not resampled\n"));
+	    err = E_DATA;
+	}
     } else if (op == DS_DELETE) {
 	pprintf(prn, "dataset delete: not ready yet\n");
     } else if (op == DS_KEEP) {
