@@ -44,9 +44,20 @@ typedef enum {
     DS_KEEP,
     DS_SORTBY,
     DS_DSORTBY,
-    DS_RESAMPLE
+    DS_RESAMPLE,
+    DS_RESTORE,
+    DS_CLEAR
 } DatasetOp;
 
+/**
+ * dataset_is_cross_section:
+ * @p: pointer to data information struct.
+ *
+ * Attempt to determine whether a data set contains cross-sectional
+ * data (1) or not (0).
+ */
+#define dataset_is_cross_section(p) ((p)->structure == CROSS_SECTION)
+
 /**
  * dataset_is_time_series:
  * @p: pointer to data information struct.
@@ -267,6 +278,9 @@ DATAINFO *create_new_dataset (double ***pZ, /* data matrix */
 			      int markers   /* case markers or not? */
 			      );
 
+DATAINFO *
+create_auxiliary_dataset (double ***pZ, int nvar, int nobs);
+
 void destroy_dataset (double **Z, DATAINFO *pdinfo);
 
 void clear_datainfo (DATAINFO *pdinfo, int code);
@@ -331,7 +345,8 @@ int dataset_copy_variable_as (int v, const char *newname,
 int overwrite_err (const DATAINFO *pdinfo, int v);
 
 int dataset_drop_listed_variables (int *list, double ***pZ, 
-				   DATAINFO *pdinfo, int *renumber);
+				   DATAINFO *pdinfo, int *renumber,
+				   PRN *prn);
 
 int dataset_drop_variable (int v, double ***pZ, DATAINFO *pdinfo); 
 
@@ -363,6 +378,11 @@ int var_set_display_name (DATAINFO *pdinfo, int i,
 
 const char *var_get_graph_name (const DATAINFO *pdinfo, int i);
 
+unsigned int get_resampling_seed (void);
+
+int dataset_resample (int n, unsigned int seed,
+		      double ***pZ, DATAINFO *pdinfo);
+
 int dataset_op_from_string (const char *s);
 
 int modify_dataset (int op, const int *list, const char *s, 
