@@ -22,6 +22,9 @@
 #include "libgretl.h"
 #include "swap_bytes.h"
 #include "gretl_www.h"
+#include "libset.h"
+#include "gretl_string_table.h"
+#include "dbread.h"
 
 #include <glib.h>
 #include <unistd.h>
@@ -1512,12 +1515,134 @@ const char *get_db_name (void)
     return db_name;
 }
 
-int db_set_sample (const char *line, DATAINFO *pdinfo)
+/* Handling of DSN setup for ODBC: grab the dsn, username
+   and password strings.
+*/
+
+static char *get_dsn_field (const char *tag, const char *src)
 {
-    char cmd[5], start[OBSLEN], stop[OBSLEN];
+    const char *p;
+    char needle[12];
+    char *ret = NULL;
+    
+    sprintf(needle, "%s=", tag);
+    p = strstr(src, needle);
+
+    if (p != NULL) {
+	p += strlen(needle);
+	if (*p == '"' || *p == '\'') {
+	    ret = gretl_quoted_string_strdup(p, NULL);
+	} else {
+	    ret = gretl_strndup(p, strcspn(p, " "));
+	}
+    }
+
+    return ret;
+}
+
+static ODBC_info gretl_odinfo;
+
+static void ODBC_info_clear_read (void)
+{
+    int i;
+
+    free(gretl_odinfo.query);
+    gretl_odinfo.query = NULL;
+
+    free(gretl_odinfo.x);
+    gretl_odinfo.x = NULL;
+
+    free_strings_array(gretl_odinfo.S, gretl_odinfo.nrows);
+    gretl_odinfo.S = NULL;
+
+    gretl_odinfo.nrows = 0;
+    gretl_odinfo.ncols = 0;
+
+    for (i=0; i<ODBC_MAXCOLS; i++) {
+	gretl_odinfo.coltypes[i] = 0;
+    }
+
+    if (gretl_odinfo.fmts != NULL) {
+	for (i=0; i<gretl_odinfo.ncols; i++) {
+	    free(gretl_odinfo.fmts[i]);
+	}
+	free(gretl_odinfo.fmts);
+	gretl_odinfo.fmts = NULL;
+    }
+
+    gretl_odinfo.nrows = 0;
+    gretl_odinfo.ncols = 0;
+}
+
+void ODBC_info_clear_all (void)
+{
+    free(gretl_odinfo.dsn);
+    gretl_odinfo.dsn = NULL;
+
+    free(gretl_odinfo.username);
+    gretl_odinfo.username = NULL;
+
+    free(gretl_odinfo.password);
+    gretl_odinfo.password = NULL;
+
+    ODBC_info_clear_read();
+}
+
+int set_odbc_dsn (const char *line, PRN *prn)
+{
+    void *handle = NULL;
+    int (*check_dsn) (ODBC_info *);
+    char *dbname = NULL;
+    char *uname = NULL;
+    char *pword = NULL;
+    int err = 0;
+
+    /* skip command word */
+    line += strcspn(line, " ");
+    line += strspn(line, " ");
+
+    ODBC_info_clear_all();
+
+    dbname = get_dsn_field("dsn", line);
+    if (dbname == NULL) {
+	pputs(prn, "You must specify a DSN using 'dsn=...'\n");
+	return E_DATA;
+    }
+
+    uname = get_dsn_field("user", line);
+    pword = get_dsn_field("password", line);
+
+    gretl_odinfo.dsn = dbname;
+    gretl_odinfo.username = uname;
+    gretl_odinfo.password = pword;
+
+    gretl_error_clear();
+    
+    check_dsn = get_plugin_function("gretl_odbc_check_dsn", &handle);
+
+    if (check_dsn == NULL) {
+        err = 1;
+    } else {
+        err = (* check_dsn) (&gretl_odinfo);
+        close_plugin(handle);
+    }
+
+    if (err) {
+	ODBC_info_clear_all();
+    } else if (gretl_messages_on()) {
+	pprintf(prn, "Connected to ODBC data source '%s'\n", 
+		gretl_odinfo.dsn);
+    }
+
+    return err;
+}
+
+int db_set_sample (const char *s, DATAINFO *pdinfo)
+{
+    char start[OBSLEN], stop[OBSLEN];
     int t1 = 0, t2 = 0;
 
-    if (sscanf(line, "%4s %8s %8s", cmd, start, stop) != 3) {
+    if (sscanf(s, "%10s %10s", start, stop) != 2) {
 	sprintf(gretl_errmsg, _("error reading smpl line"));
 	return 1;
     }
@@ -1556,14 +1681,15 @@ int db_set_sample (const char *line, DATAINFO *pdinfo)
     return 0;
 }
 
-static const char *
-get_word_and_advance (const char *s, char *word, size_t maxlen)
+static char *
+get_word_and_advance (char *s, char *word, size_t maxlen)
 {
     size_t i = 0;
 
     while (isspace(*s)) s++;
 
     *word = 0;
+
     while (*s && !isspace(*s)) {
 	if (i < maxlen) word[i++] = *s;
 	s++;
@@ -1575,10 +1701,10 @@ get_word_and_advance (const char *s, char *word, size_t maxlen)
     else return NULL;
 }
 
-static const char *
-get_compact_method_and_advance (const char *s, CompactMethod *method)
+static char *
+get_compact_method_and_advance (char *s, CompactMethod *method)
 {
-    const char *p;
+    char *p;
 
     *method = COMPACT_NONE;
 
@@ -1647,12 +1773,257 @@ static void free_dbZ (double **dbZ)
     }
 }
 
+static int parse_odbc_format_chunk (char **ps, int i)
+{
+    const char *numchars = "0123456789";
+    char *chunk = NULL;
+    char *p = *ps;
+    int n, err = 0;
+
+    /* advance to '%' */
+    while (*p && *p != '%') p++; 
+    if (*p == '\0') {
+	return E_PARSE;
+    }
+
+    p++; /* move past '%' */
+
+    /* zero padding? */
+    if (*p == '0') {
+	p++;
+    }
+
+    /* optional width? */
+    n = strspn(p, numchars);
+    if (n == 1) {
+	p++;
+    } else if (n > 0) {
+	return E_PARSE;
+    } 
+
+    /* optional dot plus precision? */
+    if (*p == '.') {
+	p++;
+	n = strspn(p, numchars);
+	if (n == 1) {
+	    p++;
+	} else {
+	    return E_PARSE;
+	}
+    }
+
+    /* now we should have a conversion character */
+    if (*p == 'd') {
+	gretl_odinfo.coltypes[i] = GRETL_TYPE_INT;
+    } else if (*p == 's') {
+	gretl_odinfo.coltypes[i] = GRETL_TYPE_STRING;
+    } else if (*p == 'f' || *p == 'g') {
+	gretl_odinfo.coltypes[i] = GRETL_TYPE_DOUBLE;
+    } else if (*p == 'D') {
+	*p = 's';
+	gretl_odinfo.coltypes[i] = GRETL_TYPE_DATE;
+    } else {
+	return E_PARSE;
+    }
+
+    /* append any trailing fixed chars */
+    p++;
+    while (*p && *p != '%') p++; 
+    n = p - *ps;
+
+    chunk = gretl_strndup(*ps, n);
+    if (chunk == NULL) {
+	err = E_ALLOC;
+    } else {
+	err = strings_array_add(&gretl_odinfo.fmts, 
+				&gretl_odinfo.ncols, 
+				chunk);
+    }
+
+    *ps = p;
+
+#if 1
+    fprintf(stderr, "set coltype[%d] = %d, fmt='%s'\n", i, 
+	    gretl_odinfo.coltypes[i], gretl_odinfo.fmts[i]);
+#endif
+
+    return err;
+}
+
+static int parse_odbc_format (char *fmt)
+{
+    char *s = fmt;
+    int i, err = 0;
+
+    for (i=0; i<ODBC_MAXCOLS && !err && *s; i++) {
+	err = parse_odbc_format_chunk(&s, i);
+    }
+
+    if (!err && *s != '\0') {
+	err = E_PARSE;
+    }
+
+    free(fmt);
+
+    return err;
+}
+
+static char *odbc_get_query (char *s, int *err)
+{
+    char *p, *query = NULL;
+
+    if (*s == '"') {
+	query = gretl_quoted_string_strdup(s, NULL);
+    } else {
+	p = get_string_by_name(s);
+	if (p != NULL) {
+	    query = gretl_strdup(p);
+	} else {
+	    query = gretl_strdup(s);
+	}
+    }
+
+    if (query == NULL) {
+	*err = E_ALLOC;
+    } else if (*query == '\0') {
+	gretl_errmsg_set(_("Expected an SQL query string"));
+	*err = E_PARSE;
+    }	
+
+    return query;
+}
+
+static int odbc_get_varname (char *s, char *vname)
+{
+    int n = gretl_varchar_spn(s);
+    int err = 0;
+
+    if (n == 0 || n > VNAMELEN - 1) {
+	gretl_errmsg_set(_("Expected a valid variable name"));
+	err = E_PARSE;
+    } else {
+	*vname = '\0';
+	strncat(vname, s, n);
+	err = check_varname(vname);
+    }
+
+    return err;
+}
+
+static int odbc_get_series (char *line, double ***pZ, DATAINFO *pdinfo, 
+			    PRN *prn)
+{
+    void *handle = NULL;
+    int (*get_data) (ODBC_info *);
+    char vname[VNAMELEN];
+    char *format = NULL;
+    int err = 0;
+
+    if (gretl_odinfo.dsn == NULL) {
+	strcpy(gretl_errmsg, _("No database has been opened"));
+	return 1;
+    } 
+
+    if (pdinfo->n == 0) {
+	strcpy(gretl_errmsg, _("No series length has been defined"));
+	return 1;
+    }
+
+    line += strcspn(line, " ");
+    line += strspn(line, " ");
+
+    err = odbc_get_varname(line, vname);
+    if (err) {
+	return err;
+    }
+
+    line += strlen(vname);
+    line += strspn(line, " ");
+
+    if (!strncmp(line, "obs-format=", 11)) {
+	line += 11;
+	format = gretl_quoted_string_strdup(line, (const char **) &line);
+	if (format == NULL) {
+	    err = E_PARSE;
+	} else {
+	    err = parse_odbc_format(format);
+	}
+    } 
+
+    if (!err) {
+	line += strspn(line, " ");
+	gretl_odinfo.query = odbc_get_query(line, &err);
+    }
+
+    if (!err) {
+	fprintf(stderr, "SQL query: '%s'\n", gretl_odinfo.query);
+	gretl_error_clear();
+
+	get_data = get_plugin_function("gretl_odbc_get_data", &handle);
+
+	if (get_data == NULL) {
+	    err = 1;
+	} else {
+	    err = (*get_data) (&gretl_odinfo);
+	    close_plugin(handle);
+	}
+    }
+
+    if (!err) {
+	int n = gretl_odinfo.nrows;
+	int s, t, v;
+
+	if (gretl_messages_on()) {
+	    pprintf(prn, "Retrieved %d observations via ODBC\n", n);
+	}
+
+	if (pdinfo->v == 0) {
+	    /* the data matrix is still empty */
+	    pdinfo->v = 2;
+	    err = start_new_Z(pZ, pdinfo, 0);
+	} else {
+	    err = dataset_add_series(1, pZ, pdinfo);
+	}
+	if (!err) {
+	    v = pdinfo->v - 1;
+	    strcpy(pdinfo->varname[v], vname);
+	    strcpy(VARLABEL(pdinfo, v), "ODBC data");
+	    if (gretl_odinfo.S != NULL) {
+		for (t=0; t<pdinfo->n; t++) {
+		    (*pZ)[v][t] = NADBL;
+		}
+		for (s=0; s<n; s++) {
+		    t = dateton(gretl_odinfo.S[s], pdinfo);
+		    if (t >= 0 && t < pdinfo->n) {
+			(*pZ)[v][t] = gretl_odinfo.x[s];
+		    } else {
+			fprintf(stderr, "Bad obs '%s'\n", gretl_odinfo.S[s]);
+		    }
+		}
+	    } else {
+		s = 0;
+		for (t=0; t<pdinfo->n; t++) {
+		    if (t>=pdinfo->t1 && t<=pdinfo->t2 && s<n) {
+			(*pZ)[v][t] = gretl_odinfo.x[s++];
+		    } else {
+			(*pZ)[v][t] = NADBL;
+		    }
+		}
+	    }
+	}
+    }
+
+    ODBC_info_clear_read();
+
+    return err;
+}
+
 /* main function for getting a series out of a database, using the
    command-line client or in script or console mode
 */
 
-int db_get_series (const char *line, double ***pZ, DATAINFO *pdinfo, 
-		   PRN *prn)
+int db_get_series (char *line, double ***pZ, DATAINFO *pdinfo, 
+		   gretlopt opt, PRN *prn)
 {
     char series[VNAMELEN];
     CompactMethod method;
@@ -1660,6 +2031,10 @@ int db_get_series (const char *line, double ***pZ, DATAINFO *pdinfo,
     double **dbZ;
     int err = 0;
 
+    if (opt & OPT_O) {
+	return odbc_get_series(line, pZ, pdinfo, prn);
+    }
+
 #if DB_DEBUG
     fprintf(stderr, "db_get_series: line='%s', pZ=%p, pdinfo=%p\n", 
 	    line, (void *) pZ, (void *) pdinfo);
@@ -1793,7 +2168,7 @@ static void maybe_fclose (FILE *fp)
 
 #define DBUFLEN 1024
 
-static int db_delete_series (const char *line, const int *list,
+static int db_delete_series (char *line, const int *list,
 			     const char *fname, PRN *prn)
 {
     dbnumber buf[DBUFLEN];
@@ -1828,14 +2203,14 @@ static int db_delete_series (const char *line, const int *list,
 	return err;
     }
 
-    strcpy(tmp1, gretl_user_dir());
+    strcpy(tmp1, gretl_dot_dir());
     strcat(tmp1, "tmpidx");
     f1 = tempfile_open(tmp1, &err);
     if (err) {
 	goto bailout;
     }
 
-    strcpy(tmp2, gretl_user_dir());
+    strcpy(tmp2, gretl_dot_dir());
     strcat(tmp2, "tmpbin");
     f2 = tempfile_open(tmp2, &err);
     if (err) {
@@ -1945,7 +2320,7 @@ static int db_delete_series (const char *line, const int *list,
 	    err = rename(tmp2, src2);
 	}
 	if (err) {
-	    gretl_errmsg_set_from_errno();
+	    gretl_errmsg_set_from_errno("db_delete_series");
 	}
 #endif
     } else {
@@ -1960,7 +2335,7 @@ static int db_delete_series (const char *line, const int *list,
     return err;
 }
 
-int db_delete_series_by_name (const char *line, PRN *prn)
+int db_delete_series_by_name (char *line, PRN *prn)
 {
     return db_delete_series(line, NULL, NULL, prn);
 }
@@ -2606,51 +2981,44 @@ static int get_n_ok_months (const DATAINFO *pdinfo,
 
 static int 
 weeks_to_months_exec (double **mZ, const double **Z, const DATAINFO *pdinfo, 
-		      int mn)
+		      CompactMethod method)
 { 
     char obsstr[OBSLEN];
-    int *den;
+    int *mn = NULL;
     int yr, mon, day;
     int monbak = 0;
     int i, s, t = 0;
     int err = 0;
 
-    den = malloc(pdinfo->v * sizeof *den);
-    if (den == NULL) {
+    mn = malloc(pdinfo->v * sizeof *mn);
+    if (mn == NULL) {
 	return E_ALLOC;
     }
 
     for (i=1; i<pdinfo->v; i++) {
 	/* initialize all series, first obs */
 	if (var_is_series(pdinfo, i)) {
-	    mZ[i][0] = 0.0;
-	    den[i] = 0;
+	    mZ[i][0] = NADBL;
+	    mn[i] = 0;
 	}
     }    
 
     for (s=0; s<pdinfo->n; s++) {
 	/* loop across the weekly obs in this month */
-#if WEEKLY_DEBUG
-	fprintf(stderr, "\n** weekly to monthly monthly loop, s = %d\n", s);
-#endif
 	ntodate_full(obsstr, s, pdinfo);
 	sscanf(obsstr, "%d/%d/%d", &yr, &mon, &day);
-#if WEEKLY_DEBUG
-	fprintf(stderr, " month = %d\n", mon);
-#endif
-
 	if (monbak > 0 && mon != monbak) {
 	    /* new month: finalize the previous one */
 	    for (i=1; i<pdinfo->v; i++) {
 		if (var_is_series(pdinfo, i)) {
-#if WEEKLY_DEBUG
-		    fprintf(stderr, " finalizing monthly obs %d, var %d, den = %d\n", 
-			    t, i, den[i]);
-#endif
-		    if (den[i] > 0) {
-			mZ[i][t] /= (double) den[i];
-		    } else {
-			mZ[i][t] = NADBL;
+		    if (method == COMPACT_EOP) {
+			if (s > 0) {
+			    mZ[i][t] = Z[i][s-1];
+			}
+		    } else if (method == COMPACT_AVG) {
+			if (mn[i] > 0) {
+			    mZ[i][t] /= (double) mn[i];
+			}
 		    }
 		}
 	    }
@@ -2660,39 +3028,46 @@ weeks_to_months_exec (double **mZ, const double **Z, const DATAINFO *pdinfo,
 		for (i=1; i<pdinfo->v; i++) {
 		    /* initialize all series, current obs */
 		    if (var_is_series(pdinfo, i)) {
-			mZ[i][t] = 0.0;
-			den[i] = 0;
+			if (method == COMPACT_SOP) {
+			    mZ[i][t] = Z[i][s];
+			} else {
+			    mZ[i][t] = NADBL;
+			}
+			mn[i] = 0;
 		    }
 		}  		
 	    }
 	} 
 
-	/* cumulate non-missing weekly observations */
+	/* cumulate non-missing weekly observations? */
 	for (i=1; i<pdinfo->v; i++) {
 	    if (var_is_series(pdinfo, i)) {
-		if (!na(Z[i][s])) {
-		    mZ[i][t] += Z[i][s];
-		    den[i] += 1;
+		if (method == COMPACT_SOP) {
+		    ; /* handled above */
+		} else if (method == COMPACT_EOP) {
+		    mZ[i][t] = Z[i][s];
+		} else if (!na(Z[i][s])) {
+		    if (na(mZ[i][t])) {
+			mZ[i][t] = Z[i][s];
+		    } else {
+			mZ[i][t] += Z[i][s];
+		    }
+		    mn[i] += 1;
 		}
 		if (mon == monbak && s == pdinfo->n - 1) {
-#if WEEKLY_DEBUG
-		    fprintf(stderr, " finalizing monthly obs %d, var %d, den = %d\n", 
-			    t, i, den[i]);
-#endif
 		    /* reached the end: ship out last obs */
-		    if (den[i] > 0) {
-			mZ[i][t] /= (double) den[i];
-		    } else {
+		    if (method == COMPACT_EOP) {
 			mZ[i][t] = NADBL;
+		    } else if (method == COMPACT_AVG && mn[i] > 0) {
+			mZ[i][t] /= (double) mn[i];
 		    }
 		}		    
 	    }
 	}
-
 	monbak = mon;
     }
 
-    free(den);
+    free(mn);
 
     return err;
 }
@@ -2746,10 +3121,8 @@ weeks_to_months_check (const DATAINFO *pdinfo, int *startyr, int *endyr,
     return mcount;
 }
 
-/* for now, averaging is the only compaction option in this case */
-
 static int weekly_dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
-				      CompactMethod default_method)
+				      CompactMethod method)
 {
     double **mZ = NULL;
     DATAINFO minfo;
@@ -2788,7 +3161,7 @@ static int weekly_dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
 
     /* compact series */
     if (!err && nseries > 0) {
-	err = weeks_to_months_exec(mZ, (const double **) *pZ, pdinfo, minfo.n);
+	err = weeks_to_months_exec(mZ, (const double **) *pZ, pdinfo, method);
     }
 
     if (err) {
