@@ -40,9 +40,10 @@ typedef enum {
 
 typedef float dbnumber;
 typedef struct dbwrapper_ dbwrapper;
-typedef struct _SERIESINFO SERIESINFO;
+typedef struct SERIESINFO_ SERIESINFO;
+typedef struct ODBC_info_ ODBC_info;
 
-struct _SERIESINFO {
+struct SERIESINFO_ {
     int t1, t2, v;
     char varname[VNAMELEN];
     char descrip[MAXLABEL];
@@ -61,6 +62,21 @@ struct dbwrapper_ {
     SERIESINFO *sinfo;
 };
 
+#define ODBC_MAXCOLS 3
+
+struct ODBC_info_ {
+    char *dsn;
+    char *username;
+    char *password;
+    char *query;
+    char **fmts;
+    char coltypes[ODBC_MAXCOLS];
+    double *x;
+    char **S;
+    int nrows;
+    int ncols;
+};
+
 int get_native_db_data (const char *dbbase, SERIESINFO *sinfo, 
 			double **Z);
 
@@ -90,12 +106,14 @@ int set_db_name (const char *fname, int filetype, const PATHS *ppaths,
 
 const char *get_db_name (void);
 
-int db_set_sample (const char *line, DATAINFO *pdinfo);
+int set_odbc_dsn (const char *line, PRN *prn);
 
-int db_get_series (const char *line, double ***pZ, DATAINFO *datainfo, 
-		   PRN *prn);
+int db_set_sample (const char *s, DATAINFO *pdinfo);
 
-int db_delete_series_by_name (const char *line, PRN *prn);
+int db_get_series (char *line, double ***pZ, DATAINFO *datainfo, 
+		   gretlopt opt, PRN *prn);
+
+int db_delete_series_by_name (char *line, PRN *prn);
 
 int db_delete_series_by_number (const int *list, const char *fname);
 
