@@ -90,6 +90,10 @@ double gretl_restricted_mean (int t1, int t2, const double *x,
 
 double gretl_quantile (int t1, int t2, const double *x, double p);
 
+int gretl_array_quantiles (double *a, int n, double *p, int k);
+
+double gretl_array_quantile (double *a, int n, double p);
+
 double gretl_median (int t1, int t2, const double *x);
 
 double gretl_sst (int t1, int t2, const double *x);
@@ -108,7 +112,8 @@ double gretl_restricted_stddev (int t1, int t2, const double *x,
 
 double gretl_long_run_variance (int t1, int t2, const double *x, int m);
 
-double gretl_covar (int t1, int t2, const double *x, const double *y);
+double gretl_covar (int t1, int t2, const double *x, const double *y,
+		    int *missing);
 
 double gretl_corr (int t1, int t2, const double *x, const double *y,
 		   int *missing);
@@ -169,9 +174,12 @@ int periodogram (int varno, int width,
 		 const double **Z, const DATAINFO *pdinfo, 
 		 gretlopt opt, PRN *prn);
 
-Summary *summary (const int *list, const double **Z, 
-		  const DATAINFO *pdinfo,
-		  PRN *prn);
+Summary *get_summary (const int *list, const double **Z, 
+		      const DATAINFO *pdinfo,
+		      PRN *prn, int *err);
+
+int list_summary (const int *list, const double **Z, 
+		  const DATAINFO *pdinfo, PRN *prn);
 
 void print_summary (const Summary *summ,
 		    const DATAINFO *pdinfo,
@@ -214,7 +222,7 @@ int mahalanobis_distance (const int *list, double ***pZ,
 
 MahalDist *get_mahal_distances (const int *list, double ***pZ,
 				DATAINFO *pdinfo, gretlopt opt,
-				PRN *prn);
+				PRN *prn, int *err);
 
 void free_mahal_dist (MahalDist *md);
 
@@ -229,4 +237,12 @@ double gretl_gini (int t1, int t2, const double *x);
 int gini (int vnum, const double **Z, DATAINFO *pdinfo, 
 	  gretlopt opt, PRN *prn);
 
+int shapiro_wilk (const double *x, int t1, int t2, double *W, double *pval);
+
+int gretl_normality_test (const char *param,
+			  const double **Z,
+			  const DATAINFO *pdinfo,
+			  gretlopt opt,
+			  PRN *prn);
+
 #endif /* DESCRIBE_H */
