@@ -21,6 +21,7 @@
 
 #include "libgretl.h"
 #include "libset.h"
+#include "missing_private.h"
 
 #define TINY 1.0e-13
 
@@ -398,128 +399,37 @@ static int logit_probit_vcv (MODEL *dmod, gretlopt opt, const double **Z)
     return err;
 }
 
-#if 0 /* not yet */
+/*
+  If min1 > max0, then there exists a separating hyperplane between
+  all the zeros and all the ones; in this case, the likelihood has
+  no maximum (despite having a supremum at 1) and no MLE exists.
+*/
 
-static double 
-matrix_probit_llhood (const gretl_matrix *y,
-		      const gretl_matrix *yhat)
+static int perfect_pred_check (const double *y, MODEL *dmod)
 {
-    double q, ll = 0.0;
+    double max0 = -1.0e200;
+    double min1 = 1.0e200;
+    double yht;
     int t;
 
-    for (t=0; t<y->rows; t++) {
-	q = 2.0 * y->val[t] - 1.0;
-	ll += log(normal_cdf(q * yhat->val[t]));
+    for (t=dmod->t1; t<=dmod->t2; t++) {
+	yht = dmod->yhat[t];
+	if (!na(yht)) {
+	    if (y[t] == 0 && yht > max0) {
+		max0 = yht;
+	    } 
+	    if (y[t] == 1 && yht < min1) {
+		min1 = yht;
+	    }
+	}
     }
 
-    return ll;
+    return (min1 > max0);
 }
 
-int gretl_matrix_probit (const gretl_matrix *X,
-			 const gretl_matrix *y,
-			 gretl_matrix **pb)
-{
-    gretl_matrix *bX = NULL;
-    gretl_matrix *by = NULL;
-    gretl_matrix *bi = NULL;
-    gretl_matrix *b = NULL;
-    gretl_matrix *yhat = NULL;
-    double ll, lldiff, llbak = -1.0e9;
-    double wt, xti, yt, fx, Fx;
-    double tol = 1.0e-9;
-    int nobs = X->rows;
-    int nc = X->cols;
-    int itermax = 250;
-    int i, t, iter;
-    int err = 0;
-
-    bX = gretl_matrix_alloc(nobs, nc);
-    by = gretl_column_vector_alloc(nobs);
-    bi = gretl_column_vector_alloc(nc);
-    yhat = gretl_column_vector_alloc(nobs);
-    b = gretl_column_vector_alloc(nc);
-
-    if (bX == NULL || by == NULL || bi == NULL || b == NULL) {
-	gretl_matrix_free(bX);
-	gretl_matrix_free(by);
-	gretl_matrix_free(bi);
-	gretl_matrix_free(b);
-	return E_ALLOC;
-    }
-
-    err = gretl_matrix_ols(y, X, b, NULL, NULL, NULL);
-
-    gretl_matrix_multiply(X, b, yhat);
-
-    for (iter=0; iter<itermax; iter++) {
-	/* construct BRMR dataset */
-	for (t=0; t<nobs; t++) {
-	    yt = yhat->val[t];
-	    fx = normal_pdf(yt);
-	    Fx = normal_cdf(yt);
-
-	    if (Fx < 1.0) {
-		wt = 1.0 / sqrt(Fx * (1.0 - Fx));
-	    } else {
-		wt = 0.0;
-	    }
-
-	    by->val[t] = wt * (y->val[t] - Fx);
-	    wt *= fx;
-	    for (i=0; i<nc; i++) {
-		xti = gretl_matrix_get(X, t, i);
-		gretl_matrix_set(bX, t, i, wt * xti);
-	    }
-	}
-
-	ll = matrix_probit_llhood(y, yhat);
-
-	lldiff = fabs(ll - llbak);
-	if (lldiff < tol) {
-	    break; 
-	}
-
-	llbak = ll;
-
-	err = gretl_matrix_ols(by, bX, bi, NULL, NULL, NULL);
-	if (err) {
-	    fprintf(stderr, "logit_probit: err = %d\n", err);
-	    if (iter > 0) {
-		err = E_NOCONV;
-	    }
-	    break;
-	}
-
-	/* update coefficient estimates */
-	gretl_matrix_add_to(b, bi);
-
-	/* calculate yhat */
-	gretl_matrix_multiply(X, b, yhat);
-    }
-
-    if (!err && lldiff > tol) {
-	err = E_NOCONV;
-    }
-
-    gretl_matrix_free(bX);
-    gretl_matrix_free(by);
-    gretl_matrix_free(bi);
-    gretl_matrix_free(yhat);
-
-    if (!err) {
-	*pb = b;
-    } else {
-	gretl_matrix_free(b);
-    }
-
-    return err;
-}
-
-#endif
-
 /* BRMR, Davidson and MacKinnon, ETM, p. 461 */
 
-static int do_BRMR (const int *list, MODEL *dmod, int ci,
+static int do_BRMR (const int *list, MODEL *dmod, int ci, 
 		    double *beta, const double **Z, 
 		    const DATAINFO *pdinfo, PRN *prn)
 {
@@ -547,6 +457,14 @@ static int do_BRMR (const int *list, MODEL *dmod, int ci,
     }
 
     for (iter=0; iter<itermax; iter++) {
+
+	if (perfect_pred_check(yvar, dmod)) {
+	    gretl_errmsg_sprintf("Perfect prediction detected at iteration %d;\nno MLE exists",
+				 iter);
+	    err = E_NOCONV;
+	    break;
+	}
+
 	/* construct BRMR dataset */
 	s = 0;
 	for (t=dmod->t1; t<=dmod->t2; t++) {
@@ -599,6 +517,7 @@ static int do_BRMR (const int *list, MODEL *dmod, int ci,
 	llbak = dmod->lnL;
 
 	err = gretl_matrix_ols(y, X, b, NULL, NULL, NULL);
+
 	if (err) {
 	    fprintf(stderr, "logit_probit: err = %d\n", err);
 	    if (iter > 0) {
@@ -607,6 +526,12 @@ static int do_BRMR (const int *list, MODEL *dmod, int ci,
 	    break;
 	}
 
+#if LPDEBUG > 1
+	gretl_matrix_print(y, "y, in BRMR");
+	gretl_matrix_print(X, "X, in BRMR");
+	gretl_matrix_print(b, "b, from BRMR");
+#endif
+
 	/* update coefficient estimates */
 	for (i=0; i<nc; i++) {
 	    beta[i] += b->val[i];
@@ -639,12 +564,87 @@ static int do_BRMR (const int *list, MODEL *dmod, int ci,
     return err;
 }
 
+static char *classifier_check (int *list, double **Z, DATAINFO *pdinfo,
+			       gretlopt opt, PRN *prn, int *err)
+{
+    char *mask = NULL;
+    const double *y = Z[list[1]];
+    int i, v, ni, t;
+
+    for (i=2; i<=list[0]; i++) {
+	v = list[i];
+	if (v == 0) {
+	    continue;
+	}
+	ni = gretl_isdummy(pdinfo->t1, pdinfo->t2, Z[v]);
+	if (ni > 0) {
+	    int same[2] = {0};
+	    int diff[2] = {0};
+	    int maskval, pc = 1;
+
+	    for (t=pdinfo->t1; t<=pdinfo->t2 && pc; t++) {
+		if (Z[v][t] > 0) {
+		    if (y[t] > 0) {
+			same[0] += 1;
+		    } else {
+			diff[0] += 1;
+		    }
+		} else {
+		    if (y[t] == 0) {
+			same[1] += 1;
+		    } else {
+			diff[1] += 1;
+		    }
+		}
+		if (same[0] && diff[0] && same[1] && diff[1]) {
+		    pc = 0;
+		}
+	    }
+
+	    if (pc) {
+		if (!(same[0] && diff[0])) {
+		    maskval = 1;
+		    pprintf(prn, "Note: %s != 0 predicts %s perfectly\n",
+			    pdinfo->varname[v], (same[0])? "success" : "failure");
+		} else {
+		    maskval = 0;
+		    pprintf(prn, "Note: %s == 0 predicts %s perfectly\n",
+			    pdinfo->varname[v], (diff[1])? "success" : "failure");
+		}
+
+		if (mask == NULL) {
+		    mask = malloc(pdinfo->n + 1);
+		    if (mask == NULL) {
+			*err = E_ALLOC;
+		    } else {
+			memset(mask, '0', pdinfo->n);
+			mask[pdinfo->n] = 0;
+		    }
+		}
+
+		if (mask != NULL) {
+		    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+			if (Z[v][t] == maskval) {
+			    mask[t-pdinfo->t1] = '1';
+			}
+		    }
+		    pprintf(prn, "%d observations not used\n", ni);
+		}
+		gretl_list_delete_at_pos(list, i--);
+	    }
+	}
+    }
+
+    return mask;
+}
+
 static MODEL 
-binary_logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo, 
+binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo, 
 		     int ci, gretlopt opt, PRN *prn)
 {
-    int i, t, depvar = list[1];
-    int nx = list[0] - 1;
+    int *list = NULL;
+    char *mask = NULL;
+    int i, t, depvar, nx;
     int oldt1 = pdinfo->t1;
     int oldt2 = pdinfo->t2;
     int *act_pred = NULL;
@@ -654,11 +654,9 @@ binary_logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
     double *beta = NULL;
 
     gretl_model_init(&dmod);
-    
-    /* allocate space for means of indep vars, coeffs */
-    xbar = malloc(nx * sizeof *xbar);
-    beta = malloc(nx * sizeof *beta);
-    if (xbar == NULL || beta == NULL) {
+
+    list = gretl_list_copy(inlist);
+    if (list == NULL) {
 	dmod.errcode = E_ALLOC;
 	goto bailout;
     }
@@ -674,15 +672,42 @@ binary_logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
     varlist_adjust_sample(list, &pdinfo->t1, &pdinfo->t2, 
 			  (const double **) *pZ);
 
-    dmod = lsq(list, pZ, pdinfo, OLS, OPT_A);
-    if (dmod.errcode == 0 && dmod.ncoeff != nx) {
-	dmod.errcode = E_DATA;
+    mask = classifier_check(list, *pZ, pdinfo, opt, prn,
+			    &dmod.errcode);
+    if (dmod.errcode) {
+	goto bailout;
+    }
+
+    depvar = list[1];
+    nx = list[0] - 1;
+
+    /* allocate space for means of indep vars, coeffs */
+    xbar = malloc(nx * sizeof *xbar);
+    beta = malloc(nx * sizeof *beta);
+    if (xbar == NULL || beta == NULL) {
+	dmod.errcode = E_ALLOC;
+	goto bailout;
+    }
+
+    if (mask != NULL) {
+	dmod.errcode = copy_to_reference_missmask(mask);
+    }
+
+    if (!dmod.errcode) {
+	dmod = lsq(list, pZ, pdinfo, OLS, OPT_A);
+	if (dmod.errcode == 0 && dmod.ncoeff != nx) {
+	    dmod.errcode = E_DATA;
+	}
     }
 
     if (dmod.errcode) {
 	goto bailout;
     }
 
+#if LPDEBUG
+    printmodel(&dmod, pdinfo, OPT_NONE, prn);
+#endif
+
     for (i=0; i<dmod.ncoeff; i++) {
 	beta[i] = dmod.coeff[i];
 	xbar[i] = 0.0;
@@ -694,8 +719,9 @@ binary_logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
 	xbar[i] /= dmod.nobs;
     }
 
-    dmod.errcode = do_BRMR(list, &dmod, ci, beta, (const double **) *pZ, 
-			   pdinfo, prn);
+    dmod.errcode = do_BRMR(list, &dmod, ci, beta, 
+			   (const double **) *pZ, pdinfo, 
+			   (opt & OPT_V)? prn : NULL);
     if (dmod.errcode) {
 	goto bailout;
     }
@@ -788,6 +814,8 @@ binary_logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     free(xbar);
     free(beta);
+    free(list);
+    free(mask);
 
     pdinfo->t1 = oldt1;
     pdinfo->t2 = oldt2;
@@ -800,18 +828,10 @@ ordered_model_ok (double **Z, const DATAINFO *pdinfo, int v)
 {
     if (!var_is_discrete(pdinfo, v) && 
 	!gretl_is_oprobit_ok(pdinfo->t1, pdinfo->t2, Z[v])) {
-	sprintf(gretl_errmsg, "The variable '%s' is not discrete",
+	sprintf(gretl_errmsg, _("The variable '%s' is not discrete"),
 		pdinfo->varname[v]);
 	return 0;
-    } else {
-	double m = gretl_min(pdinfo->t1, pdinfo->t2, Z[v]);
-
-	if (m != 0.0) {
-	    sprintf(gretl_errmsg, "The minimum value of '%s' is not 0",
-		    pdinfo->varname[v]);
-	    return 0;
-	}
-    }
+    } 
 
     return 1;
 }
