@@ -308,6 +308,7 @@ ldepvar_std_errors (MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
 static int compute_ar_stats (MODEL *pmod, const double **Z, double rho)
 {
     int i, t, yno = pmod->list[1];
+    int pwe = gretl_model_get_int(pmod, "pwe");
     double x, pw1 = 0.0;
 
     if (gretl_model_add_arinfo(pmod, 1)) {
@@ -315,7 +316,7 @@ static int compute_ar_stats (MODEL *pmod, const double **Z, double rho)
 	return 1;
     }
 
-    if (pmod->ci == PWE) {
+    if (pwe) {
 	pw1 = sqrt(1.0 - rho * rho);
     }
 
@@ -324,7 +325,7 @@ static int compute_ar_stats (MODEL *pmod, const double **Z, double rho)
     gretl_model_set_double(pmod, "rho_in", rho);
 
     for (t=pmod->t1; t<=pmod->t2; t++) {
-	if (t == pmod->t1 && pmod->ci == PWE) {
+	if (t == pmod->t1 && pwe) {
 	    x = pw1 * Z[yno][t];
 	    for (i=pmod->ifc; i<pmod->ncoeff; i++) {
 		x -= pmod->coeff[i] * pw1 * Z[pmod->list[i+2]][t];
@@ -696,7 +697,7 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
     int missv = 0, misst = 0;
     int jackknife = 0;
     int use_qr = libset_get_bool(USE_QR);
-    int pwe = (ci == PWE || (opt & OPT_P));
+    int pwe = (opt & OPT_P);
     int yno, i;
 
     gretl_error_clear();
@@ -716,9 +717,13 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
     gretl_model_smpl_init(&mdl, pdinfo);
     model_stats_init(&mdl);
 
-    if (pwe) {
-	gretl_model_set_int(&mdl, "pwe", 1);
-    }
+    if (ci == AR1) {
+	if (opt & OPT_P) {
+	    gretl_model_set_int(&mdl, "pwe", 1);
+	} else if (opt & OPT_H) {
+	    gretl_model_set_int(&mdl, "hilu", 1);
+	}
+    } 
 
     if (list[0] == 1 || pdinfo->v == 1) {
 	fprintf(stderr, "E_DATA: lsq: list[0] = %d, pdinfo->v = %d\n",
@@ -867,7 +872,7 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
     model_depvar_stats(&mdl, (const double **) *pZ);
 
     /* Doing an autoregressive procedure? */
-    if (ci == CORC || ci == HILU || ci == PWE) {
+    if (ci == AR1) {
 	if (compute_ar_stats(&mdl, (const double **) *pZ, rho)) { 
 	    goto lsq_abort;
 	}
@@ -876,7 +881,7 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 		goto lsq_abort;
 	    }
 	}
-	if (ci == HILU && (opt & OPT_B)) {
+	if ((opt & OPT_H) && (opt & OPT_B)) {
 	    gretl_model_set_int(&mdl, "no-corc", 1);
 	}
     }
@@ -1154,6 +1159,12 @@ static int make_ess (MODEL *pmod, const double **Z)
 
 #define SMALLDIFF 9.0e-16
 
+/* The heuristic used here is that the model effectively
+   contains a constant or intercept if the means of y and
+   yhat are the same, where "the same" means that the
+   relative difference is less than SMALLDIFF.
+*/
+
 int check_for_effective_const (MODEL *pmod, const double *y)
 {
     double x1 = 0.0, x2 = 0.0;
@@ -1168,6 +1179,9 @@ int check_for_effective_const (MODEL *pmod, const double *y)
     }
 
     reldiff = fabs((x1 - x2) / x2);
+#if 0
+    fprintf(stderr, "check_for_effective_const: reldiff = %g\n", reldiff);
+#endif
 
     if (reldiff < SMALLDIFF) {
 	gretl_model_set_int(pmod, "effconst", 1);
@@ -1515,19 +1529,22 @@ static void diaginv (double *xpx, double *xpy, double *diag, int nv)
 int makevcv (MODEL *pmod, double sigma)
 {
     int dec, mst, kk, i, j, kj, icnt, m, k, l = 0;
-    const int nv = pmod->ncoeff;
-    const int nxpx = (nv * nv + nv) / 2; 
+    int nv, nxpx;
     double d;
 
     if (pmod->vcv != NULL) {
+	/* already done */
 	return 0;
     }
 
     if (pmod->xpx == NULL) {
+	/* raw material not available */
 	fprintf(stderr, "makevcv: pmod->xpx = NULL\n");
-	return E_DATA;
+	return E_BADSTAT;
     }
 
+    nv = pmod->ncoeff;
+    nxpx = (nv * nv + nv) / 2; 
     mst = nxpx;
     kk = nxpx - 1;
 
@@ -1787,13 +1804,14 @@ static int hilu_plot (double *ssr, double *rho, int n)
     return 0;
 }
 
-static double autores (MODEL *pmod, const double **Z, int ci)
+static double autores (MODEL *pmod, const double **Z, gretlopt opt)
 {
     int t, v, t1 = pmod->t1;
     double x, num = 0.0, den = 0.0;
     double rhohat;
 
-    if (ci == CORC || ci == HILU) {
+    if (!(opt & OPT_P)) {
+	/* not using Prais-Winsten */
 	t1--;
     }
 
@@ -1821,13 +1839,12 @@ static double autores (MODEL *pmod, const double **Z, int ci)
  * @list: dependent variable plus list of regressors.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
- * @ci: %CORC for Cochrane-Orcutt, %HILU for Hildreth-Lu,
- *      %PWE for Prais-Winsten estimator.
- * @err: pointer for error code.
- * @opt: option flags: may include %OPT_B to suppress Cochrane-Orcutt
- *       fine-tuning of Hildreth-Lu results, %OPT_P to generate
- *       a gnuplot graph of the search in case @ci = %HILU.
+ * @opt: option flags: may include %OPT_H to use Hildreth-Lu,
+ *       %OPT_P to use Prais-Winsten, %OPT_B to suppress Cochrane-Orcutt
+ *       fine-tuning of Hildreth-Lu results, %OPT_G to generate
+ *       a gnuplot graph of the search in Hildreth-Lu case.
  * @prn: gretl printing struct.
+ * @err: location to receeve error code.
  *
  * Estimate the quasi-differencing coefficient for use with the
  * Cochrane-Orcutt, Hildreth-Lu or Prais-Winsten procedures for
@@ -1838,7 +1855,7 @@ static double autores (MODEL *pmod, const double **Z, int ci)
  */
 
 double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
-		     GretlCmdIndex ci, int *err, gretlopt opt, PRN *prn)
+		     gretlopt opt, PRN *prn, int *err)
 {
     double rho = 0.0, rho0 = 0.0, diff;
     double finalrho = 0.0, essmin = 1.0e8;
@@ -1848,8 +1865,8 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
     int missv = 0, misst = 0;
     gretlopt lsqopt = OPT_A;
     int quiet = (opt & OPT_Q);
-    int ascii = !(opt & OPT_P);
-    MODEL corc_model;
+    int ascii = !(opt & OPT_G);
+    MODEL armod;
 
     gretl_error_clear();
     *err = 0;
@@ -1863,23 +1880,23 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 	goto bailout;
     }
 
-    gretl_model_init(&corc_model);
+    gretl_model_init(&armod);
 
-    if (ci == PWE) {
+    if (opt & OPT_P) {
 	/* Prais-Winsten treatment of first observation */
 	lsqopt |= OPT_P;
     } 
 
-    if (ci == HILU) { 
+    if (opt & OPT_H) { 
 	/* Do Hildreth-Lu first */
 	for (rho = -0.990, iter = 0; rho < 1.0; rho += .01, iter++) {
-	    clear_model(&corc_model);
-	    corc_model = ar1_lsq(list, pZ, pdinfo, OLS, OPT_A, rho);
-	    if ((*err = corc_model.errcode)) {
-		clear_model(&corc_model);
+	    clear_model(&armod);
+	    armod = ar1_lsq(list, pZ, pdinfo, OLS, OPT_A, rho);
+	    if ((*err = armod.errcode)) {
+		clear_model(&armod);
 		goto bailout;
 	    }
-	    ess = corc_model.ess;
+	    ess = armod.ess;
 	    if (ascii && !quiet) {
 		char num[16];
 		int chk;
@@ -1914,13 +1931,13 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 	if (finalrho > 0.989) {
 	    /* try exploring this funny region? */
 	    for (rho = 0.99; rho <= 0.999; rho += .001) {
-		clear_model(&corc_model);
-		corc_model = ar1_lsq(list, pZ, pdinfo, OLS, OPT_A, rho);
-		if ((*err = corc_model.errcode)) {
-		    clear_model(&corc_model);
+		clear_model(&armod);
+		armod = ar1_lsq(list, pZ, pdinfo, OLS, OPT_A, rho);
+		if ((*err = armod.errcode)) {
+		    clear_model(&armod);
 		    goto bailout;
 		}
-		ess = corc_model.ess;
+		ess = armod.ess;
 		if (ess < essmin) {
 		    essmin = ess;
 		    finalrho = rho;
@@ -1931,13 +1948,13 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 	if (finalrho > 0.9989) {
 	    /* this even funnier one? */
 	    for (rho = 0.9991; rho <= 0.9999; rho += .0001) {
-		clear_model(&corc_model);
-		corc_model = ar1_lsq(list, pZ, pdinfo, OLS, OPT_A, rho);
-		if ((*err = corc_model.errcode)) {
-		    clear_model(&corc_model);
+		clear_model(&armod);
+		armod = ar1_lsq(list, pZ, pdinfo, OLS, OPT_A, rho);
+		if ((*err = armod.errcode)) {
+		    clear_model(&armod);
 		    goto bailout;
 		}
-		ess = corc_model.ess;
+		ess = armod.ess;
 		if (ess < essmin) {
 		    essmin = ess;
 		    finalrho = rho;
@@ -1957,27 +1974,27 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 	}
     } else { 
 	/* Go straight to Cochrane-Orcutt (or Prais-Winsten) */
-	corc_model = lsq(list, pZ, pdinfo, OLS, OPT_A);
-	if (!corc_model.errcode && corc_model.dfd == 0) {
-	    corc_model.errcode = E_DF;
+	armod = lsq(list, pZ, pdinfo, OLS, OPT_A);
+	if (!armod.errcode && armod.dfd == 0) {
+	    armod.errcode = E_DF;
 	}
-	if ((*err = corc_model.errcode)) {
-	    clear_model(&corc_model);
+	if ((*err = armod.errcode)) {
+	    clear_model(&armod);
 	    goto bailout;
 	}
-	rho0 = rho = corc_model.rho;
+	rho0 = rho = armod.rho;
     }
 
     if (na(rho)) {
 	*err = E_NOCONV;
-	clear_model(&corc_model);
+	clear_model(&armod);
 	goto bailout;
     }
 
-    if (ci != HILU || !(opt & OPT_B)) {
+    if (!(opt & OPT_H) || !(opt & OPT_B)) {
 
 	if (!quiet) {
-	    if (ci == HILU) {
+	    if (opt & OPT_H) {
 		pputs(prn, _("\nFine-tune rho using the CORC procedure...\n\n"));
 	    } else {
 		pputs(prn, _("\nPerforming iterative calculation of rho...\n\n"));
@@ -1991,36 +2008,34 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 	diff = 1.0;
 
 	while (diff > 0.001) {
-	    if (!quiet) {
-		pprintf(prn, "          %10d %12.5f", ++iter, rho);
-	    }
-	    clear_model(&corc_model);
-	    corc_model = ar1_lsq(list, pZ, pdinfo, OLS, lsqopt, rho);
+	    clear_model(&armod);
+	    armod = ar1_lsq(list, pZ, pdinfo, OLS, lsqopt, rho);
 #if AR_DEBUG
-	    fprintf(stderr, "corc_model: t1=%d, first two uhats: %g, %g\n",
-		    corc_model.t1, 
-		    corc_model.uhat[corc_model.t1],
-		    corc_model.uhat[corc_model.t1+1]);
+	    fprintf(stderr, "armod: t1=%d, first two uhats: %g, %g\n",
+		    armod.t1, 
+		    armod.uhat[armod.t1],
+		    armod.uhat[armod.t1+1]);
 #endif
-	    if ((*err = corc_model.errcode)) {
-		clear_model(&corc_model);
+	    if ((*err = armod.errcode)) {
+		clear_model(&armod);
 		goto bailout;
 	    }
 	    if (!quiet) {
-		pprintf(prn, "   %g\n", corc_model.ess);
+		pprintf(prn, "          %10d %12.5f", ++iter, rho);
+		pprintf(prn, "   %g\n", armod.ess);
 	    }
 
-	    rho = autores(&corc_model, (const double **) *pZ, ci);
+	    rho = autores(&armod, (const double **) *pZ, opt);
 
 #if AR_DEBUG
-	    pputs(prn, "CORC model (using rho-transformed data)\n");
-	    printmodel(&corc_model, pdinfo, OPT_NONE, prn);
+	    pputs(prn, "AR1 model (using rho-transformed data)\n");
+	    printmodel(&armod, pdinfo, OPT_NONE, prn);
 	    pprintf(prn, "autores gives rho = %g\n", rho);
 #endif
 
 	    if (rho > .99999 || rho < -.99999) {
 		*err = E_NOCONV;
-		clear_model(&corc_model);
+		clear_model(&armod);
 		goto bailout;
 	    }
 
@@ -2034,7 +2049,7 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    clear_model(&corc_model);
+    clear_model(&armod);
 
  bailout:
 
@@ -2122,6 +2137,8 @@ int *augment_regression_list (const int *orig, int aux,
 		    }
 		    vnew = xpxgenr(vi, vj, pZ, pdinfo);
 		    if (vnew > 0) {
+			/* ensure uniqueness of generated varnames */
+			sprintf(pdinfo->varname[vnew], "X%d_X%d", i-1, j-1);
 			list[++k] = vnew;
 		    }
 		}
@@ -2139,6 +2156,39 @@ int *augment_regression_list (const int *orig, int aux,
     return list;
 }
 
+/* For observation s, see if the regression list contains a variable
+   that has a single non-zero value at that particular observation.
+   We run this check on observations showing an OLS residual of zero.
+*/
+
+static int observation_is_dummied (const MODEL *pmod,
+				   int *list, const double **Z,
+				   int s)
+{
+    int i, t, v;
+    int ret = 0;
+
+    for (i=list[0]; i>=2; i--) {
+	v = list[i];
+	if (v == 0) {
+	    continue;
+	}
+	ret = 1;
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    if ((t == s && Z[v][t] == 0.0) || (t != s && Z[v][t] != 0.0)) {
+		ret = 0;
+		break;
+	    }
+	}
+	if (ret) {
+	    gretl_list_delete_at_pos(list, i);
+	    break;
+	}
+    }
+
+    return ret;
+}
+
 /* get_hsk_weights: take the residuals from the model pmod, square them
    and take logs; find the fitted values for this series using an
    auxiliary regression including the original independent variables
@@ -2150,13 +2200,20 @@ static int get_hsk_weights (MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
 {
     int oldv = pdinfo->v;
     int t, t1 = pdinfo->t1, t2 = pdinfo->t2;
+    int *lcpy = NULL;
     int *list = NULL;
     int err = 0, shrink = 0;
     double xx;
     MODEL aux;
 
+    lcpy = gretl_list_copy(pmod->list);
+    if (lcpy == NULL) {
+	return E_ALLOC;
+    }
+
     /* allocate space for an additional variable */
     if (dataset_add_series(1, pZ, pdinfo)) {
+	free(lcpy);
 	return E_ALLOC;
     }
 
@@ -2165,6 +2222,13 @@ static int get_hsk_weights (MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
 	xx = pmod->uhat[t];
 	if (na(xx)) {
 	    (*pZ)[oldv][t] = NADBL;
+	} else if (xx == 0.0) {
+	    if (observation_is_dummied(pmod, lcpy, (const double **) *pZ, t)) {
+		(*pZ)[oldv][t] = NADBL;
+	    } else {
+		fprintf(stderr, "hsk: got a zero residual, could be a problem!\n");
+		(*pZ)[oldv][t] = -1.0e16; /* ?? */
+	    }
 	} else {
 	    (*pZ)[oldv][t] = log(xx * xx);
 	}
@@ -2172,12 +2236,12 @@ static int get_hsk_weights (MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
 
     /* build regression list, adding the squares of the original
        independent vars */
-    list = augment_regression_list(pmod->list, AUX_SQ, pZ, pdinfo);
+    list = augment_regression_list(lcpy, AUX_SQ, pZ, pdinfo);
     if (list == NULL) {
 	return E_ALLOC;
     }
 
-    list[1] = oldv; /* the newly added uhat-squared */
+    list[1] = oldv; /* the newly added log(uhat-squared) */
 
     pdinfo->t1 = pmod->t1;
     pdinfo->t2 = pmod->t2;
@@ -2209,6 +2273,7 @@ static int get_hsk_weights (MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
     }
 
     free(list);
+    free(lcpy);
 
     return err;
 }
@@ -2450,13 +2515,27 @@ static void print_HET_1 (double z, double pval, PRN *prn)
 	    _("with p-value"), z, pval);
 }
 
-static void print_whites_test (double TR2, int df, double pval, PRN *prn)
+static void print_whites_test (double LM, int df, double pval, 
+			       gretlopt opt, PRN *prn)
 {
-    pprintf(prn, "\n%s\n", _("White's test for heteroskedasticity"));
-    pprintf(prn, "\n%s: TR^2 = %f,\n", _("Test statistic"), TR2);
+    if (opt & OPT_B) {
+	pprintf(prn, "\n%s\n", _("Breusch-Pagan test for heteroskedasticity"));
+	if (opt & OPT_R) {
+	    pprintf(prn, "%s\n", _("with Koenker robust variance estimator"));
+	}
+	pprintf(prn, "\n%s: LM = %f,\n", _("Test statistic"), LM);
+    } else {
+	if (opt & OPT_X) {
+	    pprintf(prn, "\n%s\n", _("White's test for heteroskedasticity (no cross products)"));
+	} else {
+	    pprintf(prn, "\n%s\n", _("White's test for heteroskedasticity"));
+	}
+	pprintf(prn, "\n%s: TR^2 = %f,\n", _("Test statistic"), LM);
+    }
+
     pprintf(prn, "%s = P(%s(%d) > %f) = %f\n\n", 
 	    _("with p-value"), _("Chi-square"), 
-	    df, TR2, pval); 
+	    df, LM, pval); 
 }
 
 /* For White's test, see if we have enough degrees of freedom
@@ -2606,7 +2685,7 @@ static int tsls_hetero_test (MODEL *pmod, double ***pZ,
 	    }	  
 	}
 
-	record_test_result(x, pval, _("HET_1"));
+	record_test_result(z, pval, _("HET_1"));
     }
 
     clear_model(&ptmod);
@@ -2624,13 +2703,71 @@ static int tsls_hetero_test (MODEL *pmod, double ***pZ,
     return err;
 }
 
+/* Compute LM statistic as per Breusch and Pagan (Econometrica, 1979),
+   with the option to use the robust variance estimator proposed by
+   Koenker (Journal of Econometrics, 1981).
+*/
+
+static double get_BP_LM (MODEL *pmod, int *list, MODEL *aux,
+			 double ***pZ, DATAINFO *pdinfo,
+			 gretlopt opt, int *err)
+{
+    double s2, u2t, gt;
+    double V = 0.0, LM = NADBL;
+    int t, v = list[1];
+
+    s2 = pmod->ess / pmod->nobs;
+
+    if (opt & OPT_R) {
+	/* calculate robust variance estimate a la Koenker */
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    u2t = pmod->uhat[t] * pmod->uhat[t];
+	    V += (u2t - s2) * (u2t - s2);
+	}
+	V /= pmod->nobs;
+    }
+
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	u2t = pmod->uhat[t] * pmod->uhat[t];
+	if (opt & OPT_R) {
+	    gt = u2t - s2;
+	} else {
+	    gt = u2t / s2;
+	}
+	(*pZ)[v][t] = gt;
+    }
+
+    *aux = lsq(list, pZ, pdinfo, OLS, OPT_A);
+    *err = aux->errcode;
+
+    if (!*err) {
+	double RSS = aux->tss - aux->ess;
+
+	if (RSS < 0) {
+	    *err = E_DATA;
+	} else {
+	    if (opt & OPT_R) {
+		gretl_model_set_int(aux, "robust", 1);
+		LM = RSS / V;
+	    } else {
+		LM = .5 * RSS;
+	    }
+	    gretl_model_set_double(aux, "BPLM", LM);
+	    aux->aux = AUX_BP;
+	}
+    }
+
+    return LM;
+}
+
 /**
  * whites_test:
  * @pmod: pointer to model.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
  * @opt: if flags include %OPT_S, save results to model; %OPT_Q
- * means don't print the auxiliary regression.
+ * means don't print the auxiliary regression;  %OPT_B means
+ * do the simpler Breusch-Pagan variant.
  * @prn: gretl printing struct.
  *
  * Runs White's test for heteroskedasticity on the given model.
@@ -2641,10 +2778,12 @@ static int tsls_hetero_test (MODEL *pmod, double ***pZ,
 int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
 		 gretlopt opt, PRN *prn)
 {
-    int aux, lo, ncoeff, yno, t;
+    int lo, ncoeff, yno, t;
+    int BP = (opt & OPT_B);
+    int aux = AUX_NONE;
     int v = pdinfo->v;
     int *list = NULL;
-    double zz;
+    double zz, LM;
     MODEL white;
     int err = 0;
 
@@ -2667,10 +2806,16 @@ int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* what can we do, with the degrees of freedom available? */
-    aux = get_whites_aux(pmod, (const double **) *pZ);
-    if (aux == AUX_NONE) {
-	return E_DF;
-    } 
+    if (!BP) {
+	if (opt & OPT_X) { 
+	    aux = AUX_SQ;
+	} else {
+	    aux = get_whites_aux(pmod, (const double **) *pZ);
+	    if (aux == AUX_NONE) {
+		return E_DF;
+	    }
+	}
+    }
 
     gretl_model_init(&white);
 
@@ -2697,10 +2842,14 @@ int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (!err) {
-	/* build aux regression list, adding squares and
-	   (possibly) cross-products of the original 
-	   independent vars */
-	list = augment_regression_list(pmod->list, aux, pZ, pdinfo);
+	if (BP) {
+	    list = gretl_list_copy(pmod->list);
+	} else {
+	    /* build aux regression list, adding squares and
+	       (possibly) cross-products of the original 
+	       independent vars */
+	    list = augment_regression_list(pmod->list, aux, pZ, pdinfo);
+	}
 	if (list == NULL) {
 	    err = E_ALLOC;
 	} else {
@@ -2710,35 +2859,50 @@ int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 
     if (!err) {
 	/* run auxiliary regression */
-	white = lsq(list, pZ, pdinfo, OLS, OPT_A);
-	err = white.errcode;
+	if (BP) {
+	    LM = get_BP_LM(pmod, list, &white, pZ, pdinfo, opt, &err);
+	} else {
+	    int qrbak = libset_get_bool(USE_QR);
+
+	    libset_set_bool(USE_QR, 1);
+	    white = lsq(list, pZ, pdinfo, OLS, OPT_A);
+	    libset_set_bool(USE_QR, qrbak);
+	    err = white.errcode;
+	    if (!err) {
+		LM = white.rsq * white.nobs;
+		white.aux = AUX_WHITE;
+	    }
+	}
     }
 
     if (!err) {
-	double TR2 = white.rsq * white.nobs;
 	int df = white.ncoeff - 1;
-	double pval = chisq_cdf_comp(TR2, df);
+	double pval = chisq_cdf_comp(df, LM);
 
 	if (opt & OPT_Q) {
-	    print_whites_test(TR2, df, pval, prn);
+	    print_whites_test(LM, df, pval, opt, prn);
 	} else {
-	    white.aux = AUX_WHITE;
 	    printmodel(&white, pdinfo, OPT_NONE, prn);
 	}
 
 	if (opt & OPT_S) {
-	    ModelTest *test = model_test_new(GRETL_TEST_WHITES);
-
+	    ModelTestType mt = BP? GRETL_TEST_BP : GRETL_TEST_WHITES;
+	    ModelTest *test = model_test_new(mt);
+	    
 	    if (test != NULL) {
-		model_test_set_teststat(test, GRETL_STAT_TR2);
+		model_test_set_teststat(test, GRETL_STAT_LM);
 		model_test_set_dfn(test, df);
-		model_test_set_value(test, TR2);
+		model_test_set_value(test, LM);
 		model_test_set_pvalue(test, pval);
 		maybe_add_test_to_model(pmod, test);
 	    }	  
 	}
 
-	record_test_result(TR2, pval, _("White's"));
+	if (BP) {
+	    record_test_result(LM, pval, "Breusch-Pagan");
+	} else {
+	    record_test_result(LM, pval, _("White's"));
+	}
     }
 
     clear_model(&white);
@@ -2821,14 +2985,13 @@ MODEL ar_func (const int *list, double ***pZ,
 
     cpos = reglist_check_for_const(reglist, (const double **) *pZ, pdinfo);
 
-    /* special case: ar 1 ; ... => use CORC */
+    /* special case: ar 1 ; ... => use AR1 */
     if (arlist[0] == 1 && arlist[1] == 1) {
-	xx = estimate_rho(reglist, pZ, pdinfo, CORC, &err, OPT_NONE, prn);
+	xx = estimate_rho(reglist, pZ, pdinfo, OPT_NONE, prn, &err);
 	if (err) {
 	    ar.errcode = err;
 	} else {
-	    ar = ar1_lsq(reglist, pZ, pdinfo, CORC, OPT_NONE, xx);
-	    printmodel(&ar, pdinfo, opt, prn); 
+	    ar = ar1_lsq(reglist, pZ, pdinfo, AR1, OPT_NONE, xx);
 	}
 	goto bailout;
     }
@@ -3036,21 +3199,21 @@ static void omitzero (MODEL *pmod, const double **Z, const DATAINFO *pdinfo,
 	zlist = gretl_null_list();
     }
 
-    for (i=offset; i<=pmod->list[0]; i++) {
+    for (i=pmod->list[0]; i>=offset; i--) {
         v = pmod->list[i];
         if (modelvar_iszero(pmod, Z[v])) {
 	    if (zlist != NULL) {
 		gretl_list_append_term(&zlist, v);
 	    }
-	    fprintf(stderr, "Deleting var at pos %d, all zero\n", i);
-	    gretl_list_delete_at_pos(pmod->list, i--);
+	    fprintf(stderr, "Deleting var %d at list pos %d: all zero\n", v, i);
+	    gretl_list_delete_at_pos(pmod->list, i);
 	}
     }
 
     if (pmod->nwt) {
 	int t, wtzero;
 
-	for (i=offset; i<=pmod->list[0]; i++) {
+	for (i=pmod->list[0]; i>=offset; i--) {
 	    v = pmod->list[i];
 	    wtzero = 1;
 	    for (t=pmod->t1; t<=pmod->t2; t++) {
@@ -3064,7 +3227,7 @@ static void omitzero (MODEL *pmod, const double **Z, const DATAINFO *pdinfo,
 		if (zlist != NULL) {
 		    gretl_list_append_term(&zlist, v);
 		}
-		gretl_list_delete_at_pos(pmod->list, i--);
+		gretl_list_delete_at_pos(pmod->list, i);
 	    }
 	}
     }
@@ -3162,7 +3325,7 @@ arch_test_save_or_print (const gretl_matrix *b, const gretl_matrix *V,
 {
     ModelTest *test = model_test_new(GRETL_TEST_ARCH);
     double LM = T * rsq;
-    double pv = chisq_cdf_comp(LM, order);
+    double pv = chisq_cdf_comp(order, LM);
 
     if (V != NULL) {
 	int i, k = order + 1;
@@ -3178,7 +3341,7 @@ arch_test_save_or_print (const gretl_matrix *b, const gretl_matrix *V,
     }
 
     if (test != NULL) {
-	model_test_set_teststat(test, GRETL_STAT_TR2);
+	model_test_set_teststat(test, GRETL_STAT_LM);
 	model_test_set_order(test, order);
 	model_test_set_dfn(test, order);
 	model_test_set_value(test, LM);
@@ -3192,7 +3355,7 @@ arch_test_save_or_print (const gretl_matrix *b, const gretl_matrix *V,
 	    gretl_model_test_print_direct(test, heading, prn);
 	}
 
-	if (opt & OPT_S) {
+	if (pmod != NULL && (opt & OPT_S)) {
 	    maybe_add_test_to_model(pmod, test);
 	} else {
 	    model_test_free(test);
@@ -3202,45 +3365,22 @@ arch_test_save_or_print (const gretl_matrix *b, const gretl_matrix *V,
     record_test_result(LM, pv, "ARCH");
 }
 
-/**
- * arch_test:
- * @pmod: model to be tested.
- * @order: lag order for ARCH process.
- * @pdinfo: information on the data set.
- * @opt: if flags include %OPT_S, save test results to model;
- * if %OPT_Q, be less verbose.
- * @prn: gretl printing struct.
- *
- * Tests @pmod for AutoRegressive Conditional Heteroskedasticity.  
- * 
- * Returns: 0 on success, non-zero code on error.
- */
-
-int arch_test (MODEL *pmod, int order, const DATAINFO *pdinfo, 
-	       gretlopt opt, PRN *prn)
+static int real_arch_test (const double *u, int T, int order, 
+			   MODEL *pmod, const DATAINFO *pdinfo, 
+			   gretlopt opt, PRN *prn)
 {
     gretl_matrix *X = NULL;
     gretl_matrix *y = NULL;
     gretl_matrix *b = NULL;
     gretl_matrix *V = NULL;
-    int T = pmod->nobs;
     int i, k, s, t;
     double x, s2, rsq;
     double *ps2 = NULL;
     int err = 0;
 
-    if (pmod->missmask != NULL) {
-	return E_MISSDATA;
-    }
-
     gretl_error_clear();
 
-    if (order == 0) {
-	/* use data frequency as default lag order */
-	order = pdinfo->pd;
-    }
-
-    if (order < 1 || order > T - pmod->list[0]) {
+    if (order < 1 || order > T - 1) {
 	sprintf(gretl_errmsg, _("Invalid lag order for arch (%d)"), order);
 	return E_DATA;
     }
@@ -3273,13 +3413,13 @@ int arch_test (MODEL *pmod, int order, const DATAINFO *pdinfo,
 
     for (i=0; i<k; i++) {
 	for (t=0; t<T; t++) {
-	    s = t + pmod->t1 + order;
+	    s = t + order;
 	    if (i == 0) {
-		x = pmod->uhat[s];
+		x = u[s];
 		gretl_vector_set(y, t, x * x);
 		gretl_matrix_set(X, t, i, 1.0);
 	    } else {
-		x = pmod->uhat[s - i];
+		x = u[s - i];
 		gretl_matrix_set(X, t, i, x * x);
 	    }
 	}
@@ -3306,6 +3446,48 @@ int arch_test (MODEL *pmod, int order, const DATAINFO *pdinfo,
     return err;
 }
 
+/**
+ * arch_test:
+ * @pmod: model to be tested.
+ * @order: lag order for ARCH process.
+ * @pdinfo: information on the data set.
+ * @opt: if flags include %OPT_S, save test results to model;
+ * if %OPT_Q, be less verbose.
+ * @prn: gretl printing struct.
+ *
+ * Tests @pmod for AutoRegressive Conditional Heteroskedasticity.  
+ * 
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int arch_test (MODEL *pmod, int order, const DATAINFO *pdinfo, 
+	       gretlopt opt, PRN *prn)
+{
+    int err;
+
+    if (pmod->missmask != NULL) {
+	err = E_MISSDATA;
+    } else {
+	const double *u = pmod->uhat + pmod->t1;
+
+	if (order == 0) {
+	    /* use data frequency as default lag order */
+	    order = pdinfo->pd;
+	}
+
+	err = real_arch_test(u, pmod->nobs, order, pmod, pdinfo, 
+			     opt, prn);
+    }
+
+    return err;
+}
+
+int array_arch_test (const double *u, int n, int order, 
+		     gretlopt opt, PRN *prn)
+{
+    return real_arch_test(u, n, order, NULL, NULL, opt, prn);
+}
+
 /**
  * arch_model:
  * @list: dependent variable plus list of regressors.
@@ -3470,7 +3652,7 @@ MODEL arch_model (const int *list, int order, double ***pZ, DATAINFO *pdinfo,
 
 MODEL lad (const int *list, double ***pZ, DATAINFO *pdinfo)
 {
-    MODEL lad_model;
+    MODEL lmod;
     void *handle;
     int (*lad_driver) (MODEL *, double **, DATAINFO *);
 
@@ -3478,26 +3660,87 @@ MODEL lad (const int *list, double ***pZ, DATAINFO *pdinfo)
        the lad_driver function will overwrite the coefficients etc.
     */
 
-    lad_model = lsq(list, pZ, pdinfo, OLS, OPT_A);
+    lmod = lsq(list, pZ, pdinfo, OLS, OPT_A);
 
-    if (lad_model.errcode) {
-        return lad_model;
+    if (lmod.errcode) {
+        return lmod;
     }
 
     lad_driver = get_plugin_function("lad_driver", &handle);
 
     if (lad_driver == NULL) {
 	fprintf(stderr, I_("Couldn't load plugin function\n"));
-	lad_model.errcode = E_FOPEN;
-	return lad_model;
+	lmod.errcode = E_FOPEN;
+	return lmod;
     }
 
-    (*lad_driver) (&lad_model, *pZ, pdinfo);
+    (*lad_driver) (&lmod, *pZ, pdinfo);
     close_plugin(handle);
 
-    set_model_id(&lad_model);
+    if (lmod.errcode == 0) {
+	set_model_id(&lmod);
+    }
 
-    return lad_model;
+    return lmod;
+}
+
+/**
+ * quantreg:
+ * @parm: 
+ * @list: model specification: dependent var and regressors.
+ * @pZ: pointer to data array.
+ * @pdinfo: information on the data set.
+ * @opt:
+ * @prn:
+ *
+ * Estimate the model given in @list using the method of 
+ * quantile regression.
+ * 
+ * Returns: a #MODEL struct, containing the estimates.
+ */
+
+MODEL quantreg (const char *parm, const int *list, 
+		double ***pZ, DATAINFO *pdinfo,
+		gretlopt opt, PRN *prn)
+{
+    MODEL qmod;
+    void *handle;
+    int (*rq_driver) (const char *, MODEL *, double ***, DATAINFO *,
+		      gretlopt, PRN *);
+    gretlopt olsopt = (OPT_A | OPT_M);
+
+    /* Run an initial OLS to "set the model up" and check for errors.
+       the driver function will overwrite the coefficients, etc.
+       For now we make life easier by rejecting within-sample missing
+       values (OPT_M).
+    */
+
+    if (opt & OPT_R) {
+	olsopt |= OPT_R;
+    }
+
+    qmod = lsq(list, pZ, pdinfo, OLS, olsopt);
+
+    if (qmod.errcode) {
+        return qmod;
+    }
+
+    rq_driver = get_plugin_function("rq_driver", &handle);
+
+    if (rq_driver == NULL) {
+	fprintf(stderr, I_("Couldn't load plugin function\n"));
+	qmod.errcode = E_FOPEN;
+	return qmod;
+    }
+
+    (*rq_driver) (parm, &qmod, pZ, pdinfo, opt, prn);
+    close_plugin(handle);
+
+    if (qmod.errcode == 0) {
+	set_model_id(&qmod);
+    }
+
+    return qmod;
 }
 
 /**
@@ -3589,7 +3832,6 @@ MODEL tobit_model (const int *list, double ***pZ, DATAINFO *pdinfo, PRN *prn)
     tmod = (*tobit_estimate) (list, pZ, pdinfo, prn);
 
     close_plugin(handle);
-
     set_model_id(&tmod);
 
     return tmod;
@@ -3773,8 +4015,9 @@ MODEL garch (const int *list, double ***pZ, DATAINFO *pdinfo, gretlopt opt,
 MODEL mp_ols (const int *list, const double **Z, DATAINFO *pdinfo)
 {
     void *handle = NULL;
-    int (*mplsq)(const int *, const int *, const double **, 
-		 DATAINFO *, char *, MODEL *, gretlopt);
+    int (*mplsq)(const int *, const int *, const int *, 
+		 const double **, DATAINFO *, char *, 
+		 MODEL *, gretlopt);
     MODEL mpmod;
 
     gretl_model_init(&mpmod);
@@ -3789,17 +4032,17 @@ MODEL mp_ols (const int *list, const double **Z, DATAINFO *pdinfo)
 	int *base = NULL;
 	int *poly = NULL;
 
-	gretl_list_split_on_separator(list, &base, &poly);
-	if (base == NULL || poly == NULL) {
-	    mpmod.errcode = E_ALLOC;
+	mpmod.errcode = gretl_list_split_on_separator(list, &base, &poly);
+	if (mpmod.errcode == 0 && (base == NULL || poly == NULL)) {
+	    mpmod.errcode = E_ARGS;
 	} else {
-	    mpmod.errcode = (*mplsq)(base, poly, Z, pdinfo,  
+	    mpmod.errcode = (*mplsq)(base, poly, NULL, Z, pdinfo,  
 				     gretl_errmsg, &mpmod, OPT_S);
 	}
 	free(base);
 	free(poly);
     } else {
-	mpmod.errcode = (*mplsq)(list, NULL, Z, pdinfo,  
+	mpmod.errcode = (*mplsq)(list, NULL, NULL, Z, pdinfo,  
 				 gretl_errmsg, &mpmod, OPT_S); 
     }
 
