@@ -29,10 +29,14 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 	    GretlCmdIndex ci, gretlopt opt, double rho);
 
 double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
-		     GretlCmdIndex ci, int *err, gretlopt opt, PRN *prn);
+		     gretlopt opt, PRN *prn, int *err);
 
 MODEL lad (const int *list, double ***pZ, DATAINFO *pdinfo); 
 
+MODEL quantreg (const char *parm, const int *list, 
+		double ***pZ, DATAINFO *pdinfo,
+		gretlopt opt, PRN *prn);
+
 MODEL arma (const int *list, const char *pqspec,
 	    const double **Z, const DATAINFO *pdinfo, 
 	    gretlopt opt, PRN *prn);
@@ -44,7 +48,7 @@ MODEL poisson_model (const int *list, double ***pZ, DATAINFO *pdinfo,
 		     PRN *prn);
 
 MODEL heckit_model (const int *list, double ***pZ, DATAINFO *pdinfo, 
-		    gretlopt, PRN *prn);
+		    gretlopt opt, PRN *prn);
 
 MODEL garch (const int *list, double ***pZ, DATAINFO *pdinfo, gretlopt opt,
 	     PRN *prn);
@@ -74,6 +78,9 @@ MODEL ar_func (const int *list,
 int arch_test (MODEL *pmod, int order, const DATAINFO *pdinfo, 
 	       gretlopt opt, PRN *prn);
 
+int array_arch_test (const double *u, int n, int order, 
+		     gretlopt opt, PRN *prn);
+
 MODEL arch_model (const int *list, int order, 
 		  double ***pZ, DATAINFO *pdinfo, 
 		  gretlopt opt, PRN *prn);
