@@ -29,7 +29,7 @@ typedef enum {
 
 struct FITRESID_ {
     int model_ID;
-    int model_ci;
+    int asymp;
     int model_t1;
     int method;
     double *actual;
@@ -42,38 +42,38 @@ struct FITRESID_ {
     int df;
     int t0, t1, t2;
     int nobs;
-    int err;
     char depvar[VNAMELEN];
 };
 
 void free_fit_resid (FITRESID *fr);
 
 FITRESID *get_fit_resid (const MODEL *pmod, const double **Z, 
-			 const DATAINFO *pdinfo);
+			 const DATAINFO *pdinfo, int *err);
 
-FITRESID *get_forecast (MODEL *pmod, int t0, int t1, int t2,
+FITRESID *get_forecast (MODEL *pmod, int t1, int t2, int pre_n,
 			double ***pZ, DATAINFO *pdinfo,
-			gretlopt opt);
+			gretlopt opt, int *err);
 
-FITRESID *get_VAR_forecast (GRETL_VAR *var, int i, int t0, int t1, int t2,
-			    const double **Z, DATAINFO *pdinfo,
-			    gretlopt opt);
+FITRESID *get_system_forecast (void *p, int ci, int i, 
+			       int t1, int t2, int pre_n,
+			       const double **Z, DATAINFO *pdinfo,
+			       gretlopt opt, int *err);
 
-int display_forecast (const char *str, double ***pZ, DATAINFO *pdinfo, 
-		      gretlopt opt, PRN *prn);
-
-int add_forecast (const char *str, MODEL *pmod, 
-		  double ***pZ, DATAINFO *pdinfo,
-		  gretlopt opt);
+int do_forecast (const char *str, double ***pZ, DATAINFO *pdinfo, 
+		 gretlopt opt, PRN *prn);
 
 void forecast_options_for_model (MODEL *pmod, const double **Z,
 				 const DATAINFO *pdinfo,
 				 int *dyn_ok, int *add_obs_ok,
 				 int *dt2max, int *st2max);
 
+gretl_matrix *get_forecast_matrix (int idx, int *err);
+
 FITRESID *
 rolling_OLS_one_step_fcast (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
-			    int t0, int t1, int t2);
+			    int t1, int t2, int pre_n, int *err);
+
+void forecast_matrix_cleanup (void);
 
 #endif /* FORECAST_H */
 
