@@ -22,6 +22,7 @@
 #include "genparse.h"
 #include "libset.h"
 #include "monte_carlo.h"
+#include "gretl_string_table.h"
 
 #include <errno.h>
 
@@ -95,7 +96,7 @@ int sort_series (const double *x, double *y, int f,
 	}
     }
 
-    if (f == DSORT) {
+    if (f == F_DSORT) {
 	if (markers) {
 	    qsort(vm, i, sizeof *vm, inverse_compare_vms);
 	} else {
@@ -242,7 +243,7 @@ int rank_series (const double *x, double *y, int f,
 	}
     }
     
-    if (f == DSORT) {
+    if (f == F_DSORT) {
 	qsort(sx, m, sizeof *sx, gretl_inverse_compare_doubles);
     } else {
 	qsort(sx, m, sizeof *sx, gretl_compare_doubles);
@@ -267,13 +268,13 @@ int rank_series (const double *x, double *y, int f,
  * diff_series:
  * @x: array of original data.
  * @y: array into which to write the result.
- * @f: function, %DIF, %SDIF or %LDIF.
+ * @f: function, %F_DIFF, %F_SDIFF or %F_LDIFF.
  * @pdinfo: data set information.
  *
  * Calculates the differenced counterpart to the input 
- * series @x.  If @f = %SDIF, the seasonal difference is
- * computed; if @f = %LDIF, the log difference, and if
- * @f = DIF, the ordinary first difference.
+ * series @x.  If @f = %F_SDIFF, the seasonal difference is
+ * computed; if @f = %F_LDIFF, the log difference, and if
+ * @f = %F_DIFF, the ordinary first difference.
  *
  * Returns: 0 on success, non-zero error code on failure.
  */
@@ -282,7 +283,7 @@ int diff_series (const double *x, double *y, int f,
 		 const DATAINFO *pdinfo)
 {
     int t1 = pdinfo->t1;
-    int k = (f == SDIF)? pdinfo->pd : 1;
+    int k = (f == F_SDIFF)? pdinfo->pd : 1;
     int t, err = 0;
 
     if (t1 < k) {
@@ -300,9 +301,9 @@ int diff_series (const double *x, double *y, int f,
 	    continue;
 	}
 
-	if (f == DIF || f == SDIF) {
+	if (f == F_DIFF || f == F_SDIFF) {
 	    y[t] = x[t] - x[t-k];
-	} else if (f == LDIF) {
+	} else if (f == F_LDIFF) {
 	    if (x[t] <= 0.0 || x[t-k] <= 0.0) {
 		err = E_LOGS; /* FIXME: should be warning? */
 	    } else {
@@ -355,7 +356,12 @@ int orthdev_series (const double *x, double *y, const DATAINFO *pdinfo)
 
 	if (n > 0) {
 	    xbar /= n;
-	    y[t] = sqrt(n / (n + 1.0)) * (x[t] - xbar);
+	    /* Lead one period, for compatibility with first diffs.
+	       I.e. we lose the first observation rather than the
+	       last.  This is for arbond.  Cf. Doornik, Bond and
+	       Arellano, DPD documentation.
+	    */
+	    y[t+1] = sqrt(n / (n + 1.0)) * (x[t] - xbar);
 	}
     }
 
@@ -418,18 +424,14 @@ int cum_series (const double *x, double *y, const DATAINFO *pdinfo)
 {
     int t, s = pdinfo->t1;
 
-    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	if (na(x[t])) {
-	    s++;
-	} else {
-	    break;
-	}
+    for (t=pdinfo->t1; t<=pdinfo->t2 && na(x[t]); t++) {
+	s++;
     }
 
     if (s < pdinfo->t2) {
-	y[s] = (na(x[s]))? 0.0 : x[s];
-	for (t=s+1; t<=pdinfo->t2; t++) {
-	    y[t] = y[t-1] + (na(x[t])? 0.0 : x[t]);
+	y[s] = x[s];
+	for (t=s+1; t<=pdinfo->t2 && !na(x[t]); t++) {
+	    y[t] = y[t-1] + x[t];
 	}
     }
 
@@ -470,6 +472,128 @@ int resample_series (const double *x, double *y, const DATAINFO *pdinfo)
     return 0;
 }
 
+/**
+ * filter_series:
+ * @x: array of original data.
+ * @y: array into which to write the result.
+ * @pdinfo: data set information.
+ * @A: matrix for autoregressive polynomial.
+ * @C: matrix for moving average polynomial.
+ *
+ * Filters x according to y_t = C(L)/A(L) x_t; the matrices A and C
+ * must have two columns, or be empty. The rows of A and C must
+ * contain the coefficients for the powers other than 0 (assumed 1),
+ * with the exponents on column one and the corresponding coefficient
+ * on column two. For example, the matrix corresponding to the
+ * polynomial C(L) = 1 - 0.75 L^2 should be given as { 2, -0.75 };
+ * negative exponents (leads) are allowed for the C polynomial only.
+ *
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
+
+int filter_series (const double *x, double *y, const DATAINFO *pdinfo, 
+		   gretl_matrix *A, gretl_matrix *C, double y0)
+{
+    int t1 = pdinfo->t1;
+    int t2 = pdinfo->t2;
+    int t, s, i, m, n;
+    int amin = 0, amax = 0;
+    int cmin = 0, cmax = 0;
+    double coef, *e;
+    int err = 0;
+
+    if (!gretl_is_null_matrix(C)) {
+	for (i=0; i<C->rows; i++) {
+	    m = gretl_matrix_get(C, i, 0);
+	    if (m < cmin) {
+		cmin = m; 
+	    } else if (m > cmax) {
+		cmax = m; 
+	    } 
+	}
+    }
+
+    if (!gretl_is_null_matrix(A)) {
+	for (i=0; i<A->rows; i++) {
+	    m = gretl_matrix_get(A, i, 0);
+	    if (m < amin) {
+		amin = m; 
+	    } else if (m > amax) {
+		amax = m; 
+	    } 
+	}
+    }
+
+    if (amin < 0) {
+	return E_DATA;
+    } 
+
+    err = array_adjust_t1t2(x, &t1, &t2);
+    if (err) {
+	return E_DATA;
+    } 
+
+    n = t2 - t1 + 1;
+    e = malloc(n * sizeof *e);
+
+    if (e == NULL) {
+	return E_ALLOC;
+    }
+
+    s = 0;
+    if (gretl_is_null_matrix(C)) {
+	for (t=t1; t<=t2; t++) {
+	    e[s++] = x[t];
+	}
+    } else {
+	for (t=t1; t<=t2; t++) {
+	    if (s >= cmax && t <= t2+cmin) {
+		e[s] = x[t];
+		for (i=0; i<C->rows; i++) {
+		    m = gretl_matrix_get(C, i, 0);
+		    coef = gretl_matrix_get(C, i, 1);
+		    e[s] += x[t-m] * coef;
+		} 
+	    } else {
+		e[s] = NADBL;
+	    }
+	    s++;
+	} 
+    }
+
+    s = 0;
+    if (gretl_is_null_matrix(A)) {
+	for (t=t1; t<=t2; t++) {
+	    y[t] = e[s++];
+	}
+    } else {
+	double xlag;
+
+	for (t=t1; t<=t2-amin; t++) {
+	    y[t] = e[s];
+	    for (i=0; i<A->rows; i++) {
+		m = gretl_matrix_get(A, i, 0);
+		coef = gretl_matrix_get(A, i, 1);
+#if 1
+		xlag = (t < m || na(y[t-m]))? y0 : y[t-m];
+		y[t] -= coef * xlag;
+#else
+		xlag = (s < m)? y0 : y[t-m];
+		if (!na(xlag)) {
+		    y[t] -= coef * xlag;
+		}
+#endif
+	    } 
+	    s++;
+	}
+    }
+
+    free(e);
+
+    return err;
+}
+
 int panel_mean_series (const double *x, double *y, const DATAINFO *pdinfo)
 {
     const int *unit;
@@ -1241,7 +1365,7 @@ make_panel_time_var (double *x, const DATAINFO *pdinfo)
  * will correctly represent the location in time of
  * each observation.  The index is 1-based.
  *
- * Returns: 0 on success, non-zero code on error.
+ * Returns: 0 on success, non-zero on error.
  */
 
 int gen_time (double ***pZ, DATAINFO *pdinfo, int tm)
@@ -1363,10 +1487,12 @@ const double *gretl_plotx (const DATAINFO *pdinfo)
 {
     static double *x;
     static int ptype;
-    static int T;
+    static int Tbak;
+    static double sd0bak;
 
-    int t, y1;
+    int t, y1, T;
     int new_ptype;
+    double sd0;
     float rm;
 
     if (pdinfo == NULL) {
@@ -1375,12 +1501,15 @@ const double *gretl_plotx (const DATAINFO *pdinfo)
 	x = NULL;
 	ptype = 0;
 	T = 0;
+	sd0 = 0;
 	return NULL;
     }
 
     new_ptype = plotvar_code(pdinfo);
+    T = pdinfo->n;
+    sd0 = pdinfo->sd0;
 
-    if (x != NULL && new_ptype == ptype && T == pdinfo->n) {
+    if (x != NULL && new_ptype == ptype && Tbak == T && sd0 == sd0bak) {
 	/* a suitable array is already at hand */
 	return x;
     }
@@ -1389,16 +1518,17 @@ const double *gretl_plotx (const DATAINFO *pdinfo)
 	free(x);
     }
 
-    x = malloc(pdinfo->n * sizeof *x);
+    x = malloc(T * sizeof *x);
     if (x == NULL) {
 	return NULL;
     }
 
-    T = pdinfo->n;
+    Tbak = T;
     ptype = new_ptype;
+    sd0bak = sd0;
 
-    y1 = (int) pdinfo->sd0;
-    rm = pdinfo->sd0 - y1;
+    y1 = (int) sd0;
+    rm = sd0 - y1;
 
     switch (ptype) {
     case PLOTVAR_ANNUAL: 
@@ -1687,7 +1817,9 @@ int check_declarations (char ***pS, parser *p)
 {
     char **S;
     const char *s;
-    int i, m, n = 1;
+    int i, n = 1;
+
+    gretl_error_clear();
 
     if (p->lh.substr == NULL) {
 	p->err = E_ALLOC;
@@ -1695,9 +1827,16 @@ int check_declarations (char ***pS, parser *p)
     }
 
     s = p->lh.substr;
+    s += strspn(s, " ");
+
     while (*s) {
-	if (*s == ',') n++;
-	s++;
+	if (*s == ',' || *s == ' ') {
+	    n++;
+	    s++;
+	    s += strspn(s, " ");
+	} else {
+	    s++;
+	}
     }
 
     S = strings_array_new(n);
@@ -1709,30 +1848,31 @@ int check_declarations (char ***pS, parser *p)
     s = p->lh.substr;
     for (i=0; i<n; i++) {
 	S[i] = gretl_word_strdup(s, &s);
-    }
-
-    m = 0;
-    for (i=0; i<n; i++) {
-	if (varindex(p->dinfo, S[i]) < p->dinfo->v || 
-	    get_matrix_by_name(S[i]) ||
-	    get_list_by_name(S[i])) {
-	    /* variable already exists */
-	    free(S[i]);
-	    S[i] = NULL;
-	} else if (check_varname(S[i])) {
-	    /* invalid name */
+	if (S[i] == NULL) {
 	    p->err = E_DATA;
-	} else {
-	    m++;
+	    break;
 	}
     }
 
-    if (m == 0) {
+    if (*s != '\0') {
 	p->err = E_DATA;
-	strcpy(gretl_errmsg, "Invalid declaration");
+    }
+
+    for (i=0; i<n && !p->err; i++) {
+	if (varindex(p->dinfo, S[i]) < p->dinfo->v || 
+	    get_matrix_by_name(S[i]) ||
+	    get_list_by_name(S[i]) ||
+	    get_string_by_name(S[i])) {
+	    /* variable already exists */
+	    p->err = E_DATA;
+	} else if (check_varname(S[i])) {
+	    /* invalid name */
+	    p->err = E_DATA;
+	} 
     }
 
     if (p->err) {
+	gretl_errmsg_set(_("Invalid declaration"));
 	free_strings_array(S, n);
     } else {
 	*pS = S;
@@ -1741,3 +1881,286 @@ int check_declarations (char ***pS, parser *p)
     return n;
 }
 
+/* cross-sectional mean of observations on variables in @list
+   at time @t */
+
+static double mean_at_obs (const int *list, const double **Z, int t)
+{
+    double xi, xsum = 0.0;
+    int i;
+
+    for (i=1; i<=list[0]; i++) {
+	xi = Z[list[i]][t];
+	if (na(xi)) {
+	    return NADBL;
+	}
+	xsum += xi;
+    }
+
+    return xsum / list[0];
+}
+
+/* weighted cross-sectional mean of observations on variables in @list
+   at time @t, weights given in @wlist */
+
+static double weighted_mean_at_obs (const int *list, const int *wlist,
+				    const double **Z, int t,
+				    double *pwsum, int *pm)
+{
+    double w, xi, wsum = 0.0, wxbar = 0.0;
+    int i, m = 0;
+
+    for (i=1; i<=list[0]; i++) {
+	w = Z[wlist[i]][t];
+	if (na(w) || w < 0.0) {
+	    return NADBL;
+	}
+	if (w > 0.0) {
+	    wsum += w;
+	    m++;
+	}
+    }
+
+    if (wsum <= 0.0) {
+	return NADBL;
+    }
+
+    if (pwsum != NULL) {
+	*pwsum = wsum;
+    }
+
+    if (pm != NULL) {
+	*pm = m;
+    }
+
+    for (i=1; i<=list[0]; i++) {
+	w = Z[wlist[i]][t] / wsum;
+	if (w > 0) {
+	    xi = Z[list[i]][t];
+	    if (na(xi)) {
+		return NADBL;
+	    }
+	    wxbar += xi * w;
+	}
+    }
+
+    return wxbar;
+}
+
+/* Computes weighted mean of the variables in @list using the
+   (possibly time-varying) weights given in @wlist, or the
+   unweighted mean if @wlist is NULL.
+*/
+
+static int x_sectional_weighted_mean (double *x, const int *list, 
+				      const int *wlist,
+				      const double **Z, 
+				      const DATAINFO *pdinfo)
+{
+    int n = list[0];
+    int t, v;
+
+    if (n == 0) {
+	return 0; /* all NAs */
+    } else if (n == 1) {
+	v = list[1];
+	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	    x[t] = Z[v][t]; 
+	}
+	return 0;
+    }
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	if (wlist != NULL) {
+	    x[t] = weighted_mean_at_obs(list, wlist, Z, t, NULL, NULL);
+	} else {
+	    x[t] = mean_at_obs(list, Z, t);
+	}
+    }
+
+    return 0;
+}
+
+/* Computes weighted sample variance of the variables in @list using
+   the (possibly time-varying) weights given in @wlist, or the
+   unweighted sample variance if @wlist is NULL
+*/
+
+static int x_sectional_wtd_variance (double *x, const int *list,
+				     const int *wlist,
+				     const double **Z, 
+				     const DATAINFO *pdinfo)
+{
+    double xdev, xbar, wsum;
+    int m = 0, n = list[0];
+    int i, t, v;
+
+    if (n == 0) {
+	return 0; /* all NAs */
+    } else if (n == 1) {
+	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	    x[t] = 0.0;
+	}
+	return 0;
+    }
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	if (wlist != NULL) {
+	    xbar = weighted_mean_at_obs(list, wlist, Z, t, &wsum, &m);
+	} else {
+	    xbar = mean_at_obs(list, Z, t);
+	}
+	if (na(xbar)) {
+	    x[t] = NADBL;
+	    continue;
+	}
+	if (wlist != NULL && m < 2) {
+	    x[t] = (m == 1)? 0.0 : NADBL;
+	    continue;
+	}
+	x[t] = 0.0;
+	for (i=1; i<=list[0]; i++) {
+	    v = list[i];
+	    xdev = Z[v][t] - xbar;
+	    if (wlist != NULL) {
+		x[t] += xdev * xdev * Z[wlist[i]][t] / wsum;
+	    } else {
+		x[t] += xdev * xdev;
+	    }
+	}
+	if (wlist != NULL) {
+	    x[t] *= m / (m - 1);
+	} else {
+	    x[t] /= (n - 1);
+	}
+    }
+
+    return 0;
+}
+
+static int x_sectional_wtd_stddev (double *x, const int *list, 
+				   const int *wlist,
+				   const double **Z, 
+				   const DATAINFO *pdinfo)
+{
+    int t, err;
+
+    err = x_sectional_wtd_variance(x, list, wlist, Z, pdinfo);
+
+    if (!err) {
+	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	    if (!na(x[t])) {
+		x[t] = sqrt(x[t]);
+	    }
+	}
+    }
+
+    return err;
+}
+
+static int x_sectional_extremum (int f, double *x, const int *list, 
+				 const int *wlist,
+				 const double **Z, 
+				 const DATAINFO *pdinfo)
+{
+    double xit, xx;
+    int i, t, err = 0;
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	xx = (f == F_MIN)? NADBL : -NADBL;
+	for (i=1; i<=list[0]; i++) {
+	    xit = Z[list[i]][t];
+	    if (!na(xit)) { 
+		if (f == F_MAX && xit > xx) {
+		    xx = xit;
+		} else if (f == F_MIN && xit < xx) {
+		    xx = xit;
+		}
+	    }
+	}
+	if (xx == -NADBL) {
+	    x[t] = NADBL;
+	} else {
+	    x[t] = xx;
+	}
+    }
+
+    return err;
+}
+
+int cross_sectional_stat (double *x, const int *list, 
+			  const double **Z, 
+			  const DATAINFO *pdinfo,
+			  int f)
+{
+    if (f == F_MEAN) {
+	return x_sectional_weighted_mean(x, list, NULL, Z, pdinfo);
+    } else if (f == F_VCE) {
+	return x_sectional_wtd_variance(x, list, NULL, Z, pdinfo);
+    } else if (f == F_SD) {
+	return x_sectional_wtd_stddev(x, list, NULL, Z, pdinfo);
+    } else if (f == F_MIN || f == F_MAX) {
+	return x_sectional_extremum(f, x, list, NULL, Z, pdinfo);
+    } else {
+	return E_DATA;
+    }
+}
+
+int x_sectional_weighted_stat (double *x, const int *list, 
+			       const int *wlist,
+			       const double **Z, 
+			       const DATAINFO *pdinfo,
+			       int f)
+{
+    if (wlist[0] != list[0]) {
+	gretl_errmsg_sprintf("Weighted stats: data list has %d members but weight "
+			     "list has %d", list[0], wlist[0]);
+	return E_DATA;
+    }
+
+    if (f == F_WMEAN) {
+	return x_sectional_weighted_mean(x, list, wlist, Z, pdinfo);
+    } else if (f == F_WVAR) {
+	return x_sectional_wtd_variance(x, list, wlist, Z, pdinfo);
+    } else if (f == F_WSD) {
+	return x_sectional_wtd_stddev(x, list, wlist, Z, pdinfo);
+    } else {
+	return E_DATA;
+    }
+}
+
+/* writes to the series @y a linear combination of the variables given
+   in @list, using the coefficients given in the vector @b.
+*/
+
+int list_linear_combo (double *y, const int *list, 
+		       const gretl_vector *b, const double **Z, 
+		       const DATAINFO *pdinfo)
+{
+    int nb = gretl_vector_get_length(b);
+    int nl = list[0];
+    int err = 0;
+
+    if (nb != nl) {
+	err = E_DATA;
+    } else {
+	int i, t;
+	double xit, yt;
+
+	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	    yt = 0;
+	    for (i=0; i<nl; i++) {
+		xit = Z[list[i+1]][t];
+		if (na(xit)) {
+		    yt = NADBL;
+		    break;
+		} else {
+		    yt += xit * gretl_vector_get(b, i);
+		}
+	    }
+	    y[t] = yt;
+	}
+    }
+
+    return err;
+}
