@@ -43,6 +43,9 @@ int resample_series (const double *x, double *y,
 int fracdiff_series (const double *x, double *y, double d,
 		     const DATAINFO *pdinfo);
 
+int filter_series(const double *x, double *y, const DATAINFO *pdinfo, 
+		  gretl_matrix *A, gretl_matrix *C, double y0);
+
 int panel_mean_series (const double *x, double *y, const DATAINFO *pdinfo);
 
 int panel_sd_series (const double *x, double *y, const DATAINFO *pdinfo);
@@ -77,4 +80,8 @@ int get_observation_number (const char *s, const DATAINFO *pdinfo);
 int get_t_from_obs_string (const char *s, const double **Z, 
 			   const DATAINFO *pdinfo);
 
+int list_linear_combo (double *y, const int *list, 
+		       const gretl_vector *b, const double **Z, 
+		       const DATAINFO *pdinfo);
+
 #endif /* GENFUNCS_H */
