@@ -25,6 +25,8 @@
 #include "gretl_func.h"
 #include "gretl_string_table.h"
 
+#include <glib.h>
+
 #define NUMLEN 32
 #define MAXQUOTE 64
 
@@ -77,6 +79,8 @@ struct str_table dvars[] = {
     { R_T1,        "$t1" },
     { R_T2,        "$t2" },
     { R_STOPWATCH, "$stopwatch" },
+    { R_NSCAN,     "$nscan" },
+    { R_PUNIT,     "$unit" },
     { 0,           NULL },
 };
 
@@ -96,6 +100,7 @@ struct str_table mvars[] = {
     { M_UHAT,    "$uhat" },
     { M_YHAT,    "$yhat" },
     { M_AHAT,    "$ahat" },
+    { M_SAMPLE,  "$sample" },
     { M_H,       "$h" },
     { M_COEFF,   "$coeff" },
     { M_SE,      "$stderr" },
@@ -108,147 +113,191 @@ struct str_table mvars[] = {
     { M_JS00,    "$s00" },
     { M_JS11,    "$s11" },
     { M_JS01,    "$s01" },
+    { M_HAUSMAN, "$hausman" },
+    { M_SARGAN,  "$sargan" },
+    { M_SYSGAM,  "$sysGamma" },
+    { M_SYSA,    "$sysA" },
+    { M_SYSB,    "$sysB" },
+    { M_FCAST,   "$fcast" },
+    { M_FCERR,   "$fcerr" },
+    { M_COEFF_CI, "$coeff_ci" },
     { 0,         NULL }
 };
 
 struct str_table funcs[] = {
-    { ABS,      "abs" },
-    { SIN,      "sin" },
-    { COS,      "cos" },
-    { TAN,      "tan" },
-    { ATAN,     "atan" },
-    { LOG,      "log" },
-    { LOG,      "ln" },
-    { LOG10,    "log10" },
-    { LOG2,     "log2" },
-    { EXP,      "exp" },
-    { SQRT,     "sqrt" },
-    { DIF,      "diff" },
-    { LDIF,     "ldiff" },
-    { SDIF,     "sdiff" },
-    { TOINT,    "int" },
-    { CEIL,     "ceil" },
-    { FLOOR,    "floor" },
-    { SORT,     "sort" }, 
-    { DSORT,    "dsort" }, 
-    { SORTBY,   "sortby" }, 
-    { RANKING,  "ranking" },
-    { ODEV,     "orthdev" },
-    { NOBS,     "nobs" },
-    { T1,       "firstobs" },
-    { T2,       "lastobs" },
-    { RUNIFORM, "uniform" }, 
-    { RNORMAL,  "normal" }, 
-    { RPOISSON, "poisson" },
-    { CUM,      "cum" }, 
-    { MISSING,  "missing" },
-    { OK,       "ok" },        /* opposite of missing */
-    { MISSZERO, "misszero" },
-    { LRVAR,    "lrvar" },
-    { QUANTILE, "quantile" },
-    { MEDIAN,   "median" },
-    { GINI,     "gini" },
-    { ZEROMISS, "zeromiss" },
-    { SUM,      "sum" },
-    { MEAN,     "mean" },
-    { MIN,      "min" },
-    { MAX,      "max" },
-    { SD,       "sd" },
-    { VCE,      "var" },
-    { SST,      "sst" },
-    { CNORM,    "cnorm" },
-    { DNORM,    "dnorm" },
-    { QNORM,    "qnorm" },
-    { GAMMA,    "gammafunc" },
-    { LNGAMMA,  "lngamma" },
-    { RESAMPLE, "resample" },
-    { PMEAN,    "pmean" },     /* panel mean */
-    { PSD,      "psd" },       /* panel std dev */
-    { HPFILT,   "hpfilt" },    /* Hodrick-Prescott filter */
-    { BKFILT,   "bkfilt" },    /* Baxter-King filter */
-    { FRACDIF,  "fracdiff" },  /* fractional difference */
-    { COV,      "cov" },
-    { COR,      "corr" },
-    { MOVAVG,   "movavg" },
-    { IMAT,     "I" },
-    { ZEROS,    "zeros" },
-    { ONES,     "ones" },
-    { SEQ,      "seq" },
-    { MUNIF,    "muniform" },
-    { MNORM,    "mnormal" },
-    { SUMR,     "sumr" },
-    { SUMC,     "sumc" },
-    { MEANR,    "meanr" },
-    { MEANC,    "meanc" },
-    { MINC,     "minc" },
-    { MAXC,     "maxc" },
-    { MINR,     "minr" },
-    { MAXR,     "maxr" },
-    { IMINC,    "iminc" },
-    { IMAXC,    "imaxc" },
-    { IMINR,    "iminr" },
-    { IMAXR,    "imaxr" }, 
-    { FFT,      "fft" },
-    { FFTI,     "ffti" },
-    { CMULT,    "cmult" },
-    { CDIV,     "cdiv" },
-    { MCOV,     "mcov" },
-    { MCORR,    "mcorr" },
-    { MXTAB,    "mxtab" },
-    { CDEMEAN,  "cdemean" },
-    { CHOL,     "cholesky" },
-    { INV,      "inv" },
-    { GINV,     "ginv" },
-    { DIAG,     "diag" },
-    { TRANSP,   "transp" },
-    { TVEC,     "vec" },
-    { VECH,     "vech" },
-    { UNVECH,   "unvech" },
-    { UPPER,    "upper" },
-    { LOWER,    "lower" },
-    { ROWS,     "rows" },
-    { COLS,     "cols" },
-    { DET,      "det" },
-    { LDET,     "ldet" },
-    { TRACE,    "tr" },
-    { NORM1,    "onenorm" },
-    { INFNORM,  "infnorm" },
-    { RCOND,    "rcond" },
-    { RANK,     "rank" },
-    { QFORM,    "qform" },
-    { MLAG,     "mlag" },
-    { QR,       "qrdecomp" },
-    { EIGSYM,   "eigensym" },
-    { EIGGEN,   "eigengen" },
-    { NULLSPC,  "nullspace" },
-    { PRINCOMP, "princomp" },
-    { MEXP,     "mexp" },
-    { FDJAC,    "fdjac" },
-    { BFGSMAX,  "BFGSmax" },
-    { OBSNUM,   "obsnum" },
-    { ISSERIES, "isseries" },
-    { ISLIST,   "islist" },
-    { ISSTRING, "isstring" },
-    { ISNULL,   "isnull" },
-    { LISTLEN,  "nelem" },
-    { CDF,      "cdf" },
-    { INVCDF,   "invcdf" },
-    { PVAL,     "pvalue" },
-    { CRIT,     "critical" },
-    { RANDGEN,  "randgen" },
-    { MAKEMASK, "makemask" },
-    { VALUES,   "values" },
-    { MSHAPE,   "mshape" },
-    { SVD,      "svd" },
-    { MOLS,     "mols" },
-    { MREAD,    "mread" },
-    { MWRITE,   "mwrite" },
+    { F_ABS,      "abs" },
+    { F_SIN,      "sin" },
+    { F_COS,      "cos" },
+    { F_TAN,      "tan" },
+    { F_ASIN,     "asin" },
+    { F_ACOS,     "acos" },
+    { F_ATAN,     "atan" },
+    { F_LOG,      "log" },
+    { F_LOG,      "ln" },
+    { F_LOG10,    "log10" },
+    { F_LOG2,     "log2" },
+    { F_EXP,      "exp" },
+    { F_SQRT,     "sqrt" },
+    { F_DIFF,     "diff" },
+    { F_LDIFF,    "ldiff" },
+    { F_SDIFF,    "sdiff" },
+    { F_LLAG,     "lags" },
+    { F_TOINT,    "int" },
+    { F_ROUND,    "round" },
+    { F_CEIL,     "ceil" },
+    { F_FLOOR,    "floor" },
+    { F_SORT,     "sort" }, 
+    { F_DSORT,    "dsort" }, 
+    { F_SORTBY,   "sortby" }, 
+    { F_RANKING,  "ranking" },
+    { F_ODEV,     "orthdev" },
+    { F_NOBS,     "nobs" },
+    { F_T1,       "firstobs" },
+    { F_T2,       "lastobs" },
+    { F_RUNIFORM, "uniform" }, 
+    { F_RNORMAL,  "normal" }, 
+    { F_RPOISSON, "genpois" },
+    { F_CUM,      "cum" }, 
+    { F_MISSING,  "missing" },
+    { F_DATAOK,   "ok" },        /* opposite of missing */
+    { F_MISSZERO, "misszero" },
+    { F_LRVAR,    "lrvar" },
+    { F_QUANTILE, "quantile" },
+    { F_MEDIAN,   "median" },
+    { F_GINI,     "gini" },
+    { F_ZEROMISS, "zeromiss" },
+    { F_SUM,      "sum" },
+    { F_MEAN,     "mean" },
+    { F_MIN,      "min" },
+    { F_MAX,      "max" },
+    { F_SD,       "sd" },
+    { F_VCE,      "var" },
+    { F_SST,      "sst" },
+    { F_CNORM,    "cnorm" },
+    { F_DNORM,    "dnorm" },
+    { F_QNORM,    "qnorm" },
+    { F_GAMMA,    "gammafun" },
+    { F_LNGAMMA,  "lngamma" },
+    { F_RESAMPLE, "resample" },
+    { F_PMEAN,    "pmean" },     /* panel mean */
+    { F_PSD,      "psd" },       /* panel std dev */
+    { F_HPFILT,   "hpfilt" },    /* Hodrick-Prescott filter */
+    { F_BKFILT,   "bkfilt" },    /* Baxter-King filter */
+    { F_FRACDIFF, "fracdiff" },  /* fractional difference */
+    { F_COV,      "cov" },
+    { F_COR,      "corr" },
+    { F_MOVAVG,   "movavg" },
+    { F_IMAT,     "I" },
+    { F_ZEROS,    "zeros" },
+    { F_ONES,     "ones" },
+    { F_SEQ,      "seq" },
+    { F_MUNIF,    "muniform" },
+    { F_MNORM,    "mnormal" },
+    { F_SUMR,     "sumr" },
+    { F_SUMC,     "sumc" },
+    { F_MEANR,    "meanr" },
+    { F_MEANC,    "meanc" },
+    { F_SDC,      "sdc" },
+    { F_MINC,     "minc" },
+    { F_MAXC,     "maxc" },
+    { F_MINR,     "minr" },
+    { F_MAXR,     "maxr" },
+    { F_IMINC,    "iminc" },
+    { F_IMAXC,    "imaxc" },
+    { F_IMINR,    "iminr" },
+    { F_IMAXR,    "imaxr" }, 
+    { F_FFT,      "fft" },
+    { F_FFTI,     "ffti" },
+    { F_CMULT,    "cmult" },
+    { F_CDIV,     "cdiv" },
+    { F_MCOV,     "mcov" },
+    { F_MCORR,    "mcorr" },
+    { F_MXTAB,    "mxtab" },
+    { F_CDEMEAN,  "cdemean" },
+    { F_CHOL,     "cholesky" },
+    { F_INV,      "inv" },
+    { F_INVPD,    "invpd" },
+    { F_GINV,     "ginv" },
+    { F_DIAG,     "diag" },
+    { F_TRANSP,   "transp" },
+    { F_VEC,      "vec" },
+    { F_VECH,     "vech" },
+    { F_UNVECH,   "unvech" },
+    { F_UPPER,    "upper" },
+    { F_LOWER,    "lower" },
+    { F_ROWS,     "rows" },
+    { F_COLS,     "cols" },
+    { F_DET,      "det" },
+    { F_LDET,     "ldet" },
+    { F_TRACE,    "tr" },
+    { F_NORM1,    "onenorm" },
+    { F_INFNORM,  "infnorm" },
+    { F_RCOND,    "rcond" },
+    { F_RANK,     "rank" },
+    { F_QFORM,    "qform" },
+    { F_MLAG,     "mlag" },
+    { F_QR,       "qrdecomp" },
+    { F_EIGSYM,   "eigensym" },
+    { F_EIGGEN,   "eigengen" },
+    { F_NULLSPC,  "nullspace" },
+    { F_PRINCOMP, "princomp" },
+    { F_MEXP,     "mexp" },
+    { F_FDJAC,    "fdjac" },
+    { F_BFGSMAX,  "BFGSmax" },
+    { F_OBSNUM,   "obsnum" },
+    { F_ISSERIES, "isseries" },
+    { F_ISLIST,   "islist" },
+    { F_ISSTRING, "isstring" },
+    { F_ISNULL,   "isnull" },
+    { F_LISTLEN,  "nelem" },
+    { F_PDF,      "pdf" },
+    { F_CDF,      "cdf" },
+    { F_INVCDF,   "invcdf" },
+    { F_PVAL,     "pvalue" },
+    { F_CRIT,     "critical" },
+    { F_RANDGEN,  "randgen" },
+    { F_MAKEMASK, "makemask" },
+    { F_VALUES,   "values" },
+    { F_MSHAPE,   "mshape" },
+    { F_SVD,      "svd" },
+    { F_MOLS,     "mols" },
+    { F_MREAD,    "mread" },
+    { F_MWRITE,   "mwrite" },
+    { F_MRSEL,    "selifr" },
+    { F_MCSEL,    "selifc" },
+    { F_POLROOTS, "polroots" },
+    { F_DUMIFY,   "dummify" },
+    { F_WMEAN,    "wmean" },
+    { F_WVAR,     "wvar" },
+    { F_WSD,      "wsd" },
+    { F_XPX,      "xpx" },
+    { F_FILTER,   "filter" },
+    { F_TRIMR,    "trimr" },
+    { F_GETENV,   "getenv" },
+    { F_ARGNAME,  "argname" },
+    { F_OBSLABEL, "obslabel" },
+    { F_READFILE, "readfile" },
+    { F_BACKTICK, "grab" },
+    { F_STRSTR,   "strstr" },
+    { F_STRLEN,   "strlen" },
+    { F_VARNAME,  "varname" },
+    { F_VARNUM,   "varnum" },
+    { F_COLNAMES, "colnames" },
+    { F_LJUNGBOX, "ljungbox" },
+    { F_MSORTBY,  "msortby" },
+    { F_LINCOMB,  "lincomb" },
     { 0,        NULL }
 };
 
 struct str_table func_alias[] = {
-    { GAMMA,    "gamma" },
-    { 0,        NULL }
+    { F_GAMMA,     "gammafunc" },
+    { F_GAMMA,     "gamma" },
+    { F_RPOISSON,  "poisson" },
+    { F_PVAL,      "pval" },
+    { F_LOG,       "logs" },
+    { F_OBSLABEL,  "date" },
+    { F_BACKTICK,  "$" },
+    { 0,          NULL }
 };
 
 int const_lookup (const char *s)
@@ -277,6 +326,21 @@ const char *constname (int c)
     return "unknown";
 }
 
+static GHashTable *gretl_function_hash_init (void)
+{
+    GHashTable *ht;
+    int i;
+
+    ht = g_hash_table_new(g_str_hash, g_str_equal);
+
+    for (i=0; funcs[i].str != NULL; i++) {
+	g_hash_table_insert(ht, (gpointer) funcs[i].str, 
+			    GINT_TO_POINTER(funcs[i].id));
+    }
+
+    return ht;
+}
+
 enum {
     NO_ALIAS,
     ALLOW_ALIAS
@@ -284,15 +348,31 @@ enum {
 
 static int real_function_lookup (const char *s, int a)
 {
-    int i;
+    static GHashTable *fht;
+    gpointer p;
+    int ret = 0;
 
-    for (i=0; funcs[i].id != 0; i++) {
-	if (!strcmp(s, funcs[i].str)) {
-	    return funcs[i].id;
+    if (s == NULL) {
+	/* cleanup signal */
+	if (fht != NULL) {
+	    g_hash_table_destroy(fht);
+	    fht = NULL;
 	}
+	return 0;
     }
 
-    if (a == ALLOW_ALIAS) {
+    if (fht == NULL) {
+	fht = gretl_function_hash_init();
+    }
+    
+    p = g_hash_table_lookup(fht, s);
+    if (p != NULL) {
+	ret = GPOINTER_TO_INT(p);
+    }
+
+    if (ret == 0 && a == ALLOW_ALIAS) {
+	int i;
+
 	for (i=0; func_alias[i].id != 0; i++) {
 	    if (!strcmp(s, func_alias[i].str)) {
 		return func_alias[i].id;
@@ -300,7 +380,12 @@ static int real_function_lookup (const char *s, int a)
 	} 
     }   
 
-    return 0;
+    return ret;
+}
+
+void gretl_function_hash_cleanup (void)
+{
+    real_function_lookup(NULL, 0);
 }
 
 int function_lookup (const char *s)
@@ -457,10 +542,26 @@ const char *mvarname (int t)
     return "unknown";
 }
 
+int genr_function_word (const char *s)
+{
+    int ret = 0;
+
+    ret = real_function_lookup(s, NO_ALIAS);
+    if (!ret) {
+	ret = dvar_lookup(s);
+    }
+    if (!ret) {
+	ret = mvar_lookup(s);
+    }
+
+    return ret;
+}
+
 static void undefined_symbol_error (const char *s, parser *p)
 {
     parser_print_input(p);
     pprintf(p->prn, _("The symbol '%s' is undefined\n"), s);
+    sprintf(gretl_errmsg, _("The symbol '%s' is undefined\n"), s);
     p->err = E_UNKVAR;
 }
 
@@ -468,6 +569,7 @@ static void function_noargs_error (const char *s, parser *p)
 {
     parser_print_input(p);
     pprintf(p->prn, _("'%s': no argument was given\n"), s);
+    sprintf(gretl_errmsg, _("'%s': no argument was given\n"), s);
     p->err = 1;
 }
 
@@ -490,19 +592,25 @@ static char *get_quoted_string (parser *p)
     int n = parser_charpos(p, '"');
     char *s = NULL;
 
-    if (n > 0) {
+    if (n >= 0) {
 	s = gretl_strndup(p->point, n);
 	parser_advance(p, n + 1);
-    } else if (n == 0) {	
-	parser_print_input(p);
-	pprintf(p->prn, _("Empty string\n"));
-	p->err = E_PARSE;
     } else {
 	parser_print_input(p);
 	pprintf(p->prn, _("Unmatched '%c'\n"), '"');
 	p->err = E_PARSE;
     }
 
+    if (!p->err) {
+	if (p->ch == '.' && parser_charpos(p, '$') == 0) {
+	    /* maybe quoted name of saved object followed by 
+	       dollar variable? */
+	    p->sym = OVAR;
+	} else {
+	    p->sym = STR;
+	}
+    }
+
     return s;
 }
 
@@ -592,6 +700,22 @@ NODE *obs_node (parser *p)
     return ret;
 }
 
+static void look_up_string_variable (const char *s, parser *p)
+{
+    const char *val = get_string_by_name(s + 1);
+
+    if (val != NULL) {
+	p->idstr = gretl_strdup(s);
+	if (p->idstr == NULL) {
+	    p->err = E_ALLOC;
+	} else {
+	    p->sym = VSTR;
+	}
+    } else {
+	undefined_symbol_error(s, p);
+    }
+}
+
 static void look_up_dollar_word (const char *s, parser *p)
 {
     p->idnum = dvar_lookup(s);
@@ -624,7 +748,11 @@ static void look_up_word (const char *s, parser *p)
 	    } else {
 		p->idnum = varindex(p->dinfo, s);
 		if (p->idnum < p->dinfo->v) {
-		    p->sym = UVAR;
+		    p->sym = (var_is_scalar(p->dinfo, p->idnum))?
+			USCALAR : USERIES;
+		} else if (!strcmp(s, "time")) {
+		    p->sym = DUM;
+		    p->idnum = DUM_TREND;
 		} else if (get_matrix_by_name(s)) {
 		    p->sym = UMAT;
 		    p->idstr = gretl_strdup(s);
@@ -638,8 +766,12 @@ static void look_up_word (const char *s, parser *p)
 		    p->sym = UFUN;
 		    p->idstr = gretl_strdup(s);
 		} else if (string_is_defined(s)) {
-		    p->sym = STR;
-		    p->idstr = gretl_strdup(get_named_string(s));
+		    p->sym = VSTR;
+		    p->idstr = gretl_strdup(s);
+		} else if (p->targ == LIST &&
+			   varname_match_any(p->dinfo, s)) {
+		    p->sym = LIST;
+		    p->idstr = gretl_strdup(s);
 		} else {
 		    err = 1;
 		}
@@ -667,7 +799,7 @@ static void word_check_next_char (const char *s, parser *p)
 
     if (p->ch == '(') {
 	/* series (lag) or function */
-	if (p->sym == UVAR && var_is_series(p->dinfo, p->idnum)) {
+	if (p->sym == USERIES) {
 	    if (p->idnum == p->lh.v) {
 		p->flags |= P_AUTOREG;
 	    }
@@ -675,7 +807,7 @@ static void word_check_next_char (const char *s, parser *p)
 	} else if (p->sym == MVAR && model_data_matrix(p->idnum)) {
 	    /* old-style "$coeff(x1)" etc. */
 	    p->sym = DMSTR;
-	} else if (!func_symb(p->sym) && !func2_symb(p->sym) &&
+	} else if (!func1_symb(p->sym) && !func2_symb(p->sym) &&
 		   !funcn_symb(p->sym) && p->sym != UFUN) {
 	    p->err = 1;
 	} 
@@ -686,7 +818,7 @@ static void word_check_next_char (const char *s, parser *p)
 	} else if (p->sym == MVAR && could_be_matrix(p->idnum)) {
 	    /* slice of $ matrix */
 	    p->sym = DMSL;
-	} else if (p->sym == UVAR && var_is_series(p->dinfo, p->idnum)) {
+	} else if (p->sym == USERIES) {
 	    /* observation from series */
 	    p->sym = OBS;
 	} else {
@@ -696,6 +828,10 @@ static void word_check_next_char (const char *s, parser *p)
 	if (p->sym == UOBJ) {
 	    /* name of saved object followed by dollar variable? */
 	    p->sym = OVAR;
+	} else if (p->sym == STR) {
+	    /* maybe quoted name of saved object followed by 
+	       dollar variable? */
+	    p->sym = OVAR;
 	} else {
 	    p->err = 1;
 	}	    
@@ -706,16 +842,27 @@ static void word_check_next_char (const char *s, parser *p)
     }
 }
 
+static int is_word_char (parser *p)
+{
+    if (strchr(wordchars, p->ch) != NULL) {
+	return 1;
+    } else if (p->targ == LIST && p->ch == '*') {
+	return 1;
+    }
+
+    return 0;
+}
+
 static void getword (parser *p)
 {  
     char word[32];
     int i = 0;
 
-    /* we know the first char is acceptable (and might be '$') */
+    /* we know the first char is acceptable (and might be '$' or '@') */
     word[i++] = p->ch;
     parser_getc(p);
 
-    while (p->ch != 0 && strchr(wordchars, p->ch) != NULL && i < 31) {
+    while (p->ch != 0 && is_word_char(p) && i < 31) {
 	word[i++] = p->ch;
 	parser_getc(p);
     }
@@ -731,11 +878,11 @@ static void getword (parser *p)
 	parser_getc(p);
     }
 
-    if (p->getstr) {
+    if (p->flags & P_GETSTR) {
 	/* uninterpreted string wanted */
 	p->sym = STR;
 	p->idstr = gretl_strdup(word);
-	p->getstr = 0;
+	p->flags ^= P_GETSTR;
 	return;
     }
 
@@ -746,13 +893,15 @@ static void getword (parser *p)
 	return;
     }
 
-    if (*word == '$' || !strcmp(word, "t") || !strcmp(word, "obs")) {
+    if ((*word == '$' && word[1]) || !strcmp(word, "t") || !strcmp(word, "obs")) {
 	look_up_dollar_word(word, p);
+    } else if (*word == '@') {
+	look_up_string_variable(word, p);
     } else {
 	look_up_word(word, p);
     }
 
-    if (!p->err) {
+    if (!p->err && *word != '@') {
 	word_check_next_char(word, p);
     }
 
@@ -837,10 +986,14 @@ static double getdbl (parser *p)
 {
     char xstr[NUMLEN] = {0};
     double d = NADBL;
+    int gotcol = 0;
     int i = 0;
 
     while (ok_dbl_char(p->ch, xstr, i - 1) && i < NUMLEN - 1) {
 	xstr[i++] = p->ch;
+	if (p->ch == ':') {
+	    gotcol = 1;
+	}
 	parser_getc(p);
     }  
 
@@ -853,7 +1006,7 @@ static double getdbl (parser *p)
     fprintf(stderr, "getdbl: xstr = '%s'\n", xstr);
 #endif
     
-    if (strchr(xstr, ':')) {
+    if (gotcol) {
 	if (p->dinfo->pd == 1) {
 	    p->err = E_PDWRONG;
 	} else {
@@ -872,22 +1025,10 @@ static double getdbl (parser *p)
     return d;
 }
 
-#if 0 /* later? */
-static void deprecation_note (parser *p)
-{
-    if (p->sym == B_AND) {
-	pprintf(p->prn, "Note: the recommended form for logical "
-		"AND is '&&'\n");
-    } else if (p->sym == B_OR) {
-	pprintf(p->prn, "Note: the recommended form for logical "
-		"OR is '||'\n");
-    }
-}
-#endif
+#define word_start_special(c) (c == '$' || c == '@' || c == '_')
 
-#define matrix_gen(p) (p->lh.t == MAT || p->targ == MAT)
-
-#define unary_context(p) (p->sym < F2_MAX || p->sym == COM)
+#define lag_range_sym(p) (p->ch == 't' && *p->point == 'o' && \
+			  *(p->point + 1) == ' ')
 
 void lex (parser *p)
 {
@@ -918,9 +1059,14 @@ void lex (parser *p)
 	    parser_getc(p);
 	    return;
         case '*': 
+	    if (p->targ == LIST) {
+		/* treat '*' as wildcard */
+		getword(p);
+		return;
+	    }
 	    parser_getc(p);
 	    if (p->ch == '*') {
-		p->sym = KRON;
+		p->sym = B_KRON;
 		parser_getc(p);
 	    } else {
 		p->sym = B_MUL;
@@ -943,34 +1089,21 @@ void lex (parser *p)
 	    parser_getc(p);
 	    return;
         case '&': 
-#if 0 /* for later, but modified */
-	    if (unary_context(p)) {
-		p->sym = U_ADDR;
-		parser_getc(p);
-		return;
-	    }
-	    p->sym = B_AND;
 	    parser_getc(p);
 	    if (p->ch == '&') {
+		p->sym = B_AND;
 		parser_getc(p);
 	    } else {
-		deprecation_note(p);
+		p->sym = U_ADDR;
 	    }
-#else
-	    p->sym = B_AND;
-	    parser_getc(p);
-	    if (p->ch == '&') {
-		/* make "&&" equal to "&" */
-		parser_getc(p);
-	    }
-#endif
 	    return;
         case '|': 
-	    p->sym = (matrix_gen(p))? MRCAT : B_OR;
 	    parser_getc(p);
 	    if (p->ch == '|') {
 		p->sym = B_OR;
 		parser_getc(p);
+	    } else {
+		p->sym = B_VCAT;
 	    }
 	    return;
         case '!': 
@@ -983,8 +1116,12 @@ void lex (parser *p)
 	    }
 	    return;
         case '=': 
-	    p->sym = B_EQ;
 	    parser_getc(p);
+	    if (p->ch == '=') {
+		/* allow "==" as synonym for "=" */
+		parser_getc(p);
+	    }
+	    p->sym = B_EQ;
 	    return;
         case '>': 
 	    parser_getc(p);
@@ -1008,47 +1145,43 @@ void lex (parser *p)
 	    }
 	    return;
         case '(': 
-	    p->sym = LPR;
+	    p->sym = G_LPR;
 	    parser_getc(p);
 	    return;
         case ')': 
-	    p->sym = RPR;
+	    p->sym = G_RPR;
 	    parser_getc(p);
 	    return;
         case '[': 
-	    p->sym = LBR;
+	    p->sym = G_LBR;
 	    parser_getc(p);
 	    return;
         case '{': 
-	    p->sym = LCB;
+	    p->sym = G_LCB;
 	    parser_getc(p);
 	    return;
         case '}': 
-	    p->sym = RCB;
+	    p->sym = G_RCB;
 	    parser_getc(p);
 	    return;
         case ']': 
-	    p->sym = RBR;
+	    p->sym = G_RBR;
 	    parser_getc(p);
 	    return;
         case '~':
-	    p->sym = MCCAT;
-	    parser_getc(p);
-	    return;
-        case '`': 
-	    p->sym = MRCAT;
+	    p->sym = B_HCAT;
 	    parser_getc(p);
 	    return;
         case ',': 
-	    p->sym = COM;
+	    p->sym = P_COM;
 	    parser_getc(p);
 	    return;
         case ';': 
-	    p->sym = SEMI;
+	    p->sym = P_SEMI;
 	    parser_getc(p);
 	    return;
         case ':': 
-	    p->sym = COL;
+	    p->sym = P_COL;
 	    parser_getc(p);
 	    return;
         case '?': 
@@ -1057,58 +1190,69 @@ void lex (parser *p)
 	    return;
 	case '.':
 	    if (*p->point == '$') {
-		p->sym = DOT;
+		p->sym = P_DOT;
 		parser_getc(p);
 		return;
 	    }
 	    parser_getc(p);
 	    if (p->ch == '*') {
-		p->sym = DOTMULT;
+		p->sym = B_DOTMULT;
 		parser_getc(p);
 		return;
 	    } else if (p->ch == '/') {
-		p->sym = DOTDIV;
+		p->sym = B_DOTDIV;
 		parser_getc(p);
 		return;
 	    } else if (p->ch == '^') {
-		p->sym = DOTPOW;
+		p->sym = B_DOTPOW;
 		parser_getc(p);
 		return;
 	    } else if (p->ch == '+') {
-		p->sym = DOTADD;
+		p->sym = B_DOTADD;
 		parser_getc(p);
 		return;
 	    } else if (p->ch == '-') {
-		p->sym = DOTSUB;
+		p->sym = B_DOTSUB;
 		parser_getc(p);
 		return;
 	    } else if (p->ch == '=') {
-		p->sym = DOTEQ;
+		p->sym = B_DOTEQ;
 		parser_getc(p);
 		return;
 	    } else if (p->ch == '>') {
-		p->sym = DOTGT;
+		p->sym = B_DOTGT;
 		parser_getc(p);
 		return;
 	    } else if (p->ch == '<') {
-		p->sym = DOTLT;
+		p->sym = B_DOTLT;
 		parser_getc(p);
 		return;
 	    } else {
-		/* not a "dot operator", back up */
+		/* not a "dot operator", so back up */
 		parser_ungetc(p);
 	    }
         default: 
+	    if (p->targ == LIST && lag_range_sym(p)) {
+		p->sym = B_RANGE;
+		parser_getc(p);
+		parser_getc(p);
+		return;
+	    }
+	    if (bare_data_type(p->sym) || closing_sym(p->sym) ||
+		(p->targ == LIST && p->sym == LAG)) {
+		p->sym = B_LCAT;
+		return;
+	    }
 	    if (isdigit(p->ch) || (p->ch == '.' && isdigit(*p->point))) {
 		p->xval = getdbl(p);
 		p->sym = NUM;
 		return;
-	    } else if (islower(p->ch) || isupper(p->ch) || p->ch == '$') {
+	    } else if (islower(p->ch) || isupper(p->ch) || 
+		       word_start_special(p->ch)) {
 		getword(p);
 		return;
 	    } else if (p->ch == '"') {
 		p->idstr = get_quoted_string(p);
-		p->sym = STR;
 		return;
 	    } else {
 		parser_print_input(p);
@@ -1122,8 +1266,8 @@ void lex (parser *p)
 
 const char *getsymb (int t, const parser *p)
 {  
-    if ((t > OP_MAX && t < FUNC_MAX) ||
-	(t > FUNC_MAX && t < F2_MAX) ||
+    if ((t > F1_MIN && t < F1_MAX) ||
+	(t > F1_MAX && t < F2_MAX) ||
 	(t > F2_MAX && t < FN_MAX)) {
 	return funname(t);
     }
@@ -1155,12 +1299,16 @@ const char *getsymb (int t, const parser *p)
 	return "LIST";
     } else if (t == OVAR) {
 	return "OVAR";
+    } else if (t == VSTR) {
+	return "VSTR";
+    } else if (t == EMPTY) {
+	return "EMPTY";
     }
 
     if (p != NULL) {
 	if (t == NUM) {
 	    return fromdbl(p->xval); 
-	} else if (t == UVAR) {
+	} else if (t == USCALAR || t == USERIES) {
 	    return p->dinfo->varname[p->idnum];
 	} else if (t == UMAT || t == UOBJ ||
 		   t == LOOPIDX) {
@@ -1213,53 +1361,56 @@ const char *getsymb (int t, const parser *p)
 	return "<=";
     case B_AND: 
 	return "&&";
+    case B_RANGE:
+	return " to ";
     case U_ADDR:
 	return "&";
     case B_OR: 
 	return "||";	
     case U_NOT: 
 	return "!";
-    case LPR: 
+    case G_LPR: 
 	return "(";
-    case RPR: 
+    case G_RPR: 
 	return ")";
-    case LBR: 
+    case G_LBR: 
 	return "[";
-    case RBR: 
+    case G_RBR: 
 	return "]";
-    case LCB: 
+    case G_LCB: 
 	return "{";
-    case RCB: 
+    case G_RCB: 
 	return "}";
-    case DOTMULT: 
+    case B_DOTMULT: 
 	return ".*";
-    case DOTDIV: 
+    case B_DOTDIV: 
 	return "./";
-    case DOTPOW: 
+    case B_DOTPOW: 
 	return ".^";
-    case DOTADD: 
+    case B_DOTADD: 
 	return ".+";
-    case DOTSUB: 
+    case B_DOTSUB: 
 	return ".-";
-    case DOTEQ: 
+    case B_DOTEQ: 
 	return ".=";
-    case DOTGT: 
+    case B_DOTGT: 
 	return ".>";
-    case DOTLT: 
+    case B_DOTLT: 
 	return ".<";
-    case KRON: 
+    case B_KRON: 
 	return "**";
-    case MCCAT: 
+    case B_HCAT: 
 	return "~";
-    case MRCAT: 
+    case B_VCAT: 
+    case B_LCAT:
 	return "|";
-    case COM: 
+    case P_COM: 
 	return ",";
-    case DOT: 
+    case P_DOT: 
 	return ".";
-    case SEMI: 
+    case P_SEMI: 
 	return ";";
-    case COL: 
+    case P_COL: 
 	return ":";
     case QUERY: 
 	return "?";
