@@ -40,7 +40,8 @@ static void gen_write_message (const parser *p, int oldv, PRN *prn)
 	    pprintf(prn, _("Modified series %s (ID %d)"),
 		    p->lh.name, p->lh.v);
 	} else {
-	    double x = (*p->Z)[p->lh.v][p->lh.obs];
+	    int t = lh_obs(p);
+	    double x = (*p->Z)[p->lh.v][t];
 
 	    if (p->lh.v < oldv) {
 		pprintf(prn, _("Replaced scalar %s (ID %d)"),
@@ -64,16 +65,28 @@ static void gen_write_message (const parser *p, int oldv, PRN *prn)
 		    p->lh.name, p->lh.v);
 	}
     } else if (p->targ == MAT) {
-	if (p->lh.m0 != NULL && p->lh.substr != NULL && 
-	    *p->lh.substr !='\0') {
+	if ((p->flags & P_LHMAT) && p->lh.substr != NULL && 
+	    *p->lh.substr != '\0') {
 	    pprintf(prn, _("Modified matrix %s"), p->lh.name);
-	} else if (p->lh.m0 != NULL) {
+	} else if (p->flags & P_LHMAT) {
 	    pprintf(prn, _("Replaced matrix %s"), p->lh.name);
 	} else {
 	    pprintf(prn, _("Generated matrix %s"), p->lh.name);
 	}
     } else if (p->targ == LIST) {
-	pprintf(prn, _("Generated list %s"), p->lh.name);
+	if (p->flags & P_LHLIST) {
+	    pprintf(prn, _("Replaced list %s"), p->lh.name);
+	} else {
+	    pprintf(prn, _("Generated list %s"), p->lh.name);
+	}
+    } else if (p->targ == STR) {
+	if (p->flags & P_LHSTR) {
+	    pprintf(prn, _("Replaced string %s"), p->lh.name);
+	} else {
+	    pprintf(prn, _("Generated string %s"), p->lh.name);
+	}
+    } else {
+	return;
     }
 
     pputc(prn, '\n');
@@ -83,7 +96,7 @@ static void gen_write_warning (const parser *p, PRN *prn)
 {
     if (prn != NULL && !repeating_function_exec()) {
 	if (*p->warning != '\0') {
-	    pprintf(prn, "%s: %s: %s\n", _("Warning"),
+	    pprintf(prn, "%s: %s\n", 
 		    p->warning, _("missing values were generated"));
 	} else {
 	    pprintf(prn, "%s: %s\n", _("Warning"),
@@ -92,6 +105,45 @@ static void gen_write_warning (const parser *p, PRN *prn)
     }
 }
 
+static int maybe_record_lag_info (parser *p)
+{
+    const char *s = p->input;
+    int n = strlen(p->lh.name);
+    char vname[VNAMELEN];
+    int lag;
+
+    if (!strncmp(s, "genr ", 5)) {
+	s += 5;
+    } else if (!strncmp(s, "series ", 7)) {
+	s += 7;
+    }
+
+    s += strspn(s, " ");
+
+    if (!strncmp(s, p->lh.name, n)) {
+	s += n;
+	s += strspn(s, " ");
+	if (*s == '=') s++;
+	s += strspn(s, " ");
+    }
+
+    if (sscanf(s, "%15[^ ()](%d)", vname, &lag) == 2) {
+	s = strchr(s, ')');
+	if (s != NULL && string_is_blank(s + 1)) {
+	    int pv = varindex(p->dinfo, vname);
+
+	    if (pv < p->dinfo->v) {
+		strcpy(p->dinfo->varinfo[p->lh.v]->parent, 
+		   p->dinfo->varname[pv]);
+		p->dinfo->varinfo[p->lh.v]->transform = LAGS;
+		p->dinfo->varinfo[p->lh.v]->lag = -lag;
+	    }
+	}
+    }
+
+    return 0;
+}
+
 static void gen_write_label (parser *p, int oldv)
 {
     char tmp[MAXLABEL];
@@ -109,6 +161,8 @@ static void gen_write_label (parser *p, int oldv)
 	return;
     }
 
+    maybe_record_lag_info(p);
+
     *tmp = '\0';
 
     if (p->lh.v < oldv && p->targ == VEC) {
@@ -177,6 +231,27 @@ int function_from_string (const char *s)
     return 0;
 }
 
+/**
+ * is_gretl_function_call:
+ * @s: the string to check.
+ *
+ * Returns: 1 if @s starts with the name of a built-in 
+ * gretl function, otherwise 0.
+ */
+
+int is_gretl_function_call (const char *s)
+{
+    char c, word[9];
+
+    sscanf(s, "%8[^ (]", word);
+    c = s[strlen(word)];
+    if (c == ' ' || c == '(') {
+	return function_lookup(word);
+    } else {
+	return 0;
+    }
+}
+
 /* "reserved word" data */
 
 static const char *res1[] = {
@@ -199,16 +274,15 @@ static const char *res2[] = {
  * gretl_reserved_word:
  * @str: string to be tested.
  *
- * Returns non-zero if @str is a reserved word that cannot figure as the
- * name of a user-defined variable, otherwise 0.
+ * Returns non-zero if @str is a reserved word that cannot 
+ * figure as the name of a user-defined variable, otherwise 0.
  */
 
 int gretl_reserved_word (const char *str)
 {
     const char *ruses[] = {
 	N_("a constant"),
-	N_("an internal variable"),
-	N_("a function")
+	N_("an internal variable")
     };
     static int n1 = sizeof res1 / sizeof res1[0];
     static int n2 = sizeof res2 / sizeof res2[0];
@@ -226,10 +300,6 @@ int gretl_reserved_word (const char *str)
 	}
     }
 
-    if (!ret && function_from_string(str)) {
-	ret = 3;
-    }
-
     if (ret > 0) {
 	sprintf(gretl_errmsg, _("'%s' refers to %s and may not be used as a "
 			    "variable name"), str, _(ruses[ret - 1])); 
@@ -316,8 +386,9 @@ static int gen_special (const char *s, const char *line,
 			double ***pZ, DATAINFO *pdinfo, 
 			PRN *prn, parser *p)
 {
+    const char *msg = NULL;
     int orig_v = pdinfo->v;
-    int msg = 0;
+    int write_label = 0;
     int err = 0;
 
     if (!strcmp(s, "markers")) {
@@ -327,36 +398,42 @@ static int gen_special (const char *s, const char *line,
 
 	if (di0 == 0) {
 	    err = 1;
-	} else if (di0 == orig_v) {
-	    pputs(prn, _("Periodic dummy variables generated.\n"));
-	} else {
-	    pputs(prn, _("Periodic dummy variables already present.\n"));
+	} else { 
+	    if (di0 == orig_v) {
+		msg = N_("Periodic dummy variables generated.\n");
+	    } else {
+		msg = N_("Periodic dummy variables already present.\n");
+	    }
 	}
     } else if (!strcmp(s, "timedum")) {
 	err = panel_dummies(pZ, pdinfo, OPT_T);
 	if (!err) {
-	    pputs(prn, _("Panel dummy variables generated.\n"));
+	    msg = N_("Panel dummy variables generated.\n");
 	}
     } else if (!strcmp(s, "unitdum")) {
 	err = panel_dummies(pZ, pdinfo, OPT_NONE);
 	if (!err) {
-	    pputs(prn, _("Panel dummy variables generated.\n"));
+	    msg = N_("Panel dummy variables generated.\n");
 	}
     } else if (!strcmp(s, "time")) {
 	err = gen_time(pZ, pdinfo, 1);
-	msg = 1;
+	write_label = 1;
     } else if (!strcmp(s, "index")) {
 	err = gen_time(pZ, pdinfo, 0);
-	msg = 1;
+	write_label = 1;
     } else if (!strcmp(s, "unit")) {
 	err = gen_unit(pZ, pdinfo);
-	msg = 1;
+	write_label = 1;
     } else if (!strcmp(s, "weekday")) {
 	err = gen_wkday(pZ, pdinfo);
-	msg = 1;
+	write_label = 1;
     } 
 
-    if (!err && msg) {
+    if (msg != NULL && gretl_messages_on()) {
+	pputs(prn, _(msg));
+    }
+
+    if (!err && write_label) {
 	strcpy(p->lh.name, s);
 	p->lh.v = varindex(pdinfo, s);
 	p->Z = pZ;
@@ -438,6 +515,7 @@ static int is_gen_special (const char *s, char *spec, const char **rem)
                         !(f & P_DISCARD) && \
                         !(f & P_PRIVATE) && \
                         !(f & P_UFUN) && \
+                        !(f & P_QUIET) && \
                         !(f & P_DECL))
 
 int generate (const char *line, double ***pZ, DATAINFO *pdinfo,
@@ -457,6 +535,10 @@ int generate (const char *line, double ***pZ, DATAINFO *pdinfo,
 	flags |= P_UFUN;
     }
 
+    if (opt & OPT_Q) {
+	flags |= P_QUIET;
+    }
+
 #if GDEBUG
     fprintf(stderr, "\n*** generate: line = '%s'\n", line);
 #endif
@@ -484,7 +566,7 @@ int generate (const char *line, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    if (!p.err && p.warn) {
+    if (!p.err && p.warn && gretl_warnings_on()) {
 	gen_write_warning(&p, prn);
     }
 
@@ -533,14 +615,15 @@ double *generate_series (const char *s, double ***pZ,
     *err = realgen(s, &p, pZ, pdinfo, NULL, P_SERIES | P_PRIVATE);
 
     if (!*err) {
-	if (p.ret->t == VEC) {
-	    if (p.ret->flags & TMP_NODE) {
+	NODE *n = p.ret;
+
+	if (n->t == VEC) {
+	    if (n->flags & TMP_NODE) {
 		/* steal the generated series */
-		x = p.ret->v.xvec;
-		p.ret->v.xvec = NULL;
+		x = n->v.xvec;
+		n->v.xvec = NULL;
 	    } else {
-		/* copy it */
-		x = copyvec(p.ret->v.xvec, p.dinfo->n);
+		x = copyvec(n->v.xvec, p.dinfo->n);
 	    }
 	} else {
 	    *err = E_TYPES;
@@ -552,6 +635,110 @@ double *generate_series (const char *s, double ***pZ,
     return x;
 }
 
+/* retrieve a matrix result directly */
+
+gretl_matrix *generate_matrix (const char *s, double ***pZ, 
+			       DATAINFO *pdinfo, int *err)
+{
+    gretl_matrix *m = NULL;
+    parser p;
+
+    *err = realgen(s, &p, pZ, pdinfo, NULL, P_MATRIX | P_PRIVATE);
+
+    if (!*err) {
+	NODE *n = p.ret;
+
+	if (n->t == MAT) {
+	    if (n->flags & TMP_NODE) {
+		/* steal the generated matrix */
+		m = n->v.m;
+		n->v.m = NULL;
+	    } else {
+		m = gretl_matrix_copy(n->v.m);
+		if (m == NULL) {
+		    *err = E_ALLOC;
+		}
+	    }
+	} else if (n->t == NUM) {
+	    if (xna(n->v.xval)) {
+		*err = E_NAN;
+	    } else {
+		m = gretl_matrix_alloc(1, 1);
+		if (m == NULL) {
+		    *err = E_ALLOC;
+		} else {
+		    m->val[0] = n->v.xval;
+		}
+	    }
+	} else {
+	    *err = E_TYPES;
+	}
+    }
+
+    gen_cleanup(&p);
+
+    return m;
+}
+
+/* retrieve a string result directly */
+
+char *generate_string (const char *s, double ***pZ, 
+		       DATAINFO *pdinfo, int *err)
+{
+    parser p;
+    char *ret = NULL;
+
+    *err = realgen(s, &p, pZ, pdinfo, NULL, P_STRING | P_PRIVATE);
+
+    if (!*err) {
+	if (p.ret->t == STR) {
+	    ret = gretl_strdup(p.ret->v.str);
+	} else {
+	    *err = E_TYPES;
+	}
+    }
+
+    gen_cleanup(&p);
+
+    return ret;
+}
+
+/* retrieve a list result directly */
+
+int *generate_list (const char *s, double ***pZ, 
+		    DATAINFO *pdinfo, int *err)
+{
+    parser p;
+    int *ret = NULL;
+
+    *err = realgen(s, &p, pZ, pdinfo, NULL, P_LIST | P_PRIVATE);
+
+    if (!*err) {
+	if (p.ret->t == LIST) {
+	    const int *nlist = get_list_by_name(p.ret->v.str);
+
+	    if (nlist == NULL) {
+		*err = E_DATA;
+	    } else {
+		ret = gretl_list_copy(nlist);
+	    }
+	} else if (p.ret->t == LVEC) {
+	    ret = p.ret->v.ivec;
+	    p.ret->v.ivec = NULL;
+	} else {
+	    *err = E_TYPES;
+	}
+    }
+
+    if (ret == NULL && !*err) {
+	*err = E_ALLOC;
+    }
+
+    gen_cleanup(&p);
+
+    return ret;
+}
+
 /* retrieve and print a variable from "within" a saved
    object
 */
@@ -575,9 +762,10 @@ int print_object_var (const char *oname, const char *param,
    probably multiple times */
 
 parser *genr_compile (const char *s, double ***pZ, DATAINFO *pdinfo, 
-		      int *err)
+		      gretlopt opt, int *err)
 {
     parser *p = malloc(sizeof *p);
+    int flags = P_COMPILE;
 
 #if GDEBUG
     fprintf(stderr, "\n*** genr_compile: s = '%s'\n", s);
@@ -588,7 +776,15 @@ parser *genr_compile (const char *s, double ***pZ, DATAINFO *pdinfo,
 	return NULL;
     }
 
-    *err = realgen(s, p, pZ, pdinfo, NULL, P_COMPILE | P_PRIVATE);
+    if (opt & OPT_P) {
+	flags |= P_PRIVATE;
+    }
+
+    *err = realgen(s, p, pZ, pdinfo, NULL, flags);
+
+#if GDEBUG
+    fprintf(stderr, "genr_compile: err = %d\n", *err);
+#endif
 
     return p;
 }
@@ -596,19 +792,28 @@ parser *genr_compile (const char *s, double ***pZ, DATAINFO *pdinfo,
 /* run a previously compiled generator */
 
 int execute_genr (parser *p, double ***pZ, DATAINFO *pdinfo,
-		  PRN *prn)
+		  gretlopt opt, PRN *prn)
 {
+    int flags = P_EXEC;
+
 #if GDEBUG
     fprintf(stderr, "\n*** execute_genr: p=%p, LHS='%s'\n", 
 	    (void *) p, p->lh.name);
 #endif
 
-    realgen(NULL, p, pZ, pdinfo, prn, P_EXEC);
-    gen_save_or_print(p, prn);
+    if (opt & OPT_L) {
+	flags |= P_LOOP;
+    } else if (opt & OPT_S) {
+	flags |= P_SLAVE;
+    }
+
+    realgen(NULL, p, pZ, pdinfo, prn, flags);
+
+    if (p->err == 0) {
+	gen_save_or_print(p, prn);
+    }
+
     gen_cleanup(p);
-#if PRESERVE_AUX_NODES
-    p->ecount += 1;
-#endif
 
 #if GDEBUG
     fprintf(stderr, "execute_genr: returning %d\n", p->err);
@@ -636,6 +841,20 @@ void destroy_genr (parser *p)
     }
 }
 
+void genr_set_loopline (parser *p, int i)
+{
+    p->loopline = i;
+}
+
+int genr_get_loopline (parser *p)
+{
+    if (p == NULL) {
+	return -1;
+    } else {
+	return p->loopline;
+    }
+}
+
 int genr_get_output_varnum (const parser *p)
 {
     return p->lh.v;
@@ -650,6 +869,11 @@ gretl_matrix *genr_get_output_matrix (const parser *p)
     }
 }
 
+const char *genr_get_formula (const GENERATOR *genr)
+{
+    return genr->input;
+}
+
 int genr_is_print (const parser *p)
 {
     return (p->flags & P_PRINT);
