@@ -33,8 +33,10 @@ typedef enum {
     R_TEST_PVAL,  /* p-value from last explicit test performed */
     R_TEST_LNL,   /* log-likelihood from last test (if applicable) */
     R_STOPWATCH,  /* stopwatch */ 
+    R_NSCAN,      /* number of items scanned via sscanf */
     R_SCALAR_MAX, /* separator: scalars vs series */
     R_INDEX,      /* consecutive observations index */
+    R_PUNIT,      /* 1-based panel unit index */
     R_MAX
 } RetrievalIndex;
 
@@ -51,12 +53,13 @@ typedef enum {
     M_BIC,        /* Bayesian info criterion */
     M_HQC,        /* Hannan-Quinn criterion */
     M_TRSQ,       /* T * R-squared, last model */
-    M_SCALAR_MAX, /* -- separator -- */
+    M_SCALAR_MAX, /* -- separator, scalars/series -- */
     M_UHAT,       /* residuals */
     M_YHAT,       /* fitted values */
     M_AHAT,       /* per-unit intercepts in panel model */
     M_H,          /* GARCH predicted variances */
-    M_SERIES_MAX, /* -- separator -- */
+    M_SAMPLE,     /* observations used in estimation */
+    M_SERIES_MAX, /* -- separator, series/matrices -- */
     M_COEFF,      /* parameter estimates */
     M_SE,         /* parameter standard errors */
     M_VCV,        /* parameter covariance matrix */
@@ -68,6 +71,14 @@ typedef enum {
     M_JS00,       /* VECM residual covariance matrix (1st differences) */
     M_JS11,       /* VECM residual covariance matrix (levels) */
     M_JS01,       /* VECM residual cross-product matrix */
+    M_HAUSMAN,    /* Hausman test after tsls */
+    M_SARGAN,     /* Sargan over-identification test after tsls */
+    M_SYSGAM,     /* Parameter matrix Gamma (simultaneous systems) */
+    M_SYSA,       /* Parameter matrix A (simultaneous systems) */
+    M_SYSB,       /* Parameter matrix B (simultaneous systems) */
+    M_FCAST,      /* last forecast generated via fcasterr */
+    M_FCERR,      /* standard errors associated with M_FCAST */
+    M_COEFF_CI,   /* (assymetric) confidence intervals for coeffs */
     M_MAX         /* sentinel */
 } ModelDataIndex;
 
@@ -81,15 +92,21 @@ int generate (const char *line, double ***pZ, DATAINFO *pdinfo,
 	      gretlopt opt, PRN *prn);
 
 GENERATOR *genr_compile (const char *s, double ***pZ, DATAINFO *pdinfo, 
-			 int *err);
- 
+			 gretlopt opt, int *err);
+
 int execute_genr (GENERATOR *genr, double ***pZ, DATAINFO *pdinfo,
-		  PRN *prn);
+		  gretlopt opt, PRN *prn);
 
 void destroy_genr (GENERATOR *genr);
 
+void genr_set_loopline (GENERATOR *genr, int i);
+
+int genr_get_loopline (GENERATOR *genr);
+
 int genr_get_output_varnum (const GENERATOR *genr);
 
+const char *genr_get_formula (const GENERATOR *genr);
+
 gretl_matrix *genr_get_output_matrix (const GENERATOR *genr);
 
 int varindex (const DATAINFO *pdinfo, const char *varname);
@@ -104,6 +121,15 @@ double generate_scalar (const char *s, double ***pZ,
 double *generate_series (const char *s, double ***pZ, 
 			 DATAINFO *pdinfo, int *err);
 
+gretl_matrix *generate_matrix (const char *s, double ***pZ, 
+			       DATAINFO *pdinfo, int *err);
+
+char *generate_string (const char *s, double ***pZ, 
+		       DATAINFO *pdinfo, int *err);
+
+int *generate_list (const char *s, double ***pZ, 
+		    DATAINFO *pdinfo, int *err);
+
 int print_object_var (const char *oname, const char *param,
 		      double ***pZ, DATAINFO *pdinfo,
 		      PRN *prn);
@@ -112,7 +138,7 @@ int gretl_reserved_word (const char *str);
 
 int genr_special_word (const char *s);
 
-/* following functions used in nls.c */
+int genr_function_word (const char *s);
 
 int genr_is_print (const GENERATOR *p);
 
@@ -122,7 +148,13 @@ void genr_unset_na_check (GENERATOR *genr);
 
 int function_from_string (const char *s);
 
+int is_gretl_function_call (const char *s);
+
+int function_lookup (const char *s);
+
 int const_lookup (const char *s);
 
+void gretl_function_hash_cleanup (void);
+
 #endif /* GENMAIN_H */
 
