@@ -25,15 +25,14 @@
 
 #define GENDEBUG 0
 
-/* this may be a good idea, but perhaps risky? */
-#define PRESERVE_AUX_NODES 0
+/* operators, types, punctuation */
 
 enum {
               U_NEG = 1,
               U_POS,
               U_NOT,
               U_ADDR,
-              U_MAX,   /* separator: end of unary operators */ 
+              U_MAX,      /* SEPARATOR: end of unary operators */ 
               B_ASN,
               B_ADD,
               B_SUB,
@@ -50,175 +49,47 @@ enum {
               B_AND,
   /* 20 */    B_OR,
               B_TRMUL,
-              LPR,     /* left paren */
-              RPR,     /* right paren */
-              LBR,     /* left bracket */
-              RBR,     /* right bracket */
-              LCB,     /* left curly bracket */
-              RCB,     /* right curly bracket */
-              DOTMULT,
-              DOTDIV,
-  /* 30 */    DOTPOW,
-              DOTADD,
-              DOTSUB,
-              DOTEQ,
-              DOTGT,
-              DOTLT,
-              KRON,     /* Kronecker product */
-              MCCAT,    /* matrix concatenation (columns) */
-              MRCAT,    /* matrix concatenation (rows) */
-              OP_MAX,   /* separator: end of operators */
-  /* 40 */    ABS,
-              TOINT,
-              CEIL,
-	      FLOOR,
-	      SIN,
-	      COS,
-	      TAN,
-	      ATAN,
-	      LOG,
-	      LOG10,
-  /* 50 */    LOG2,
-	      EXP,
-	      SQRT,
-	      DIF,	  /* first difference */
-	      LDIF,	  /* log difference */
-	      SDIF,	  /* seasonal difference */
-	      SORT,	  /* ascending sort */
-	      DSORT,	  /* descending sort */
-	      RANKING,    
-	      ODEV,	  /* orthogonal deviation */
-  /* 60 */    NOBS,
-              T1,
-	      T2,   
-	      CUM,
-	      MISSING,
-	      OK,
-	      MISSZERO,
-	      ZEROMISS,
-	      MEDIAN,
-	      GINI,
-  /* 70 */    SUM,   
-	      MEAN,
-	      MIN,
-	      MAX,
-	      SD,
-	      VCE,	  /* variance */
-	      SST,
-	      CNORM,
-	      DNORM,
-	      QNORM,
-  /* 80 */    GAMMA,	
-	      LNGAMMA,
-	      HPFILT,
-	      BKFILT,
-	      RESAMPLE,
-	      PMEAN,
-	      PSD,
-	      IMAT,
-	      SUMR,
-	      SUMC,
-  /* 90 */    MEANR,	 
-	      MEANC,
-	      MCOV,
-	      MCORR,
-	      CDEMEAN,
-	      CHOL,
-	      INV,
-	      GINV,
-	      DIAG,
-	      TRANSP,
-  /* 100 */   TVEC,
-	      VECH,	
-	      UNVECH,
-	      ROWS,
-	      COLS,
-	      DET,
-	      LDET,
-	      TRACE,
-	      NORM1,
-	      INFNORM,
-  /* 110 */   RCOND,
-	      RANK, 
-	      OBSNUM,
-	      ISSERIES,
-	      ISLIST,	 
-	      ISSTRING,
-	      ISNULL,
-	      LISTLEN,
-	      PVAL,
-	      CDF,
-  /* 120 */   INVCDF,
-	      CRIT,  
-	      RANDGEN,
-	      RPOISSON,
-	      MAKEMASK,
-	      VALUES,
-	      NULLSPC,
-	      MEXP,
-	      MINC,
-	      MAXC,
-  /* 130 */   MINR,
-	      MAXR,
-	      IMINC, 
-	      IMAXC,
-	      IMINR,
-	      IMAXR,
-	      FFT,
-	      FFTI,
-	      UPPER,
-	      LOWER,
-  /* 140 */   MREAD,
-	      FUNC_MAX,	  /* separator: end of single-arg functions */
-	      COR,
-	      COV,
-	      SORTBY,
-	      RUNIFORM,
-	      RNORMAL,
-	      FRACDIF,
-	      ZEROS,
-	      ONES,
-  /* 150 */   SEQ,
-	      MUNIF,
-	      MNORM,
-	      QFORM,
-	      MLAG,
-	      QR,
-	      EIGSYM,	 
-	      EIGGEN,
-	      FDJAC,
-	      BFGSMAX,
-  /* 160 */   LRVAR,
-	      PRINCOMP,
-	      QUANTILE,
-	      CMULT,	  /* complex multiplication */
-	      CDIV,	  /* complex division */
-	      MXTAB,
-	      MWRITE,
-	      MOVAVG,
-	      F2_MAX,	  /* separator: end of two-arg functions */
-	      MSHAPE,
-  /* 170 */   SVD,
-	      MOLS,
-	      FN_MAX,	  /* separator: end of n-arg functions */
-	      COM,	  /* comma */
-	      DOT,	  /* period */
-	      SEMI,	  /* semi-colon */
-	      COL,	  /* colon */
-	      CON,	  /* named constant */
-	      DUM,	  /* dummy variable */
-	      UVAR,	  /* user variable (scalar or series) */
-  /* 180 */   UMAT,	  /* user-defined matrix */
+	      B_RANGE,
+              B_DOTMULT,
+	      B_DOTDIV,
+	      B_DOTPOW,
+              B_DOTADD,
+              B_DOTSUB,
+              B_DOTEQ,
+              B_DOTGT,
+  /* 30 */    B_DOTLT,
+              B_KRON,     /* Kronecker product */
+              B_HCAT,     /* horizontal concatenation */
+              B_VCAT,     /* vertical concatenation */
+	      B_LCAT,     /* list concatentation */
+	      OP_MAX,     /* SEPARATOR: end of binary operators */
+              G_LPR,      /* grouping: left paren */
+              G_RPR,      /* right paren */
+              G_LBR,      /* left bracket */
+              G_RBR,      /* right bracket */
+  /* 40 */    G_LCB,      /* left curly bracket */
+	      G_RCB,      /* right curly bracket */  
+	      P_COM,	  /* punctuation: comma */
+	      P_DOT,	  /* period */
+	      P_SEMI,	  /* semi-colon */
+	      P_COL,	  /* colon */
+	      PUNCT_MAX,  /* SEPARATOR: end of grouping and punctuation marks */
+              CON,	  /* named constant */
+              DUM,	  /* dummy variable */
+	      USCALAR,	  /* user variable, scalar */
+  /* 50 */    USERIES,    /* user variable, series */
+	      UMAT,	  /* user-defined matrix */
 	      UOBJ,	  /* user-defined object (e.g. model) */
 	      NUM,	  /* scalar, evaluated */
 	      VEC,	  /* series, evaluated */
 	      IVEC,	  /* vector of integers, evaluated */
+	      LVEC,       /* list, as array of ints */
 	      MAT,	  /* matrix, evaluated */
 	      OBS,	  /* observation from a series */
 	      MSL,	  /* matrix plus subspec */
-	      DMSL,	  /* "dollar" matrix plus subspec */
+  /* 60 */    DMSL,	  /* "dollar" matrix plus subspec */
 	      DMSTR,	  /* "dollar" matrix plus old-style string subspec */
-  /* 190 */   MSL2,	  /* unevaluated matrix subspec */
+	      MSL2,	  /* unevaluated matrix subspec */
 	      MSPEC,	  /* evaluated matrix subspec */
 	      SUBSL,	  /* row or column component of MSPEC */
 	      MDEF,	  /* explicit matrix definition {...} */
@@ -226,19 +97,190 @@ enum {
 	      DVAR,	  /* $ dataset variable (scalar or series) */
 	      MVAR,	  /* $ model var (scalar, series, or matrix) */
 	      OVAR,	  /* object variable: variable "under" an object */
-	      LOOPIDX,	  /* loop index variable */
+  /* 70 */    LOOPIDX,	  /* loop index variable */
 	      LIST,	  /* reference to named list */
-  /* 200 */   STR,	  /* string */
-	      EROOT,	  /* dummy root for (...) expression */
-	      UFUN,	  /* user-defined function */
+	      STR,	  /* string */
 	      FARGS,	  /* set of n function arguments */
-	      EMPTY,
+	      EMPTY,      /* "null" */
 	      ABSENT,
-	      INC,   
+	      DTYPE_MAX,  /* SEPARATOR: end of "bare" types */
+	      EROOT,	  /* dummy root for (...) expression */
+              UFUN,	  /* user-defined function */
+	      VSTR,       /* string variable */
+  /* 80 */    INC,   
 	      DEC,
 	      QUERY,
-	      EOT,	    /* end of transmission */
-  /* 210 */   UNK 
+	      EOT,	  /* end of transmission */
+	      UNK 
+};
+
+/* functions: don't collide with the enumeration above */
+
+enum {
+    F1_MIN = 1 << 8,
+    F_ABS,
+    F_TOINT,
+    F_CEIL,
+    F_FLOOR,
+    F_ROUND,
+    F_SIN,
+    F_COS,
+    F_TAN,
+    F_ASIN,
+    F_ACOS,
+    F_ATAN,
+    F_LOG,
+    F_LOG10,
+    F_LOG2,
+    F_EXP,
+    F_SQRT,
+    F_DIFF,	  /* first difference */
+    F_LDIFF,	  /* log difference */
+    F_SDIFF,	  /* seasonal difference */
+    F_SORT,	  /* ascending sort */
+    F_DSORT,	  /* descending sort */
+    F_RANKING,    
+    F_ODEV,	  /* orthogonal deviation */
+    F_NOBS,
+    F_T1,
+    F_T2,   
+    F_CUM,
+    F_MISSING,
+    F_DATAOK,
+    F_MISSZERO,
+    F_ZEROMISS,
+    F_MEDIAN,
+    F_GINI,
+    F_SUM,   
+    F_MEAN,
+    F_MIN,
+    F_MAX,
+    F_SD,
+    F_VCE,	  /* variance */
+    F_SST,
+    F_CNORM,
+    F_DNORM,
+    F_QNORM,
+    F_GAMMA,	
+    F_LNGAMMA,
+    F_HPFILT,
+    F_BKFILT,
+    F_RESAMPLE,
+    F_PMEAN,
+    F_PSD,
+    F_IMAT,
+    F_SUMR,
+    F_SUMC,
+    F_MEANR,	 
+    F_MEANC,
+    F_SDC,
+    F_MCOV,
+    F_MCORR,
+    F_CDEMEAN,
+    F_CHOL,
+    F_INV,
+    F_INVPD,
+    F_GINV,
+    F_DIAG,
+    F_TRANSP,
+    F_VEC,
+    F_VECH,	
+    F_UNVECH,
+    F_ROWS,
+    F_COLS,
+    F_DET,
+    F_LDET,
+    F_TRACE,
+    F_NORM1,
+    F_INFNORM,
+    F_RCOND,
+    F_RANK, 
+    F_OBSNUM,
+    F_ISSERIES,
+    F_ISLIST,	 
+    F_ISSTRING,
+    F_ISNULL,
+    F_LISTLEN,
+    F_PDF,
+    F_PVAL,
+    F_CDF,
+    F_INVCDF,
+    F_CRIT,  
+    F_RANDGEN,
+    F_RPOISSON,
+    F_MAKEMASK,
+    F_VALUES,
+    F_NULLSPC,
+    F_MEXP,
+    F_MINC,
+    F_MAXC,
+    F_MINR,
+    F_MAXR,
+    F_IMINC, 
+    F_IMAXC,
+    F_IMINR,
+    F_IMAXR,
+    F_FFT,
+    F_FFTI,
+    F_UPPER,
+    F_LOWER,
+    F_MREAD,
+    F_POLROOTS,
+    F_DUMIFY,
+    F_XPX,
+    F_GETENV,
+    F_ARGNAME,
+    F_OBSLABEL,
+    F_READFILE,
+    F_BACKTICK,
+    F_STRLEN,
+    F_VARNAME,
+    F_VARNUM,
+    F1_MAX,	  /* SEPARATOR: end of single-arg functions */
+    F_COR,
+    F_COV,
+    F_SORTBY,
+    F_RUNIFORM,
+    F_RNORMAL,
+    F_FRACDIFF,
+    F_ZEROS,
+    F_ONES,
+    F_SEQ,
+    F_MUNIF,
+    F_MNORM,
+    F_QFORM,
+    F_MLAG,
+    F_QR,
+    F_EIGSYM,	 
+    F_EIGGEN,
+    F_FDJAC,
+    F_BFGSMAX,
+    F_LRVAR,
+    F_PRINCOMP,
+    F_QUANTILE,
+    F_CMULT,	  /* complex multiplication */
+    F_CDIV,	  /* complex division */
+    F_MXTAB,
+    F_MWRITE,
+    F_MOVAVG,
+    F_MRSEL,
+    F_MCSEL,
+    F_LLAG,
+    F_WMEAN,
+    F_WVAR,
+    F_WSD,
+    F_STRSTR,
+    F_COLNAMES,
+    F_LJUNGBOX,
+    F_MSORTBY,
+    F_LINCOMB,
+    F2_MAX,	  /* SEPARATOR: end of two-arg functions */
+    F_MSHAPE,
+    F_SVD,
+    F_MOLS,
+    F_FILTER,
+    F_TRIMR,
+    FN_MAX,	  /* SEPARATOR: end of n-arg functions */
 };
 
 enum {
@@ -250,19 +292,24 @@ enum {
 enum {
     DUM_NULL = 1,
     DUM_DIAG,
-    DUM_DATASET
+    DUM_DATASET,
+    DUM_TREND
 };
 
 #define GENSTRLEN 128
+#define NO_VNUM -1
 
-#define func_symb(s) (s > OP_MAX && s < FUNC_MAX)
-#define func2_symb(s) (s > FUNC_MAX && s < F2_MAX)
+#define func1_symb(s) (s > F1_MIN && s < F1_MAX)
+#define func2_symb(s) (s > F1_MAX && s < F2_MAX)
 #define funcn_symb(s) (s > F2_MAX && s < FN_MAX)
-#define string_arg_func(s) (s == ISSERIES || s == ISNULL || s == ISLIST || \
-                            s == LISTLEN || s == ISSTRING || s == OBSNUM)
 
-#define string0_func(s) (s == PVAL || s == CDF || s == INVCDF || \
-                         s == CRIT || s == RANDGEN)
+#define string_arg_func(s) (s == F_ISSERIES || s == F_ISNULL || \
+			    s == F_ISLIST || s == F_LISTLEN || \
+			    s == F_ISSTRING || s == F_OBSNUM || \
+			    s == F_BACKTICK || s == F_VARNUM)
+
+#define string0_func(s) (s == F_PVAL || s == F_CDF || s == F_INVCDF || \
+                         s == F_CRIT || s == F_RANDGEN || s == F_PDF)
 
 #define unary_op(s) (s >= 1 && s < U_MAX)
 #define binary_op(s) (s > U_MAX && s < OP_MAX)
@@ -272,11 +319,11 @@ enum {
                    s == MSL2 || s == SUBSL || s == LAG || \
                    s == OBS)
 
-#define b1sym(s) (unary_op(s) || func_symb(s) || funcn_symb(s) || \
-                  s == LPR || s == EROOT)
+#define b1sym(s) (unary_op(s) || func1_symb(s) || funcn_symb(s) || \
+                  s == G_LPR || s == EROOT)
 
 #define evalb1(s) (b1sym(s) && !(string0_func(s)) && s != U_ADDR && \
-                   !func2_symb(s))
+                   !func2_symb(s) && s != EROOT)
 
 #define b2sym(s) (evalb2(s) || s == DMSTR || s == DMSL || \
                   s == OVAR || s == UFUN)
@@ -286,11 +333,16 @@ enum {
 #define bnsym(s) (s == MDEF || s == FARGS)
 
 #define freestr(s) (s == STR || s == UMAT || s == UOBJ || \
-                    s == LOOPIDX || s == LIST)
+                    s == LOOPIDX || s == LIST || s == VSTR)
+
+#define bare_data_type(s) (s > PUNCT_MAX && s < DTYPE_MAX)
+
+#define closing_sym(s) (s == G_RPR || s == G_RBR || s == G_RCB)
+
 
 /* functions where the right-hand "argument" is actually a return
    location */
-#define r_return(s) (s == QR || s == EIGSYM || s == EIGGEN)
+#define r_return(s) (s == F_QR || s == F_EIGSYM || s == F_EIGGEN)
 
 #define dollar_node(n) (n->t == DVAR || n->t == MVAR || \
                         (n->t == MSL && n->v.b2.l->v.str[0] == '$'))
@@ -339,6 +391,7 @@ enum {
 struct node {
     short t;       /* type indentifier */
     char flags;    /* AUX_NODE etc., see above */
+    int vnum;      /* associated variable ID number */
     union val v;   /* value (of whatever type) */
 };
 
@@ -350,14 +403,24 @@ enum {
     P_PRINT   = 1 <<  4, /* command just prints an existing var */
     P_SCALAR  = 1 <<  5, /* return a scalar result (only) */ 
     P_SERIES  = 1 <<  6, /* return a series result (only) */
-    P_PRIVATE = 1 <<  7, /* generating a private or internal var */
-    P_COMPILE = 1 <<  8, /* just compiling tree, not evaluating */
-    P_EXEC    = 1 <<  9, /* evaluating pre-built tree */ 
-    P_SLICE   = 1 << 10, /* compute matrix slice specification */
-    P_UFUN    = 1 << 11, /* user function call with no assignment */
-    P_SORT    = 1 << 12, /* doing a simple sort of a variable */
-    P_NATEST  = 1 << 13, /* testing for NAs in expression */
-    P_UFRET   = 1 << 14  /* returning value generated by user function */
+    P_MATRIX  = 1 <<  7, /* return a matrix result (only) */
+    P_STRING  = 1 <<  8, /* return a string result (only) */
+    P_LIST    = 1 <<  9, /* return a list result (only) */
+    P_PRIVATE = 1 << 10, /* generating a private or internal var */
+    P_COMPILE = 1 << 11, /* just compiling tree, not evaluating */
+    P_EXEC    = 1 << 12, /* evaluating pre-built tree */ 
+    P_SLICE   = 1 << 13, /* compute matrix slice specification */
+    P_UFUN    = 1 << 14, /* user function call with no assignment */
+    P_SORT    = 1 << 15, /* doing a simple sort of a variable */
+    P_NATEST  = 1 << 16, /* testing for NAs in expression */
+    P_UFRET   = 1 << 17, /* returning value generated by user function */
+    P_LHLIST  = 1 << 18, /* there was a pre-existing LHS list */
+    P_LHSTR   = 1 << 19, /* there was a pre-existing LHS string */
+    P_LHMAT   = 1 << 20, /* there was a pre-existing LHS matrix */
+    P_QUIET   = 1 << 21, /* don't print any messages or labels */
+    P_GETSTR  = 1 << 22, /* state: flag acceptance of plain strings */
+    P_LOOP    = 1 << 23, /* executing pre-built genr in loop context */
+    P_SLAVE   = 1 << 24  /* running as "slave" of NLS/MLE/GMM */
 };
 
 struct lhinfo {
@@ -390,8 +453,8 @@ struct parser_ {
     NODE **aux;        /* auxiliary nodes used in evaluation */
     int n_aux;         /* the number of the above */
     int aux_i;         /* the current ID of the above */
-    int ecount;        /* number of times this parser has been exec'd */
     char warning[64];  /* to hold a warning, if needed */
+    int loopline;      /* line number within loop, if applicable */
     /* below: parser state variables */
     int obs;
     int sym;
@@ -399,7 +462,6 @@ struct parser_ {
     double xval;
     int idnum;
     char *idstr;
-    int getstr;
     int err;
     int warn;
 };
@@ -407,6 +469,7 @@ struct parser_ {
 #define starting(p) (p->flags & P_START)
 #define autoreg(p) (p->flags & P_AUTOREG)
 #define simple_ufun_call(p) (p->flags & P_UFUN)
+#define lh_obs(p) ((p->lh.obs > 0)? p->lh.obs : 0)
 
 int parser_getc (parser *p);
 void parser_ungetc (parser *p);
@@ -421,7 +484,6 @@ NODE *obs_node (parser *p);
 NODE *msl_node_direct (parser *p);
 void context_error (int c, parser *p);
 const char *getsymb (int t, const parser *p);
-int function_lookup (const char *s);
 
 int realgen (const char *s, parser *p, double ***Z, 
 	     DATAINFO *pdinfo, PRN *prn, int flags);
@@ -441,9 +503,20 @@ const char *dumname (int t);
 /* handling declarations of variables */
 int check_declarations (char ***pS, parser *p);
 
-/* handling sorted market strings */
+/* handling sorted marker strings */
 int maybe_pick_up_sorted_markers (parser *p);
 
+/* in genfuncs.c, used only internally */
+int cross_sectional_stat (double *x, const int *list, 
+			  const double **Z, 
+			  const DATAINFO *pdinfo,
+			  int f);
+int x_sectional_weighted_stat (double *x, const int *list, 
+			       const int *wlist,
+			       const double **Z, 
+			       const DATAINFO *pdinfo,
+			       int f);
+
 /* helper functions for manual, gretl.lang file */
 int gen_func_count (void);
 const char *gen_func_name (int i);
