@@ -53,6 +53,7 @@ static NODE *newempty (int t)
 	n->t = t;
 	n->v.idnum = 0;
 	n->flags = 0;
+	n->vnum = NO_VNUM;
     }
 
     return n;
@@ -68,11 +69,22 @@ static NODE *newref (parser *p, int t)
 #endif
 
     if (n != NULL) {
-	n->t = t;
-	if (t == UMAT || t == UOBJ || t == LOOPIDX || t == LIST) {
-	    n->v.str = p->idstr;
+	if (t == USCALAR) {
+	    n->vnum = p->idnum;
+	    n->t = NUM;
+	    n->v.xval = (*p->Z)[n->vnum][0];
+	} else if (t == USERIES) {
+	    n->vnum = p->idnum;
+	    n->t = VEC;
+	    n->v.xvec = (*p->Z)[n->vnum];
 	} else {
-	    n->v.idnum = p->idnum;
+	    n->t = t;
+	    if (t == UMAT || t == UOBJ || t == LOOPIDX || t == LIST) {
+		n->v.str = p->idstr;
+	    } else {
+		n->v.idnum = p->idnum;
+	    }
+	    n->vnum = NO_VNUM;
 	}
 	n->flags = 0;
     }
@@ -80,7 +92,7 @@ static NODE *newref (parser *p, int t)
     return n;
 }
 
-static NODE *newstr (parser *p)
+static NODE *newstr (parser *p, int t)
 {  
     NODE *n;
 
@@ -97,9 +109,10 @@ static NODE *newstr (parser *p)
 #endif
 
     if (n != NULL) {
-	n->t = STR;
+	n->t = t;
 	n->v.str = p->idstr;
 	n->flags = 0;
+	n->vnum = NO_VNUM;
     }
 
     return n;
@@ -120,6 +133,7 @@ NODE *newdbl (double x)
 	n->t = NUM;
 	n->v.xval = x;
 	n->flags = 0;
+	n->vnum = NO_VNUM;
     }
 
     return n;
@@ -140,6 +154,7 @@ static NODE *newb1 (int t, NODE *b)
 	n->t = t;
 	n->v.b1.b = b;
 	n->flags = 0;
+	n->vnum = NO_VNUM;
     }
 
     return n;
@@ -161,6 +176,7 @@ static NODE *newb2 (int t, NODE *l, NODE *r)
 	n->v.b2.l = l;
 	n->v.b2.r = r;
 	n->flags = 0;
+	n->vnum = NO_VNUM;
     }
 
     return n;
@@ -183,6 +199,7 @@ static NODE *newb3 (int t, NODE *l, NODE *m, NODE *r)
 	n->v.b3.m = m;
 	n->v.b3.r = r;
 	n->flags = 0;
+	n->vnum = NO_VNUM;
     }
 
     return n;
@@ -204,6 +221,7 @@ static NODE *newbn (int t)
 	n->v.bn.n_nodes = 0;
 	n->v.bn.n = NULL;
 	n->flags = 0;
+	n->vnum = NO_VNUM;
     }
 
     return n;
@@ -225,8 +243,8 @@ static int push_bn_node (NODE *t, NODE *n)
 
 #if SDEBUG
     fprintf(stderr, "push_bn_node: n_nodes now = %d, "
-	    "added node of type %d\n", t->v.bn.n_nodes,
-	    n->t);
+	    "added node of type %d (vnum = %d)\n", 
+	    t->v.bn.n_nodes, n->t, n->vnum);
 #endif
 
     return 0;
@@ -234,9 +252,18 @@ static int push_bn_node (NODE *t, NODE *n)
 
 static void expected_symbol_error (int c, parser *p)
 {
+    const char *found = getsymb(p->sym, p);
+
     parser_print_input(p);
     pprintf(p->prn, _("Expected '%c' but found '%s'\n"), c, 
-	    getsymb(p->sym, p));
+	    found);
+    if (found != NULL) {
+	if (!strcmp(found, "&")) {
+	    pputs(p->prn, "(for logical AND, please use \"&&\")\n");
+	} else if (!strcmp(found, "|")) {
+	    pputs(p->prn, "(for logical OR, please use \"||\")\n");
+	}
+    }   
     p->err = 1;
 }
 
@@ -291,7 +318,8 @@ static NODE *base (parser *p, NODE *up)
 	t = newdbl(p->xval);
 	lex(p);
 	break;
-    case UVAR: 
+    case USCALAR: 
+    case USERIES:
     case UMAT:
     case UOBJ:
     case CON: 
@@ -317,14 +345,15 @@ static NODE *base (parser *p, NODE *up)
 	break;
     case B_SUB:
     case B_ADD:
-    case DOT:
+    case B_RANGE:
+    case P_DOT:
 	lex(p);
 	t = expr(p);
 	break;
-    case LPR: /* left paren '(' */
+    case G_LPR: /* left paren '(' */
 	lex(p);
 	t = expr(p);
-	if (p->sym == RPR) {
+	if (p->sym == G_RPR) {
 	    if (up != NULL && up->t != LAG && unary_apost(p)) {
 		set_transpose(up);
 		parser_getc(p);
@@ -334,14 +363,14 @@ static NODE *base (parser *p, NODE *up)
 	    expected_symbol_error(')', p);
 	}
 	break;
-    case LBR: /* left bracket '[' */
+    case G_LBR: /* left bracket '[' */
 	if (up == NULL) {
 	    goto deferr;
 	}
 	if (up->t == OBS) {
 	    t = obs_node(p);
 	}
-	if (p->sym == RBR) {
+	if (p->sym == G_RBR) {
 	    if (up->t == MSL || up->t == DMSL) {
 		if (p->ch == '\'') {
 		    set_transpose(up);
@@ -366,55 +395,99 @@ static NODE *base (parser *p, NODE *up)
     return t;
 }
 
-/* grab parenthesized string argument, for some special
-   functions */
+/* Grab a string argument.  Note: we have a mechanism in genlex.c for
+   retrieving arguments that take the form of quoted string literals
+   or names of string variables.  The special use of this function is
+   to grab a literal string without requiring the user to wrap it in
+   quotes; we use it only where we know the only acceptable argument
+   is a string.  This function assumes that the argument in question
+   is the last (or only) argument to a function: we look for a closing
+   parenthesis and flag an error if we don't find one.
+*/
 
 static NODE *get_string_arg (parser *p)
 {
-    char str[GENSTRLEN] = {0};
-    int i, close = -1;
+    int wrapped = 0;
 
-    if (p->ch != ')') {
-	/* allow for empty arg string "()" */
-	int j, started = 0;
+    while (p->ch == ' ') {
+	parser_getc(p);
+    }
 
-	close = parser_charpos(p, ')');
+    if (p->ch == '"') {
+	wrapped = 1;
+	parser_getc(p);
+    }
 
-	if (close < 0 || close > GENSTRLEN - 2) {
-	    p->err = E_PARSE;
-	    if (close > 0) {
-		pprintf(p->prn, _("String is too long (%d versus %d max)\n"),
-			close, GENSTRLEN);
-	    } else {
-		unmatched_symbol_error('(', p);
+    if (p->ch == ')') {
+	/* allow empty arg string "()" */
+	p->idstr = gretl_strdup("");
+    } else {
+	int i, paren = 1, quoted = wrapped;
+	int close = -1, started = 0;
+	const char *s = p->point;
+
+	/* find length of string to closing paren */
+	i = 0;
+	while (*s) {
+	    if (!quoted && *s == ')') paren--;
+	    if (paren == 0) {
+		close = i;
+		break;
 	    }
+	    if (*s == '"') {
+		quoted = !quoted;
+	    } else if (!quoted && *s == '(') {
+		paren++;
+	    }
+	    s++;
+	    i++;
+	}
+
+	if (close < 0) {
+	    unmatched_symbol_error('(', p);
 	    return NULL;
 	}
 
-	j = 0;
 	for (i=0; i<=close; i++) {
 	    if (!started && !isspace(p->ch)) {
+		p->idstr = gretl_strndup(p->point - 1, close - i + 1);
 		started = 1;
 	    }
-	    if (started) {
-		str[j++] = p->ch;
-	    }
 	    parser_getc(p);
 	}
     }
 
+    if (p->idstr == NULL) {
+	p->err = E_ALLOC;
+	return NULL;
+    }
+
     parser_getc(p);
     lex(p);
 
-    tailstrip(str);
-    p->idstr = gretl_strdup(str); /* leakage possible? */
+    tailstrip(p->idstr);
 
-    return newstr(p);
+    if (wrapped) {
+	int n = strlen(p->idstr);
+
+	if (p->idstr[n-1] == '"') {
+	    p->idstr[n-1] = '\0';
+	} else {
+	    unmatched_symbol_error('"', p);
+	}
+    }
+
+#if SDEBUG
+    fprintf(stderr, "get_string_arg: '%s'\n", p->idstr);
+#endif
+
+    return newstr(p, STR);
 }
 
 enum {
     BOTH_OPT = 1,
-    RIGHT_OPT
+    RIGHT_OPT,
+    RIGHT_STR
 };
 
 /* gather an unknown number of comma-separated arguments
@@ -429,7 +502,7 @@ static void get_multi_args (NODE *t, parser *p)
     fprintf(stderr, "get_multi_args, p->sym = %d\n", p->sym);
 #endif    
 
-    if (p->sym == LPR) {
+    if (p->sym == G_LPR) {
 	lex(p);
 	while (p->ch != 0 && !p->err) {
 	    n = expr(p);
@@ -438,12 +511,12 @@ static void get_multi_args (NODE *t, parser *p)
 	    } else {
 		p->err = push_bn_node(t, n);
 	    }
-	    if (p->sym == COM) {
+	    if (p->sym == P_COM) {
 		/* in case acceptance of plain strings was set,
 		   turn it off after the first arg */
-		p->getstr = 0;
+		p->flags &= ~P_GETSTR;
 		lex(p);
-	    } else if (p->sym == RPR) {
+	    } else if (p->sym == G_RPR) {
 		break;
 	    }
 	}
@@ -452,7 +525,7 @@ static void get_multi_args (NODE *t, parser *p)
     }
 
     if (cexp == 0 && !p->err) {
-	if (p->sym == RPR) {
+	if (p->sym == G_RPR) {
 	    lex(p);
 	} else {
 	    unmatched_symbol_error('(', p);
@@ -473,7 +546,7 @@ static void get_matrix_def (NODE *t, parser *p, int *sub)
     fprintf(stderr, "get_matrix_def, p->sym = %d\n", p->sym);
 #endif    
 
-    if (p->sym == LCB) {
+    if (p->sym == G_LCB) {
 	lex(p);
 	while (p->ch != 0 && !p->err) {
 	    n = expr(p);
@@ -482,13 +555,13 @@ static void get_matrix_def (NODE *t, parser *p, int *sub)
 	    } else {
 		p->err = push_bn_node(t, n);
 	    }
-	    if (p->sym == COM) {
+	    if (p->sym == P_COM) {
 		lex(p);
-	    } else if (p->sym == SEMI) {
+	    } else if (p->sym == P_SEMI) {
 		n = newempty(EMPTY);
 		p->err = push_bn_node(t, n);
 		lex(p);
-	    } else if (p->sym == RCB) {
+	    } else if (p->sym == G_RCB) {
 		if (p->ch == '\'') {
 		    set_transpose(t);
 		    parser_getc(p);
@@ -504,7 +577,7 @@ static void get_matrix_def (NODE *t, parser *p, int *sub)
     }
 
     if (cexp == 0) {
-	if (p->sym == RCB) {
+	if (p->sym == G_RCB) {
 	    lex(p);
 	} else {
 	    unmatched_symbol_error('{', p);
@@ -526,42 +599,42 @@ static void get_slice_parts (NODE *t, parser *p)
 
     set_matrix_slice_on();
 
-    if (p->sym == LBR) {
+    if (p->sym == G_LBR) {
 	lex(p);
-	if (p->sym == COM) {
+	if (p->sym == P_COM) {
 	    /* empty row spec, OK */
 	    t->v.b2.l = newempty(EMPTY);
 	} else {
 	    t->v.b2.l = expr(p);
 	}
-	if (p->sym == COL) {
+	if (p->sym == P_COL) {
 	    /* need second part of colon-separated range */
 	    t->v.b2.l = newb2(SUBSL, t->v.b2.l, NULL);
 	    lex(p);
 	    t->v.b2.l->v.b2.r = expr(p);
 	}
-	if (p->sym == RBR) {
-	    /* co comma, no second arg string: may be OK */
+	if (p->sym == G_RBR) {
+	    /* no comma, no second arg string: may be OK */
 	    t->v.b2.r = newempty(ABSENT);
 	    lex(p);
 	    set_matrix_slice_off();
 	    return;
 	}
-	if (p->sym == COM) {
+	if (p->sym == P_COM) {
 	    lex(p);
-	    if (p->sym == RBR) {
+	    if (p->sym == G_RBR) {
 		/* empty column spec, OK */
 		t->v.b2.r = newempty(EMPTY);
 	    } else {
 		t->v.b2.r = expr(p);
 	    }
-	    if (p->sym == COL) {
+	    if (p->sym == P_COL) {
 		/* need second part of colon-separated range */
 		t->v.b2.r = newb2(SUBSL, t->v.b2.r, NULL);
 		lex(p);
 		t->v.b2.r->v.b2.r = expr(p);
 	    }
-	    if (p->sym == RBR) {
+	    if (p->sym == G_RBR) {
 		if (p->ch == '\'') {
 		    set_transpose(t); /* ?? */
 		    parser_getc(p);
@@ -595,9 +668,9 @@ static void get_args (NODE *t, parser *p, int opt)
     fprintf(stderr, "get_args...\n");
 #endif    
 
-    if (p->sym == LPR) {
+    if (p->sym == G_LPR) {
 	lex(p);
-	if (p->sym == RPR && opt == BOTH_OPT) {
+	if (p->sym == G_RPR && opt == BOTH_OPT) {
 	    /* no args, but it's OK */
 	    t->v.b2.l = newempty(EMPTY);
 	    t->v.b2.r = newempty(EMPTY);
@@ -605,23 +678,28 @@ static void get_args (NODE *t, parser *p, int opt)
 	    return;
 	}
 	t->v.b2.l = expr(p);
-	if (p->sym == RPR && opt == RIGHT_OPT) {
+	if (p->sym == G_RPR && opt == RIGHT_OPT) {
 	    /* no second arg, but it's OK */
 	    t->v.b2.r = newempty(EMPTY);
 	    lex(p);
 	    return;
 	}	    
-	if (p->sym == COM) {
-	    lex(p);
-	    t->v.b2.r = expr(p);
-	    if (p->sym == RPR) {
-		if (p->ch == '\'') {
-		    set_transpose(t);
-		    parser_getc(p);
-		}
-		lex(p);
+	if (p->sym == P_COM) {
+	    if (opt == RIGHT_STR) {
+		/* the second arg must be a literal string */
+		t->v.b2.r = get_string_arg(p);
 	    } else {
-		cexp = ')';
+		lex(p);
+		t->v.b2.r = expr(p);
+		if (p->sym == G_RPR) {
+		    if (p->ch == '\'') {
+			set_transpose(t);
+			parser_getc(p);
+		    }
+		    lex(p);
+		} else {
+		    cexp = ')';
+		}
 	    }
 	} else {
 	    cexp = ',';
@@ -671,12 +749,14 @@ static NODE *powterm (parser *p)
     }
 
 #if SDEBUG
-    fprintf(stderr, "powterm: p->sym = %d, p->ch = %c\n",
-	    p->sym, p->ch? p->ch : '0');
+    fprintf(stderr, "powterm: p->sym = %d, p->ch = '%c' (%d)\n",
+	    p->sym, p->ch? p->ch : '0', p->ch);
 #endif
 
-    if (p->sym == RUNIFORM || p->sym == RNORMAL) {
+    if (p->sym == F_RUNIFORM || p->sym == F_RNORMAL) {
 	opt = BOTH_OPT;
+    } else if (p->sym == F_FDJAC || p->sym == F_BFGSMAX) {
+	opt = RIGHT_STR;
     }
 
     if (func2_symb(p->sym)) {
@@ -691,7 +771,7 @@ static NODE *powterm (parser *p)
 	    lex(p);
 	    t->v.b1.b = newbn(FARGS);
 	    if (t != NULL) {
-		p->getstr = 1;
+		p->flags |= P_GETSTR;
 		get_multi_args(t->v.b1.b, p);
 	    }
 	}	
@@ -701,7 +781,7 @@ static NODE *powterm (parser *p)
 	    lex(p);
 	    t->v.b1.b = get_string_arg(p);
 	}	
-    } else if (func_symb(p->sym)) {
+    } else if (func1_symb(p->sym)) {
 	t = newb1(p->sym, NULL);
 	if (t != NULL) {
 	    lex(p);
@@ -710,15 +790,21 @@ static NODE *powterm (parser *p)
     } else if (p->sym == LAG || p->sym == OBS) {
 	t = newb2(p->sym, NULL, NULL);
 	if (t != NULL) {
-	    t->v.b2.l = newref(p, UVAR); 
+	    t->v.b2.l = newref(p, USERIES); 
 	    lex(p);
 	    t->v.b2.r = base(p, t);
 	}
+    } else if (func2_symb(p->sym)) {
+	t = newb2(p->sym, NULL, NULL);
+	if (t != NULL) {
+	    lex(p);
+	    get_args(t, p, opt);
+	}
     } else if (p->sym == MSL || p->sym == DMSL) {
 	t = newb2(p->sym, NULL, NULL);
 	if (t != NULL) {
 	    if (p->sym == MSL) {
-		t->v.b2.l = newstr(p);
+		t->v.b2.l = newstr(p, STR);
 	    } else {
 		t->v.b2.l = newref(p, MVAR);
 	    }
@@ -738,17 +824,17 @@ static NODE *powterm (parser *p)
     } else if (p->sym == OVAR) {
 	t = newb2(p->sym, NULL, NULL);
 	if (t != NULL) {
-	    t->v.b2.l = newstr(p);
+	    t->v.b2.l = newstr(p, STR);
 	    get_ovar_ref(t, p);
 	}
-    } else if (p->sym == LPR) {
+    } else if (p->sym == G_LPR) {
 	/* dummy root for parenthesized expressions, to facilitate
 	   taking the transpose of matrix stuff, e.g. (A*B)' */
 	t = newb1(EROOT, NULL);
 	if (t != NULL) {
 	    t->v.b1.b = base(p, t);
 	}
-    } else if (p->sym == LCB) {
+    } else if (p->sym == G_LCB) {
 	/* explicit matrix definition, possibly followed by
 	   a "subslice" specification */
 	int sub = 0;
@@ -770,7 +856,7 @@ static NODE *powterm (parser *p)
     } else if (p->sym == UFUN) {
 	t = newb2(p->sym, NULL, NULL);
 	if (t != NULL) {
-	    t->v.b2.l = newstr(p);
+	    t->v.b2.l = newstr(p, STR);
 	    lex(p);
 	    t->v.b2.r = newbn(FARGS);
 	    if (t != NULL) {
@@ -786,8 +872,8 @@ static NODE *powterm (parser *p)
 		get_multi_args(t->v.b1.b, p);
 	    }
 	}
-    } else if (p->sym == STR) {
-	t = newstr(p);
+    } else if (p->sym == STR || p->sym == VSTR) {
+	t = newstr(p, p->sym);
 	lex(p);
     } else {
 	t = base(p, NULL);
@@ -800,19 +886,6 @@ static NODE *powterm (parser *p)
     return t;
 }
 
-#if 0
-/* chunk that may be needed, modified */
-if (unary_op(sym)) {
-    if (p->ch == 0) {
-	/* no input left: provoke error */
-	p->sym = sym;
-	t = base(p, NULL);
-    } else {
-	/* continue as usual */
-    }
-}
-#endif
-
 static NODE *factor (parser *p)
 {  
     int sym = p->sym == B_SUB ? U_NEG : 
@@ -843,7 +916,7 @@ static NODE *factor (parser *p)
 		set_transpose(t);
 	    } else {
 		while (!p->err && (p->sym == B_POW || 
-				   p->sym == DOTPOW ||
+				   p->sym == B_DOTPOW ||
 				   p->sym == B_TRMUL)) {
 		    t = newb2(p->sym, t, NULL);
 		    if (t != NULL) {
@@ -871,8 +944,8 @@ static NODE *term (parser *p)
     }
 
     while (!p->err && (p->sym == B_MUL || p->sym == B_DIV || 
-		       p->sym == B_MOD || p->sym == DOTMULT || 
-		       p->sym == DOTDIV || p->sym == KRON)) {
+		       p->sym == B_MOD || p->sym == B_DOTMULT || 
+		       p->sym == B_DOTDIV || p->sym == B_KRON)) {
 	t = newb2(p->sym, t, NULL);
 	if (t != NULL) {
 	    lex(p);
@@ -896,8 +969,9 @@ static NODE *expr4 (parser *p)
     }
 
     while (!p->err && (p->sym == B_ADD || p->sym == B_SUB || 
-		       p->sym == DOTADD || p->sym == DOTSUB ||
-		       p->sym == MCCAT || p->sym == MRCAT)) {
+		       p->sym == B_DOTADD || p->sym == B_DOTSUB ||
+		       p->sym == B_HCAT || p->sym == B_VCAT ||
+		       p->sym == B_LCAT || p->sym == B_RANGE)) {
 	t = newb2(p->sym, t, NULL);
 	if (t != NULL) {
 	    lex(p);
@@ -921,7 +995,7 @@ static NODE *expr3 (parser *p)
     }
 
     while (!p->err && (p->sym == B_GT || p->sym == B_LT || 
-		       p->sym == DOTGT || p->sym == DOTLT || 
+		       p->sym == B_DOTGT || p->sym == B_DOTLT || 
 		       p->sym == B_GTE || p->sym == B_LTE)) {
 	t = newb2(p->sym, t, NULL);
 	if (t != NULL) {
@@ -946,7 +1020,7 @@ static NODE *expr2 (parser *p)
     }
 
     while (!p->err && (p->sym == B_EQ || p->sym == B_NEQ ||
-		       p->sym == DOTEQ)) {
+		       p->sym == B_DOTEQ)) {
 	t = newb2(p->sym, t, NULL);
 	if (t != NULL) {
 	    lex(p);
@@ -1020,7 +1094,7 @@ NODE *expr (parser *p)
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b3.m = expr(p);
-	    if (p->sym == COL) {
+	    if (p->sym == P_COL) {
 		lex(p);
 		t->v.b3.r = expr(p);
 	    } else {
