@@ -37,6 +37,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { ADF,      "adf" }, 
     { APPEND,   "append" },
     { AR,       "ar" },  
+    { AR1,      "ar1" },
     { ARBOND,   "arbond" },
     { ARCH,     "arch" },
     { ARMA,     "arima" },
@@ -46,7 +47,6 @@ static struct gretl_cmd gretl_cmds[] = {
     { COEFFSUM, "coeffsum" },
     { COINT,    "coint" },
     { COINT2,   "coint2" },
-    { CORC,     "corc" }, 
     { CORR,     "corr" },     
     { CORRGM,   "corrgm" },   
     { CRITERIA, "criteria" },
@@ -58,6 +58,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { DIFFTEST, "difftest" },
     { DISCRETE, "discrete" },
     { DUMMIFY,  "dummify" },
+    { ELIF,     "elif" },
     { ELSE,     "else" },
     { END,      "end" },
     { ENDIF,    "endif" },
@@ -66,8 +67,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { EQUATION, "equation" },
     { ESTIMATE, "estimate" },
     { FCAST,    "fcast" }, 
-    { FCASTERR, "fcasterr" },
-    { FIT,      "fit" }, 
+    { FOREIGN,  "foreign" }, 
     { FREQ,     "freq" }, 
     { FUNC,     "function" },
     { FUNCERR,  "error" },
@@ -80,7 +80,6 @@ static struct gretl_cmd gretl_cmds[] = {
     { HCCM,     "hccm" }, 
     { HECKIT,   "heckit" },
     { HELP,     "help" },    
-    { HILU,     "hilu" },    
     { HSK,      "hsk" }, 
     { HURST,    "hurst" },
     { IF,       "if" },
@@ -102,13 +101,14 @@ static struct gretl_cmd gretl_cmds[] = {
     { MLE,      "mle" },
     { MODELTAB, "modeltab" },
     { MPOLS,    "mpols" },
-    { MULTIPLY, "multiply" },
     { NLS,      "nls" },
+    { NORMTEST, "normtest" },
     { NULLDATA, "nulldata" },
     { OLS,      "ols" },     
     { OMIT,     "omit" },
     { OMITFROM, "omitfrom" },
     { OPEN,     "open" },
+    { ORTHDEV,  "orthdev" },
     { OUTFILE,  "outfile" },
     { PANEL,    "panel" },
     { PCA,      "pca" },
@@ -118,11 +118,10 @@ static struct gretl_cmd gretl_cmds[] = {
     { PRINT,    "print" }, 
     { PRINTF,   "printf" },
     { PROBIT,   "probit" },
-    { PVALUE,   "pvalue" },  
-    { PWE,      "pwe" },
+    { PVALUE,   "pvalue" }, 
+    { QUANTREG, "quantreg" },
     { QLRTEST,  "qlrtest" }, 
     { QUIT,     "quit" }, 
-    { REMEMBER, "remember" },
     { RENAME,   "rename" },
     { RESET,    "reset" },
     { RESTRICT, "restrict" },
@@ -140,9 +139,9 @@ static struct gretl_cmd gretl_cmds[] = {
     { SMPL,     "smpl" },
     { SPEARMAN, "spearman" },
     { SPRINTF,  "sprintf" },
-    { SQUARE,   "square" },  
+    { SQUARE,   "square" },
+    { SSCANF,   "sscanf" },
     { STORE,    "store" },   
-    { STRING,   "string" },
     { SUMMARY,  "summary" },
     { SYSTEM,   "system" },
     { TABPRINT, "tabprint" },
@@ -165,6 +164,7 @@ static struct gretl_cmd gretl_cmd_aliases[] = {
     { GENR, "series" },   
     { GENR, "scalar" },  
     { GENR, "matrix" },  
+    { GENR, "string" },
     { LOOP, "for" },   
     { LOOP, "foreach" },
     { LOOP, "while" },
@@ -208,7 +208,7 @@ int gretl_command_number (const char *s)
     if (ht == NULL) {
 	gretl_command_hash_init();
     }
-    
+
     p = g_hash_table_lookup(ht, s);
     if (p != NULL) {
 	ret = GPOINTER_TO_INT(p);
