@@ -67,6 +67,7 @@ static const char *gretl_error_messages[] = {
     N_("Incompatible options"),                                  /* E_BADOPT */
     N_("The restrictions do not identify the parameters"),       /* E_NOIDENT */
     N_("External command failed"),                               /* E_EXTERNAL */
+    N_("Maximum length of command line (8192 bytes) exceeded"),  /* E_TOOLONG */        
     NULL,                                                        /* E_DB_DUP */
     NULL,                                                        /* E_OK */
     NULL                                                         /* E_MAX */
@@ -171,6 +172,25 @@ void gretl_errmsg_set (const char *str)
     }
 }
 
+/**
+ * gretl_errmsg_sprintf:
+ * @fmt: format string.
+ *
+ * If %gretl_errmsg is currently blank, print a formatted
+ * message into place.
+ */
+
+void gretl_errmsg_sprintf (const char *fmt, ...)
+{
+    if (*gretl_errmsg == '\0' && fmt != NULL) {
+	va_list args;
+
+	va_start(args, fmt);
+	vsnprintf(gretl_errmsg, ERRLEN, fmt, args);
+	va_end(args);
+    }
+}
+
 /**
  * gretl_errmsg_set_from_errno:
  *
@@ -180,16 +200,21 @@ void gretl_errmsg_set (const char *str)
  * append the new error info to the message.
  */
 
-void gretl_errmsg_set_from_errno (void)
+void gretl_errmsg_set_from_errno (const char *s)
 {
     char *msg = NULL;
 
     if (errno) {
 	msg = strerror(errno);
+	errno = 0;
     }
 
     if (msg != NULL) {
-	gretl_errmsg_set(msg);
+	if (s != NULL) {
+	    gretl_errmsg_sprintf("%s: %s", s, msg);
+	} else {
+	    gretl_errmsg_set(msg);
+	}
     }
 }
 
@@ -203,5 +228,28 @@ void gretl_error_clear (void)
 {
     *gretl_errmsg = '\0';
     error_printed = 0;
+    errno = 0;
 }
 
+#if 0 /* not yet */
+
+static char *warnbuf;
+
+void gretl_warning_add (const char *s)
+{
+    free(warnbuf);
+    warnbuf = gretl_strdup(s);
+}
+
+const char *gretl_warning_get (void)
+{
+    if (warnbuf != NULL) {
+	return warnbuf;
+    } else {
+	return "";
+    }
+}
+
+#endif
+
+
