@@ -64,9 +64,10 @@ enum gretl_error_codes {
     E_BADOPT,      /* 40 */
     E_NOIDENT,     /* 41 */
     E_EXTERNAL,    /* 42 */
-    E_DB_DUP,      /* 43 : duplicate vars found when saving to database */
-    E_OK,          /* 44 : not really an error */
-    E_MAX          /* 45 */
+    E_TOOLONG,     /* 43 : command line too long */
+    E_DB_DUP,      /* 44 : duplicate vars found when saving to database */
+    E_OK,          /* 45 : not really an error */
+    E_MAX          /* 46 */
 }; 
 
 void errmsg (int err, PRN *prn);
@@ -77,7 +78,9 @@ const char *gretl_errmsg_get (void);
 
 void gretl_errmsg_set (const char *str);
 
-void gretl_errmsg_set_from_errno (void);
+void gretl_errmsg_sprintf (const char *fmt, ...);
+
+void gretl_errmsg_set_from_errno (const char *s);
 
 void gretl_error_clear (void);
 
