@@ -36,6 +36,7 @@ typedef struct fn_param_ fn_param;
 
 struct fn_param_ {
     char *name;
+    char *descrip;
     char type;
     char flags;
     double deflt;
@@ -345,6 +346,12 @@ const char *fn_param_name (const ufunc *fun, int i)
 	fun->params[i].name;
 }
 
+const char *fn_param_descrip (const ufunc *fun, int i)
+{
+    return (i < 0 || i >= fun->n_params)? NULL :
+	fun->params[i].descrip;
+}
+
 double fn_param_default (const ufunc *fun, int i)
 {
     return (i < 0 || i >= fun->n_params)? NADBL :
@@ -485,7 +492,7 @@ ufunc *get_user_function_by_name (const char *name)
        look first for functions from the same package */
 
     for (i=0; i<n_ufuns; i++) {
-	if (!strcmp(name, (ufuns[i])->name)) {
+	if (!strcmp(name, ufuns[i]->name)) {
 	    fun = ufuns[i];
 	    if (fun->pkgID == ID || ID == 0) {
 		break;
@@ -498,7 +505,7 @@ ufunc *get_user_function_by_name (const char *name)
     if (ID > 0 && fun == NULL) {
 	/* fall back on unpackaged functions */
 	for (i=0; i<n_ufuns; i++) {
-	    if (!strcmp(name, (ufuns[i])->name)) {
+	    if (!strcmp(name, ufuns[i]->name)) {
 		fun = ufuns[i];
 		if (fun->pkgID == 0) {
 		    break;
@@ -533,6 +540,7 @@ static fn_param *allocate_params (int n)
 
     for (i=0; i<n; i++) {
 	params[i].name = NULL;
+	params[i].descrip = NULL;
 	params[i].type = 0;
 	params[i].flags = 0;
 	params[i].deflt = NADBL;
@@ -580,6 +588,7 @@ static void free_params_array (fn_param *params, int n)
 
     for (i=0; i<n; i++) {
 	free(params[i].name);
+	free(params[i].descrip);
     }
     free(params);
 }
@@ -686,7 +695,8 @@ static const char *arg_type_string (int t)
     case GRETL_TYPE_SCALAR_REF: return "scalar *";
     case GRETL_TYPE_SERIES_REF: return "series *";
     case GRETL_TYPE_MATRIX_REF: return "matrix *";
-    case GRETL_TYPE_STRING:     return "string";	
+    case GRETL_TYPE_STRING:     return "string";
+    case GRETL_TYPE_NONE:       return "null";
     }
 
     return "unknown";
@@ -763,7 +773,8 @@ static int field_to_type (const char *s)
     }
 }    
 
-static int func_read_params (xmlNodePtr node, ufunc *fun)
+static int func_read_params (xmlNodePtr node, xmlDocPtr doc,
+			     ufunc *fun)
 {
     xmlNodePtr cur;
     char *field;
@@ -789,12 +800,14 @@ static int func_read_params (xmlNodePtr node, ufunc *fun)
 
     cur = node->xmlChildrenNode;
     n = 0;
+
     while (cur != NULL && !err) {
 	if (!xmlStrcmp(cur->name, (XUC) "param")) {
 	    if (gretl_xml_get_prop_as_string(cur, "name", &field)) {
 		fun->params[n].name = field;
 	    } else {
 		err = E_DATA;
+		break;
 	    }
 	    if (gretl_xml_get_prop_as_string(cur, "type", &field)) {
 		fun->params[n].type = field_to_type(field);
@@ -802,12 +815,12 @@ static int func_read_params (xmlNodePtr node, ufunc *fun)
 		if (gretl_scalar_type(fun->params[n].type)) {
 		    gretl_xml_get_prop_as_double(cur, "default", 
 						 &fun->params[n].deflt);
-		}
-		if (fun->params[n].type == GRETL_TYPE_INT) {
-		    gretl_xml_get_prop_as_double(cur, "min", 
-						 &fun->params[n].min);
-		    gretl_xml_get_prop_as_double(cur, "max", 
-						 &fun->params[n].max);
+		    if (fun->params[n].type != GRETL_TYPE_BOOL) {
+			gretl_xml_get_prop_as_double(cur, "min", 
+						     &fun->params[n].min);
+			gretl_xml_get_prop_as_double(cur, "max", 
+						     &fun->params[n].max);
+		    }
 		}
 		if (gretl_xml_get_prop_as_bool(cur, "optional")) {
 		    fun->params[n].flags |= ARG_OPTIONAL;
@@ -817,7 +830,10 @@ static int func_read_params (xmlNodePtr node, ufunc *fun)
 		}
 	    } else {
 		err = E_DATA;
+		break;
 	    }
+	    gretl_xml_child_get_string(cur, doc, "description", 
+				       &fun->params[n].descrip);
 	    n++;
 	}	    
 	cur = cur->next;
@@ -874,6 +890,7 @@ static int func_read_code (xmlNodePtr node, xmlDocPtr doc, ufunc *fun)
 	}
 	s = line;
 	while (isspace(*s)) s++;
+	tailstrip(s);
 	err = strings_array_add(&fun->lines, &fun->n_lines, s);
     }
 
@@ -891,33 +908,33 @@ static void print_opt_flags (fn_param *param, PRN *prn)
     }
 }
 
-static void print_deflt_min_max (fn_param *param, PRN *prn)
+static void print_param_description (fn_param *param, PRN *prn)
 {
-    double x = param->min;
-    double y = param->max;
-    double z = param->deflt;
+    if (param->descrip != NULL && *param->descrip != '\0') {
+	pprintf(prn, " \"%s\"", param->descrip);
+    }
+}
 
-    if (na(x) && na(y) && na(z)) {
+static void print_min_max_deflt (fn_param *param, PRN *prn)
+{
+    double mi = param->min;
+    double ma = param->max;
+    double d = param->deflt;
+
+    if (na(mi) && na(ma) && na(d)) {
 	return; /* no-op */
-    } else if (na(x) && na(y) && !na(z)) {
-	pprintf(prn, "[%g]", z);
+    } else if (na(mi) && na(ma)) {
+	/* default value only */
+	pprintf(prn, "[%g]", d);
 	return;
     }
 
     pputc(prn, '[');
-    if (na(x)) {
-	pputc(prn, ':');
-    } else {
-	pprintf(prn, "%g:", x);
-    }
-    if (na(y)) {
-	pputc(prn, ':');
-    } else {
-	pprintf(prn, "%g:", y);
-    }  
-    if (!na(z)) {
-	pprintf(prn, "%g", z);
-    }   
+    if (!na(mi)) pprintf(prn, "%g", mi);
+    pputc(prn, ':');
+    if (!na(ma)) pprintf(prn, "%g", ma);
+    pputc(prn, ':');
+    if (!na(d)) pprintf(prn, "%g", d);
     pputc(prn, ']');
 }
 
@@ -947,11 +964,12 @@ static void print_function_start (ufunc *fun, PRN *prn)
 		pprintf(prn, "[%g]", fun->params[i].deflt);
 	    }
 	} else if (gretl_scalar_type(fun->params[i].type)) {
-	    print_deflt_min_max(&fun->params[i], prn);
+	    print_min_max_deflt(&fun->params[i], prn);
 	} else if (gretl_ref_type(fun->params[i].type) || 
 		   fun->params[i].type == GRETL_TYPE_LIST) {
 	    print_opt_flags(&fun->params[i], prn);
 	}
+	print_param_description(&fun->params[i], prn);
 	if (i == fun->n_params - 1) {
 	    pputc(prn, ')');
 	} else {
@@ -1060,7 +1078,7 @@ static int read_ufunc_from_xml (xmlNodePtr node, xmlDocPtr doc, fnpkg *pkg)
 	if (!xmlStrcmp(cur->name, (XUC) "help")) {
 	    gretl_xml_node_get_string(cur, doc, &fun->help);
 	} else if (!xmlStrcmp(cur->name, (XUC) "params")) {
-	    err = func_read_params(cur, fun);
+	    err = func_read_params(cur, doc, fun);
 	    if (err) {
 		fprintf(stderr, "%s: error parsing function parameters\n",
 			fun->name);
@@ -1096,36 +1114,43 @@ static int read_ufunc_from_xml (xmlNodePtr node, xmlDocPtr doc, fnpkg *pkg)
     return err;
 }
 
-static int wordmatch (const char *line, const char *s)
+static int wordmatch (const char *s, const char *test)
 {
-    int n = strlen(s);
+    int n = strlen(test);
     
-    return (strncmp(line, s, n) == 0 && 
-	    (line[n] == '\0' || isspace(line[n])));
+    return (!strncmp(s, test, n) && (s[n] == '\0' || isspace(s[n])));
 }
 
-static void adjust_indent (const char *line, int *this_indent,
-			   int *next_indent)
+void adjust_indent (const char *s, int *this_indent, int *next_indent)
 {
     int ti = *next_indent;
     int ni = *next_indent;
 
-    if (wordmatch(line, "loop")) {
+    if (wordmatch(s, "loop")) {
 	ni++;
-    } else if (wordmatch(line, "if")) {
+    } else if (wordmatch(s, "if")) {
 	ni++;
-    } else if (wordmatch(line, "nls")) {
+    } else if (wordmatch(s, "nls")) {
 	ni++;
-    } else if (wordmatch(line, "mle")) {
+    } else if (wordmatch(s, "mle")) {
 	ni++;
-    } else if (wordmatch(line, "gmm")) {
+    } else if (wordmatch(s, "gmm")) {
 	ni++;
-    } else if (wordmatch(line, "end") ||
-	       wordmatch(line, "endloop")) {
+    } else if (wordmatch(s, "function")) {
+	ni++;
+    } else if (wordmatch(s, "restrict")) {
+	ni++;
+    } else if (wordmatch(s, "system")) {
+	ni++;
+    } else if (wordmatch(s, "foreign")) {
+	ni++;
+    } else if (wordmatch(s, "end") ||
+	       wordmatch(s, "endif") ||
+	       wordmatch(s, "endloop")) {
 	ti--;
 	ni--;
-    } else if (wordmatch(line, "else") ||
-	       wordmatch(line, "elif")) {
+    } else if (wordmatch(s, "else") ||
+	       wordmatch(s, "elif")) {
 	ni = ti;
 	ti--;
     } 
@@ -1171,7 +1196,15 @@ static int write_function_xml (const ufunc *fun, FILE *fp)
 	    if (fun->params[i].flags & ARG_CONST) {
 		fputs(" const=\"true\"", fp);
 	    }
-	    fputs("/>\n", fp);
+	    if (fun->params[i].descrip != NULL) {
+		fputs(">\n", fp);
+		gretl_xml_put_tagged_string("description", 
+					    fun->params[i].descrip,
+					    fp);
+		fputs("  </param>\n", fp);
+	    } else {
+		fputs("/>\n", fp);
+	    }
 	}
 	fputs(" </params>\n", fp);
 
@@ -2352,16 +2385,21 @@ static int check_func_name (const char *fname, ufunc **pfun, PRN *prn)
     int i, err = 0;
 
     if (!isalpha((unsigned char) *fname)) {
-	strcpy(gretl_errmsg, "function names must start with a letter");
+	strcpy(gretl_errmsg, _("Function names must start with a letter"));
 	err = 1;
     } else if (gretl_command_number(fname)) {
-	sprintf(gretl_errmsg, "'%s' is the name of a gretl command",
+	sprintf(gretl_errmsg, _("'%s' is the name of a gretl command"),
+		fname);
+	err = 1;
+    } else if (function_lookup(fname)) {
+	sprintf(gretl_errmsg, _("'%s' is the name of a built-in function"),
 		fname);
 	err = 1;
     } else {
 	for (i=0; i<n_ufuns; i++) {
 	    if (!strcmp(fname, ufuns[i]->name)) {
-		pprintf(prn, "Redefining function '%s'\n", fname);
+		pprintf(prn, _("Redefining function '%s'"), fname);
+		pputc(prn, '\n');
 		if (pfun != NULL) {
 		    clear_ufunc_data(ufuns[i]);
 		    *pfun = ufuns[i];
@@ -2376,7 +2414,7 @@ static int check_func_name (const char *fname, ufunc **pfun, PRN *prn)
     return err;
 }
 
-static int maybe_delete_function (const char *fname)
+static int maybe_delete_function (const char *fname, PRN *prn)
 {
     ufunc *fun = get_user_function_by_name(fname);
     int err = 0;
@@ -2391,6 +2429,9 @@ static int maybe_delete_function (const char *fname)
 	err = 1;
     } else {
 	delete_ufunc_from_list(fun);
+	if (gretl_messages_on()) {
+	    pprintf(prn, _("Deleted function '%s'\n"), fname);
+	}
     } 
 
     return err;
@@ -2398,67 +2439,105 @@ static int maybe_delete_function (const char *fname)
 
 static int comma_count (const char *s)
 {
+    int quoted = 0;
     int nc = 0;
 
     while (*s) {
-	if (*s == ',') nc++;
+	if (*s == '"') {
+	    quoted = !quoted;
+	} else if (!quoted && *s == ',') {
+	    nc++;
+	}
 	s++;
     }
 
     return nc;
 }
 
-static int read_deflt_min_max (char *s, fn_param *param,
-			       int *namelen)
+static int read_min_max_deflt (char **ps, fn_param *param)
 {
-    char *p = strchr(s, '[');
+    char *p = *ps;
+    double x, y, z;
     int err = 0;
 
-    if (p != NULL) {
-	double x, y, z;
-
-	if (param->type == GRETL_TYPE_BOOL) {
-	    if (sscanf(p, "[%lf]", &x) == 1) {
-		param->deflt = x;
-	    } else {
-		err = E_DATA;
-	    }
+    if (param->type == GRETL_TYPE_BOOL) {
+	if (sscanf(p, "[%lf]", &x) == 1) {
+	    param->deflt = x;
 	} else {
-	    if (sscanf(p, "[%lf:%lf:%lf]", &x, &y, &z) == 3) {
-		param->min = x;
-		param->max = y;
-		param->deflt = z;
-	    } else if (sscanf(p, "[%lf::%lf]", &x, &y) == 2) {
-		param->min = x;
-		param->deflt = y;
-	    } else if (sscanf(p, "[:%lf:%lf]", &x, &y) == 2) {
-		param->max = x;
-		param->deflt = y;
-	    } else if (sscanf(p, "[%lf]", &x) == 1) {
-		param->deflt = x;
-	    } else {
-		err = E_DATA;
-	    }
+	    err = E_PARSE;
 	}
-	if (!err) {
-	    *namelen = p - s;
+    } else {
+	if (sscanf(p, "[%lf:%lf:%lf]", &x, &y, &z) == 3) {
+	    param->min = x;
+	    param->max = y;
+	    param->deflt = z;
+	} else if (sscanf(p, "[%lf::%lf]", &x, &y) == 2) {
+	    param->min = x;
+	    param->deflt = y;
+	} else if (sscanf(p, "[%lf:%lf:]", &x, &y) == 2) {
+	    param->min = x;
+	    param->max = y;
+	} else if (sscanf(p, "[:%lf:%lf]", &x, &y) == 2) {
+	    param->max = x;
+	    param->deflt = y;
+	} else if (sscanf(p, "[%lf]", &x) == 1) {
+	    param->deflt = x;
+	} else {
+	    err = E_PARSE;
+	}
+    }
+
+    if (!err) {
+	p = strchr(p, ']');
+	if (p == NULL) {
+	    err = E_PARSE;
+	} else {
+	    *ps = p + 1;
 	}
     }
 
     return err;
 }
 
-static int read_param_option (char *s, fn_param *param,
-			      int *namelen)
+static int read_param_option (char **ps, fn_param *param)
 {
-    char *p = strstr(s, "[null]");
+    int err = E_PARSE;
 
-    if (p != NULL) {
+    if (!strncmp(*ps, "[null]", 6)) {
 	param->flags |= ARG_OPTIONAL;
-	*namelen -= 6;
+	err = 0;
+	*ps += 6;
     }
 
-    return 0;
+    return err;
+}
+
+static int read_param_comment (char **ps, fn_param *param)
+{
+    char *p = *ps + 1;
+    int len = 0;
+    int err = E_PARSE;
+
+    while (*p) {
+	if (*p == '"') {
+	    err = 0;
+	    break;
+	}
+	len++;
+	p++;
+    }
+
+    if (!err && len > 0) {
+	p = *ps + 1;
+	param->descrip = gretl_strndup(p, len);
+	if (param->descrip == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    *ps = p + len + 1;
+	}
+    }
+
+    return err;
 }
 
 static int parse_function_param (char *s, fn_param *param, int i)
@@ -2468,6 +2547,10 @@ static int parse_function_param (char *s, fn_param *param, int i)
     int type, len;
     int err = 0;
 
+#if FN_DEBUG
+    fprintf(stderr, "parse_function_param: s = '%s'\n", s);
+#endif
+
     while (isspace(*s)) s++;
 
     if (!strncmp(s, "const ", 6)) {
@@ -2477,7 +2560,7 @@ static int parse_function_param (char *s, fn_param *param, int i)
     }
 
     /* get arg or return type */
-    len = strcspn(s, " ");
+    len = gretl_varchar_spn(s);
     if (len > 21) {
 	err = E_PARSE;
     } else {
@@ -2500,44 +2583,64 @@ static int parse_function_param (char *s, fn_param *param, int i)
     }
     
     while (isspace(*s)) s++;
-    len = strcspn(s, " ");
+    len = gretl_varchar_spn(s);
     if (len == 0) {
-	sprintf(gretl_errmsg, "parameter %d: name is missing", i);
+	sprintf(gretl_errmsg, "parameter %d: name is missing", i + 1);
 	err = E_PARSE;
-    }
-
-    if (err) {
-	return err;
-    }    
-
-    if (gretl_scalar_type(type)) {
-	param->type = type;
-	err = read_deflt_min_max(s, param, &len);
-    }
-
-    if (gretl_ref_type(type) || type == GRETL_TYPE_LIST) {
-	param->type = type;
-	err = read_param_option(s, param, &len);
-    }    
-
-    if (!err) {
+    } else {
 	name = gretl_strndup(s, len);
 	if (name == NULL) {
 	    err = E_ALLOC;
 	}
+    }	
+
+    if (err) {
+	return err;
+    }
+
+    param->type = type;
+
+    s += len;
+    s += strspn(s, " ");
+
+    if (gretl_scalar_type(type)) {
+	if (*s == '[') { 
+	    err = read_min_max_deflt(&s, param);
+	}
+    }
+
+    if (gretl_ref_type(type) || type == GRETL_TYPE_LIST) {
+	if (*s == '[') { 
+	    err = read_param_option(&s, param);
+	}
+    } 
+
+    s += strspn(s, " ");
+
+    if (*s == '"') {
+	err = read_param_comment(&s, param);
+    } 
+
+    if (!err && *s != '\0') {
+	/* got trailing unparseable stuff */
+	err = E_PARSE;
     }
 
     if (!err) {
-	param->type = type;
 	param->name = name;
+    } else {
+	free(name);
+	free(param->descrip);
+	param->descrip = NULL;
     }
 
 #if FN_DEBUG
     if (!err) {
 	fprintf(stderr, " param[%d] = '%s', ptype = %d\n", 
 		i, name, type);
-	fprintf(stderr, "min=%g, max=%g, deflt=%g\n", 
+	fprintf(stderr, "  min=%g, max=%g, deflt=%g\n", 
 		param->min, param->max, param->deflt);
+	fprintf(stderr, "  comment = '%s'\n", param->descrip); 
     }
 #endif
 
@@ -2633,7 +2736,17 @@ static int parse_fn_definition (char *fname,
     }
 
     if (!err) {
-	charsub(s, ',', 0);
+	int quoted = 0;
+
+	p = s;
+	while (*p) {
+	    if (*p == '"') {
+		quoted = !quoted;
+	    } else if (!quoted && *p == ',') {
+		*p = '\0';
+	    }
+	    p++;
+	}
 	p = s;
 	for (i=0; i<np && !err; i++) {
 	    err = parse_function_param(p, &params[i], i);
@@ -2669,7 +2782,7 @@ int gretl_start_compiling_function (const char *line, PRN *prn)
 
     if (nf == 2) {
 	if (!strcmp(extra, "clear") || !strcmp(extra, "delete")) {
-	    return maybe_delete_function(fname);
+	    return maybe_delete_function(fname, prn);
 	}
     } 
 
@@ -2779,10 +2892,9 @@ static int real_function_append_line (const char *line, ufunc *fun)
     }
 
     if (string_is_blank(line)) {
-	return 0;
-    }
-
-    if (end_of_function(line)) {
+	/* return 0; */
+	err = strings_array_add(&fun->lines, &fun->n_lines, "");
+    } else if (end_of_function(line)) {
 	if (fun->n_lines == 0) {
 	    sprintf(gretl_errmsg, "%s: empty function", fun->name);
 	    err = 1;
@@ -2924,6 +3036,33 @@ int update_function_from_script (const char *fname, int idx)
     return err;
 }
 
+#define LIST_RENAME 0
+
+#if LIST_RENAME
+
+static void rename_list_var (DATAINFO *pdinfo, int v)
+{
+    if (strlen(pdinfo->varname[v]) < VNAMELEN - 1) {
+	char tmp[VNAMELEN];
+	
+	strcpy(tmp, "_");
+	strcat(tmp, pdinfo->varname[v]);
+	strcpy(pdinfo->varname[v], tmp);
+    }
+}
+
+static void restore_listvar_name (DATAINFO *pdinfo, int v)
+{
+    if (*pdinfo->varname[v] == '_') {
+	char tmp[VNAMELEN];
+	
+	strcpy(tmp, pdinfo->varname[v] + 1);
+	strcpy(pdinfo->varname[v], tmp);
+    }
+}
+
+#endif
+
 /* Given a named list of variables supplied as an argument to a
    function, copy the list under the name assigned by the function,
    and make the variables referenced in that list accessible to the
@@ -2936,7 +3075,7 @@ static int localize_list (const char *oldname, fn_param *fp,
 {
     const int *list;
     int level = fn_executing + 1;
-    int i, err;
+    int i, v, err;
 
     list = get_list_by_name(oldname);
 
@@ -2953,11 +3092,15 @@ static int localize_list (const char *oldname, fn_param *fp,
 
     if (!err) {
 	for (i=1; i<=list[0]; i++) {
-	    if (list[i] != 0) {
-		STACK_LEVEL(pdinfo, list[i]) = level;
+	    v = list[i];
+	    if (v != 0) {
+		STACK_LEVEL(pdinfo, v) = level;
 		if (fp->flags & ARG_CONST) {
-		    set_var_const(pdinfo, list[i]);
+		    set_var_const(pdinfo, v);
 		}
+#if LIST_RENAME
+		rename_list_var(pdinfo, v);
+#endif
 	    }
 	}
     }
@@ -2965,6 +3108,47 @@ static int localize_list (const char *oldname, fn_param *fp,
     return err;
 }
 
+static void boolify_local_var (const char *vname, double **Z,
+			       const DATAINFO *pdinfo)
+{
+    int v = varindex(pdinfo, vname);
+
+    if (v < pdinfo->v && Z[v][0] != 0.0 && !na(Z[v][0])) {
+	Z[v][0] = 1.0;
+    }
+}
+
+static int localize_matrix_ref (struct fnarg *arg, const char *name)
+{
+    user_matrix *u = arg->val.um;
+
+    arg->upname = gretl_strdup(user_matrix_get_name(u));
+    if (arg->upname == NULL) {
+	return E_ALLOC;
+    }
+
+    user_matrix_adjust_level(u, 1);
+    user_matrix_set_name(u, name);
+
+    return 0;
+}
+
+static int localize_variable_ref (struct fnarg *arg, DATAINFO *pdinfo,
+				  const char *name)
+{
+    int v = arg->val.idnum;
+
+    arg->upname = gretl_strdup(pdinfo->varname[v]);
+    if (arg->upname == NULL) {
+	return E_ALLOC;
+    } 
+
+    STACK_LEVEL(pdinfo, v) += 1;
+    strcpy(pdinfo->varname[v], name);
+
+    return 0;
+}
+
 /* Scalar function arguments only: if the arg is not supplied, use the
    default that is contained in the function specification, if any.
 */
@@ -3007,10 +3191,17 @@ static int allocate_function_args (ufunc *fun,
 					       pZ, pdinfo);
 	    } else if (arg->type == GRETL_TYPE_NONE) {
 		err = add_scalar_arg_default(fp, pZ, pdinfo);
+	    } else if (arg->type == GRETL_TYPE_MATRIX) {
+		/* "cast" to scalar */
+		err = dataset_add_scalar_as(arg->val.m->val[0], fp->name, 
+					    pZ, pdinfo);
 	    } else {
 		err = dataset_add_scalar_as(arg->val.x, fp->name, 
 					    pZ, pdinfo);
 	    }
+	    if (!err && fp->type == GRETL_TYPE_BOOL) {
+		boolify_local_var(fp->name, *pZ, pdinfo);
+	    }
 	} else if (fp->type == GRETL_TYPE_SERIES) {
 	    if (arg->type == GRETL_TYPE_UVAR) {
 		err = dataset_copy_variable_as(arg->val.idnum, fp->name,
@@ -3031,24 +3222,12 @@ static int allocate_function_args (ufunc *fun,
 	    err = add_string_as(arg->val.str, fp->name);
 	} else if (fp->type == GRETL_TYPE_SCALAR_REF ||
 		   fp->type == GRETL_TYPE_SERIES_REF) {
-	    int v = arg->val.idnum;
-
-	    arg->upname = gretl_strdup(pdinfo->varname[v]);
-	    if (arg->upname == NULL) {
-		err = E_ALLOC;
-	    } else {
-		STACK_LEVEL(pdinfo, v) += 1;
-		strcpy(pdinfo->varname[v], fp->name);
+	    if (arg->type != GRETL_TYPE_NONE) {
+		err = localize_variable_ref(arg, pdinfo, fp->name);
 	    }
 	} else if (fp->type == GRETL_TYPE_MATRIX_REF) {
-	    user_matrix *u = arg->val.um;
-
-	    arg->upname = gretl_strdup(user_matrix_get_name(u));
-	    if (arg->upname == NULL) {
-		err = E_ALLOC;
-	    } else {	    
-		user_matrix_adjust_level(u, 1);
-		user_matrix_set_name(u, fp->name);
+	    if (arg->type != GRETL_TYPE_NONE) {
+		err = localize_matrix_ref(arg, fp->name);
 	    }
 	}
 
@@ -3192,44 +3371,88 @@ get_matrix_return (const char *mname, int action, int *err)
     return ret;
 }
 
-static int unlocalize_list (const char *lname, DATAINFO *pdinfo)
+static int unlocalize_list (const char *lname, double **Z, DATAINFO *pdinfo)
 {
     const int *list = get_list_by_name(lname);
     int d = gretl_function_depth();
-    int i, vi, err = 0;
+    const char *vname;
+    int upd = d - 1;
+    int i, j, t, vi;
 
 #if UDEBUG
-    fprintf(stderr, "unlocalize_list: %s, d = %d\n", lname, d);
+    fprintf(stderr, "unlocalize_list: '%s', function depth = %d\n", lname, d);
     printlist(list, lname);
 #endif	    
 
     if (list == NULL) {
-	err = E_DATA;
-    } else {
-	for (i=1; i<=list[0]; i++) {
-	    vi = list[i];
-	    if (vi != 0 && vi < pdinfo->v) {
-		if (STACK_LEVEL(pdinfo, vi) == d) {
-		    STACK_LEVEL(pdinfo, vi) -= 1;
+	return E_DATA;
+    }
+
+    for (i=1; i<=list[0]; i++) {
+	int overwrite = 0;
+
+	vi = list[i];
+#if LIST_RENAME
+	restore_listvar_name(pdinfo, vi);
+#endif
+	vname = pdinfo->varname[vi];
+	if (vi > 0 && vi < pdinfo->v && STACK_LEVEL(pdinfo, vi) == d) {
+	    for (j=1; j<pdinfo->v; j++) { 
+		if (STACK_LEVEL(pdinfo, j) == upd && 
+		    !strcmp(pdinfo->varname[j], vname)) {
+		    overwrite = 1;
+		    break;
 		}
-		unset_var_const(pdinfo, vi);
 	    }
+	    if (overwrite) {
+		for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+		    Z[j][t] = Z[vi][t];
+		}
+	    } else {
+		STACK_LEVEL(pdinfo, vi) = upd;
+	    }
+	}
+	if (vi > 0 && vi < pdinfo->v) {
+	    unset_var_const(pdinfo, vi);
 	}
     }
 
-    return err;
+    return 0;
 }
 
 static char *
-get_list_return (const char *lname, DATAINFO *pdinfo, int *err)
+get_string_return (const char *sname, double **Z, DATAINFO *pdinfo, 
+		   int *err)
 {
     
+    const char *s = get_string_by_name(sname);
+    char *ret = NULL;
+
+    if (s == NULL) {
+	*err = E_DATA;
+    } else {
+	ret = gretl_strdup(s);
+	if (ret == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    return ret;
+}
+
+static char *
+get_list_return (const char *lname, double **Z, DATAINFO *pdinfo, 
+		 int *err)
+{
     char *ret = gretl_strdup(lname);
 
     if (ret == NULL) {
 	*err = E_ALLOC;
     } else {
-	*err = unlocalize_list(lname, pdinfo);
+#if UDEBUG
+	fprintf(stderr, "get_list_return: calling unlocalize_list\n");
+#endif
+	*err = unlocalize_list(lname, Z, pdinfo);
 	if (!*err) {
 	    *err = named_list_lower_level(lname);
 	}
@@ -3261,6 +3484,11 @@ function_assign_returns (ufunc *u, fnargs *args, int rtype,
     fn_param *fp;
     int i, err = 0;
 
+#if UDEBUG
+    fprintf(stderr, "function_assign_returns: retname = '%s', rtype = %d\n", 
+	    u->retname, rtype);
+#endif
+
     if (*perr == 0) {
 	/* direct return value */
 	if (rtype == GRETL_TYPE_DOUBLE) {
@@ -3270,7 +3498,9 @@ function_assign_returns (ufunc *u, fnargs *args, int rtype,
 	} else if (rtype == GRETL_TYPE_MATRIX) {
 	    *(gretl_matrix **) ret = get_matrix_return(u->retname, GET_COPY, &err);
 	} else if (rtype == GRETL_TYPE_LIST) {
-	    *(char **) ret = get_list_return(u->retname, pdinfo, &err);
+	    *(char **) ret = get_list_return(u->retname, Z, pdinfo, &err);
+	} else if (rtype == GRETL_TYPE_STRING) {
+	    *(char **) ret = get_string_return(u->retname, Z, pdinfo, &err);
 	}
 
 	if (err == E_UNKVAR) {
@@ -3306,10 +3536,18 @@ function_assign_returns (ufunc *u, fnargs *args, int rtype,
 		user_matrix_set_name(u, arg->upname);
 	    }
 	} else if (fp->type == GRETL_TYPE_LIST) {
-	    unlocalize_list(fp->name, pdinfo);
+#if UDEBUG
+	    fprintf(stderr, "function_assign_returns: calling unlocalize_list on '%s'\n",
+		    fp->name);
+#endif	    
+	    unlocalize_list(fp->name, Z, pdinfo);
 	}
     }
 
+#if UDEBUG
+    fprintf(stderr, "function_assign_returns: returning %d\n", err);
+#endif
+
     return err;
 }
 
@@ -3372,14 +3610,14 @@ static int stop_fncall (ufunc *u, double ***pZ, DATAINFO *pdinfo,
 		err = anyerr;
 	    }
 	} else {
-	    for (i=orig_v; i<pdinfo->v; i++) {
+	    for (i=pdinfo->v-1; i>=orig_v; i--) {
 		if (STACK_LEVEL(pdinfo, i) == d) {
-		    anyerr = dataset_drop_variable(i--, pZ, pdinfo);
+		    anyerr = dataset_drop_variable(i, pZ, pdinfo);
 		    if (anyerr && !err) {
 			err = anyerr;
 		    }
 		} 
-	    }
+	    }	    
 	}
     } else {
 	/* delete only variables copied from arguments */
@@ -3406,13 +3644,20 @@ static int stop_fncall (ufunc *u, double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
-static void start_fncall (ufunc *u, fnargs *args)
+static void start_fncall (ufunc *u, fnargs *args, PRN *prn)
 {
     set_executing_on(u);
     u->args = args;
     push_program_state();
-    set_gretl_echo(0);
-    set_gretl_messages(0);
+
+    if (gretl_debugging_on()) {
+	set_gretl_echo(1);
+	set_gretl_messages(1);
+	pprintf(prn, "*** executing function %s\n", u->name);
+    } else {
+	set_gretl_echo(0);
+	set_gretl_messages(0);
+    }
 }
 
 static char funcerr_msg[256];
@@ -3476,6 +3721,8 @@ static double arg_get_double_val (struct fnarg *arg,
 	return arg->val.x;
     } else if (arg->type == GRETL_TYPE_UVAR) {
 	return Z[arg->val.idnum][0];
+    } else if (arg->type == GRETL_TYPE_MATRIX) {
+	return arg->val.m->val[0];
     } else {
 	return NADBL;
     }
@@ -3503,6 +3750,10 @@ static int check_function_args (ufunc *u, fnargs *args,
 	    ; /* OK */
 	} else if (fp->type == GRETL_TYPE_SERIES && uvar_series(arg, pdinfo)) {
 	    ; /* OK */
+	} else if (gretl_scalar_type(fp->type) && 
+		   arg->type == GRETL_TYPE_MATRIX &&
+		   gretl_matrix_is_scalar(arg->val.m)) {
+	    ; /* OK */
 	} else if (fp->type != arg->type) {
 	    pprintf(prn, "argv[%d] is of wrong type (got %s, should be %s)\n", 
 		    i, arg_type_string(arg->type), arg_type_string(fp->type));
@@ -3531,7 +3782,7 @@ static int check_function_args (ufunc *u, fnargs *args,
     return err;
 }
 
-static void fn_state_init (CMD *cmd, ExecState *state)
+static void fn_state_init (CMD *cmd, ExecState *state, int *indent0)
 {
     cmd->list = NULL;
     cmd->param = NULL;
@@ -3540,6 +3791,9 @@ static void fn_state_init (CMD *cmd, ExecState *state)
 
     state->cmd = NULL;
     state->models = NULL;
+    state->submask = NULL;
+
+    *indent0 = gretl_if_state_record();
 }
 
 int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
@@ -3554,9 +3808,9 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
     int orig_v = pdinfo->v;
     int orig_t1 = pdinfo->t1;
     int orig_t2 = pdinfo->t2;
-    int started = 0;
+    int indent0, started = 0;
     int i, err = 0;
- 
+
     *funcerr_msg = '\0';
 
 #if FN_DEBUG
@@ -3569,7 +3823,7 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
     }
 
     /* precaution */
-    fn_state_init(&cmd, &state);
+    fn_state_init(&cmd, &state, &indent0);
 
 #if FN_DEBUG
     fprintf(stderr, "gretl_function_exec: argc = %d\n", args->argc);
@@ -3600,22 +3854,35 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 	*line = '\0';
 	gretl_exec_state_init(&state, FUNCTION_EXEC, line, &cmd, 
 			      models, prn);
-	if (pdinfo->submode) {
-	    state.subinfo = pdinfo;
+	if (pdinfo->submask != NULL) {
+	    state.submask = copy_datainfo_submask(pdinfo);
 	}
 	state.callback = func_exec_callback;
     }
 
     if (!err) {
-	start_fncall(u, args);
+	start_fncall(u, args, prn);
 	started = 1;
     }
 
     /* get function lines in sequence and check, parse, execute */
 
     for (i=0; i<u->n_lines && !err; i++) {
+	if (*u->lines[i] == '\0') {
+	    continue;
+	}
+
 	strcpy(line, u->lines[i]);
-	err = maybe_exec_line(&state, pZ, &pdinfo);
+	if (gretl_echo_on()) {
+	    pprintf(prn, "? %s\n", line);
+	}
+
+	err = maybe_exec_line(&state, pZ, pdinfo);
+	if (err) {
+	    fprintf(stderr, "error on line %d of function %s\n", i+1, u->name);
+	    fprintf(stderr, "> %s\n", line);
+	}
+
 	if (state.funcerr) {
 #if UDEBUG
 	    fprintf(stderr, "funcerr: gretl_function_exec: i=%d, line: '%s'\n", 
@@ -3624,17 +3891,19 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 	    pprintf(prn, "%s: %s\n", u->name, state.cmd->param);
 	    set_funcerr_message(u, state.cmd->param);
 	}
+
 	if (gretl_execute_loop()) { 
 #if UDEBUG
 	    fprintf(stderr, "gretl_function_exec: calling gretl_loop_exec\n");
 #endif
-	    err = gretl_loop_exec(&state, pZ, &pdinfo);
+	    err = gretl_loop_exec(&state, pZ, pdinfo);
 	    if (state.funcerr) {
 		pprintf(prn, "%s: %s\n", u->name, state.cmd->param);
 		set_funcerr_message(u, state.cmd->param);
 	    }
 	    if (err) {
 		fprintf(stderr, "function_exec: breaking on error in loop\n");
+		fprintf(stderr, "error on line %d of function %s\n", i+1, u->name);
 		break;
 	    }
 #if UDEBUG
@@ -3643,21 +3912,32 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 	}
     }
 
+#if UDEBUG
+    fprintf(stderr, "gretl_function_exec: finished main exec, err = %d\n", err);
+#endif
+
     /* restore the sample that was in place on entry */
 
     if (complex_subsampled()) {
-	if (state.subinfo == NULL) {
-	    /* we weren't sub-sampled on entry: easy */
-	    restore_full_sample(pZ, &pdinfo, NULL);
-	} else if (state.subinfo != pdinfo) {
-	    /* we're differently sub-sampled: complex */
-	    restore_full_sample(pZ, &pdinfo, &state);
+	if (state.submask == NULL) {
+	    /* we were not sub-sampled on entry */
+	    restore_full_sample(pZ, pdinfo, NULL);
+	} else if (submask_cmp(state.submask, pdinfo->submask)) {
+	    /* we were sub-sampled differently on entry */
+	    restore_full_sample(pZ, pdinfo, NULL);
+	    restrict_sample_from_mask(state.submask, pZ, pdinfo);
 	} 
-    } 
+    }
 
     pdinfo->t1 = orig_t1;
     pdinfo->t2 = orig_t2;
 
+    if (err) {
+	gretl_if_state_clear();
+    } else {
+	err = gretl_if_state_check(indent0);
+    }
+
     function_assign_returns(u, args, rtype, *pZ, pdinfo,
 			    ret, descrip, prn, &err);
 
@@ -3681,7 +3961,7 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 /* look up name of supplied argument based on name of variable
    inside function */
 
-char *gretl_func_get_arg_name (const char *argvar)
+char *gretl_func_get_arg_name (const char *argvar, int *err)
 {
     ufunc *u = currently_called_function();
     char *ret = NULL;
@@ -3701,6 +3981,9 @@ char *gretl_func_get_arg_name (const char *argvar)
 
     if (ret == NULL) {
 	ret = gretl_strdup("");
+	if (ret == NULL) {
+	    *err = E_ALLOC;
+	}
     }
 
     return ret;
