@@ -52,6 +52,8 @@ int fn_param_type (const ufunc *fun, int i);
 
 const char *fn_param_name (const ufunc *fun, int i);
 
+const char *fn_param_descrip (const ufunc *fun, int i);
+
 double fn_param_default (const ufunc *fun, int i);
 
 double fn_param_minval (const ufunc *fun, int i);
@@ -87,7 +89,7 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 			 void *ret, char **descrip,
 			 PRN *prn);
 
-char *gretl_func_get_arg_name (const char *argvar);
+char *gretl_func_get_arg_name (const char *argvar, int *err);
 
 void set_drop_function_vars (int s);
 
@@ -155,4 +157,8 @@ void fn_args_free (fnargs *args);
 
 int push_fn_arg (fnargs *args, int type, void *p);
 
+void adjust_indent (const char *line, int *this_indent,
+		    int *next_indent);
+
+
 #endif /* GRETL_FUNC_H */
