@@ -170,7 +170,6 @@ static int gretl_cpage;
 # endif
 static int using_utf8;
 
-
 /* Use g_get_charset() to determine the current local character set,
    and record this information.  If we get an "ISO-XXXX-Y" locale,
    record the numerical elements as gretl_cset_maj and gretl_cset_min
@@ -217,12 +216,24 @@ void set_gretl_charset (const char *s)
 		gretl_cset_maj = gretl_cset_min = 0;
 	    }
 	} 
-# ifdef WIN32
+#ifdef WIN32
 	if (p == NULL) {
 	    sscanf(gretl_charset, "cp%d", &gretl_cpage);
 	}
-# endif
+#endif
     }
+
+#ifdef WIN32
+    fprintf(stderr, "codepage = %d\n", gretl_cpage);
+    if (gretl_cpage != 1250) {
+	char *e = getenv("GRETL_CPAGE");
+
+	if (e != NULL && !strcmp(e, "1250")) {
+	    gretl_cpage = 1250;
+	    fprintf(stderr, "revised codepage to 1250\n");
+	}
+    }
+#endif
 }
 
 static const char *get_gretl_charset (void)
@@ -244,60 +255,6 @@ static const char *get_gretl_charset (void)
     return NULL;
 }
 
-static const char *gnuplot_encoding_string (void)
-{
-    static char gp_enc[12] = {0};
-
-    if (gretl_cset_maj != 8859) {
-	return NULL;
-    }
-
-    if (*gp_enc != '\0') {
-	return gp_enc;
-    }
-
-    if (gretl_cset_min == 1 || 
-	gretl_cset_min == 2 ||
-	gretl_cset_min == 15) {
-	sprintf(gp_enc, "iso_8859_%d", gretl_cset_min);
-	return gp_enc;
-    } else if (gretl_cset_min == 9) {
-	static int l5 = -1;
-
-	if (l5 < 0) {
-	    l5 = gnuplot_has_latin5();
-	}
-	if (l5) {
-	    strcpy(gp_enc, "iso_8859_9");
-	    return gp_enc;
-	}
-    }
-
-    return NULL;
-}
-
-void pprint_gnuplot_encoding (const char *termstr, PRN *prn)
-{
-    if (strstr(termstr, "postscript")) {
-	const char *enc = gnuplot_encoding_string();
-
-	if (enc != NULL) {
-	    pprintf(prn, "set encoding %s\n", enc);
-	}
-    }
-}
-
-void fprint_gnuplot_encoding (const char *termstr, FILE *fp)
-{
-    if (strstr(termstr, "postscript")) {
-	const char *enc = gnuplot_encoding_string();
-
-	if (enc != NULL) {
-	    fprintf(fp, "set encoding %s\n", enc);
-	}
-    }
-}
-
 int iso_latin_version (void)
 {
     char *lang = NULL;
@@ -315,6 +272,8 @@ int iso_latin_version (void)
 	return 1;
     } else if (gretl_cpage == 1250) {
 	return 2;
+    } else if (gretl_cpage == 1254) {
+	return 9;
     }
 # endif
 
@@ -331,50 +290,24 @@ int iso_latin_version (void)
     return 1;
 }
 
-static const char *get_gp_charset (void)
-{
-    if (iso_latin_version() == 2) {
-#ifdef WIN32
-	return "CP1250";
-#else
-	return "ISO-8859-2";
-#endif
-    } else if (iso_latin_version() == 9) {
-	return "ISO-8859-9";
-    } else {
-	return "ISO-8859-1";
-    }
-}
-
 char *gp_gettext (const char *msgid)
 {
-    static const char *cset;
-    char *ret;
-
-    if (gnuplot_png_terminal() == GP_PNG_CAIRO) {
-	/* UTF-8 wanted */
-	return gettext(msgid);
-    }
-
-    if (cset == NULL) {
-	cset = get_gp_charset();
-    }
-
-    bind_textdomain_codeset(PACKAGE, cset);
-    ret = gettext(msgid);
-    bind_textdomain_codeset(PACKAGE, "UTF-8");
-
-    return ret;
+    return gettext(msgid);
 } 
 
 char *iso_gettext (const char *msgid)
 {
-    static int iso_ok = -1;
+    static int iso_switch = -1;
     static const char *cset;
     static int cli;
     char *ret;
 
-    /* command line program: switching of codesets is not required */
+    /* command line program: switching of codesets should not be 
+       required, since unlike the GUI program there's no need
+       to force UTF-8 as the default.  Though there might be 
+       a problem with gnuplot, perhaps?
+    */
+
     if (!strcmp(msgid, "@CLI_INIT")) {
 	cli = 1;
 	return NULL;
@@ -384,14 +317,30 @@ char *iso_gettext (const char *msgid)
 	return gettext(msgid);
     }
 
-    if (iso_ok < 0) {
+    /* iso_switch: we'll reckon that if the system character set is
+       not UTF-8, and is an ISO-8859-N or Windows CP12NN 8-bit set,
+       then we should probably recode when printing translated strings
+       in the context of writing TeX, CSV and RTF files.  If
+       iso_switch is non-zero (once it's determinate) this makes the
+       I_() gettext macro use the system encoding, otherwise I_() is
+       equivalent to plain _(), which always spits out UTF-8.  There's
+       a possible FIXME here: perhaps we should always recode for RTF
+       files -- is a UTF-8 encoded RTF file kosher?  But this seems
+       like a corner case.
+    */
+
+    if (iso_switch < 0) {
 	/* not yet determined */
 	cset = get_gretl_charset();
-	fprintf(stderr, "get_gretl_charset gave %s\n", cset);
-	iso_ok = (cset != NULL);
+	if (cset == NULL) {
+	    fprintf(stderr, "get_gretl_charset: using UTF-8\n");
+	} else {
+	    fprintf(stderr, "get_gretl_charset gave %s\n", cset);
+	}
+	iso_switch = (cset != NULL);
     }
 
-    if (iso_ok) {
+    if (iso_switch) {
 	bind_textdomain_codeset(PACKAGE, cset);
 	ret = gettext(msgid);
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
@@ -671,274 +620,6 @@ char *get_month_name (char *mname, int m)
 
 #ifdef ENABLE_NLS
 
-struct l2sym {
-    int l2val;   /* ISO-8859-2 (or CP1250) character code */
-    int ucs2val; /* corresponding UCS-2 code */
-};
-
-# ifndef WIN32
-
-/* ISO-8859-2 */
-
-static struct l2sym l2table[] = { 
-    { 161, 260 }, /*  A; */
-    { 162, 728 }, /*  '( */
-    { 163, 321 }, /*  L/ */
-    { 165, 317 }, /*  L< */
-    { 166, 346 }, /*  S' */
-    { 169, 352 }, /*  S< */
-    { 170, 350 }, /*  S, */
-    { 171, 356 }, /*  T< */
-    { 172, 377 }, /*  Z' */
-    { 174, 381 }, /*  Z< */
-    { 175, 379 }, /*  Z. */
-    { 177, 261 }, /*  a; */
-    { 178, 731 }, /*  '; */
-    { 179, 322 }, /*  l/ */
-    { 181, 318 }, /*  l< */
-    { 182, 347 }, /*  s' */
-    { 183, 711 }, /*  '< */
-    { 185, 353 }, /*  s< */
-    { 186, 351 }, /*  s, */
-    { 187, 357 }, /*  t< */
-    { 188, 378 }, /*  z' */
-    { 189, 733 }, /*  '" */
-    { 190, 382 }, /*  z< */
-    { 191, 380 }, /*  z. */
-    { 192, 340 }, /*  R' */
-    { 195, 258 }, /*  A( */
-    { 197, 313 }, /*  L' */
-    { 198, 262 }, /*  C' */
-    { 200, 268 }, /*  C< */
-    { 202, 280 }, /*  E; */
-    { 204, 282 }, /*  E< */
-    { 207, 270 }, /*  D< */
-    { 208, 272 }, /*  D/ */
-    { 209, 323 }, /*  N' */
-    { 210, 327 }, /*  N< */
-    { 213, 336 }, /*  O" */
-    { 216, 344 }, /*  R< */
-    { 217, 366 }, /*  U0 */
-    { 219, 368 }, /*  U" */
-    { 222, 354 }, /*  T, */
-    { 224, 341 }, /*  r' */
-    { 227, 259 }, /*  a( */
-    { 229, 314 }, /*  l' */
-    { 230, 263 }, /*  c' */
-    { 232, 269 }, /*  c< */
-    { 234, 281 }, /*  e; */
-    { 236, 283 }, /*  e< */
-    { 239, 271 }, /*  d< */
-    { 240, 273 }, /*  d/ */
-    { 241, 324 }, /*  n' */
-    { 242, 328 }, /*  n< */
-    { 245, 337 }, /*  o" */
-    { 248, 345 }, /*  r< */
-    { 249, 367 }, /*  u0 */
-    { 251, 369 }, /*  u" */
-    { 254, 355 }, /*  t, */
-    { 255, 729 }  /*  '. */
-};
-
-static int l2_to_ucs (int c)
-{
-    int i, n = sizeof l2table / sizeof l2table[0];
-
-    for (i=0; i<n; i++) {
-	if (c == l2table[i].l2val) {
-	    return l2table[i].ucs2val;
-	}
-    }
-
-    return c;
-}
-
-static int ucs_to_l2 (int c)
-{
-    int i, n = sizeof l2table / sizeof l2table[0];
-
-    for (i=0; i<n; i++) {
-	if (c == l2table[i].ucs2val) {
-	    return l2table[i].l2val;
-	}
-    }
-
-    return c;
-}
-
-# endif
-
-/* Windows codepage 1250 */
-
-static struct l2sym cptable[] = { 
-    { 128, 8364 }, /*  Eu */
-    { 130, 8218 }, /*  .9 */
-    { 132, 8222 }, /*  :9 */
-    { 133, 8230 }, /*  .3 */
-    { 134, 8224 }, /*  /- */
-    { 135, 8225 }, /*  /= */
-    { 137, 8240 }, /*  %0 */
-    { 138,  352 }, /*  S< */
-    { 139, 8249 }, /*  <1 */
-    { 140,  346 }, /*  S' */
-    { 141,  356 }, /*  T< */
-    { 142,  381 }, /*  Z< */
-    { 143,  377 }, /*  Z' */
-    { 145, 8216 }, /*  '6 */
-    { 146, 8217 }, /*  '9 */
-    { 147, 8220 }, /*  "6 */
-    { 148, 8221 }, /*  "9 */
-    { 149, 8226 }, /*  sb */
-    { 150, 8211 }, /*  -N */
-    { 151, 8212 }, /*  -M */
-    { 153, 8482 }, /*  TM */
-    { 154,  353 }, /*  s< */
-    { 155, 8250 }, /*  >1 */
-    { 156,  347 }, /*  s' */
-    { 157,  357 }, /*  t< */
-    { 158,  382 }, /*  z< */
-    { 159,  378 }, /*  z' */
-    { 161,  711 }, /*  '< */
-    { 162,  728 }, /*  '( */
-    { 163,  321 }, /*  L/ */
-    { 165,  260 }, /*  A; */
-    { 170,  350 }, /*  S, */
-    { 175,  379 }, /*  Z. */
-    { 178,  731 }, /*  '; */
-    { 179,  322 }, /*  l/ */
-    { 185,  261 }, /*  a; */
-    { 186,  351 }, /*  s, */
-    { 188,  317 }, /*  L< */
-    { 189,  733 }, /*  '" */
-    { 190,  318 }, /*  l< */
-    { 191,  380 }, /*  z. */
-    { 192,  340 }, /*  R' */
-    { 195,  258 }, /*  A( */
-    { 197,  313 }, /*  L' */
-    { 198,  262 }, /*  C' */
-    { 200,  268 }, /*  C< */
-    { 202,  280 }, /*  E; */
-    { 204,  282 }, /*  E< */
-    { 207,  270 }, /*  D< */
-    { 208,  272 }, /*  D/ */
-    { 209,  323 }, /*  N' */
-    { 210,  327 }, /*  N< */
-    { 213,  336 }, /*  O" */
-    { 216,  344 }, /*  R< */
-    { 217,  366 }, /*  U0 */
-    { 219,  368 }, /*  U" */
-    { 222,  354 }, /*  T, */
-    { 224,  341 }, /*  r' */
-    { 227,  259 }, /*  a( */
-    { 229,  314 }, /*  l' */
-    { 230,  263 }, /*  c' */
-    { 232,  269 }, /*  c< */
-    { 234,  281 }, /*  e; */
-    { 236,  283 }, /*  e< */
-    { 239,  271 }, /*  d< */
-    { 240,  273 }, /*  d/ */
-    { 241,  324 }, /*  n' */
-    { 242,  328 }, /*  n< */
-    { 245,  337 }, /*  o" */
-    { 248,  345 }, /*  r< */
-    { 249,  367 }, /*  u0 */
-    { 251,  369 }, /*  u" */
-    { 254,  355 }, /*  t, */
-    { 255,  729 }  /*  '. */
-};
-
-# ifdef WIN32
-
-static int cp_to_ucs (int c)
-{
-    int i, n = sizeof cptable / sizeof cptable[0];
-
-    for (i=0; i<n; i++) {
-	if (c == cptable[i].l2val) {
-	    return cptable[i].ucs2val;
-	}
-    }
-
-    return c;
-}
-
-# endif
-
-static int ucs_to_cp (int c)
-{
-    int i, n = sizeof cptable / sizeof cptable[0];
-
-    for (i=0; i<n; i++) {
-	if (c == cptable[i].ucs2val) {
-	    return cptable[i].l2val;
-	}
-    }
-
-    return c;
-}
-
-char *sprint_l2_to_html (char *targ, const char *s, size_t len)
-{
-    unsigned char c;
-    char *p = targ;
-
-    *p = '\0';
-
-    while ((c = *s)) {
-# ifndef WIN32
-	if (c > 160) {
-	    sprintf(p, "&#%d;", l2_to_ucs(c));
-	    p = strchr(p, ';') + 1;
-	} else if (c > 127) {
-	    sprintf(p, "&#%d;", c);
-	    p = strchr(p, ';') + 1;
-	} else {
-	    *p++ = c;
-	}
-# else
-	if (c > 127) {
-	    sprintf(p, "&#%d;", cp_to_ucs(c));
-	    p = strchr(p, ';') + 1;
-	} else {
-	    *p++ = c;
-	}
-# endif
-	s++;
-	if (p - targ > len - 8) {
-	    break;
-	}
-    }
-
-    *p = '\0';
-
-    return targ;
-}
-
-char *sprint_html_to_l2 (char *targ, const char *s)
-{
-    int u;
-    char *p = targ;
-
-    *p = '\0';
-
-    while (*s) {
-	if (sscanf(s, "&#%d;", &u)) {
-# ifdef WIN32
-	    *p++ = ucs_to_cp(u);
-# else
-	    *p++ = ucs_to_l2(u);
-# endif
-	    s = strchr(s, ';') + 1;
-	} else {
-	    *p++ = *s++;
-	}
-    }
-
-    *p = '\0';
-
-    return targ;
-}
-
 char *sprint_l2_to_ascii (char *targ, const char *s, size_t len)
 {
     iso_to_ascii_translate(targ, s, 2);
@@ -946,97 +627,96 @@ char *sprint_l2_to_ascii (char *targ, const char *s, size_t len)
     return targ;
 }
 
-int print_as_html (const char *s, FILE *fp)
+enum {
+    ENC_ISO_LATIN,
+    ENC_CODEPAGE
+};
+
+/* Construct a codeset string to pass to g_convert, to convert UTF-8
+   encoded strings for gnuplot formats that can't handle UTF-8.  At
+   this point we don't check if gnuplot can deal with the target
+   codeset via its "set encoding" command, we're just trying to get
+   the text localized: the graph may work even without an explicit
+   "set encoding".
+*/
+
+static char *get_gp_encoding_set (char *s, int targ)
 {
-    unsigned char c;
-    int nread = 0;
+    int latin = iso_latin_version();
 
-    while ((c = *s)) {
-        if (c > 160) {
-# ifdef WIN32
-	    fprintf(fp, "&#%d;", cp_to_ucs(c));
-# else
-            fprintf(fp, "&#%d;", l2_to_ucs(c));
-# endif
-        } else if (c > 127) {
-            fprintf(fp, "&#%d;", c);
-        } else {
-            fputc(c, fp);
-        }
-        s++;
-        nread++;
-    }
-
-    return nread;
-}
-
-int print_as_locale (const char *s, FILE *fp)
-{
-    int u, nwrote = 0;
-
-    while (*s) {
-	if (sscanf(s, "&#%d;", &u)) {
-# ifdef WIN32
-	    fputc(ucs_to_cp(u), fp);
-# else
-	    fputc(ucs_to_l2(u), fp);
-# endif
-	    s = strchr(s, ';') + 1;
+    if (targ == ENC_ISO_LATIN) {
+	strcpy(s, "ISO-8859-");
+	if (latin == 2) {
+	    strcat(s, "2");
+	} else if (latin == 9) {
+	    strcat(s, "9");
+	} else if (latin == 15) {
+	    strcat(s, "15");
 	} else {
-	    fputc(*s++, fp);
+	    /* default is ISO-8859-1 */
+	    strcat(s, "1");
 	}
-	nwrote++;
-    }
-
-    return nwrote;
-}
-
-int pprint_as_latin (PRN *prn, const char *s, int emf)
-{
-    int u, nwrote = 0;
-
-    while (*s) {
-	if (sscanf(s, "&#%d;", &u)) {
-# ifdef WIN32
-	    pputc(prn, ucs_to_cp(u));
-# else
-	    if (emf) {
-		pputc(prn, ucs_to_cp(u));
-	    } else {
-		pputc(prn, ucs_to_l2(u));
-	    }
-# endif
-	    s = strchr(s, ';') + 1;
+    } else {
+	strcpy(s, "CP125");
+	if (latin == 2) {
+	    strcat(s, "0");
+	} else if (latin == 9) {
+	    strcat(s, "4");
 	} else {
-	    pputc(prn, *s++);
-	}
-	nwrote++;
+	    /* default is CP1252 */
+	    strcat(s, "2");
+	} 
     }
 
-    return nwrote;
+    return s;
 }
 
+/* convert from UTF-8 to ISO latin, for, e.g., EPS graphs */
+
 char *utf8_to_latin (const char *s)
 {
+    char to_set[12];
     gsize read, wrote;
+    GError *err = NULL;
+    char *ret = NULL;
 
-    if (iso_latin_version() == 2) {
-# ifdef WIN32
-	return g_convert(s, -1, "CP1250", "UTF-8",
-			 &read, &wrote, NULL);
-# else
-	return g_convert(s, -1, "ISO-8859-2", "UTF-8",
-			 &read, &wrote, NULL);
-# endif
-    } else {
-	return g_convert(s, -1, "ISO-8859-1", "UTF-8",
-			 &read, &wrote, NULL);
+    get_gp_encoding_set(to_set, ENC_ISO_LATIN);
+
+    ret = g_convert(s, -1, to_set, "UTF-8",
+		    &read, &wrote, &err);
+
+    if (err != NULL) {
+	gretl_errmsg_set(err->message);
+	g_error_free(err);
     }
+
+    return ret;
 }
 
-/* allow TAB, CR, LF, FF */
+/* convert from UTF-8 to Windows codepage, for EMF graphs */
 
-#define ascii_ctrl(a) (a==9 || a==10 || a==12 || a==13)
+char *utf8_to_cp (const char *s)
+{
+    char to_set[8];
+    gsize read, wrote;
+    GError *err = NULL;
+    char *ret = NULL;
+
+    get_gp_encoding_set(to_set, ENC_CODEPAGE);
+
+    ret = g_convert(s, -1, to_set, "UTF-8",
+		    &read, &wrote, &err);
+
+    if (err != NULL) {
+	gretl_errmsg_set(err->message);
+	g_error_free(err);
+    }
+
+    return ret;
+}
+
+#define ascii_ctrl(a) (a == '\t' || a == '\n' || \
+                       a == '\r' || a == CTRLZ)
 
 int gretl_is_ascii (const char *buf)
 {
@@ -1044,7 +724,7 @@ int gretl_is_ascii (const char *buf)
 
     while (*buf) {
 	a = *buf;
-	if ((a < 32 && !(ascii_ctrl(a))) || a > 126) {
+	if (a > 126 || (a < 32 && !(ascii_ctrl(a)))) {
 	    return 0;
 	}
 	buf++;
