@@ -44,26 +44,14 @@ char *maybe_iso_gettext (const char *msgid);
 
 void set_gretl_charset (const char *s);
 
-void pprint_gnuplot_encoding (const char *termstr, PRN *prn);
-
-void fprint_gnuplot_encoding (const char *termstr, FILE *fp);
-
 int iso_latin_version (void);
 
-char *sprint_l2_to_html (char *targ, const char *s, size_t len);
-
 char *sprint_l2_to_ascii (char *targ, const char *s, size_t len);
 
-char *sprint_html_to_l2 (char *targ, const char *s);
-
-int print_as_html (const char *s, FILE *fp);
-
-int print_as_locale (const char *s, FILE *fp);
-
-int pprint_as_latin (PRN *prn, const char *s, int emf);
-
 char *utf8_to_latin (const char *s);
 
+char *utf8_to_cp (const char *s);
+
 int gretl_is_ascii (const char *buf);
 
 int get_utf_width (const char *str, int width);
