@@ -35,9 +35,17 @@ struct saved_list_ {
     int level;
 };
 
+enum {
+    LIST_OMIT_TIGHT,
+    LIST_OMIT_SLOPPY
+};
+
 static saved_list **list_stack;
 static int n_lists;
 
+static int *real_gretl_list_omit (const int *orig, const int *omit, 
+				  int minpos, int mode, int *err);
+
 static saved_list *saved_list_new (const int *list, const char *name)
 {
     saved_list *sl = malloc(sizeof *sl);
@@ -149,21 +157,127 @@ const char *get_list_name_by_index (int idx)
 int *get_list_by_name (const char *name)
 {
     int *ret = NULL;
-    saved_list *sl = get_saved_list_by_name(name);
+    saved_list *sl;
 
-    if (sl != NULL) {
-	ret = sl->list;
+    if (name != NULL) {
+	sl = get_saved_list_by_name(name);
+	if (sl != NULL) {
+	    ret = sl->list;
+	}
     }
 
     return ret;
 }
 
+/**
+ * append_to_list_by_name:
+ * @targ: the name of the target list.
+ * @add: list to add.
+ *
+ * If @targ is the name of a saved list, append the list
+ * @add to it.
+ *
+ * Returns: 0 on success, non-zero code on failure.
+ */
+
+int append_to_list_by_name (const char *targ, const int *add)
+{
+    saved_list *sl = get_saved_list_by_name(targ);
+    int err = 0;
+
+    if (sl == NULL) {
+	err = E_UNKVAR;
+    } else {
+	int *tmp = gretl_list_copy(sl->list);
+
+	if (tmp == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    err = gretl_list_add_list(&tmp, add);
+	    if (!err) {
+		free(sl->list);
+		sl->list = tmp;
+	    }
+	}
+    } 
+
+    return err;
+}
+
+/**
+ * subtract_from_list_by_name:
+ * @targ: the name of the target list.
+ * @sub: sub-list to remove.
+ *
+ * If @targ is the name of a saved list, remove from @targ
+ * any elements of @sub that it contains.
+ *
+ * Returns: 0 on success, non-zero code on failure.
+ */
+
+int subtract_from_list_by_name (const char *targ, const int *sub)
+{
+    saved_list *sl = get_saved_list_by_name(targ);
+    int err = 0;
+
+    if (sl == NULL) {
+	err = E_UNKVAR;
+    } else {
+	int *tmp = real_gretl_list_omit(sl->list, sub, 1, 
+					LIST_OMIT_SLOPPY,
+					&err);
+
+	if (!err) {
+	    free(sl->list);
+	    sl->list = tmp;
+	}
+    } 
+
+    return err;
+}
+
+/**
+ * replace_list_by_name:
+ * @targ: the name of the target list.
+ * @new: replacement list
+ *
+ * If @targ is the name of a saved list, replace the
+ * list of that name with @new.
+ *
+ * Returns: 0 on success, non-zero code on failure.
+ */
+
+int replace_list_by_name (const char *targ, const int *new)
+{
+    saved_list *sl = get_saved_list_by_name(targ);
+    int err = 0;
+
+    if (sl == NULL) {
+	err = E_UNKVAR;
+    } else {
+	int *tmp = gretl_list_copy(new);
+
+	if (tmp == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    free(sl->list);
+	    sl->list = tmp;
+	}	    
+    }
+
+    return err;
+}
+
 static int real_remember_list (const int *list, const char *name, 
 			       int force_new, PRN *prn)
 {
     saved_list *orig = NULL;
     int err = 0;
 
+    if (list == NULL) {
+	return E_DATA;
+    }
+
 #if LDEBUG
     fprintf(stderr, "remember_list (in): name='%s', force_new=%d,"
 	    " n_lists=%d\n", name, force_new, n_lists);
@@ -203,7 +317,9 @@ static int real_remember_list (const int *list, const char *name,
 	    if (gretl_messages_on()) {
 		const char *realname = list_stack[n_lists]->name;
 
-		pprintf(prn, _("Added list '%s'\n"), realname);
+		if (list[0] > 0) {
+		    pprintf(prn, _("Added list '%s'\n"), realname);
+		}
 		if (strlen(realname) < strlen(name)) {
 		    pprintf(prn, _("Warning: the name was truncated to %d characters\n"), 
 			    LNAMELEN - 1);
@@ -242,27 +358,74 @@ int remember_list (const int *list, const char *name, PRN *prn)
     return real_remember_list(list, name, 0, prn);
 }
 
+static int destroy_saved_list (saved_list *sl)
+{
+    saved_list **lstack;
+    int found = 0;
+    int i, j, err = 0;
+
+    for (i=0; i<n_lists; i++) {
+	if (list_stack[i] == NULL) {
+	    break;
+	}
+	if (list_stack[i] == sl) {
+	    free_saved_list(sl);
+	    for (j=i; j<n_lists - 1; j++) {
+		list_stack[j] = list_stack[j+1];
+	    }
+	    list_stack[n_lists - 1] = NULL;
+	    found = 1;
+	    break;
+	} 
+    }
+
+    if (!found) {
+	err = E_DATA;
+    } else {
+	n_lists--;
+	if (n_lists == 0) {
+	    free(list_stack);
+	    list_stack = NULL;
+	} else {
+	    lstack = realloc(list_stack, n_lists * sizeof *list_stack);
+	    if (lstack == NULL) {
+		err = E_ALLOC;
+	    } else {
+		list_stack = lstack;
+	    } 
+	}
+    }   
+
+    return err;
+}
+
 /**
  * rename_saved_list:
  * @orig: the original name of the list.
  * @new: the new name to be given.
  *
- * Overwrites the name of a saved list.
+ * Renames a saved list from @orig to @new.  If there is
+ * already a list called @new, it is destroyed.
  *
  * Returns: 0 on success, non-zero on error.
  */
 
 int rename_saved_list (const char *orig, const char *new)
 {
-    saved_list *sl;
+    saved_list *sl0, *sl1;
     int err = 0;
 
-    sl = get_saved_list_by_name(orig);
-    if (sl == NULL) {
+    sl0 = get_saved_list_by_name(orig);
+    if (sl0 == NULL) {
 	err = 1;
     } else {
-	*sl->name = '\0';
-	strncat(sl->name, new, LNAMELEN - 1);
+	/* is there already a list called @new? */
+	sl1 = get_saved_list_by_name(new);
+	if (sl1 != NULL) {
+	    err = destroy_saved_list(sl1);
+	}
+	*sl0->name = '\0';
+	strncat(sl0->name, new, LNAMELEN - 1);
     } 
 
     return err;
@@ -415,6 +578,18 @@ int destroy_saved_lists_at_level (int level)
     return err;
 }
 
+int delete_list_by_name (const char *name)
+{
+    saved_list *sl;
+
+    sl = get_saved_list_by_name(name);
+    if (sl == NULL) {
+	return E_UNKVAR;
+    } else {
+	return destroy_saved_list(sl);
+    }
+}
+
 static int var_is_deleted (const int *dlist, int dmin, int i)
 {
     int v = dmin + i - 1;
@@ -905,10 +1080,11 @@ int in_gretl_list (const int *list, int k)
 {
     int i, ret = 0;
 
-    for (i=1; i<=list[0]; i++) {
-	if (list[i] == k) {
-	    ret = i;
-	    break;
+    if (list != NULL) {
+	for (i=1; i<=list[0] && !ret; i++) {
+	    if (list[i] == k) {
+		ret = i;
+	    }
 	}
     }
 
@@ -1037,22 +1213,8 @@ int gretl_list_purge_const (int *list, const double **Z,
     return gotc;
 }
 
-/**
- * gretl_list_add:
- * @orig: an array of integers, the first element of which holds
- * a count of the number of elements following.
- * @add: list of variables to be added.
- * @err: location to receive error code.
- *
- * Creates a list containing the union of elements of @orig 
- * and the elements of @add.  If one or more elements of
- * @add were already present in @orig, the error code is
- * %E_ADDDUP.
- *
- * Returns: new list on success, %NULL on error.
- */
-
-int *gretl_list_add (const int *orig, const int *add, int *err)
+static int *real_gretl_list_union (const int *orig, const int *add, 
+				   int *err, int duperr)
 {
     int i, j, k;
     int *big;
@@ -1061,7 +1223,7 @@ int *gretl_list_add (const int *orig, const int *add, int *err)
 
     *err = 0;
 
-    big = malloc((norig + nadd + 1) * sizeof *big);
+    big = gretl_list_new(norig + nadd);
     if (big == NULL) {
 	*err = E_ALLOC;
 	return NULL;
@@ -1079,9 +1241,13 @@ int *gretl_list_add (const int *orig, const int *add, int *err)
 	for (j=1; j<=norig; j++) {
 	    if (add[i] == orig[j]) {
 		/* a "new" var was already present */
-		free(big);
-		*err = E_ADDDUP;
-		return NULL;
+		if (duperr) {
+		    free(big);
+		    *err = E_ADDDUP;
+		    return NULL;
+		} else {
+		    match = 1;
+		}
 	    }
 	}
 	if (!match) {
@@ -1090,7 +1256,7 @@ int *gretl_list_add (const int *orig, const int *add, int *err)
 	}
     }
 
-    if (big[0] == norig) {
+    if (duperr && big[0] == norig) {
 	free(big);
 	*err = E_NOADD;
 	return NULL;
@@ -1099,6 +1265,92 @@ int *gretl_list_add (const int *orig, const int *add, int *err)
     return big;
 }
 
+/**
+ * gretl_list_add:
+ * @orig: an array of integers, the first element of which holds
+ * a count of the number of elements following.
+ * @add: list of variables to be added.
+ * @err: location to receive error code.
+ *
+ * Creates a list containing the union of elements of @orig 
+ * and the elements of @add.  If one or more elements of
+ * @add were already present in @orig, the error code is
+ * %E_ADDDUP.
+ *
+ * Returns: new list on success, %NULL on error.
+ */
+
+int *gretl_list_add (const int *orig, const int *add, int *err)
+{
+    return real_gretl_list_union(orig, add, err, 1);
+}
+
+/**
+ * gretl_list_union:
+ * @l1: list of integers.
+ * @l2: list of integers.
+ * @err: location to receive error code.
+ *
+ * Creates a list holding the union of @l1 and @l2.
+ *
+ * Returns: new list on success, %NULL on error.
+ */
+
+int *gretl_list_union (const int *l1, const int *l2, int *err)
+{
+    return real_gretl_list_union(l1, l2, err, 0);
+}
+
+/**
+ * gretl_list_intersection:
+ * @l1: list of integers.
+ * @l2: list of integers.
+ * @err: location to receive error code.
+ *
+ * Creates a list holding the intersection of @l1 and @l2.
+ *
+ * Returns: new list on success, %NULL on error.
+ */
+
+int *gretl_list_intersection (const int *l1, const int *l2, int *err)
+{
+    int *ret = NULL;
+    int i, j;
+    int n = 0;
+
+    for (i=1; i<=l1[0]; i++) {
+	for (j=1; j<=l2[0]; j++) {
+	    if (l2[j] == l1[i]) {
+		n++;
+		break;
+	    }
+	}
+    }
+
+    if (n == 0) {
+	ret = gretl_null_list();
+    } else {
+	ret = gretl_list_new(n);
+	if (ret != NULL) {
+	    n = 1;
+	    for (i=1; i<=l1[0]; i++) {
+		for (j=1; j<=l2[0]; j++) {
+		    if (l2[j] == l1[i]) {
+			ret[n++] = l1[i];
+			break;
+		    }
+		}
+	    }
+	}
+    }
+
+    if (ret == NULL) {
+	*err = E_ALLOC;
+    }
+
+    return ret;
+}
+
 /**
  * gretl_list_omit_last:
  * @orig: an array of integers, the first element of which holds
@@ -1163,26 +1415,12 @@ static int list_count (const int *list)
     return k;
 }
 
-/**
- * gretl_list_omit:
- * @orig: an array of integers, the first element of which holds
- * a count of the number of elements following.
- * @omit: list of variables to drop.
- * @minpos: minimum position to check.  This should be 2 for a regular
- * regression list, to skip the dependent var in position 1; but in
- * other contexts it may be 1 to start from the first element of @orig.
- * @err: pointer to receive error code.
- *
- * Creates a list containing the elements of @orig that are not
- * present in @omit.
- *
- * Returns: new list on success, %NULL on error.
- */
-
-int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
+static int *real_gretl_list_omit (const int *orig, const int *omit, 
+				  int minpos, int mode, int *err)
 {
     const int nomit = omit[0];
     const int norig = list_count(orig);
+    int real_nomit = nomit;
     int *smal = NULL;
     int i, j, k;
 
@@ -1193,10 +1431,14 @@ int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
 	int pos = in_gretl_list(orig, omit[i]);
 
 	if (pos < minpos) {
-	    sprintf(gretl_errmsg, _("Variable %d was not in the original list"),
-		    omit[i]);
-	    *err = 1;
-	    return NULL;
+	    if (mode == LIST_OMIT_TIGHT) {
+		sprintf(gretl_errmsg, _("Variable %d was not in the original list"),
+			omit[i]);
+		*err = 1;
+		return NULL;
+	    } else {
+		real_nomit--;
+	    }
 	}
     }
 
@@ -1208,7 +1450,7 @@ int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
 	}
     }
 
-    if (nomit == norig) {
+    if (real_nomit == norig) {
 	/* omitting all */
 	smal = gretl_null_list();
 	if (smal == NULL) {
@@ -1217,7 +1459,7 @@ int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
 	return smal;
     }
 
-    smal = gretl_list_new(norig - nomit);
+    smal = gretl_list_new(norig - real_nomit);
     if (smal == NULL) {
 	*err = E_ALLOC;
 	return NULL;
@@ -1246,6 +1488,27 @@ int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
     return smal;
 }
 
+/**
+ * gretl_list_omit:
+ * @orig: an array of integers, the first element of which holds
+ * a count of the number of elements following.
+ * @omit: list of variables to drop.
+ * @minpos: minimum position to check.  This should be 2 for a regular
+ * regression list, to skip the dependent var in position 1; but in
+ * other contexts it may be 1 to start from the first element of @orig.
+ * @err: pointer to receive error code.
+ *
+ * Creates a list containing the elements of @orig that are not
+ * present in @omit.
+ *
+ * Returns: new list on success, %NULL on error.
+ */
+
+int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
+{
+    return real_gretl_list_omit(orig, omit, minpos, LIST_OMIT_TIGHT, err);
+}
+
 /**
  * gretl_list_diff:
  * @targ: target list (must be pre-allocated).
@@ -1359,7 +1622,7 @@ int *gretl_list_diff_new (const int *biglist, const int *sublist,
  * Adds @src onto the end of @targ.  The length of @targ becomes the
  * sum of the lengths of the two original lists.
  *
- * Returns: 0 on success, %E_ALLOC on failure.
+ * Returns: 0 on success, non-zero on failure.
  */
 
 int gretl_list_add_list (int **targ, const int *src)
@@ -1369,7 +1632,17 @@ int gretl_list_add_list (int **targ, const int *src)
     int n2 = src[0];
     int i, err = 0;
 
+    if (targ == NULL || *targ == NULL) {
+	return E_DATA;
+    }
+
+    if (src == NULL || *src == 0) {
+	/* no-op */
+	return 0;
+    }
+
     big = realloc(*targ, (n1 + n2 + 1) * sizeof *big);
+
     if (big == NULL) {
 	err = E_ALLOC;
     } else {
@@ -1614,16 +1887,17 @@ int gretl_list_has_separator (const int *list)
 int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2)
 {
     int *list1 = NULL, *list2 = NULL;
-    int i, n = -1;
+    int i, n = 0;
+    int err = 0;
 
-    for (i=1; i<list[0] && n<0; i++) {
+    for (i=1; i<=list[0] && n==0; i++) {
 	if (list[i] == LISTSEP) {
 	    n = i;
 	}
     }
 
-    if (n < 0) {
-	return E_PARSE;
+    if (n == 0) {
+	err = E_PARSE;
     }
 
     if (n > 1) {
@@ -1636,20 +1910,22 @@ int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2)
 	}
     }
 
-    list2 = gretl_list_new(list[0] - n);
-    if (list2 == NULL) {
-	free(list1);
-	return E_ALLOC;
-    }
+    if (n < list[0]) {
+	list2 = gretl_list_new(list[0] - n);
+	if (list2 == NULL) {
+	    free(list1);
+	    return E_ALLOC;
+	}
 
-    for (i=1; i<=list2[0]; i++) {
-	list2[i] = list[i + n];
+	for (i=1; i<=list2[0]; i++) {
+	    list2[i] = list[i + n];
+	}
     }
 
     *plist1 = list1;
     *plist2 = list2;
     
-    return 0;
+    return err;
 }
 
 static int real_list_dup (const int *list, int start, int stop)
@@ -1707,7 +1983,7 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
 		break;
 	    }
 	}
-    } else if (ci == TSLS || ci == HECKIT) {
+    } else if (ci == TSLS || ci == HECKIT || ci == EQUATION) {
 	multi = 1;
 	for (i=2; i<list[0]; i++) {
 	    if (list[i] == LISTSEP) {
@@ -2088,3 +2364,43 @@ int gretl_serialize_lists (const char *fname)
 
     return 0;
 }
+
+/**
+ * gretl_list_print:
+ * @lname: name of list.
+ * @pdinfo:
+ * @prn:
+ * 
+ * Prints to @prn the given @list of variables, by name.
+ */
+
+void gretl_list_print (const char *lname, const DATAINFO *pdinfo,
+		       PRN *prn)
+{
+    const int *list = get_list_by_name(lname);
+    int testlen = 62;
+    int i, li, len = 0;
+
+    if (list == NULL) {
+	pprintf(prn, _("Unknown variable '%s'"), lname);
+	pputc(prn, '\n');
+    } else if (list[0] == 0) {
+	pputs(prn, "null\n");
+    } else {
+	for (i=1; i<=list[0]; i++) {
+	    li = list[i];
+	    if (li == LISTSEP) {
+		len += pputs(prn, "; ");
+	    } else if (li < 0 || li >= pdinfo->v) {
+		len += pputs(prn, "?? ");
+	    } else {
+		len += pprintf(prn, "%s ", pdinfo->varname[li]);
+		if (len > testlen && i < list[0]) {
+		    pputs(prn, "\\\n "); 
+		    len = 1;
+		}
+	    }
+	}
+	pputc(prn, '\n');
+    }
+}
