@@ -49,6 +49,10 @@ int gretl_list_purge_const (int *list, const double **Z,
 
 int *gretl_list_add (const int *orig, const int *add, int *err);
 
+int *gretl_list_union (const int *l1, const int *l2, int *err);
+
+int *gretl_list_intersection (const int *l1, const int *l2, int *err);
+
 int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err);
 
 int *gretl_list_omit_last (const int *orig, int *err);
@@ -98,6 +102,12 @@ const char *get_list_name_by_index (int idx);
 
 int *get_list_by_name (const char *name);
 
+int append_to_list_by_name (const char *targ, const int *add);
+
+int subtract_from_list_by_name (const char *targ, const int *sub);
+
+int replace_list_by_name (const char *targ, const int *new);
+
 int remember_list (const int *list, const char *name, PRN *prn);
 
 int copy_named_list_as (const char *orig, const char *new);
@@ -108,6 +118,8 @@ int rename_saved_list (const char *orig, const char *new);
 
 int create_named_null_list (const char *name);
 
+int delete_list_by_name (const char *name);
+
 int destroy_saved_lists_at_level (int level);
 
 int gretl_lists_revise (const int *dlist, int dmin);
@@ -118,4 +130,8 @@ int load_user_lists_file (const char *fname);
 
 int gretl_serialize_lists (const char *fname);
 
+void gretl_list_print (const char *lname, 
+		       const DATAINFO *pdinfo,
+		       PRN *prn);
+
 #endif /* GRETL_LIST_H */
