@@ -59,9 +59,9 @@ struct _gretl_matrix {
  * gretl_matrix_get:
  * @m: matrix.
  * @i: row.
- * @column.
+ * @j: column.
  * 
- * Gives the @i, @j element of @m.
+ * Retrieves the @i, @j element of @m.
  */
 
 #define gretl_matrix_get(m,i,j) (m->val[(j)*m->rows+(i)])
@@ -77,16 +77,16 @@ struct _gretl_matrix {
 #define gretl_vector_get(v,i) (v->val[i])
 
 /**
- * gretl_matrix_get:
+ * gretl_matrix_set:
  * @m: matrix.
  * @i: row.
- * @column.
+ * @j: column.
  * @x: value to set.
  * 
  * Sets the @i, @j element of @m to @x.
  */
 
-#define gretl_matrix_set(m,i,j,x) (m->val[(j)*m->rows+(i)]=x)
+#define gretl_matrix_set(m,i,j,x) ((m)->val[(j)*(m)->rows+(i)]=x)
 
 /**
  * gretl_vector_set:
@@ -97,7 +97,7 @@ struct _gretl_matrix {
  * Sets element @i of @v to @x.
  */
 
-#define gretl_vector_set(v,i,x) (v->val[i]=x)
+#define gretl_vector_set(v,i,x) ((v)->val[i]=x)
 
 /**
  * gretl_matrix_cols:
@@ -173,6 +173,10 @@ int get_gretl_matrix_err (void);
 
 void clear_gretl_matrix_err (void);
 
+void gretl_matrix_print (const gretl_matrix *m, const char *msg);
+
+int gretl_matrix_xna_check (const gretl_matrix *m);
+
 int gretl_matrix_is_symmetric (const gretl_matrix *m);
 
 int gretl_matrix_is_idempotent (const gretl_matrix *m);
@@ -201,6 +205,9 @@ gretl_matrix *gretl_matrix_seq (int start, int end);
 
 gretl_matrix *gretl_matrix_copy (const gretl_matrix *m);
 
+int gretl_matrix_copy_row (gretl_matrix *dest, int di,
+			   const gretl_matrix *src, int si);
+
 int gretl_matrix_inscribe_I (gretl_matrix *m, int row, int col, int n);
 
 gretl_matrix *gretl_matrix_copy_transpose (const gretl_matrix *m);
@@ -213,6 +220,9 @@ int gretl_matrix_random_fill (gretl_matrix *m, int dist);
 
 gretl_matrix *gretl_random_matrix_new (int r, int c, int dist);
 
+gretl_matrix *gretl_matrix_resample (const gretl_matrix *m, int r,
+				     int *err);
+
 double gretl_vector_mean (const gretl_vector *v);
 
 double gretl_vector_variance (const gretl_vector *v);
@@ -245,6 +255,9 @@ gretl_matrix *gretl_matrix_complex_divide (const gretl_matrix *a,
 
 gretl_matrix *gretl_matrix_exp (const gretl_matrix *m, int *err);
 
+gretl_matrix *gretl_matrix_polroots (const gretl_matrix *a,
+				     int *err);
+
 void gretl_matrix_raise (gretl_matrix *m, double x);
 
 void gretl_matrix_free (gretl_matrix *m);
@@ -362,6 +375,9 @@ void gretl_matrix_demean_by_column (gretl_matrix *m);
 
 gretl_matrix *gretl_matrix_vcv (gretl_matrix *m);
 
+gretl_matrix *gretl_matrix_quantiles (const gretl_matrix *m,
+				      double p, int *err);
+
 double gretl_matrix_determinant (gretl_matrix *a, int *err);
 
 double gretl_matrix_log_determinant (gretl_matrix *a, int *err);
@@ -374,7 +390,7 @@ double gretl_matrix_one_norm (const gretl_matrix *m);
 
 double gretl_matrix_infinity_norm (const gretl_matrix *m);
 
-int gretl_LU_solve (gretl_matrix *a, gretl_vector *b);
+int gretl_LU_solve (gretl_matrix *a, gretl_matrix *b);
 
 int gretl_cholesky_decomp_solve (gretl_matrix *a, gretl_matrix *b);
 
@@ -403,11 +419,15 @@ int gretl_matrix_moore_penrose (gretl_matrix *A);
 
 int gretl_SVD_invert_matrix (gretl_matrix *a);
 
+int gretl_invpd (gretl_matrix *a);
+
 int gretl_matrix_SVD (const gretl_matrix *a, gretl_matrix **pu, 
 		      gretl_vector **ps, gretl_matrix **pvt);
 
 double gretl_symmetric_matrix_rcond (const gretl_matrix *m, int *err);
 
+double gretl_matrix_rcond (const gretl_matrix *m, int *err);
+
 int gretl_general_eigen_sort (gretl_matrix *evals, gretl_matrix *evecs, 
 			      int rank);
 
@@ -449,6 +469,11 @@ int
 gretl_matrix_inplace_colcat (gretl_matrix *a, const gretl_matrix *b,
 			     const char *mask);
 
+gretl_matrix *gretl_matrix_cumcol (const gretl_matrix *m, int *err);
+
+gretl_matrix *gretl_matrix_diffcol (const gretl_matrix *m, 
+				    double missval, int *err);
+
 gretl_matrix *gretl_matrix_lag (const gretl_matrix *m, int k, 
 				double missval);
 
@@ -525,15 +550,6 @@ int
 gretl_matrix_diagonal_sandwich (const gretl_vector *d, const gretl_matrix *X,
 				gretl_matrix *DXD);
 
-void 
-gretl_matrix_print_to_prn (const gretl_matrix *m, const char *msg, PRN *prn);
-
-void gretl_matrix_print (const gretl_matrix *m, const char *msg);
-
-void gretl_packed_matrix_print (const gretl_matrix *m, const char *msg);
-
-void debug_print_matrix (const gretl_matrix *m, const char *msg);
-
 void gretl_matrix_set_t1 (gretl_matrix *m, int t);
 
 void gretl_matrix_set_t2 (gretl_matrix *m, int t);
@@ -567,6 +583,10 @@ gretl_matrix *gretl_matrix_values (const double *x, int n,
 gretl_matrix *gretl_matrix_shape (const gretl_matrix *A, 
 				  int r, int c);
 
+gretl_matrix *gretl_matrix_trim_rows (const gretl_matrix *A, 
+				      int ttop, int tbot,
+				      int *err);
+
 gretl_matrix *gretl_matrix_minmax (const gretl_matrix *A, 
 				   int mm, int rc, int idx,
 				   int *err);
@@ -580,6 +600,17 @@ gretl_matrix *matrix_matrix_xtab (const gretl_matrix *x,
 				  const gretl_matrix *y,
 				  int *err);
 
+gretl_matrix *gretl_matrix_bool_sel(const gretl_matrix *A, 
+				    const gretl_matrix *sel, 
+				    int rowsel, int *err);
+
+gretl_matrix *gretl_matrix_sort_by_column (const gretl_matrix *m, 
+					   int k, int *err);
+
 void lapack_mem_free (void);
 
+void set_blas_nmk_min (int n);
+
+int get_blas_nmk_min (void);
+
 #endif /* GRETL_MATRIX_H */
