@@ -27,9 +27,10 @@ typedef enum {
     ARMA_DSPEC = 1 << 1, /* input list includes differences */
     ARMA_X12A  = 1 << 2, /* using X-12-ARIMA to generate estimates */
     ARMA_EXACT = 1 << 3, /* using exact ML */
-    ARMA_VECH  = 1 << 4  /* using vech representation when computing
+    ARMA_VECH  = 1 << 4, /* using vech representation when computing
 			    variance matrix of state for Kalman filter
 			 */
+    ARMA_LS    = 1 << 5  /* using conditional ML, and O/NLS == CML */
 } ArmaFlags;
 
 typedef struct CoeffIntervals_ CoeffIntervals;
@@ -37,6 +38,7 @@ typedef struct CoeffIntervals_ CoeffIntervals;
 struct CoeffIntervals_ {
     int asy;
     int ncoeff;
+    double alpha;
     double t;
     char **names;
     double *coeff;
@@ -46,20 +48,13 @@ struct CoeffIntervals_ {
 };
 
 #define AR_MODEL(c) (c == AR || \
+		     c == AR1 || \
                      c == ARMA || \
-                     c == CORC || \
-                     c == GARCH || \
-                     c == HILU || \
-                     c == PWE)
+                     c == GARCH)
 
-#define AR1_MODEL(c) (c == CORC || \
-                      c == HILU || \
-                      c == PWE)
+#define AR1_MODEL(c) (c == AR1)
 
-#define SIMPLE_AR_MODEL(c) (c == AR || \
-                            c == CORC || \
-                            c == HILU || \
-                            c == PWE)
+#define SIMPLE_AR_MODEL(c) (c == AR || c == AR1)
 
 #define ML_ESTIMATOR(c) (c == ARMA || \
                          c == GARCH || \
@@ -74,12 +69,10 @@ struct CoeffIntervals_ {
                    c == PROBIT || \
                    c == TOBIT)
 
-#define LSQ_MODEL(c) (c == CORC || \
+#define LSQ_MODEL(c) (c == AR1 || \
                       c == HCCM || \
-                      c == HILU || \
                       c == HSK || \
                       c == OLS || \
-                      c == PWE || \
                       c == WLS)
 
 #define ASYMPTOTIC_MODEL(c) (c == ARBOND || \
@@ -99,14 +92,16 @@ struct CoeffIntervals_ {
 	                     c == CHOW || \
 	                     c == CUSUM || \
 	                     c == FCAST || \
-	                     c == FCASTERR || \
-	                     c == FIT || \
                              c == LEVERAGE || \
 	                     c == LMTEST || \
                              c == OMIT || \
 	                     c == RESTRICT || \
+	                     c == TESTUHAT || \
                              c == VIF)
 
+#define RQ_SPECIAL_MODEL(m) (m->ci == LAD && \
+                             NULL != gretl_model_get_data(m, "rq_tauvec"))
+
 typedef enum {
     GRETL_TEST_ADD,
     GRETL_TEST_ARCH,
@@ -128,6 +123,7 @@ typedef enum {
     GRETL_TEST_PANEL_BP,
     GRETL_TEST_PANEL_TIMEDUM,
     GRETL_TEST_HET_1,
+    GRETL_TEST_BP,
     GRETL_TEST_MAX
 } ModelTestType;
 
@@ -168,6 +164,9 @@ int gretl_model_set_data_with_destructor (MODEL *pmod, const char *key, void *pt
 int gretl_model_set_data (MODEL *pmod, const char *key, void *ptr, 
 			  GretlType type, size_t size);
 
+int gretl_model_set_matrix_as_data (MODEL *pmod, const char *key, 
+				    gretl_matrix *m);
+
 int gretl_model_set_list_as_data (MODEL *pmod, const char *key, int *list);
 
 int gretl_model_set_string_as_data (MODEL *pmod, const char *key, char *str);
@@ -205,6 +204,8 @@ CoeffIntervals *
 gretl_model_get_coeff_intervals (const MODEL *pmod, 
 				 const DATAINFO *pdinfo);
 
+int reset_coeff_intervals (CoeffIntervals *cf, double alpha);
+
 int gretl_model_get_depvar (const MODEL *pmod);
 
 const char *gretl_model_get_depvar_name (const MODEL *pmod,
@@ -240,6 +241,8 @@ int gretl_model_get_coeff_separator (const MODEL *pmod, const char **ps, int *pp
 
 int gretl_model_new_vcv (MODEL *pmod, int *nelem);
 
+int gretl_model_write_vcv (MODEL *pmod, const gretl_matrix *V);
+
 VMatrix *gretl_model_get_vcv (MODEL *pmod, const DATAINFO *pdinfo);
 
 int gretl_model_add_arinfo (MODEL *pmod, int nterms);
@@ -306,6 +309,8 @@ int mle_criteria (MODEL *pmod, int addk);
 
 double coeff_pval (int ci, double x, int df);
 
+int exact_fit_check (const MODEL *pmod, PRN *prn);
+
 int gretl_model_allocate_params (MODEL *pmod, int k);
 
 int gretl_model_add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo,
@@ -319,6 +324,8 @@ int gretl_model_add_panel_varnames (MODEL *pmod, const DATAINFO *pdinfo,
 
 void gretl_model_add_allocated_varnames (MODEL *pmod, char **vnames);
 
+int gretl_model_add_y_median (MODEL *pmod, const double *y);
+
 char *gretl_model_get_fitted_formula (const MODEL *pmod, int xvar,
 				      const double **Z,
 				      const DATAINFO *pdinfo);
