@@ -20,7 +20,7 @@
 /* panel estimation and dataset handling for gretl */
 
 #include "libgretl.h"
-#include "f2c.h"
+#include "gretl_f2c.h"
 #include "clapack_double.h"
 #include "gretl_model.h"
 #include "gretl_panel.h"
@@ -551,6 +551,90 @@ static void femod_regular_vcv (MODEL *pmod)
     }
 }
 
+#if 0 /* not ready yet */
+
+/* See Cameron and Trivedi, Microeconometrics, 21.3.4 */
+
+static int panel_autocorr_1 (MODEL *pmod, const panelmod_t *pan)
+{
+    double *ubar;
+    double *ctt, *css, *cst;
+    double uit, uis, rho;
+    int i, t, ti;
+    int n = 0;
+ 
+    ubar = malloc(pan->T * sizeof *ubar);
+    ctt = malloc(pan->T * sizeof *ctt);
+    css = malloc(pan->T * sizeof *css);
+    cst = malloc(pan->T * sizeof *cst);
+
+    if (ubar == NULL) {
+	return E_ALLOC;
+    }
+
+    for (t=0; t<pan->T; t++) {
+	ubar[t] = 0.0;
+	ctt[t] = css[t] = cst[t] = 0.0;
+    }
+
+    for (i=0; i<pan->nunits; i++) {
+	if (pan->unit_obs[i] > 0) {
+	    for (t=0; t<pan->T; t++) {
+		ti = panel_index(i, t);
+		uit = pmod->uhat[ti];
+		if (!na(uit)) {
+		    ubar[t] += uit;
+		}
+	    }
+	    n++;
+	}
+    }
+
+    for (t=0; t<pan->T; t++) {
+	ubar[t] /= n;
+    }
+
+    for (i=0; i<pan->nunits; i++) {
+	if (pan->unit_obs[i] > 0) {
+	    for (t=0; t<pan->T; t++) {
+		ti = panel_index(i, t);
+		uit = pmod->uhat[ti];
+		if (t > 0) {
+		    uis = pmod->uhat[ti-1];
+		} else {
+		    uis = NADBL;
+		}
+		if (!na(uit)) {
+		    ctt[t] += (uit - ubar[t]) * (uit - ubar[t]);
+		}
+		if (!na(uis)) {
+		    css[t] += (uis - ubar[t-1]) * (uis - ubar[t-1]);
+		}
+		if (!na(uis) && !na(uit)) {
+		    cst[t] += (uis - ubar[t-1]) * (uit - ubar[t-1]);
+		}		
+	    }
+	}
+    }
+
+    for (t=1; t<pan->T; t++) {
+	ctt[t] /= (n - 1);
+	css[t] /= (n - 1);
+	cst[t] /= (n - 1);
+	rho = cst[t] / sqrt(css[t] * ctt[t]);
+	fprintf(stderr, "rho(%d) = %g\n", t, rho);
+    }
+
+    free(ubar);
+    free(ctt);
+    free(cst);
+    free(css);
+    
+    return 0;
+}
+
+#endif
+
 /* Durbin-Watson statistic for the fixed effects model.  We only
    use units that have at least two time-series observations.
 */
@@ -635,25 +719,6 @@ static int hausman_allocate (panelmod_t *pan)
 
 /* printing routines for the "panel diagnostics" test */
 
-#if 0
-
-static void print_panel_coeff (const MODEL *pmod,
- 			       const char *vname,
- 			       int i, PRN *prn)
-{
-    model_coeff mc;
-
-    model_coeff_init(&mc);
-    mc.b = pmod->coeff[i];
-    mc.se = pmod->sderr[i];
-    mc.tval = mc.b / mc.se;
-    mc.pval = student_pvalue_2(mc.tval, pmod->dfd);
-    strcpy(mc.name, vname);
-    print_coeff(&mc, prn);
-}
-
-#else
-
 static void print_panel_coeff (const MODEL *pmod,
  			       const char *vname,
  			       int i, PRN *prn)
@@ -663,13 +728,11 @@ static void print_panel_coeff (const MODEL *pmod,
     char pvstr[18];
  
     sprintf(errstr, "(%.5g)", pmod->sderr[i]);
-    sprintf(pvstr, "[%.5f]", student_pvalue_2(tstat, pmod->dfd));
+    sprintf(pvstr, "[%.5f]", student_pvalue_2(pmod->dfd, tstat));
     pprintf(prn, "%*s: %14.5g %15s %15s\n", VNAMELEN, vname,
  	    pmod->coeff[i], errstr, pvstr);
 }
 
-#endif
-
 static void print_re_model_top (const panelmod_t *pan, PRN *prn)
 {
     pputs(prn, "Variance estimators:\n");
@@ -719,7 +782,6 @@ within_groups_dataset (const double **Z, const DATAINFO *pdinfo,
 		       double ***wZ, panelmod_t *pan)
 {
     DATAINFO *winfo = NULL;
-    double *xbar = NULL;
     int *vlist = NULL;
     int i, j, vj;
     int s, t, bigt;
@@ -744,12 +806,7 @@ within_groups_dataset (const double **Z, const DATAINFO *pdinfo,
 
     vlist = real_varying_list(pan);
     if (vlist == NULL) {
-	goto bailout;
-    }
-
-    xbar = malloc(vlist[0] * sizeof *xbar);
-    if (xbar == NULL) {
-	goto bailout;
+	return NULL;
     }
 
 #if PDEBUG
@@ -757,61 +814,62 @@ within_groups_dataset (const double **Z, const DATAINFO *pdinfo,
 	    pan->vlist[0], pan->NT);
 #endif
 
-    winfo = create_new_dataset(wZ, pan->vlist[0], pan->NT, 0);
+    winfo = create_auxiliary_dataset(wZ, pan->vlist[0], pan->NT);
     if (winfo == NULL) {
-	goto bailout;
+	free(vlist);
+	return NULL;
     }
 
-    s = 0;
+    for (j=1; j<=vlist[0]; j++) {
+	double xbar, gxbar = 0.0;
 
-    for (i=0; i<pan->nunits; i++) { 
-	int Ti = pan->unit_obs[i];
-	int got = 0;
+	vj = vlist[j];
+	gxbar = 0.0;
+	s = 0;
 
-	if (Ti == 0) {
-	    continue;
-	}
+	for (i=0; i<pan->nunits; i++) {
+	    int Ti = pan->unit_obs[i];
+	    int got = 0;
 
-	/* first pass: find the group means */
-	for (j=0; j<vlist[0]; j++) {
-	    xbar[j] = 0.0;
-	}
-	for (t=0; t<pan->T; t++) {
-	    bigt = panel_index(i, t);
-	    if (!panel_missing(pan, bigt)) {
-		for (j=0; j<vlist[0]; j++) {
-		    vj = vlist[j+1];
-		    xbar[j] += Z[vj][bigt];
+	    if (Ti == 0) {
+		continue;
+	    }
+
+	    /* first pass: find the group mean */
+	    xbar = 0.0;
+	    for (t=0; t<pan->T; t++) {
+		bigt = panel_index(i, t);
+		if (!panel_missing(pan, bigt)) {
+		    xbar += Z[vj][bigt];
+		}
+	    }
+
+	    gxbar += xbar;
+	    xbar /= Ti;
+	    
+	    /* second pass: calculate de-meaned values */
+	    got = 0;
+	    for (t=0; t<pan->T && got<Ti; t++) { 
+		bigt = panel_index(i, t);
+		if (!panel_missing(pan, bigt)) {
+		    (*wZ)[j][s] = Z[vj][bigt] - xbar;
+		    got++;
+		    if (pan->small2big != NULL) {
+			pan->small2big[s] = bigt;
+			pan->big2small[bigt] = s;
+		    }
+		    s++;
 		}
 	    }
 	}
-	for (j=0; j<vlist[0]; j++) {
-	    xbar[j] /= Ti;
-	}	
 
-	/* second pass: calculate de-meaned values */
-	for (t=0; t<pan->T && got<Ti; t++) { 
-	    bigt = panel_index(i, t);
-	    if (!panel_missing(pan, bigt)) {
-		for (j=0; j<vlist[0]; j++) {
-		    vj = vlist[j+1];
-		    (*wZ)[j+1][s] = Z[vj][bigt] - xbar[j];
-		}
-		got++;
-		if (pan->small2big != NULL) {
-		    pan->small2big[s] = bigt;
-		    pan->big2small[bigt] = s;
-		}
-		s++;
-	    }
-	}	    
+	/* wZ = data - group mean + grand mean */
+	gxbar /= pan->NT;
+	for (s=0; s<pan->NT; s++) {
+	    (*wZ)[j][s] += gxbar;
+	}
     }
 
- bailout:
-
-    free(vlist);
-    free(xbar);
-
     return winfo;
 }
 
@@ -860,7 +918,7 @@ random_effects_dataset (const double **Z, const DATAINFO *pdinfo,
 	    v1 + v2, pan->NT);
 #endif
 
-    reinfo = create_new_dataset(reZ, v1 + v2, pan->NT, 0);
+    reinfo = create_auxiliary_dataset(reZ, v1 + v2, pan->NT);
     if (reinfo == NULL) {
 	return NULL;
     }
@@ -940,6 +998,8 @@ random_effects_dataset (const double **Z, const DATAINFO *pdinfo,
     return reinfo;
 }
 
+#define BETWEEN_DEBUG 0
+
 /* Construct a mini-dataset containing the group means, in
    order to run the group-means or "between" regression.
 */
@@ -965,7 +1025,7 @@ group_means_dataset (panelmod_t *pan,
 	    gv, gn);
 #endif
 
-    ginfo = create_new_dataset(gZ, gv, gn, 0);
+    ginfo = create_auxiliary_dataset(gZ, gv, gn);
     if (ginfo == NULL) {
 	return NULL;
     }
@@ -977,11 +1037,15 @@ group_means_dataset (panelmod_t *pan,
 	if (vj == 0) {
 	    continue;
 	}
-	
+
+#if BETWEEN_DEBUG
+	strcpy(ginfo->varname[k], pdinfo->varname[vj]);
+#else
 	if (pan->opt & OPT_B) {
 	    /* will save the "between" model: so name the variables */
 	    strcpy(ginfo->varname[k], pdinfo->varname[vj]);
 	}
+#endif
 
 	s = 0;
 	for (i=0; i<pan->nunits; i++) { 
@@ -998,34 +1062,54 @@ group_means_dataset (panelmod_t *pan,
 		    x += Z[vj][bigt];
 		}
 	    }
-	    (*gZ)[k][s] = x / Ti;
-	    s++;
+	    (*gZ)[k][s++] = x / Ti;
 	}
 	k++;
     }
 
+#if BETWEEN_DEBUG
+    if (1) {
+	PRN *prn = gretl_print_new(GRETL_PRINT_STDERR, NULL);
+
+	printdata(NULL, NULL, (const double **) *gZ, ginfo, OPT_O, prn);
+	gretl_print_destroy(prn);
+    }
+#endif
+
     return ginfo;
 }
 
 /* spruce up the between model and attach it to pan */
 
-static int save_between_model (MODEL *pmod, DATAINFO *ginfo, 
-			       panelmod_t *pan)
+static int save_between_model (MODEL *pmod, const int *blist,
+			       DATAINFO *ginfo, panelmod_t *pan)
 {
-    int i, err = 0;
+    int *droplist;
+    int i, j, dpos;
+    int err = 0;
 
     pmod->ci = PANEL;
     gretl_model_set_int(pmod, "between", 1);
-    set_model_id(pmod);
     pmod->dw = NADBL;
 
     gretl_model_add_panel_varnames(pmod, ginfo, NULL);
 
+    droplist = gretl_model_get_data(pmod, "droplist");
+
+    /* replace both the model's regression list and its list of
+       dropped variables, if any, with the ID numbers of the
+       corresponding variables in the main dataset 
+    */
+    j = 1;
     for (i=1; i<=pmod->list[0]; i++) {
-	pmod->list[i] = pan->pooled->list[i];
+	if (i > 1 && droplist != NULL) {
+	    while ((dpos = in_gretl_list(droplist, blist[j]))) {
+		droplist[dpos] = pan->pooled->list[j];
+		j++;
+	    }
+	}
+	pmod->list[i] = pan->pooled->list[j++];
     }
-    
-    /* FIXME handle droplist? */
 
     *pan->realmod = *pmod;
 
@@ -1038,10 +1122,10 @@ static int save_between_model (MODEL *pmod, DATAINFO *ginfo,
 static int
 between_variance (panelmod_t *pan, double ***gZ, DATAINFO *ginfo)
 {
-    MODEL bmod;
     gretlopt bopt;
+    MODEL bmod;
     int *blist;
-    int i, j;
+    int i, j, k;
     int err = 0;
 
     blist = gretl_list_new(ginfo->v);
@@ -1049,14 +1133,16 @@ between_variance (panelmod_t *pan, double ***gZ, DATAINFO *ginfo)
 	return E_ALLOC;
     }
 
-    j = 1;
-    for (i=1; i<=blist[0]; i++) { 
-	if (pan->pooled->list[i] != 0) {
-	    blist[i] = j++;
-	}
+    j = k = 1;
+    for (i=1; i<=ginfo->v; i++) { 
+	if (pan->pooled->list[i] == 0) {
+	    blist[k++] = 0;
+	} else {
+	    blist[k++] = j++;
+	} 
     }
 
-    bopt = (pan->opt & OPT_D)? OPT_A : OPT_A | OPT_Z;
+    bopt = (pan->opt & OPT_B)? OPT_NONE : OPT_A;
     bmod = lsq(blist, gZ, ginfo, OLS, bopt);
 
     if (bmod.errcode == 0) {
@@ -1068,8 +1154,8 @@ between_variance (panelmod_t *pan, double ***gZ, DATAINFO *ginfo)
 #endif
     }
 
-    if (pan->opt & OPT_B) {
-	err = save_between_model(&bmod, ginfo, pan);
+    if (!err && (pan->opt & OPT_B)) {
+	err = save_between_model(&bmod, blist, ginfo, pan);
     } else {
 	clear_model(&bmod);
     }
@@ -1193,7 +1279,7 @@ static void panel_df_correction (MODEL *pmod, int k)
     int i;
 
     pmod->dfd -= k;
-    pmod->dfn += k - 1; 
+    pmod->dfn += k;
 
     pmod->sigma *= dfcorr;
 
@@ -1222,7 +1308,7 @@ static int print_fe_results (panelmod_t *pan,
 
     /* print the slope coefficients, for varying regressors */
     for (i=0; i<pmod->ncoeff; i++) {
-	int vi = pan->vlist[i+3];
+	int vi = pan->vlist[i+2];
 
 	print_panel_coeff(pmod, pdinfo->varname[vi], i, prn);
     }
@@ -1237,7 +1323,7 @@ static int print_fe_results (panelmod_t *pan,
 
     pprintf(prn, _("Joint significance of differing group means:\n"));
     pprintf(prn, " F(%d, %d) = %g %s %g\n", pan->Fdfn, pan->Fdfd, pan->F, 
-	    _("with p-value"), snedecor_cdf_comp(pan->F, pan->Fdfn, pan->Fdfd));
+	    _("with p-value"), snedecor_cdf_comp(pan->Fdfn, pan->Fdfd, pan->F));
 
     pputs(prn, _("(A low p-value counts against the null hypothesis that "
 		 "the pooled OLS model\nis adequate, in favor of the fixed "
@@ -1308,7 +1394,7 @@ static int time_dummies_wald_test (panelmod_t *pan, MODEL *wmod)
 	    model_test_set_teststat(test, GRETL_STAT_WALD_CHISQ);
 	    model_test_set_dfn(test, k);
 	    model_test_set_value(test, x);
-	    model_test_set_pvalue(test, chisq_cdf_comp(x, k));
+	    model_test_set_pvalue(test, chisq_cdf_comp(k, x));
 	    maybe_add_test_to_model(wmod, test);
 	}
     }		
@@ -1330,7 +1416,7 @@ static void save_fixed_effects_F (panelmod_t *pan, MODEL *wmod)
 	model_test_set_dfn(test, pan->Fdfn);
 	model_test_set_dfd(test, pan->Fdfd);
 	model_test_set_value(test, pan->F);
-	model_test_set_pvalue(test, snedecor_cdf_comp(pan->F, pan->Fdfn, pan->Fdfd));
+	model_test_set_pvalue(test, snedecor_cdf_comp(pan->Fdfn, pan->Fdfd, pan->F));
 	maybe_add_test_to_model(wmod, test);
     }	    
 }
@@ -1368,8 +1454,6 @@ fix_panelmod_list (MODEL *targ, panelmod_t *pan)
 		gretl_list_delete_at_pos(targ->list, i--);
 	    }
 	}
-	/* and remove the const */
-	gretl_list_delete_at_pos(targ->list, 2);
     }
 
 #if PDEBUG
@@ -1399,6 +1483,11 @@ static int fix_within_stats (MODEL *fmod, panelmod_t *pan)
     fmod->tss = pan->pooled->tss;
     fmod->ifc = 1;
 
+    gretl_model_set_double(fmod, "rsq_within", fmod->rsq);
+    fmod->fstt = (fmod->rsq / (1.0 - fmod->rsq)) * 
+	((double) fmod->dfd / (fmod->ncoeff - 1));
+    gretl_model_set_double(fmod, "F_variables", fmod->fstt); /* ?? */
+
     /* should we modify R^2 in this way? */
     fmod->rsq = 1.0 - (fmod->ess / fmod->tss);
 
@@ -1415,7 +1504,7 @@ static int fix_within_stats (MODEL *fmod, panelmod_t *pan)
 	    ((double) fmod->dfd / fmod->dfn);
     }
 
-    fmod->ncoeff = fmod->dfn + 1;
+    fmod->ncoeff = fmod->dfn + 1; /* number of params estimated */
     ls_criteria(fmod);
     fmod->ncoeff = nc;
 
@@ -1460,11 +1549,12 @@ static int fe_model_add_ahat (MODEL *pmod, const double **Z,
 	    bigt = panel_index(i, t);
 	    if (!na(pmod->uhat[bigt])) {
 		ahi += Z[pmod->list[1]][bigt];
-		for (j=0; j<pmod->ncoeff; j++) {
+		for (j=1; j<pmod->ncoeff; j++) {
 		    ahi -= pmod->coeff[j] * Z[pmod->list[j+2]][bigt];
 		}
 	    }
 	}
+
 	ahi /= Ti;
 
 	for (t=0; t<pan->T; t++) {
@@ -1572,7 +1662,7 @@ fix_panel_hatvars (MODEL *pmod, const DATAINFO *dinfo,
 
 #if PDEBUG > 1
 
-static void verbose_femod_print (MODEL *femod, const double *wZ,
+static void verbose_femod_print (MODEL *femod, double **wZ,
 				 DATAINFO *winfo, PRN *prn)
 {
     int i, j;
@@ -1613,7 +1703,7 @@ fixed_effects_model (panelmod_t *pan, const double **Z,
 
     gretl_model_init(&femod);
 
-    felist = gretl_list_new(pan->vlist[0] - 1);
+    felist = gretl_list_new(pan->vlist[0]); 
     if (felist == NULL) {
 	femod.errcode = E_ALLOC;
 	return femod;
@@ -1626,8 +1716,10 @@ fixed_effects_model (panelmod_t *pan, const double **Z,
 	return femod;
     } 
 
-    for (i=1; i<=felist[0]; i++) {
-	felist[i] = i;
+    felist[1] = 1;
+    felist[2] = 0;
+    for (i=3; i<=felist[0]; i++) {
+	felist[i] = i - 1;
     }
 
     femod = lsq(felist, &wZ, winfo, OLS, lsqopt);
@@ -1641,7 +1733,7 @@ fixed_effects_model (panelmod_t *pan, const double **Z,
     } else {
 	/* we estimated a bunch of group means, and have to
 	   subtract degrees of freedom */
-	panel_df_correction(&femod, pan->N_fe);
+	panel_df_correction(&femod, pan->N_fe - 1);
 #if PDEBUG > 1
 	verbose_femod_print(&femod, wZ, winfo, prn);
 #endif
@@ -1781,6 +1873,9 @@ static int save_fixed_effects_model (MODEL *pmod, panelmod_t *pan,
     fe_model_add_ahat(pmod, Z, pdinfo, pan);
     set_model_id(pmod);
     panel_dwstat(pmod, pan);
+#if 0
+    panel_autocorr_1(pmod, pan);
+#endif
     save_fixed_effects_F(pan, pmod);
     time_dummies_wald_test(pan, pmod);
 
@@ -1826,8 +1921,8 @@ static int within_variance (panelmod_t *pan,
 	}
 
 	if (pan->bdiff != NULL && pan->sigma != NULL) {
-	    for (i=0; i<femod.ncoeff; i++) {
-		pan->bdiff[i] = femod.coeff[i];
+	    for (i=1; i<femod.ncoeff; i++) {
+		pan->bdiff[i-1] = femod.coeff[i];
 	    }
 	    femod_regular_vcv(&femod);
 	    pan->sigma_e = femod.sigma;
@@ -1910,7 +2005,7 @@ static void print_hausman_result (panelmod_t *pan, PRN *prn)
 	pprintf(prn, _("\nHausman test statistic:\n"
 		       " H = %g with p-value = prob(chi-square(%d) > %g) = %g\n"),
 		pan->H, pan->nbeta, pan->H, 
-		chisq_cdf_comp(pan->H, pan->nbeta));
+		chisq_cdf_comp(pan->nbeta, pan->H));
 	pputs(prn, _("(A low p-value counts against the null hypothesis that "
 		     "the random effects\nmodel is consistent, in favor of the fixed "
 		     "effects model.)\n"));
@@ -1931,7 +2026,7 @@ static void save_hausman_result (panelmod_t *pan)
 	model_test_set_teststat(test, GRETL_STAT_WALD_CHISQ);
 	model_test_set_dfn(test, pan->nbeta);
 	model_test_set_value(test, pan->H);
-	model_test_set_pvalue(test, chisq_cdf_comp(pan->H, pan->nbeta));
+	model_test_set_pvalue(test, chisq_cdf_comp(pan->nbeta, pan->H));
 	maybe_add_test_to_model(pan->realmod, test);
     }	    
 }
@@ -2001,7 +2096,7 @@ static int random_effects (panelmod_t *pan,
 	int *biglist = gretl_list_add(relist, hlist, &err);
 
 	if (!err) {
-	    remod = lsq(biglist, &reZ, reinfo, OLS, lsqopt);
+	    remod = lsq(biglist, &reZ, reinfo, OLS, OPT_A);
 	    if (remod.errcode == 0) {
 		URSS = remod.ess;
 	    }
@@ -2076,7 +2171,7 @@ static void save_breusch_pagan_result (panelmod_t *pan)
 	model_test_set_teststat(test, GRETL_STAT_WALD_CHISQ);
 	model_test_set_dfn(test, 1);
 	model_test_set_value(test, pan->BP);
-	model_test_set_pvalue(test, chisq_cdf_comp(pan->BP, 1));
+	model_test_set_pvalue(test, chisq_cdf_comp(1, pan->BP));
 	maybe_add_test_to_model(pan->realmod, test);
     }	    
 }
@@ -2127,7 +2222,7 @@ breusch_pagan_LM (panelmod_t *pan, const DATAINFO *pdinfo, PRN *prn)
     if (pan->opt & OPT_V) {
 	pprintf(prn, _("\nBreusch-Pagan test statistic:\n"
 		       " LM = %g with p-value = prob(chi-square(1) > %g) = %g\n"), 
-		pan->BP, pan->BP, chisq_cdf_comp(pan->BP, 1));
+		pan->BP, pan->BP, chisq_cdf_comp(1, pan->BP));
 
 	pputs(prn, _("(A low p-value counts against the null hypothesis that "
 		     "the pooled OLS model\nis adequate, in favor of the random "
@@ -2398,6 +2493,10 @@ int panel_diagnostics (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 
 	if (err) { 
 	    pputs(prn, _("Couldn't estimate group means regression\n"));
+	    if (err == E_SINGULAR) {
+		/* treat this as non-fatal */
+		err = 0;
+	    }
 	} else {
 	    random_effects(&pan, (const double **) *pZ, pdinfo, 
 			   (const double **) gZ, ginfo, prn);
@@ -2513,7 +2612,9 @@ static void save_pooled_model (MODEL *pmod, panelmod_t *pan,
     }
 }
 
-#define estimator_specified(o) (o & (OPT_U|OPT_B|OPT_P))
+/* fixed effects | random effects | between | pooled */
+
+#define estimator_specified(o) (o & (OPT_F|OPT_U|OPT_B|OPT_P))
 
 /* real_panel_model:
  * @list: list containing model specification.
@@ -2779,7 +2880,7 @@ print_wald_test (double W, int nunits, const int *unit_obs, PRN *prn)
 	    _("Distribution free Wald test for heteroskedasticity"),
 	    _("based on the FGLS residuals"));
     pprintf(prn, "%s(%d) = %g, ",  _("Chi-square"), df, W);
-    pprintf(prn, _("with p-value = %g\n\n"), chisq_cdf_comp(W, df));
+    pprintf(prn, _("with p-value = %g\n\n"), chisq_cdf_comp(df, W));
 }
 
 /* Wald test for groupwise heteroskedasticity, without assuming
@@ -2860,7 +2961,7 @@ ml_hetero_test (MODEL *pmod, double s2, const double *uvar,
 	model_test_set_teststat(test, GRETL_STAT_LR);
 	model_test_set_dfn(test, df);
 	model_test_set_value(test, x2);
-	model_test_set_pvalue(test, chisq_cdf_comp(x2, df));
+	model_test_set_pvalue(test, chisq_cdf_comp(df, x2));
 	maybe_add_test_to_model(pmod, test);
     } else {
 	err = 1;
@@ -3220,8 +3321,10 @@ int panel_autocorr_test (MODEL *pmod, int order,
     nv = pmod->list[0] + order;
 
     /* create temporary reduced dataset */
-    tmpinfo = create_new_dataset(&tmpZ, nv, nobs, 0);
-    if (tmpinfo == NULL) return E_ALLOC;
+    tmpinfo = create_auxiliary_dataset(&tmpZ, nv, nobs);
+    if (tmpinfo == NULL) {
+	return E_ALLOC;
+    }
 
     make_reduced_data_info(tmpinfo, pdinfo, order);
 
@@ -3288,8 +3391,9 @@ int panel_autocorr_test (MODEL *pmod, int order,
 	gretl_model_set_int(&aux, "BG_order", order);
 	printmodel(&aux, tmpinfo, OPT_NONE, prn);
 	trsq = aux.rsq * aux.nobs;
+	/* FIXME LMF */
 	LMF = (aux.rsq / (1.0 - aux.rsq)) * dfd / order; 
-	pval = snedecor_cdf_comp(LMF, order, dfd);
+	pval = snedecor_cdf_comp(order, dfd, LMF);
 
 	pprintf(prn, "\n%s: LMF = %f,\n", _("Test statistic"), LMF);
 	pprintf(prn, "%s = P(F(%d,%d) > %g) = %.3g\n", _("with p-value"), 
@@ -3298,7 +3402,7 @@ int panel_autocorr_test (MODEL *pmod, int order,
 	pprintf(prn, "\n%s: TR^2 = %f,\n", 
 		_("Alternative statistic"), trsq);
 	pprintf(prn, "%s = P(%s(%d) > %g) = %.3g\n\n", 	_("with p-value"), 
-		_("Chi-square"), order, trsq, chisq_cdf_comp(trsq, order));
+		_("Chi-square"), order, trsq, chisq_cdf_comp(order, trsq));
 
 	if (opt & OPT_S) {
 	    ModelTest *test = model_test_new(GRETL_TEST_AUTOCORR);
@@ -3342,16 +3446,15 @@ int panel_autocorr_test (MODEL *pmod, int order,
 
 int switch_panel_orientation (double **Z, DATAINFO *pdinfo)
 {
-    int sts = (pdinfo->structure == STACKED_TIME_SERIES);
+    int oldmode = pdinfo->structure;
     double *tmp;
     char **markers = NULL;
     double pdx;
-    int pd = pdinfo->pd;
-    int nblocks = pdinfo->n / pd;
-    int i, j, t;
+    int T, n;
+    int i, j, s, t;
 
-    if (pdinfo->structure != STACKED_TIME_SERIES &&
-	pdinfo->structure != STACKED_CROSS_SECTION) {
+    if (oldmode != STACKED_TIME_SERIES &&
+	oldmode != STACKED_CROSS_SECTION) {
 	return E_DATA;
     }
 
@@ -3360,19 +3463,39 @@ int switch_panel_orientation (double **Z, DATAINFO *pdinfo)
 	return E_ALLOC;
     }
 
+    if (oldmode == STACKED_CROSS_SECTION) {
+	n = pdinfo->pd;
+	T = pdinfo->n / n;
+    } else {
+	T = pdinfo->pd;
+	n = pdinfo->n / T;
+    }
+
     /* copy the data series across in transformed order */
     for (i=1; i<pdinfo->v; i++) {
 	if (var_is_scalar(pdinfo, i)) {
 	    continue;
 	}
 	for (t=0; t<pdinfo->n; t++) {
+	    /* transcribe to tmp in original order */
 	    tmp[t] = Z[i][t];
 	}
-	for (j=0; j<pd; j++) {
-	    for (t=0; t<nblocks; t++) {
-		Z[i][j * nblocks + t] = tmp[j + pd * t];
+	s = 0;
+	if (oldmode == STACKED_CROSS_SECTION) {
+	    /* convert to stacked time-series */
+	    for (j=0; j<n; j++) {
+		for (t=0; t<T; t++) {
+		    Z[i][s++] = tmp[t * n + j];
+		}
 	    }
-	}
+	} else {
+	    /* convert to stacked cross-sections */
+	    for (t=0; t<T; t++) {
+		for (j=0; j<n; j++) {
+		    Z[i][s++] = tmp[j * T + t];
+		}
+	    } 
+	}  
     }
 
     /* rearrange observations markers if relevant */
@@ -3382,9 +3505,18 @@ int switch_panel_orientation (double **Z, DATAINFO *pdinfo)
 	    for (t=0; t<pdinfo->n; t++) {
 		strcpy(markers[t], pdinfo->S[t]);
 	    }
-	    for (j=0; j<pd; j++) {
-		for (t=0; t<nblocks; t++) {
-		    strcpy(pdinfo->S[j * nblocks + t], markers[j + pd * t]);
+	    s = 0;
+	    if (oldmode == STACKED_CROSS_SECTION) {
+		for (j=0; j<n; j++) {
+		    for (t=0; t<T; t++) {
+			strcpy(pdinfo->S[s++], markers[t * n + j]);
+		    }
+		}
+	    } else {
+		for (t=0; t<T; t++) {
+		    for (j=0; j<n; j++) {
+			strcpy(pdinfo->S[s++], markers[j * T + t]);
+		    }
 		}
 	    }
 	    free_strings_array(markers, pdinfo->n);
@@ -3394,17 +3526,25 @@ int switch_panel_orientation (double **Z, DATAINFO *pdinfo)
 	}
     }
 
-    /* change the datainfo setup */
-    pdinfo->structure = (sts)? STACKED_CROSS_SECTION : STACKED_TIME_SERIES;
-    pdinfo->pd = nblocks;
-
     pdinfo->sd0 = 1.0;
     pdx = 0.1;
-    while (nblocks /= 10) {
-	pdx *= 0.1;
-    }
-    pdinfo->sd0 += pdx;
 
+    /* change the datainfo setup */
+    if (oldmode == STACKED_CROSS_SECTION) {
+	pdinfo->structure = STACKED_TIME_SERIES;
+	pdinfo->pd = T;
+	while (T /= 10) {
+	    pdx *= 0.1;
+	}	
+    } else {
+	pdinfo->structure = STACKED_CROSS_SECTION;
+	pdinfo->pd = n;
+	while (n /= 10) {
+	    pdx *= 0.1;
+	}
+    }
+	
+    pdinfo->sd0 += pdx;
     ntodate(pdinfo->stobs, 0, pdinfo);
     ntodate(pdinfo->endobs, pdinfo->n - 1, pdinfo);
 
@@ -3565,7 +3705,7 @@ static int compose_panel_indices (DATAINFO *pdinfo, int T,
 }
 
 static int panel_data_sort_by (double **Z, DATAINFO *pdinfo,
-			       int uv, int tv)
+			       int uv, int tv, int *ustrs)
 {
     int n = pdinfo->n;
     char **S = NULL;
@@ -3602,21 +3742,33 @@ static int panel_data_sort_by (double **Z, DATAINFO *pdinfo,
 	  compare_obs);
 
     for (i=1; i<pdinfo->v; i++) {
-	for (t=0; t<n; t++) {
-	    tmp[t] = Z[i][t];
-	}
-	for (t=0; t<n; t++) {
-	    Z[i][t] = tmp[s.points[t].obsnum];
+	if (var_is_series(pdinfo, i)) {
+	    for (t=0; t<n; t++) {
+		tmp[t] = Z[i][t];
+	    }
+	    for (t=0; t<n; t++) {
+		Z[i][t] = tmp[s.points[t].obsnum];
+	    }
 	}	
     }
 
     if (S != NULL) {
+	*ustrs = 1;
 	for (t=0; t<n; t++) {
 	    strcpy(S[t], pdinfo->S[t]);
+	    if (S[t][0] == '\0') {
+		*ustrs = 0;
+	    }
 	}
 	for (t=0; t<n; t++) {
 	    strcpy(pdinfo->S[t], S[s.points[t].obsnum]);
 	}
+	for (t=1; t<n && *ustrs; t++) {
+	    if (Z[uv][t] == Z[uv][t-1] &&
+		strcmp(pdinfo->S[t], pdinfo->S[t-1])) {
+		*ustrs = 0;
+	    }
+	}
 	free_strings_array(S, n);
     }	
 
@@ -3675,9 +3827,10 @@ static int normalize_uid_tid (const double *tid, int T,
 static int pad_panel_dataset (const double *uid, int uv, int nunits,
 			      const double *tid, int tv, int nperiods, 
 			      double **Z, DATAINFO *pdinfo,
-			      char *mask)
+			      int ustrs, char *mask)
 {
     double **bigZ = NULL;
+    char **S = NULL;
     int *nuid = NULL;
     int *ntid = NULL;
     int n_scalars = 0;
@@ -3704,6 +3857,10 @@ static int pad_panel_dataset (const double *uid, int uv, int nunits,
 	pdinfo->v += n_scalars;
     }
 
+    if (!err && pdinfo->S != NULL && ustrs) {
+	S = strings_array_new_with_length(pdinfo->n, OBSLEN);
+    }
+
     if (err) {
 	pdinfo->n = n_orig;
 	pdinfo->t2 = t2_orig;
@@ -3720,6 +3877,9 @@ static int pad_panel_dataset (const double *uid, int uv, int nunits,
 		    bigZ[j++][s] = Z[i][t];
 		}
 	    }
+	    if (S != NULL) {
+		strcpy(S[s], pdinfo->S[t]);
+	    }
 	    if (mask != NULL && s > tref) {
 		/* recording the padding in "mask" */
 		for (j=tref; j<s; j++) {
@@ -3768,20 +3928,48 @@ static int pad_panel_dataset (const double *uid, int uv, int nunits,
     free(nuid);
     free(ntid);
 
+    if (pdinfo->S != NULL) {
+	/* expand the obs (unit) marker strings appropriately */
+	if (S == NULL) {
+	    dataset_destroy_obs_markers(pdinfo);
+	} else {
+	    char si[OBSLEN];
+
+	    for (i=0; i<nunits; i++) {
+		t = i * nperiods;
+		for (j=0; j<nperiods; j++) {
+		    if (S[t][0] != '\0') {
+			strcpy(si, S[t]);
+			break;
+		    }
+		    t++;
+		}
+		t = i * nperiods;
+		for (j=0; j<nperiods; j++) {
+		    strcpy(S[t++], si);
+		}
+	    }
+	    free_strings_array(pdinfo->S, n_orig);
+	    pdinfo->S = S;
+	}
+    }
+
     return err;
 }
 
-static int non_negative (const double *x, int n)
+static int check_index_values (const double *x, int n)
 {
     int i;
 
     for (i=1; i<n; i++) {
 	if (x[i] < 0) {
-	    return 0;
+	    return E_DATA;
+	} else if (na(x[i])) {
+	    return E_MISSDATA;
 	}
     }    
 
-    return 1;
+    return 0;
 }
 
 static int uv_tv_from_line (const char *line, const DATAINFO *pdinfo,
@@ -3858,6 +4046,7 @@ int set_panel_structure_from_vars (int uv, int tv,
     int fulln = 0;
     int nunits = 0;
     int nperiods = 0;
+    int ustrs = 0;
     int err = 0;
 
     /* FIXME sub-sampled dataset (needs to be disallowed?) */
@@ -3903,7 +4092,7 @@ int set_panel_structure_from_vars (int uv, int tv,
 #endif
 
     /* sort full dataset by unit and period */
-    err = panel_data_sort_by(Z, pdinfo, uv, tv);
+    err = panel_data_sort_by(Z, pdinfo, uv, tv, &ustrs);
 
 #if PDEBUG
     print_unit_var(uv, Z, n, 1);
@@ -3916,7 +4105,8 @@ int set_panel_structure_from_vars (int uv, int tv,
 	rearrange_id_array(tid, nperiods, n);
 	err = pad_panel_dataset(uid, uv, nunits, 
 				tid, tv, nperiods, 
-				Z, pdinfo, mask);
+				Z, pdinfo, ustrs, 
+				mask);
     }
 
     if (!err) {
@@ -3939,8 +4129,8 @@ int set_panel_structure_from_line (const char *line,
 				   double **Z, 
 				   DATAINFO *pdinfo)
 {
-    int n = pdinfo->n; /* ? */
-    int uv, tv;
+    int n = pdinfo->n;
+    int uv = 0, tv = 0;
     int err = 0;
 
     if (!strncmp(line, "setobs", 6)) {
@@ -3948,31 +4138,53 @@ int set_panel_structure_from_line (const char *line,
     }
 
     err = uv_tv_from_line(line, pdinfo, &uv, &tv);
-    if (err) {
-	return err;
+
+    if (!err) {
+	err = check_index_values(Z[uv], n);
     }
 
-    if (!non_negative(Z[uv], n) || !non_negative(Z[tv], n)) {
-	return E_DATA;
+    if (!err) {
+	err = check_index_values(Z[tv], n);
     }
 
-    return set_panel_structure_from_vars(uv, tv, Z, pdinfo);
+    if (!err) {
+	err = set_panel_structure_from_vars(uv, tv, Z, pdinfo);
+    }
+
+    return err;
 }
 
+static int find_time_var (const DATAINFO *pdinfo)
+{
+    const char *tnames[] = {
+	"year",
+	"Year",
+	"period",
+	"Period",
+	NULL
+    };
+    int i, v;
+
+    for (i=0; tnames[i] != NULL; i++) {
+	v = varindex(pdinfo, tnames[i]);
+	if (v < pdinfo->v && var_is_series(pdinfo, v)) {
+	    return v;
+	}
+    }
+
+    return 0;
+}
+    
 /* utility functions */
 
 int guess_panel_structure (double **Z, DATAINFO *pdinfo)
 {
-    int ret, v = varindex(pdinfo, "year");
+    int ret, v = find_time_var(pdinfo);
 
-    if (v == pdinfo->v) {
-	v = varindex(pdinfo, "Year");
-    }
-
-    if (v == pdinfo->v) {
+    if (v == 0) {
 	ret = 0; /* can't guess */
     } else if (floateq(Z[v][0], Z[v][1])) { 
-	/* "year" is same for first two obs */
+	/* "year" or whatever is same for first two obs */
 	pdinfo->structure = STACKED_CROSS_SECTION; 
 	ret = STACKED_CROSS_SECTION;
     } else {
@@ -4182,68 +4394,6 @@ int *panel_list_add (const MODEL *orig, const int *add, int *err)
     return newlist;
 } 
 
-#if 0 /* not ready */
-
-static int panel_obs_freq (FreqDist *fr, int *x, int n)
-{
-    int *ifreq = NULL, *ivals = NULL;
-    int *sorted = NULL;
-    int i, t, last;
-    int err = 0;
-
-    sorted = malloc(n * sizeof *sorted);
-    if (sorted == NULL) {
-	return E_ALLOC;
-    }
-	
-    for (t=0; t<n; t++) {
-	sorted[t] = x[t];
-    }
-	
-    qsort(sorted, n, sizeof *sorted, gretl_compare_ints); 
-    nbins = count_distinct_int_values(sorted, n);
-
-    ifreq = malloc(nbins * sizeof *ifreq);
-    ivals = malloc(nbins * sizeof *ivals);
-    if (ifreq == NULL || ivals == NULL) {
-	err = E_ALLOC;
-	goto bailout;
-    }
-
-    ivals[0] = last = sorted[0];
-    ifreq[0] = i = 1;
-
-    for (t=1; t<n; t++) {
-	if (sorted[t] != last) {
-	    last = sorted[t];
-	    ifreq[i] = 1;
-	    ivals[i++] = last;
-	} else {
-	    ifreq[i-1] += 1;
-	}
-    }
-
-    if (freq_add_arrays(freq, nbins)) {
-	*err = E_ALLOC;
-    } else {
-	for (k=0; k<nbins; k++) {
-	    freq->endpt[k] = freq->midpt[k] = ivals[k];
-	    freq->f[k] = ifreq[k];
-	}
-	freq->endpt[nbins] = xmax;
-    }
-
- bailout:
-
-    free(sorted);
-    free(ivals);
-    free(ifreq);
-
-    return err;
-}
-
-#endif
-
 /* calculate the within and between standard deviations for a given
    variable in a panel data set */
 
