@@ -22,6 +22,7 @@
 #include "libgretl.h"
 #include "libset.h"
 #include "gretl_string_table.h"
+#include "gretl_www.h"
 
 #include <unistd.h>
 
@@ -35,15 +36,21 @@
 
 #include <glib.h>
 
-enum {
-    CURRENT_DIR,
-    DATA_SEARCH,
-    SCRIPT_SEARCH,
-    FUNCS_SEARCH,
-    USER_SEARCH
+struct INTERNAL_PATHS {
+    char dotdir[MAXLEN];
+    char workdir[MAXLEN];
+    char gnuplot[MAXLEN];
+    char plotfile[MAXLEN];
+    char libpath[MAXLEN];
+    char x12a[MAXLEN];
+    char x12adir[MAXLEN];
+    char tramo[MAXLEN];
+    char tramodir[MAXLEN];
+    char pngfont[128];
+    unsigned char status;
 };
 
-static void ensure_slash (char *str);
+static struct INTERNAL_PATHS gretl_paths;
 
 static int add_suffix (char *fname, const char *sfx)
 {
@@ -81,15 +88,30 @@ static int fname_is_utf8 (const unsigned char *s)
 
 static int fopen_use_utf8;
 
+/**
+ * set_fopen_use_utf8:
+ *
+ * Sets gretl's internal state so as to ensure that filenames
+ * are given in UTF-8 when passed to the C library's fopen().
+ */
+
 void set_fopen_use_utf8 (void)
 {
     fopen_use_utf8 = 1;
 }
 
-/* Below: try to guard against a situation where a filename
-   is UTF-8 encoded, but we should be using locale encoding
-   with the stdio functions.
-*/
+/**
+ * gretl_fopen:
+ * @fname: name of file to be opened.
+ * @mode: mode in which to open the file.
+ *
+ * A wrapper for the C library's fopen(): provides a guard
+ * against the situation where a filename is UTF-8 encoded, 
+ * but on the current platform we should be using locale
+ * encoding for the stdio functions.
+ *
+ * Returns: file pointer, or %NULL on failure.
+ */
 
 FILE *gretl_fopen (const char *fname, const char *mode)
 {
@@ -97,6 +119,8 @@ FILE *gretl_fopen (const char *fname, const char *mode)
     gsize wrote;
     FILE *fp = NULL;
 
+    errno = 0;
+
     if (mode != NULL && *mode == 'r') {
 	/* opening for reading */
 	fp = fopen(fname, mode);
@@ -124,9 +148,26 @@ FILE *gretl_fopen (const char *fname, const char *mode)
 	}
     }
 
+    if (errno != 0) {
+	gretl_errmsg_set_from_errno(fname);
+    }
+
     return fp;
 }
 
+/**
+ * gretl_gzopen:
+ * @fname: name of gzipped file to be opened.
+ * @mode: mode in which to open the file.
+ *
+ * A wrapper for zlib's gzopen(): provides a guard
+ * against the situation where a filename is UTF-8 encoded, 
+ * but on the current platform we should be using local
+ * encoding for the stdio functions.
+ *
+ * Returns: pointer to gzip stream, or %NULL on failure.
+ */
+
 gzFile gretl_gzopen (const char *fname, const char *mode)
 {
     gchar *fconv;
@@ -183,6 +224,17 @@ int gretl_mkdir (const char *path)
 
 #else
 
+/**
+ * gretl_mkdir:
+ * @path: name of directory to be created.
+ *
+ * Calls the underlying library function to create the
+ * specified directory with mode 0755.  If the directory in
+ * question already exists, this does not count as an error.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
 int gretl_mkdir (const char *path)
 {
     int err = 0;
@@ -218,6 +270,17 @@ static int gretl_isdir (const char *path)
    in the directory above the one to be deleted at the
    outset */
 
+/**
+ * gretl_deltree:
+ * @path: name of directory to be deleted.
+ *
+ * Carries out recursive deletion of the specified directory.
+ * Note: the current working directory should be set to one
+ * level above @path when this function is called. FIXME.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
 int gretl_deltree (const char *path)
 {
     const char *fname;
@@ -249,7 +312,7 @@ int gretl_deltree (const char *path)
     }
 
     if (err) {
-	gretl_errmsg_set_from_errno();
+	gretl_errmsg_set_from_errno(path);
 	err = E_FOPEN;
     }
     
@@ -258,7 +321,13 @@ int gretl_deltree (const char *path)
 
 #endif
 
-/* like access(), returns 0 on success */
+/**
+ * gretl_write_access:
+ * @fname: name of file to test.
+ *
+ * Returns: 0 on success (meaning that the current user has
+ * write access to @fname), non-zero on failure.
+ */
 
 int gretl_write_access (char *fname)
 {
@@ -277,6 +346,14 @@ int gretl_write_access (char *fname)
     return err;
 }
 
+/**
+ * gretl_is_xml_file:
+ * @fname: name of file to test.
+ *
+ * Returns: 1 if @fname appears to be a (possibly gzipped) XML file,
+ * otherwise 0.
+ */
+
 int gretl_is_xml_file (const char *fname)
 {
     gzFile fz;
@@ -295,21 +372,34 @@ int gretl_is_xml_file (const char *fname)
     return ret;
 } 
 
+/**
+ * gretl_path_prepend:
+ * @file: target filename.
+ * @path: path to prepend.
+ *
+ * Creates a path string by prepending @path, plus an appropriate
+ * separator if needed, to @file.  The result is written back into
+ * @file: this variable is assumed to have storage for at least
+ * #MAXLEN characters.
+ *
+ * Returns: 0 on success, or 1 if the final path string would
+ * exceed #MAXLEN characters (including nul-termination).
+ */
+
 int gretl_path_prepend (char *file, const char *path)
 {
     char temp[MAXLEN];
-    int n, pathlen = strlen(file) + strlen(path) + 1;
+    int n = strlen(file) + strlen(path) + 1;
 
-    if (pathlen > MAXLEN) {
+    if (n > MAXLEN) {
 	return 1;
     }
 
     strcpy(temp, path);
     n = strlen(temp);
 
-    if (temp[n - 1] != SLASH && n < MAXLEN - 1) {
-	temp[n] = SLASH;
-	temp[n + 1] = '\0';
+    if (temp[n-1] != SLASH && n < MAXLEN - 1) {
+	strcat(temp, SLASHSTR);
     }
 
     strcat(temp, file);
@@ -543,6 +633,22 @@ static int dotpath (const char *fname)
     return 0;
 }
 
+static char *fname_strstr (char *fname, char *dname)
+{
+#ifdef WIN32
+    char lfname[MAXLEN], ldname[MAXLEN];
+
+    *lfname = *ldname = '\0';
+    strncat(lfname, fname, MAXLEN - 1);
+    strncat(ldname, dname, MAXLEN - 1);
+    lower(lfname);
+    lower(ldname);
+    return strstr(lfname, ldname);
+#else
+    return strstr(fname, dname);
+#endif
+}
+
 /* note: for our purposes we count filenames beginning with "./" or
    "../" as absolute 
 */
@@ -552,22 +658,27 @@ int gretl_path_is_absolute (const char *fname)
     return g_path_is_absolute(fname) || dotpath(fname);
 }
 
+static void real_make_path_absolute (char *targ, const char *src,
+				     const char *dirname)
+{
+    int offset = 0;
+
+    strcpy(targ, dirname);
+    trim_slash(targ);
+    strcat(targ, SLASHSTR);
+    if (*src == '.' && src[1] == SLASH && strlen(src) > 2) {
+	offset = 2;
+    }
+    strcat(targ, src + offset);
+}
+
 static void make_path_absolute (char *fname, const char *orig)
 {
     char thisdir[MAXLEN];
-    int offset = 0;
 
-    if (getcwd(thisdir, MAXLEN-1) != NULL) {
-#ifdef WIN32		
-	lower(thisdir); /* hmmm */
-#endif
-	if (strstr(fname, thisdir) == NULL) {
-	    strcpy(fname, thisdir);
-	    strcat(fname, SLASHSTR);
-	    if (*orig == '.' && orig[1] == SLASH && strlen(orig) > 2) {
-		offset = 2;
-	    }
-	    strcat(fname, orig + offset);
+    if (getcwd(thisdir, MAXLEN - 1) != NULL) {
+	if (fname_strstr(fname, thisdir) == NULL) {
+	    real_make_path_absolute(fname, orig, thisdir);
 	}
     }
 }
@@ -584,7 +695,7 @@ static int shelldir_open_dotfile (char *fname, char *orig)
     int ret = 0;
 
     if (sdir != NULL && *sdir != '\0') {
-	sprintf(fname, "%s%c%s", sdir, SLASH, orig);
+	real_make_path_absolute(fname, orig, sdir);
 	test = gretl_fopen(fname, "r");
 	if (test != NULL) {
 	    fclose(test);
@@ -620,7 +731,7 @@ char *addpath (char *fname, PATHS *ppaths, int script)
 
     if (dotpath(fname) && shelldir_open_dotfile(fname, orig)) {
 	return fname;
-    }
+    }  
 
     /* try opening filename as given */
     test = gretl_fopen(fname, "r");
@@ -636,6 +747,8 @@ char *addpath (char *fname, PATHS *ppaths, int script)
     }
 
     if (ppaths != NULL) {
+	char trydir[MAXLEN];
+
 	/* try looking where script was found */
 	if (*ppaths->currdir != '\0') {
 	    if ((fname = search_dir(fname, ppaths->currdir, CURRENT_DIR))) {
@@ -647,22 +760,21 @@ char *addpath (char *fname, PATHS *ppaths, int script)
 	strcpy(fname, orig);
 
 	if (script) {
-	    /* for a script, try system script dir (and subdirs) */
-	    if ((fname = search_dir(fname, ppaths->scriptdir, SCRIPT_SEARCH))) { 
+	    sprintf(trydir, "%sscripts", ppaths->gretldir);
+	    if ((fname = search_dir(fname, trydir, SCRIPT_SEARCH))) { 
 		return fname;
 	    } else {
-		char fndir[MAXLEN];
-
 		fname = tmp;
 		strcpy(fname, orig);
-		sprintf(fndir, "%sfunctions", ppaths->gretldir);
-		if ((fname = search_dir(fname, fndir, FUNCS_SEARCH))) { 
+		sprintf(trydir, "%sfunctions", ppaths->gretldir);
+		if ((fname = search_dir(fname, trydir, FUNCS_SEARCH))) { 
 		    return fname;
 		}
 	    }
 	} else {
-	    /* for a data file, try system data dir (and subdirs) */
-	    if ((fname = search_dir(fname, ppaths->datadir, DATA_SEARCH))) { 
+	    /* data file */
+	    sprintf(trydir, "%sdata", ppaths->gretldir);
+	    if ((fname = search_dir(fname, trydir, DATA_SEARCH))) { 
 		return fname;
 	    }
 	} 
@@ -671,10 +783,29 @@ char *addpath (char *fname, PATHS *ppaths, int script)
     /* or try looking in user's dir (and subdirs) */
     fname = tmp;
     strcpy(fname, orig);
-    if ((fname = search_dir(fname, gretl_user_dir(), USER_SEARCH))) { 
+    if ((fname = search_dir(fname, gretl_work_dir(), USER_SEARCH))) { 
 	return fname;
     }
 
+    /* try looking in default workdir? */
+    if (ppaths != NULL) {
+	char *dwork = gretl_default_workdir(ppaths);
+
+	if (dwork != NULL) {
+	    int ok = 0;
+
+	    fname = tmp;
+	    strcpy(fname, orig);
+	    if ((fname = search_dir(fname, dwork, USER_SEARCH))) { 
+		ok = 1;
+	    }
+	    free(dwork);
+	    if (ok) {
+		return fname;
+	    }
+	}
+    }
+
 #ifdef WIN32
     /* try looking on the desktop? */
     if (1) {
@@ -697,30 +828,24 @@ char *addpath (char *fname, PATHS *ppaths, int script)
     fname = tmp;
     strcpy(fname, orig);
 
+    gretl_error_clear();
+
     return NULL;
 }
 
-static int get_quoted_filename (const char *line, char *fname)
+static int get_quoted_filename (const char *s, char *fname)
 {
-    char *p;
-    int quote = '"';
     int ret = 0;
 
-    p = strchr(line, quote);
-    if (p == NULL) {
-	quote = '\'';
-	p = strchr(line, quote);
-    }
+    if (*s == '"' || *s == '\'') {
+	const char *p = strchr(s + 1, *s);
 
-    if (p != NULL) {
-	char *q = strrchr(line, quote);
-
-	if (q != NULL) {
-	    size_t len = q - p;
+	if (p != NULL) {
+	    size_t len = p - s;
 
 	    if (len > 0) {
 		*fname = 0;
-		strncat(fname, p+1, len-1);
+		strncat(fname, s+1, len-1);
 		ret = 1;
 	    } 
 	}
@@ -772,12 +897,16 @@ int getopenfile (const char *line, char *fname, PATHS *ppaths,
     int script = (opt & OPT_S)? 1 : 0;
     char *fullname;
 
+    /* skip past command word */
+    line += strcspn(line, " ");
+    line += strspn(line, " ");
+
     if (get_quoted_filename(line, fname)) {
 	/* if the filename was quoted, we'll leave it as is */
 	return 0; 
     }
 
-    if (sscanf(line, "%*s %s", fname) != 1) {
+    if (sscanf(line, "%s", fname) != 1) {
 	return E_PARSE;
     }
 
@@ -786,7 +915,7 @@ int getopenfile (const char *line, char *fname, PATHS *ppaths,
     }
 
     /* handle tilde == HOME */
-    if (*fname == '~') {
+    if (fname[0] == '~' && fname[1] == '/') {
 	substitute_homedir(fname);
     }
 
@@ -794,42 +923,46 @@ int getopenfile (const char *line, char *fname, PATHS *ppaths,
     fullname = addpath(fname, ppaths, script);
 
     if (ppaths != NULL && fullname != NULL && script) {
-	int n, spos = slashpos(fname);
+	int spos = slashpos(fname);
 
 	if (spos) {
-	    strncpy(ppaths->currdir, fname, (size_t) spos);
-	    n = strlen(ppaths->currdir);
-	    ppaths->currdir[n] = SLASH;
-	    ppaths->currdir[n+1] = '\0';
+	    *ppaths->currdir = '\0';
+	    strncat(ppaths->currdir, fname, spos + 1);
 	} else {
 	    ppaths->currdir[0] = '.';
 	    ppaths->currdir[1] = SLASH;
 	    ppaths->currdir[2] = '\0';
-	}	    
+	}
     }
 
     return 0;
 }
 
+int has_system_prefix (const char *fname, const PATHS *ppaths, 
+		       int locus)
+{
+    int n = strlen(ppaths->gretldir);
+    int ret = 0;
+
+    if (strlen(fname) < n) return 0;
+    
+    if (!strncmp(fname, ppaths->gretldir, n)) {
+	if (locus == DATA_SEARCH &&
+	    !strncmp(fname + n, "data", 4)) {
+	    ret = 1;
+	} else if (locus == SCRIPT_SEARCH &&
+		   !strncmp(fname + n, "scripts", 7)) { 
+	    ret = 1;
+	}
+    }
+
+    return ret;
+}
+
 enum paths_status_flags {
     STRING_TABLE_WRITTEN = 1 << 0
 };
 
-struct INTERNAL_PATHS {
-    char userdir[MAXLEN];
-    char gnuplot[MAXLEN];
-    char plotfile[MAXLEN];
-    char libpath[MAXLEN];
-    char x12a[MAXLEN];
-    char x12adir[MAXLEN];
-    char tramo[MAXLEN];
-    char tramodir[MAXLEN];
-    char pngfont[128];
-    unsigned char status;
-};
-
-static struct INTERNAL_PATHS gretl_paths;
-
 static void set_gretl_libpath (const char *path)
 {
 #ifdef WIN32
@@ -869,7 +1002,7 @@ static int set_tramo_x12a_dirs (PATHS *ppaths, int baddir)
 	return baddir;
     }
 
-    strcpy(dirname, ppaths->userdir);
+    strcpy(dirname, ppaths->dotdir);
     n = strlen(dirname);
 
     if (n > 0 && (dirname[n-1] == '\\' || dirname[n-1] == '/')) {
@@ -877,7 +1010,7 @@ static int set_tramo_x12a_dirs (PATHS *ppaths, int baddir)
     }
 
 # ifdef HAVE_X12A
-    build_path(ppaths->x12adir, ppaths->userdir, "x12arima", NULL);
+    build_path(ppaths->x12adir, ppaths->dotdir, "x12arima", NULL);
     err = gretl_mkdir(ppaths->x12adir);
     if (err) {
 	*ppaths->x12adir = '\0';
@@ -885,7 +1018,7 @@ static int set_tramo_x12a_dirs (PATHS *ppaths, int baddir)
 # endif
 
 # ifdef HAVE_TRAMO
-    build_path(ppaths->tramodir, ppaths->userdir, "tramo", NULL);
+    build_path(ppaths->tramodir, ppaths->dotdir, "tramo", NULL);
     if (gretl_mkdir(ppaths->tramodir)) {
 	*ppaths->tramodir = '\0';
 	return E_FOPEN;
@@ -919,7 +1052,8 @@ static int set_tramo_x12a_dirs (PATHS *ppaths, int baddir)
 
 static void copy_paths_to_internal (PATHS *paths)
 {
-    strcpy(gretl_paths.userdir,  paths->userdir);
+    strcpy(gretl_paths.dotdir,   paths->dotdir);
+    strcpy(gretl_paths.workdir,  paths->workdir);
     strcpy(gretl_paths.gnuplot,  paths->gnuplot);
     strcpy(gretl_paths.x12a,     paths->x12a);
     strcpy(gretl_paths.x12adir,  paths->x12adir);
@@ -927,13 +1061,14 @@ static void copy_paths_to_internal (PATHS *paths)
     strcpy(gretl_paths.tramodir, paths->tramodir);
     strcpy(gretl_paths.pngfont,  paths->pngfont);
 
-    gretl_insert_builtin_string("gretldir",  paths->gretldir);
-    gretl_insert_builtin_string("userdir",   paths->userdir);
-    gretl_insert_builtin_string("gnuplot",   paths->gnuplot);
-    gretl_insert_builtin_string("x12a",      paths->x12a);
-    gretl_insert_builtin_string("x12adir",   paths->x12adir);
-    gretl_insert_builtin_string("tramo",     paths->tramo);
-    gretl_insert_builtin_string("tramodir",  paths->tramodir);
+    gretl_insert_builtin_string("gretldir", paths->gretldir);
+    gretl_insert_builtin_string("dotdir",   paths->dotdir);
+    gretl_insert_builtin_string("workdir",  paths->workdir);
+    gretl_insert_builtin_string("gnuplot",  paths->gnuplot);
+    gretl_insert_builtin_string("x12a",     paths->x12a);
+    gretl_insert_builtin_string("x12adir",  paths->x12adir);
+    gretl_insert_builtin_string("tramo",    paths->tramo);
+    gretl_insert_builtin_string("tramodir", paths->tramodir);
 
     if (*paths->tramo) {
 	char s[MAXLEN];
@@ -973,37 +1108,119 @@ const char *gretl_lib_path (void)
     return gretl_paths.libpath;
 }
 
-const char *gretl_user_dir (void)
+const char *gretl_dot_dir (void)
 {
-    return gretl_paths.userdir;
+    return gretl_paths.dotdir;
+}
+
+const char *gretl_work_dir (void)
+{
+    return gretl_paths.workdir;
 }
 
 #ifdef WIN32
 
-static void correct_blank_userdir (PATHS *paths)
+static void correct_blank_dotdir (PATHS *paths)
 {
-    char *home = appdata_path();
+    char *base = appdata_path();
 
-    if (home != NULL) {
-	sprintf(paths->userdir, "%s\\gretl\\", home);
-	free(home);
+    if (base != NULL) {
+	sprintf(paths->dotdir, "%s\\gretl\\", base);
+	free(base);
+    } 
+}
+
+/* if the default workdir is a valid directory, and
+   not equal to the current workdir, return the path
+   to it
+*/
+
+char *gretl_default_workdir (PATHS *paths)
+{
+    char *base = mydocs_path();
+    char *ret = NULL;
+    int ok = 0;
+
+    if (base != NULL) {
+	ret = g_strdup_printf("%s\\gretl\\", base);
+	if (strcmp(ret, paths->workdir)) {
+	    DIR *dir = win32_opendir(ret);
+
+	    if (dir != NULL) {
+		closedir(dir);
+		ok = 1;
+	    }
+	}
+	free(base);
+    }
+
+    if (ret && !ok) {
+	free(ret);
+	ret = NULL;
+    }
+
+    return ret;
+}
+
+static void correct_blank_workdir (PATHS *paths)
+{
+    char *base = mydocs_path();
+
+    if (base != NULL) {
+	sprintf(paths->workdir, "%s\\gretl\\", base);
+	free(base);
     } 
 }
 
 #else
 
-static void correct_blank_userdir (PATHS *paths)
+static void correct_blank_dotdir (char *path)
 {
     char *home = getenv("HOME");
 
     if (home != NULL) {
-	sprintf(paths->userdir, "%s/gretl/", home);
+	sprintf(path, "%s/.gretl/", home);
+    } 
+}
+
+char *gretl_default_workdir (PATHS *paths)
+{
+    char *home = getenv("HOME");
+    char *ret = NULL;
+    int ok = 0;
+
+    if (home != NULL) {
+	ret = g_strdup_printf("%s/gretl/", home);
+	if (strcmp(ret, paths->workdir)) {
+	    DIR *dir = opendir(ret);
+
+	    if (dir != NULL) {
+		closedir(dir);
+		ok = 1;
+	    }
+	}
+    }
+
+    if (ret && !ok) {
+	free(ret);
+	ret = NULL;
+    }
+
+    return ret;
+}
+
+static void correct_blank_workdir (char *path)
+{
+    char *home = getenv("HOME");
+
+    if (home != NULL) {
+	sprintf(path, "%s/gretl/", home);
     } 
 }
 
 #endif
 
-static int validate_userdir (const char *dirname)
+static int validate_writedir (const char *dirname)
 {
     int err = 0;
 
@@ -1041,29 +1258,31 @@ static int validate_userdir (const char *dirname)
     return err;
 }
 
-int set_gretl_user_dir (const char *path, PATHS *ppaths)
+int set_gretl_work_dir (const char *path, PATHS *ppaths)
 {
-    int err = validate_userdir(path);
+    DIR *test;
 
-    if (err) {
-	return err;
+    errno = 0;
+
+#ifdef WIN32
+    test = win32_opendir(path);
+#else
+    test = opendir(path);
+#endif
+    if (test == NULL) {
+	gretl_errmsg_set_from_errno(path);
+	return E_FOPEN;
+    } 
+
+    closedir(test);
+
+    if (path != ppaths->workdir) {
+	strcpy(ppaths->workdir, path);
+	ensure_slash(ppaths->workdir);
+	strcpy(gretl_paths.workdir, ppaths->workdir);
+	gretl_insert_builtin_string("workdir", ppaths->workdir);
     }
 
-    if (path != ppaths->userdir) {
-	strcpy(ppaths->userdir, path);
-    }
-
-    ensure_slash(ppaths->userdir);
-    set_tramo_x12a_dirs(ppaths, 0);
-
-    strcpy(gretl_paths.userdir, ppaths->userdir);
-    strcpy(gretl_paths.x12adir, ppaths->x12adir);
-    strcpy(gretl_paths.tramodir, ppaths->tramodir);
-
-    gretl_insert_builtin_string("userdir", ppaths->userdir);
-    gretl_insert_builtin_string("x12adir",  ppaths->x12adir);
-    gretl_insert_builtin_string("tramodir", ppaths->tramodir);
-
     return 0;
 }
 
@@ -1127,20 +1346,12 @@ int gretl_string_table_written (void)
     return ret;
 }
 
-static void ensure_slash (char *str)
-{
-    if (str[strlen(str) - 1] != SLASH) {
-	strcat(str, SLASHSTR);
-    }
-}
-
 void show_paths (const PATHS *ppaths)
 {
     printf(_("gretl: using these basic search paths:\n"));
     printf("gretldir: %s\n", ppaths->gretldir);
-    printf("userdir: %s\n", ppaths->userdir);
-    printf("datadir: %s\n", ppaths->datadir);
-    printf("scriptdir: %s\n", ppaths->scriptdir);
+    printf("workdir: %s\n", ppaths->workdir);
+    printf("dotdir: %s\n", ppaths->dotdir);
     printf("gnuplot: %s\n", ppaths->gnuplot);
 }
 
@@ -1148,7 +1359,7 @@ void show_paths (const PATHS *ppaths)
 
 int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 {
-    char envstr[MAXLEN];
+    static char envstr[MAXLEN];
     int err = 0;
 
     if (opt & OPT_D) {
@@ -1174,24 +1385,31 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 	    ppaths->dbhost[0] = '\0';
 	}
 
-	shelldir_init();
+	shelldir_init(NULL);
 	ppaths->currdir[0] = '\0';
-	*gretl_paths.plotfile = '\0';
+	ppaths->dotdir[0] = '\0';
+	ppaths->workdir[0] = '\0';
+	gretl_paths.plotfile[0] = '\0';
 	strcpy(ppaths->pngfont, "verdana 8");
     } else {
 	ensure_slash(ppaths->gretldir);
-	if (*ppaths->userdir == '\0') {
-	    correct_blank_userdir(ppaths);
+	if (*ppaths->dotdir == '\0') {
+	    correct_blank_dotdir(ppaths);
+	}
+	err = validate_writedir(ppaths->dotdir);
+	if (*ppaths->workdir == '\0') {
+	    correct_blank_workdir(ppaths);
+	}
+	if (strcmp(ppaths->dotdir, ppaths->workdir)) { 
+	    err += validate_writedir(ppaths->workdir);
 	}
-	err = validate_userdir(ppaths->userdir);
 #if defined(HAVE_X12A) || defined(HAVE_TRAMO)
-	err = set_tramo_x12a_dirs(ppaths, err);
+	if (!err) {
+	    err = set_tramo_x12a_dirs(ppaths, err);
+	}
 #endif
     }
 
-    sprintf(ppaths->datadir, "%sdata\\", ppaths->gretldir);
-    sprintf(ppaths->scriptdir, "%sscripts\\", ppaths->gretldir);
-
     if (opt & OPT_X) {
 	/* gui program */
 	gretl_set_gui_mode(1);
@@ -1207,16 +1425,22 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 	}
     } else { 
 	sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretlcli_hlp.txt"));
+	strcpy(ppaths->cli_helpfile, ppaths->helpfile);
     }
 
     sprintf(envstr, "GTKSOURCEVIEW_LANGUAGE_DIR=%sshare\\gtksourceview-1.0"
 	    "\\language-specs", ppaths->gretldir);
     putenv(envstr);
 
-    ensure_slash(ppaths->userdir);
+    ensure_slash(ppaths->dotdir);
+    ensure_slash(ppaths->workdir);
     set_gretl_libpath(ppaths->gretldir);
     copy_paths_to_internal(ppaths);
 
+    if (!(opt & OPT_D)) {
+	shelldir_init(ppaths->workdir);
+    }
+
     return err;
 }
 
@@ -1272,15 +1496,18 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 	strcpy(ppaths->gnuplot, "gnuplot");
 	strcpy(ppaths->pngfont, "Vera 9");
 	ppaths->currdir[0] = '\0';	
-	shelldir_init();
+	shelldir_init(NULL);
 
 	/* try to set a default userdir */
 	home = getenv("HOME");
 	if (home != NULL) {
-	    strcpy(ppaths->userdir, home);
-	    strcat(ppaths->userdir, "/gretl/");
+	    strcpy(ppaths->dotdir, home);
+	    strcat(ppaths->dotdir, "/.gretl/");
+	    strcpy(ppaths->workdir, home);
+	    strcat(ppaths->workdir, "/gretl/");
 	} else {
-	    *ppaths->userdir = '\0';
+	    *ppaths->dotdir = '\0';
+	    *ppaths->workdir = '\0';
 	}
 
 #ifdef HAVE_X12A 	 
@@ -1293,17 +1520,20 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 
 	*gretl_paths.plotfile = '\0';
     } else {
-	/* check validity of gretldir, userdir */
+	/* check validity of main directories */
 	check_gretldir(ppaths);
-	if (*ppaths->userdir == '\0') {
-	    correct_blank_userdir(ppaths);
+	if (*ppaths->dotdir == '\0') {
+	    correct_blank_dotdir(ppaths->dotdir);
+	}
+	if (*ppaths->workdir == '\0') {
+	    correct_blank_workdir(ppaths->workdir);
+	}
+	err = validate_writedir(ppaths->dotdir);
+	if (strcmp(ppaths->dotdir, ppaths->workdir)) {
+	    err += validate_writedir(ppaths->workdir);
 	}
-	err = validate_userdir(ppaths->userdir);
     }
 
-    sprintf(ppaths->datadir, "%sdata/", ppaths->gretldir);
-    sprintf(ppaths->scriptdir, "%sscripts/", ppaths->gretldir);
-
     if (opt & OPT_X) {
 	gretl_set_gui_mode(1);
 	if (opt & OPT_N) {
@@ -1318,17 +1548,18 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 	}
     } else {
 	sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretlcli.hlp"));
+	strcpy(ppaths->cli_helpfile, ppaths->helpfile);
     }
 
     if (getenv("GTKSOURCEVIEW_LANGUAGE_DIR") == NULL) {
-	char envstr[MAXLEN];
+	static char envstr[MAXLEN];
 
 	sprintf(envstr, "GTKSOURCEVIEW_LANGUAGE_DIR=%sgtksourceview",
 		ppaths->gretldir);
 	putenv(envstr);
     }
 
-    ensure_slash(ppaths->userdir);
+    ensure_slash(ppaths->dotdir);
     set_gretl_libpath(ppaths->gretldir);
 
 #if defined(HAVE_X12A) || defined(HAVE_TRAMO)
@@ -1339,28 +1570,232 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 
     copy_paths_to_internal(ppaths);
 
+#ifdef OSX_BUILD
+    if (!(opt & OPT_D)) {
+	shelldir_init(ppaths->workdir);
+    }
+#endif
+
     return err;
 }
 
 #endif /* win32 versus unix */
 
 /* for writing a file, name given by user: if the path is not
-   absolute, switch to the gretl user dir (for a plain filename and
-   use_cwd not set) or to the current "shelldir" (for a filename
-   beginning with '.', or if use_cwd is set).
+   absolute, switch to the gretl "workdir" (for a plain filename and
+   STATE_USE_CWD not set), or to the current "shelldir" (for a filename
+   beginning with '.', or if STATE_USE_CWD is set).
 */
 
-void gretl_maybe_switch_dir (const char *fname)
+const char *gretl_maybe_switch_dir (const char *fname)
 {
-    if (!g_path_is_absolute(fname)) {
+    if (fname[0] == '~' && fname[1] == '/') {
+	char *home = getenv("HOME");
+	
+	if (home != NULL) {
+	    chdir(home);
+	    fname += 2;
+	}
+    } else if (!g_path_is_absolute(fname)) {
 	if (dotpath(fname) || libset_get_bool(USE_CWD)) {
 	    char *sdir = get_shelldir();
 
-	    if (sdir != NULL) {
+	    if (sdir != NULL && *sdir != '\0') {
 		chdir(sdir);
 	    }
 	} else {
-	    chdir(gretl_paths.userdir);
+	    chdir(gretl_paths.workdir);
 	}
     }
+
+    return fname;
 }
+
+/* remove '.' and '..' from @path */
+
+int gretl_normalize_path (char *path)
+{
+    char tmp[FILENAME_MAX];
+    char *pcpy, *pbit, *s = path;
+    char **S, **P = NULL;
+    int i, n = 0;
+    int err = 0;
+
+    if (*path == '\0' || strstr(path, SLASHSTR) == NULL) {
+	return 0;
+    }
+
+    pcpy = gretl_strdup(path);
+    if (pcpy == NULL) {
+	return E_ALLOC;
+    }
+
+    *tmp = '\0';
+    s = pcpy;
+
+#ifdef WIN32
+    /* may be ok for a filename to start with a double backslash */
+    if (!strncmp(path, "\\\\", 2)) {
+	strcpy(tmp, SLASHSTR);
+	s++;
+    } else if (*path && path[1] == ':') {
+	strncat(tmp, path, 2);
+	s += 2;
+    }
+#endif
+
+    while ((pbit = strtok(s, SLASHSTR)) != NULL && !err) {
+	if (strcmp(pbit, ".")) {
+	    S = realloc(P, (n+1) * sizeof *P);
+	    if (S == NULL) {
+		err = E_ALLOC;
+	    } else {
+		P = S;
+		P[n++] = pbit;
+	    }
+	}
+	s = NULL;
+    }
+
+    if (!err) {
+	int j;
+
+	for (i=n-1; i>0; i--) {
+	    if (P[i] != NULL && !strcmp(P[i], "..")) {
+		for (j=i-1; j>0; j--) {
+		    if (P[j] != NULL && strcmp(P[j], "..")) {
+			P[j] = NULL;
+			break;
+		    }
+		}
+	    }
+	}
+	for (i=0; i<n; i++) {
+	    if (P[i] != NULL && strcmp(P[i], "..")) {
+		strcat(tmp, SLASHSTR);
+		strcat(tmp, P[i]);
+	    }
+	}
+	strcpy(path, tmp);
+	free(P);
+    }
+
+    free(pcpy);
+    
+    return err;
+}
+
+#ifndef WIN32
+
+static void rc_set_gp_colors (const char *gpcolors)
+{
+    char cstr[N_GP_COLORS][8];
+    int i, nc;
+
+    *cstr[0] = *cstr[1] = *cstr[2] = *cstr[3] = '\0';
+
+    nc = sscanf(gpcolors, "%7s %7s %7s %7s", 
+		cstr[0], cstr[1], cstr[2], cstr[3]);
+
+    for (i=0; i<nc; i++) {
+	set_graph_palette_from_string(i, cstr[i]);
+    }
+}
+
+static int rc_bool (const char *s)
+{
+    if (!strcmp(s, "true") || !strcmp(s, "1")) {
+	return 1;
+    } else {
+	return 0;
+    }	
+}
+
+int cli_read_rc (PATHS *paths) 
+{
+    FILE *fp;
+    char rcfile[FILENAME_MAX];
+    char line[MAXLEN], key[32], val[MAXLEN];
+    char dbproxy[21] = {0};
+    char *home;
+    int usecwd = 0;
+    int use_proxy = 0;
+    int err = 0;
+
+    home = getenv("HOME");
+    if (home == NULL) {
+	return 1;
+    }
+
+    sprintf(rcfile, "%s/.gretl2rc", home);
+    fp = gretl_fopen(rcfile, "r");
+    if (fp == NULL) {
+	return 1;
+    }
+
+    while (fgets(line, MAXLEN, fp) != NULL) {
+	if (*line == '#') {
+	    continue;
+	}
+	if (!strncmp(line, "recent", 6)) {
+	    break;
+	}
+	if (sscanf(line, "%s", key) == 1) {
+	    strcpy(val, line + strlen(key) + 3); 
+	    chopstr(val); 
+	    if (!strcmp(key, "gretldir")) {
+		*paths->gretldir = '\0';
+		strncat(paths->gretldir, val, MAXLEN - 1);
+	    } else if (!strcmp(key, "userdir")) {
+		*paths->workdir = '\0';
+		strncat(paths->workdir, val, MAXLEN - 1);
+	    } else if (!strcmp(key, "shellok")) {
+		libset_set_bool(SHELL_OK, rc_bool(val));
+	    } else if (!strcmp(key, "usecwd")) {
+		usecwd = rc_bool(val);
+		libset_set_bool(USE_CWD, usecwd);
+	    } else if (!strcmp(key, "gnuplot")) {
+		*paths->gnuplot = '\0';
+		strncat(paths->gnuplot, val, MAXLEN - 1);
+	    } else if (!strcmp(key, "binbase")) {
+		*paths->binbase = '\0';
+		strncat(paths->binbase, val, MAXLEN - 1);
+	    } else if (!strcmp(key, "ratsbase")) {
+		*paths->ratsbase = '\0';
+		strncat(paths->ratsbase, val, MAXLEN - 1);
+	    } else if (!strcmp(key, "dbhost")) {
+		*paths->dbhost = '\0';
+		strncat(paths->dbhost, val, 32 - 1);
+	    } else if (!strcmp(key, "dbproxy")) {
+		strncat(dbproxy, val, 21 - 1);
+	    } else if (!strcmp(key, "useproxy")) {
+		use_proxy = rc_bool(val);
+	    } else if (!strcmp(key, "x12a")) {
+		*paths->x12a = '\0';
+		strncat(paths->x12a, val, FILENAME_MAX - 1);
+	    } else if (!strcmp(key, "Gp_colors")) {
+		rc_set_gp_colors(val);
+	    } 
+	}
+    }
+
+    fclose(fp);
+
+    if (usecwd) {
+	char *s, cwd[MAXLEN];
+
+	s = getcwd(cwd, MAXLEN);
+	if (s != NULL) {
+	    *paths->workdir = '\0';
+	    strncat(paths->workdir, s, MAXLEN - 2);
+	    ensure_slash(paths->workdir);
+	}
+    }
+
+    err = gretl_set_paths(paths, OPT_NONE);
+    gretl_www_init(paths->dbhost, dbproxy, use_proxy);
+
+    return err;
+}
+
+#endif
