@@ -20,17 +20,27 @@
 #ifndef GRETL_PATHS_H
 #define GRETL_PATHS_H
 
+typedef enum {
+    CURRENT_DIR,
+    DATA_SEARCH,
+    SCRIPT_SEARCH,
+    FUNCS_SEARCH,
+    USER_SEARCH
+} SearchLocation;
+
 void set_string_table_written (void);
 
 int gretl_string_table_written (void);
 
 int gretl_path_prepend (char *file, const char *path);
 
+int gretl_normalize_path (char *path);
+
 void set_fopen_use_utf8 (void);
 
-FILE *gretl_fopen (const char *filename, const char *mode);
+FILE *gretl_fopen (const char *fname, const char *mode);
 
-gzFile gretl_gzopen (const char *filename, const char *mode);
+gzFile gretl_gzopen (const char *fname, const char *mode);
 
 int gretl_mkdir (const char *path);
 
@@ -47,17 +57,24 @@ int getopenfile (const char *line, char *fname, PATHS *ppaths,
 
 int gretl_path_is_absolute (const char *fname);
 
+int has_system_prefix (const char *fname, const PATHS *ppaths,
+		       int locus);
+
 void show_paths (const PATHS *ppaths);
 
 int gretl_set_paths (PATHS *ppaths, gretlopt opt);
 
 const char *gretl_lib_path (void);
 
-const char *gretl_user_dir (void);
+const char *gretl_dot_dir (void);
 
-int set_gretl_user_dir (const char *path, PATHS *ppaths);
+const char *gretl_work_dir (void);
 
-void gretl_maybe_switch_dir (const char *fname);
+char *gretl_default_workdir (PATHS *paths);
+
+int set_gretl_work_dir (const char *path, PATHS *ppaths);
+
+const char *gretl_maybe_switch_dir (const char *fname);
 
 const char *gretl_gnuplot_path (void);
 
@@ -75,4 +92,10 @@ const char *gretl_png_font (void);
 
 void set_gretl_png_font (const char *s, PATHS *ppaths);
 
+#ifndef WIN32
+
+int cli_read_rc (PATHS *paths);
+
+#endif
+
 #endif /* GRETL_PATHS_H */
