@@ -20,18 +20,20 @@
 #include "libgretl.h"
 #include "libset.h"
 #include <stdarg.h>
+#include <errno.h>
 #include <glib.h>
 
 struct PRN_ {
-    FILE *fp;
-    FILE *fpaux;    
-    char *buf;
-    size_t bufsize;
-    int savepos;
-    PrnFormat format;
-    int fixed;
-    char delim;
-    char *fname;
+    FILE *fp;          /* file to which to print, or NULL */
+    FILE *fpaux;       /* auxiliary file (output redirected) */
+    char *buf;         /* buffer to which to print, or NULL */
+    size_t bufsize;    /* allocated size of buffer */
+    size_t blen;       /* string length of buffer */
+    int savepos;       /* saved poistion in stream or buffer */
+    PrnFormat format;  /* plain, TeX, RTF */
+    int fixed;         /* = 1 for fixed-size buffer */
+    char delim;        /* CSV field delimiter */
+    char *fname;       /* file name, or NULL */
 };
 
 #undef PRN_DEBUG
@@ -81,33 +83,52 @@ void gretl_print_destroy (PRN *prn)
 static int prn_add_tempfile (PRN *prn)
 {
     char fname[MAXLEN];
-    int fd;
+    int fd, err = 0;
 
-    sprintf(fname, "%sprntmp.XXXXXX", gretl_user_dir());
+    errno = 0;
+
+    sprintf(fname, "%sprntmp.XXXXXX", gretl_dot_dir());
 
 #ifdef WIN32
     fd = g_mkstemp(fname);
 #else
     fd = mkstemp(fname);
 #endif
+
     if (fd == -1) {
-	return E_FOPEN;
+	err = E_FOPEN;
     } else {
 	prn->fname = gretl_strdup(fname);
-	prn->fp = fdopen(fd, "w");
+	if (prn->fname == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    prn->fp = fdopen(fd, "w");
+	    if (prn->fp == NULL) {
+		err = E_FOPEN;
+	    }
+	}
     }
 
-    return 0;
+    if (errno != 0) {
+	gretl_errmsg_set_from_errno("prn_add_tempfile");
+	
+    }
+
+    return err;
 }
 
 static PRN *real_gretl_print_new (PrnType ptype, 
 				  const char *fname,
-				  char *buf)
+				  char *buf,
+				  int *perr)
 {
     PRN *prn = malloc(sizeof *prn);
+    int err = 0;
 
     if (prn == NULL) {
-	fprintf(stderr, _("gretl_prn_new: out of memory\n"));
+	if (perr != NULL) {
+	    *perr = E_ALLOC;
+	}
 	return NULL;
     }
 
@@ -120,6 +141,7 @@ static PRN *real_gretl_print_new (PrnType ptype,
     prn->fpaux = NULL;
     prn->buf = NULL;
     prn->bufsize = 0;
+    prn->blen = 0;
     prn->savepos = -1;
     prn->format = GRETL_FORMAT_TXT;
     prn->fixed = 0;
@@ -129,13 +151,13 @@ static PRN *real_gretl_print_new (PrnType ptype,
     if (ptype == GRETL_PRINT_FILE) {
 	prn->fp = gretl_fopen(fname, "w");
 	if (prn->fp == NULL) {
-	    fprintf(stderr, _("gretl_prn_new: couldn't open %s\n"), fname);
+	    err = E_FOPEN;
 	    free(prn);
 	    prn = NULL;
 	} 
     } else if (ptype == GRETL_PRINT_TEMPFILE) {
-	if (prn_add_tempfile(prn)) {
-	    fprintf(stderr, "gretl_prn_new: couldn't open tempfile\n");
+	err = prn_add_tempfile(prn);
+	if (err) {
 	    free(prn);
 	    prn = NULL;
 	} else {
@@ -156,7 +178,7 @@ static PRN *real_gretl_print_new (PrnType ptype,
 	    int p = pprintf(prn, "@init");
 
 	    if (p < 0) {
-		fprintf(stderr, _("gretl_prn_new: out of memory\n"));
+		err = E_ALLOC;
 		free(prn);
 		prn = NULL;
 	    }
@@ -166,12 +188,17 @@ static PRN *real_gretl_print_new (PrnType ptype,
 	}
     }
 
+    if (perr != NULL) {
+	*perr = err;
+    }
+
     return prn;
 }
 
 /**
  * gretl_print_new:
  * @ptype: code indicating the desired printing mode.
+ * @err: location to receive error code, or %NULL.
  * 
  * Create and initialize a gretl printing struct. If @ptype
  * is %GRETL_PRINT_BUFFER, output will go to an automatically
@@ -187,7 +214,7 @@ static PRN *real_gretl_print_new (PrnType ptype,
  * Returns: pointer to newly created struct, or %NULL on failure.
  */
 
-PRN *gretl_print_new (PrnType ptype)
+PRN *gretl_print_new (PrnType ptype, int *err)
 {
     if (ptype == GRETL_PRINT_FILE) {
 	fprintf(stderr, "gretl_print_new: needs a filename\n");
@@ -198,12 +225,13 @@ PRN *gretl_print_new (PrnType ptype)
     fprintf(stderr, "gretl_print_new() called, type = %d\n", ptype);
 #endif
 
-    return real_gretl_print_new(ptype, NULL, NULL);
+    return real_gretl_print_new(ptype, NULL, NULL, err);
 }
 
 /**
  * gretl_print_new_with_filename:
  * @fname: name of the file to be opened for writing.
+ * @err: location to receive error code.
  * 
  * Create and initialize a gretl printing struct, with
  * output directed to the named file.  
@@ -211,18 +239,19 @@ PRN *gretl_print_new (PrnType ptype)
  * Returns: pointer to newly created struct, or %NULL on failure.
  */
 
-PRN *gretl_print_new_with_filename (const char *fname)
+PRN *gretl_print_new_with_filename (const char *fname, int *err)
 {
     if (fname == NULL) {
 	fprintf(stderr, _("gretl_prn_new: Must supply a filename\n"));
 	return NULL;
     }
 
-    return real_gretl_print_new(GRETL_PRINT_FILE, fname, NULL);
+    return real_gretl_print_new(GRETL_PRINT_FILE, fname, NULL, err);
 }
 
 /**
  * gretl_print_new_with_tempfile:
+ * @err: location to receive error code.
  * 
  * Create and initialize a gretl printing struct, with
  * output directed to a temporary file, which is deleted
@@ -231,9 +260,9 @@ PRN *gretl_print_new_with_filename (const char *fname)
  * Returns: pointer to newly created struct, or %NULL on failure.
  */
 
-PRN *gretl_print_new_with_tempfile (void)
+PRN *gretl_print_new_with_tempfile (int *err)
 {
-    return real_gretl_print_new(GRETL_PRINT_TEMPFILE, NULL, NULL);
+    return real_gretl_print_new(GRETL_PRINT_TEMPFILE, NULL, NULL, err);
 }
 
 /**
@@ -268,7 +297,7 @@ PRN *gretl_print_new_with_buffer (char *buf)
     if (buf == NULL) {
 	return NULL;
     } else {
-	return real_gretl_print_new(GRETL_PRINT_BUFFER, NULL, buf);
+	return real_gretl_print_new(GRETL_PRINT_BUFFER, NULL, buf, NULL);
     }
 }
 
@@ -311,6 +340,30 @@ const char *gretl_print_get_buffer (PRN *prn)
     return (prn != NULL)? prn->buf : NULL;
 }
 
+/**
+ * gretl_print_steal_buffer:
+ * @prn: printing struct.
+ * 
+ * Obtain a pointer to the buffer associated with @prn,
+ * if any.  The pointer on @prn itself is set to %NULL 
+ * and the caller takes responsibility for freeing the 
+ * buffer.
+ *
+ * Returns: the buffer, or %NULL on failure.
+ */
+
+char *gretl_print_steal_buffer (PRN *prn)
+{
+    char *buf = NULL;
+
+    if (prn != NULL) {
+	buf = prn->buf;
+	prn->buf = NULL;
+    }
+
+    return buf;
+}
+
 /**
  * gretl_print_read_tempfile:
  * @prn: printing struct.
@@ -475,22 +528,24 @@ void gretl_print_toggle_doc_flag (PRN *prn)
     }
 }
 
-static int realloc_prn_buffer (PRN *prn, size_t blen)
+static int realloc_prn_buffer (PRN *prn)
 {
+    size_t newlen;
     char *tmp;
     int err = 0;
 
 #if PRN_DEBUG
     fprintf(stderr, "%d bytes left\ndoing realloc(%p, %d)\n",
-	    prn->bufsize - blen, prn->buf, 2 * prn->bufsize);
+	    prn->bufsize - prn->blen, prn->buf, 2 * prn->bufsize);
 #endif
-    prn->bufsize *= 2; 
 
-    tmp = realloc(prn->buf, prn->bufsize); 
+    newlen = prn->bufsize * 2;
+    tmp = realloc(prn->buf, newlen); 
 
     if (tmp == NULL) {
 	err = 1;
     } else {
+	prn->bufsize = newlen;
 	prn->buf = tmp;
 #if PRN_DEBUG
 	fprintf(stderr, "realloc: prn->buf is %d bytes at %p\n",
@@ -498,11 +553,33 @@ static int realloc_prn_buffer (PRN *prn, size_t blen)
 #endif
     }
 
-    memset(prn->buf + blen, 0, 1);
+    memset(prn->buf + prn->blen, 0, 1);
 
     return err;
 }
 
+static int pprintf_init (PRN *prn)
+{
+    int ret = 0;
+
+    prn->bufsize = 2048;
+    prn->buf = malloc(prn->bufsize);
+
+#if PRN_DEBUG
+    fprintf(stderr, "pprintf: malloc'd %d bytes at %p\n", prn->bufsize,  
+	    (void *) prn->buf); 
+#endif
+
+    if (prn->buf == NULL) {
+	ret = -1;
+    } else {
+	memset(prn->buf, 0, 1);
+	prn->blen = 0;
+    }
+
+    return ret;
+}
+
 /**
  * pprintf:
  * @prn: gretl printing struct.
@@ -521,7 +598,6 @@ static int realloc_prn_buffer (PRN *prn, size_t blen)
 int pprintf (PRN *prn, const char *template, ...)
 {
     va_list args;
-    size_t blen;
     int rem, plen = 0;
 
     if (prn == NULL || prn->fixed) {
@@ -536,45 +612,33 @@ int pprintf (PRN *prn, const char *template, ...)
     }
 
     if (strncmp(template, "@init", 5) == 0) {
-	prn->bufsize = 2048;
-	prn->buf = malloc(prn->bufsize);
-#if PRN_DEBUG
-  	fprintf(stderr, "pprintf: malloc'd %d bytes at %p\n", prn->bufsize,  
-		(void *) prn->buf); 
-#endif
-	if (prn->buf == NULL) {
-	    return -1;
-	}
-	memset(prn->buf, 0, 1);
-	return 0;
+	return pprintf_init(prn);
     }
 
     if (prn->buf == NULL) {
 	return 0;
     }
 
-    blen = strlen(prn->buf);
-
-    if (prn->bufsize - blen < 1024) {
-	if (realloc_prn_buffer(prn, blen)) {
+    if (prn->bufsize - prn->blen < 1024) {
+	if (realloc_prn_buffer(prn)) {
 	    return -1;
 	}
     }
 
-    rem = prn->bufsize - blen - 1;
+#if PRN_DEBUG
+    fprintf(stderr, "printing at %p\n", (void *) (prn->buf + prn->blen));
+#endif
 
+    rem = prn->bufsize - prn->blen - 1;
     va_start(args, template);
-#if PRN_DEBUG
-    fprintf(stderr, "printing at %p\n", (void *) (prn->buf + blen));
-#endif
-    plen = vsnprintf(prn->buf + blen, rem, template, args);
+    plen = vsnprintf(prn->buf + prn->blen, rem, template, args);
     va_end(args);
-#if PRN_DEBUG
-    fprintf(stderr, "printed %d byte(s)\n", strlen(prn->buf) - blen);
-#endif
 
     if (plen >= rem) {
 	fputs("pprintf warning: string was truncated\n", stderr);
+	prn->blen += rem;
+    } else {
+	prn->blen += plen;
     }
 
     return plen;
@@ -591,10 +655,11 @@ int pprintf (PRN *prn, const char *template, ...)
 
 int pputs (PRN *prn, const char *s)
 {
-    size_t blen;
     int slen, bytesleft;
 
-    if (prn == NULL || prn->fixed) return 0;
+    if (prn == NULL || prn->fixed) {
+	return 0;
+    }
 
     slen = strlen(s);
 
@@ -603,17 +668,17 @@ int pputs (PRN *prn, const char *s)
 	return slen;
     }
 
-    if (prn->buf == NULL) return 0;
+    if (prn->buf == NULL) {
+	return 0;
+    }
 
-    blen = strlen(prn->buf);
+    bytesleft = prn->bufsize - prn->blen;
 
-    bytesleft = prn->bufsize - blen;
-
-    while (prn->bufsize - blen < 1024 || bytesleft <= slen) {
-	if (realloc_prn_buffer(prn, blen)) {
+    while (prn->bufsize - prn->blen < 1024 || bytesleft <= slen) {
+	if (realloc_prn_buffer(prn)) {
 	    return -1;
 	}
-	bytesleft = prn->bufsize - blen;
+	bytesleft = prn->bufsize - prn->blen;
     }	
 
 #if PRN_DEBUG
@@ -621,7 +686,8 @@ int pputs (PRN *prn, const char *s)
 	    (int) prn->bufsize, (int) blen, bytesleft, slen);
 #endif
 
-    strcpy(prn->buf + blen, s);
+    strcpy(prn->buf + prn->blen, s);
+    prn->blen += slen;
 
     return slen;
 }
@@ -637,30 +703,32 @@ int pputs (PRN *prn, const char *s)
 
 int pputc (PRN *prn, int c)
 {
-    size_t blen;
-
-    if (prn == NULL || prn->fixed) return 0;
+    if (prn == NULL || prn->fixed) {
+	return 0;
+    }
 
     if (prn->fp != NULL) {
 	fputc(c, prn->fp);
 	return 1;
     }
 
-    if (prn->buf == NULL) return 0;
+    if (prn->buf == NULL) {
+	return 0;
+    }
 
-    blen = strlen(prn->buf);
-
-    if (prn->bufsize - blen < 1024) {
-	if (realloc_prn_buffer(prn, blen)) {
+    if (prn->bufsize - prn->blen < 1024) {
+	if (realloc_prn_buffer(prn)) {
 	    return -1;
 	}
     }
 
 #if PRN_DEBUG
-    fprintf(stderr, "pputc: adding char at %p\n", (void *) (prn->buf + blen));
+    fprintf(stderr, "pputc: adding char at %p\n", (void *) (prn->buf + prn->blen));
 #endif
-    prn->buf[blen] = c;
-    prn->buf[blen + 1] = '\0';
+
+    prn->buf[prn->blen] = c;
+    prn->buf[prn->blen + 1] = '\0';
+    prn->blen += 1;
 
     return 1;
 }
@@ -734,6 +802,8 @@ int printing_is_redirected (PRN *prn)
 	if (prn->fpaux != NULL || 
 	    (prn->fp != NULL && prn->buf != NULL)) {
 	    ret = 1;
+	} else if (prn->fixed) {
+	    ret = 1;
 	}
     }
 
@@ -755,13 +825,18 @@ int print_start_redirection (PRN *prn, FILE *fp)
     int err = 0;
 
     if (prn != NULL) {
-	/* flush and save the current stream */
+	/* flush the current stream */
 	if (prn->fp != NULL) {
 	    fflush(prn->fp);
 	}
-	prn->fpaux = prn->fp;
-	/* hook output to specified file */
-	prn->fp = fp;
+	if (fp == NULL) {
+	    /* disable printing */
+	    prn->fixed = 1;
+	} else {
+	    /* hook output to specified file */
+	    prn->fpaux = prn->fp;
+	    prn->fp = fp;
+	}
     } else {
 	err = 1;
     }
@@ -783,10 +858,14 @@ int print_end_redirection (PRN *prn)
 {
     int err = 0;
 
-    if (prn != NULL && prn->fp != NULL) {
-	fclose(prn->fp);
-	prn->fp = prn->fpaux;
-	prn->fpaux = NULL;
+    if (prn != NULL) {
+	if (prn->fixed) {
+	    prn->fixed = 0;
+	} else if (prn->fp != NULL) {
+	    fclose(prn->fp);
+	    prn->fp = prn->fpaux;
+	    prn->fpaux = NULL;
+	}
     } else {
 	err = 1;
     }
@@ -912,10 +991,69 @@ char prn_delim (PRN *prn)
     return (prn != NULL)? prn->delim : ',';
 }
 
+/**
+ * gretl_print_has_buffer:
+ * @prn: gretl printing struct.
+ * 
+ * Returns: 1 if @prn is using a buffer for printing,
+ * otherwise 0 (e.g., if a file is being used).
+ */
+
+int gretl_print_has_buffer (PRN *prn)
+{
+    return (prn != NULL && prn->buf != NULL);
+}
+
+/**
+ * gretl_print_alloc:
+ * @prn: gretl printing struct.
+ * @s: desired size in bytes.
+ *
+ * If @prn is using a buffer for printing, try to
+ * ensure that at least @s bytes are available for
+ * further printing.  May speed things up if we
+ * know there is a large amount of text to be
+ * printed.
+ * 
+ * Returns: 0 on success, non-zero on failure.
+ */
+
+int gretl_print_alloc (PRN *prn, size_t s)
+{
+    size_t rem;
+    int err = 0;
+
+    if (prn == NULL || prn->buf == NULL || prn->fixed) {
+	return E_DATA;
+    }
+
+    rem = prn->bufsize - prn->blen;
+
+    if (rem <= s) {
+	size_t newlen = prn->blen + s + 1;
+	char *tmp;
+
+	tmp = realloc(prn->buf, newlen); 
+	if (tmp == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    prn->buf = tmp;
+	    prn->bufsize = newlen;
+	    memset(prn->buf + prn->blen, 0, 1);
+#if PRN_DEBUG
+	    fprintf(stderr, "gretl_print_alloc: realloc'd prn->buf "
+		    "to %d bytes\n", newlen);
+#endif
+	}
+    }
+
+    return err;
+}
+
 /**
  * set_up_verbose_printer.
  * @opt: %OPT_V for verbosity.
- * @prn: currently active printing struct
+ * @prn: currently active printing struct.
  *
  * Returns: printing struct for verbose output.
  */
@@ -926,7 +1064,9 @@ PRN *set_up_verbose_printer (gretlopt opt, PRN *prn)
 
     if (opt & OPT_V) {
 	if (iter_print_func_installed()) {
-	    vprn = gretl_print_new_with_tempfile();
+	    int err;
+
+	    vprn = gretl_print_new_with_tempfile(&err);
 	} else {
 	    vprn = prn;
 	}
