@@ -243,13 +243,14 @@ get_R_sys_column (const gretl_restriction *rset, int i, int j)
 {
     const rrow *r = rset->rows[i];
     equation_system *sys = rset->obj;
-    const int *list;
     int col = r->bnum[j];
-    int k;
+    int k, n;
 
     for (k=0; k<r->eq[j]; k++) {
-	list = system_get_list(sys, k);
-	col += list[0] - 1;
+	n = system_get_list_length(sys, k);
+	if (n > 0) {
+	    col += n - 1;
+	}
     }
 
     return col;
@@ -470,8 +471,8 @@ static int rset_alloc_matrices (gretl_restriction *rset, int nc)
     rset->q = gretl_zero_matrix_new(rset->k, 1);
 
 #if RDEBUG
-    fprintf(stderr, "rset_alloc_matrices: on output R=%p, q=%p\n",
-	    (void *) rset->R, (void *) rset->q);
+    fprintf(stderr, "rset_alloc_matrices: on output R=%p (%dx%d), q=%p (%dx%d)\n",
+	    (void *) rset->R, rset->k, nc, (void *) rset->q, rset->k, 1);
 #endif
 
     if (rset->R == NULL || rset->q == NULL) {
@@ -854,6 +855,27 @@ static int pick_apart (gretl_restriction *r, const char *s,
     return 0;
 }
 
+static int bbit_trailing_garbage (const char *s)
+{
+    if (*s) {
+	while (isspace(*s)) s++;
+	if (*s) {
+	    if (*s == '/') {
+		gretl_errmsg_sprintf(_("%s: division not allowed here"), s-1);
+	    }
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+/* We got a leading letter ('a' or 'b') and now we parse
+   what follows.  The simplest case is just a number, giving
+   e.g., "b1".  Then we assess the validity of what we've
+   found.
+*/
+
 static int parse_b_bit (gretl_restriction *r, const char *s, 
 			int *eq, int *bnum,
 			const DATAINFO *pdinfo)
@@ -861,7 +883,12 @@ static int parse_b_bit (gretl_restriction *r, const char *s,
     int err = E_PARSE;
 
     if (isdigit((unsigned char) *s)) {
-	sscanf(s, "%d", bnum);
+	char *test;
+
+	*bnum = strtol(s, &test, 10);
+	if (bbit_trailing_garbage(test)) {
+	    return err;
+	}
 	if (r->type == GRETL_OBJ_VAR) {
 	    *eq = EQN_UNSPEC;
 	}
@@ -872,8 +899,8 @@ static int parse_b_bit (gretl_restriction *r, const char *s,
 
     if (*bnum < 1) {
 	if (*gretl_errmsg == '\0') {
-	    sprintf(gretl_errmsg, _("Coefficient number (%d) is out of range"), 
-		    *bnum);
+	    gretl_errmsg_sprintf(_("Coefficient number (%d) is out of range"), 
+				 *bnum);
 	}
 	err = 1;
     } else {
@@ -888,8 +915,8 @@ static int parse_b_bit (gretl_restriction *r, const char *s,
 	    err = E_PARSE;
 	}
     } else if (*eq < 1) {
-	sprintf(gretl_errmsg, _("Equation number (%d) is out of range"), 
-		*eq);
+	gretl_errmsg_sprintf(_("Equation number (%d) is out of range"), 
+			     *eq);
 	err = 1;
     } else {
 	*eq -= 1; /* convert to zero base */
@@ -912,9 +939,16 @@ parse_coeff_chunk (gretl_restriction *r, const char *s, double *x,
 
     while (isspace((unsigned char) *s)) s++;
 
+#if RDEBUG
+    fprintf(stderr, "parse_coeff_chunk: s='%s'\n", s);
+#endif
+
     if (ok_letter(r, *s)) {
 	*letter = *s;
 	s++;
+#if RDEBUG
+	fprintf(stderr, " first branch: s='%s'\n", s);
+#endif
 	err = parse_b_bit(r, s, eq, bnum, pdinfo);
 	*x = 1.0;
     } else if (sscanf(s, "%lf", x)) {
@@ -928,8 +962,8 @@ parse_coeff_chunk (gretl_restriction *r, const char *s, double *x,
 	}
     }
 
-    if (err && *gretl_errmsg == '\0') {
-	sprintf(gretl_errmsg, _("parse error in '%s'\n"), s0);
+    if (err) {
+	gretl_errmsg_sprintf(_("parse error in '%s'\n"), s0);
     } 
 
 #if RDEBUG
@@ -1577,7 +1611,7 @@ restriction_set_start (const char *line, gretlopt opt, int *err)
 #endif
 
     if (!strncmp(line, "restrict", 8)) {
-	name = get_system_name_from_line(line);
+	name = get_system_name_from_line(line, SYSNAME_RST);
     }
 
     if (name != NULL) {
@@ -1766,14 +1800,14 @@ restriction_set_print_result (gretl_restriction *rset,
 
     if (asym) {
 	rset->code = GRETL_STAT_WALD_CHISQ;
-	rset->pval = chisq_cdf_comp(rset->test, rset->k);
+	rset->pval = chisq_cdf_comp(rset->k, rset->test);
 	pprintf(prn, "\n%s: %s(%d) = %g, ", _("Test statistic"), 
 		(robust)? _("Robust chi^2"): "chi^2",
 		rset->k, rset->test);
     } else {
 	rset->code = GRETL_STAT_F;
 	rset->test /= rset->k;
-	rset->pval = snedecor_cdf_comp(rset->test, rset->k, pmod->dfd);
+	rset->pval = snedecor_cdf_comp(rset->k, pmod->dfd, rset->test);
 	pprintf(prn, "\n%s: %s(%d, %d) = %g, ", _("Test statistic"), 
 		(robust)? _("Robust F"): "F",
 		rset->k, pmod->dfd, rset->test);
@@ -1817,14 +1851,16 @@ test_restriction_set (gretl_restriction *rset, PRN *prn)
 #endif
 
     err = check_R_matrix(rset->R);
-    if (err) {
-	goto bailout;
+
+    if (!err) {
+	b = gretl_coeff_vector_from_model(pmod, rset->mask, &err);
     }
 
-    b = gretl_coeff_vector_from_model(pmod, rset->mask);
-    vcv = gretl_vcv_matrix_from_model(pmod, rset->mask);
-    if (b == NULL || vcv == NULL) {
-	err = E_ALLOC;
+    if (!err) {
+	vcv = gretl_vcv_matrix_from_model(pmod, rset->mask, &err);
+    }
+
+    if (err) {
 	goto bailout;
     }
 
@@ -2111,6 +2147,10 @@ gretl_sum_test (const int *list, MODEL *pmod, DATAINFO *pdinfo,
 	return E_NOTIMP;
     }
 
+    if (exact_fit_check(pmod, prn)) {
+	return 0;
+    }
+
     r = restriction_set_new(pmod, GRETL_OBJ_EQN, OPT_Q | OPT_C);
     if (r == NULL) {
 	return 1;
