@@ -236,7 +236,7 @@ int gretl_string_table_print (gretl_string_table *st, DATAINFO *pdinfo,
     if (st == NULL) return E_DATA;
 
     strcpy(stname, "string_table.txt");
-    gretl_path_prepend(stname, gretl_user_dir());
+    gretl_path_prepend(stname, gretl_work_dir());
 
     fp = gretl_fopen(stname, "w");
     if (fp == NULL) {
@@ -296,7 +296,8 @@ static saved_string *saved_strings;
 
 static saved_string built_ins[] = {
     { "gretldir", 0, NULL },
-    { "userdir",  0, NULL },
+    { "dotdir",   0, NULL },
+    { "workdir",  0, NULL },
     { "gnuplot",  0, NULL },
     { "x12a",     0, NULL },
     { "x12adir",  0, NULL },
@@ -374,10 +375,9 @@ static saved_string *get_saved_string_by_name (const char *name,
     return NULL;
 }
 
-/* for use in the context of the "printf" and "sprintf" commands:
-   access a saved string by name */
+/* access a saved string by name */
 
-char *get_named_string (const char *name)
+char *get_string_by_name (const char *name)
 {
     int i, n, d;
 
@@ -405,44 +405,6 @@ char *get_named_string (const char *name)
     return NULL;
 }
 
-static int append_to_saved_string (const char *name, char **s)
-{
-    saved_string *str;
-    char *tmp;
-    int n;
-
-    str = get_saved_string_by_name(name, NULL);
-    if (str == NULL) {
-	return E_UNKVAR;
-    }
-
-    if (str->s != NULL) {
-	n = strlen(str->s) + strlen(*s) + 1;
-    } else {
-	n = strlen(*s) + 1;
-    }
-
-    tmp = malloc(n);
-
-    if (tmp == NULL) {
-	return E_ALLOC;
-    }
-
-    if (str->s != NULL) {
-	strcpy(tmp, str->s);
-	free(str->s);
-    } else {
-	*tmp = '\0';
-    }
-
-    strcat(tmp, *s);
-    free(*s);
-    *s = NULL;
-    str->s = tmp;
-    
-    return 0;
-}
-
 static saved_string *add_named_string (const char *name)
 {
     int n = n_saved_strings;
@@ -491,6 +453,61 @@ int add_string_as (const char *s, const char *name)
     return err;
 }
 
+static int destroy_saved_string (saved_string *S)
+{
+    int i, j, ns = n_saved_strings - 1;
+    int err = 0;
+
+    for (i=0; i<n_saved_strings; i++) {
+	if (&saved_strings[i] == S) {
+	    free(saved_strings[i].s);
+	    for (j=i; j<ns; j++) {
+		saved_strings[j] = saved_strings[j+1];
+	    }
+	    break;
+	}
+    } 
+
+    if (ns == 0) {
+	free(saved_strings);
+	saved_strings = NULL;
+    } else {
+	saved_string *tmp = realloc(saved_strings, ns * sizeof *tmp);
+
+	if (tmp == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    saved_strings = tmp;
+	}
+    }
+
+    n_saved_strings = ns;
+
+    return err;
+}
+
+int delete_saved_string (const char *name, PRN *prn)
+{
+    saved_string *S;
+    int err, builtin = 0;
+
+    S = get_saved_string_by_name(name, &builtin);
+    if (S == NULL) {
+	err = E_UNKVAR;
+    } else if (builtin) {
+	sprintf(gretl_errmsg, _("You cannot delete '%s'"), name);
+	err = E_DATA;
+    } else {
+	err = destroy_saved_string(S);
+	if (!err && prn != NULL && gretl_messages_on()) {
+	    pprintf(prn, _("Deleted string %s"), name);
+	    pputc(prn, '\n');
+	}
+    }
+
+    return err;
+}
+
 void saved_strings_cleanup (void)
 {
     int i;
@@ -559,6 +576,7 @@ int destroy_saved_strings_at_level (int d)
     return err;
 }
 
+#if 0
 static void copy_unescape (char *targ, const char *src, int n)
 {
     int c, i = 0;
@@ -579,97 +597,11 @@ static void copy_unescape (char *targ, const char *src, int n)
 
     *targ = '\0';
 }
-
-static char *retrieve_string_var (const char **pline, 
-				  const char *s,
-				  int *err)
-{
-    char sname[VNAMELEN] = {0};
-    char *p, *ret = NULL;
-    int n, spn;
-
-    spn = n = gretl_varchar_spn(s);
-    if (n >= VNAMELEN) {
-	n = VNAMELEN - 1;
-    }
-
-    strncat(sname, s, n);
-    p = get_named_string(sname);
-
-    if (p == NULL) {
-	*err = E_UNKVAR;
-    } else {
-	ret = gretl_strdup(p);
-	if (ret == NULL) {
-	    *err = E_ALLOC;
-	} else {
-	    *pline = s + spn;
-	}
-    }
-
-    return ret;
-}
-
-#ifdef WIN32
-
-/* FIXME */
+#endif
 
 static int shell_grab (const char *arg, char **sout)
 {
-    gchar *serr = NULL;
-    GError *gerr = NULL;
-    char *wdir;
-    int status, err = 0;
- 
-    if (!libset_get_bool(SHELL_OK)) {
-	strcpy(gretl_errmsg, _("The shell command is not activated."));
-	return 1;
-    }
-
-    wdir = get_shelldir();
-    if (wdir != NULL && chdir(wdir)) {
-	sprintf(gretl_errmsg, _("Couldn't open %s"), wdir);
-	return E_FOPEN;
-    }
-
-    arg += strspn(arg, " \t");
-
-    g_spawn_command_line_sync(arg, sout, &serr, 
-			      &status, &gerr);
-
-    if (gerr != NULL) {
-	gretl_errmsg_set(gerr->message);
-	g_error_free(gerr);
-	err = 1;
-    }
-
-    if (serr != NULL) {
-	g_free(serr);
-    }
-
-    if (sout != NULL && *sout != NULL) {
-	/* trim trailing newline */
-	int n = strlen(*sout);
-
-	if ((*sout)[n-1] == '\n') {
-	    (*sout)[n-1] = '\0';
-	}
-    }
-
-    return err;
-}
-
-#else
-
-static int shell_grab (const char *arg, char **sout)
-{
-    gchar *serr = NULL;
-    GError *gerr = NULL;
-    gchar *argv[5];
-    const char *theshell, *namep;
-    char shellnam[40];
-    char *wdir;
-    int status, err = 0;
+    int err = 0;
     
     if (arg == NULL || *arg == '\0') {
 	return E_PARSE;
@@ -680,55 +612,7 @@ static int shell_grab (const char *arg, char **sout)
 	return 1;
     }
 
-    wdir = get_shelldir();
-    if (wdir == NULL || *wdir == '\0') {
-	wdir = NULL;
-    }
-
-    arg += strspn(arg, " \t");
-    theshell = getenv("SHELL"); 
-
-    if (theshell == NULL) {
-# ifdef HAVE_PATHS_H
-	theshell =_PATH_BSHELL;
-# else
-	theshell = "/bin/sh"; 
-# endif
-    }
-
-    namep = strrchr(theshell, '/');
-    if (namep == NULL) {
-	namep = theshell;
-    }
-
-    strcpy(shellnam, "-");
-    strcat(shellnam, ++namep);
-    if (strcmp(namep, "sh") != 0) {
-	shellnam[0] = '+';
-    }
-
-    argv[0] = g_strdup(theshell);
-    argv[1] = shellnam;
-    argv[2] = g_strdup("-c");
-    argv[3] = g_strdup(arg);
-    argv[4] = NULL;
-
-    g_spawn_sync(wdir, argv, NULL, 0, NULL, NULL,
-		 sout, &serr, &status, &gerr); 
-
-    g_free(argv[0]);
-    g_free(argv[2]);
-    g_free(argv[3]);
-
-    if (gerr != NULL) {
-	gretl_errmsg_set(gerr->message);
-	g_error_free(gerr);
-	err = 1;
-    }
-
-    if (serr != NULL) {
-	g_free(serr);
-    }
+    gretl_shell_grab(arg, sout);
 
     if (sout != NULL && *sout != NULL) {
 	/* trim trailing newline */
@@ -742,65 +626,26 @@ static int shell_grab (const char *arg, char **sout)
     return err;
 }
 
-#endif
-
-static char *gretl_backtick (const char **pline, int *err)
+char *gretl_backtick (const char *arg, int *err)
 {
-    const char *s = *pline + 2;
-    char *arg = NULL;
     char *val = NULL;
-    int n = 0, pcount = 1;
 
-    while (*s && pcount) {
-	if (*s == '(') pcount++;
-	else if (*s == ')') pcount--;
-	s++;
-	n++;
-    }
-
-    if (pcount || n < 2) {
-	*err = E_PARSE;
-	return NULL;
-    }
-
-    arg = gretl_strndup(*pline + 2, n - 1);
-    if (arg == NULL) {
-	*err = E_ALLOC;
-	return NULL;
-    }
-
-    fprintf(stderr, "arg = '%s'\n", arg);
-
-    *pline = s;
     *err = shell_grab(arg, &val);
 
-    free(arg);
+    if (!*err && val == NULL) {
+	val = gretl_strdup("");
+	if (val == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
     
     return val;
 }
 
-static char *gretl_getenv (const char **pline, int *err)
+char *gretl_getenv (const char *key, int *err)
 {
-    const char *s = strchr(*pline, '"') + 1;
-    char *p = strchr(s, '"');
-    char *key = NULL;
     char *test = NULL;
     char *val = NULL;
-    int n;
-
-    if (p == NULL || *(p+1) != ')') {
-	*err = E_PARSE;
-	return NULL;
-    }
-
-    n = strcspn(s, "\"");
-    key = gretl_strndup(s, n);
-    if (key == NULL) {
-	*err = E_ALLOC;
-	return NULL;
-    }
-
-    *pline = p + 2;
 
     test = getenv(key);
     if (test == NULL) {
@@ -813,95 +658,68 @@ static char *gretl_getenv (const char **pline, int *err)
 	*err = E_ALLOC;
     }
 
-    free(key);
-    
     return val;
 }
 
-static char *retrieve_arg_name (const char **pline, int *err)
+#if 0
+static char *strim (char *s)
 {
-    char argvar[VNAMELEN];
+    int n;
+
+    tailstrip(s);
+    n = strlen(s);
+    if (n > 0 && s[n-1] == '"') {
+	s[n-1] = '\0';
+    }
+
+    return s;
+}
+#endif
+
+char *retrieve_date_string (int t, const DATAINFO *pdinfo, int *err)
+{
+    char datestr[OBSLEN] = {0};
     char *ret = NULL;
 
-    if (sscanf(*pline, "argname(%15[^)])", argvar) == 1) {
-	*pline = strchr(*pline, ')') + 1;
-	ret = gretl_func_get_arg_name(argvar);
+    if (t > 0 && t <= pdinfo->n) {
+	ntodate(datestr, t - 1, pdinfo);
+	ret = gretl_strdup(datestr);
+	if (ret == NULL) {
+	    *err = E_ALLOC;
+	}
     } else {
-	*err = E_PARSE;
+	*err = E_DATA;
     }
 
     return ret;
 }
 
-static char *get_string_element (const char **pline, int *err)
+char *retrieve_file_content (const char *fname, int *err)
 {
-    const char *line = *pline;
-    const char *s;
-    char *cpy;
-    int closed = 0;
-    int bs = 0;
-    int n = 0;
+    char *ret = NULL;
 
-    line += strspn(line, " \t");
-    *pline = line;
-
-    if (*line == '@') {
-	/* should be saved string variable */
-	return retrieve_string_var(pline, line + 1, err);
-    }
-
-    if (!strncmp(line, "getenv(\"", 8)) {
-	return gretl_getenv(pline, err);
-    }
-
-    if (!strncmp(line, "argname(", 8)) {
-	return retrieve_arg_name(pline, err);
-    }
-
-    if (!strncmp(line, "$(", 2)) {
-	return gretl_backtick(pline, err);
-    }    
-
-    if (*line != '"') {
-	*err = E_PARSE;
-	return NULL;
-    }
-
-    line++;
-    s = line;
-    while (*s) {
-	/* allow for escaped quotes */
-	if (*s == '\\') {
-	    bs++;
-	} else if (*s == '"' && (bs % 2 == 0)) {
-	    closed = 1;
-	    break;
-	}
-	if (*s != '\\') {
-	    bs = 0;
-	}
-	s++;
-	n++;
-    }
-
-    if (!closed) {
-	fprintf(stderr, "quote not closed\n");
-	*err = E_PARSE;
-	return NULL;
-    }
-
-    cpy = malloc(n + 1);
-
-    if (cpy == NULL) {
-	*err = E_ALLOC;
+    if (fname == NULL || *fname == '\0') {
+	*err = E_DATA;
     } else {
-	copy_unescape(cpy, line, n);
-    }
+	char fullname[FILENAME_MAX];
+	GError *gerr = NULL;
+	gsize len = 0;
+	
 
-    s++; /* eat closing quote */
-    *pline = s;
+	*fullname = '\0';
+	strncat(fullname, fname, FILENAME_MAX - 1);
+	addpath(fullname, NULL, 0);
 
-    return cpy;
+	g_file_get_contents(fullname, &ret, &len, &gerr);
+
+	if (gerr != NULL) {
+	    gretl_errmsg_set(gerr->message);
+	    g_error_free(gerr);
+	    *err = E_FOPEN;
+	}
+    } 
+
+    return ret;
 }
 
 int string_is_defined (const char *sname)
@@ -915,142 +733,68 @@ int string_is_defined (const char *sname)
     
     str = get_saved_string_by_name(sname, &builtin);
 
-    return (str != NULL && str->s != NULL && str->s[0]);
+    return (str != NULL && str->s != NULL);
 }
 
-/* for use in "sprintf" command */
-
-int save_named_string (const char *name, const char *s, PRN *prn)
+int is_user_string (const char *sname)
 {
-    saved_string *str;
-    int builtin = 0;
+    int i, d;
 
-    if (s == NULL) {
-	return E_DATA;
+    if (*sname == '@' && *(sname + 1) != '@') {
+	sname++;
     }
 
-    str = get_saved_string_by_name(name, &builtin);
-    
-    if (str != NULL && builtin) {
-	pprintf(prn, "You cannot overwrite '%s'\n", name);
-	return E_DATA;
-    }
+    d = gretl_function_depth();
 
-    if (str == NULL) {
-	str = add_named_string(name);
-	if (str == NULL) {
-	    return E_ALLOC;
-	}
-    }
-
-    if (str->s != NULL) {
-	free(str->s);
-    }
-
-    str->s = gretl_strdup(s);
-    if (str->s == NULL) {
-	return E_ALLOC;
-    }
-
-    if (gretl_messages_on()) {
-	if (str->s[0] == '\0') {
-	    pprintf(prn, "Saved empty string as '%s'\n", name);
-	} else {
-	    pprintf(prn, "Saved string as '%s'\n", name);
+    for (i=0; i<n_saved_strings; i++) {
+	if (saved_strings[i].level == d &&
+	    !strcmp(sname, saved_strings[i].name)) {
+	    return 1;
 	}
     }
 
     return 0;
 }
 
-/* respond to commands of the forms:
-
-     string <name> = "<s1>" "<s2>" ... "<sn>"
-     string <name> += "<s2>" "<s2>"  ... "<sn>"
-*/
-
-int process_string_command (const char *line, PRN *prn)
+int save_named_string (const char *name, const char *s, PRN *prn)
 {
     saved_string *str;
-    char *s1 = NULL;
-    char targ[VNAMELEN];
     int builtin = 0;
-    int n, add = 0;
+    int add = 0;
     int err = 0;
 
-    /* skip "string" plus any following space */
-    line += 6;
-    line += strspn(line, " \t");
-
-    n = gretl_varchar_spn(line);
-    if (n == 0 || n >= VNAMELEN) {
-	return E_PARSE;
-    }
-
-    *targ = '\0';
-    strncat(targ, line, n);
-    line += n;
-
-    /* eat any space before operator */
-    line += strspn(line, " \t");
-
-    if (*line == '\0') {
-	/* just a call to echo an existing string? */
-	str = get_saved_string_by_name(targ, &builtin);
-	if (str == NULL) {
-	    return E_UNKVAR;
-	} else {
-	    pprintf(prn, " %s\n", str->s);
-	    return 0;
-	}
-    }
-
-    /* operator must be '=' or '+=' */
-    if (!strncmp(line, "+=", 2)) {
-	add = 1;
-    } else if (*line != '=') {
-	return E_PARSE;
-    }
-
-    line += (add)? 2 : 1;
-
-    /* set up the target */
-    str = get_saved_string_by_name(targ, &builtin);
+    str = get_saved_string_by_name(name, &builtin);
 
     if (str != NULL && builtin) {
-	pprintf(prn, "You cannot overwrite '%s'\n", targ);
+	if (prn == NULL) {
+	    sprintf(gretl_errmsg, _("You cannot overwrite '%s'\n"), name);
+	} else {
+	    pprintf(prn, _("You cannot overwrite '%s'\n"), name);
+	}
 	return E_DATA;
     }	
 
-    if (str == NULL) {
-	if (add) {
-	    return E_UNKVAR;
-	} else {
-	    str = add_named_string(targ);
-	    if (str == NULL) {
-		return E_ALLOC;
-	    }
-	}
-    } else if (!add) {
+    if (str != NULL) {
 	free(str->s);
 	str->s = NULL;
-    }
-
-    /* add strings(s) to target */
-    while (!err && *line != '\0') {
-	s1 = get_string_element(&line, &err);
-	if (!err) {
-	    err = append_to_saved_string(targ, &s1);
+    } else {
+	str = add_named_string(name);
+	if (str == NULL) {
+	    return E_ALLOC;
 	}
+	add = 1;
+    } 
+
+    str->s = gretl_strdup(s);
+    if (str->s == NULL) {
+	err = E_ALLOC;
     }
 
-    if (err) {
-	free(s1);
-    } else if (gretl_messages_on()) {
-	if (str->s[0] == '\0') {
-	    pprintf(prn, "Saved empty string as '%s'\n", targ);
+    if (prn != NULL && !err && gretl_messages_on() && *s != '\0') {
+	if (add) {
+	    pprintf(prn, _("Generated string %s\n"), name); 
 	} else {
-	    pprintf(prn, "Saved string as '%s'\n", targ);
+	    pprintf(prn, _("Replaced string %s\n"), name);
 	}
     }
 
@@ -1133,7 +877,7 @@ static void too_long (void)
 
 #define var_context(s,i) (i > 8 && !strncmp(s - 9, "isstring(", 9))
 
-int substitute_named_strings (char *line)
+int substitute_named_strings (char *line, int *subst)
 {
     char sname[VNAMELEN];
     int len = strlen(line);
@@ -1146,11 +890,17 @@ int substitute_named_strings (char *line)
 	return 0;
     }
 
-    if (!strncmp(line, "string", 6)) {
-	/* when defining a string, let @foo be handled as a variable */
+    /* when using the genr apparatus for a string, let @foo be handled 
+       as a variable (FIXME?) */
+    if (!strncmp(line, "string ", 7)) {
 	return 0;
     }
 
+    if (!strncmp(line, "sscanf", 6)) {
+	/* when scanning, let @foo be handled as a variable */
+	return 0;
+    }    
+
     if (!strncmp(line, "printf", 6) || !strncmp(line, "sprintf", 7)) {
 	pf = 1;
 	s = strchr(s, '"');
@@ -1203,6 +953,7 @@ int substitute_named_strings (char *line)
 			len += m - (n + 1);
 			s += m - 1;
 			i += m - 1;
+			*subst = 1;
 		    }
 		    if (freeit) {
 			free(sub);
