@@ -40,20 +40,34 @@ int save_named_string (const char *name, const char *s, PRN *prn);
 
 int add_string_as (const char *s, const char *name);
 
-char *get_named_string (const char *name);
+char *get_string_by_name (const char *name);
 
-int process_string_command (const char *line, PRN *prn);
+int process_string_command (const char *line, 
+			    double ***pZ, DATAINFO *pdinfo,
+			    PRN *prn);
 
-int substitute_named_strings (char *line);
+int substitute_named_strings (char *line, int *subst);
 
 int string_is_defined (const char *sname);
 
+int is_user_string (const char *sname);
+
 void saved_strings_cleanup (void);
 
+int delete_saved_string (const char *name, PRN *prn);
+
 int destroy_saved_strings_at_level (int d);
 
 int is_codevar (const char *s);
 
 int set_codevars (const char *s);
 
+char *gretl_getenv (const char *key, int *err);
+
+char *retrieve_date_string (int t, const DATAINFO *pdinfo, int *err);
+
+char *retrieve_file_content (const char *fname, int *err);
+
+char *gretl_backtick (const char *arg, int *err);
+
 #endif /* GRETL_STRING_TABLE_H */
