@@ -28,6 +28,7 @@
 #include "loop_private.h"
 #include "texprint.h"
 #include "gretl_string_table.h"
+#include "forecast.h"
 
 #include <errno.h>
 #include <time.h>
@@ -385,9 +386,8 @@ int gretl_isdiscrete (int t1, int t2, const double *x)
  *
  * Checks the variable @x over the range @t1 to @t2 for its
  * suitability as the dependent variable in an ordered probit
- * analysis.  The criterion used is that the variable has a
- * minimum value of zero and has only non-negative integer 
- * values.
+ * analysis.  The criterion used is that the variable has 
+ * only non-negative integer values.
  * 
  * Returns: 1 if the test succeeds, otherwise 0.
  */
@@ -395,7 +395,6 @@ int gretl_isdiscrete (int t1, int t2, const double *x)
 int gretl_is_oprobit_ok (int t1, int t2, const double *x)
 {
     int t, n = 0, d = 1;
-    double xmin = 1;
 
     for (t=t1; t<=t2; t++) {
 	if (na(x[t])) {
@@ -406,12 +405,9 @@ int gretl_is_oprobit_ok (int t1, int t2, const double *x)
 	    d = 0;
 	    break;
 	}
-	if (x[t] < xmin) {
-	    xmin = x[t];
-	}
     }
 
-    return (d && xmin == 0 && n > 0);
+    return (d > 0 && n > 0);
 }
 
 /**
@@ -1327,13 +1323,10 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
     case AR:
 	*tmpmod = ar_func(cmd.list, pZ, pdinfo, OPT_NONE, NULL);
 	break;
-    case CORC:
-    case HILU:
-    case PWE:
-	rho = estimate_rho(cmd.list, pZ, pdinfo, cmd.ci, 
-			   &err, cmd.opt, NULL);
+    case AR1:
+	rho = estimate_rho(cmd.list, pZ, pdinfo, cmd.opt, NULL, &err);
 	if (!err) {
-	    *tmpmod = ar1_lsq(cmd.list, pZ, pdinfo, cmd.ci, 0, rho);
+	    *tmpmod = ar1_lsq(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt, rho);
 	}
 	break;
     case HSK:
@@ -1385,12 +1378,17 @@ double get_xvalue (int i, const double **Z, const DATAINFO *pdinfo)
 
 #ifndef WIN32
 
-static int font_not_found (const char *s)
+static int non_fatal (const char *s)
 {
-    /* "Could not find/open font when opening font X, using default" 
-       or "gnuplot_x11: Some character sets not available" */
+    /* 
+       "Could not find/open font when opening font X, using default" 
+       "gnuplot_x11: Some character sets not available" 
+       "Warning: empty y2 range..."
+    */
 
-    if (strstr(s, "using default") || strstr(s, "character sets not available")) {
+    if (strstr(s, "using default") || 
+	strstr(s, "character sets not available") ||
+	strstr(s, "Warning: empty ")) {
 	return 1;
     } else {
 	return 0;
@@ -1422,7 +1420,7 @@ int gretl_spawn (char *cmdline)
 	ret = 1;
     } else if (errout && *errout) {
 	fprintf(stderr, "stderr: '%s'\n", errout);
-	if (!font_not_found(errout)) {
+	if (!non_fatal(errout)) {
 	    gretl_errmsg_set(errout);
 	    fprintf(stderr, "gretl_errmsg: '%s'\n", gretl_errmsg);
 	    ret = 1;
@@ -1512,27 +1510,32 @@ void libgretl_init (void)
     set_gretl_tex_preamble(); 
 }
 
-void libgretl_session_cleanup (void)
+void libgretl_session_cleanup (int mode)
 {
     gretl_saved_objects_cleanup();
     gretl_transforms_cleanup();
     gretl_lists_cleanup();
-    destroy_user_matrices();
     gretl_plotx(NULL);
+
+    if (mode != SESSION_PRESERVE_MATRICES) {
+	destroy_user_matrices();
+    }
 }
 
 void libgretl_cleanup (void)
 {
     const char *p;
 
-    libgretl_session_cleanup();
+    libgretl_session_cleanup(SESSION_CLEAR_FULL);
 
     gretl_rand_free();
     gretl_functions_cleanup();
     libset_cleanup();
     gretl_command_hash_cleanup();
+    gretl_function_hash_cleanup();
     saved_strings_cleanup();
     lapack_mem_free();
+    forecast_matrix_cleanup();
 
     p = strstr(gretl_plotfile(), "gpttmp");
     if (p != NULL) {
