@@ -147,21 +147,37 @@ void cli_read_registry (char *callname, PATHS *ppaths)
     char valstr[MAXLEN];
     char dbproxy[21];
     int use_proxy = 0;
+    char *tmp;
     int drive = callname[0];
 
     ppaths->gretldir[0] = '\0';
-
     read_reg_val(HKEY_LOCAL_MACHINE, "gretl", "gretldir", ppaths->gretldir);
     if (ppaths->gretldir[0] == '\0') {
 	sprintf(ppaths->gretldir, "%c:\\userdata\\gretl\\", drive);
     }
 
-    ppaths->userdir[0] = '\0';
-    read_reg_val(HKEY_CURRENT_USER, "gretl", "userdir", ppaths->userdir);
-    if (ppaths->userdir[0] == '\0') {
-	sprintf(ppaths->userdir, "%c:\\userdata\\gretl\\user\\", drive);
+    ppaths->workdir[0] = '\0';
+    read_reg_val(HKEY_CURRENT_USER, "gretl", "userdir", ppaths->workdir);
+    if (ppaths->workdir[0] == '\0') {
+	tmp = mydocs_path();
+
+	if (tmp != NULL) {
+	    sprintf(ppaths->workdir, "%s\\gretl\\", tmp);
+	    free(tmp);
+	} else {
+	    sprintf(ppaths->workdir, "%c:\\userdata\\gretl\\user\\", drive);
+	}
     }
 
+    ppaths->dotdir[0] = '\0';
+    tmp = appdata_path();
+    if (tmp != NULL) {
+	sprintf(ppaths->dotdir, "%s\\gretl\\", tmp);
+	free(tmp);
+    } else {
+	sprintf(ppaths->dotdir, "%c:\\userdata\\gretl\\user\\", drive);
+    }    
+
     ppaths->gnuplot[0] = '\0';
     read_reg_val_with_fallback(HKEY_LOCAL_MACHINE, HKEY_CLASSES_ROOT,
 			       "gretl", "gnuplot", ppaths->gnuplot);
@@ -209,6 +225,7 @@ void cli_read_registry (char *callname, PATHS *ppaths)
 	libset_set_bool(SHELL_OK, 0);
     }
 
+    gretl_set_paths(ppaths, OPT_NONE);
     gretl_www_init(ppaths->dbhost, dbproxy, use_proxy);
 }
 
@@ -311,6 +328,7 @@ static char *win_special_path (int folder)
     LPITEMIDLIST id_list;
     DWORD result;
     LPMALLOC allocator;
+    char *ret = NULL;
 
     if (SHGetSpecialFolderLocation(NULL, folder | CSIDL_FLAG_CREATE, 
 				   &id_list) != S_OK) {
@@ -319,12 +337,16 @@ static char *win_special_path (int folder)
 
     result = SHGetPathFromIDList(id_list, dpath);
 
+    if (result) {
+	ret = gretl_strdup(dpath);
+    }
+
     if (SHGetMalloc(&allocator) == S_OK) {
 	allocator->lpVtbl->Free(allocator, id_list);
 	allocator->lpVtbl->Release(allocator);
     }
 
-    return (result == TRUE)? gretl_strdup(dpath) : NULL;
+    return ret;
 }
 
 char *desktop_path (void)
@@ -358,16 +380,22 @@ static char *compose_command_line (const char *arg)
     return cmdline;
 }
 
-#if 1
-
 #define BUFSIZE 4096 
  
-static int read_from_pipe (HANDLE hwrite, HANDLE hread, PRN *prn) 
+static int read_from_pipe (HANDLE hwrite, HANDLE hread, 
+			   char **sout, PRN *inprn) 
 { 
     DWORD dwread;
     CHAR buf[BUFSIZE];
+    PRN *prn;
     int ok;
 
+    if (sout != NULL) {
+	prn = gretl_print_new(GRETL_PRINT_BUFFER, NULL);
+    } else {
+	prn = inprn;
+    }
+
     /* close the write end of the pipe */
     ok = CloseHandle(hwrite);
     
@@ -375,14 +403,21 @@ static int read_from_pipe (HANDLE hwrite, HANDLE hread, PRN *prn)
 	fputs("Closing handle failed\n", stderr); 
     } else {
 	/* read output from the child process */
-	for (;;) { 
+	while (1) { 
 	    memset(buf, '\0', BUFSIZE);
-	    if (!ReadFile(hread, buf, BUFSIZE, &dwread, 
-			  NULL) || dwread == 0) break;
+	    if (!ReadFile(hread, buf, BUFSIZE, &dwread, NULL) || 
+		dwread == 0) {
+		break;
+	    }
 	    pputs(prn, buf);
 	} 
     }
 
+    if (sout != NULL) {
+	*sout = gretl_print_steal_buffer(prn);
+	gretl_print_destroy(prn);
+    }
+
     return ok;
 } 
 
@@ -391,20 +426,14 @@ run_child_with_pipe (const char *arg, HANDLE hwrite, HANDLE hread)
 { 
     PROCESS_INFORMATION pinfo; 
     STARTUPINFO sinfo;
-    char *wdir, *cmdline = NULL;
+    char *cmdline = NULL;
     int ok;
     
-    wdir = get_shelldir();
-    if (wdir != NULL && chdir(wdir)) {
-	sprintf(gretl_errmsg, _("Couldn't open %s"), wdir);
-	return E_FOPEN;
-    }
-
     cmdline = compose_command_line(arg);
  
-    ZeroMemory(&pinfo, sizeof(PROCESS_INFORMATION));
- 
+    ZeroMemory(&pinfo, sizeof pinfo);
     ZeroMemory(&sinfo, sizeof sinfo);
+
     sinfo.cb = sizeof sinfo;
     sinfo.hStdError = hwrite;
     sinfo.hStdOutput = hwrite;
@@ -417,16 +446,15 @@ run_child_with_pipe (const char *arg, HANDLE hwrite, HANDLE hread)
 		       NULL,          // process security attributes 
 		       NULL,          // primary thread security attributes 
 		       TRUE,          // handles are inherited 
-		       CREATE_NEW_CONSOLE | HIGH_PRIORITY_CLASS,
+		       CREATE_NO_WINDOW,
 		       NULL,          // use parent's environment 
-		       wdir,          
+		       get_shelldir(),          
 		       &sinfo,
 		       &pinfo);
    
     if (!ok) {
 	win_show_last_error();
     } else {
-	/* WaitForSingleObject(pinfo.hProcess, INFINITE); */
 	CloseHandle(pinfo.hProcess);
 	CloseHandle(pinfo.hThread);
     }
@@ -436,38 +464,36 @@ run_child_with_pipe (const char *arg, HANDLE hwrite, HANDLE hread)
     return ok;
 }
 
-int run_cmd_new (const char *arg, PRN *prn) 
+static int run_cmd_with_pipes (const char *arg, char **sout, PRN *prn) 
 { 
     HANDLE hread, hwrite;
     SECURITY_ATTRIBUTES sa; 
     int ok; 
  
-    /* Set the bInheritHandle flag so pipe handles are inherited */
+    /* set the bInheritHandle flag so pipe handles are inherited */
     sa.nLength = sizeof(SECURITY_ATTRIBUTES); 
     sa.bInheritHandle = TRUE; 
     sa.lpSecurityDescriptor = NULL; 
 
-    /* Create a pipe for the child process's STDOUT */ 
+    /* create pipe for the child process's STDOUT */ 
     ok = CreatePipe(&hread, &hwrite, &sa, 0);
 
     if (!ok) {
 	win_show_last_error();
     } else {
-	/* Ensure that the read handle to the child process's pipe for 
+	/* ensure that the read handle to the child process's pipe for 
 	   STDOUT is not inherited */
 	SetHandleInformation(hread, HANDLE_FLAG_INHERIT, 0);
 	ok = run_child_with_pipe(arg, hwrite, hread);
 	if (ok) {
-	    /* Read from child's output pipe */
-	    read_from_pipe(hwrite, hread, prn); 
+	    /* read from child's output pipe */
+	    read_from_pipe(hwrite, hread, sout, prn); 
 	}
     }
  
     return 0; 
 } 
 
-#endif
-
 static int run_cmd_wait (const char *cmd, PRN *prn)
 {
     STARTUPINFO si;
@@ -475,12 +501,6 @@ static int run_cmd_wait (const char *cmd, PRN *prn)
     char *cmdline = NULL;
     int ok, err = 0;
 
-#if 1
-    if (getenv("GRETL_SHELL_NEW")) {
-	return run_cmd_new(cmd, prn);
-    }
-#endif
-
     ZeroMemory(&si, sizeof si);
     ZeroMemory(&pi, sizeof pi);
 
@@ -510,6 +530,11 @@ static int run_cmd_wait (const char *cmd, PRN *prn)
     return err;
 }
 
+int gretl_shell_grab (const char *arg, char **sout)
+{
+    return run_cmd_with_pipes(arg, sout, NULL);
+}
+
 int gretl_shell (const char *arg, PRN *prn)
 {
     UINT winret;
@@ -539,6 +564,8 @@ int gretl_shell (const char *arg, PRN *prn)
 	if (winret <= 31) {
 	    err = 1;
 	}
+    } else if (getenv("GRETL_SHELL_NEW")) {
+	err = run_cmd_with_pipes(arg, NULL, prn);
     } else {
 	err = run_cmd_wait(arg, prn);
     } 
@@ -668,3 +695,46 @@ int win32_delete_dir (const char *path)
 
     return err;
 }
+
+char *slash_convert (char *str, int which)
+{
+    char *p;
+
+    if (str == NULL) {
+	return NULL;
+    }
+
+    p = str;
+    while (*p) {
+	if (which == FROM_BACKSLASH) {
+	    if (*p == '\\') *p = '/';
+	} else if (which == TO_BACKSLASH) {
+	    if (*p == '/') *p = '\\';
+	}
+	p++;
+    }
+
+    return str;
+}
+
+char *R_path_from_registry (void)
+{
+    char tmp[MAX_PATH] = {0}; 
+    char *ret = NULL;
+    int err;
+
+    err = read_reg_val(HKEY_LOCAL_MACHINE, "R-core\\R", "InstallPath", tmp);
+
+    if (err) {
+	err = read_reg_val(HKEY_LOCAL_MACHINE, "R", "InstallPath", tmp);
+    }
+
+    if (!err) {
+	strcat(tmp, "\\bin\\");
+	strcat(tmp, "Rterm.exe");
+	ret = gretl_strdup(tmp);
+    }
+
+    return ret;
+}
+
