@@ -22,6 +22,7 @@
 #include "gretl_panel.h"
 #include "gretl_func.h"
 #include "usermat.h"
+#include "dbread.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -29,6 +30,8 @@
 
 #undef XML_DEBUG
 
+#define GRETLDATA_VERSION "1.1"
+
 #ifdef WIN32
 # include <glib.h>
 static xmlDocPtr gretl_xmlParseFile (const char *fname)
@@ -402,14 +405,21 @@ void gretl_xml_put_matrix (const gretl_matrix *m, const char *name,
 	return;
     }
 
-    fprintf(fp, "<gretl-matrix name=\"%s\" rows=\"%d\" cols=\"%d\">\n", 
-	    name, m->rows, m->cols);
+    if (name == NULL) {
+	fprintf(fp, "<gretl-matrix rows=\"%d\" cols=\"%d\">\n", 
+		m->rows, m->cols);
+    } else {
+	fprintf(fp, "<gretl-matrix name=\"%s\" rows=\"%d\" cols=\"%d\">\n", 
+		name, m->rows, m->cols);
+    }
+
     for (i=0; i<m->rows; i++) {
 	for (j=0; j<m->cols; j++) {
 	    fprintf(fp, "%.15g ", gretl_matrix_get(m, i, j));
 	}
 	fputc('\n', fp);
     }
+
     fputs("</gretl-matrix>\n", fp); 
 }
 
@@ -504,12 +514,12 @@ int gretl_xml_get_prop_as_double (xmlNodePtr node, const char *tag,
     char *p, *s = (char *) xmlGetProp(node, (XUC) tag);
     int ret = 0;
 
+    *x = NADBL;
+
     if (s != NULL) {
 	p = s;
 	p += strspn(p, " \r\n");
-	if (!strncmp(p, "NA", 2)) {
-	    *x = NADBL;
-	} else {
+	if (strncmp(p, "NA", 2)) {
 	    *x = atof(p);
 	}
 	free(s);
@@ -760,6 +770,43 @@ int *gretl_xml_node_get_list (xmlNodePtr node, xmlDocPtr doc, int *err)
     return list;
 }
 
+/**
+ * gretl_xml_child_get_string:
+ * @node: XML node pointer.
+ * @doc: XML document pointer.
+ * @name: name of child node.
+ * @pstr: location to receive string.
+ * 
+ * Returns: 1 if a string is found and read successfully, 0
+ * otherwise.
+ */
+
+int gretl_xml_child_get_string (xmlNodePtr node, xmlDocPtr doc, 
+				const char *name, char **pstr)
+{
+    xmlNodePtr cur;
+    xmlChar *tmp;
+    int ret = 0;
+
+    *pstr = NULL;
+
+    cur = node->xmlChildrenNode;
+
+    while (cur != NULL) {
+	if (!xmlStrcmp(cur->name, (XUC) name)) {
+	    tmp = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+	    if (tmp != NULL) {
+		*pstr = (char *) tmp;
+		ret = 1;
+	    }
+	    break;
+	}
+	cur = cur->next;
+    }
+
+    return ret;
+}
+
 static const char *skip_to_next (const char *s)
 {
     s += strspn(s, " \r\n");
@@ -956,16 +1003,18 @@ char **gretl_xml_get_strings_array (xmlNodePtr node, xmlDocPtr doc,
 }
 
 /**
- * gretl_xml_get_matrix:
+ * xml_get_user_matrix:
  * @node: XML node pointer.
  * @doc: XML document pointer.
+ * @colnames: location to receive column strings, if present.
  * @err: location to receive error code.
  * 
  * Returns: allocated gretl matrix read from @node, or %NULL 
  * on failure.
  */
 
-gretl_matrix *gretl_xml_get_matrix (xmlNodePtr node, xmlDocPtr doc, int *err)
+static gretl_matrix *xml_get_user_matrix (xmlNodePtr node, xmlDocPtr doc, 
+					  char **colnames, int *err)
 {
     gretl_matrix *m = NULL;
     xmlChar *tmp;
@@ -1007,6 +1056,10 @@ gretl_matrix *gretl_xml_get_matrix (xmlNodePtr node, xmlDocPtr doc, int *err)
 	return NULL;
     }
 
+    if (colnames != NULL) {
+	*colnames = (char *) xmlGetProp(node, (XUC) "colnames");
+    }
+
     m = gretl_matrix_alloc(rows, cols);
     if (m == NULL) {
 	*err = E_ALLOC;
@@ -1046,30 +1099,48 @@ gretl_matrix *gretl_xml_get_matrix (xmlNodePtr node, xmlDocPtr doc, int *err)
     return m;
 }
 
+/**
+ * gretl_xml_get_matrix:
+ * @node: XML node pointer.
+ * @doc: XML document pointer.
+ * @err: location to receive error code.
+ * 
+ * Returns: allocated gretl matrix read from @node, or %NULL 
+ * on failure.
+ */
+
+gretl_matrix *gretl_xml_get_matrix (xmlNodePtr node, xmlDocPtr doc, 
+				    int *err)
+{
+    return xml_get_user_matrix(node, doc, NULL, err);
+}
+
 /**
  * gretl_xml_get_submask:
  * @node: XML node pointer.
  * @doc: XML document pointer.
  * @pmask: location to receive allocated mask.
- * @pmode: location to receive sub-sampling mode (or %NULL).
  * 
  * Returns: 0 on success, non-zero on failure.
  */
 
-int gretl_xml_get_submask (xmlNodePtr node, xmlDocPtr doc,
-			   char **pmask, int *pmode)
+int gretl_xml_get_submask (xmlNodePtr node, xmlDocPtr doc, char **pmask)
 {
     char *mask = NULL;
-    int i, len, mode = 0;
+    int i, len;
     int err = 0;
 
     if (!gretl_xml_get_prop_as_int(node, "length", &len)) {
 	return 1;
     }
 
-    gretl_xml_get_prop_as_int(node, "mode", &mode);
+    if (len == 0) {
+	*pmask = RESAMPLED;
+	return 0;
+    }
 
     mask = calloc(len, 1);
+
     if (mask == NULL) {
 	err = 1;
     } else {
@@ -1094,9 +1165,6 @@ int gretl_xml_get_submask (xmlNodePtr node, xmlDocPtr doc,
 
     if (!err) {
 	*pmask = mask;
-	if (pmode != NULL) {
-	    *pmode = mode;
-	}
     }
 
     return err;
@@ -1227,9 +1295,9 @@ int gretl_write_matrix_as_gdt (const char *fname,
 
     gzprintf(fz, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
 	     "<!DOCTYPE gretldata SYSTEM \"gretldata.dtd\">\n\n"
-	     "<gretldata name=\"%s\" frequency=\"1\" "
+	     "<gretldata version=\"%s\" name=\"%s\" frequency=\"1\" "
 	     "startobs=\"1\" endobs=\"%d\" type=\"cross-section\">\n", 
-	     datname, T);
+	     GRETLDATA_VERSION, datname, T);
 
     free(xmlbuf);
 
@@ -1282,7 +1350,7 @@ int gretl_write_matrix_as_gdt (const char *fname,
  * @list: list of variables to write (or %NULL to write all).
  * @Z: data matrix.
  * @pdinfo: data information struct.
- * @fmt: if %GRETL_DATA_GZIPPED write gzipped data, else uncompressed.
+ * @opt: if %OPT_Z write gzipped data, else uncompressed.
  * @ppaths: pointer to paths information (or NULL).
  * 
  * Write out in xml a data file containing the values of the given set
@@ -1293,11 +1361,11 @@ int gretl_write_matrix_as_gdt (const char *fname,
 
 int gretl_write_gdt (const char *fname, const int *list, 
 		     const double **Z, const DATAINFO *pdinfo, 
-		     GretlDataFormat fmt, PATHS *ppaths)
+		     gretlopt opt, PATHS *ppaths)
 {
     FILE *fp = NULL;
     gzFile *fz = Z_NULL;
-    int gz = (fmt == GRETL_DATA_GZIPPED);
+    int gz = (opt & OPT_Z);
     int tsamp = pdinfo->t2 - pdinfo->t1 + 1;
     int panelobs = 0;
     int *pmax = NULL;
@@ -1384,15 +1452,15 @@ int gretl_write_gdt (const char *fname, const int *list,
     if (gz) {
 	gzprintf(fz, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
 		 "<!DOCTYPE gretldata SYSTEM \"gretldata.dtd\">\n\n"
-		 "<gretldata name=\"%s\" frequency=\"%s\" "
+		 "<gretldata version=\"%s\" name=\"%s\" frequency=\"%s\" "
 		 "startobs=\"%s\" endobs=\"%s\" ", 
-		 xmlbuf, freqstr, startdate, enddate);
+		 GRETLDATA_VERSION, xmlbuf, freqstr, startdate, enddate);
     } else {
 	fprintf(fp, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
 		"<!DOCTYPE gretldata SYSTEM \"gretldata.dtd\">\n\n"
-		"<gretldata name=\"%s\" frequency=\"%s\" "
+		"<gretldata version=\"%s\" name=\"%s\" frequency=\"%s\" "
 		"startobs=\"%s\" endobs=\"%s\" ", 
-		xmlbuf, freqstr, startdate, enddate);
+		GRETLDATA_VERSION, xmlbuf, freqstr, startdate, enddate);
     }
 
     if (gz) {
@@ -1470,7 +1538,36 @@ int gretl_write_gdt (const char *fname, const int *list,
 		    fprintf(fp, "\n displayname=\"%s\"", xmlbuf);
 		}
 	    }
-	} 
+	}
+
+	if (*PARENT(pdinfo, v)) {
+	    uerr = gretl_xml_encode_to_buf(xmlbuf, PARENT(pdinfo, v), sizeof xmlbuf);
+	    if (!uerr) {
+		if (gz) {
+		    gzprintf(fz, "\n parent=\"%s\"", xmlbuf);
+		} else {
+		    fprintf(fp, "\n parent=\"%s\"", xmlbuf);
+		}
+	    }
+	}
+
+	if (pdinfo->varinfo[i]->transform != 0) {
+	    const char *tr = gretl_command_word(pdinfo->varinfo[i]->transform);
+
+	    if (gz) {
+		gzprintf(fz, "\n transform=\"%s\"", tr);
+	    } else {
+		fprintf(fp, "\n transform=\"%s\"", tr);
+	    }
+	}
+
+	if (pdinfo->varinfo[i]->lag != 0) {
+	    if (gz) {
+		gzprintf(fz, "\n lag=\"%d\"", pdinfo->varinfo[i]->lag);
+	    } else {
+		fprintf(fp, "\n lag=\"%d\"", pdinfo->varinfo[i]->lag);
+	    }
+	}	    
 
 	if (COMPACT_METHOD(pdinfo, v) != COMPACT_NONE) {
 	    const char *meth = compact_method_to_string(COMPACT_METHOD(pdinfo, v));
@@ -1644,11 +1741,30 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
 		var_set_display_name(pdinfo, i, (char *) tmp);
 		free(tmp);
 	    }
+	    tmp = xmlGetProp(cur, (XUC) "parent");
+	    if (tmp != NULL) {
+		strcpy(pdinfo->varinfo[i]->parent, (char *) tmp); 
+		free(tmp);
+	    }
+	    tmp = xmlGetProp(cur, (XUC) "transform");
+	    if (tmp != NULL) {
+		int ci = gretl_command_number((char *) tmp);
+
+		pdinfo->varinfo[i]->transform = ci; 
+		free(tmp);
+	    }
+	    tmp = xmlGetProp(cur, (XUC) "lag");
+	    if (tmp != NULL) {
+		pdinfo->varinfo[i]->transform = atoi((char *) tmp); 
+		free(tmp);
+	    }
+
 	    tmp = xmlGetProp(cur, (XUC) "compact-method");
 	    if (tmp != NULL) {
 		COMPACT_METHOD(pdinfo, i) = compact_string_to_int((char *) tmp);
 		free(tmp);
 	    }
+
 	    tmp = xmlGetProp(cur, (XUC) "discrete");
 	    if (tmp != NULL) {
 		if (!strcmp((char *) tmp, "true")) {
@@ -1905,6 +2021,19 @@ static long get_filesize (const char *fname)
     }
 }
 
+static double get_gdt_version (xmlNodePtr node)
+{
+    xmlChar *tmp = xmlGetProp(node, (XUC) "version");
+    double v = 1.0;
+
+    if (tmp != NULL) {
+	v = dot_atof((char *) tmp);
+	free(tmp);
+    }
+
+    return v;
+}
+
 static int xml_get_data_structure (xmlNodePtr node, int *dtype)
 {
     xmlChar *tmp = xmlGetProp(node, (XUC) "type");
@@ -2035,6 +2164,56 @@ static int xml_get_endobs (xmlNodePtr node, char *endobs, int caldata)
     return err;
 }
 
+static int lag_from_label (int v, const DATAINFO *pdinfo, int *lag)
+{
+    const char *test = VARLABEL(pdinfo, v);
+    char pm, vname[VNAMELEN];
+    int pv = 0;
+
+    if (sscanf(test, "= %15[^(](t %c %d)", vname, &pm, lag) == 3) {
+	pv = varindex(pdinfo, vname);
+	pv = (pv < pdinfo->v)? pv : 0;
+    }
+
+    return pv;
+}
+
+static int dummy_child_from_label (int v, const DATAINFO *pdinfo)
+{
+    const char *test = VARLABEL(pdinfo, v);
+    char vname[VNAMELEN];
+    double val;
+    int pv = 0;
+
+    if (sscanf(test, _("dummy for %s = %lf"), vname, &val) == 2 ||
+	sscanf(test, "dummy for %s = %lf", vname, &val) == 2) {
+	pv = varindex(pdinfo, vname);
+	pv = (pv < pdinfo->v)? pv : 0;
+    }
+
+    return pv;
+}
+
+static void record_transform_info (double **Z, DATAINFO *pdinfo)
+{
+    int i, p, pv;
+
+    for (i=1; i<pdinfo->v; i++) {
+	pv = lag_from_label(i, pdinfo, &p);
+	if (pv > 0) {
+	    strcpy(pdinfo->varinfo[i]->parent, pdinfo->varname[pv]);
+	    pdinfo->varinfo[i]->transform = LAGS;
+	    pdinfo->varinfo[i]->lag = p;
+	} else {
+	    pv = dummy_child_from_label(i, pdinfo);
+	    if (pv > 0) {
+		strcpy(pdinfo->varinfo[i]->parent, pdinfo->varname[pv]);
+		pdinfo->varinfo[i]->transform = DUMMIFY;
+	    }
+	}
+    }
+}
+
 static void data_read_message (const char *fname, DATAINFO *pdinfo, PRN *prn)
 {
     pprintf(prn, M_("\nRead datafile %s\n"), fname);
@@ -2047,10 +2226,10 @@ static void data_read_message (const char *fname, DATAINFO *pdinfo, PRN *prn)
 
 /**
  * gretl_read_gdt:
- * @pZ: pointer to data set.
- * @ppdinfo: pointer to data information struct.
  * @fname: name of file to try.
  * @ppaths: path information struct.
+ * @pZ: pointer to data set.
+ * @pdinfo: pointer to data information struct.
  * @opt: use %OPT_P to display gui progress bar.
  * @prn: where messages should be written.
  * 
@@ -2061,8 +2240,9 @@ static void data_read_message (const char *fname, DATAINFO *pdinfo, PRN *prn)
  * Returns: 0 on successful completion, non-zero otherwise.
  */
 
-int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
-		    PATHS *ppaths, gretlopt opt, PRN *prn) 
+int gretl_read_gdt (char *fname, PATHS *ppaths,
+		    double ***pZ, DATAINFO *pdinfo, 
+		    gretlopt opt, PRN *prn) 
 {
     DATAINFO *tmpdinfo;
     double **tmpZ = NULL;
@@ -2071,6 +2251,7 @@ int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
     int newdata = (*pZ == NULL);
     int gotvars = 0, gotobs = 0, err = 0;
     int caldata = 0;
+    double gdtversion = 1.0;
     long fsz, progress = 0L;
 
     gretl_error_clear();
@@ -2117,6 +2298,8 @@ int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
 	goto bailout;
     }
 
+    gdtversion = get_gdt_version(cur);
+
     /* set some datainfo parameters */
 
     err = xml_get_data_structure(cur, &tmpdinfo->structure);
@@ -2195,7 +2378,7 @@ int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
 
     data_read_message(fname, tmpdinfo, prn);
 
-    err = merge_or_replace_data(pZ, ppdinfo, &tmpZ, &tmpdinfo, prn);
+    err = merge_or_replace_data(pZ, pdinfo, &tmpZ, &tmpdinfo, opt, prn);
 
     if (!err && newdata && ppaths != NULL && fname != ppaths->datfile) {
 	strcpy(ppaths->datfile, fname);
@@ -2211,18 +2394,22 @@ int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
     /* pre-process stacked cross-sectional panels: put into canonical
        stacked time series form
     */
-    if (!err && (*ppdinfo)->structure == STACKED_CROSS_SECTION) {
-	err = switch_panel_orientation(*pZ, *ppdinfo);
+    if (!err && pdinfo->structure == STACKED_CROSS_SECTION) {
+	err = switch_panel_orientation(*pZ, pdinfo);
     }
 
-    if (!err && (*ppdinfo)->structure == STACKED_TIME_SERIES) {
-	if ((*ppdinfo)->paninfo == NULL) {
-	    err = dataset_add_default_panel_indices(*ppdinfo);
+    if (!err && pdinfo->structure == STACKED_TIME_SERIES) {
+	if (pdinfo->paninfo == NULL) {
+	    err = dataset_add_default_panel_indices(pdinfo);
 	} else {
-	    err = dataset_finalize_panel_indices(*ppdinfo);
+	    err = dataset_finalize_panel_indices(pdinfo);
 	}
     }
 
+    if (!err && gdtversion < 1.1) {
+	record_transform_info(*pZ, pdinfo);
+    }
+
     if (err && tmpdinfo != NULL) {
 	destroy_dataset(tmpZ, tmpdinfo);
     }
@@ -2325,6 +2512,7 @@ int load_user_matrix_file (const char *fname)
     xmlDocPtr doc = NULL;
     xmlNodePtr cur = NULL;
     gretl_matrix *m;
+    char *colnames;
     char *name;
     int err = 0;
 
@@ -2342,10 +2530,15 @@ int load_user_matrix_file (const char *fname)
 	    if (name == NULL) {
 		err = 1;
 	    } else {
-		m = gretl_xml_get_matrix(cur, doc, &err);
+		colnames = NULL;
+		m = xml_get_user_matrix(cur, doc, &colnames, &err);
 		if (m != NULL) {
 		    err = user_matrix_add(m, name);
+		    if (!err && colnames != NULL) {
+			umatrix_set_colnames_from_string(m, colnames);
+		    }
 		}
+		free(colnames);
 		free(name);
 	    }
 	}
