@@ -62,8 +62,7 @@ typedef enum {
 
 #define HIDDEN_COMMAND(c) (c == ADDTO || \
                            c == FUNCERR || \
-                           c == OMITFROM || \
-                           c == REMEMBER)
+                           c == OMITFROM)
     
 /* functions follow */
 
@@ -92,8 +91,7 @@ int get_command_index (char *line, CMD *cmd, const DATAINFO *pdinfo);
 
 int command_number (const char *cmd);
 
-int cli_help (const char *cmdword, const char *helpfile, 
-	      int locale, PRN *prn);
+int cli_help (const char *cmdword, PATHS *paths, gretlopt opt, PRN *prn);
 
 int parseopt (const char **argv, int argc, char *fname, 
 	      int *force_lang);
@@ -105,14 +103,16 @@ void echo_function_call (const char *line, unsigned char flags, PRN *prn);
 
 void safe_print_line (const char *line, int *plen, PRN *prn);
 
-int gretl_cmd_exec (ExecState *s, double ***pZ, DATAINFO **ppdinfo);
+int gretl_cmd_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo);
 
-int call_pca_plugin (VMatrix *corrmat, double ***pZ,
-		     DATAINFO *pdinfo, gretlopt *pflag,
+int call_pca_plugin (VMatrix *cmat, double ***pZ,
+		     DATAINFO *pdinfo, gretlopt opt,
 		     PRN *prn);
 
 int ready_for_command (const char *line);
 
+int gretl_shell_grab (const char *arg, char **sout);
+
 #endif /* INTERACT_H */
 
 
