@@ -21,6 +21,7 @@
 
 #include "libgretl.h" 
 #include "var.h"  
+#include "johansen.h"
 #include "vartest.h"
 #include "matrix_extra.h"
 
@@ -455,7 +456,7 @@ static int init_VECM_dataset (irfboot *b, GRETL_VAR *var,
 	nv += var->rlist[0];
     }
 
-    b->dinfo = create_new_dataset(&b->Z, nv, pdinfo->n, 0);
+    b->dinfo = create_auxiliary_dataset(&b->Z, nv, pdinfo->n);
     if (b->dinfo == NULL) {
 	return E_ALLOC;
     }   
