@@ -22,10 +22,6 @@
 
 #define KDEBUG 0
 
-enum {
-    KALMAN_ARMA_LL = 1 << 0
-};
-
 struct kalman_ {
     int flags;  /* for recording any options */
 
@@ -404,6 +400,28 @@ matrix_diff (const gretl_matrix *a, const gretl_matrix *b)
 /* below: if postmult is non-zero, we're post-multiplying by the
    transpose of F */
 
+#if 0
+
+/* no-brainer, just let the compiler handle things */
+
+static inline int multiply_by_F (kalman *K, const gretl_matrix *A, 
+				 gretl_matrix *B, int postmult)
+{
+    if (postmult) {
+	gretl_matrix_multiply_mod(A, GRETL_MOD_NONE,
+				  K->F, GRETL_MOD_TRANSPOSE,
+				  B, GRETL_MOD_NONE);
+    } else {
+	gretl_matrix_multiply(K->F, A, B);
+    }
+
+    return 0;
+}
+
+#else
+
+/* carefully optimized */
+
 static int multiply_by_F (kalman *K, const gretl_matrix *A, 
 			  gretl_matrix *B, int postmult)
 {
@@ -501,6 +519,8 @@ static int multiply_by_F (kalman *K, const gretl_matrix *A,
     return ret;
 }
 
+#endif
+
 /* Simplified version of the function below, for ARMA:
    in this case we have H = (r x 1) and S = (r x 1),
    although F and P are (r x r).
@@ -616,10 +636,12 @@ static void kalman_print_state (kalman *K, int t)
 {
     int j;
 
+    /* if (t > 5) return; */
+
     fprintf(stderr, "Iteration %d:\n", t);
 
     for (j=0; j<K->n; j++) {
-	fprintf(stderr, "y[%d] = %.8g, err[%d] = %.8g\n", j, 
+	fprintf(stderr, "y[%d] = %.10g, err[%d] = %.10g\n", j, 
 		gretl_matrix_get(K->y, t, j), 
 		j, gretl_vector_get(K->e, j));
     }
@@ -634,11 +656,18 @@ void kalman_set_nonshift (kalman *K, int n)
     K->nonshift = n;
 }
 
-void kalman_use_ARMA_ll (kalman *K)
+void kalman_set_options (kalman *K, int opts)
 {
-    K->flags |= KALMAN_ARMA_LL;
+    K->flags |= opts;
 }
 
+int kalman_get_options (kalman *K)
+{
+    return K->flags;
+}
+
+#define kalman_ll_average(K) (K->flags & KALMAN_AVG_LL)
+
 /* Read from the appropriate row of x (T x k) and multiply by A' to
    form A'x_t.  Note this complication: if there's a constant as well
    as other exogenous vars present, the constant does _not_ have a
@@ -859,10 +888,15 @@ int kalman_forecast (kalman *K)
     } 
 
     if (arma_ll(K) && !na(K->loglik)) {
-	double k = -(K->T / 2.0) * LN_2_PI;
+	if (kalman_ll_average(K)) {
+	    K->loglik = -0.5 * 
+		(LN_2_PI + 1 + log(K->SSRw / K->T) + K->sumVt / K->T);
+	} else {
+	    double k = -(K->T / 2.0) * LN_2_PI;
 
-	K->loglik = k - (K->T / 2.0) * (1.0 + log(K->SSRw / K->T))
-	    - 0.5 * K->sumVt;
+	    K->loglik = k - (K->T / 2.0) * (1.0 + log(K->SSRw / K->T))
+		- 0.5 * K->sumVt;
+	}
 	if (isnan(K->loglik) || isinf(K->loglik)) {
 	    K->loglik = NADBL;
 	} 
