@@ -112,7 +112,8 @@ typedef enum {
     GRETL_TYPE_SCALAR_REF,
     GRETL_TYPE_SERIES_REF,
     GRETL_TYPE_MATRIX_REF,
-    GRETL_TYPE_UVAR
+    GRETL_TYPE_UVAR,
+    GRETL_TYPE_DATE
 } GretlType;
 
 #define gretl_scalar_type(t) (t == GRETL_TYPE_BOOL || \
@@ -159,7 +160,7 @@ enum progress_return_flags {
 enum test_stats {
     GRETL_STAT_NONE,
     GRETL_STAT_NORMAL_CHISQ,
-    GRETL_STAT_TR2,
+    GRETL_STAT_LM,
     GRETL_STAT_F,
     GRETL_STAT_LMF,
     GRETL_STAT_HARVEY_COLLIER,
@@ -254,8 +255,10 @@ typedef struct equation_system_ equation_system;
 struct VARINFO_ {
     char label[MAXLABEL];
     char display_name[MAXDISP];
-    char gp_display_name[MAXDISP];
+    char parent[VNAMELEN];
     int flags;
+    int transform;
+    int lag;
     char compact_method;
     char stack_level;
     char line_width;
@@ -289,11 +292,9 @@ struct DATAINFO_ {
     char markers;       /* NO_MARKERS (0), REGULAR MARKERS or DAILY_DATE_STRINGS */
     char delim;         /* default delimiter for "CSV" files */
     char decpoint;      /* character used to represent decimal point */
-    char submode;       /* mode of sub-sampling in force, if any */
     char **S;           /* to hold observation markers */
     char *descrip;      /* to hold info on data sources etc. */
     char *submask;      /* subsampling mask */
-    void *data;         /* all-purpose pointer */
 };
 
 /* wrapper for the two main elements of a gretl data set */
@@ -304,10 +305,9 @@ struct DATASET_ {
 
 struct PATHS_ {
     char currdir[MAXLEN];
-    char userdir[MAXLEN];
+    char workdir[MAXLEN];
+    char dotdir[MAXLEN];
     char gretldir[MAXLEN];
-    char datadir[MAXLEN];
-    char scriptdir[MAXLEN];
     char helpfile[MAXLEN];
     char cmd_helpfile[MAXLEN];
     char cli_helpfile[MAXLEN];
@@ -398,6 +398,7 @@ struct MODEL_ {
 
 #define VARLABEL(p,i)        ((p->varinfo[i])->label)
 #define DISPLAYNAME(p,i)     ((p->varinfo[i])->display_name)
+#define PARENT(p,i)          ((p->varinfo[i])->parent)
 #define COMPACT_METHOD(p,i)  ((p->varinfo[i])->compact_method)
 #define STACK_LEVEL(p,i)     ((p->varinfo[i])->stack_level)
 #define SORTED_MARKER(p,i,t) ((p->varinfo[i])->sorted_markers[t])
@@ -418,10 +419,10 @@ struct MODEL_ {
 #include "gretl_prn.h"
 #include "pvalues.h"
 #include "dataio.h"
-#include "dbread.h"
 #include "strutils.h"
 #include "describe.h"
 #include "printout.h"
+#include "printscan.h"
 #include "modelprint.h"
 #include "graphing.h"
 #include "random.h"
