@@ -22,7 +22,9 @@
 #include "libgretl.h"
 #include "libset.h"
 #include "usermat.h"
+#include "matrix_extra.h"
 #include "gretl_string_table.h"
+#include "gretl_func.h"
 
 #include <unistd.h>
 #include <errno.h>
@@ -48,7 +50,8 @@ enum {
     STATE_USE_LBFGS      = 1 << 9,  /* prefer LBFGS to BFGS? */
     STATE_SHELL_OK       = 1 << 10, /* "shell" facility is approved? */
     STATE_MAX_VERBOSE    = 1 << 11, /* verbose output from maximizer? */
-    STATE_USE_FCP        = 1 << 12  /* use FCP garch code */
+    STATE_USE_FCP        = 1 << 12, /* use FCP garch code */
+    STATE_WARN_ON        = 1 << 13  /* print numerical warning messages */
 };    
 
 /* for values that really want a non-negative integer */
@@ -82,9 +85,11 @@ struct set_vars_ {
     double bhhh_toler;          /* convergence tolerance, BHHH */
     int garch_vcv;              /* GARCH vcv variant */
     int garch_robust_vcv;       /* GARCH vcv variant, robust estimation */
+    int arma_vcv;               /* ARMA vcv variant */
     int bkbp_k;                 /* Baxter-King k */
     int bkbp_p0;                /* Baxter-King periods[0] */
     int bkbp_p1;                /* Baxter-King periods[1] */
+    int rq_maxiter;             /* max iterations for quantreg, simplex */
     gretl_matrix *initvals;     /* parameter initializer */
     struct robust_opts ropts;   /* robust standard error options */
     char shelldir[MAXLEN];      /* working dir for shell commands */
@@ -92,9 +97,13 @@ struct set_vars_ {
 
 #define ECHO "echo"
 #define MESSAGES "messages"
+#define WARNINGS "warnings"
+#define GRETL_DEBUG "debug"
+#define BLAS_NMK_MIN "blas_nmk_min"
 
 #define libset_boolvar(s) (!strcmp(s, ECHO) || \
                            !strcmp(s, MESSAGES) || \
+                           !strcmp(s, WARNINGS) || \
                            !strcmp(s, FORCE_DECP) || \
 			   !strcmp(s, FORCE_HC) || \
 			   !strcmp(s, HALT_ON_ERR) || \
@@ -122,10 +131,14 @@ struct set_vars_ {
 		       !strcmp(s, HORIZON) || \
 		       !strcmp(s, LONGDIGITS) || \
 		       !strcmp(s, LOOP_MAXITER) || \
-		       !strcmp(s, VECM_NORM))
+                       !strcmp(s, RQ_MAXITER) || \
+		       !strcmp(s, VECM_NORM) || \
+		       !strcmp(s, GRETL_DEBUG) || \
+		       !strcmp(s, BLAS_NMK_MIN))
 
 /* global state */
 set_vars *state;
+int gretl_debug;
 
 static int boolvar_get_flag (const char *s);
 static const char *hac_lag_string (void);
@@ -166,6 +179,12 @@ static const char *garch_vcv_strs[] = {
     NULL
 };
 
+static const char *arma_vcv_strs[] = {
+    "hessian",
+    "op",
+    NULL
+};
+
 static const char *hac_kernel_strs[] = {
     "bartlett", 
     "parzen", 
@@ -189,6 +208,8 @@ static const char **libset_option_strings (const char *s)
 {
     if (!strcmp(s, GARCH_VCV)) {
 	return garch_vcv_strs;
+    } else if (!strcmp(s, ARMA_VCV)) {
+	return arma_vcv_strs;
     } else if (!strcmp(s, HAC_KERNEL)) {
 	return hac_kernel_strs;
     } else if (!strcmp(s, HC_VERSION)) {
@@ -216,12 +237,25 @@ static void coded_var_show_opts (const char *s, PRN *prn)
     }
 }
 
+static const char *get_arma_vcv_str (int v)
+{
+    if (v == VCV_HESSIAN) {
+	return arma_vcv_strs[0];
+    } else if (v == VCV_OP) {
+	return arma_vcv_strs[1];
+    } else {
+	return "unknown";
+    }
+}
+
 static const char *libset_option_string (const char *s)
 {
     if (!strcmp(s, HAC_LAG)) {
 	return hac_lag_string(); /* special */
     } else if (!strcmp(s, GARCH_VCV)) {
 	return garch_vcv_strs[state->garch_vcv];
+    } else if (!strcmp(s, ARMA_VCV)) {
+	return get_arma_vcv_str(state->arma_vcv);
     } else if (!strcmp(s, HAC_KERNEL)) {
 	return hac_kernel_strs[state->ropts.hkern];
     } else if (!strcmp(s, HC_VERSION)) {
@@ -259,8 +293,11 @@ static int check_for_state (void)
 
 static int flag_to_bool (set_vars *sv, int flag)
 {
-    if (!sv) return 0;
-    return (sv->flags & flag)? 1 : 0;
+    if (!sv) {
+	return 0;
+    } else {
+	return (sv->flags & flag)? 1 : 0;
+    }
 }
 
 static void state_vars_copy (set_vars *sv)
@@ -274,6 +311,7 @@ static void state_vars_copy (set_vars *sv)
     sv->horizon = state->horizon;
     sv->bootrep = state->bootrep;
     sv->loop_maxiter = state->loop_maxiter;
+    sv->rq_maxiter = state->rq_maxiter;
     sv->nls_toler = state->nls_toler;
     sv->delim = state->delim; 
     sv->longdigits = state->longdigits; 
@@ -299,13 +337,14 @@ static void state_vars_init (set_vars *sv)
 #if PDEBUG
     fprintf(stderr, "state_vars_init called\n");
 #endif
-    sv->flags = STATE_ECHO_ON | STATE_MSGS_ON | STATE_HALT_ON_ERR;
+    sv->flags = STATE_ECHO_ON | STATE_MSGS_ON | STATE_WARN_ON | STATE_HALT_ON_ERR;
     sv->seed = 0;
     sv->hp_lambda = NADBL;
     sv->horizon = UNSET_INT;
     sv->bootrep = 1000;
     sv->nls_toler = NADBL;
     sv->loop_maxiter = 250;
+    sv->rq_maxiter = 1000;
     sv->delim = UNSET_INT;
     sv->longdigits = 10;
     sv->vecm_norm = NORM_PHILLIPS;
@@ -316,6 +355,7 @@ static void state_vars_init (set_vars *sv)
     sv->bhhh_maxiter = 500;
     sv->bhhh_toler = NADBL;
     sv->garch_vcv = VCV_UNSET;
+    sv->arma_vcv = VCV_HESSIAN;
     sv->garch_robust_vcv = VCV_UNSET;
 
     sv->bkbp_k = UNSET_INT;
@@ -451,6 +491,17 @@ int gretl_messages_on (void)
     return flag_to_bool(state, STATE_MSGS_ON);
 }
 
+int gretl_warnings_on (void)
+{
+    if (check_for_state()) return 1;
+    return flag_to_bool(state, STATE_WARN_ON);
+}
+
+int gretl_debugging_on (void)
+{
+    return gretl_debug;
+}
+
 char get_csv_delim (const DATAINFO *pdinfo)
 {
     check_for_state();
@@ -594,7 +645,7 @@ libset_numeric_string (const char *s, int *pi, double *px, int *err)
 	if (*test != '\0') {
 	    ret = 0;
 	} else if (errno == ERANGE) {
-	    gretl_errmsg_set_from_errno();
+	    gretl_errmsg_set_from_errno(s);
 	    *err = 1;
 	}
     } else {
@@ -603,7 +654,7 @@ libset_numeric_string (const char *s, int *pi, double *px, int *err)
 	if (*test != '\0') {
 	    ret = 0;
 	} else if (errno == ERANGE) {
-	    gretl_errmsg_set_from_errno();
+	    gretl_errmsg_set_from_errno(s);
 	    *err = 1;
 	} else {
 	    *pi = (int) li;
@@ -615,19 +666,22 @@ libset_numeric_string (const char *s, int *pi, double *px, int *err)
     return ret;
 }
 
-/* all the libset ints/doubles are non-negative */
+static int negval_invalid (const char *var)
+{
+    return (var == NULL || strcmp(var, BLAS_NMK_MIN)); 
+}
 
-static int libset_get_scalar (const char *s, double **Z,
-			      DATAINFO *pdinfo,
+static int libset_get_scalar (const char *var, const char *arg, 
+			      double **Z, DATAINFO *pdinfo,
 			      int *pi, double *px)
 {
     double x = NADBL;
     int v, err = 0;
 
-    if (libset_numeric_string(s, pi, px, &err)) {
+    if (libset_numeric_string(arg, pi, px, &err)) {
 	if (err) {
 	    err = E_DATA;
-	} else if (pi != NULL && *pi < 0) {
+	} else if (pi != NULL && negval_invalid(var) && *pi < 0) {
 	    err = E_DATA;
 	} else if (px != NULL && *px < 0.0) {
 	    err = E_DATA;
@@ -635,10 +689,10 @@ static int libset_get_scalar (const char *s, double **Z,
 	return err;
     }
 
-    v = varindex(pdinfo, s);
+    v = varindex(pdinfo, arg);
 
     if (v >= pdinfo->v) {
-	sprintf(gretl_errmsg, "'%s': unrecognized name", s);
+	sprintf(gretl_errmsg, "'%s': unrecognized name", arg);
 	return E_UNKVAR;
     } else if (var_is_series(pdinfo, v)) {
 	return E_DATATYPE;
@@ -646,7 +700,7 @@ static int libset_get_scalar (const char *s, double **Z,
 	x = Z[v][0];
     }
 
-    if (x < 0.0) {
+    if (negval_invalid(var) && x < 0.0) {
 	return E_DATA;
     }
 
@@ -704,6 +758,14 @@ static int parse_libset_int_code (const char *s,
 		return 0;
 	    }
 	}
+    } else if (!strcmp(s, ARMA_VCV)) {
+	if (!strcmp(code, "op")) {
+	    state->arma_vcv = VCV_OP;
+	    return 0;
+	} else if (!strcmp(code, "hessian")) {
+	    state->arma_vcv = VCV_HESSIAN;
+	    return 0;
+	}
     } else if (!strcmp(s, HAC_KERNEL)) {
 	for (i=0; i<KERNEL_MAX; i++) {
 	    if (!strcmp(code, hac_kernel_strs[i])) {
@@ -826,9 +888,9 @@ static int set_bkbp_limits (const char *s0, const char *s1,
     int p0, p1;
     int err = 0;
 
-    err = libset_get_scalar(s0, Z, pdinfo, &p0, NULL);
+    err = libset_get_scalar(NULL, s0, Z, pdinfo, &p0, NULL);
     if (!err) {
-	err = libset_get_scalar(s1, Z, pdinfo, &p1, NULL);
+	err = libset_get_scalar(NULL, s1, Z, pdinfo, &p1, NULL);
     }
 
     if (err) {
@@ -879,15 +941,27 @@ static int set_initvals (const char *s, const DATAINFO *pdinfo, PRN *prn)
     return err;
 }
 
-void shelldir_init (void)
+void shelldir_init (const char *s)
 {
-    char *test = getcwd(state->shelldir, MAXLEN);
+    if (s != NULL) {
+	int n;
 
-    if (test == NULL) {
 	*state->shelldir = '\0';
+	strncat(state->shelldir, s, MAXLEN - 1);
+	n = strlen(state->shelldir);
+	if (n > 0 && (state->shelldir[n-1] == '\\' ||
+		      state->shelldir[n-1] == '/')) {
+	    state->shelldir[n-1] = '\0';
+	}	
     } else {
-	gretl_insert_builtin_string("shelldir", state->shelldir);
+	char *test = getcwd(state->shelldir, MAXLEN);
+
+	if (test == NULL) {
+	    *state->shelldir = '\0';
+	} 
     }
+
+    gretl_insert_builtin_string("shelldir", state->shelldir);
 }
 
 static int set_shelldir (const char *s)
@@ -921,6 +995,43 @@ static int set_shelldir (const char *s)
     return err;
 }
 
+static int (*workdir_callback)();
+
+void set_workdir_callback (int (*callback)())
+{
+    workdir_callback = callback;
+}
+
+static int set_workdir (const char *s)
+{
+    if (gretl_function_depth() > 0) {
+	gretl_errmsg_set("set workdir: cannot be done inside a function");
+	return 1;
+    }
+
+    if (workdir_callback == NULL) {
+	return E_DATA;
+    }
+
+    /* skip past "set workdir" and space */
+    s += 11;
+    s += strspn(s, " ");
+
+    if (*s != '\0') {
+	char workdir[MAXLEN];
+	int n = 0;
+
+	if (*s == '"') {
+	    n = sscanf(s+1, "%511[^\"]", workdir);
+	} else {
+	    n = sscanf(s, "%511s", workdir);
+	}
+	return (*workdir_callback)(workdir);
+    } else {
+	return E_DATA;
+    }
+}
+
 static int parse_set_plotfile (const char *s)
 {
     char *fname;
@@ -982,6 +1093,7 @@ static void libset_print_bool (const char *s, PRN *prn)
 }
 
 #define coded_intvar(s) (!strcmp(s, GARCH_VCV) || \
+			 !strcmp(s, ARMA_VCV) || \
 			 !strcmp(s, HAC_LAG) || \
 			 !strcmp(s, HAC_KERNEL) || \
                          !strcmp(s, HC_VERSION) || \
@@ -1047,6 +1159,9 @@ static int display_settings (PRN *prn)
     libset_print_int(LOOP_MAXITER, prn);
     libset_print_bool(MAX_VERBOSE, prn);
     libset_print_bool(MESSAGES, prn);
+    libset_print_bool(WARNINGS, prn);
+    libset_print_int(GRETL_DEBUG, prn);
+    libset_print_int(BLAS_NMK_MIN, prn);
     libset_print_bool(SHELL_OK, prn);
 
     if (*state->shelldir) {
@@ -1063,6 +1178,7 @@ static int display_settings (PRN *prn)
     libset_print_double(BFGS_TOLER, prn);
     libset_print_int(BHHH_MAXITER, prn);
     libset_print_double(BHHH_TOLER, prn);
+    libset_print_int(RQ_MAXITER, prn);
     print_initvals(state->initvals, prn);
     libset_print_bool(USE_LBFGS, prn);
     libset_print_double(NLS_TOLER, prn);
@@ -1077,6 +1193,7 @@ static int display_settings (PRN *prn)
 
     libset_print_int(BOOTREP, prn);
     libset_print_int(GARCH_VCV, prn);
+    libset_print_int(ARMA_VCV, prn);
     libset_print_bool(FORCE_HC, prn);
     libset_print_int(HAC_LAG, prn);
     libset_print_int(HAC_KERNEL, prn);
@@ -1103,6 +1220,8 @@ static int display_settings (PRN *prn)
 
     libset_print_int(HORIZON, prn);
     libset_print_int(VECM_NORM, prn);
+
+    pputc(prn, '\n');
     
     return 0;
 }
@@ -1199,6 +1318,8 @@ int execute_set_line (const char *line, double **Z, DATAINFO *pdinfo,
 	    return set_shelldir(line);
 	} else if (!strcmp(setobj, "codevars")) {
 	    return set_codevars(line);
+	} else if (!strcmp(setobj, "workdir")) {
+	    return set_workdir(line);
 	}
     }
 
@@ -1231,7 +1352,7 @@ int execute_set_line (const char *line, double **Z, DATAINFO *pdinfo,
 		libset_set_double(setobj, NADBL);
 		err = 0;
 	    } else {
-		err = libset_get_scalar(setarg, Z, pdinfo, NULL, &x);
+		err = libset_get_scalar(NULL, setarg, Z, pdinfo, NULL, &x);
 		if (!err) {
 		    err = libset_set_double(setobj, x);
 		}
@@ -1244,11 +1365,13 @@ int execute_set_line (const char *line, double **Z, DATAINFO *pdinfo,
 		err = 0;
 	    }
 	} else if (!strcmp(setobj, "seed")) {
-	    err = libset_get_scalar(setarg, Z, pdinfo, &k, NULL);
+	    err = libset_get_scalar(NULL, setarg, Z, pdinfo, &k, NULL);
 	    if (!err) {
 		gretl_rand_set_seed((unsigned int) k);
-		pprintf(prn, 
-			_("Pseudo-random number generator seeded with %d\n"), k);
+		if (gretl_messages_on() && !gretl_looping_quietly()) {
+		    pprintf(prn, 
+			    _("Pseudo-random number generator seeded with %d\n"), k);
+		}
 		state->seed = k;
 	    }
 	} else if (!strcmp(setobj, HORIZON)) {
@@ -1257,7 +1380,7 @@ int execute_set_line (const char *line, double **Z, DATAINFO *pdinfo,
 		state->horizon = UNSET_INT;
 		err = 0;
 	    } else {
-		err = libset_get_scalar(setarg, Z, pdinfo, &k, NULL);
+		err = libset_get_scalar(NULL, setarg, Z, pdinfo, &k, NULL);
 		if (!err) {
 		    state->horizon = k;
 		} else {
@@ -1267,16 +1390,20 @@ int execute_set_line (const char *line, double **Z, DATAINFO *pdinfo,
 	} else if (coded_intvar(setobj)) {
 	    err = parse_libset_int_code(setobj, setarg);
 	} else if (libset_int(setobj)) {
-	    err = libset_get_scalar(setarg, Z, pdinfo, &k, NULL);
+	    err = libset_get_scalar(setobj, setarg, Z, pdinfo, &k, NULL);
 	    if (!err) {
 		err = libset_set_int(setobj, k);
 	    }
-	} 	    
+	} else {
+	    err = E_UNKVAR;
+	}
     } else if (nw == 3) {
 	if (!strcmp(setobj, "bkbp_limits")) {
 	    err = set_bkbp_limits(setarg, setarg2, Z, pdinfo, prn);
 	} else if (!strcmp(setobj, "linewidth")) {
 	    err = set_line_width(setarg, setarg2, pdinfo, prn);
+	} else {
+	    err = E_UNKVAR;
 	}
     }
 		    
@@ -1359,6 +1486,8 @@ int libset_get_int (const char *s)
 	return state->bfgs_maxiter;
     } else if (!strcmp(s, BHHH_MAXITER)) {
 	return state->bhhh_maxiter;
+    } else if (!strcmp(s, RQ_MAXITER)) {
+	return state->rq_maxiter;
     } else if (!strcmp(s, BKBP_K)) {
 	return state->bkbp_k;
     } else if (!strcmp(s, BOOTREP)) {
@@ -1367,6 +1496,8 @@ int libset_get_int (const char *s)
 	return state->garch_vcv;
     } else if (!strcmp(s, GARCH_ROBUST_VCV)) {
 	return state->garch_robust_vcv;
+    } else if (!strcmp(s, ARMA_VCV)) {
+	return state->arma_vcv;
     } else if (!strcmp(s, HAC_KERNEL)) {
 	return state->ropts.hkern;
     } else if (!strcmp(s, HC_VERSION)) {
@@ -1379,6 +1510,10 @@ int libset_get_int (const char *s)
 	return state->loop_maxiter;
     } else if (!strcmp(s, VECM_NORM)) {
 	return state->vecm_norm;
+    } else if (!strcmp(s, GRETL_DEBUG)) {
+	return gretl_debug;
+    } else if (!strcmp(s, BLAS_NMK_MIN)) {
+	return get_blas_nmk_min();
     } else {
 	fprintf(stderr, "libset_get_int: unrecognized "
 		"variable '%s'\n", s);	
@@ -1397,6 +1532,9 @@ static int intvar_min_max (const char *s, int *min, int *max,
     } else if (!strcmp(s, BHHH_MAXITER)) {
 	*min = 1;
 	*var = &state->bhhh_maxiter;
+    } else if (!strcmp(s, RQ_MAXITER)) {
+	*min = 1;
+	*var = &state->rq_maxiter;
     } else if (!strcmp(s, BKBP_K)) {
 	*min = 1;
 	*var = &state->bkbp_k;
@@ -1424,6 +1562,10 @@ static int intvar_min_max (const char *s, int *min, int *max,
 	*min = 0;
 	*max = NORM_MAX;
 	*var = &state->vecm_norm;
+    } else if (!strcmp(s, GRETL_DEBUG)) {
+	*min = 0;
+	*max = 4;
+	*var = &gretl_debug;
     } else {
 	fprintf(stderr, "libset_set_int: unrecognized "
 		"variable '%s'\n", s);	
@@ -1443,6 +1585,11 @@ int libset_set_int (const char *s, int k)
 	return 1;
     }
 
+    if (!strcmp(s, BLAS_NMK_MIN)) {
+	set_blas_nmk_min(k);
+	return 0;
+    }
+
     err = intvar_min_max(s, &min, &max, &ivar);
 
     if (!err) {
@@ -1463,7 +1610,7 @@ static int read_cli_shell_status (void)
     FILE *fp;
     int ok = 0;
 
-    sprintf(shellstamp, "%s.gretl_shell_stamp", gretl_user_dir());
+    sprintf(shellstamp, "%s.gretl_shell_stamp", gretl_dot_dir());
     fp = fopen(shellstamp, "r");
     if (fp != NULL) {
 	ok = 1;
@@ -1480,6 +1627,8 @@ static int boolvar_get_flag (const char *s)
 	return STATE_ECHO_ON;
     } else if (!strcmp(s, MESSAGES)) {
 	return STATE_MSGS_ON;
+    } else if (!strcmp(s, WARNINGS)) {
+	return STATE_WARN_ON;
     } else if (!strcmp(s, USE_QR)) {
 	return STATE_USE_QR;
     } else if (!strcmp(s, USE_LBFGS)) {
@@ -1554,6 +1703,11 @@ int libset_get_bool (const char *s)
 {
     int flag, ret = 0;
 
+    if (!strcmp(s, MAX_VERBOSE) && gretl_debug > 1) {
+	/* strong debugging turns on max_verbose */
+	return 1;
+    }
+
     if (check_for_state()) {
 	return 0;
     }
