@@ -18,9 +18,10 @@
  */
 
 #include "libgretl.h"
+#include "usermat.h"
 
 /* functions that convert between gretl_matrix and other
-   datatypes */
+   datatypes; also printing of gretl_matrices */
 
 /**
  * gretl_vector_from_array:
@@ -160,6 +161,7 @@ static int count_selection (const char *s, int n)
  * @pmod: pointer to model
  * @select: char array indicating which rows and colums to select
  * (or %NULL for the full matrix).
+ * @err: location to receive error code.
  *
  * Produces all or part of the covariance matrix for @pmod 
  * in the form of a gretl_matrix.  Storage is allocated, to be freed
@@ -171,7 +173,7 @@ static int count_selection (const char *s, int n)
  */
 
 gretl_matrix *
-gretl_vcv_matrix_from_model (MODEL *pmod, const char *select)
+gretl_vcv_matrix_from_model (MODEL *pmod, const char *select, int *err)
 {
     gretl_matrix *vcv;
     int i, j, idx, nc;
@@ -179,7 +181,8 @@ gretl_vcv_matrix_from_model (MODEL *pmod, const char *select)
     int k = pmod->ncoeff;
 
     /* first ensure the model _has_ a vcv */
-    if (makevcv(pmod, pmod->sigma)) {
+    *err = makevcv(pmod, pmod->sigma);
+    if (*err) {
 	return NULL;
     }
 
@@ -190,11 +193,13 @@ gretl_vcv_matrix_from_model (MODEL *pmod, const char *select)
     }
     
     if (nc == 0) {
+	*err = E_DATA;
 	return NULL;
     }
 
     vcv = gretl_matrix_alloc(nc, nc);
     if (vcv == NULL) {
+	*err = E_ALLOC;
 	return NULL;
     }
 
@@ -226,6 +231,7 @@ gretl_vcv_matrix_from_model (MODEL *pmod, const char *select)
  * @pmod: pointer to model
  * @select: char array indicating which rows to select
  * (or %NULL for the full vector).
+ * @err: location to receive error code.
  *
  * Produces all or part of the coefficient vector for @pmod  
  * in the form of a gretl column vector.  Storage is allocated, to be freed
@@ -237,7 +243,7 @@ gretl_vcv_matrix_from_model (MODEL *pmod, const char *select)
  */
 
 gretl_vector *
-gretl_coeff_vector_from_model (const MODEL *pmod, const char *select)
+gretl_coeff_vector_from_model (const MODEL *pmod, const char *select, int *err)
 {
     gretl_vector *b;
     int i, j, nc;
@@ -250,11 +256,13 @@ gretl_coeff_vector_from_model (const MODEL *pmod, const char *select)
     }
     
     if (nc == 0) {
+	*err = E_DATA;
 	return NULL;
     }
 
     b = gretl_column_vector_alloc(nc);
     if (b == NULL) {
+	*err = E_ALLOC;
 	return NULL;
     }
 
@@ -798,28 +806,359 @@ int gretl_matrix_write_as_text (gretl_matrix *A, const char *fname)
     int r = A->rows;
     int c = A->cols;
     int i, j, err = 0;
-    FILE *f;
+    FILE *fp;
 
-    f = fopen(fname, "w");
+    fname = gretl_maybe_switch_dir(fname);
 
-    if (f == NULL) {
+    fp = fopen(fname, "w");
+
+    if (fp == NULL) {
 	return E_FOPEN;
     }
 
-    fprintf(f, "%d\t%d\n", r, c);
+    fprintf(fp, "%d\t%d\n", r, c);
     
     gretl_push_c_numeric_locale();
 
     for (i=0; i<r; i++) {
 	for (j=0; j<c; j++) {
-	    fprintf(f, "%26.18E\t", gretl_matrix_get(A, i, j));
+	    fprintf(fp, "%26.18E\t", gretl_matrix_get(A, i, j));
 	}
-	fputc('\n', f);
+	fputc('\n', fp);
     }
 
     gretl_pop_c_numeric_locale();
     
-    fclose(f);
+    fclose(fp);
 
     return err;
 }
+
+static void make_numstr (char *s, double x)
+{
+    if (x == -0.0) {
+	x = 0.0;
+    }   
+
+    sprintf(s, "%#.5g", x);
+
+    /* remove surplus, or add deficient, zeros */
+
+    if (strstr(s, ".00000")) {
+	s[strlen(s) - 1] = 0;
+    } else {
+	char *p = s;
+	int n = 0;
+
+	while (*p) {
+	    if (isdigit(*p)) {
+		n++;
+	    } else if (isalpha(*p)) {
+		n = 0;
+		break;
+	    }
+	    p++;
+	}
+	if (n > 0 && n < 5) {
+	    strncat(s, "0", 1);
+	}
+    }
+}
+
+static int max_numchars (const gretl_matrix *m)
+{
+    char s[24];
+    int i, n = m->rows * m->cols;
+    int c, cmax = 0;
+
+    for (i = 0; i < n && cmax < 6; i++) {
+	sprintf(s, "%g", m->val[i]);
+	c = strlen(s);
+	if (c > cmax) {
+	    cmax = c;
+	}
+    }
+
+    return cmax;
+}
+
+static void 
+real_matrix_print_to_prn (const gretl_matrix *m, const char *msg, 
+			  int packed, int errout, int plain,
+			  const char **heads, PRN *prn)
+{
+    char numstr[32];
+    double x;
+    int i, j;
+
+    if (prn == NULL) {
+	return;
+    }
+
+    if (m == NULL || m->val == NULL) {
+	if (msg != NULL && *msg != '\0') {
+	    pprintf(prn, "%s: matrix is NULL\n", msg);
+	} else {
+	    pputs(prn, "matrix is NULL\n");
+	}
+	return;
+    }
+
+    if (msg != NULL && *msg != '\0' && !plain) {
+	pprintf(prn, "%s (%d x %d)", msg, m->rows, m->cols);
+	if (!(m->t1 == 0 && m->t2 == 0)) {
+	    pprintf(prn, " [t1 = %d, t2 = %d]\n\n", m->t1 + 1, m->t2 + 1);
+	} else {
+	    pputs(prn, "\n\n");
+	}
+    }
+
+    if (heads == NULL) {
+	heads = user_matrix_get_column_names(m);
+    }
+
+    if (heads != NULL) {
+	for (j=0; j<m->cols; j++) {
+	    pprintf(prn, "%12.12s ", heads[j]);
+	}
+	pputc(prn, '\n');
+    }
+
+    if (packed) {
+	int v, n;
+
+	v = gretl_vector_get_length(m);
+	if (v == 0) {
+	    pputs(prn, " not a packed matrix\n");
+	    return;
+	}
+
+	n = (sqrt(1.0 + 8.0 * v) - 1.0) / 2.0;
+
+	for (i=0; i<n; i++) {
+	    for (j=0; j<n; j++) {
+		x = m->val[ijton(i, j, n)];
+		make_numstr(numstr, x);
+		pprintf(prn, "%12s ", numstr);
+	    }
+	    pputc(prn, '\n');
+	}
+    } else {
+	int cmax = 0;
+
+	if (heads == NULL) {
+	    cmax = max_numchars(m);
+	    if (cmax > 5) {
+		cmax = 0;
+	    }
+	} 
+
+	for (i=0; i<m->rows; i++) {
+	    for (j=0; j<m->cols; j++) {
+		x = gretl_matrix_get(m, i, j);
+		if (cmax) {
+		    sprintf(numstr, "%g", x);
+		    pprintf(prn, "%*s ", cmax + 2, numstr);
+		} else {
+		    make_numstr(numstr, x);
+		    pprintf(prn, "%12s ", numstr);
+		}
+	    }
+	    pputc(prn, '\n');
+	}
+    }
+
+    pputc(prn, '\n');
+}
+
+/**
+ * gretl_matrix_print_to_prn:
+ * @m: matrix to print.
+ * @msg: accompanying message text (or %NULL if no message is wanted).
+ * @prn: pointer to gretl printing struct.
+ *
+ * Prints the matrix @m to @prn.
+ */
+
+void 
+gretl_matrix_print_to_prn (const gretl_matrix *m, const char *msg, PRN *prn)
+{
+    real_matrix_print_to_prn(m, msg, 0, 0, 0, NULL, prn);
+}
+
+/**
+ * gretl_matrix_print_with_col_heads:
+ * @m: matrix to print.
+ * @title: accompanying title (or %NULL if no title is wanted).
+ * @heads: array of strings to identify the columns.
+ * @prn: pointer to gretl printing struct.
+ *
+ * Prints the matrix @m to @prn, with column headings given
+ * by @heads.
+ */
+
+void gretl_matrix_print_with_col_heads (const gretl_matrix *m, 
+					const char *title,
+					const char **heads,
+					PRN *prn)
+{
+    real_matrix_print_to_prn(m, title, 0, 0, 0, heads, prn);
+}
+
+static void maybe_print_col_heads (const gretl_matrix *m, 
+				   const char *fmt,
+				   int wid, int prec,
+				   int icast, PRN *prn)
+{
+    const char **heads;
+
+    heads = user_matrix_get_column_names(m);
+    
+    if (heads != NULL) {
+	char wtest[32];
+	double x;
+	int j, n;
+
+	x = gretl_matrix_get(m, 0, 0);
+
+	if (icast) {
+	    if (wid >= 0 && prec >= 0) {
+		snprintf(wtest, 32, fmt, wid, prec, (int) x);
+	    } else if (wid >= 0 || prec >= 0) {
+		n = (wid >= 0)? wid : prec;
+		snprintf(wtest, 32, fmt, n, (int) x);
+	    } else {
+		snprintf(wtest, 32, fmt, (int) x);
+	    }
+	} else {
+	    if (wid >= 0 && prec >= 0) {
+		snprintf(wtest, 32, fmt, wid, prec, x);
+	    } else if (wid >= 0 || prec >= 0) {
+		n = (wid >= 0)? wid : prec;
+		snprintf(wtest, 32, fmt, n, x);
+	    } else {
+		snprintf(wtest, 32, fmt, x);
+	    }
+	}
+
+	n = strlen(wtest);
+	for (j=0; j<m->cols; j++) {
+	    pprintf(prn, "%*s", n, heads[j]);
+	}
+	pputc(prn, '\n');
+    }
+}
+
+void gretl_matrix_print_with_format (const gretl_matrix *m, 
+				     const char *fmt,
+				     int wid, int prec,
+				     PRN *prn)
+{
+    if (prn == NULL) {
+	return;
+    }
+
+    if (gretl_is_null_matrix(m) || fmt == NULL || *fmt == '\0') {
+	real_matrix_print_to_prn(m, NULL, 0, 0, 1, NULL, prn);
+    } else {
+	int intcast = 0;
+	double x;
+	int i, j, c;
+
+	c = fmt[strlen(fmt)-1];
+	if (c == 'd' || c == 'u' || c == 'x' || c == 'l') {
+	    intcast = 1;
+	}
+
+	maybe_print_col_heads(m, fmt, wid, prec, intcast, prn);
+
+	for (i=0; i<m->rows; i++) {
+	    for (j=0; j<m->cols; j++) {
+		x = gretl_matrix_get(m, i, j);
+		if (intcast) {
+		    if (wid >= 0 && prec >= 0) {
+			pprintf(prn, fmt, wid, prec, (int) x);
+		    } else if (wid >= 0 || prec >= 0) {
+			c = (wid >= 0)? wid : prec;
+			pprintf(prn, fmt, c, (int) x);
+		    } else {
+			pprintf(prn, fmt, (int) x);
+		    }
+		} else {
+		    if (wid >= 0 && prec >= 0) {
+			pprintf(prn, fmt, wid, prec, x);
+		    } else if (wid >= 0 || prec >= 0) {
+			c = (wid >= 0)? wid : prec;
+			pprintf(prn, fmt, c, x);
+		    } else {
+			pprintf(prn, fmt, x);
+		    }
+		}
+	    }
+	    pputc(prn, '\n');
+	}
+    }
+}
+
+/**
+ * gretl_packed_matrix_print:
+ * @m: packed matrix to print.
+ * @msg: accompanying message text (or %NULL if no message is wanted).
+ *
+ * Prints the symmetric matrix @m (packed as lower triangle)
+ * to stderr.
+ */
+
+void gretl_packed_matrix_print (const gretl_matrix *m, const char *msg)
+{
+    PRN *prn;
+    int err = 0;
+
+    prn = gretl_print_new(GRETL_PRINT_STDERR, &err);
+
+    if (!err) {
+	real_matrix_print_to_prn(m, msg, 1, 0, 0, NULL, prn);
+	gretl_print_destroy(prn);
+    }
+}
+
+/**
+ * debug_print_matrix:
+ * @m: matrix to print.
+ * @msg: accompanying message text (or %NULL if no message is wanted).
+ *
+ * Prints the matrix @m to stderr, as with gretl_matrix_print(), but
+ * appends the address of the matrix struct.
+ */
+
+void debug_print_matrix (const gretl_matrix *m, const char *msg)
+{
+    char full[64] = {0};
+
+    if (msg != NULL) {
+	strncpy(full, msg, 32);
+	sprintf(full + strlen(full), " (%p)", (void *) m);
+    } else {
+	sprintf(full, " (%p)", (void *) m);
+    }
+
+    if (m != NULL) {
+	int i, n = m->rows * m->cols;
+	int d = (int) ceil(log10((double) n));
+
+	fprintf(stderr, "%s\n", full);
+	for (i=0; i<n; i++) {
+	    fprintf(stderr, "val[%0*d] = % .10E\n", d, i, m->val[i]);
+	}
+    } else {
+	PRN *prn;
+	int err = 0;
+
+	prn = gretl_print_new(GRETL_PRINT_STDERR, &err);
+	if (!err) {
+	    gretl_matrix_print_to_prn(m, full, prn);
+	    gretl_print_destroy(prn);
+	}
+    }
+}
+
