@@ -32,10 +32,10 @@ gretl_matrix *gretl_matrix_from_2d_array (const double **X,
 gretl_matrix *gretl_matrix_from_scalar (double x);
 
 gretl_matrix *
-gretl_vcv_matrix_from_model (MODEL *pmod, const char *select);
+gretl_vcv_matrix_from_model (MODEL *pmod, const char *select, int *err);
 
 gretl_vector *
-gretl_coeff_vector_from_model (const MODEL *pmod, const char *select);
+gretl_coeff_vector_from_model (const MODEL *pmod, const char *select, int *err);
 
 gretl_matrix *
 gretl_covariance_matrix_from_varlist (const int *list, const double **Z, 
@@ -68,4 +68,21 @@ gretl_matrix *gretl_matrix_read_from_text (const char *fname, int *err);
 
 int gretl_matrix_write_as_text (gretl_matrix *A, const char *fname);
 
+void 
+gretl_matrix_print_to_prn (const gretl_matrix *m, const char *msg, PRN *prn);
+
+void gretl_packed_matrix_print (const gretl_matrix *m, const char *msg);
+
+void debug_print_matrix (const gretl_matrix *m, const char *msg);
+
+void gretl_matrix_print_with_col_heads (const gretl_matrix *m, 
+					const char *title,
+					const char **heads,
+					PRN *prn);
+
+void gretl_matrix_print_with_format (const gretl_matrix *m, 
+				     const char *fmt,
+				     int wid, int prec,
+				     PRN *prn);
+
 #endif /* MATRIX_EXTRA_H */
