@@ -1,20 +1,20 @@
-/*
- *  Copyright (c) by Allin Cottrell
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /* mechanisms for handling missing observations */
@@ -285,7 +285,7 @@ int repack_missing_daily_obs (MODEL *pmod, double **Z,
 	mobs->missvec = missvec;
 	mobs->misscount = misscount;
 	err = gretl_model_set_data(pmod, "missobs", mobs, 
-				   MODEL_DATA_STRUCT,
+				   GRETL_TYPE_STRUCT,
 				   sizeof *mobs);
     }
 
@@ -412,6 +412,7 @@ int adjust_t1t2 (MODEL *pmod, const int *list, int *t1, int *t2,
 {
     int i, t, dwt = 0, t1min = *t1, t2max = *t2;
     int vi, missobs, ret = 0;
+    int move_ends = 1;
     double xx;
 
     if (pmod != NULL && gretl_model_get_int(pmod, "wt_dummy")) {
@@ -516,18 +517,26 @@ int adjust_t1t2 (MODEL *pmod, const int *list, int *t1, int *t2,
 	    /* no valid observations */
 	    pmod->errcode = E_MISSDATA;
 	    ret = 1;
-	} else if (missobs > 0) {	
+	} else if (missobs > 0) {
+#if 1
+	    pmod->missmask = model_missmask(list, *t1, *t2, 
+					    n, Z, dwt, NULL);
+	    move_ends = 0;
+#else	
 	    pmod->missmask = model_missmask(list, t1min, t2max, 
 					    n, Z, dwt, NULL);
+#endif
 	    if (pmod->missmask == NULL) {
 		pmod->errcode = E_ALLOC;
 		ret = 1;
 	    }
 	}
-    }    
+    } 
 
-    *t1 = t1min; 
-    *t2 = t2max;
+    if (move_ends) {
+	*t1 = t1min; 
+	*t2 = t2max;
+    }
 
     return ret;
 }
@@ -535,7 +544,7 @@ int adjust_t1t2 (MODEL *pmod, const int *list, int *t1, int *t2,
 /* drop first/last observations from sample if missing obs 
    encountered -- also check for missing vals within the
    remaining sample: return non-zero if there are such.
-   Adjust the t1 andt2 members of pdinfo if need be.
+   Adjust the t1 and t2 members of pdinfo if need be.
 */
 
 int list_adjust_t1t2 (const int *list, const double **Z, DATAINFO *pdinfo)
@@ -671,7 +680,7 @@ static char *refmask;
 
 /* Set the "reference" mask based on a target model */
 
-void set_reference_missmask (const MODEL *pmod)
+void set_reference_missmask_from_model (const MODEL *pmod)
 {
 #if MASKDEBUG
     fprintf(stderr, "set_reference_missmask: using model = %p\n", 
@@ -682,6 +691,57 @@ void set_reference_missmask (const MODEL *pmod)
     } 
 }
 
+int copy_to_reference_missmask (const char *mask)
+{
+    if (refmask != NULL) {
+	free(refmask);
+    }
+
+    refmask = gretl_strdup(mask);
+
+#if MASKDEBUG
+    fprintf(stderr, "copy_to_reference_missmask: refmask = %p\n", 
+	    (void *) refmask);
+#endif
+
+    return (refmask == NULL)? E_ALLOC : 0;
+}
+
+int set_reference_missmask_from_list (const int *list,
+				      const double **Z,
+				      const DATAINFO *pdinfo)
+{
+    int T = pdinfo->t2 - pdinfo->t1 + 1;
+    char *mask;
+    int nmiss = 0;
+    int err = 0;
+
+    mask = model_missmask(list, pdinfo->t1, pdinfo->t2, 
+			  pdinfo->n, Z, 0, &nmiss);
+
+    if (nmiss == T) {
+	return E_DATA;
+    } else if (nmiss == 0 && mask != NULL) {
+	free(mask);
+	mask = NULL;
+    } else if (mask == NULL) {
+	err = E_ALLOC;
+    }
+
+#if MASKDEBUG
+    fprintf(stderr, "set_reference_missmask_from_list\n");
+    printlist(list, "list");
+    fprintf(stderr, "nmiss = %d, mask = %p\n", nmiss, (void *) mask);
+#endif
+
+    if (!err) {
+	free(refmask);
+	refmask = mask;
+    }
+
+    return err;
+}
+
 /* attach the reference missing obs mask to a specified model */
 
 int apply_reference_missmask (MODEL *pmod)
@@ -777,3 +837,35 @@ int set_miss (const int *list, const char *param, double **Z,
 
     return ret;
 }
+
+/**
+ * missing_obs_fraction:
+ * @Z: data array.
+ * @pdinfo: pointer to data information struct.
+ * 
+ * Returns: the fraction of the observations in @Z for which
+ * all variables have missing values (empty rows).
+ */
+
+double missing_obs_fraction (const double **Z, const DATAINFO *pdinfo)
+{
+    int missrow, totmiss = 0;
+    int i, t;
+
+    if (pdinfo->n == 0) {
+	return 0.0;
+    }
+
+    for (t=0; t<pdinfo->n; t++) {
+	missrow = 1;
+	for (i=1; i<pdinfo->v; i++) {
+	    if (!var_is_scalar(pdinfo, i) && !na(Z[i][t])) {
+		missrow = 0;
+		break;
+	    }
+	}
+	totmiss += missrow;
+    }
+
+    return (double) totmiss / pdinfo->n;
+}
