@@ -34,7 +34,7 @@ static struct MODELSPEC *mspec;
  * model_ci_from_modelspec:
  * @i: index number of model within modelspec array.
  *
- * Returns: the command index (e.g. OLS, CORC) associated
+ * Returns: the command index (e.g. OLS, AR1) associated
  * with the model specification.
  */
 
@@ -104,7 +104,7 @@ void free_modelspec (void)
 	while (mspec[i].cmd != NULL) {
 	    free(mspec[i].cmd);
 	    if (mspec[i].submask != NULL) {
-		free(mspec[i].submask);
+		free_subsample_mask(mspec[i].submask);
 	    }
 	    i++;
 	}
@@ -193,6 +193,10 @@ static int submask_match (const char *s1, const char *s2, int n)
 {
     int t;
 
+    if (s1 == RESAMPLED || s2 == RESAMPLED) {
+	return s1 == RESAMPLED && s2 == RESAMPLED;
+    }
+
     for (t=0; t<n; t++) {
 	if (s1[t] != s2[t]) return 0;
     }
