@@ -31,7 +31,7 @@ int ok_in_loop (int ci);
 int gretl_loop_append_line (ExecState *s, double ***pZ, 
 			    DATAINFO *pdinfo);
 
-int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO **ppdinfo); 
+int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo); 
 
 int model_is_in_loop (const MODEL *pmod);
 
