@@ -25,14 +25,18 @@
 #include "libgretl.h" 
 #include "libset.h"
 #include "usermat.h"
+#include "matrix_extra.h"
 #include "gretl_func.h"
 #include "nlspec.h"
 #include "cmd_private.h"
 
+#include "gretl_f2c.h"
 #include "../../minpack/minpack.h"  
 
 #define NLS_DEBUG 0
 #define ML_DEBUG 0
+#define BFGS_DEBUG 0
+
 #define RSTEPS 4
 
 enum {
@@ -172,7 +176,7 @@ static int nls_genr_setup (nlspec *s)
 	    j++;
 	}
 	
-	genrs[i] = genr_compile(formula, s->Z, s->dinfo, &err);
+	genrs[i] = genr_compile(formula, s->Z, s->dinfo, OPT_P, &err);
 
 	if (err) {
 	    fprintf(stderr, "genr_compile: genrs[%d] = %p, err = %d\n", i, 
@@ -185,7 +189,7 @@ static int nls_genr_setup (nlspec *s)
 	   while we're at it
 	*/
 	genr_set_na_check(genrs[i]);
-	err = execute_genr(genrs[i], s->Z, s->dinfo, s->prn);
+	err = execute_genr(genrs[i], s->Z, s->dinfo, OPT_S, s->prn);
 	genr_unset_na_check(genrs[i]);
 
 	if (err) {
@@ -249,8 +253,10 @@ static int nls_genr_setup (nlspec *s)
     return err;
 }
 
-/* if i == 0 we're calculating the function; if i > 0 we're calculating
-   a derivative */
+/* If i == 0 we're calculating the function; if i > 0 we're calculating
+   a derivative.  Either way, we recalculate any auxiliary variables
+   first.
+*/
 
 static int nls_auto_genr (nlspec *s, int i)
 {
@@ -272,7 +278,7 @@ static int nls_auto_genr (nlspec *s, int i)
 #if NLS_DEBUG
 	fprintf(stderr, " generating aux var %d:\n %s\n", j, s->aux[j]);
 #endif
-	s->generr = execute_genr(s->genrs[j], s->Z, s->dinfo, s->prn);
+	s->generr = execute_genr(s->genrs[j], s->Z, s->dinfo, OPT_S, s->prn);
     }
 
     if (i == 0 && s->nlfunc == NULL) {
@@ -282,10 +288,11 @@ static int nls_auto_genr (nlspec *s, int i)
 
     j = s->naux + i;
 #if NLS_DEBUG
-    fprintf(stderr, " j = naux + i = %d+%d = %d: executing genr[%d]\n", 
+    fprintf(stderr, " j = naux+i = %d+%d = %d: executing genr[%d]:\n", 
 	    s->naux, i, j, j);
+    fprintf(stderr, " %s\n", genr_get_formula(s->genrs[j]));
 #endif
-    s->generr = execute_genr(s->genrs[j], s->Z, s->dinfo, s->prn);
+    s->generr = execute_genr(s->genrs[j], s->Z, s->dinfo, OPT_S, s->prn);
 
     /* make sure we have a correct pointer to matrix deriv */
     if (!s->generr && i > 0 && matrix_deriv(s, i-1)) {
@@ -762,6 +769,11 @@ static int nl_missval_check (nlspec *s)
 	}
     }
 
+    if (t2 - t1 + 1 == 0) {
+	fprintf(stderr, "nl_missval_check: no valid data\n");
+	return E_DATA;
+    }
+
     if (t2 - t1 + 1 < s->ncoeff) {
 	return E_DF;
     }
@@ -886,6 +898,10 @@ int BFGS_numeric_gradient (double *b, double *g, int n,
 		return 1;
 	    }
 	    g[i] = (f2 - f1) / (2.0 * h);
+#if BFGS_DEBUG > 1
+	    fprintf(stderr, "g[%d] = (%.16g - %.16g) / (2.0 * %g) = %g\n",
+		    i, f2, f1, h, g[i]);
+#endif
 	}
     }
 
@@ -900,15 +916,15 @@ static int nl_function_calc (double *f, void *p)
 {
     nlspec *s = (nlspec *) p;
     const double *y;
-    int t;
+    int t, err = 0;
 
 #if NLS_DEBUG > 1
     fprintf(stderr, "\n*** nl_function_calc called\n");
 #endif
 
     /* calculate residual given current parameter estimates */
-    if (nl_calculate_fvec(s)) {
-	return 1;
+    if ((err = nl_calculate_fvec(s))) {
+	return err;
     }
 
     s->crit = 0.0;
@@ -1371,12 +1387,7 @@ static int mle_build_vcv (MODEL *pmod, nlspec *spec, int *vcvopt)
     }
 
     if (!err) {
-	for (i=0; i<k; i++) {
-	    for (j=0; j<=i; j++) {
-		x = gretl_matrix_get(V, i, j);
-		pmod->vcv[ijton(i, j, k)] = x;
-	    }
-	}
+	err = gretl_model_write_vcv(pmod, V);
     }
 
     gretl_matrix_free(G);
@@ -1399,6 +1410,10 @@ static int mle_add_vcv (MODEL *pmod, nlspec *spec)
 	for (i=0; i<n; i++) {
 	    pmod->vcv[i] = spec->hessvec[i];
 	}
+	for (i=0; i<k; i++) {
+	    x = pmod->vcv[ijton(i, i, k)];
+	    pmod->sderr[i] = sqrt(x);
+	}
 	vcvopt = VCV_HESSIAN;
     } else {
 	/* either OPG or QML */
@@ -1406,10 +1421,6 @@ static int mle_add_vcv (MODEL *pmod, nlspec *spec)
     }
 
     if (!err) {
-	for (i=0; i<k; i++) {
-	    x = pmod->vcv[ijton(i, i, k)];
-	    pmod->sderr[i] = sqrt(x);
-	}
 	gretl_model_set_int(pmod, "ml_vcv", vcvopt);
     }    
 
@@ -1551,7 +1562,7 @@ static int transcribe_nls_function (MODEL *pmod, const char *s)
 static void 
 print_GNR_dataset (const int *list, double **gZ, DATAINFO *gdinfo)
 {
-    PRN *prn = gretl_print_new(GRETL_PRINT_STDERR);
+    PRN *prn = gretl_print_new(GRETL_PRINT_STDERR, NULL);
     int t1 = gdinfo->t1;
 
     fprintf(stderr, "gdinfo->t1 = %d, gdinfo->t2 = %d\n",
@@ -1595,7 +1606,7 @@ static MODEL GNR (double *uhat, double *jac, nlspec *spec,
 
     /* number of variables = 1 (const) + 1 (depvar) + spec->ncoeff
        (derivatives) */
-    gdinfo = create_new_dataset(&gZ, spec->ncoeff + 2, pdinfo->n, 0);
+    gdinfo = create_auxiliary_dataset(&gZ, spec->ncoeff + 2, pdinfo->n);
     if (gdinfo == NULL) {
 	gretl_model_init(&gnr);
 	gnr.errcode = E_ALLOC;
@@ -2122,9 +2133,7 @@ static int lm_calculate (nlspec *spec, double *fvec, double *jac,
     case 1:
     case 2:
     case 3:
-    case 4: /* is this right? */
-	pprintf(prn, _("Convergence achieved after %d iterations\n"),
-		spec->iters);
+    case 4:
 	break;
     case 5:
     case 6:
@@ -2152,11 +2161,21 @@ static int
 nls_calc_approx (integer *m, integer *n, double *x, double *fvec,
 		 integer *iflag, void *p)
 {
-    /* write current parameter values into dataset Z */
-    update_coeff_values(x, p);
+    int erru, errc, err;
 
-    /* calculate function at x, results into fvec */    
-    if (nl_function_calc(fvec, p)) {
+    /* write current parameter values into dataset Z */
+    err = erru = update_coeff_values(x, p);
+
+    /* calculate function at x, results into fvec */  
+    if (!err) {
+	err = errc = nl_function_calc(fvec, p);
+    }
+    
+    if (err) {
+	/* flag error to minpack */
+	fprintf(stderr, "nls_calc_approx: got error %d from %s\n", 
+		err, (erru)? "update_coeff_values" : 
+		"nl_function_calc");
 	*iflag = -1;
     }
 
@@ -2220,8 +2239,6 @@ lm_approximate (nlspec *spec, double *fvec, double *jac, PRN *prn)
     case 2:
     case 3:
     case 4:
-	pprintf(prn, _("Convergence achieved after %d iterations\n"),
-		spec->iters);
 	break;
     case 5:
     case 6:
@@ -2361,7 +2378,7 @@ static int screen_bad_aux (const char *line, const DATAINFO *pdinfo)
     get_aux_command_word(word, line);
     ci = gretl_command_number(word);
 
-    if (ci == GENR) {
+    if (ci == GENR || ci == PRINT) {
 	err = 0;
     } else if (plausible_genr_start(line, pdinfo)) {
 	err = 0;
@@ -2975,8 +2992,6 @@ static double **triangular_array_new (int n)
     return m;
 }
 
-#define BFGS_DEBUG 0
-
 /* apparatus for constructing numerical approximation to
    the Hessian */
 
@@ -3272,37 +3287,6 @@ static void BFGS_get_user_values (double *b, int n, int *maxit,
     }	
 }
 
-/**
- * BFGS_max:
- * @b: array of adjustable coefficients.
- * @n: number elements in array @b.
- * @maxit: the maximum number of iterations to allow.
- * @reltol: relative tolerance for terminating iteration.
- * @fncount: location to receive count of function evaluations.
- * @grcount: location to receive count of gradient evaluations.
- * @cfunc: pointer to function used to calculate maximand.
- * @crittype: code for type of the maximand/minimand: should
- * be %C_LOGLIK, %C_GMM or %C_OTHER.  Used only in printing
- * iteration info.
- * @gradfunc: pointer to function used to calculate the 
- * gradient, or %NULL for default numerical calculation.
- * @data: pointer that will be passed as the last
- * parameter to the callback functions @cfunc and @gradfunc.
- * @opt: may contain %OPT_V for verbose operation.
- * @prn: printing struct (or %NULL).  Only used if @opt
- * includes %OPT_V.
- *
- * Obtains the set of values for @b which jointly maximize the
- * criterion value as calculated by @cfunc.  Uses the BFGS
- * variable-metric method.  Based on Pascal code in J. C. Nash,
- * "Compact Numerical Methods for Computers," 2nd edition, converted
- * by p2c then re-crafted by B. D. Ripley for gnu R.  Revised for 
- * gretl by Allin Cottrell.
- * 
- * Returns: 0 on successful completion, non-zero error code
- * on error.
- */
-
 int BFGS_orig (double *b, int n, int maxit, double reltol,
 	       int *fncount, int *grcount, BFGS_CRIT_FUNC cfunc, 
 	       int crittype, BFGS_GRAD_FUNC gradfunc, void *data, 
@@ -3311,7 +3295,7 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
     int crit_ok, done;
     double *g = NULL, *t = NULL, *X = NULL, *c = NULL, **H = NULL;
     int verbose = (opt & OPT_V);
-    int ndelta, fcount, gcount;
+    int fcount, gcount, ndelta = 0;
     double d, fmax, f, f0, sumgrad;
     int i, j, ilast, iter;
     double s, steplen = 0.0;
@@ -3343,11 +3327,22 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
 	goto bailout;
     }
 
+#if BFGS_DEBUG
+    fprintf(stderr, "BFGS: first evaluation of f = %g\n", f);
+#endif
+
     f0 = fmax = f;
     iter = ilast = fcount = gcount = 1;
     gradfunc(b, g, n, cfunc, data);
     reverse_gradient(g, n);
 
+#if BFGS_DEBUG
+    fprintf(stderr, "initial gradient:\n");
+    for (i=0; i<n; i++) {
+	fprintf(stderr, " g[%d] = %g\n", i, g[i]);
+    }
+#endif
+
     do {
 	if (verbose) {
 	    reverse_gradient(g, n);
@@ -3413,7 +3408,7 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
 	    done = fabs(fmax - f) <= reltol * (fabs(fmax) + reltol);
 
 #if BFGS_DEBUG
-	    fprintf(stderr, "LHS=%g, RHS=%g; done = %d\n",
+	    fprintf(stderr, "convergence test: LHS=%g, RHS=%g; done = %d\n",
 		    fabs(fmax - f), reltol * (fabs(fmax) + reltol),
 		    done);
 #endif
@@ -3464,6 +3459,9 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
 		ndelta = n;
 		ilast = gcount;
 	    }
+	} else if (sumgrad == 0.0) {
+	    fprintf(stderr, "gradient is exactly zero!\n");
+	    break;
 	} else {
 	    /* heading in the wrong direction */
 	    if (ilast == gcount) {
@@ -3472,7 +3470,7 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
 		*/
 		ndelta = 0;
 		if (gcount == 1) {
-		    err = (broken_gradient(g, n))? E_NAN : E_DATA;
+		    err = (broken_gradient(g, n))? E_NAN : E_NOCONV;
 		}
 	    } else {
 		/* reset for another attempt */
@@ -3523,6 +3521,10 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
     free(c);
     free_triangular_array(H, n);
 
+#if BFGS_DEBUG
+    fprintf(stderr, "BFGS_max: returning %d\n", err);
+#endif
+
     return err;
 }
 
@@ -3719,6 +3721,7 @@ int BFGS_max (double *b, int n, int maxit, double reltol,
 typedef struct umax_ umax;
 
 struct umax_ {
+    GretlType gentype;
     gretl_matrix *b;
     int ncoeff;
     GENERATOR *g;
@@ -3729,8 +3732,9 @@ struct umax_ {
     PRN *prn;
 };
 
-static void umax_init (umax *u)
+static void umax_init (umax *u, GretlType t)
 {
+    u->gentype = t;
     u->b = NULL;
     u->ncoeff = 0;
     u->g = NULL;
@@ -3743,8 +3747,10 @@ static void umax_init (umax *u)
 
 static void umax_clear (umax *u)
 {
-    /* drop any "$" variables created */
-    dataset_drop_listed_variables(NULL, u->Z, u->dinfo, NULL);
+    if (u->dinfo != NULL) {
+	/* drop any "$" variables created */
+	dataset_drop_listed_variables(NULL, u->Z, u->dinfo, NULL, NULL);
+    }
     destroy_genr(u->g);
 }
 
@@ -3759,7 +3765,7 @@ static double user_get_criterion (const double *b, void *p)
 	u->b->val[i] = b[i];
     }
 
-    err = execute_genr(u->g, u->Z, u->dinfo, u->prn); 
+    err = execute_genr(u->g, u->Z, u->dinfo, OPT_S, u->prn); 
 
     if (err) {
 	return NADBL;
@@ -3790,13 +3796,17 @@ static int user_gen_setup (umax *u,
     GENERATOR *g;
     int err = 0;
 
-    sprintf(formula, "$umax=%s", fncall);
+    if (u->gentype == GRETL_TYPE_MATRIX) {
+	sprintf(formula, "matrix $umax=%s", fncall);
+    } else {
+	sprintf(formula, "$umax=%s", fncall);
+    }
 
-    g = genr_compile(formula, pZ, pdinfo, &err);
+    g = genr_compile(formula, pZ, pdinfo, OPT_P, &err);
 
     if (!err) {
 	/* see if the formula actually works */
-	err = execute_genr(g, pZ, pdinfo, u->prn);
+	err = execute_genr(g, pZ, pdinfo, OPT_S, u->prn);
     }
 
     if (!err) {
@@ -3822,7 +3832,7 @@ double user_BFGS (gretl_matrix *b, const char *fncall,
     int fcount = 0, gcount = 0;
     double tol;
 
-    umax_init(&u);
+    umax_init(&u, GRETL_TYPE_DOUBLE);
 
     u.ncoeff = gretl_vector_get_length(b);
     if (u.ncoeff == 0) {
@@ -3875,7 +3885,7 @@ static int user_calc_fvec (integer *m, integer *n, double *x, double *fvec,
 	u->b->val[i] = x[i];
     }
 
-    err = execute_genr(u->g, u->Z, u->dinfo, u->prn); 
+    err = execute_genr(u->g, u->Z, u->dinfo, OPT_S, u->prn); 
     if (err) {
 	fprintf(stderr, "execute_genr: err = %d\n", err); 
     }
@@ -3933,10 +3943,12 @@ gretl_matrix *fdjac (gretl_matrix *theta, const char *fncall,
 
     *err = 0;
 
-    umax_init(&u);
+    umax_init(&u, GRETL_TYPE_MATRIX);
 
     n = gretl_vector_get_length(theta);
     if (n == 0) {
+	fprintf(stderr, "fdjac: gretl_vector_get_length gave %d for theta\n",
+		n);
 	*err = E_DATA;
 	return NULL;
     }
@@ -3946,11 +3958,12 @@ gretl_matrix *fdjac (gretl_matrix *theta, const char *fncall,
 
     *err = user_gen_setup(&u, fncall, pZ, pdinfo);
     if (*err) {
-	fprintf(stderr, "ldjac: error %d from user_gen_setup\n", *err);
+	fprintf(stderr, "fdjac: error %d from user_gen_setup\n", *err);
 	goto bailout;
     }
 
     if (u.m_out == NULL) {
+	fprintf(stderr, "fdjac: u.m_out is NULL\n");
 	*err = E_TYPES; /* FIXME */
 	goto bailout;
     }
