@@ -110,27 +110,24 @@ static void make_ranking (const double *sz, int m,
 		    rz[j] = avg;
 		}
 	    }
-	    *ties = 1;
+	    if (ties != NULL) {
+		*ties = 1;
+	    }
 	} 
 
 	r += cases;
     }
 }
 
-static int real_spearman_rho (const double *x, const double *y, int n, 
-			      double *rho, double *zval,
-			      double **rxout, double **ryout, 
-			      int *pm)
+static int rankcorr_get_rankings (const double *x, const double *y, int n, 
+				  double **rxout, double **ryout, 
+				  int *pm, int *ties)
 {
     double *sx = NULL, *sy = NULL;
     double *rx = NULL, *ry = NULL;
-    double sd, sr;
-    int i, m, ties = 0;
-
-    *rho = *zval = NADBL;
+    int i, m = 0;
 
     /* count non-missing pairs */
-    m = 0;
     for (i=0; i<n; i++) {
 	if (!na(x[i]) && !na(y[i])) {
 	    m++;
@@ -174,8 +171,39 @@ static int real_spearman_rho (const double *x, const double *y, int n,
     }
 
     /* make rankings by comparing "raw" x, y with sorted */
-    make_ranking(sx, m, x, y, n, rx, &ties, RANK_X);
-    make_ranking(sy, m, x, y, n, ry, &ties, RANK_Y);
+    make_ranking(sx, m, x, y, n, rx, ties, RANK_X);
+    make_ranking(sy, m, x, y, n, ry, ties, RANK_Y);
+
+    /* save the ranks */
+    *rxout = rx;
+    *ryout = ry;
+
+    if (pm != NULL) {
+	*pm = m;
+    }
+
+    free(sx);
+    free(sy);
+
+    return 0;
+}
+
+static int real_spearman_rho (const double *x, const double *y, int n, 
+			      double *rho, double *zval,
+			      double **rxout, double **ryout, 
+			      int *pm)
+{
+    double *rx = NULL, *ry = NULL;
+    double sd, sr;
+    int i, m, ties = 0;
+    int err = 0;
+
+    *rho = *zval = NADBL;
+
+    err = rankcorr_get_rankings(x, y, n, &rx, &ry, &m, &ties);
+    if (err) {
+	return err;
+    }
 
     if (ties == 0) {
 	/* calculate rho and z-score, no ties */
@@ -195,23 +223,44 @@ static int real_spearman_rho (const double *x, const double *y, int n,
     }
 
     /* save the ranks, if wanted */
-    if (rxout != NULL) {
+    if (rxout != NULL && ry != NULL) {
 	*rxout = rx;
-    } else {
-	free(rx);
-    }
-    if (ryout != NULL) {
 	*ryout = ry;
     } else {
+	free(rx);
 	free(ry);
     }
 
     *pm = m;
 
-    free(sx);
-    free(sy);
+    return err;
+}
 
-    return 0;
+static void print_raw_and_ranked (int vx, int vy,
+				  const double *x, const double *y, 
+				  const double *rx, const double *ry,
+				  const DATAINFO *pdinfo,
+				  PRN *prn)
+{
+    int T = pdinfo->t2 - pdinfo->t1 + 1;
+    int t, i = 0;
+
+    obs_marker_init(pdinfo);
+    pprintf(prn, "\n     %s ", _("Obs"));
+    pprintf(prn, "%13s%13s%13s%13s\n\n", pdinfo->varname[vx], _("rank"),
+	    pdinfo->varname[vy], _("rank"));
+
+    for (t=0; t<T; t++) {
+	print_obs_marker(t + pdinfo->t1, pdinfo, prn);
+	if (!(na(x[t])) && !(na(y[t]))) {
+	    gretl_printxn(x[t], 15, prn);
+	    pprintf(prn, "%15g", rx[i]);
+	    gretl_printxn(y[t], 15, prn);
+	    pprintf(prn, "%15g", ry[i]);
+	    i++;
+	}
+	pputc(prn, '\n');
+    }
 }
 
 /**
@@ -235,7 +284,7 @@ int spearman (const int *list, const double **Z, const DATAINFO *pdinfo,
     double *rx = NULL, *ry = NULL;
     const double *x, *y;
     double rho, zval;
-    int vx, vy, t, m;
+    int vx, vy, m;
     int err;
 
     if (list[0] != 2) {
@@ -274,7 +323,7 @@ int spearman (const int *list, const double **Z, const DATAINFO *pdinfo,
 
 	pputs(prn, _("Under the null hypothesis of no correlation:\n "));
 	pprintf(prn, _("t(%d) = %g, with two-tailed p-value %.4f\n"), m - 2,
-		tval, student_pvalue_2(tval, m - 2));
+		tval, student_pvalue_2(m - 2, tval));
     } else if (m >= 7) {
 	double pval = spearman_signif(fabs(rho), m);
 
@@ -293,26 +342,7 @@ int spearman (const int *list, const double **Z, const DATAINFO *pdinfo,
  skipit:
 
     if (rx != NULL && ry != NULL) { 
-	/* print raw and ranked data */
-	int i = 0;
-
-	obs_marker_init(pdinfo);
-	pprintf(prn, "\n     %s ", _("Obs"));
-	pprintf(prn, "%13s%13s%13s%13s\n\n", pdinfo->varname[vx], _("rank"),
-	       pdinfo->varname[vy], _("rank"));
-
-	for (t=0; t<T; t++) {
-	    print_obs_marker(t + pdinfo->t1, pdinfo, prn);
-	    if (!(na(x[t])) && !(na(y[t]))) {
-		gretl_printxn(x[t], 15, prn);
-		pprintf(prn, "%15g", rx[i]);
-		gretl_printxn(y[t], 15, prn);
-		pprintf(prn, "%15g", ry[i]);
-		i++;
-	    }
-	    pputc(prn, '\n');
-	}
-
+	print_raw_and_ranked(vx, vy, x, y, rx, ry, pdinfo, prn);
 	free(rx);
 	free(ry);
     }
@@ -515,9 +545,22 @@ int kendall (const int *list, const double **Z, const DATAINFO *pdinfo,
 	pputs(prn, _("Under the null hypothesis of no correlation:\n "));
 	pprintf(prn, _("z-score = %g, with two-tailed p-value %.4f\n"), z,
 		normal_pvalue_2(z));
-	pputc(prn, '\n');
     }
 
+    if (opt & OPT_V) {
+	double *rx = NULL, *ry = NULL;
+
+	rankcorr_get_rankings(x, y, T, &rx, &ry, NULL, NULL);
+
+	if (rx != NULL && ry != NULL) { 
+	    print_raw_and_ranked(vx, vy, x, y, rx, ry, pdinfo, prn);
+	    free(rx);
+	    free(ry);
+	}
+    }
+
+    pputc(prn, '\n');
+
     /* clean up */
     free(xy);
 
@@ -662,10 +705,11 @@ double lockes_test (const double *x, int t1, int t2)
 
 /**
  * runs_test:
- * @varno: ID number of the variable to process.
+ * @v: ID number of the variable to process.
  * @Z: data matrix.
  * @pdinfo: information on the data set.
- * @opt: %OPT_D to use first difference of variable.
+ * @opt: %OPT_D to use first difference of variable, %OPT_E
+ * to assume positive and negative are equiprobable.
  * @prn: gretl printing struct.
  *
  * Performs, and prints the results of, the runs test for randomness
@@ -675,14 +719,16 @@ double lockes_test (const double *x, int t1, int t2)
  * Returns: 0 on successful completion, non-zero on error.
  */
 
-int runs_test (int varno, const double **Z, const DATAINFO *pdinfo, 
+int runs_test (int v, const double **Z, const DATAINFO *pdinfo, 
 	       gretlopt opt, PRN *prn)
 {
-    double xt, *x, mean, sd;
-    double z, pval;
+    double xt, *x, mu, s2, sigma;
+    double N2, z, pval;
+    int Np, Nm;
     int t, n, runs = 1;
 
     n = pdinfo->t2 - pdinfo->t1 + 1;
+
     x = malloc(n * sizeof *x);
     if (x == NULL) {
 	return E_ALLOC;
@@ -694,8 +740,8 @@ int runs_test (int varno, const double **Z, const DATAINFO *pdinfo,
 	double xt1;
 
 	for (t=pdinfo->t1 + 1; t<=pdinfo->t2; t++) {
-	    xt = Z[varno][t];
-	    xt1 = Z[varno][t-1];
+	    xt = Z[v][t];
+	    xt1 = Z[v][t-1];
 	    if (na(xt) || na(xt1)) {
 		continue;
 	    }
@@ -703,7 +749,7 @@ int runs_test (int varno, const double **Z, const DATAINFO *pdinfo,
 	} 	
     } else {
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	    xt = Z[varno][t];
+	    xt = Z[v][t];
 	    if (na(xt)) {
 		continue;
 	    }
@@ -717,16 +763,38 @@ int runs_test (int varno, const double **Z, const DATAINFO *pdinfo,
 	return 1;
     }
 
+    Np = (x[0] > 0);
+    Nm = 1 - Np;
+
     for (t=1; t<n; t++) {
-	if ((x[t] > 0 && x[t-1] < 0) || (x[t] < 0 && x[t-1] > 0)) { 
+	if (x[t] > 0) {
+	    Np++;
+	} else {
+	    Nm++;
+	}
+	if ((x[t] > 0 && x[t-1] <= 0) || (x[t] <= 0 && x[t-1] > 0)) { 
 	    runs++;
 	}
     }
 
-    mean = (1.0 + n / 2.0);
-    sd = sqrt((double) n - 1) / 2.0;
-    z = fabs((runs - mean) / sd);
-    pval = normal_pvalue_2(z);
+    if (opt & OPT_E) {
+	mu = 1.0 + n / 2.0;
+	s2 = ((double) n - 1) / 4.0;
+    } else {
+	/* don't assume that + and - are equiprobable */
+	N2 = 2.0 * Np * Nm;
+	mu = 1.0 + N2 / n;
+	s2 = (N2 * (N2 - n)) / (n*n * (n-1));
+    }
+
+    if (s2 <= 0) {
+	sigma = 0;
+	z = pval = NADBL;
+    } else {
+	sigma = sqrt(s2);
+	z = (runs - mu) / sigma;
+	pval = normal_pvalue_2(z);
+    }	
 
     if (opt & OPT_D) {
 	pprintf(prn, "\n%s\n", _("Runs test (first difference)"));
@@ -735,10 +803,24 @@ int runs_test (int varno, const double **Z, const DATAINFO *pdinfo,
     }
 
     pprintf(prn, _("\nNumber of runs (R) in the variable '%s' = %d\n"), 
-	    pdinfo->varname[varno], runs);
-    pprintf(prn, _("Under the null hypothesis of randomness, R "
-	    "follows N(%g, %g)\n"), mean, sd);
-    pprintf(prn, _("z-score = %g, with two-tailed p-value %g\n"), z, pval);
+	    pdinfo->varname[v], runs);
+
+    if (na(z)) {
+	pprintf(prn, _("Test statistic cannot be computed: try "
+		       "the deviation from the median?\n"));
+    } else {
+	if (opt & OPT_E) {
+	    pprintf(prn, _("Under the null hypothesis of independence and equal "
+			   "probability of positive\nand negative values, R "
+			   "follows N(%g, %g)\n"), mu, sigma);
+	} else {
+	    pprintf(prn, _("Under the null hypothesis of independence, R "
+			   "follows N(%g, %g)\n"), mu, sigma);
+	}
+	pprintf(prn, _("z-score = %g, with two-tailed p-value %g\n"), z, pval);
+    }
+
+    pputc(prn, '\n');
 
     record_test_result(z, pval, "runs");
   
@@ -1078,11 +1160,11 @@ static int sign_test (const double *x, const double *y,
     pprintf(prn, _("Under the null hypothesis of no difference, W "
 		   "follows B(%d, %.1f)\n"), n, 0.5);
     pprintf(prn, "  %s(W <= %d) = %g\n", _("Prob"), w, 
-	    binomial_cdf(w, n, 0.5));
+	    binomial_cdf(0.5, n, w));
     if (w == 0) {
 	pv = 1.0;
     } else {
-	pv = binomial_cdf_comp(w - 1, n, 0.5);
+	pv = binomial_cdf_comp(0.5, n, w - 1);
     }
     pprintf(prn, "  %s(W >= %d) = %g\n\n", _("Prob"), w, pv);
 
