@@ -25,6 +25,7 @@
 #include "objstack.h"
 #include "usermat.h"
 #include "modelspec.h"
+#include "forecast.h"
 
 #define ODEBUG 0
 
@@ -52,7 +53,7 @@ static GretlObjType get_stacked_type_by_data (void *ptr)
 	}
     }
 
-    return GRETL_OBJ_UNKNOWN;
+    return GRETL_OBJ_NULL;
 }
 
 /**
@@ -266,6 +267,11 @@ void gretl_object_unref (void *ptr, GretlObjType type)
 {
     int rc = 999;
 
+    if (ptr == NULL) {
+	/* no-op */
+	return;
+    }
+
     if (type == GRETL_OBJ_ANY) {
 	type = get_stacked_type_by_data(ptr);
     } 
@@ -333,12 +339,16 @@ void set_as_last_model (void *ptr, GretlObjType type)
     if (last_model.ptr != ptr || last_model.type != type) {
 	last_model.ptr = ptr;
 	last_model.type = type;
-	gretl_object_ref(ptr, type);
+	if (ptr != NULL) {
+	    gretl_object_ref(ptr, type);
+	}
     }
 
 #if ODEBUG
-    fprintf(stderr, " refcount on \"last_model\" = %d\n",
-	    gretl_object_get_refcount(last_model.ptr, last_model.type));
+    if (last_model.ptr != NULL) {
+	fprintf(stderr, " refcount on \"last_model\" = %d\n",
+		gretl_object_get_refcount(last_model.ptr, last_model.type));
+    }
 #endif
 }
 
@@ -817,7 +827,7 @@ static double real_get_obj_scalar (void *p, GretlObjType type, int idx)
 	equation_system *sys = (equation_system *) p;
 
 	if (idx == M_T) {
-	    x = sys->n_obs;
+	    x = sys->T;
 	} else if (idx == M_LNL) {
 	    x = sys->ll;
 	} else if (idx == M_ESS) {
@@ -903,7 +913,7 @@ void set_genr_model (MODEL *pmod)
 
 void unset_genr_model (void)
 {
-    genr_model.type = GRETL_OBJ_UNKNOWN;
+    genr_model.type = GRETL_OBJ_NULL;
     genr_model.ptr = NULL;
 }
 
@@ -942,18 +952,27 @@ static stacker *find_smatch (const char *oname)
     return smatch;
 }
 
-/* retrieve the type of a named model, or of the last model
-   if @name is NULL */
+/* retrieve the type and command index of a named model, or of 
+   the last model if @name is NULL */
 
-GretlObjType gretl_model_get_type (const char *name)
+GretlObjType gretl_model_get_type_and_ci (const char *name,
+					  int *ci)
 {
     stacker *smatch = find_smatch(name);
+    GretlObjType ret = GRETL_OBJ_NULL;
 
-    if (smatch == NULL) {
-	return GRETL_OBJ_UNKNOWN;
-    } else {
-	return smatch->type;
+    *ci = 0;
+
+    if (smatch != NULL) {
+	ret = smatch->type;
+	if (ret == GRETL_OBJ_EQN) {
+	    MODEL *pmod = smatch->ptr;
+
+	    *ci = pmod->ci;
+	}
     }
+
+    return ret;
 }
 
 double saved_object_get_scalar (const char *oname, int idx, int *err)
@@ -1005,12 +1024,15 @@ gretl_matrix *
 saved_object_get_matrix (const char *oname, int idx, int *err)
 {
     gretl_matrix *M = NULL;
-    stacker *smatch;
 
-    smatch = find_smatch(oname);
+    if (idx == M_FCAST || idx == M_FCERR) {
+	M = get_forecast_matrix(idx, err);
+    } else {
+	stacker *smatch = find_smatch(oname);
 
-    if (smatch != NULL) {
-	M = real_get_obj_matrix(smatch->ptr, smatch->type, idx, err);
+	if (smatch != NULL) {
+	    M = real_get_obj_matrix(smatch->ptr, smatch->type, idx, err);
+	}
     }
 
     if (M == NULL && !*err) {
@@ -1180,8 +1202,11 @@ int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo,
 	    err = 1;
 	}
     } else if (type == GRETL_OBJ_SYS) {
-	if (ci != RESTRICT && ci != TESTUHAT) {
-	    err = E_NOTIMP;
+	err = E_NOTIMP;
+	if (ci == RESTRICT || ci == TESTUHAT || ci == FCAST) {
+	    err = 0;
+	} else if (ci == LMTEST && ((opt & OPT_A) || (opt & OPT_H))) {
+	    err = 0;
 	}
     } else if (type == GRETL_OBJ_VAR) {
 	GRETL_VAR *var = (GRETL_VAR *) ptr;
@@ -1191,7 +1216,7 @@ int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo,
 
 	if (ci == RESTRICT && r > 0) {
 	    err = 0;
-	} else if (ci == TESTUHAT || ci == FCASTERR) {
+	} else if (ci == TESTUHAT || ci == FCAST) {
 	    err = 0;
 	} else if (ci == LMTEST && ((opt & OPT_A) || (opt & OPT_H))) {
 	    err = 0;
@@ -1223,7 +1248,7 @@ int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo, PRN *prn)
 	if ((pmod->ci == LOGIT || pmod->ci == PROBIT) &&
 	    gretl_model_get_int(pmod, "ordered")) {
 	    err = E_NOTIMP;
-	} else {
+	} else if (!exact_fit_check(pmod, prn)) {
 	    err = model_error_dist(ptr, pZ, pdinfo, prn);
 	}
     } else if (type == GRETL_OBJ_SYS) {
@@ -1237,6 +1262,60 @@ int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo, PRN *prn)
     return err;
 }
 
+int highest_numbered_var_in_saved_object (const DATAINFO *pdinfo)
+{
+    GretlObjType type;
+    void *ptr;
+    int i, mvm, vmax = 0;
+
+    for (i=-1; i<n_obj; i++) {
+	if (i < 0) {
+	    ptr = get_last_model(&type);
+	} else {
+	    ptr = ostack[i].ptr;
+	    type = ostack[i].type;
+	}
+	if (ptr == NULL) {
+	    continue;
+	}
+	if (type == GRETL_OBJ_EQN) {
+	    mvm = highest_numbered_var_in_model((MODEL *) ptr, pdinfo);
+	    if (mvm > vmax) {
+		vmax = mvm;
+	    }
+	} else if (type == GRETL_OBJ_VAR) {
+	    mvm = gretl_VAR_get_highest_variable((GRETL_VAR *) ptr);
+	    if (mvm > vmax) {
+		vmax = mvm;
+	    }
+	} else if (type == GRETL_OBJ_SYS) {
+	    mvm = highest_numbered_var_in_system((equation_system *) ptr, pdinfo);
+	    if (mvm > vmax) {
+		vmax = mvm;
+	    }
+	}
+    }
+
+    return vmax;
+}
+
+int check_variable_deletion_list (int *list, const DATAINFO *pdinfo)
+{
+    int pruned = 0;
+    int i, vsave;
+
+    vsave = highest_numbered_var_in_saved_object(pdinfo);
+
+    for (i=1; i<=list[0]; i++) {
+	if (list[i] <= vsave) {
+	    gretl_list_delete_at_pos(list, i--);
+	    pruned = 1;
+	}
+    }
+
+    return pruned;
+}
+
 void gretl_saved_objects_cleanup (void)
 {
     int i;
