@@ -18,18 +18,18 @@
  */
 
 #include "libgretl.h"
+#include <errno.h>
 
 /* model commands plus ADD and OMIT */
 #define vcv_opt_ok(c) (c == ADD || \
                        c == AR || \
+		       c == AR1 || \
                        c == ARBOND || \
                        c == ARMA || \
-                       c == CORC || \
                        c == HECKIT || \
                        c == GARCH || \
                        c == GMM || \
                        c == HCCM || \
-                       c == HILU || \
                        c == HSK || \
                        c == LAD || \
                        c == LOGISTIC || \
@@ -42,7 +42,7 @@
                        c == PANEL || \
                        c == POISSON || \
                        c == PROBIT || \
-                       c == PWE || \
+		       c == QUANTREG || \
                        c == TOBIT || \
                        c == TSLS || \
                        c == WLS)
@@ -68,9 +68,6 @@ struct gretl_option gretl_opts[] = {
     { ADD,      OPT_I, "silent" },
     { ADD,      OPT_Q, "quiet" },
     { ADD,      OPT_T, "inst" },
-    { ADDTO,    OPT_B, "both" },
-    { ADDTO,    OPT_Q, "quiet" },
-    { ADDTO,    OPT_T, "inst" },
     { ADF,      OPT_N, "nc" }, 
     { ADF,      OPT_C, "c" }, 
     { ADF,      OPT_D, "seasonals" },
@@ -80,10 +77,19 @@ struct gretl_option gretl_opts[] = {
     { ADF,      OPT_Q, "quiet" },
     { ADF,      OPT_F, "difference" },
     { ADF,      OPT_E, "test-down" },
+    { ADF,      OPT_G, "gls" },
+    { AR1,      OPT_B, "no-corc"},
+    { AR1,      OPT_H, "hilu"},
+    { AR1,      OPT_P, "pwe"},
+    { AR1,      OPT_Q, "quiet"},
+    { APPEND,   OPT_T, "time-series" },
     { ARBOND,   OPT_A, "asymptotic" },
     { ARBOND,   OPT_D, "time-dummies" },
+    { ARBOND,   OPT_H, "orthdev" },
     { ARBOND,   OPT_T, "two-step" },
     { ARMA,     OPT_C, "conditional" },
+    { ARMA,     OPT_G, "opg" },
+    { ARMA,     OPT_H, "hessian" },
     { ARMA,     OPT_N, "nc" },    
     { ARMA,     OPT_Q, "quiet" },
     { ARMA,     OPT_V, "verbose" },
@@ -100,11 +106,15 @@ struct gretl_option gretl_opts[] = {
     { COINT2,   OPT_R, "rc" },
     { COINT2,   OPT_T, "ct" },
     { COINT2,   OPT_V, "verbose" },
-    { CORC,     OPT_Q, "quiet" },
     { CORR,     OPT_K, "kendall" },
     { CORR,     OPT_S, "spearman" },
     { CORR,     OPT_U, "uniform" },
+    { CORR,     OPT_V, "verbose" },
+    { CORRGM,   OPT_Q, "quiet" },
+    { CUSUM,    OPT_Q, "quiet" },
     { CUSUM,    OPT_R, "squares" },
+    { DATA,     OPT_O, "odbc" },
+    { DATAMOD,  OPT_P, "preserve" },
     { DELEET,   OPT_D, "db" },
     { DIFFTEST, OPT_G, "sign" },
     { DIFFTEST, OPT_R, "rank-sum" },
@@ -116,14 +126,18 @@ struct gretl_option gretl_opts[] = {
     { EQNPRINT, OPT_O, "complete" },
     { EQNPRINT, OPT_T, "t-ratios" },
     { TABPRINT, OPT_O, "complete" },
+    { TABPRINT, OPT_R, "rtf" },
     { ESTIMATE, OPT_M, "geomean" },
     { ESTIMATE, OPT_N, "no-df-corr" },
+    { ESTIMATE, OPT_Q, "quiet" },
     { ESTIMATE, OPT_T, "iterate" },
+    { ESTIMATE, OPT_V, "verbose" },
     { FCAST,    OPT_D, "dynamic" },
     { FCAST,    OPT_S, "static" },
-    { FCASTERR, OPT_D, "dynamic" },
-    { FCASTERR, OPT_O, "plot" },
-    { FCASTERR, OPT_S, "static" },
+    { FCAST,    OPT_Q, "quiet" },
+    { FCAST,    OPT_O, "out-of-sample" },
+    { FOREIGN,  OPT_D, "send-data" },
+    { FOREIGN,  OPT_Q, "quiet" },
     { FREQ,     OPT_O, "gamma" },
     { FREQ,     OPT_Q, "quiet" },
     { FREQ,     OPT_S, "silent" },
@@ -132,6 +146,7 @@ struct gretl_option gretl_opts[] = {
     { GARCH,    OPT_R, "robust" },
     { GARCH,    OPT_V, "verbose" },
     { GMM,      OPT_I, "iterate" },
+    { GMM,      OPT_Q, "quiet" },
     { GMM,      OPT_T, "two-step" },
     { GMM,      OPT_V, "verbose" },
     { GNUPLOT,  OPT_O, "with-lines" },
@@ -143,11 +158,12 @@ struct gretl_option gretl_opts[] = {
     { GNUPLOT,  OPT_S, "suppress-fitted" },
     { GNUPLOT,  OPT_T, "time-series" },
     { GNUPLOT,  OPT_Z, "dummy" },
+    { GNUPLOT,  OPT_C, "control" },
     { GRAPH,    OPT_O, "tall" },
     { HECKIT,   OPT_M, "ml" },
     { HECKIT,   OPT_T, "two-step" },
     { HECKIT,   OPT_V, "verbose" },
-    { HILU,     OPT_B, "no-corc" },
+    { HELP,     OPT_F, "func" },
     { HSK,      OPT_Q, "quiet" },
     { KPSS,     OPT_T, "trend" },
     { KPSS,     OPT_V, "verbose" },
@@ -155,12 +171,15 @@ struct gretl_option gretl_opts[] = {
     { KPSS,     OPT_F, "difference" },
     { LEVERAGE, OPT_S, "save" },
     { LMTEST,   OPT_A, "autocorr" },
+    { LMTEST,   OPT_B, "breusch-pagan" },
     { LMTEST,   OPT_H, "arch" },
     { LMTEST,   OPT_L, "logs" },
     { LMTEST,   OPT_S, "squares" }, 
     { LMTEST,   OPT_P, "panel" },
+    { LMTEST,   OPT_R, "robust" },
     { LMTEST,   OPT_Q, "quiet" },
     { LMTEST,   OPT_W, "white" },
+    { LMTEST,   OPT_X, "white-nocross" },
     { LOGIT,    OPT_P, "p-values" },
     { LOGIT,    OPT_R, "robust" },
     { LOGIT,    OPT_V, "verbose" },
@@ -171,41 +190,46 @@ struct gretl_option gretl_opts[] = {
     { MAHAL,    OPT_V, "vcv" },
     { MEANTEST, OPT_O, "unequal-vars" },
     { MLE,      OPT_H, "hessian" },
+    { MLE,      OPT_Q, "quiet" },
     { MLE,      OPT_R, "robust" },
     { MLE,      OPT_V, "verbose" },
     { MPOLS,    OPT_O, "vcv" },
     { MPOLS,    OPT_Q, "quiet" },
     { MPOLS,    OPT_S, "simple-print" },
     { NLS,      OPT_O, "vcv" },
+    { NLS,      OPT_Q, "quiet" },
     { NLS,      OPT_R, "robust" },
     { NLS,      OPT_V, "verbose" },
+    { NORMTEST, OPT_A, "all" },
+    { NORMTEST, OPT_D, "dhansen" },
+    { NORMTEST, OPT_W, "swilk" },
+    { NORMTEST, OPT_J, "jbera" },
+    { NORMTEST, OPT_L, "lillie" },
+    { NORMTEST, OPT_Q, "quiet" },
+    { NULLDATA, OPT_P, "preserve" },
     { OLS,      OPT_F, "print-final" },
     { OLS,      OPT_N, "no-df-corr" },
     { OLS,      OPT_O, "vcv" }, 
     { OLS,      OPT_R, "robust" },
     { OLS,      OPT_Q, "quiet" },
     { OLS,      OPT_S, "simple-print" },
-    { OMIT,     OPT_A, "automatic" },
+    { OMIT,     OPT_A, "auto" },
     { OMIT,     OPT_B, "both" },
     { OMIT,     OPT_I, "silent" },
     { OMIT,     OPT_P, "bootstrap" },
     { OMIT,     OPT_Q, "quiet" },
     { OMIT,     OPT_T, "inst" },
     { OMIT,     OPT_W, "wald" },
-    { OMITFROM, OPT_B, "both" },
-    { OMITFROM, OPT_P, "bootstrap" },
-    { OMITFROM, OPT_Q, "quiet" },
-    { OMITFROM, OPT_T, "inst" },
-    { OMITFROM, OPT_W, "wald" },
     { OPEN,     OPT_C, "coded" },
-    { OPEN,     OPT_B, "box1" },
-    { OPEN,     OPT_O, "octave" },
+    { OPEN,     OPT_O, "odbc" },
+    { OPEN,     OPT_P, "preserve" },
     { OPEN,     OPT_W, "www" },
     { OUTFILE,  OPT_A, "append" },
     { OUTFILE,  OPT_C, "close" },
     { OUTFILE,  OPT_W, "write" },
     { PANEL,    OPT_B, "between" },
     { PANEL,    OPT_D, "time-dummies" },
+    { PANEL,    OPT_F, "fixed-effects" },
     { PANEL,    OPT_H, "hausman-reg" },
     { PANEL,    OPT_O, "vcv" },
     { PANEL,    OPT_P, "pooled" },
@@ -217,6 +241,7 @@ struct gretl_option gretl_opts[] = {
     { PANEL,    OPT_V, "verbose" },
     { PANEL,    OPT_W, "unit-weights" },
     { POISSON,  OPT_V, "verbose" },
+    { PCA,      OPT_C, "covariance" },
     { PCA,      OPT_A, "save-all" },
     { PCA,      OPT_O, "save" },
     { PERGM,    OPT_O, "bartlett" },
@@ -229,7 +254,13 @@ struct gretl_option gretl_opts[] = {
     { PROBIT,   OPT_P, "p-values" },
     { PROBIT,   OPT_R, "robust" },
     { PROBIT,   OPT_V, "verbose" },
+    { QUANTREG, OPT_I, "intervals" },
+    { QUANTREG, OPT_N, "no-df-corr" },
+    { QUANTREG, OPT_R, "robust" },
     { QUIT,     OPT_X, "exit" },
+    { RESET,    OPT_C, "cubes-only" },
+    { RESET,    OPT_Q, "quiet" },
+    { RESET,    OPT_R, "squares-only" },
     { RESTRICT, OPT_B, "bootstrap" },
     { RESTRICT, OPT_F, "full" },
     { RESTRICT, OPT_J, "jitter" },
@@ -237,6 +268,8 @@ struct gretl_option gretl_opts[] = {
     { RESTRICT, OPT_V, "verbose" },
     { RESTRICT, OPT_L, "lbfgs" },
     { RESTRICT, OPT_N, "no-scaling" },
+    { RUNS,     OPT_D, "difference" },
+    { RUNS,     OPT_E, "equal" },
     { SCATTERS, OPT_L, "with-lines" },
     { SETINFO,  OPT_C, "continuous" },
     { SETINFO,  OPT_D, "discrete" },
@@ -266,6 +299,7 @@ struct gretl_option gretl_opts[] = {
     { STORE,    OPT_Z, "gzipped" },
     { STORE,    OPT_X, "omit-obs" },
     { SYSTEM,   OPT_T, "iterate" },
+    { SYSTEM,   OPT_V, "verbose" },
     { TOBIT,    OPT_V, "verbose" },
     { TSLS,     OPT_Q, "quiet" },
     { TSLS,     OPT_R, "robust" },  
@@ -289,6 +323,7 @@ struct gretl_option gretl_opts[] = {
     { VECM,     OPT_V, "verbose" },
     { WLS,      OPT_R, "robust" },    
     { WLS,      OPT_Q, "quiet" },
+    { XCORRGM,  OPT_Q, "quiet" },
     { XTAB,     OPT_C, "column" },
     { XTAB,     OPT_R, "row" },
     { XTAB,     OPT_Z, "zeros" },
@@ -489,7 +524,9 @@ static gretlopt get_short_opts (char *line, int ci, int *err)
 		    for (i=0; i<n; i++) {
 			opt = opt_from_flag(p[i]);
 			if (!opt_is_valid(opt, ci, p[i])) {
-			    *err = 1;
+			    if (err != NULL) {
+				*err = 1;
+			    }
 			    return 0L;
 			}
 			ret |= opt;
@@ -560,18 +597,84 @@ static int valid_long_opt (int ci, const char *lopt)
 
     return opt;
 }
+
+/* The following apparatus is rather funky, and will probably have to
+   be completely redone at some point.  For now, it allows passing a
+   numerical value (two-sided p-value) in connection with the --auto
+   option to the omit command.  We may want to make this mechanism
+   available in connection with other option flags for other commands:
+   in that case a redesign will be needed.  AC, 2008-01-08.
+*/
+
+static int optval_ci;
+static gretlopt optval_opt;
+static double optval_double = NADBL;
+static char optval_str[32];
+
+double get_optval_double (int ci, gretlopt opt)
+{
+    double x = NADBL;
+
+    if (ci == optval_ci && optval_opt == opt) {
+	x = optval_double;
+    }
+
+    optval_double = NADBL;
+
+    return x;
+}
+
+void set_optval_double (int ci, gretlopt opt, double x)
+{
+    optval_ci = ci;
+    optval_opt = opt;
+    optval_double = x;
+    gretl_push_c_numeric_locale();
+    sprintf(optval_str, "%g", x);
+    gretl_pop_c_numeric_locale();
+}
+
+static void unset_optval_double (void)
+{
+    optval_ci = 0;
+    optval_opt = OPT_NONE;
+    optval_double = NADBL;
+    *optval_str = '\0';
+}
+
+static int valid_optval (int ci, gretlopt opt, const char *val)
+{
+    double x;
+
+    if ((ci == OMIT && opt == OPT_A) ||
+	(ci == QUANTREG && opt == OPT_I)) {
+	if (numeric_string(val)) {
+	    x = dot_atof(val);
+	    if (x > 0.0 && x < 1.0) {
+		set_optval_double(ci, opt, x);
+		return 1;
+	    } else {
+		unset_optval_double();
+	    }
+	}
+    }
+
+    return 0;
+}
   
 static gretlopt get_long_opts (char *line, int ci, int *err)
 {
     char *s = line;
     char longopt[32];
+    char optval[32];
     gretlopt match, ret = 0L;
 
     while ((s = strstr(s, "--")) != NULL) {
 	match = 0;
 	*longopt = '\0';
 	if (maybe_opt_start(line, s)) {
-	    sscanf(s + 2, "%31s", longopt);
+	    sscanf(s + 2, "%31[^ =]", longopt);
+	    /* sscanf(s + 2, "%31s", longopt); */
 	    match = valid_long_opt(ci, longopt);
 	    if (match > 0) {
 		/* recognized an acceptable option flag */
@@ -587,6 +690,11 @@ static gretlopt get_long_opts (char *line, int ci, int *err)
 
 	if (match > 0) {
 	    gretl_delete(s, 0, 2 + strlen(longopt));
+	    if (*s == '=' && sscanf(s + 1, "%31[^ =]", optval) == 1) {
+		if (valid_optval(ci, match, optval)) {
+		    gretl_delete(s, 0, 1 + strlen(optval));
+		}
+	    }
 	} else {
 	    s += 2;
 	}
@@ -614,6 +722,12 @@ static void tail_strip (char *s)
     }
 }
 
+#define ar1_alias(s) (!strcmp(s, "corc") || \
+		      !strcmp(s, "hilu") || \
+		      !strcmp(s, "pwe"))
+
+#define smpl_alias(s) (!strcmp(s, "sample"))
+
 /**
  * get_gretl_options:
  * @line: command line to parse.
@@ -662,6 +776,10 @@ gretlopt get_gretl_options (char *line, int *err)
 	ci = GMM;
     } else if (strstr(line, "end restrict")) {
 	ci = RESTRICT;
+    } else if (ar1_alias(cmdword)) {
+	ci = AR1;
+    } else if (smpl_alias(cmdword)) {
+	ci = SMPL;
     } else {
 	ci = gretl_command_number(cmdword);
     }
@@ -672,6 +790,13 @@ gretlopt get_gretl_options (char *line, int *err)
 	return oflags;
     }
 
+    /* some commands are basically aliases */
+    if (ci == OMITFROM) {
+	ci = OMIT;
+    } else if (ci == ADDTO) {
+	ci = ADD;
+    }
+
     /* smpl: in some contexts options don't make sense */
     if (ci == SMPL && strchr(line, ';')) {
 	return oflags;
@@ -679,7 +804,7 @@ gretlopt get_gretl_options (char *line, int *err)
 
     if (ci != SETINFO && ci != TABPRINT && ci != EQNPRINT) {
 	/* try for short-form options (e.g. "-o") */
-	opt = get_short_opts(line, ci, &myerr);
+	opt = get_short_opts(line, ci, (ci == SMPL)? NULL : &myerr);
 	if (!myerr && opt) {
 	    oflags |= opt;
 	}
@@ -715,7 +840,7 @@ gretlopt get_gretl_options (char *line, int *err)
 
 const char *print_flags (gretlopt oflags, int ci)
 {
-    static char flagstr[256];
+    static char flagstr[512];
     char fbit[32];
     int i;
 
@@ -730,6 +855,12 @@ const char *print_flags (gretlopt oflags, int ci)
 	return flagstr;
     }
 
+    if (ci == OMITFROM) {
+	ci = OMIT;
+    } else if (ci == ADDTO) {
+	ci = ADD;
+    }
+
     /* special: -o (--vcv) can be used with several model
        commands */
     if ((oflags & OPT_O) && vcv_opt_ok(ci)) {
@@ -741,6 +872,12 @@ const char *print_flags (gretlopt oflags, int ci)
 	if (ci == gretl_opts[i].ci && (oflags & gretl_opts[i].o)) {
 	    sprintf(fbit, " --%s", gretl_opts[i].longopt);
 	    strcat(flagstr, fbit);
+	    if (*optval_str != '\0' && gretl_opts[i].o == optval_opt 
+		&& ci == optval_ci) {
+		sprintf(fbit, "=%s", optval_str);
+		strcat(flagstr, fbit);
+		*optval_str = '\0';
+	    }
 	}
     }
 
