@@ -32,4 +32,8 @@ char **get_all_option_strings (int *pn);
 
 int incompatible_options (gretlopt opt, gretlopt test);
 
+double get_optval_double (int ci, gretlopt opt);
+
+void set_optval_double (int ci, gretlopt opt, double x);
+
 #endif /* OPTIONS_H */
