@@ -69,6 +69,9 @@ GPT_SPEC *plotspec_new (void)
     for (i=0; i<4; i++) {
 	spec->range[i][0] = NADBL;
 	spec->range[i][1] = NADBL;
+	if (i < 3) {
+	    spec->logbase[i] = 0.0;
+	}
     }
 
     spec->b_ols = NULL;
@@ -92,7 +95,7 @@ GPT_SPEC *plotspec_new (void)
     spec->boxwidth = 0;
     spec->samples = 0;
 
-    spec->termtype[0] = 0;
+    spec->termtype = GP_TERM_NONE;
 
     return spec;
 }
@@ -195,32 +198,6 @@ static char *escape_quotes (const char *s)
     }
 }
 
-static void 
-gp_string (FILE *fp, const char *fmt, const char *s, int png)
-{
-    int done = 0;
-
-#ifdef ENABLE_NLS  
-    static int pngterm = -1;
-
-    if (png && iso_latin_version() == 2) {
-	if (pngterm < 0) {
-	    pngterm = gnuplot_png_terminal();
-	}
-	if (pngterm != GP_PNG_CAIRO) {
-	    char htmlstr[128];
-
-	    sprint_l2_to_html(htmlstr, s, sizeof htmlstr);
-	    fprintf(fp, fmt, htmlstr);
-	}
-    } 
-#endif
-
-    if (!done) {
-	fprintf(fp, fmt, s);
-    }
-}
-
 const char *gp_justification_string (int j)
 {
     if (j == GP_JUST_LEFT) {
@@ -235,12 +212,12 @@ const char *gp_justification_string (int j)
 }
 
 static void 
-print_plot_labelspec (const GPT_LABEL *lbl, int png, FILE *fp)
+print_plot_labelspec (const GPT_LABEL *lbl, FILE *fp)
 {
     char *label = escape_quotes(lbl->text);
 
-    gp_string(fp, "set label \"%s\" ", (label != NULL)? 
-	      label : lbl->text, png);
+    fprintf(fp, "set label \"%s\" ", (label != NULL)? 
+	    label : lbl->text);
 
     gretl_push_c_numeric_locale();
 
@@ -356,16 +333,23 @@ void print_plot_ranges_etc (const GPT_SPEC *spec, FILE *fp)
     gretl_push_c_numeric_locale();
 
     for (i=0; i<4; i++) {
+	if (i < 3 && spec->logbase[i] > 0.0) {
+	    fprintf(fp, "set logscale %s %g\n", rstrs[i], spec->logbase[i]);
+	}
+
 	if (na(spec->range[i][0]) || na(spec->range[i][1]) ||
 	    spec->range[i][0] == spec->range[i][1]) {
 	    continue;
 	}
+
 	if ((i == 2 && !(spec->flags & GPT_Y2AXIS)) ||
 	    (i == 3 && !(spec->flags & GPT_PARAMETRIC))) {
 	    continue;
 	}
+
 	fprintf(fp, "set %srange [%.7g:%.7g]\n", rstrs[i], 
 		spec->range[i][0], spec->range[i][1]);
+
 	if (i == 4 && spec->code == PLOT_PROB_DIST && spec->samples == 0) {
 	    int ns = spec->range[i][1] - spec->range[i][0] + 1;
 
@@ -380,18 +364,6 @@ void print_plot_ranges_etc (const GPT_SPEC *spec, FILE *fp)
     gretl_pop_c_numeric_locale();
 }
 
-static int get_mono_output (const GPT_SPEC *spec)
-{
-    if (strstr(spec->termtype, " mono")) {
-	return 1;
-    } else if (strstr(spec->termtype, "postscr") && 
-	       !strstr(spec->termtype, "color")) {
-	return 1;
-    } else {
-	return 0;
-    }
-}
-
 #define show_fit(s) (s->fit == PLOT_FIT_OLS || \
                      s->fit == PLOT_FIT_QUADRATIC || \
                      s->fit == PLOT_FIT_INVERSE || \
@@ -401,9 +373,10 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 {
     int i, k, t;
     int png = get_png_output(spec);
-    int mono = get_mono_output(spec);
+    int mono = (spec->flags & GPT_MONO);
     int started_data_lines = 0;
     int n_lines = spec->n_lines;
+    double et, yt;
     double *x[4];
     int any_y2 = 0;
     int miss = 0;
@@ -426,24 +399,24 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
     }
 
     if (!string_is_blank(spec->titles[0])) {
-	gp_string(fp, "set title \"%s\"\n", spec->titles[0], png);
+	fprintf(fp, "set title \"%s\"\n", spec->titles[0]);
     }
 
     if (!string_is_blank(spec->titles[1])) {
-	gp_string(fp, "set xlabel \"%s\"\n", spec->titles[1], png);
+	fprintf(fp, "set xlabel \"%s\"\n", spec->titles[1]);
     }
 
     if (!string_is_blank(spec->titles[2])) {
-	gp_string(fp, "set ylabel \"%s\"\n", spec->titles[2], png);
+	fprintf(fp, "set ylabel \"%s\"\n", spec->titles[2]);
     }
 
     if ((spec->flags & GPT_Y2AXIS) && !string_is_blank(spec->titles[3])) {
-	gp_string(fp, "set y2label \"%s\"\n", spec->titles[3], png);
+	fprintf(fp, "set y2label \"%s\"\n", spec->titles[3]);
     }
 
     for (i=0; i<MAX_PLOT_LABELS; i++) {
 	if (!string_is_blank(spec->labels[i].text)) {
-	    print_plot_labelspec(&(spec->labels[i]), png, fp);
+	    print_plot_labelspec(&(spec->labels[i]), fp);
 	}
     }
 
@@ -523,7 +496,9 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 	} else {
 	    fputs("set style fill solid 0.6\n", fp);
 	}
-    }  
+    } else if (spec->code == PLOT_FORECAST) {
+	fputs("set style fill solid 0.4\n", fp);
+    }
 
     if (spec->flags & GPT_ALL_MARKERS) {
 	print_data_labels(spec, fp);
@@ -543,8 +518,12 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 	if (strcmp(spec->lines[i].scale, "NA")) {
 	    if (!strcmp(spec->lines[i].scale, "1.0")) {
 		fputs("'-' using 1", fp);
-		for (k=2; k<=spec->lines[i].ncols; k++) {
-		    fprintf(fp, ":%d", k);
+		if (spec->lines[i].ncols == 2) {
+		    fputs(":($2)", fp);
+		} else {
+		    for (k=2; k<=spec->lines[i].ncols; k++) {
+			fprintf(fp, ":%d", k);
+		    }
 		}
 		fputc(' ', fp);
 	    } else {
@@ -559,7 +538,7 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 	    fprintf(fp, "axes x1y%d ", spec->lines[i].yaxis);
 	}
 
-	gp_string(fp, "title \"%s", spec->lines[i].title, png);
+	fprintf(fp, "title \"%s", spec->lines[i].title);
 
 	if (any_y2) {
 	    if (spec->lines[i].yaxis == 1) {
@@ -602,6 +581,7 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 
 	if (!started_data_lines) {
 	    x[0] = spec->data;
+	    /* see below for subsequent adjustment of x[1] */
 	    x[1] = x[0] + spec->nobs;
 	    started_data_lines = 1;
 	} 
@@ -610,6 +590,7 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 	x[3] = x[2] + spec->nobs;
 
 	for (t=0; t<spec->nobs; t++) {
+	    /* print x-axis value */
 	    if (na(x[0][t])) {
 		fputs("? ", fp);
 		miss = 1;
@@ -617,6 +598,28 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 		fprintf(fp, "%.8g ", x[0][t]);
 	    }
 
+	    /* conversion, if needed, between (y, ydelta) and
+	       (ylow, yhigh)
+	     */
+	    if (spec->lines[i].ncols == 3) {
+		if (spec->flags & GPT_FILL_SWITCH) { 
+		    yt = x[1][t];
+		    et = x[2][t];
+		    if (!na(yt) && !na(et)) {
+			x[1][t] = yt - et;
+			x[2][t] = yt + et;
+		    }
+		} else if (spec->flags & GPT_ERR_SWITCH) {
+		    if (!na(x[1][t]) && !na(x[2][t])) {
+			et = (x[2][t] - x[1][t]) / 2.0;
+			yt = x[1][t] + et;
+			x[1][t] = yt;
+			x[2][t] = et;
+		    }
+		}
+	    } 
+
+	    /* print y-axis value(s) */
 	    for (j=1; j<spec->lines[i].ncols; j++) {
 		if (na(x[j][t])) {
 		    fputs("? ", fp);
@@ -640,7 +643,7 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 
     gretl_pop_c_numeric_locale();
 
-    if (png && gnuplot_png_terminal() == GP_PNG_CAIRO) {
+    if (png && gnuplot_has_bbox()) {
 	print_plot_bounding_box_request(fp);
     }
 
