@@ -59,10 +59,11 @@ typedef struct {
     char yvarname[MAXDISP];    /* name of y variable */
     char titles[4][MAXTITLE];  /* main, x, y, y2 */
     double range[4][2];        /* axis range specifiers */
+    double logbase[3];         /* axis log-scales base (0 for linear) */
     char keyspec[MAXTITLE];    /* position of key (or none) */
     char xtics[16];            /* x-axis tic marks */
     char mxtics[4];            /* minor tics */
-    char termtype[MAXTITLE];   /* gnuplot "term" setting */
+    int termtype;              /* gnuplot "terminal" code */
     int n_lines;               /* number of lines */
     int samples;               /* number of samples for parametric plots */
     float boxwidth;            /* when using box style for frequency plots */
