@@ -24,7 +24,7 @@
 #include "libset.h"
 #include "forecast.h"
 #include "gretl_func.h"
-#include "gretl_string_table.h"
+#include "matrix_extra.h"
 
 #include <time.h>
 
@@ -74,7 +74,7 @@ void session_time (PRN *prn)
     PRN *myprn = NULL;
 
     if (prn == NULL) {
-	myprn = gretl_print_new(GRETL_PRINT_STDOUT);
+	myprn = gretl_print_new(GRETL_PRINT_STDOUT, NULL);
 	prn = myprn;
     }
 
@@ -111,7 +111,7 @@ void gui_logo (PRN *prn)
     PRN *myprn = NULL;
 
     if (prn == NULL) {
-	myprn = gretl_print_new(GRETL_PRINT_STDOUT);
+	myprn = gretl_print_new(GRETL_PRINT_STDOUT, NULL);
 	prn = myprn;
     }
 	
@@ -168,9 +168,9 @@ print_coeff_interval (const CoeffIntervals *cf, int i, PRN *prn)
     if (isnan(cf->maxerr[i])) {
 	pprintf(prn, "%*s", UTF_WIDTH(_("undefined"), 10), _("undefined"));
     } else {
-	pprintf(prn, " (%#.*g, %#.*g)", 
-		GRETL_DIGITS, cf->coeff[i] - cf->maxerr[i],
-		GRETL_DIGITS, cf->coeff[i] + cf->maxerr[i]);
+	pprintf(prn, " %#12.6g %#12.6g", 
+		cf->coeff[i] - cf->maxerr[i],
+		cf->coeff[i] + cf->maxerr[i]);
     }
 
     pputc(prn, '\n');
@@ -214,17 +214,18 @@ void print_centered (const char *s, int width, PRN *prn)
 
 void text_print_model_confints (const CoeffIntervals *cf, PRN *prn)
 {
+    double tail = cf->alpha / 2;
     int i;
 
     if (cf->asy) {
-	pprintf(prn, "z(.025) = %.4f\n\n", cf->t);
+	pprintf(prn, "z(%g) = %.4f\n\n", tail, cf->t);
     } else {
-	pprintf(prn, "t(%d, .025) = %.3f\n\n", cf->df, cf->t);
+	pprintf(prn, "t(%d, %g) = %.3f\n\n", cf->df, tail, cf->t);
     }
 
     /* xgettext:no-c-format */
-    pputs(prn, _("      VARIABLE         COEFFICIENT      95% CONFIDENCE "
-	    "INTERVAL\n\n"));      
+    pprintf(prn, _("      VARIABLE         COEFFICIENT      %g%% CONFIDENCE "
+		   "INTERVAL\n\n"), 100 * (1 - cf->alpha));      
 
     for (i=0; i<cf->ncoeff; i++) {
 	print_coeff_interval(cf, i, prn);
@@ -238,7 +239,7 @@ static void print_freq_test (const FreqDist *freq, PRN *prn)
     double pval = NADBL;
 
     if (freq->dist == D_NORMAL) {
-	pval = chisq_cdf_comp(freq->test, 2);
+	pval = chisq_cdf_comp(2, freq->test);
 	pprintf(prn, "\n%s:\n", 
 		_("Test for null hypothesis of normal distribution"));
 	pprintf(prn, "%s(2) = %.3f %s %.5f\n", 
@@ -521,7 +522,7 @@ void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
 
 	pputc(prn, '\n');
 	pprintf(prn, _("Pearson chi-square test = %g (%d df, p-value = %g)"), 
-		pearson, df, chisq_cdf_comp(pearson, df));
+		pearson, df, chisq_cdf_comp(df, pearson));
 	pputc(prn, '\n');
 	if (n5p < 0.80) {
 	    pputs(prn, "Warning: Less than of 80% of cells had expected "
@@ -545,7 +546,7 @@ void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
 
 void print_smpl (const DATAINFO *pdinfo, int fulln, PRN *prn)
 {
-    if (!gretl_messages_on()) {
+    if (!gretl_messages_on() || pdinfo->v == 0 || gretl_looping_quietly()) {
 	return;
     }
 
@@ -620,7 +621,9 @@ char *gretl_fix_exponent (char *s)
     int n;
 
     if ((p = strstr(s, "+00")) || (p = strstr(s, "-00"))) {
-	memmove(p+1, p+2, strlen(p+1));
+	if (*(p + 3)) {
+	    memmove(p+1, p+2, strlen(p+1));
+	}
     }
 
     n = strlen(s);
@@ -724,6 +727,7 @@ void
 print_contemp_covariance_matrix (const gretl_matrix *m, 
 				 double ldet, PRN *prn)
 {
+    int tex = tex_format(prn);
     int rows = gretl_matrix_rows(m);
     int cols = gretl_matrix_cols(m);
     int jmax = 1;
@@ -731,13 +735,26 @@ print_contemp_covariance_matrix (const gretl_matrix *m,
     double x;
     int i, j;
 
-    pprintf(prn, "%s\n(%s)\n\n",
-	    _("Cross-equation VCV for residuals"),
-	    _("correlations above the diagonal"));
+    if (tex) {
+	pputs(prn, "\\begin{center}\n");
+	pprintf(prn, "%s \\\\\n", I_("Cross-equation VCV for residuals"));
+	pprintf(prn, "(%s)\n\n", I_("correlations above the diagonal"));
+	pputs(prn, "\\[\n\\begin{array}{");
+	for (j=0; j<cols; j++) {
+	    pputc(prn, 'c');
+	}
+	pputs(prn, "}\n");
+    } else {
+	pprintf(prn, "%s\n", _("Cross-equation VCV for residuals"));
+	pprintf(prn, "(%s)\n\n", _("correlations above the diagonal"));
+    }
 
     for (i=0; i<rows; i++) {
 	for (j=0; j<jmax; j++) {
 	    pprintf(prn, "%#13.5g", gretl_matrix_get(m, i, j));
+	    if (tex && j < cols - 1) {
+		pputs(prn, " & ");
+	    }
 	}
 	for (j=jmax; j<cols; j++) {
 	    x = gretl_matrix_get(m, i, i) * gretl_matrix_get(m, j, j);
@@ -745,16 +762,40 @@ print_contemp_covariance_matrix (const gretl_matrix *m,
 	    x = gretl_matrix_get(m, i, j) / x;
 	    sprintf(numstr,"(%.3f)", x); 
 	    pprintf(prn, "%13s", numstr);
+	    if (tex && j < cols - 1) {
+		pputs(prn, " & ");
+	    }	    
+	}
+	if (tex) {
+	    pputs(prn, "\\\\\n");
+	} else {
+	    pputc(prn, '\n');
 	}
-	pputc(prn, '\n');
 	if (jmax < cols) {
 	    jmax++;
 	}
     }
 
+    if (tex) {
+	pputs(prn, "\\end{array}\n\\]\n");
+    }    
+
     if (!na(ldet)) {
-	pprintf(prn, "\n%s = %g\n", _("log determinant"), ldet);
+	if (tex) {
+	    if (ldet < 0) {
+		pprintf(prn, "\n%s = ", I_("log determinant"));
+		pprintf(prn, "$-$%g\n", -ldet);
+	    } else {
+		pprintf(prn, "\n%s = %g\n", I_("log determinant"), ldet);
+	    }
+	} else {
+	    pprintf(prn, "\n%s = %g\n", _("log determinant"), ldet);
+	}
     }
+
+    if (tex) {
+	pputs(prn, "\n\\end{center}\n");
+    } 
 }
 
 /**
@@ -786,9 +827,9 @@ int outcovmx (MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
     return err;
 }
 
-static void outxx (const double xx, int ci, int wid, PRN *prn)
+static void outxx (double x, int ci, int wid, PRN *prn)
 {
-    if (isnan(xx) || na(xx)) { 
+    if (isnan(x) || na(x)) { 
 	if (ci == CORR) {
 	    pprintf(prn, "%*s", UTF_WIDTH(_("NA"), wid), 
 		    _("NA"));
@@ -796,14 +837,16 @@ static void outxx (const double xx, int ci, int wid, PRN *prn)
 	    bufspace(wid, prn);
 	}
     } else if (ci == CORR) {
-	pprintf(prn, " %*.4f", wid - 1, xx);
+	pprintf(prn, " %*.4f", wid - 1, x);
     } else {
 	char numstr[18];
 
-	if (xx > -0.001 && xx < 0.001) {
-	    sprintf(numstr, "%.5e", xx);
+	if (x == -0) x = 0.0;
+
+	if (x != 0 && x > -0.001 && x < 0.001) {
+	    sprintf(numstr, "%.5e", x);
 	} else {
-	    sprintf(numstr, "%g", xx);
+	    sprintf(numstr, "%g", x);
 	}
 	gretl_fix_exponent(numstr);
 	pprintf(prn, "%*s", wid, numstr);
@@ -1070,55 +1113,22 @@ static void fcast_print_x (double x, int n, int pmax, PRN *prn)
 
 static void printstr_long (PRN *prn, double xx, int d, int *ls)
 {
-    int lwrd;    
     char str[64];
+    int n;
 
     if (na(xx)) {
-	strcpy(str, "NA");
+	strcpy(str, "NA  ");
+	n = 4;
     } else {
-	sprintf(str, "%#.*E", d, xx);
+	sprintf(str, "%#.*E  ", d, xx);
+	n = strlen(str);
     }
-    strcat(str, "  ");
-    lwrd = strlen(str);
-    if (*ls + lwrd > 78) {
-	*ls = 0;
+    if (*ls + n > 78) {
 	pputc(prn, '\n');
+	*ls = 0;
     }
     pputs(prn, str);
-    *ls += lwrd;
-}
-
-static void printstr (PRN *prn, double xx, int *ls)
-{
-    int lwrd;
-    char str[32];
-
-    if (na(xx)) {
-	strcpy(str, "NA");
-    } else {
-	printxx(xx, str, 0);
-    }
-    strcat(str, "  ");
-    lwrd = strlen(str);
-    if (*ls + lwrd > 78) {
-	*ls = 0;
-	pputc(prn, '\n');
-    }
-    pputs(prn, str);
-    *ls += lwrd;
-}
-
-static int really_const (int t1, int t2, const double *x)
-{
-    int t;
-
-    for (t=t1+1; t<=t2; t++) {
-	if (x[t] != x[t1]) {
-	    return 0;
-	}
-    }
-
-    return 1;
+    *ls += n;
 }
 
 /* prints series z from current sample t1 to t2 */
@@ -1126,7 +1136,6 @@ static int really_const (int t1, int t2, const double *x)
 static void printz (const double *z, const DATAINFO *pdinfo, 
 		    PRN *prn, gretlopt opt)
 {
-    int t1 = pdinfo->t1, t2 = pdinfo->t2;
     int t, dig = 10, ls = 0;
     double xx;
 
@@ -1135,18 +1144,27 @@ static void printz (const double *z, const DATAINFO *pdinfo,
 	opt = OPT_T;
     }
 
-    if (really_const(t1, t2, z)) {
-	if (opt & OPT_T) {
-	    printstr_long(prn, z[t1], dig, &ls);
-	} else {
-	    printstr(prn, z[t1], &ls);
-	}
-    } else for (t=t1; t<=t2; t++) {
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	xx = z[t];
 	if (opt & OPT_T) {
 	    printstr_long(prn, xx, dig, &ls);
 	} else {
-	    printstr(prn, xx, &ls);
+	    char str[32];
+	    int n;
+
+	    if (na(xx)) {
+		strcpy(str, "NA  ");
+		n = 4;
+	    } else {
+		sprintf(str, "%#.*g  ", GRETL_DIGITS, xx);
+		n = strlen(str);
+	    }
+	    if (ls + n > 78) {
+		pputc(prn, '\n');
+		ls = 0;
+	    }
+	    pputs(prn, str);
+	    ls += n;
 	}
     }
 
@@ -1660,10 +1678,11 @@ int printdata (const int *list, const char *mstr,
 	       const double **Z, const DATAINFO *pdinfo, 
 	       gretlopt opt, PRN *prn)
 {
+    int pause = gretl_get_text_pause();
     int j, v, v1, v2, jc, nvjc, lineno, ncol;
     int screenvar = 0;
     int scalars = 0;
-    int nvars = 0, sortvar = 0;
+    int sortvar = 0;
     int maxlen = 0, bplen = 13;
     int *plist = NULL;
     int *pmax = NULL; 
@@ -1671,31 +1690,36 @@ int printdata (const int *list, const char *mstr,
     char line[128];
     int err = 0;
 
-    int pause = gretl_get_text_pause();
-
     printdata_blocks = 0;
 
-    if (list == NULL || list[0] == 0) {
-	if (mstr != NULL) {
-	    goto endprint;
+    if (list != NULL && list[0] == 0) {
+	/* explicitly empty list given */
+	if (mstr == NULL) {
+	    return 0; /* no-op */
 	} else {
+	    goto endprint;
+	}
+    } else if (list == NULL) {
+	/* no list given */
+	if (mstr == NULL) {
+	    int nvars = 0;
+
 	    plist = full_var_list(pdinfo, &nvars);
-	}
+	    if (nvars == 0) {
+		/* no-op */
+		return 0;
+	    }
+	} else {
+	    goto endprint;
+	} 
     } else {
-	nvars = list[0];
-	if (nvars > 0) {
-	    plist = gretl_list_copy(list);
-	}
+	plist = gretl_list_copy(list);
     }
 
+    /* at this point plist should have something in it */
     if (plist == NULL) {
-	if (nvars == 0) {
-	    pputs(prn, "No data\n");
-	    goto endprint;
-	} else {
-	    return E_ALLOC;
-	}
-    }
+	return E_ALLOC;
+    } 
 
     if (gretl_list_has_separator(plist)) {
 	err = adjust_print_list(plist, &screenvar, opt);
@@ -1708,6 +1732,7 @@ int printdata (const int *list, const char *mstr,
     lineno = 1;
 
     /* screen out any scalars and print them first */
+
     for (j=1; j<=plist[0]; j++) {
 	int len, v = plist[j];
 
@@ -1723,26 +1748,39 @@ int printdata (const int *list, const char *mstr,
 	    }
 	}
     }
-
+    
     if (scalars) {
 	pputc(prn, '\n');
     }
 
     if (plist[0] == 0) {
+	/* no series left after elimination of scalars */
 	pputc(prn, '\n');
 	goto endprint;
     }
 
+    if (gretl_print_has_buffer(prn)) {
+	/* how big a job do we have? */
+	int T = pdinfo->t2 - pdinfo->t1 + 1;
+	int nx = plist[0] * T;
+
+	if (nx > 1000) {
+	    err = gretl_print_alloc(prn, nx * 12);
+	}
+    }
+
     if (!(opt & OPT_O)) { 
 	/* not by observations, but by variable */
 	pputc(prn, '\n');
 	for (j=1; j<=plist[0]; j++) {
+	    int vj = plist[j];
+
 	    if (plist[0] > 1) {
-		pprintf(prn, _("Varname: %s\n"), pdinfo->varname[plist[j]]);
+		pprintf(prn, _("Varname: %s\n"), pdinfo->varname[vj]);
 	    }
-	    print_var_smpl(plist[j], Z, pdinfo, prn);
+	    print_var_smpl(vj, Z, pdinfo, prn);
 	    pputc(prn, '\n');
-	    printz(Z[plist[j]], pdinfo, prn, opt);
+	    printz(Z[vj], pdinfo, prn, opt);
 	    pputc(prn, '\n');
 	}
 	goto endprint;
@@ -1769,17 +1807,20 @@ int printdata (const int *list, const char *mstr,
 	ncol = 5;
     }
 
-    /* print data by observations */
+    /* main block: print data by observations */
+
     for (j=0; j<=plist[0]/ncol; j++) {
 	char obs_string[OBSLEN];
 
 	jc = j * ncol;
 	nvjc = plist[0] - jc;
-	v1 = jc +1;
+	v1 = jc + 1;
+
 	if (nvjc) {
 	    /* starting a new block of variables */
 	    v2 = (ncol > nvjc)? nvjc : ncol;
 	    v2 += jc;
+
 	    varheading(plist, v1, v2, bplen, pdinfo, prn);
 	    printdata_blocks++;
 
@@ -1803,7 +1844,7 @@ int printdata (const int *list, const char *mstr,
 		} else {
 		    get_obs_string(obs_string, t, pdinfo);
 		}
-		
+
 		sprintf(line, "%8s ", obs_string);
 		
 		for (v=v1; v<=v2; v++) {
@@ -1963,8 +2004,11 @@ int print_data_sorted (const int *list, const int *obsvec,
     return 0;
 }
 
+#define SIGMA_MIN 1.0e-18
+
 int
-text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, PRN *prn)
+text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, 
+		      PRN *prn)
 {
     int onestep = fr->method == FC_ONESTEP;
     int t, anyast = 0;
@@ -2006,7 +2050,7 @@ text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, PRN *prn)
 		MSE += et * et;
 		AE += fabs(et);
 		effn++;
-	    } else {
+	    } else if (fr->sigma > SIGMA_MIN) {
 		ast = (fabs(et) > 2.5 * fr->sigma);
 		if (ast) {
 		    anyast = 1;
@@ -2049,7 +2093,8 @@ text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, PRN *prn)
 	} else {
 	    err = plot_fcast_errs(t0, fr->t2, obs, 
 				  fr->actual, fr->fitted, NULL, 
-				  fr->depvar, (ts)? pdinfo->pd : 0);
+				  fr->depvar, (ts)? pdinfo->pd : 0,
+				  OPT_NONE);
 	}
     }
 
@@ -2060,19 +2105,19 @@ text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, PRN *prn)
 /**
  * text_print_forecast:
  * @fr: pointer to structure containing forecasts.
- * @pZ: pointer to data array.
  * @pdinfo: dataset information.
  * @opt: if includes %OPT_P, make a plot of the forecasts.
  * @prn: printing structure.
  *
  * Print the forecasts in @fr to @prn, and also plot the
- * forecasts if %OPT_P is given.
+ * forecasts if %OPT_P is given.  If %OPT_F is given in
+ * addition to %OPT_P, use fill style for the confidence
+ * bands when plotting a forecast with standard errors.
  *
  * Returns: 0 on success, non-zero error code on error.
  */
 
-int text_print_forecast (const FITRESID *fr, 
-			 double ***pZ, DATAINFO *pdinfo, 
+int text_print_forecast (const FITRESID *fr, DATAINFO *pdinfo, 
 			 gretlopt opt, PRN *prn)
 {
     int do_errs = (fr->sderr != NULL);
@@ -2093,7 +2138,7 @@ int text_print_forecast (const FITRESID *fr,
     }
 
     if (do_errs && !(opt & OPT_Q)) {
-	if (fr->model_ci == ARMA || fr->model_ci == VECM) {
+	if (fr->asymp) {
 	    pprintf(prn, _(" For 95%% confidence intervals, z(.025) = %.2f\n"), 
 		    1.96);
 	} else {
@@ -2158,14 +2203,16 @@ int text_print_forecast (const FITRESID *fr,
     if ((opt & OPT_P) && fr->nobs > 0) {
 	const double *obs = gretl_plotx(pdinfo);
 	int ts = dataset_is_time_series(pdinfo);
-	
+
+	/* FIXME daily data x-axis */
 
 	if (obs == NULL) {
 	    err = 1;
 	} else {
 	    err = plot_fcast_errs(fr->t0, fr->t2, obs, 
 				  fr->actual, fr->fitted, maxerr, 
-				  fr->depvar, (ts)? pdinfo->pd : 0);
+				  fr->depvar, (ts)? pdinfo->pd : 0,
+				  opt);
 	}
     }
 
@@ -2192,16 +2239,16 @@ int print_fit_resid (const MODEL *pmod, const double **Z,
 		     const DATAINFO *pdinfo, PRN *prn)
 {
     FITRESID *fr;
+    int err = 0;
 
-    fr = get_fit_resid(pmod, Z, pdinfo);
-    if (fr == NULL) {
-	return 1;
+    fr = get_fit_resid(pmod, Z, pdinfo, &err);
+
+    if (!err) {
+	text_print_fit_resid(fr, pdinfo, prn);
+	free_fit_resid(fr);
     }
 
-    text_print_fit_resid(fr, pdinfo, prn);
-    free_fit_resid(fr);
-
-    return 0;
+    return err;
 }
 
 static void print_iter_val (double x, int i, int k, PRN *prn)
@@ -2284,651 +2331,6 @@ print_iter_info (int iter, double crit, int type, int k,
     }
 }
 
-/* apparatus for user-defined printf statements */
-
-#define PRINTF_DEBUG 0
-
-static int printf_escape (int c, PRN *prn)
-{
-    switch (c) {
-    case 'n':
-	pputc(prn, '\n');
-	break;
-    case 't':
-	pputc(prn, '\t');
-	break;
-    case 'v':
-	pputc(prn, '\v');
-	break;
-    case '\\':
-	pputc(prn, '\\');
-	break;
-    case '"':
-	pputc(prn, '"');
-	break;
-    default:
-	/* treat as literal backslash */
-	pputc(prn, '\\');
-	pputc(prn, c);
-    }
-
-    return 0;
-}
-
-/* various string argument variants, optionally followed
-   by "+ offset" */
-
-static char *printf_get_string (const char *s, const double **Z,
-				const DATAINFO *pdinfo, 
-				int t, int *err)
-{
-    char *ret = NULL;
-    char tmpstr[16], strarg[16];
-    const char *p = NULL, *q = NULL;
-    int v, offset = 0;
-    int len = 0;
-
-#if PRINTF_DEBUG
-    fprintf(stderr, "printf_get_string: looking at '%s'\n", s);
-#endif
-
-    if (*s == '@') {
-	char sname[VNAMELEN] = {0};
-	int n = gretl_varchar_spn(s + 1);
-
-	len = n;
-	if (len >= VNAMELEN) {
-	    len = VNAMELEN - 1;
-	}
-	strncat(sname, s + 1, len);
-	p = get_named_string(sname);
-	if (p != NULL) {
-	    len = strlen(p);
-	    q = s + 1 + n;
-	}
-    } else if (*s == '"') {
-	/* literal string */
-	p = strrchr(s + 1, '"');
-	if (p != NULL) {
-	    q = p + 1;
-	    len = p - s - 1;
-	    p = s + 1;
-	}
-    } else if (sscanf(s, "varname(%d)", &v)) {
-	/* name of variable identified by number */
-	if (v >= 0 && v < pdinfo->v) {
-	    p = pdinfo->varname[v];
-	    len = strlen(p);
-	    q = strchr(s, ')') + 1;
-	}
-    } else if (sscanf(s, "argname(%15[^)])", strarg)) {
-	/* name of function argument */
-	char *aname = gretl_func_get_arg_name(strarg);
-
-	if (aname != NULL) {
-	    strcpy(tmpstr, aname);
-	    p = tmpstr;
-	    len = strlen(p);
-	    q = strchr(s, ')') + 1;
-	    free(aname);
-	}
-    } else if (sscanf(s, "date(%15[^)])", strarg)) {
-	/* date string */
-	t = -1;
-	if (isdigit(*strarg)) {
-	    t = atoi(strarg);
-	} else {
-	    v = varindex(pdinfo, strarg);
-	    if (v < pdinfo->v) {
-		t = Z[v][0];
-	    }
-	}
-	if (t > 0 && t <= pdinfo->n) {
-	    ntodate(tmpstr, t - 1, pdinfo);
-	    p = tmpstr;
-	    len = strlen(p);
-	    q = strchr(s, ')') + 1;
-	}
-    } else if (!strncmp(s, "marker", 6) && pdinfo->S != NULL) {
-	/* observation label */
-	p = pdinfo->S[t];
-	len = strlen(p);
-	q = s + 6;
-    }
-
-    if (p != NULL) {
-	if (q != NULL) {
-	    while (isspace(*q)) q++;
-	    if (*q == '+') {
-		q++;
-		offset = atoi(q);
-		len -= offset;
-	    }
-	}
-	if (len >= 0) {
-	    ret = gretl_strndup(p + offset, len);
-	}
-    }
-
-    if (ret == NULL) {
-	ret = gretl_strdup("NA");
-    }
-
-    if (ret == NULL) {
-	*err = E_ALLOC;
-    }
-
-    return ret;
-}
-
-static double printf_get_scalar (char *s, double ***pZ,
-				 DATAINFO *pdinfo, int t, 
-				 int *err)
-{
-    double x = NADBL;
-    int v;
-
-#if PRINTF_DEBUG
-    fprintf(stderr, "printf_get_scalar: looking at '%s'\n", s);
-#endif
-
-    if (numeric_string(s)) {
-	return atof(s);
-    }
-
-    v = varindex(pdinfo, s);
-
-    if (v < pdinfo->v && var_is_series(pdinfo, v)) {
-	char genstr[32];
-
-	sprintf(genstr, "%s[%d]", s, t + 1);
-	x = generate_scalar(genstr, pZ, pdinfo, err);
-    } else {
-	x = generate_scalar(s, pZ, pdinfo, err);
-    }
-
-#if PRINTF_DEBUG
-    fprintf(stderr, "printf_get_scalar: returning %g\n", x);
-#endif
-
-    return x;
-}
-
-/* dup argv (s) up to the next free comma */
-
-static char *get_next_arg (const char *s, int *len, int *err)
-{
-    const char *p;
-    char *arg = NULL;
-    int par = 0, br = 0;
-    int quoted = 0;
-    int n = 0;
-
-    if (s == NULL) {
-	*err = E_PARSE;
-	return NULL;
-    }
-
-    *len = strspn(s, ", ");
-    s += *len;
-    p = s;
-    
-    while (*p) {
-	if (*p == '"' && (p == s || *(p-1) != '\\')) {
-	    quoted = !quoted;
-	}
-	if (!quoted) {
-	    if (*p == '(') par++;
-	    else if (*p == ')') par--;
-	    else if (*p == '[') br++;
-	    else if (*p == ']') br--;
-	}
-	if (!quoted && !par && !br && *p == ',') {
-	    break;
-	}
-	p++;
-    }
-
-    n = p - s;
-    *len += n;
-
-    if (n > 0) {
-	arg = gretl_strndup(s, n);
-	if (arg == NULL) {
-	    *err = E_ALLOC;
-	}
-    } else {
-	*err = E_PARSE;
-    }
-
-#if PRINTF_DEBUG
-    fprintf(stderr, "get_next_arg: got '%s'\n", arg);
-#endif
-
-    return arg;
-}
-
-/* dup format string up to the end of the current conversion:
-   e.g. "%10.4f", "%6g", "%.8g", %3s", "%.*s", "%#12.4g"
-*/
-
-static char *get_format_chunk (const char *s, int *fc, 
-			       int *len, int *wstar, int *pstar,
-			       int *err)
-{
-    const char *cnvchars = "eEfgGxduxs";
-    const char *numchars = "0123456789";
-    char *chunk = NULL;
-    const char *p = s;
-    int n;
-
-    p++; /* % */
-
-    /* '#', if present, must come first */
-    if (*p == '#') {
-	p++;
-    }
-
-    /* 'width' could be < 0 */
-    if (*p == '-') {
-	p++;
-    }
-
-    /* optional width? */
-    n = strspn(p, numchars);
-    if (n == 0 && *p == '*') {
-	/* variable version */
-	*wstar = 1;
-	p++;
-    } else if (n > 3) {
-	*err = E_PARSE;
-	return NULL;
-    } else {
-	p += n;
-    }
-
-    /* optional dot plus precision? */
-    if (*p == '.') {
-	p++;
-	n = strspn(p, numchars);
-	if (n == 0 && *p == '*') {
-	    *pstar = 1;
-	    p++;
-	} else if (n > 3) {
-	    *err = E_PARSE;
-	    return NULL;
-	} else {
-	    p += n;
-	}
-    }
-
-    /* now we should have a conversion character */
-    if (*p == '\0' || strchr(cnvchars, *p) == NULL) {
-	fprintf(stderr, "bad conversion '%c'\n", *p);
-	*err = E_PARSE;
-	return NULL;
-    }
-
-    *fc = *p++;
-    *len = n = p - s;
-    
-    if (n > 0) {
-	chunk = gretl_strndup(s, n);
-	if (chunk == NULL) {
-	    *err = E_ALLOC;
-	}
-    } else {
-	*err = E_PARSE;
-    }
-
-#if PRINTF_DEBUG
-    fprintf(stderr, "get_format_chunk: got '%s'\n", chunk);
-#endif
-
-    return chunk;
-}
-
-/* extract the next conversion spec from *pfmt, find and evaluate the
-   corresponding elements in *pargs, and print the result */
-
-static int print_arg (char **pfmt, char **pargs, 
-		      double ***pZ, DATAINFO *pdinfo,
-		      int t, PRN *prn)
-{
-    const char *intconv = "dxul";
-    char *fmt = NULL;
-    char *arg = NULL;
-    char *str = NULL;
-    double x = NADBL;
-    int flen = 0, alen = 0;
-    int wstar = 0, pstar = 0;
-    int wid = 0, prec = 0;
-    int fc = 0;
-    int err = 0;
-
-#if PRINTF_DEBUG
-    fprintf(stderr, "print_arg: *pfmt='%s', *pargs='%s'\n",
-	    *pfmt, *pargs);
-#endif
-
-    /* select current conversion format */
-    fmt = get_format_chunk(*pfmt, &fc, &flen, &wstar, &pstar, &err);
-    if (err) {
-	return err;
-    }
-
-    *pfmt += flen;
-
-    if (wstar) {
-	/* evaluate field width specifier */
-	arg = get_next_arg(*pargs, &alen, &err);
-	if (!err) {
-	    x = printf_get_scalar(arg, pZ, pdinfo, t, &err);
-	    if (!err && (na(x) || fabs(x) > 255)) {
-		err = E_DATA;
-	    }
-	}
-	free(arg);
-	if (err) {
-	    goto bailout;
-	}
-	*pargs += alen;
-	wid = x;
-    }
-
-    if (pstar) {
-	/* evaluate precision specifier */
-	arg = get_next_arg(*pargs, &alen, &err);
-	if (!err) {
-	    x = printf_get_scalar(arg, pZ, pdinfo, t, &err);
-	    if (!err && (na(x) || fabs(x) > 255)) {
-		err = E_DATA;
-	    }
-	}
-	free(arg);
-	if (err) {
-	    goto bailout;
-	}
-	*pargs += alen;
-	prec = x;
-    }
-
-    /* get next substantive arg */
-    arg = get_next_arg(*pargs, &alen, &err);
-    if (!err) {
-	if (fc == 's') {
-	    str = printf_get_string(arg, (const double **) *pZ, 
-				    pdinfo, t, &err);
-	} else {
-	    x = printf_get_scalar(arg, pZ, pdinfo, t, &err);
-	    if (!err && na(x)) {
-		fc = fmt[flen - 1] = 's';
-		str = gretl_strdup("NA");
-	    }
-	}
-	*pargs += alen;
-    }
-    free(arg);
-
-    if (err) {
-	goto bailout;
-    } 
-
-    /* do the actual printing */
-
-    if (fc == 's') {
-	if (wstar && pstar) {
-	    pprintf(prn, fmt, wid, prec, str);
-	} else if (wstar || pstar) {
-	    wid = (wstar)? wid : prec;
-	    pprintf(prn, fmt, wid, str);
-	} else {
-	    pprintf(prn, fmt, str);
-	}
-    } else if (strchr(intconv, fc)) {
-	if (wstar && pstar) {
-	    pprintf(prn, fmt, wid, prec, (int) x);
-	} else if (wstar || pstar) {
-	    wid = (wstar)? wid : prec;
-	    pprintf(prn, fmt, wid, (int) x);
-	} else {
-	    pprintf(prn, fmt, (int) x);
-	}
-    } else {
-	if (wstar && pstar) {
-	    pprintf(prn, fmt, wid, prec, x);
-	} else if (wstar || pstar) {
-	    wid = (wstar)? wid : prec;
-	    pprintf(prn, fmt, wid, x);
-	} else {
-	    pprintf(prn, fmt, x);
-	}
-    }
-
- bailout:
-
-    if (err) {
-	pputc(prn, '\n');
-    }
-
-    free(fmt);
-    free(str);
-    
-    return err;
-}
-
-/* split line into format and args, copying both parts */
-
-static int split_printf_line (const char *s, char *targ, int *sp,
-			      char **format, char **args)
-{
-    const char *p;
-    int n;
-
-    *sp = 0;
-
-    if (!strncmp(s, "printf ", 7)) {
-	s += 7;
-    } else if (!strncmp(s, "sprintf ", 8)) {
-	s += 8;
-	*sp = 1;
-    }
-
-    if (*sp) {
-	/* need a target name */
-	s += strspn(s, " ");
-	n = gretl_varchar_spn(s);
-	if (n == 0 || n >= VNAMELEN) {
-	    return E_PARSE;
-	} else {
-	    *targ = '\0';
-	    strncat(targ, s, n);
-	    s += n;
-	}
-    }
-
-    s += strspn(s, " ");
-    if (*s != '"' || *(s+1) == '\0') {
-	return E_PARSE;
-    }
-
-    s++;
-    p = s;
-
-    n = 0;
-    while (*s) {
-	if (*s == '"' && *(s-1) != '\\') {
-	    break;
-	}
-	n++;
-	s++;
-    }
-
-    if (n == 0) {
-	/* empty format string */
-	return 0; 
-    }
-
-    *format = gretl_strndup(p, n);
-    if (*format == NULL) {
-	return E_ALLOC;
-    }
-
-    s++;
-    s += strspn(s, " ");
-
-    if (*s != ',') {
-	/* empty args */
-	*args = NULL;
-	return 0;
-    }
-
-    s++;
-    s += strspn(s, " ");
-
-    *args = gretl_strdup(s);
-    if (*args == NULL) {
-	return E_ALLOC;
-    }
-
-    return 0;
-}
-
-static int real_do_printf (const char *line, double ***pZ, 
-			   DATAINFO *pdinfo, PRN *inprn, 
-			   int t)
-{
-    PRN *prn = inprn;
-    char *p, *q;
-    char targ[VNAMELEN];
-    char *format = NULL;
-    char *args = NULL;
-    int sp, err = 0;
-
-    gretl_error_clear();
-
-    *targ = '\0';
-
-    if (t < 0) {
-	t = pdinfo->t1;
-    }
-
-    err = split_printf_line(line, targ, &sp, &format, &args);
-    if (err) {
-	return err;
-    }
-
-    if (sp) {
-	/* "sprintf" */
-	if (*targ == '\0') {
-	    return E_PARSE;
-	}
-	prn = gretl_print_new(GRETL_PRINT_BUFFER);
-	if (prn == NULL) {
-	    return E_ALLOC;
-	}
-    } 
-
-#if PRINTF_DEBUG
-    fprintf(stderr, "do_printf: line = '%s'\n", line);
-    fprintf(stderr, " targ = '%s'\n", targ);
-    fprintf(stderr, " format = '%s'\n", format);
-    fprintf(stderr, " args = '%s'\n", args);
-#endif
-
-    p = format;
-    q = args;
-
-    while (*p && !err) {
-	if (*p == '%' && *(p+1) == '%') {
-	    pputc(prn, '%');
-	    p += 2;
-	} else if (*p == '%') {
-	    err = print_arg(&p, &q, pZ, pdinfo, t, prn);
-	} else if (*p == '\\') {
-	    err = printf_escape(*(p+1), prn);
-	    p += 2;
-	} else {
-	    pputc(prn, *p);
-	    p++;
-	}
-    }
-
-    if (sp) {
-	if (!err) {
-	   err = save_named_string(targ, gretl_print_get_buffer(prn),
-				   inprn);
-	}
-	gretl_print_destroy(prn);
-    } else if (err) {
-	pputc(prn, '\n');
-    }
-
-    free(format);
-    free(args);
-
-    return err;
-}
-
-/**
- * do_printf:
- * @line: command line.
- * @pZ: pointer to data array.
- * @pdinfo: dataset information.
- * @prn: printing struct.
- *
- * Implement a somewhat limited version of C's printf
- * for use in gretl scripts.
- *
- * Returns: 0 on success, non-zero on error.
- */
-
-int do_printf (const char *line, double ***pZ, 
-	       DATAINFO *pdinfo, PRN *prn)
-{
-    return real_do_printf(line, pZ, pdinfo, prn, -1);
-}
-
-/* The originating command is of form:
-
-     genr markers = format, arg1,...
-
-   We're assuming that we're just getting the part after
-   the equals sign here, in the variable s.
-*/
-
-int generate_obs_markers (const char *s, double ***pZ, DATAINFO *pdinfo)
-{
-    PRN *prn = gretl_print_new(GRETL_PRINT_BUFFER);
-    int t, err = 0;
-
-    if (prn == NULL) {
-	return E_ALLOC;
-    }
-
-    if (pdinfo->S == NULL) {
-	err = dataset_allocate_obs_markers(pdinfo);
-    }
-
-    if (!err) {
-	const char *buf;
-
-	for (t=0; t<pdinfo->n && !err; t++) {
-	    gretl_print_reset_buffer(prn);
-	    err = real_do_printf(s, pZ, pdinfo, prn, t);
-	    if (!err) {
-		buf = gretl_print_get_buffer(prn);
-		pdinfo->S[t][0] = '\0';
-		strncat(pdinfo->S[t], buf, OBSLEN - 1);
-	    }
-	}
-    }
-
-    gretl_print_destroy(prn);
-	
-    return err;
-}
-
 int in_usa (void)
 {
     static int ustime = -1;
@@ -3033,8 +2435,7 @@ static void rbuf_finalize (const char *s)
  * @buf: source buffer.
  *
  * This function works much like fgets, reading successive lines 
- * from a buffer rather than a file.  It differs from fgets in 
- * that it discards the line termination ("\n" or "\r\n") on output.  
+ * from a buffer rather than a file.  
  * Important note: use of bufgets() on a particular buffer must be 
  * preceded by a call to bufgets_init() on the same buffer, and must be
  * followed by a call to bufgets_finalize(), again on the same
@@ -3080,11 +2481,11 @@ char *bufgets (char *s, size_t size, const char *buf)
        whichever comes first */
     for (i=0; ; i++) {
 	s[i] = p[i];
-	if (p[i] == 0) {
+	if (p[i] == '\0') {
 	    break;
 	}
 	if (p[i] == '\r') {
-	    s[i] = 0;
+	    s[i] = '\0';
 	    if (p[i+1] == '\n') {
 		status = GOT_CRLF;
 	    } else {
@@ -3093,7 +2494,7 @@ char *bufgets (char *s, size_t size, const char *buf)
 	    break;
 	}
 	if (p[i] == '\n') {
-	    s[i] = 0;
+	    s[i] = '\0';
 	    status = GOT_LF;
 	    break;
 	}
@@ -3113,6 +2514,11 @@ char *bufgets (char *s, size_t size, const char *buf)
 	p += 2;
     }
 
+    /* replace newline */
+    if (status && i < size - 1) {
+	strcat(s, "\n");
+    }
+
     rbuf_set_point(buf, p);
 
     return s;
