@@ -77,8 +77,7 @@ int print_fit_resid (const MODEL *pmod,
 		     const double **Z, const DATAINFO *pdinfo, 
 		     PRN *prn);
 
-int text_print_forecast (const FITRESID *fr, 
-			 double ***pZ, DATAINFO *pdinfo, 
+int text_print_forecast (const FITRESID *fr, DATAINFO *pdinfo, 
 			 gretlopt opt, PRN *prn);
 
 void print_iter_info (int iter, double crit, int type, int k, 
@@ -99,11 +98,6 @@ void print_centered (const char *s, int width, PRN *prn);
 
 void gretl_printxn (double x, int n, PRN *prn);
 
-int do_printf (const char *line, double ***pZ, 
-	       DATAINFO *pdinfo, PRN *prn);
-
-int generate_obs_markers (const char *s, double ***pZ, DATAINFO *pdinfo);
-
 int in_usa (void);
 
 char *bufgets (char *s, size_t size, const char *buf);
