@@ -25,19 +25,57 @@
 #include "../../cephes/libprob.h"
 
 #include <errno.h>
+
+/**
+ * gamma_function:
+ * @x: argument.
+ *
+ * Returns: the gamma function of @x, or #NADBL on failure.
+ */
+
+double gamma_function (double x)
+{
+    double ret = cephes_gamma(x);
+
+    if (get_cephes_errno()) {
+	fprintf(stderr, "gamma_function: arg = %g\n", x);
+	ret = NADBL;
+    }
+
+    return ret;
+}
+
+/**
+ * log_gamma_function:
+ * @x: argument.
+ *
+ * Returns: the log gamma function of @x, or #NADBL on failure.
+ */
+
+double log_gamma_function (double x)
+{
+    double ret = cephes_lgamma(x);
+
+    if (get_cephes_errno()) {
+	fprintf(stderr, "log_gamma_function: arg = %g\n", x);
+	ret = NADBL;
+    }
+
+    return ret;
+}
  
 /**
  * binomial_cdf:
- * @k: maximum number of successes.
- * @n: number of trials.
  * @p: probability of success on each trial.
+ * @n: number of trials.
+ * @k: maximum number of successes. 
  *
  * Returns: the probability of @k or less successes on
  * @n trials given binomial probability @p, or
  * #NADBL on failure.
  */
 
-double binomial_cdf (int k, int n, double p)
+double binomial_cdf (double p, int n, int k)
 {
     double x = NADBL;
 
@@ -53,16 +91,16 @@ double binomial_cdf (int k, int n, double p)
 
 /**
  * binomial_cdf_comp:
- * @k: maximum number of successes.
- * @n: number of trials.
  * @p: probability of success on each trial.
+ * @n: number of trials.
+ * @k: maximum number of successes.
  *
  * Returns: the probability of @k + 1 or more successes on
  * @n trials given binomial probability @p, or
  * #NADBL on failure.
  */
 
-double binomial_cdf_comp (int k, int n, double p)
+double binomial_cdf_comp (double p, int n, int k)
 {
     double x = NADBL;
 
@@ -82,7 +120,7 @@ double binomial_cdf_comp (int k, int n, double p)
     equal to the given cumulative probability y.
 */
 
-static double binomial_cdf_inverse (int k, int n, double y)
+static double binomial_cdf_inverse (double y, int n, int k)
 {
     double p = NADBL;
 
@@ -98,7 +136,7 @@ static double binomial_cdf_inverse (int k, int n, double y)
 
 /* The following is no doubt horribly inefficient */
 
-static double binomial_critval (double a, double p, int n)
+static double binomial_critval (double p, int n, double a)
 {
     double pk, ac = 1 - a;
     int k;
@@ -108,7 +146,7 @@ static double binomial_critval (double a, double p, int n)
     }
 
     for (k=n; k>0; k--) {
-	pk = binomial_cdf(k, n, p);
+	pk = binomial_cdf(p, n, k);
 	if (pk < ac) {
 	    break;
 	}
@@ -277,15 +315,15 @@ double normal_pvalue_1 (double x)
 
 /**
  * student_cdf:
- * @x: the cutoff point in the distribution.
  * @df: degrees of freedom.
+ * @x: the cutoff point in the distribution.
  * 
  * Returns: the integral from minus infinity to @x of
  * the Student's t distribution with @df degrees of freedom, or
  * #NADBL on failure.
  */
 
-double student_cdf (double x, int df)
+double student_cdf (int df, double x)
 {
     double p = NADBL;
 
@@ -301,15 +339,15 @@ double student_cdf (double x, int df)
 
 /**
  * student_cdf_comp:
- * @x: the cutoff point in the distribution.
  * @df: degrees of freedom.
+ * @x: the cutoff point in the distribution.
  * 
  * Returns: the integral from @x to infinity of
  * the t distribution with @df degrees of freedom, or
  * #NADBL on failure.
  */
 
-double student_cdf_comp (double x, int df)
+static double student_cdf_comp (int df, double x)
 {
     double p = NADBL;
 
@@ -332,7 +370,15 @@ double student_cdf_comp (double x, int df)
     return p;
 }
 
-static double normal_cdf_comp (double x)
+/**
+ * normal_cdf_comp:
+ * @x: the cutoff point in the distribution.
+ * 
+ * Returns: the integral from @x to infinity of the standard 
+ * normal distribution, or #NADBL on failure.
+ */
+
+double normal_cdf_comp (double x)
 {
     double p;
 
@@ -355,15 +401,15 @@ static double normal_cdf_comp (double x)
 
 /**
  * student_pvalue_2:
- * @x: the cutoff point in the distribution.
  * @df: degrees of freedom.
+ * @x: the cutoff point in the distribution.
  * 
  * Returns: the probability that t(@df) is greater than @x
  * (two-sided, using the absolute value of @x), or
  * #NADBL on failure.
  */
 
-double student_pvalue_2 (double x, int df)
+double student_pvalue_2 (int df, double x)
 {
     double p = NADBL;
 
@@ -387,15 +433,15 @@ double student_pvalue_2 (double x, int df)
 
 /**
  * student_critval:
- * @a: right-tail probability.
  * @df: degrees of freedom.
+ * @a: right-tail probability.
  *
  * Returns: the argument x such that the integral from x to 
  * infinity of the t(@df) density is equal to the given
  * probability @a, or #NADBL on failure.
  */
 
-double student_critval (double a, double df)
+static double student_critval (double df, double a)
 {
     double x;
 
@@ -424,7 +470,17 @@ double student_critval (double a, double df)
     return x;
 }
 
-static double student_cdf_inverse (double a, double df)
+/**
+ * student_cdf_inverse:
+ * @df: degrees of freedom.
+ * @a: probability.
+ *
+ * Returns: the argument x such that the integral from 
+ * minus infinity to @x of the t(@df) density is equal to 
+ * the given probability @a, or #NADBL on failure.
+ */
+
+double student_cdf_inverse (double df, double a)
 {
     double x;
 
@@ -447,15 +503,15 @@ static double student_cdf_inverse (double a, double df)
 
 /**
  * chisq_cdf:
- * @x: the cutoff point in the distribution.
  * @df: degrees of freedom.
+ * @x: the cutoff point in the distribution.
  * 
  * Returns: the integral from 0 to @x of the chi-squared
  * distribution with @df degrees of freedom, or #NADBL
  * on failure.
  */
 
-double chisq_cdf (double x, int df)
+double chisq_cdf (int df, double x)
 {
     double p = NADBL;
 
@@ -471,15 +527,15 @@ double chisq_cdf (double x, int df)
 
 /**
  * chisq_cdf_comp:
- * @x: the cutoff point in the distribution.
  * @df: degrees of freedom.
+ * @x: the cutoff point in the distribution.
  * 
  * Returns: the integral from @x to infinity of the chi-squared
  * distribution with @df degrees of freedom, or #NADBL
  * on failure.
  */
 
-double chisq_cdf_comp (double x, int df)
+double chisq_cdf_comp (int df, double x)
 {
     double p = NADBL;
 
@@ -495,15 +551,15 @@ double chisq_cdf_comp (double x, int df)
 
 /**
  * chisq_critval:
- * @a: right-tail probability.
  * @df: degrees of freedom.
+ * @a: right-tail probability.
  * 
  * Returns: the Chi-square argument x such that the integral
  * from x to infinity of the Chi-square density is equal
  * to the given probability @a, or #NADBL on failure.
  */
 
-double chisq_critval (double a, int df)
+static double chisq_critval (int df, double a)
 {
     double x = NADBL;
 
@@ -517,7 +573,7 @@ double chisq_critval (double a, int df)
     return x;
 }
 
-static double chisq_cdf_inverse (double a, int df)
+static double chisq_cdf_inverse (int df, double a)
 {
     double x = NADBL;
 
@@ -533,15 +589,15 @@ static double chisq_cdf_inverse (double a, int df)
 
 /**
  * snedecor_cdf:
- * @x: the cutoff point in the distribution.
  * @dfn: numerator degrees of freedom.
  * @dfd: denominator degrees of freedom.
+ * @x: the cutoff point in the distribution.
  * 
  * Returns: the integral of the F distribution with @dfn and
  * @dfd degrees of freedom, from 0 to @x, or #NADBL on failure.
  */
 
-double snedecor_cdf (double x, int dfn, int dfd)
+static double snedecor_cdf (int dfn, int dfd, double x)
 {
     double p = NADBL;
 
@@ -557,16 +613,16 @@ double snedecor_cdf (double x, int dfn, int dfd)
 
 /**
  * snedecor_cdf_comp:
- * @x: the cutoff point in the distribution.
  * @dfn: numerator degrees of freedom.
  * @dfd: denominator degrees of freedom.
+ * @x: the cutoff point in the distribution.
  * 
  * Returns: the integral of the F distribution with @dfn and
  * @dfd degrees of freedom, from @x to infinity, or #NADBL 
  * on failure.
  */
 
-double snedecor_cdf_comp (double x, int dfn, int dfd)
+double snedecor_cdf_comp (int dfn, int dfd, double x)
 {
     double p = NADBL;
 
@@ -582,16 +638,16 @@ double snedecor_cdf_comp (double x, int dfn, int dfd)
 
 /**
  * snedecor_critval:
- * @a: right-tail probability.
  * @dfn: numerator degrees of freedom.
  * @dfd: denominator degrees of freedom.
+ * @a: right-tail probability.
  * 
  * Returns: the F argument x such that the integral
  * from x to infinity of the F density is equal
  * to the given probability @a, or #NADBL on failure.
  */
 
-double snedecor_critval (double a, int dfn, int dfd)
+double snedecor_critval (int dfn, int dfd, double a)
 {
     double x = NADBL;
 
@@ -605,7 +661,7 @@ double snedecor_critval (double a, int dfn, int dfd)
     return x;
 }
 
-static double snedecor_cdf_inverse (double a, int dfn, int dfd)
+static double snedecor_cdf_inverse (int dfn, int dfd, double a)
 {
     double x = NADBL;
 
@@ -619,6 +675,114 @@ static double snedecor_cdf_inverse (double a, int dfn, int dfd)
     return x;
 }
 
+/* PDFs */
+
+static double Binv (double p, double q)
+{
+    double f = NADBL;
+
+    errno = 0;
+
+    if (p > 0 && q > 0) {
+	double x1 = log_gamma_function(p + q);
+	double x2 = log_gamma_function(p);
+	double x3 = log_gamma_function(q);
+
+	if (!na(x1) && !na(x2) && !na(x3)) {
+	    f = exp(x1 - x2 - x3);
+	    if (errno) {
+		f = NADBL;
+	    }
+	}
+    }
+
+    return f;
+}    
+
+static double snedecor_pdf (int m, int n, double x)
+{
+    double fx = NADBL;
+
+    errno = 0;
+
+    if (m > 0 && n > 0 && x > 0) {
+	double xm = m, xn = n;
+	double x1 = Binv(0.5*xm, 0.5*xn);
+	double x2 = pow(xm / xn, 0.5*xm);
+	double x3 = pow(x, 0.5*xm - 1.0);
+	double x4 = pow(1.0 + xm/xn * x, 0.5 * (xm+xn));
+
+	if (!errno && !na(x1)) {
+	    fx = x1 * x2 * x3 / x4;
+	}
+    }
+
+    return fx;
+}
+
+static double chisq_pdf (int m, double x)
+{
+    double fx = NADBL;
+
+    errno = 0;
+
+    if (m > 0 && x >= 0) {
+	double m2 = m / 2.0;
+	double x1 = pow(.5, m2);
+	double x2 = gamma_function(m2);
+	double x3 = pow(x, m2 - 1.0);
+	double x4 = exp(-x / 2.0);
+
+	if (!errno && !na(x2)) {
+	    fx = (x1/x2) * x3 * x4;
+	}
+    }
+
+    return fx;
+}
+
+static double student_pdf (double m, double x)
+{
+    double fx = NADBL;
+
+    errno = 0;
+
+    if (m > 0) {
+	double x1 = Binv(0.5 * m, 0.5) / sqrt(m);
+	double x2 = m / (m + x * x);
+	double x3 = 0.5 * (m + 1.0);
+
+	if (!errno && !na(x1)) {
+	    fx = x1 * pow(x2, x3);
+	    if (errno) {
+		fx = NADBL;
+	    }
+	}
+    }
+
+    return fx;
+}
+
+static double weibull_pdf (double k, double l, double x)
+{
+    double fx = NADBL;
+
+    errno = 0;
+
+    if (k > 0 && l > 0 && x >= 0) {
+	double x1 = k / l;
+	double x2 = pow(x / l, k - 1.0);
+	double x3 = pow(x / l, k);
+	double x4 = exp(-x3);
+
+	if (!errno) {
+	    fx = x1 * x2 * x4;
+	}
+    }
+
+    return fx;
+}
+
 /**
  * normal_cdf:
  * @x: double-precision value.
@@ -642,7 +806,7 @@ double normal_cdf (double x)
  * normal_cdf_inverse:
  * @x: double-precision value.
  * 
- * Returns the argument, y, for which the area under the
+ * Returns: the argument, y, for which the area under the
  * Gaussian probability density function (integrated from
  * minus infinity to y) is equal to x, or #NADBL on failure.
  */
@@ -658,19 +822,6 @@ double normal_cdf_inverse (double x)
     return y;
 }
 
-/**
- * normal_pdf:
- * @x: double-precision value.
- * 
- * Returns: the value of the standard normal PDF evaluated
- * at @x.
- */
-
-double normal_pdf (double x)
-{
-    return (1 / sqrt(M_2PI)) * exp(-0.5 * x * x);
-}
-
 /**
  * normal_critval:
  * @a: right-tail probability.
@@ -697,6 +848,41 @@ double normal_critval (double a)
     return z;
 }
 
+/**
+ * normal_pdf:
+ * @x: double-precision value.
+ * 
+ * Returns: the value of the standard normal PDF evaluated
+ * at @x.
+ */
+
+double normal_pdf (double x)
+{
+    return (1 / sqrt(M_2PI)) * exp(-0.5 * x * x);
+}
+
+/**
+ * general_normal_pdf:
+ * @mu: mean.
+ * @sigma: standard deviation.
+ * @x: abscissa value.
+ * 
+ * Returns: the value of the normal PDF with mean @mu
+ * and standard deviation @sigma evaluated at @x.
+ */
+
+double general_normal_pdf (double mu, double sigma, double x)
+{
+    if (na(x) || na(mu) || na(sigma) || sigma <= 0.0) {
+	return NADBL;
+    } else {
+	double d = x - mu;
+
+	return (1 / (sigma * sqrt(M_2PI))) * 
+	    exp(-0.5 * d * d / (sigma * sigma));
+    }
+}
+
 /**
  * log_normal_pdf:
  * @x: double-precision value.
@@ -783,9 +969,25 @@ double bvnorm_cdf (double a, double b, double rho)
     return ret;
 }
 
+/**
+ * gamma_cdf:
+ * @s1: first parameter.
+ * @s2: second parameter.
+ * @x: reference value.
+ * @control: see below.
+ *
+ * Calculates the value of the CDF of the gamma distribution
+ * at @x.  If @control equals 1, then it is assumed that the
+ * parameters @s1 and @s2 represent the shape and scale,
+ * respectively, otherwise it is assumed they give mean and
+ * variance.
+
+ * Returns: the calculated probability, or #NADBL on failure.
+ */
+
 double gamma_cdf (double s1, double s2, double x, int control)
 {
-    double shape, scale, p;
+    double scale, shape, p;
 
     if (control == 1) {
 	shape = s1; 
@@ -793,9 +995,11 @@ double gamma_cdf (double s1, double s2, double x, int control)
     } else {
 	scale = s2 / s1; 
 	shape = s1 / scale; 
-    }	
+    }
 
-    p = gdtr(s2, s1, x);
+    /* cephes expects inverse-scale, shape, I think >8-} */
+
+    p = gdtr(1.0 / scale, shape, x);
     if (get_cephes_errno()) {
 	p = NADBL;
     }
@@ -803,10 +1007,21 @@ double gamma_cdf (double s1, double s2, double x, int control)
     return p;
 }
 
-/* Control 1 : s1, s2 = shape, scale
-           2 : s1, s2 = mean, variance
-   Returns NADBL on error.
-*/
+/**
+ * gamma_cdf_comp:
+ * @s1: first parameter.
+ * @s2: second parameter.
+ * @x: reference value.
+ * @control: see below.
+ *
+ * Calculates the complement of the CDF of the gamma distribution
+ * at @x.  If @control equals 1, then it is assumed that the
+ * parameters @s1 and @s2 represent the shape and scale,
+ * respectively, otherwise it is assumed they give mean and
+ * variance.
+
+ * Returns: the calculated probability, or #NADBL on failure.
+ */
 
 double gamma_cdf_comp (double s1, double s2, double x, int control)
 {
@@ -828,6 +1043,35 @@ double gamma_cdf_comp (double s1, double s2, double x, int control)
     return p;
 }
 
+/**
+ * gamma_pdf:
+ * @shape: shape parameter.
+ * @scale: scale parameter.
+ * @x: reference value.
+ *
+ * Returns: the value of the gamma pdf at @x, or #NADBL on failure.
+ */
+
+double gamma_pdf (double shape, double scale, double x)
+{
+    double p = NADBL;
+
+    errno = 0;
+
+    if (shape > 0 && scale > 0 && x > 0) {
+	double x1 = pow(x, shape - 1.0);
+	double x2 = exp(-x/scale);
+	double x3 = pow(scale, shape);
+	double x4 = gamma_function(shape);
+
+	if (!errno && !na(x4)) {
+	    p = x1 * x2 / (x3 * x4);
+	}
+    }
+
+    return p;
+}
+
 /**
  * poisson_cdf:
  * @lambda: mean (also variance).
@@ -874,30 +1118,6 @@ static double poisson_cdf_comp (double lambda, int k)
     return x;
 }
 
-/**
- * poisson_cdf_inverse:
- * @k: test value.
- * @p: cumulative probability.
- *
- * Returns: the Poisson parameter such that the integral
- * from 0 to @k of the Poisson density is equal to the
- * given probability @p.
- */
-
-static double poisson_cdf_inverse (int k, double p)
-{
-    double x = NADBL;
-
-    if (k >= 0 && p >= 0 && p <= 1) {
-	x = pdtri(k, p);
-	if (get_cephes_errno()) {
-	    x = NADBL;
-	}
-    }
-
-    return x;
-}
-
 static double poisson_pmf (double lambda, int k)
 {
     double den, l0, p;
@@ -932,7 +1152,7 @@ static double poisson_pmf (double lambda, int k)
    or perhaps binary search.
 */
 
-static double poisson_critval (double a, double mu)
+static double poisson_critval (double mu, double a)
 {
     double pk = 0.0;
     double ac = 1 - a;
@@ -957,6 +1177,82 @@ static double poisson_critval (double a, double mu)
     return (double) k;
 }
 
+/**
+ * poisson_cdf_inverse:
+ * @k: test value.
+ * @p: cumulative probability.
+ *
+ * Returns: the Poisson parameter such that the integral
+ * from 0 to @k of the Poisson density is equal to the
+ * given probability @p.
+ */
+
+static double poisson_cdf_inverse (int k, double p)
+{
+    double x = NADBL;
+
+    if (k >= 0 && p >= 0 && p <= 1) {
+	x = pdtri(k, p);
+	if (get_cephes_errno()) {
+	    x = NADBL;
+	}
+    }
+
+    return x;
+}
+
+static double 
+weibull_critval (double shape, double scale, double rtail)
+{
+    double ret = NADBL;
+
+    if (shape > 0 && scale > 0 && rtail > 0 && rtail < 1) {
+	ret = scale * pow(-log(rtail), 1.0 / shape);
+    }
+
+    return ret;
+}
+
+/**
+ * weibull_cdf:
+ * @shape: shape parameter > 0.
+ * @scale: scale parameter > 0.
+ * @x: test value.
+ *
+ * Returns: the probability of X <= @x, for X an r.v. that follows
+ * the Weibull distribution with parameters @shape and @scale.
+ */
+
+static double weibull_cdf (double shape, double scale, double x)
+{
+    double ret = NADBL;
+
+    if (shape > 0 && scale > 0) {
+	if (x == 0.0) {
+	    ret = 0.0;
+	} else if (x > 0.0) {
+	    ret = 1.0 - exp(-pow(x/scale, shape));
+	}
+    }
+
+    return ret;
+}
+
+static double weibull_cdf_comp (double shape, double scale, double x)
+{
+    double ret = NADBL;
+
+    if (shape > 0 && scale > 0) {
+	if (x == 0.0) {
+	    ret = 1.0;
+	} else if (x > 0.0) {
+	    ret = exp(-pow(x/scale, shape));
+	}
+    }
+
+    return ret;
+}
+
 /* order in x: [params], alpha, critval */
 
 void print_critval (char st, double *x, double c, PRN *prn)
@@ -988,6 +1284,10 @@ void print_critval (char st, double *x, double c, PRN *prn)
 	pprintf(prn, "Poisson (mean = %g)", x[0]);
 	apos = 1;
 	break;
+    case 'W':
+	pprintf(prn, "Weibull (shape = %g, scale = %g)", x[0], x[1]);
+	apos = 2;
+	break;
     }
 
     a = x[apos];
@@ -1016,6 +1316,27 @@ static void dparm_set (const double *p)
     }
 }
 
+double gretl_get_cdf_inverse (char st, double *p)
+{
+    double x = NADBL;
+
+    if (st == 'z') {
+	x = normal_cdf_inverse(p[0]);
+    } else if (st == 't') {
+	x = student_cdf_inverse(p[0], p[1]);
+    } else if (st == 'X') {
+	x = chisq_cdf_inverse((int) p[0], p[1]);
+    } else if (st == 'F') {
+	x = snedecor_cdf_inverse((int) p[0], (int) p[1], p[2]);
+    } else if (st == 'B') {
+	x = binomial_cdf_inverse((int) p[0], (int) p[1], p[2]);
+    } else if (st == 'P') {
+	x = poisson_cdf_inverse((int) p[0], p[1]);
+    } 
+
+    return x;
+}
+
 double gretl_get_critval (char st, double *p)
 {
     double x = NADBL;
@@ -1023,16 +1344,18 @@ double gretl_get_critval (char st, double *p)
     if (st == 'z') {
 	x = normal_critval(p[0]);
     } else if (st == 't') {
-	x = student_critval(p[1], p[0]);
+	x = student_critval(p[0], p[1]);
     } else if (st == 'X') {	
-	x = chisq_critval(p[1], (int) p[0]);
+	x = chisq_critval((int) p[0], p[1]);
     } else if (st == 'F') {
-	x = snedecor_critval(p[2], (int) p[0], (int) p[1]);
+	x = snedecor_critval((int) p[0], (int) p[1], p[2]);
     } else if (st == 'B') {
-	x = binomial_critval(p[2], p[0], (int) p[1]);
+	x = binomial_critval(p[0], (int) p[1], p[2]);
     } else if (st == 'P') {
-	x = poisson_critval(p[1], p[0]);
-    }
+	x = poisson_critval(p[0], p[1]);
+    } else if (st == 'W') {
+	x = weibull_critval(p[0], p[1], p[2]);
+    } 
 
     return x;
 }
@@ -1044,40 +1367,42 @@ double gretl_get_cdf (char st, double *p)
     if (st == 'z') {
 	x = normal_cdf(p[0]);
     } else if (st == 't') {
-	x = student_cdf(p[1], (int) p[0]);
+	x = student_cdf((int) p[0], p[1]);
     } else if (st == 'X') {
-	x = chisq_cdf(p[1], (int) p[0]);
+	x = chisq_cdf((int) p[0], p[1]);
     } else if (st == 'F') {
-	x = snedecor_cdf(p[2], (int) p[0], (int) p[1]);
+	x = snedecor_cdf((int) p[0], (int) p[1], p[2]);
     } else if (st == 'G') {
 	x = gamma_cdf(p[0], p[1], p[2], 1);
     } else if (st == 'B') {
-	x = binomial_cdf((int) p[2], (int) p[1], p[0]);
+	x = binomial_cdf(p[0], (int) p[1], (int) p[2]);
     } else if (st == 'D') {
-	x = bvnorm_cdf(p[0], p[1], p[2]);
+	x = bvnorm_cdf(p[1], p[2], p[0]);
     } else if (st == 'P') {
 	x = poisson_cdf(p[0], (int) p[1]);
+    } else if (st == 'W') {
+	x = weibull_cdf(p[0], p[1], p[2]);
     }
 
     return x;
 }
 
-double gretl_get_cdf_inverse (char st, double *p)
+double gretl_get_pdf (char st, double *p)
 {
     double x = NADBL;
 
     if (st == 'z') {
-	x = normal_cdf_inverse(p[0]);
+	x = normal_pdf(p[0]);
     } else if (st == 't') {
-	x = student_cdf_inverse(p[1], p[0]);
+	x = student_pdf(p[0], p[1]);
     } else if (st == 'X') {
-	x = chisq_cdf_inverse(p[1], (int) p[0]);
+	x = chisq_pdf((int) p[0], p[1]);
     } else if (st == 'F') {
-	x = snedecor_cdf_inverse(p[2], (int) p[0], (int) p[1]);
-    } else if (st == 'B') {
-	x = binomial_cdf_inverse((int) p[2], (int) p[1], p[0]);
-    } else if (st == 'P') {
-	x = poisson_cdf_inverse((int) p[0], p[1]);
+	x = snedecor_pdf((int) p[0], (int) p[1], p[2]);
+    } else if (st == 'G') {
+	x = gamma_pdf(p[0], p[1], p[2]);
+    } else if (st == 'W') {
+	x = weibull_pdf(p[0], p[1], p[2]);
     }
 
     return x;
@@ -1090,17 +1415,19 @@ double gretl_get_pvalue (char st, const double *p)
     if (st == 'z') {
 	x = normal_cdf_comp(p[0]);
     } else if (st == 't') {
-	x = student_cdf_comp(p[1], (int) p[0]);
+	x = student_cdf_comp((int) p[0], p[1]);
     } else if (st == 'X') {
-	x = chisq_cdf_comp(p[1], (int) p[0]);
+	x = chisq_cdf_comp((int) p[0], p[1]);
     } else if (st == 'F') {
-	x = snedecor_cdf_comp(p[2], (int) p[0], (int) p[1]);
+	x = snedecor_cdf_comp((int) p[0], (int) p[1], p[2]);
     } else if (st == 'G') {
 	x = gamma_cdf_comp(p[0], p[1], p[2], 1);
     } else if (st == 'B') {
-	x = binomial_cdf_comp((int) p[2], (int) p[1], p[0]);
+	x = binomial_cdf_comp(p[0], (int) p[1], (int) p[2]);
     } else if (st == 'P') {
 	x = poisson_cdf_comp(p[0], (int) p[1]);
+    } else if (st == 'W') {
+	x = weibull_cdf_comp(p[0], p[1], p[2]);
     }
 
     dparm_set(p);
@@ -1149,15 +1476,19 @@ double *gretl_get_random_series (char st, const double *p,
 
 	*err = gretl_rand_gamma(x, pdinfo->t1, pdinfo->t2, shape, scale);
     } else if (st == 'B') {
-	int n = p[0];
-	double pr = p[1];
+	double pr = p[0];
+	int n = p[1];
 
 	*err = gretl_rand_binomial(x, pdinfo->t1, pdinfo->t2, n, pr);
     } else if (st == 'P') {
 	double m = p[0]; /* FIXME? */
 
 	gretl_rand_poisson(x, pdinfo->t1, pdinfo->t2, &m, 0);
-    }
+    } else if (st == 'W') {
+	double shape = p[0], scale = p[1];
+
+	*err = gretl_rand_weibull(x, pdinfo->t1, pdinfo->t2, shape, scale);
+    }	
 
     return x;
 }
@@ -1217,7 +1548,7 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
 	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		    2 * pv, 1 - 2 * pv);
 	} else {
-	    pc = student_cdf(p[1], (int) p[0]);
+	    pc = student_cdf((int) p[0], p[1]);
 	    pprintf(prn, _("(to the left: %g)\n"), pc);
 	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		    2 * pc, 1 - 2 * pc);
@@ -1231,7 +1562,7 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
 	pprintf(prn, "\n%s(%d): ", _("Chi-square"), (int) p[0]);
 	err = print_pv_string(p[1], pv, prn);
 	if (err) return;
-	pc = chisq_cdf(p[1], p[0]);
+	pc = chisq_cdf(p[0], p[1]);
 	pprintf(prn, _("(to the left: %g)\n"), pc);
 	break;
 
@@ -1241,7 +1572,7 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
 	pprintf(prn, "\nF(%d, %d): ", (int) p[0], (int) p[1]);
 	err = print_pv_string(p[2], pv, prn);
 	if (err) return;
-	pc = snedecor_cdf(p[2], (int) p[0], (int) p[1]);
+	pc = snedecor_cdf((int) p[0], (int) p[1], p[2]);
 	pprintf(prn, _("(to the left: %g)\n"), pc);
 	break;
 
@@ -1260,11 +1591,11 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
 	pprintf(prn, _("\nBinomial (p = %g, n = %d):"
 		       "\n Prob(x > %d) = %g\n"), 
 		p[0], (int) p[1], (int) p[2], pv);
-	pc = binomial_cdf(p[2], p[1], p[0]);
+	pc = binomial_cdf(p[0], p[1], p[2]);
 	if (p[2] > 0) {
 	    pprintf(prn, _(" Prob(x <= %d) = %g\n"), (int) p[2], pc);
 	    pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) p[2],
-		    pc - binomial_cdf(p[2] - 1, p[1], p[0]));
+		    pc - binomial_cdf(p[0], p[1], p[2] - 1));
 	} else {
 	    pprintf(prn, _(" Prob(x = %d) = %g\n"), (int) p[2], pc);
 	}
@@ -1286,6 +1617,17 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
 	}
 	break;	
 
+    case 'w':
+    case 'W':
+    case '9':
+	pprintf(prn, _("\nWeibull (shape = %g, scale = %g): "), 
+		p[0], p[1]);
+	err = print_pv_string(p[2], pv, prn);
+	if (err) return;
+	pc = weibull_cdf(p[0], p[1], p[2]);
+	pprintf(prn, _("(to the left: %g)\n"), pc);
+	break;
+
     default:
 	break;
     }
@@ -1301,6 +1643,7 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
  * pvalue G mean variance x (Gamma distribution).
  * pvalue B prob n x (Binomial distribution).
  * pvalue P mean k (Poisson distribution).
+ * pvalue W shape scale x (Weibull distribution).
  * @pZ: pointer to the data array.
  * @pdinfo: data information struct.
  * @prn: gretl printing struct.
@@ -1359,3 +1702,22 @@ int batch_pvalue (const char *str,
     return err;
 }
 
+gretl_matrix *gretl_get_DW (int n, int k, int *err)
+{
+    void *handle;
+    int (*dw_lookup) (int, int, gretl_matrix **);
+    gretl_matrix *m = NULL;
+
+    dw_lookup = get_plugin_function("dw_lookup", &handle);
+
+    if (dw_lookup == NULL) {
+	*err = E_FOPEN;
+	return NULL;
+    }
+
+    *err = (*dw_lookup) (n, k, &m);
+    close_plugin(handle);
+
+    return m;
+}
+
