@@ -20,9 +20,13 @@
 #ifndef PVALUES_H
 #define PVALUES_H
 
-double binomial_cdf (int k, int n, double p);
+double gamma_function (double x);
 
-double binomial_cdf_comp (int k, int n, double p);
+double log_gamma_function (double x);
+
+double binomial_cdf (double p, int n, int k);
+
+double binomial_cdf_comp (double p, int n, int k);
 
 double x_factorial (double x);
 
@@ -32,30 +36,24 @@ double normal_pvalue_2 (double x);
 
 double normal_pvalue_1 (double x);
 
-double student_cdf (double x, int df);
+double student_pvalue_2 (int df, double x);
 
-double student_cdf_comp (double x, int df);
+double chisq_cdf (int df, double x);
 
-double student_pvalue_2 (double x, int df);
+double chisq_cdf_comp (int df, double x);
 
-double student_critval (double a, double df);
+double snedecor_cdf_comp (int dfn, int dfd, double x);
 
-double chisq_cdf (double x, int df);
-
-double chisq_cdf_comp (double x, int df);
-
-double chisq_critval (double a, int df);
-
-double snedecor_cdf (double x, int dfn, int dfd);
-
-double snedecor_cdf_comp (double x, int dfn, int dfd);
-
-double snedecor_critval (double a, int dfn, int dfd);
+double snedecor_critval (int dfn, int dfd, double a);
 
 double normal_cdf (double x);
 
 double normal_cdf_inverse (double x);
 
+double normal_cdf_comp (double x);
+
+double student_cdf_inverse (double df, double a);
+
 double normal_pdf (double x);
 
 double normal_critval (double a);
@@ -76,6 +74,8 @@ double cephes_lgamma (double x);
 
 double gretl_get_pvalue (char st, const double *p);
 
+double gretl_get_pdf (char st, double *p);
+
 double gretl_get_cdf (char st, double *p);
 
 double gretl_get_cdf_inverse (char st, double *p);
@@ -94,6 +94,8 @@ void print_pvalue (char st, double *p, double pv, PRN *prn);
 
 void print_critval (char st, double *x, double c, PRN *prn);
 
+gretl_matrix *gretl_get_DW (int n, int k, int *err);
+
 #endif /* PVALUES_H */
 
 
