@@ -25,7 +25,7 @@
 #include "gretl_panel.h"
 #include "estim_private.h"
 
-#include "f2c.h"
+#include "gretl_f2c.h"
 #include "clapack_double.h"
 
 #define QR_RCOND_MIN 1e-15 /* experiment with this? */
@@ -825,6 +825,8 @@ static int QR_decomp_plus (gretl_matrix *Q, gretl_matrix *R, int *rank)
     return err;
 }
 
+#define REDEBUG 0
+
 static void
 drop_redundant_vars (MODEL *pmod, gretl_matrix *R, int rank, gretlopt opt)
 {
@@ -841,7 +843,7 @@ drop_redundant_vars (MODEL *pmod, gretl_matrix *R, int rank, gretlopt opt)
     }
 
 #if REDEBUG
-    printlist(pmod->list, "pmod->list, into drop_redundant_vars\n");
+    printlist(pmod->list, "pmod->list, into drop_redundant_vars");
     fprintf(stderr, "rank = %d\n", rank);
 #endif
 
