@@ -134,7 +134,7 @@ void gretl_two_snormals (double *z1, double *z2)
  * @t1: start of the fill range.
  * @t2: end of the fill range.
  * @min: lower bound of range.
- * @min: upper bound of range.
+ * @max: upper bound of range.
  *
  * Fill the selected subset of array @a with pseudo-random drawings
  * from the uniform distribution on @min to @max, using the Mersenne
@@ -256,6 +256,110 @@ gretl_rand_normal_full (double *a, int t1, int t2,
     return 0;
 }
 
+/**
+ * gretl_rand_gamma:
+ * @a: target array.
+ * @t1: start of the fill range.
+ * @t2: end of the fill range.
+ * @shape: shape parameter.
+ * @scale: scale parameter.
+ *
+ * Fill the selected range of array @a with pseudo-random drawings
+ * from the specified gamma distribution.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int gretl_rand_gamma (double *a, int t1, int t2, 
+		      double shape, double scale) 
+{
+    double *U = NULL;
+    double e = 2.718281828459045235;
+    double delta, dinv = 0, d1 = 0;
+    double u, v, x, y, u0 = 0;
+    int k, i, t;
+
+    if (shape <= 0 || scale <= 0) {
+	return E_DATA;
+    }
+
+    k = shape;
+    if (k > 0) {
+	U = malloc(k * sizeof *U);
+	if (U == NULL) {
+	    return E_ALLOC;
+	}
+    }
+
+    delta = shape - k;
+
+    if (delta > 0) {
+	d1 = delta - 1;
+	dinv = 1 / delta;
+	u0 = e / (e + delta);
+    }
+
+    /* 
+       Case of shape < 1 from Kundu and Gupta, "A convenient way of
+       generating gamma random variables using generalized exponential
+       distribution", Computational Statistics and Data Analysis, 51
+       (2007).  Case of shape >= 1 taken from the Wikipedia entry on
+       the gamma distribution.
+    */
+
+    for (t=t1; t<=t2; t++) {
+	a[t] = 0.0;
+	if (shape < 1) {
+	    double ex2;
+
+	    while (1) {
+		u = gretl_one_uniform();
+		v = gretl_one_uniform();
+		x = -2 * log(1 - pow(u, dinv));
+		ex2 = exp(-x/2);
+		u0 = pow(x, d1) * ex2;
+		u0 /= pow(2.0, d1) * pow((1-ex2), d1);
+		if (v <= u0) {
+		    a[t] = x;
+		    break;
+		}
+	    }
+	} else {
+	    for (i=0; i<k; i++) {
+		U[i] = gretl_one_uniform();
+		while (U[i] == 0.0) {
+		    U[i] = gretl_one_uniform();
+		}
+	    }
+	    if (delta > 0) {
+		while (1) {
+		    u = gretl_one_uniform();
+		    v = gretl_one_uniform();
+		    if (u <= u0) {
+			x = pow(u, dinv);
+			y = v * pow(x, d1);
+		    } else {
+			x = 1 - log(u);
+			y = v * exp(-x);
+		    }
+		    if (y <= pow(x, d1) * exp(-x)) {
+			a[t] = x;
+			break;
+		    }
+		}
+	    } 
+	    for (i=0; i<k; i++) {
+		a[t] -= log(U[i]);
+	    }
+	}
+	a[t] *= scale;	
+    }
+
+    free(U);
+	
+    return 0;
+}
+
 /**
  * gretl_rand_chisq:
  * @a: target array.
@@ -265,30 +369,14 @@ gretl_rand_normal_full (double *a, int t1, int t2,
  *
  * Fill the selected range of array @a with pseudo-random drawings
  * from the Chi-Squared distribution with @v degrees of freedom, 
- * using the Mersenne Twister for uniform input and the Box-Muller 
- * method for converting to the normal distribution.
+ * using the gamma r.v. generator.
  *
  * Returns: 0 on success, non-zero on error.
  */
 
 int gretl_rand_chisq (double *a, int t1, int t2, int v) 
 {
-    double z;
-    int i, t;
-
-    if (v < 1) {
-	return E_INVARG;
-    }
-
-    for (t=t1; t<=t2; t++) {
-	a[t] = 0.0;
-	for (i=0; i<v; i++) {
-	    z = gretl_one_snormal();
-	    a[t] += z * z;
-	}
-    }
-
-    return 0;
+    return gretl_rand_gamma(a, t1, t2, 0.5*v, 2);
 }
 
 /**
@@ -420,113 +508,9 @@ int gretl_rand_binomial (double *a, int t1, int t2, int n, double p)
     return 0;
 }
 
-/**
- * gretl_rand_gamma:
- * @a: target array.
- * @t1: start of the fill range.
- * @t2: end of the fill range.
- * @shape: shape parameter.
- * @scale: scale parameter.
- *
- * Fill the selected range of array @a with pseudo-random drawings
- * from the specified gamma distribution.
- *
- * Returns: 0 on success, non-zero on error.
- */
-
-int gretl_rand_gamma (double *a, int t1, int t2, 
-		      double shape, double scale) 
-{
-    double *U = NULL;
-    double e = 2.718281828459045235;
-    double delta, dinv = 0, d1 = 0;
-    double u, v, x, y, u0 = 0;
-    int k, i, t;
-
-    if (shape <= 0 || scale <= 0) {
-	return E_DATA;
-    }
-
-    k = shape;
-    if (k > 0) {
-	U = malloc(k * sizeof *U);
-	if (U == NULL) {
-	    return E_ALLOC;
-	}
-    }
-
-    delta = shape - k;
-
-    if (delta > 0) {
-	d1 = delta - 1;
-	dinv = 1 / delta;
-	u0 = e / (e + delta);
-    }
-
-    /* 
-       Case of shape < 1 from Kundu and Gupta, "A convenient way of
-       generating gamma random variables using generalized exponential
-       distribution", Computational Statistics and Data Analysis, 51
-       (2007).  Case of shape >= 1 taken from the Wikipedia entry on
-       the gamma distribution.
-    */
-
-    for (t=t1; t<=t2; t++) {
-	a[t] = 0.0;
-	if (shape < 1) {
-	    double ex2;
-
-	    while (1) {
-		u = gretl_one_uniform();
-		v = gretl_one_uniform();
-		x = -2 * log(1 - pow(u, dinv));
-		ex2 = exp(-x/2);
-		u0 = pow(x, d1) * ex2;
-		u0 /= pow(2.0, d1) * pow((1-ex2), d1);
-		if (v <= u0) {
-		    a[t] = x;
-		    break;
-		}
-	    }
-	} else {
-	    for (i=0; i<k; i++) {
-		U[i] = gretl_one_uniform();
-		while (U[i] == 0.0) {
-		    U[i] = gretl_one_uniform();
-		}
-	    }
-	    if (delta > 0) {
-		while (1) {
-		    u = gretl_one_uniform();
-		    v = gretl_one_uniform();
-		    if (u <= u0) {
-			x = pow(u, dinv);
-			y = v * pow(x, d1);
-		    } else {
-			x = 1 - log(u);
-			y = v * exp(-x);
-		    }
-		    if (y <= pow(x, d1) * exp(-x)) {
-			a[t] = x;
-			break;
-		    }
-		}
-	    } 
-	    for (i=0; i<k; i++) {
-		a[t] -= log(U[i]);
-	    }
-	}
-	a[t] *= scale;	
-    }
-
-    free(U);
-	
-    return 0;
-}
-
 /* Poisson rv with mean m */
 
-static double genpois (double m)
+static double genpois (const double m)
 {
     double x;
 
@@ -561,7 +545,7 @@ static double genpois (double m)
  * of length greater than or equal to @t2 + 1.  
  */
 
-void gretl_rand_poisson (double *a, int t1, int t2, double *m,
+void gretl_rand_poisson (double *a, int t1, int t2, const double *m,
 			 int vec) 
 {
     int t;
@@ -571,6 +555,46 @@ void gretl_rand_poisson (double *a, int t1, int t2, double *m,
     }
 }
 
+/* f(x; k, \lambda) = (k/\lambda) (x/\lambda)^{k-1} e^{-(x/\lambda)^k} */
+
+/**
+ * gretl_rand_weibull:
+ * @a: target array.
+ * @t1: start of the fill range.
+ * @t2: end of the fill range.
+ * @shape: shape parameter > 0.
+ * @scale: scale parameter > 0.
+ *
+ * Fill the selected range of array @a with pseudo-random drawings
+ * from the Weibull distribution with shape @k and scale @lambda.
+ *
+ * Returns: 0 on success, non-zero if either parameter is out of
+ * bounds.
+ */
+
+int gretl_rand_weibull (double *a, int t1, int t2, double shape,
+			double scale) 
+{
+    int err = 0;
+
+    if (shape <= 0 || scale <= 0) {
+	err = E_DATA;
+    } else {
+	double u, kinv = 1.0 / shape;
+	int t;
+
+	for (t=t1; t<=t2; t++) {
+	    u = gretl_one_uniform();
+	    while (u == 0.0) {
+		u = gretl_one_uniform();
+	    }
+	    a[t] = scale * pow(-log(u), kinv);
+	}
+    }
+
+    return err;
+}
+
 /**
  * gretl_rand_int_max:
  * @max: the maximum value (open)
