@@ -63,9 +63,25 @@ double dot_atof (const char *s)
 {
     double x;
 
-    gretl_push_c_numeric_locale();
+#ifdef ENABLE_NLS
+    static int decpoint = 0;
+
+    if (decpoint == 0) {
+	struct lconv *lc;
+
+	lc = localeconv();
+	decpoint = *lc->decimal_point;
+    }
+    if (decpoint == '.') {
+	x = atof(s);
+    } else {
+	gretl_push_c_numeric_locale();
+	x = atof(s);
+	gretl_pop_c_numeric_locale();
+    }
+#else
     x = atof(s);
-    gretl_pop_c_numeric_locale();
+#endif
 
     return x;
 }
@@ -237,6 +253,43 @@ char *charsub (char *str, char find, char repl)
     return str;
 }
 
+/**
+ * comma_separate_numbers:
+ * @s: the string to operate on.
+ *
+ * Given a string which contains two or more numbers
+ * separated by spaces and/or commas, revise the
+ * string to ensure that all the numbers are comma-
+ * separated.
+ *
+ * Returns: the (possibly modified) string.
+ */
+
+char *comma_separate_numbers (char *s) 
+{
+    const char *numstart = "+-.0123456789";
+    char *p = s;
+    int i, n, done;
+
+    while (*s) {
+	n = strspn(s, " ,");
+	if (n > 0 && s[n] != '\0' && strchr(numstart, s[n])) {
+	    done = 0;
+	    for (i=0; i<n && !done; i++) {
+		if (s[i] == ',') {
+		    done = 1;
+		}
+	    }
+	    if (!done) {
+		*s = ',';
+	    }
+	}
+	s += (n > 0)? n : 1;
+    }
+
+    return p;
+}
+
 /**
  * has_suffix:
  * @str: the string to check.
@@ -304,6 +357,32 @@ int numeric_string (const char *str)
     return ret;
 }
 
+/**
+ * integer_string:
+ * @str: the string to examine.
+ *
+ * Returns: 1 if the given @str represents an integer, otherwise 0.
+ */
+
+int integer_string (const char *str)
+{
+    char *test;
+    int ret = 1;
+
+    if (str == NULL || *str == '\0') {
+	return 0;
+    }
+
+    errno = 0;
+
+    strtol(str, &test, 10);
+    if (*test != '\0' || errno != 0) {
+	ret = 0;
+    }
+
+    return ret;
+}
+
 /**
  * ends_with_backslash:
  * @s: the string to examine.
@@ -386,7 +465,7 @@ char *gretl_strndup (const char *src, size_t n)
 {
     char *targ = NULL;
 
-    if (src != NULL && n > 0) {
+    if (src != NULL) {
 	size_t len = strlen(src);
 
 	if (len > n) {
@@ -546,6 +625,45 @@ char *gretl_word_strdup (const char *src, const char **ptr)
     return targ;
 }
 
+/**
+ * gretl_quoted_string_strdup:
+ * @s: the source string.
+ * @ptr: location to receive end pointer, or %NULL.
+ *
+ * If @s starts with a quote (double or single), return a copy of  
+ * the portion of @s that is enclosed in quotes.  That is, 
+ * from @s + 1 up to but not including the next matching quote.
+ * If @ptr is not %NULL, on output it receives a pointer to
+ * the next byte in @s after the closing quote.
+ *
+ * Returns: the allocated string or %NULL on failure.
+ */
+
+char *gretl_quoted_string_strdup (const char *s, const char **ptr)
+{
+    char quote, *ret = NULL;
+    const char *p = NULL;
+
+    if (s != NULL && (*s == '"' || *s == '\'')) {
+	quote = *s;
+	s++;
+	p = strchr(s, quote);
+    }
+
+    if (p == NULL) {
+	if (ptr != NULL) {
+	    *ptr = NULL;
+	}
+    } else {
+	if (ptr != NULL) {
+	    *ptr = p + 1;
+	}
+	ret = gretl_strndup(s, p - s);
+    }
+
+    return ret;
+}
+
 /**
  * gretl_string_split:
  * @s: the source string.
@@ -851,25 +969,33 @@ int get_base (char *targ, const char *src, char c)
 /**
  * top_n_tail:
  * @str: the string to process.
+ * @err: location to receive error code, or %NULL
  *
  * Drop leading space and trailing space and newline from string,
  * then replace a trailing backslash (if any) with a space.
+ * If @str does not end with a newline, and @err is not %NULL,
+ * then %E_TOOLONG is written to @err.
  * 
- * Returns: 1 if a trailing backslash was found, otherwise 0.
+ * Returns: 1 if a trailing backslash or comma was found, 
+ * otherwise 0.
  */
 
-int top_n_tail (char *str)
+int top_n_tail (char *str, int *err)
 {
-    int i, len, bs = 0;
+    int i, n, cont = 0;
 
     if (str == NULL || *str == 0 || *str == '\n' || *str == '\r') {
 	return 0;
     }
 
-    len = strlen(str);
+    n = strlen(str) - 1;
+
+    if (err != NULL && str[n] != '\n') {
+	*err = E_TOOLONG;
+    }
 
     /* chop any trailing space */
-    for (i=len-1; i>=0; i--) {
+    for (i=n; i>=0; i--) {
 	if (isspace((unsigned char) str[i])) {
 	    str[i] = 0;
 	} else {
@@ -878,9 +1004,9 @@ int top_n_tail (char *str)
     }
 
     if (*str != 0) {
-	/* drop any leading spaces, also possible questionmark
-	   2007-11-07: try to catch non-breaking spaces too */
-#if 1
+	/* Drop any leading spaces, also possible questionmark.  Try
+	   to catch non-breaking spaces too -- ugh, Windows!
+	*/
 	i = 0;
 	while (isspace((unsigned char) str[i]) || 
 	       str[i] == '?' ||
@@ -888,22 +1014,28 @@ int top_n_tail (char *str)
 	       str[i] == (char) 0xA0) {
 	    i++;
 	}
-#else
-	i = strspn(str, " \t?");
-#endif
 	if (i > 0) {
 	    shift_string_left(str, i);
 	}
 
-	/* replace backslash, if present */
-	len = strlen(str);
-	if (str[len - 1] == '\\') {
-	    str[len - 1] = ' ';
-	    bs = 1;
+	/* does this line start a comment? */
+	if (strchr(str, '#') || !strncmp(str, "/*", 2)) {
+	    ; /* leave well alone */
+	} else {
+	    /* replace backslash, if present */
+	    n = strlen(str) - 1;
+	    if (n >= 0) {
+		if (str[n] == '\\') {
+		    str[n] = ' ';
+		    cont = 1;
+		} else if (str[n] == ',') {
+		    cont = 1;
+		}
+	    }
 	}
     }
 
-    return bs;
+    return cont;
 }  
 
 /**
@@ -1152,14 +1284,19 @@ int strings_array_add (char ***pS, int *n, const char *p)
     }
 
     *pS = Tmp;
-    Tmp[m] = gretl_strdup(p);
 
-    if (Tmp[m] == NULL) {
-	return E_ALLOC;
+    if (p != NULL) {
+	Tmp[m] = gretl_strdup(p);
+	if (Tmp[m] == NULL) {
+	    return E_ALLOC;
+	}
     } else {
-	*n += 1;
-	return 0;
+	Tmp[m] = NULL;
     }
+
+    *n += 1;
+
+    return 0;
 }
 
 /**
@@ -1260,36 +1397,43 @@ void free_strings_array (char **strs, int nstrs)
     }
 }
 
+static int maybe_year_plus (const char *s)
+{
+    int n = strlen(s);
+
+    if (n > 8 && isdigit(s[0]) && isdigit(s[1]) &&
+	isdigit(s[2]) && isdigit(s[3]) && !isdigit(s[4])) {
+	return 1;
+    }
+
+    return 0;
+}
+
 static char *
-real_get_obs_string (char *obs, int t, const DATAINFO *pdinfo, int full)
+real_get_obs_string (char *s, int t, const DATAINFO *pdinfo, int full)
 {
     if (pdinfo->markers && pdinfo->S != NULL) { 
 	/* data marker strings present */
-	strcpy(obs, pdinfo->S[t]);
+	if (full) {
+	    strcpy(s, pdinfo->S[t]);
+	} else {
+	    *s = '\0';
+	    if (maybe_year_plus(pdinfo->S[t])) {
+		/* daily date, year first? */
+		strncat(s, pdinfo->S[t] + 2, 8);
+	    } else {
+		strncat(s, pdinfo->S[t], 8);
+	    }
+	}
     } else {
 	if (full) {
-	    ntodate_full(obs, t, pdinfo);
+	    ntodate_full(s, t, pdinfo);
 	} else {
-	    ntodate(obs, t, pdinfo);
+	    ntodate(s, t, pdinfo);
 	}
     }
 
-    if (!full) {
-	if (strlen(obs) > 8) { 
-	    char tmp[9];
-
-	    if (isdigit(*obs) && isdigit(*(obs + 1))) {
-		strcpy(tmp, obs + 2);
-		strcpy(obs, tmp);
-	    } else {
-		*tmp = '\0';
-		strncat(tmp, obs, 8);
-		strcpy(obs, tmp);
-	    }
-	} 
-    }
-
-    return obs;
+    return s;
 }
 
 /**
@@ -1632,9 +1776,9 @@ void unescape_url (char *url)
 
 char *make_varname_unique (char *vname, int v, DATAINFO *pdinfo) 
 {
-    int i, j, conflict;
-    size_t n = strlen(vname);
     const char *add = "abcdefghijklmnopqrstuvwxyz";
+    size_t n = strlen(vname);
+    int i, j, conflict;
 
     if (n > 7) {
 	n = 7;
@@ -1642,17 +1786,17 @@ char *make_varname_unique (char *vname, int v, DATAINFO *pdinfo)
 
     for (j=0; j<26; j++) {
 	conflict = 0;
-	for (i=1; i<pdinfo->v; i++) {
+	for (i=1; i<pdinfo->v && !conflict; i++) {
 	    if (i != v && !strcmp(vname, pdinfo->varname[i])) {
 		conflict = 1;
-		break;
 	    }
 	}
-	if (!conflict) {
+	if (conflict) {
+	    vname[n] = add[j];
+	    vname[n+1] = '\0';
+	} else {
 	    break;
 	}
-	vname[n] = add[j];
-	vname[n+1] = '\0';
     }
 
     return vname;
@@ -1666,6 +1810,7 @@ int fix_varname_duplicates (DATAINFO *pdinfo)
     for (i=1; i<pdinfo->v; i++) {
 	for (j=i+1; j<pdinfo->v; j++) {
 	    if (!strcmp(pdinfo->varname[i], pdinfo->varname[j])) {
+		fprintf(stderr, "'%s' duplicated variable name\n", pdinfo->varname[i]);
 		dups = 1;
 		make_varname_unique(pdinfo->varname[j], j, pdinfo);
 	    }
@@ -1781,6 +1926,69 @@ const char *path_last_element (const char *path)
     return p;
 }
 
+/**
+ * ensure_slash:
+ * @s: string to work on.
+ *
+ * If @s does not ends with #SLASH, append this character.
+ *
+ * Returns: the (possibly) modified string.
+ */
+
+char *ensure_slash (char *s)
+{
+    int n = strlen(s);
+
+    if (n > 0 && s[n-1] != SLASH) {
+	strcat(s, SLASHSTR);
+    }
+
+    return s;
+}
+
+/**
+ * trim_slash:
+ * @s: string to work on.
+ *
+ * If @s ends with #SLASH, remove this character.
+ *
+ * Returns: the (possibly) modified string.
+ */
+
+char *trim_slash (char *s)
+{
+    int n = strlen(s);
+
+    if (n > 0 && (s[n-1] == SLASH)) {
+	s[n-1] = '\0';
+    }
+
+    return s;
+}
+
+/**
+ * gretl_string_ends_with:
+ * @s: string to examine.
+ * @test: string to test for.
+ *
+ * Returns: 1 if @s ends with @test, else 0.
+ */
+
+int gretl_string_ends_with (const char *s, const char *test)
+{
+    int nt = strlen(test);
+    int n = strlen(s);
+    int ret = 0;
+
+    if (n >= nt) {
+	const char *p = s + n - nt;
+
+	ret = !strcmp(p, test);
+    }
+
+    return ret;
+}
+
 int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
 {
     GPatternSpec *pspec;
@@ -1788,6 +1996,7 @@ int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
     int i, n = 0;
 
     pspec = g_pattern_spec_new(pattern);
+
     for (i=1; i<pdinfo->v; i++) { 
 	if (var_is_series(pdinfo, i) &&
 	    g_pattern_match_string(pspec, pdinfo->varname[i])) {
@@ -1815,7 +2024,22 @@ int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
     return list;
 }
 
-    
+int varname_match_any (const DATAINFO *pdinfo, const char *pattern)
+{
+    GPatternSpec *pspec;
+    int i, ret = 0;
 
+    pspec = g_pattern_spec_new(pattern);
 
+    for (i=1; i<pdinfo->v; i++) { 
+	if (var_is_series(pdinfo, i) &&
+	    g_pattern_match_string(pspec, pdinfo->varname[i])) {
+	    ret = 1;
+	    break;
+	}
+    }
 
+    g_pattern_spec_free(pspec);
+
+    return ret;
+}
