@@ -43,6 +43,8 @@ int has_suffix (const char *str, const char *sfx);
 
 int numeric_string (const char *str);
 
+int integer_string (const char *str);
+
 int count_fields (const char *s);
 
 double dot_atof (const char *s);
@@ -73,6 +75,8 @@ char *gretl_strdup_printf (const char *template, ...);
 
 char *gretl_word_strdup (const char *src, const char **ptr);
 
+char *gretl_quoted_string_strdup (const char *s, const char **ptr);
+
 char **gretl_string_split (const char *s, int *n);
 
 char *gretl_str_expand (char **orig, const char *add, const char *sep);
@@ -81,6 +85,8 @@ double gretl_double_from_string (const char *s, const char **ptr);
 
 char *charsub (char *str, char find, char repl);
 
+char *comma_separate_numbers (char *s);
+
 char *shift_string_left (char *str, size_t move);
 
 char *lower (char *str);
@@ -95,7 +101,7 @@ int get_base (char *targ, const char *src, char c);
 
 int equation_get_lhs_and_rhs (const char *s, char **plh, char **prh);
 
-int top_n_tail (char *str);
+int top_n_tail (char *str, int *err);
 
 char *tailstrip (char *str);
 
@@ -148,6 +154,14 @@ char *build_path (char *targ, const char *dirname, const char *fname,
 
 const char *path_last_element (const char *path);
 
+char *ensure_slash (char *s);
+
+char *trim_slash (char *s);
+
+int gretl_string_ends_with (const char *s, const char *test);
+
 int *varname_match_list (const DATAINFO *pdinfo, const char *pattern);
 
+int varname_match_any (const DATAINFO *pdinfo, const char *pattern);
+
 #endif /* STRUTILS_H */
