@@ -24,6 +24,7 @@
 #include "gretl_func.h"
 #include "gretl_panel.h"
 #include "cmd_private.h"
+#include "dbread.h"
 
 #define SUBDEBUG 0
 
@@ -36,25 +37,20 @@
   and fullinfo are used to record the addresses of the full data
   matrix and DATAINFO struct respectively.
 
-  Another issue arises: if the user replaces or clears a dataset while
-  it is subsampled, we want to free the associated full dataset also.
-  The "peerinfo" pointer is used to facilitate this.  On subsampling,
-  when fullZ and fullinfo are assigned to, peerinfo is pointed at the
-  associated subsampled DATAINFO struct.  Then, on freeing the
-  subsampled dataset, we check whether its DATAINFO address matches
-  peerinfo: if so, we free up fullZ and fullinfo.
-
+  In addition, peerinfo keeps track of the location of the DATAINFO
+  struct associated with the back-up full dataset; by means of this,
+  we can know when to free the full dataset and when not to (for
+  instance, if we're freeing an auxiliary dataset).
 */
 
 static double **fullZ;
 static DATAINFO *fullinfo;
-static const DATAINFO *peerinfo;
+static DATAINFO *peerinfo;
 
 #define SUBMASK_SENTINEL 127
 
-static int 
-restore_sample_from_subinfo (double ***pZ, DATAINFO **ppdinfo,
-			     ExecState *state);
+static int smpl_get_int (const char *s, double ***pZ, DATAINFO *pdinfo,
+			 int *err);
 
 /* accessors for full dataset, when sub-sampled */
 
@@ -93,19 +89,32 @@ static int get_submask_length (const char *s)
 {
     int n = 1;
 
-    while (*s != SUBMASK_SENTINEL) {
-	n++;
-	s++;
+    if (s == RESAMPLED) {
+	n = 0;
+    } else {
+	while (*s != SUBMASK_SENTINEL) {
+	    n++;
+	    s++;
+	}
     }
 
     return n;
 }
 
+void free_subsample_mask (char *s)
+{
+    if (s != RESAMPLED && s != NULL) {
+	free(s);
+    }
+}
+
 char *copy_subsample_mask (const char *src)
 {
     char *ret = NULL;
 
-    if (src != NULL) {
+    if (src == RESAMPLED) {
+	ret = RESAMPLED;
+    } else if (src != NULL) {
 	int n = get_submask_length(src);
 
 	ret = malloc(n * sizeof *ret);
@@ -132,7 +141,10 @@ int write_model_submask (const MODEL *pmod, FILE *fp)
 {
     int ret = 0;
 
-    if (pmod->submask != NULL) {
+    if (pmod->submask == RESAMPLED) {
+	fputs("<submask length=\"0\"></submask>\n", fp);
+	ret = 1;
+    } else if (pmod->submask != NULL) {
 	int i, n = get_submask_length(pmod->submask);
 
 	fprintf(fp, "<submask length=\"%d\">", n);
@@ -150,10 +162,15 @@ int write_datainfo_submask (const DATAINFO *pdinfo, FILE *fp)
 {
     int ret = 0;
 
-    if (complex_subsampled()) {
+    if (pdinfo->submask == RESAMPLED) {
+	unsigned int seed = get_resampling_seed();
+
+	fprintf(fp, "<resample seed=\"%u\" n=\"%d\"/>\n", seed, pdinfo->n);
+	ret = 1;
+    } else if (complex_subsampled()) {
 	int i, n = get_submask_length(pdinfo->submask);
 
-	fprintf(fp, "<submask length=\"%d\" mode=\"%d\">", n, pdinfo->submode);
+	fprintf(fp, "<submask length=\"%d\">", n);
 	for (i=0; i<n; i++) {
 	    fprintf(fp, "%d ", (int) pdinfo->submask[i]);
 	}
@@ -166,6 +183,10 @@ int write_datainfo_submask (const DATAINFO *pdinfo, FILE *fp)
 
 int submask_cmp (const char *m1, const char *m2)
 {
+    if (m1 == RESAMPLED || m2 == RESAMPLED) {
+	return m1 == RESAMPLED && m2 == RESAMPLED;
+    }
+
     while (*m1 != SUBMASK_SENTINEL && *m2 != SUBMASK_SENTINEL) {
 	if (*m1 != *m2) {
 	    return 1;
@@ -207,23 +228,20 @@ void maybe_free_full_dataset (const DATAINFO *pdinfo)
 /* we do this on "restore full sample" */
 
 static void 
-relink_to_full_dataset (double ***pZ, DATAINFO **ppdinfo, int nullit)
+relink_to_full_dataset (double ***pZ, DATAINFO *pdinfo)
 {
     *pZ = fullZ;
-    *ppdinfo = fullinfo;
-    (*ppdinfo)->submode = 0;
+    *pdinfo = *fullinfo;
 
 #if SUBDEBUG
     fprintf(stderr, "relink_to_full_dataset: fullZ = %p, fullinfo = %p\n",
 	    (void *) fullZ, (void *) fullinfo);
 #endif
 
-    if (nullit) {
-	fullZ = NULL;
-	fullinfo = NULL;
-	peerinfo = NULL;
-	fullZ = 0;
-    }
+    fullZ = NULL;
+    free(fullinfo);
+    fullinfo = NULL;
+    peerinfo = NULL;
 }
 
 /* sync malloced elements of the fullinfo struct that might
@@ -252,12 +270,17 @@ int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 {
     int err = 0;
 
+#if SUBDEBUG
+    fprintf(stderr, "attach_subsample_to_model: fullZ = %p\n",
+	    (void *) fullZ);
+#endif
+
     if (fullZ != NULL) {
 	/* sync, in case anything has moved */
 	sync_dataset_elements(pdinfo);
 
 	if (pmod->submask != NULL) {
-	    free(pmod->submask);
+	    free_subsample_mask(pmod->submask);
 	}
 
 	pmod->submask = copy_subsample_mask(pdinfo->submask);
@@ -269,6 +292,84 @@ int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo)
     return err;
 }
 
+static int resample_update_dataset (double ***RZ, DATAINFO *pdinfo)
+{
+    int newv = fullinfo->v;
+    double **newZ = NULL;
+    int drop = 0;
+    int i, j, err = 0;
+
+    /* carry back values of pre-existing scalars */
+
+    for (i=1; i<fullinfo->v && i<pdinfo->v; i++) {
+	if (var_is_scalar(pdinfo, i)) {
+	    fullZ[i][0] = (*RZ)[i][0];
+	} 
+    }
+
+    /* figure how many scalars and how many series have been added */
+
+    for (i=fullinfo->v; i<pdinfo->v; i++) {
+	if (var_is_scalar(pdinfo, i)) {
+	    newv++;
+	} else {
+	    drop++;
+	}
+    }
+
+    /* if new series have been added, drop them all */
+
+    if (drop > 0) {
+	int *list = gretl_list_new(drop);
+
+	if (list == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    j = 1;
+	    for (i=fullinfo->v; i<pdinfo->v; i++) {
+		if (var_is_series(pdinfo, i)) {
+		    list[j++] = i;
+		}
+	    }	    
+	}
+	
+	if (list != NULL) {
+	    err = dataset_drop_listed_variables(list, RZ, pdinfo, NULL, NULL);
+	    free(list);
+	}
+    }
+
+    /* if new scalars have been added, add them to fullZ */
+
+    if (!err && newv > fullinfo->v) {
+	newZ = realloc(fullZ, newv * sizeof *fullZ);
+	if (newZ == NULL) {
+	    newv = fullinfo->v;
+	    err = E_ALLOC;
+	} else {
+	    fullZ = newZ;
+	    j = fullinfo->v;
+	    for (i=fullinfo->v; i<pdinfo->v && !err; i++) {
+		if (var_is_scalar(pdinfo, i)) {
+		    fullZ[j] = malloc(sizeof **newZ);
+		    if (fullZ[j] == NULL) {
+			err = E_ALLOC;
+		    } else {
+			fullZ[j][0] = (*RZ)[i][0];
+			j++;
+		    }
+		}
+	    }
+	}
+    }
+
+    if (!err) {
+	fullinfo->v = newv;
+    }
+
+    return err;
+}
+
 /* Apparatus for updating full dataset when restoring full sample
    after sub-sampling.  
 */
@@ -361,6 +462,10 @@ static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
 	if (var_is_series(pdinfo, i)) {
 	    fullZ[i] = malloc(N * sizeof **newZ);
 	} else {
+#if SUBDEBUG
+	    fprintf(stderr, " new var %d (%s) is a scalar\n", i, 
+		    pdinfo->varname[i]);
+#endif
 	    fullZ[i] = malloc(sizeof **newZ);
 	}
 	if (fullZ[i] == NULL) {
@@ -424,53 +529,23 @@ static char *make_current_sample_mask (DATAINFO *pdinfo)
     return currmask;
 }
 
-static void destroy_subsampled_datainfo (DATAINFO *pdinfo)
-{
-    clear_datainfo(pdinfo, CLEAR_SUBSAMPLE);
-#if SUBDEBUG
-    fprintf(stderr, "destroy_subsampled_datainfo: freeing at %p\n", 
-	    (void *) pdinfo);
-#endif
-    free(pdinfo);
-}
-
-/* marker for the case where restore_full_sample is called as
-   a preliminary for imposing a restriction */
-
-static int restricting;
-
-static void flag_restricting_on (void)
-{
-    restricting = 1;
-}
-
-static void flag_restricting_off (void)
-{
-    restricting = 0;
-}
-
-static int doing_restrict (void)
-{
-    return restricting;
-}
-
 /* restore_full_sample: 
  * @pZ: pointer to data array.  
- * @ppdinfo: address of data info pointer.
+ * @pdinfo: pointer to dataset information.
  * @state: structure representing program execution state,
  * or %NULL. 
  *
  * Restore the full data range, undoing any sub-sampling that was
  * previously performed (either by shifting the endpoints of the
  * sample range or by selecting cases on some criterion or other).
+ * However, if @state is not %NULL, and if it contains a submask,
+ * the "full" range is relative to that mask.
  *
  * Returns: 0 on success, non-zero error code on failure.
  */
 
-int restore_full_sample (double ***pZ, DATAINFO **ppdinfo,
-			 ExecState *state)
+int restore_full_sample (double ***pZ, DATAINFO *pdinfo, ExecState *state)
 {
-    DATAINFO *pdinfo = *ppdinfo;
     int err = 0;
 
     gretl_error_clear();
@@ -487,10 +562,8 @@ int restore_full_sample (double ***pZ, DATAINFO **ppdinfo,
     }
 
 #if SUBDEBUG
-    fprintf(stderr, "\nrestore_full_sample: pZ=%p, ppdinfo=%p\n",
-	    (void *) pZ, (void *) ppdinfo);
-    fprintf(stderr, " *pZ=%p, *ppdinfo=%p, state=%p\n",
-	    (void *) *pZ, (void *) pdinfo, (void *) state);
+    fprintf(stderr, "\nrestore_full_sample: pZ=%p, *pZ=%p, pdinfo=%p\n",
+	    (void *) pZ, (void *) *pZ, (void *) pdinfo);
 #endif
 
     /* beyond this point we are doing a non-trivial restoration
@@ -502,7 +575,10 @@ int restore_full_sample (double ***pZ, DATAINFO **ppdinfo,
        which may have moved */
     sync_dataset_elements(pdinfo);
 
-    if (!err) {
+    if (pdinfo->submask == RESAMPLED) {
+	/* not regular subsample but resampled dataset */
+	resample_update_dataset(pZ, pdinfo);
+    } else {
 	/* update values for pre-existing series, which may have been
 	   modified via genr etc */
 	update_full_data_values((const double **) *pZ, pdinfo);
@@ -520,33 +596,28 @@ int restore_full_sample (double ***pZ, DATAINFO **ppdinfo,
 	}
     }
 
-    if (err == E_ALLOC) {
-        sprintf(gretl_errmsg, _("Out of memory expanding data set\n"));
-    } else if (err == E_NOMERGE) {
+    if (err == E_NOMERGE) {
         sprintf(gretl_errmsg, 
 		_("Missing sub-sample information; can't merge data\n"));
     }
 
     /* destroy sub-sampled data array */
+#if SUBDEBUG
+    fprintf(stderr, "restore_full_sample: freeing Z at %p\n", (void *) *pZ);
+#endif
     free_Z(*pZ, pdinfo);
 
-    if (state != NULL && state->subinfo != NULL) {
-	if (doing_restrict()) {
-	    /* preliminary to sub-sampling inside a function */
-	    if (*ppdinfo != state->subinfo) {
-		destroy_subsampled_datainfo(*ppdinfo);
-	    }
-	    relink_to_full_dataset(pZ, ppdinfo, 0);
-	} else {
-	    /* inside a function, where the dataset was sub-sampled
-	       on entry to the function: restoring the "full sample" in
-	       effect means restoring the outer sub-sample.
-	    */
-	    err = restore_sample_from_subinfo(pZ, ppdinfo, state);
-	} 
-    } else {
-	destroy_subsampled_datainfo(*ppdinfo);
-	relink_to_full_dataset(pZ, ppdinfo, 1);
+#if SUBDEBUG
+    fprintf(stderr, "restore_full_sample: clearing pdinfo at %p\n", 
+	    (void *) pdinfo);
+#endif
+    clear_datainfo(pdinfo, CLEAR_SUBSAMPLE);
+
+    relink_to_full_dataset(pZ, pdinfo);
+
+    if (!err && state != NULL && state->submask != NULL) {
+	/* "full" really means, relative to the original state */
+	err = restrict_sample_from_mask(state->submask, pZ, pdinfo);
     }
 
     return err;
@@ -617,8 +688,7 @@ static int copy_dummy_to_mask (char *mask, const double *x, int n)
     return err;
 }
 
-static int mask_from_temp_dummy (const char *line,
-				 double ***pZ, DATAINFO *pdinfo, 
+static int mask_from_temp_dummy (const char *s, double ***pZ, DATAINFO *pdinfo, 
 				 char *mask)
 {
     char formula[MAXLEN];
@@ -626,7 +696,7 @@ static int mask_from_temp_dummy (const char *line,
     int err = 0;
 
     *formula = '\0';
-    strncat(formula, line + 4, MAXLEN - 1);
+    strncat(formula, s, MAXLEN - 1);
 
     x = generate_series(formula, pZ, pdinfo, &err);
 
@@ -642,21 +712,13 @@ static int mask_from_temp_dummy (const char *line,
     return err;
 }
 
-static int mask_from_dummy (const char *line,
-			    const double **Z, const DATAINFO *pdinfo,
+static int mask_from_dummy (const char *s, const double **Z, const DATAINFO *pdinfo,
 			    char *mask)
 {
     char dname[VNAMELEN] = {0};
     int dnum, err = 0;
 
-    /* + 4 to skip the command word "smpl" */
-    sscanf(line + 4, "%15s", dname);
-
-    if (*dname == 0) {
-	strcpy(gretl_errmsg, "Unrecognized sample command");
-	return 1;
-    }
-
+    sscanf(s, "%15s", dname);
     dnum = varindex(pdinfo, dname);
 
     if (dnum == pdinfo->v) {
@@ -715,31 +777,18 @@ count_panel_units (const char *mask, const DATAINFO *pdinfo)
     return n;
 }
 
-/* construct mask for takig random sub-sample from dataset */
+/* construct mask for taking random sub-sample from dataset */
 
-static int make_random_mask (const char *oldmask, const char *line, 
-			     const double **Z, const DATAINFO *pdinfo,
+static int make_random_mask (const char *s, const char *oldmask, 
+			     double ***pZ, DATAINFO *pdinfo,
 			     char *mask)
 {
-    char s[VNAMELEN] = {0};
     unsigned u;
-    int i, subn = 0;
+    int i, subn;
     int cases = 0;
     int err = 0;
 
-    sscanf(line + 4, "%15s", s);
-    if (*s == 0) {
-	sprintf(gretl_errmsg, _("Invalid number of cases %d"), subn);
-	return 0;
-    } else if (isdigit(*s)) {
-	subn = atoi(s);
-    } else {
-	int v = varindex(pdinfo, s);
-
-	if (v < pdinfo->v && !na(Z[v][0])) {
-	    subn = Z[v][0];
-	}
-    }
+    subn = smpl_get_int(s, pZ, pdinfo, &err);
 
     if (subn <= 0 || subn >= pdinfo->n) {
 	err = 1;
@@ -765,7 +814,7 @@ static int make_random_mask (const char *oldmask, const char *line,
 	mask[i] = 0;
     }
 
-    for (i=0; (cases != subn); i++) {
+    for (i=0; cases != subn; i++) {
 	u = gretl_rand_int_max(pdinfo->n);
 	if (oldmask == NULL || oldmask[u]) {
 	    mask[u] = 1;
@@ -778,22 +827,28 @@ static int make_random_mask (const char *oldmask, const char *line,
     return err;
 }
 
-static void backup_full_dataset (double **Z, DATAINFO *pdinfo,
-				 const DATAINFO *newinfo)
+int backup_full_dataset (double **Z, DATAINFO *pdinfo)
 {
     fullZ = Z;
 
-    if (pdinfo != NULL) {
-	fullinfo = pdinfo;
-    } 
-    if (newinfo != NULL) {
-	peerinfo = newinfo;
+    if (fullinfo == NULL) {
+	fullinfo = malloc(sizeof *fullinfo);
+	if (fullinfo == NULL) {
+	    return E_ALLOC;
+	}
     }
 
+    if (pdinfo != NULL) {
+	*fullinfo = *pdinfo;
+	peerinfo = pdinfo;
+    } 
+
 #if SUBDEBUG
     fprintf(stderr, "backup_full_dataset: fullZ = %p, fullinfo = %p\n",
 	    (void *) fullZ, (void *) fullinfo);
 #endif
+
+    return 0;
 }
 
 int complex_subsampled (void)
@@ -979,14 +1034,22 @@ make_panel_submask (char *mask, const DATAINFO *pdinfo, int *err)
     return np;
 }
 
+#define needs_string_arg(m) (m == SUBSAMPLE_RANDOM || \
+	                     m == SUBSAMPLE_USE_DUMMY || \
+                             m == SUBSAMPLE_BOOLEAN)
+
 static int 
-make_restriction_mask (int mode, const char *line, const int *list, 
+make_restriction_mask (int mode, const char *s, const int *list, 
 		       double ***pZ, DATAINFO *pdinfo,
 		       PRN *prn, const char *oldmask, char **pmask)
 {
     char *mask = NULL;
     int sn = 0, err = 0;
 
+    if (needs_string_arg(mode) && (s == NULL || *s == '\0')) {
+	return E_ARGS;
+    }
+
     mask = make_submask(pdinfo->n);
     if (mask == NULL) {
 	return E_ALLOC;
@@ -996,12 +1059,11 @@ make_restriction_mask (int mode, const char *line, const int *list,
     if (mode == SUBSAMPLE_DROP_MISSING) {   
 	err = make_missing_mask(list, (const double **) *pZ, pdinfo, mask);
     } else if (mode == SUBSAMPLE_RANDOM) {
-	err = make_random_mask(oldmask, line, (const double **) *pZ,
-			       pdinfo, mask);
+	err = make_random_mask(s, oldmask, pZ, pdinfo, mask);
     } else if (mode == SUBSAMPLE_USE_DUMMY) {
-	err = mask_from_dummy(line, (const double **) *pZ, pdinfo, mask);
+	err = mask_from_dummy(s, (const double **) *pZ, pdinfo, mask);
     } else if (mode == SUBSAMPLE_BOOLEAN) {
-	err = mask_from_temp_dummy(line, pZ, pdinfo, mask);
+	err = mask_from_temp_dummy(s, pZ, pdinfo, mask);
     } else {
 	strcpy(gretl_errmsg, _("Sub-sample command failed mysteriously"));
 	err = 1;
@@ -1045,20 +1107,24 @@ make_restriction_mask (int mode, const char *line, const int *list,
     return err;
 }
 
-/* this is also used in session.c, when re-establishing
-   a previously sub-sampled data state on reopening 
-   a saved session */
+/* This is also used in session.c, when re-establishing a previously
+   sub-sampled data state on reopening a saved session, and also on
+   exit from a user function when the dataset was sub-sampled on
+   entry to the function.
+*/
 
 int 
-restrict_sample_from_mask (char *mask, int mode, 
-			   double ***pZ, DATAINFO **ppdinfo,
-			   ExecState *state)
+restrict_sample_from_mask (char *mask, double ***pZ, DATAINFO *pdinfo)
 {
     double **subZ = NULL;
-    DATAINFO *pdinfo = *ppdinfo;
     DATAINFO *subinfo;
     int err = 0;
 
+    if (mask == RESAMPLED) {
+	fprintf(stderr, "restrict_sample_from_mask: got RESAMPLED!\n");
+	return E_DATA;
+    }
+
     subinfo = datainfo_new();
     if (subinfo == NULL) {
 	return E_ALLOC;
@@ -1074,18 +1140,22 @@ restrict_sample_from_mask (char *mask, int mode,
 
     if (dataset_is_panel(pdinfo)) {
 	/* are we able to reconstitute a panel? */
-	int np, n = count_panel_units(mask, pdinfo);
+	int npad, nunits = count_panel_units(mask, pdinfo);
 
-	if (n > 1 && subinfo->n > n) {
+	if (nunits > 1 && subinfo->n > nunits) {
 	    /* add padding rows if need be */
-	    np = make_panel_submask(mask, pdinfo, &err);
+	    npad = make_panel_submask(mask, pdinfo, &err);
 	    if (err) {
 		free(subinfo);
 		return err;
 	    }
 	    subinfo->structure = STACKED_TIME_SERIES;
-	    subinfo->n += np;
-	    subinfo->pd = subinfo->n / n;
+	    subinfo->n += npad;
+	    subinfo->pd = subinfo->n / nunits;
+	    /* note: panel indices are added below */
+	} else if (nunits == 1 && subinfo->n == pdinfo->pd) {
+	    /* time series for single panel unit */
+	    subinfo->structure = SPECIAL_TIME_SERIES;
 	}
     }
 
@@ -1133,26 +1203,22 @@ restrict_sample_from_mask (char *mask, int mode,
     copy_data_to_subsample(subZ, subinfo, (const double **) *pZ, 
 			   pdinfo, mask);
 
-    if (state != NULL && state->subinfo != NULL) {
-	backup_full_dataset(*pZ, pdinfo, NULL);
-    } else {
-	backup_full_dataset(*pZ, pdinfo, subinfo);
-    }
+    err = backup_full_dataset(*pZ, pdinfo);
 
-    subinfo->submode = mode;
     subinfo->submask = copy_subsample_mask(mask);
 
     /* switch pointers */
     *pZ = subZ;
-    *ppdinfo = subinfo;
+    *pdinfo = *subinfo;
+    free(subinfo);
 
-    return 0;
+    return err;
 }
 
 /* restrict_sample: 
  * @line: command line (or %NULL).  
  * @pZ: pointer to original data array.  
- * @ppdinfo: address of original data info pointer. 
+ * @pdinfo: data info pointer. 
  * @state: structure representing program state (or %NULL).
  * @list: list of variables in case of OPT_M (or %NULL).  
  * @opt: option flags.
@@ -1175,7 +1241,7 @@ restrict_sample_from_mask (char *mask, int mode,
  */
 
 int restrict_sample (const char *line, const int *list,
-		     double ***pZ, DATAINFO **ppdinfo, 
+		     double ***pZ, DATAINFO *pdinfo, 
 		     ExecState *state, gretlopt opt, 
 		     PRN *prn)
 {
@@ -1188,11 +1254,17 @@ int restrict_sample (const char *line, const int *list,
 
 #if SUBDEBUG
     fprintf(stderr, "\nrestrict_sample: '%s'\n", line);
-    fprintf(stderr, " ppdinfo=%p, *ppdinfo=%p, pZ=%p, *pZ=%p, state=%p\n", 
-	    (void *) ppdinfo, (void *) *ppdinfo, (void *) pZ, (void *) *pZ,
+    fprintf(stderr, " pdinfo=%p, pZ=%p, *pZ=%p, state=%p\n", 
+	    (void *) pdinfo, (void *) pZ, (void *) *pZ,
 	    (void *) state);
 #endif
 
+    if (line != NULL) {
+	/* skip the command word */
+	line += strcspn(line, " ");
+	line += strspn(line, " ");
+    }
+
     mode = get_restriction_mode(opt);
     if (mode == SUBSAMPLE_UNKNOWN) {
 	strcpy(gretl_errmsg, "Unrecognized sample command");
@@ -1201,26 +1273,26 @@ int restrict_sample (const char *line, const int *list,
 
     if (!(opt & OPT_P)) {
 	/* not replacing but cumulating any existing restrictions */
-	oldmask = make_current_sample_mask(*ppdinfo);
+	oldmask = make_current_sample_mask(pdinfo);
 	if (oldmask == NULL) {
 	    return E_ALLOC;
 	}
 	free_oldmask = 1;
-    } else if (state != NULL && state->subinfo != NULL) {
-	/* subsampling within a function */
-	oldmask = state->subinfo->submask;
+    } else if (state != NULL && state->submask != NULL) {
+	/* subsampling within a function, with incoming
+	   restriction recorded in state
+	*/
+	oldmask = state->submask;
     }
 
     /* restore the full data range, for housekeeping purposes */
-    flag_restricting_on();
-    err = restore_full_sample(pZ, ppdinfo, state);
-    flag_restricting_off();
+    err = restore_full_sample(pZ, pdinfo, NULL);
     if (err) {
 	return err;
     }
 
     if (!err) {
-	err = make_restriction_mask(mode, line, list, pZ, *ppdinfo, 
+	err = make_restriction_mask(mode, line, list, pZ, pdinfo, 
 				    prn, oldmask, &mask);
     }
 
@@ -1229,21 +1301,21 @@ int restrict_sample (const char *line, const int *list,
 	int contig = 0;
 
 	if (mode != SUBSAMPLE_RANDOM &&
-	    (dataset_is_time_series(*ppdinfo) ||
-	     dataset_is_panel(*ppdinfo))) {
-	    contig = mask_contiguous(mask, *ppdinfo, &t1, &t2);
+	    (dataset_is_time_series(pdinfo) ||
+	     dataset_is_panel(pdinfo))) {
+	    contig = mask_contiguous(mask, pdinfo, &t1, &t2);
 	}
 
 	if (contig) {
-	    (*ppdinfo)->t1 = t1;
-	    (*ppdinfo)->t2 = t2;
+	    pdinfo->t1 = t1;
+	    pdinfo->t2 = t2;
 	} else {
-	    err = restrict_sample_from_mask(mask, mode, pZ, ppdinfo,
-					    state);
+	    err = restrict_sample_from_mask(mask, pZ, pdinfo);
 	}
     }
 
     free(mask);
+
     if (free_oldmask) {
 	free(oldmask);
     }
@@ -1251,75 +1323,6 @@ int restrict_sample (const char *line, const int *list,
     return err;
 }
 
-/* For use when exiting a function: re-establish the sub-sample that
-   was in force on entry, which is recorded in subinfo */
-
-static int 
-restore_sample_from_subinfo (double ***pZ, DATAINFO **ppdinfo,
-			     ExecState *state)
-{
-    double **subZ = NULL;
-    DATAINFO *pdinfo = *ppdinfo;
-    DATAINFO *subinfo = state->subinfo;
-    char *mask = subinfo->submask;
-    int i, s, t;
-
-#if SUBDEBUG
-    fprintf(stderr, "restore_sample_from_subinfo:\n"
-	    " subinfo=%p, n=%d, v=%d\n"
-	    " *ppdinfo=%p, n=%d, v=%d\n",
-	    (void *) subinfo, subinfo->n, subinfo->v,
-	    (void *) pdinfo, pdinfo->n, pdinfo->v);
-#endif
-
-    /* allow for added variables */
-    subinfo->v = pdinfo->v;
-
-    /* set up the sub-sampled Z */
-    if (allocate_Z(&subZ, subinfo)) {
-	return E_ALLOC;
-    }
-
-    /* re-link varnames, etc. */
-    subinfo->varname = pdinfo->varname;
-    subinfo->varinfo = pdinfo->varinfo;
-    subinfo->descrip = pdinfo->descrip;
-
-    /* copy data values */
-    for (i=1; i<fullinfo->v; i++) {
-	if (var_is_series(fullinfo, i)) {
-	    s = 0;
-	    for (t=0; t<fullinfo->n; t++) {
-		if (mask == NULL) {
-		    subZ[i][s++] = fullZ[i][t];
-		} else if (mask[t] == 1) {
-		    subZ[i][s++] = fullZ[i][t];
-		} else if (mask[t] == 'p') {
-		    /* panel padding */
-		    subZ[i][s++] = NADBL;
-		}
-	    }
-	} else {
-	    subZ[i][0] = fullZ[i][0];
-	}
-    }
-
-    if (*ppdinfo != subinfo) {
-	destroy_subsampled_datainfo(*ppdinfo);
-    }
-
-    /* switch pointers */
-    *pZ = subZ;
-    *ppdinfo = subinfo;
-
-#if SUBDEBUG
-    fprintf(stderr, " set *pZ = %p, *ppdinfo = %p\n", (void *) subZ, 
-	    (void *) subinfo);
-#endif
-
-    return 0;    
-}
-
 enum {
     SMPL_T1,
     SMPL_T2
@@ -1340,39 +1343,58 @@ static int panel_round (const DATAINFO *pdinfo, int t, int code)
     return t;
 }
 
-static int 
-get_sample_limit (char *s, const double **Z, DATAINFO *pdinfo,
-		  int code)
+static int smpl_get_int (const char *s, double ***pZ, DATAINFO *pdinfo,
+			 int *err)
 {
-    int v, ret = -1;
+    int k = 0;
+
+    if (integer_string(s)) {
+	k = atoi(s);
+    } else {
+	int v = varindex(pdinfo, s);
+
+	if (v < pdinfo->v && var_is_scalar(pdinfo, v)) {
+	    k = (int) (*pZ)[v][0];
+	} else {
+	    k = (int) generate_scalar(s, pZ, pdinfo, err);
+	}
+    }
+
+    return k;
+}
+
+static int get_sample_limit (char *s, double ***pZ, DATAINFO *pdinfo,
+			     int code)
+{
+    int ret = -1;
+    int err = 0;
 
     if (*s == '-' || *s == '+') {
 	/* increment/decrement form */
-	int incr = 0;
+	int incr = smpl_get_int(s + 1, pZ, pdinfo, &err);
 
-	if (isdigit((unsigned char) s[1])) {
-	    incr = atoi(s);
-	} else {
-	    v = varindex(pdinfo, s + 1);
-	    if (v < pdinfo->v) {
-		incr = (int) Z[v][0];
-		if (*s == '-') {
-		    incr = -incr;
-		}
+	if (!err) {
+	    if (*s == '-') {
+		incr = -incr;
+	    }
+	    if (dataset_is_panel(pdinfo)) {
+		incr *= pdinfo->paninfo->Tmax;
+	    }
+	    if (code == SMPL_T1) {
+		ret = pdinfo->t1 + incr;
+	    } else {
+		ret = pdinfo->t2 + incr;
 	    }
-	}
-	if (dataset_is_panel(pdinfo)) {
-	    incr *= pdinfo->paninfo->Tmax;
-	}
-	if (code == SMPL_T1) {
-	    ret = pdinfo->t1 + incr;
-	} else {
-	    ret = pdinfo->t2 + incr;
 	}
     } else {
 	/* absolute form */
-	ret = get_t_from_obs_string(s, Z, pdinfo);
-	if (dataset_is_panel(pdinfo)) {
+	ret = get_t_from_obs_string(s, (const double **) *pZ, pdinfo);
+	if (ret < 0) {
+	    ret = smpl_get_int(s, pZ, pdinfo, &err);
+	    /* convert to base 0 */
+	    if (!err) ret--;
+	}
+	if (ret >= 0 && dataset_is_panel(pdinfo)) {
 	    ret = panel_round(pdinfo, ret, code);
 	}
     }
@@ -1380,13 +1402,16 @@ get_sample_limit (char *s, const double **Z, DATAINFO *pdinfo,
     return ret;
 }
 
-int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
+int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
 {
     int nf, new_t1 = pdinfo->t1, new_t2 = pdinfo->t2;
-    char cmd[5], newstart[OBSLEN], newstop[OBSLEN];
+    char newstart[OBSLEN], newstop[OBSLEN];
 
     gretl_error_clear();
 
+    line += strcspn(line, " ");
+    line += strspn(line, " ");
+
     nf = count_fields(line);
 
 #if SUBDEBUG
@@ -1398,22 +1423,22 @@ int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
     }
 #endif
 
-    if (nf == 3 && pdinfo->n == 0) {
+    if (nf == 2 && pdinfo->n == 0) {
 	/* database special */
 	return db_set_sample(line, pdinfo);
     }
 
-    if (nf == 1) {
+    if (nf == 0) {
 	/* no-op, just print the current sample */
 	return 0;
     }
 	
-    if (nf == 2) {
-	if (sscanf(line, "%4s %10s", cmd, newstart) != 2) {
+    if (nf == 1) {
+	if (sscanf(line, "%10s", newstart) != 1) {
 	    strcpy(gretl_errmsg, _("error reading smpl line"));
 	    return 1;
 	} else {
-	    new_t1 = get_sample_limit(newstart, Z, pdinfo, SMPL_T1);
+	    new_t1 = get_sample_limit(newstart, pZ, pdinfo, SMPL_T1);
 	    if (new_t1 < 0 || new_t1 >= pdinfo->n) {
 		strcpy(gretl_errmsg, _("error in new starting obs"));
 		return 1;
@@ -1423,15 +1448,15 @@ int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
 	}
     }
 
-    /* now we're looking at nf = 3 (3 fields) case */
+    /* now we're looking at nf = 2 (2 fields) case */
 
-    if (sscanf(line, "%4s %10s %10s", cmd, newstart, newstop) != 3) {
+    if (sscanf(line, "%10s %10s", newstart, newstop) != 2) {
 	strcpy(gretl_errmsg, _("error reading smpl line"));
 	return 1;
     }
 
     if (strcmp(newstart, ";")) {
-	new_t1 = get_sample_limit(newstart, Z, pdinfo, SMPL_T1);
+	new_t1 = get_sample_limit(newstart, pZ, pdinfo, SMPL_T1);
 	if (new_t1 < 0 || new_t1 >= pdinfo->n) {
 	    strcpy(gretl_errmsg, _("error in new starting obs"));
 	    return 1;
@@ -1439,7 +1464,7 @@ int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
     }
 
     if (strcmp(newstop, ";")) {
-	new_t2 = get_sample_limit(newstop, Z, pdinfo, SMPL_T2);
+	new_t2 = get_sample_limit(newstop, pZ, pdinfo, SMPL_T2);
 	if (new_t2 < 0 || new_t2 >= pdinfo->n) {
 	    strcpy(gretl_errmsg, _("error in new ending obs"));
 	    return 1;
