@@ -31,12 +31,16 @@ typedef enum {
     SUBSAMPLE_UNKNOWN
 } SubsampleMode;
 
+#define RESAMPLED ((char *) 0xdeadbeef)
+
 double ***fetch_full_Z (void);
 
 void reset_full_Z (double ***pZ);
 
 DATAINFO *fetch_full_datainfo (void);
 
+void free_subsample_mask (char *s);
+
 char *copy_subsample_mask (const char *src);
 
 char *copy_datainfo_submask (const DATAINFO *pdinfo);
@@ -50,23 +54,22 @@ int submask_cmp (const char *m1, const char *m2);
 int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo);
 
 int restrict_sample (const char *line, const int *list,  
-		     double ***pZ, DATAINFO **ppdinfo,
+		     double ***pZ, DATAINFO *pdinfo,
 		     ExecState *state, gretlopt opt, 
 		     PRN *prn);
 
 int 
-restrict_sample_from_mask (char *mask, int mode, 
-			   double ***pZ, DATAINFO **ppdinfo,
-			   ExecState *state);
+restrict_sample_from_mask (char *mask, double ***pZ, DATAINFO *pdinfo);
 
 int complex_subsampled (void);
 
 int get_full_length_n (void);
 
-int set_sample (const char *line, const double **Z, DATAINFO *pdinfo);
+int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo);
 
-int restore_full_sample (double ***pZ, DATAINFO **ppdinfo,
-			 ExecState *state); 
+int restore_full_sample (double ***pZ, DATAINFO *pdinfo, ExecState *state);
+
+int backup_full_dataset (double **Z, DATAINFO *pdinfo);
 
 int count_missing_values (double ***pZ, DATAINFO *pdinfo, PRN *prn);
 
