@@ -41,18 +41,28 @@ enum {
     SYSTEM_SAVEIT      = 1 << 6
 };
 
+enum {
+    SYSNAME_NEW,
+    SYSNAME_EST,
+    SYSNAME_RST
+};
+
 typedef struct id_atom_ id_atom;
 typedef struct identity_ identity;
+typedef struct predet_ predet;
+typedef struct liml_data_ liml_data;
 
 struct equation_system_ {
     char *name;                 /* user-specified name for system, or NULL */
     int refcount;               /* for saving/deleting */
     int t1;                     /* starting observation number */
     int t2;                     /* ending observation number */
+    int T;                      /* number of observations per equation */
+    int df;                     /* T - average coeffs per equation */
     int method;                 /* estimation method */
-    int n_equations;            /* number of stochastic equations */
-    int n_identities;           /* number of identities */
-    int n_obs;                  /* number of observations per equation */
+    int neqns;                  /* number of stochastic equations */
+    int nidents;                /* number of identities */
+    int order;                  /* max lag of endogenous variable */
     int iters;                  /* number of iterations taken */
     char flags;                 /* to record options (e.g. save residuals) */
     double ll;                  /* log-likelihood (restricted) */
@@ -61,37 +71,50 @@ struct equation_system_ {
     double ess;                 /* total error sum of squares */
     double diag;                /* test stat for diagonal covariance matrix */
     double bdiff;               /* summary stat for change in coefficients */
+    double ldet;                /* log-determinant of covariance matrix */
     int **lists;                /* regression lists for stochastic equations */
-    int *endog_vars;            /* list of endogenous variables */
-    int *instr_vars;            /* list of instruments (exogenous vars) */
+    int *ylist;                 /* list of endogenous variables */
+    int *ilist;                 /* list of instruments */
+    int *xlist;                 /* list of truly exogenous variables */
+    int *plist;                 /* list of predetermined variables */
+    predet *pre_vars;           /* array of info on predetermined regressors */
     identity **idents;          /* set of identities */
     gretl_matrix *b;            /* coefficient estimates */
     gretl_matrix *vcv;          /* covariance matrix of coefficients */
-    gretl_matrix *sigma;        /* cross-equation covariance matrix */
+    gretl_matrix *S;            /* cross-equation covariance matrix */
     gretl_matrix *R;            /* LHS of any linear restrictions */
     gretl_matrix *q;            /* RHS of any linear restrictions */  
-    gretl_matrix *uhat;         /* residuals, all equations */
+    gretl_matrix *E;            /* residuals, all equations */
+    gretl_matrix *yhat;         /* fitted values, all equations */
+    gretl_matrix *Gamma;        /* structural form Gamma matrix (endog + identities)*/
+    gretl_matrix *B;            /* structural form B matrix (exogenous) */
+    gretl_matrix *A;            /* structural form A matrix (lagged endogenous) */
+    gretl_matrix *F;            /* forecast matrix */
+    gretl_matrix *Sr;           /* reduced-form error covariance matrix */
     MODEL **models;             /* set of pointers to per-equation models: just
 				   convenience pointers -- these should NOT be
 				   freed as part of sys cleanup
 				*/
+    liml_data *ldata;           /* extra info from LIML estimation */
 };
 
 equation_system *equation_system_start (const char *line, 
-					gretlopt opt);
+					gretlopt opt,
+					int *err);
 
-char *get_system_name_from_line (const char *s);
+char *get_system_name_from_line (const char *s, int context);
 
 int equation_system_append (equation_system *sys, 
 			    const int *list);
 
 int system_parse_line (equation_system *sys,
-		       const char *line, 
-		       const DATAINFO *pdinfo);
+		       const char *line,
+		       double ***pZ, 
+		       DATAINFO *pdinfo);
 
 int equation_system_finalize (equation_system *sys, 
 			      double ***pZ, DATAINFO *pdinfo,
-			      PRN *prn);
+			      gretlopt opt, PRN *prn);
 
 int 
 equation_system_estimate (equation_system *sys, 
@@ -103,8 +126,6 @@ int estimate_named_system (const char *line, double ***pZ, DATAINFO *pdinfo,
 
 void equation_system_destroy (equation_system *sys);
 
-const char *system_get_full_string (const equation_system *sys);
-
 int system_want_df_corr (const equation_system *sys);
 
 int system_n_restrictions (const equation_system *sys);
@@ -117,7 +138,9 @@ int system_adjust_t1t2 (equation_system *sys,
 
 int *system_get_list (const equation_system *sys, int i);
 
-int *compose_tsls_list (equation_system *sys, int i);
+int system_get_list_length (const equation_system *sys, int i);
+
+int *compose_tsls_list (const equation_system *sys, int i);
 
 int system_get_depvar (const equation_system *sys, int i);
 
@@ -132,9 +155,9 @@ const char *system_method_short_string (int method);
 int *system_get_endog_vars (const equation_system *sys);
 int *system_get_instr_vars (const equation_system *sys);
 
-void system_attach_uhat (equation_system *sys, gretl_matrix *uhat);
+void system_attach_uhat (equation_system *sys, gretl_matrix *E);
 
-void system_attach_sigma (equation_system *sys, gretl_matrix *sigma);
+void system_attach_sigma (equation_system *sys, gretl_matrix *S);
 
 void system_attach_coeffs (equation_system *sys, gretl_matrix *b);
 void system_attach_vcv (equation_system *sys, gretl_matrix *vcv);
@@ -187,6 +210,23 @@ void system_unset_save_flag (equation_system *sys);
 
 int system_save_flag_is_set (equation_system *sys);
 
+int gretl_system_print (equation_system *sys, 
+			const double **Z, const DATAINFO *pdinfo, 
+			gretlopt opt, PRN *prn);
+
+int system_print_sigma (const equation_system *sys, PRN *prn);
+
+const gretl_matrix *
+system_get_forecast_matrix (equation_system *sys, int t1, int t2,
+			    const double **Z, DATAINFO *pdinfo, 
+			    gretlopt opt, int *err);
+
+gretl_matrix *sys_get_fitted_values (equation_system *sys,
+				     int v, int t1, int t2,
+				     const double **Z, 
+				     const DATAINFO *pdinfo,
+				     int *err);
+
 #ifndef GRETLCLI
 
 equation_system *
@@ -199,4 +239,9 @@ system_save_and_print_results (equation_system *sys,
 			       double ***pZ, DATAINFO *pdinfo,
 			       gretlopt opt, PRN *prn);
 
+int system_autocorrelation_test (equation_system *sys, int order, 
+				 PRN *prn);
+
+int system_arch_test (equation_system *sys, int order, PRN *prn);
+
 #endif /* EQUATION_SYSTEM_H */
