@@ -20,6 +20,7 @@
 /* texprint.c for gretl - LaTeX output from modeling */
 
 #include "libgretl.h"
+#include "var.h"
 #include "johansen.h"
 #include "texprint.h"
 
@@ -509,7 +510,8 @@ void make_tex_coeff_name (const MODEL *pmod, const DATAINFO *pdinfo, int i,
 
 void tex_print_coeff (const model_coeff *mc, PRN *prn)
 {
-    char bstr[64], sestr[64], tstr[64], col4[64];
+    char col1[64], col2[64], col3[64], col4[64];
+    int ncols = 4;
 
     if (use_custom) {
 	tex_print_coeff_custom(mc, prn);
@@ -517,21 +519,27 @@ void tex_print_coeff (const model_coeff *mc, PRN *prn)
     }
 
     if (na(mc->b)) {
-	sprintf(bstr, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+	sprintf(col1, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
     } else {
-	tex_dcolumn_double(mc->b, bstr);
+	tex_dcolumn_double(mc->b, col1);
     }
 
-    if (na(mc->se)) {
-	sprintf(sestr, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+    if (!na(mc->lo) && !na(mc->hi)) {
+	tex_dcolumn_double(mc->lo, col2);
+	tex_dcolumn_double(mc->hi, col3);
+	ncols = 3;
     } else {
-	tex_dcolumn_double(mc->se, sestr);
-    }
+	if (na(mc->se)) {
+	    sprintf(col2, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+	} else {
+	    tex_dcolumn_double(mc->se, col2);
+	}
 
-    if (na(mc->tval)) {
-	sprintf(tstr, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
-    } else {
-	sprintf(tstr, "%.4f", mc->tval);
+	if (na(mc->tval)) {
+	    sprintf(col3, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+	} else {
+	    sprintf(col3, "%.4f", mc->tval);
+	}
     }
 
     *col4 = '\0';
@@ -546,18 +554,25 @@ void tex_print_coeff (const model_coeff *mc, PRN *prn)
 
     pprintf(prn, "%s &\n"
 	    "  %s &\n"
-	    "    %s &\n"
-	    "      %s &\n"
-	    "        %s \\\\\n",  
+	    "    %s &\n",
 	    mc->name,
-	    bstr,
-	    sestr,
-	    tstr,
-	    col4);	    
+	    col1,
+	    col2);
+
+    if (ncols == 4) {
+	pprintf(prn, 
+		"      %s &\n"
+		"        %s \\\\\n",  
+		col3,
+		col4);
+    } else {
+	pprintf(prn, 
+		"      %s \\\\\n",
+		col3);
+    }	
 }
 
-void tex_custom_coeff_table_start (const char *col1, const char *col2,
-				   PRN *prn)
+void tex_custom_coeff_table_start (const char **cols, PRN *prn)
 {
     int i, ncols = 0;
 
@@ -574,10 +589,10 @@ void tex_custom_coeff_table_start (const char *col1, const char *col2,
 
     pputs(prn, "}\n");
 
-    pprintf(prn, "\\multicolumn{1}{c}{%s} &\n", I_(col1));
+    pprintf(prn, "\\multicolumn{1}{c}{%s} &\n", I_(cols[0]));
 
     if (colspec[0][0]) {
-	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_(col2));
+	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_(cols[1]));
     }
 
     if (!colspec[1][0] && !colspec[2][0] && !colspec[3][0]) {
@@ -589,7 +604,7 @@ void tex_custom_coeff_table_start (const char *col1, const char *col2,
 	if (colspec[0][0]) {
 	    pputs(prn, " &\n");
 	}
-	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_("Std.\\ Error"));
+	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_(cols[2]));
     }
 
     if (!colspec[2][0] && !colspec[3][0]) {
@@ -601,50 +616,63 @@ void tex_custom_coeff_table_start (const char *col1, const char *col2,
 	if (colspec[0][0] || colspec[1][0]) {
 	    pputs(prn, " &\n");
 	}
-	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_("$t$-statistic"));
+	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_(cols[3]));
     }
 
     if (colspec[3][0]) {
 	if (colspec[0][0] || colspec[1][0] || colspec[2][0]) {
 	    pputs(prn, " &\n");
 	}
-	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_("p-value"));
+	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_(cols[4]));
     }
 
     pputs(prn, " \\\\\n");
 }
 
-void tex_coeff_table_start (const char *col1, const char *col2,
-			    int binary, PRN *prn)
+void tex_coeff_table_start (const char **cols, int binary, PRN *prn)
 {
 
     char pt;
 
     if (use_custom) {
-	tex_custom_coeff_table_start(col1, col2, prn);
+	tex_custom_coeff_table_start(cols, prn);
 	return;
     }
 
     pt = get_local_decpoint();
 
-    pprintf(prn, "\\vspace{1em}\n\n"
-	    "\\begin{tabular*}{\\textwidth}"
-	    "{@{\\extracolsep{\\fill}}\n"
-	    "l%% col 1: varname\n"
-	    "  D{%c}{%c}{-1}%% col 2: coeff\n"
-	    "    D{%c}{%c}{-1}%% col 3: sderr\n"
-	    "      D{%c}{%c}{-1}%% col 4: t-stat\n"
-	    "        D{%c}{%c}{4}}%% col 5: p-value (or slope)\n"
-	    "%s &\n"
-	    "  \\multicolumn{1}{c}{%s} &\n"
-	    "    \\multicolumn{1}{c}{%s} &\n"
-	    "      \\multicolumn{1}{c}{%s} &\n"
-	    "        \\multicolumn{1}{c}{%s%s} \\\\[1ex]\n",
-	    pt, pt, pt, pt, pt, pt, pt, pt, I_(col1),
-	    I_(col2), I_("Std.\\ Error"), 
-	    I_("$t$-statistic"), 
-	    (binary)? I_("Slope"): I_("p-value"),
-	    (binary)? "$^*$" : "");
+    if (cols[4] == NULL) {
+	pprintf(prn, "\\vspace{1em}\n\n"
+		"\\begin{tabular*}{\\textwidth}"
+		"{@{\\extracolsep{\\fill}}\n"
+		"l%% col 1: varname\n"
+		"  D{%c}{%c}{-1}%% col 2: coeff\n"
+		"    D{%c}{%c}{-1}%% col 3\n"
+		"      D{%c}{%c}{-1}}%% col 4\n"
+		"%s &\n"
+		"  \\multicolumn{1}{c}{%s} &\n"
+		"    \\multicolumn{1}{c}{%s} &\n"
+		"      \\multicolumn{1}{c}{%s} \\\\[1ex]\n",
+		pt, pt, pt, pt, pt, pt, 
+		I_(cols[0]), I_(cols[1]), I_(cols[2]), I_(cols[3]));
+    } else {
+	pprintf(prn, "\\vspace{1em}\n\n"
+		"\\begin{tabular*}{\\textwidth}"
+		"{@{\\extracolsep{\\fill}}\n"
+		"l%% col 1: varname\n"
+		"  D{%c}{%c}{-1}%% col 2: coeff\n"
+		"    D{%c}{%c}{-1}%% col 3: sderr\n"
+		"      D{%c}{%c}{-1}%% col 4: t-stat\n"
+		"        D{%c}{%c}{4}}%% col 5: p-value (or slope)\n"
+		"%s &\n"
+		"  \\multicolumn{1}{c}{%s} &\n"
+		"    \\multicolumn{1}{c}{%s} &\n"
+		"      \\multicolumn{1}{c}{%s} &\n"
+		"        \\multicolumn{1}{c}{%s%s} \\\\[1ex]\n",
+		pt, pt, pt, pt, pt, pt, pt, pt, 
+		I_(cols[0]), I_(cols[1]), I_(cols[2]), I_(cols[3]), I_(cols[4]),
+		(binary)? "$^*$" : "");
+    }
 }
 
 void tex_coeff_table_end (PRN *prn)
@@ -860,20 +888,24 @@ void tex_print_VAR_ll_stats (GRETL_VAR *var, PRN *prn)
     pprintf(prn, "\\noindent\n%s $= %.4f$ \\par\n", I_("HQC"), var->HQC);
 }
 
-static PRN *make_tex_prn (int ID, char *texfile,
-			  int eqn, int doc)
+static PRN *make_tex_prn (int ID, char *fname,
+			  int eqn, int doc, int *err)
 {
+    char texfile[FILENAME_MAX];
     PrnFormat fmt = GRETL_FORMAT_TEX;
     PRN *prn;
 
-    if (*texfile == '\0') {
-	sprintf(texfile, "%s%s_%d.tex", gretl_user_dir(),
+    if (*fname == '\0') {
+	sprintf(texfile, "%s%s_%d.tex", gretl_work_dir(),
 		(eqn)? "equation" : "model", ID);
+	strcpy(fname, texfile);
     } else {
-	gretl_maybe_switch_dir(texfile);
+	gretl_maybe_switch_dir(fname);
+	strcpy(texfile, fname);       
     }
 
-    prn = gretl_print_new_with_filename(texfile);
+    prn = gretl_print_new_with_filename(texfile, err);
+
     if (prn != NULL) {
 	if (eqn) {
 	    fmt |= GRETL_FORMAT_EQN;
@@ -887,6 +919,28 @@ static PRN *make_tex_prn (int ID, char *texfile,
     return prn;
 }
 
+static PRN *make_rtf_prn (int ID, char *fname, int *err)
+{
+    char rtffile[FILENAME_MAX];
+    PRN *prn;
+
+    if (*fname == '\0') {
+	sprintf(rtffile, "%smodel_%d.rtf", gretl_work_dir(), ID);
+	strcpy(fname, rtffile);
+    } else {
+	gretl_maybe_switch_dir(fname);
+	strcpy(rtffile, fname);       
+    }
+
+    prn = gretl_print_new_with_filename(rtffile, err);
+
+    if (prn != NULL) {
+	gretl_print_set_format(prn, GRETL_FORMAT_RTF);
+    }
+
+    return prn;
+}
+
 /* mechanism for customizing gretl's tex preamble */
 
 static char tex_preamble_file[MAXLEN];
@@ -916,7 +970,7 @@ void set_gretl_tex_preamble (void)
     const char *localtex = get_gretltex_local();
 
     /* first choice: localized preamble file */
-    sprintf(tex_preamble_file, "%s%s", gretl_user_dir(), localtex);
+    sprintf(tex_preamble_file, "%s%s", gretl_work_dir(), localtex);
     fp = gretl_fopen(tex_preamble_file, "r");
     if (fp == NULL) {
 	tex_preamble_file[0] = '\0';
@@ -927,7 +981,7 @@ void set_gretl_tex_preamble (void)
 #endif
 
     /* preamble file on disk */
-    sprintf(tex_preamble_file, "%s%s", gretl_user_dir(), gretltex);
+    sprintf(tex_preamble_file, "%s%s", gretl_work_dir(), gretltex);
     fp = gretl_fopen(tex_preamble_file, "r");
     if (fp == NULL) {
 	tex_preamble_file[0] = '\0';
@@ -1064,6 +1118,7 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
     int i, nc = pmod->ncoeff;
     int split = 0, offvar = 0;
     int cchars = 0, ccount = 0;
+    int sderr_ok = 1;
 
     if (pmod->ci == POISSON) {
 	offvar = gretl_model_get_int(pmod, "offset_var");
@@ -1118,19 +1173,26 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 	    pputc(prn, '+');
 	    tex_print_float(1.0, prn);
 	} else {
-	    if (opt & OPT_T) {
-		x = pmod->coeff[i] / pmod->sderr[i];
-		pprintf(prn, "%s\\underset{(%.3f)}{", 
-			(pmod->coeff[i] < 0.0)? "-" :
-			(i > 0)? "+" : "", x);
-	    } else {
-		pprintf(prn, "%s\\underset{(%.5g)}{", 
-			(pmod->coeff[i] < 0.0)? "-" :
-			(i > 0)? "+" : "", pmod->sderr[i]);
+	    if (na(pmod->sderr[i])) {
+		sderr_ok = 0;			
+		pprintf(prn, "%s{", (pmod->coeff[i] < 0.0)? "-" :
+			(i > 0)? "+" : "");
+	    } else if (sderr_ok) {
+		if (opt & OPT_T) {
+		    x = pmod->coeff[i] / pmod->sderr[i];
+		    pprintf(prn, "%s\\underset{(%.3f)}{", 
+			    (pmod->coeff[i] < 0.0)? "-" :
+			    (i > 0)? "+" : "", x);
+		} else {
+		    pprintf(prn, "%s\\underset{(%.5g)}{", 
+			    (pmod->coeff[i] < 0.0)? "-" :
+			    (i > 0)? "+" : "", pmod->sderr[i]);
+		}
 	    }
 	    tex_print_float(pmod->coeff[i], prn);
 	    pputc(prn, '}');
 	}
+
 	if (i > 0 || pmod->ifc == 0) {
 	    pputs(prn, "\\,");
 	    if (pmod->ci == ARMA) {
@@ -1236,9 +1298,13 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 
     /* additional info (R^2 etc) */
     if (pmod->ci == LAD) { 
-	sprintf(tmp, "%g", pmod->rho);
-	tex_modify_exponent(tmp);
-	pprintf(prn, "\\quad \\sum |\\hat{u}_t| = %s", tmp);
+	double SAR = gretl_model_get_double(pmod, "ladsum");
+
+	if (!na(SAR)) {
+	    sprintf(tmp, "%g", SAR);
+	    tex_modify_exponent(tmp);
+	    pprintf(prn, "\\quad \\sum |\\hat{u}_t| = %s ", tmp);
+	}
     } else {
 	if (!na(pmod->adjrsq)) {
 	    pprintf(prn, "\\quad \\bar{R}^2 = %.4f ", pmod->adjrsq);
@@ -1256,7 +1322,7 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 	if (!na(pmod->sigma)) {
 	    sprintf(tmp, "%.5g", pmod->sigma);
 	    tex_modify_exponent(tmp);
-	    pprintf(prn, "\\quad \\hat{\\sigma} = %s", tmp);
+	    pprintf(prn, "\\quad \\hat{\\sigma} = %s ", tmp);
 	}
 
 	if (!na(gretl_model_get_double(pmod, "rho_in"))) {
@@ -1269,9 +1335,15 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
     }
 
     pputs(prn, "\\notag \\\\\n");
-    pprintf(prn, "\\centerline{(%s)} \\notag\n",
-	    (opt & OPT_T)? I_("$t$-statistics in parentheses") :
-	    I_("standard errors in parentheses"));
+
+    if (sderr_ok) {
+	pprintf(prn, "\\centerline{(%s)} \\notag\n",
+		(opt & OPT_T)? I_("$t$-statistics in parentheses") :
+		I_("standard errors in parentheses"));
+    } else {
+	pputs(prn, "\\notag\n");
+    }
+
     pputs(prn, "\\end{gather}\n");
 
     if (opt & OPT_S) {
@@ -1305,6 +1377,10 @@ int tex_print_model (MODEL *pmod, const DATAINFO *pdinfo,
 {
     int ret;
 
+    if (RQ_SPECIAL_MODEL(pmod)) {
+	return E_NOTIMP;
+    }
+
     if (tex_doc_format(prn)) {
 	opt |= OPT_S;
     }
@@ -1322,7 +1398,7 @@ int tex_print_model (MODEL *pmod, const DATAINFO *pdinfo,
  * texprint:
  * @pmod: pointer to model.
  * @pdinfo: information regarding the data set.
- * @texfile: name of file to save.
+ * @fname: name of file to save.
  * @opt: if opt & %OPT_O, complete doc, else fragment;
  * if opt & %OPT_E print as equation, otherwise use tabular
  * format; if opt & %OPT_T show t-ratios rather than standard
@@ -1335,7 +1411,7 @@ int tex_print_model (MODEL *pmod, const DATAINFO *pdinfo,
  * Returns: 0 on successful completion, 1 on error.
  */
 
-int texprint (MODEL *pmod, const DATAINFO *pdinfo, char *texfile, 
+int texprint (MODEL *pmod, const DATAINFO *pdinfo, char *fname, 
 	      gretlopt opt)
 {
     PRN *prn;
@@ -1343,10 +1419,13 @@ int texprint (MODEL *pmod, const DATAINFO *pdinfo, char *texfile,
     int doc = (opt & OPT_O);
     int err = 0;
 
-    prn = make_tex_prn(pmod->ID, texfile, eqn, doc);
-    if (prn == NULL) {
-	err = 1;
-    } else {
+    if (RQ_SPECIAL_MODEL(pmod)) {
+	return E_NOTIMP;
+    }
+
+    prn = make_tex_prn(pmod->ID, fname, eqn, doc, &err);
+
+    if (!err) {
 	err = tex_print_model(pmod, pdinfo, opt, prn);
 	gretl_print_destroy(prn);
     }
@@ -1354,6 +1433,26 @@ int texprint (MODEL *pmod, const DATAINFO *pdinfo, char *texfile,
     return err;
 }
 
+int rtfprint (MODEL *pmod, const DATAINFO *pdinfo, char *fname, 
+	      gretlopt opt)
+{
+    PRN *prn;
+    int err = 0;
+
+    if (RQ_SPECIAL_MODEL(pmod)) {
+	return E_NOTIMP;
+    }
+
+    prn = make_rtf_prn(pmod->ID, fname, &err);
+
+    if (!err) {
+	err = printmodel(pmod, pdinfo, opt, prn);
+	gretl_print_destroy(prn);
+    }
+
+    return err;
+}
+
 /**
  * tex_print_obs_marker:
  * @t: observation number.
