@@ -1,21 +1,20 @@
-/* gretl - The Gnu Regression, Econometrics and Time-series Library
- *
- * Copyright (C) Allin Cottrell
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef TEXPRINT_H
@@ -27,8 +26,7 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 int tex_print_model (MODEL *pmod, const DATAINFO *pdinfo, 
 		     gretlopt opt, PRN *prn);
 
-void tex_coeff_table_start (const char *col1, const char *col2,
-			    int binary, PRN *prn);
+void tex_coeff_table_start (const char **cols, int binary, PRN *prn);
 
 void tex_coeff_table_end (PRN *prn);
 
@@ -43,7 +41,10 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *pr
 
 void tex_print_VAR_ll_stats (GRETL_VAR *var, PRN *prn);
 
-int texprint (MODEL *pmod, const DATAINFO *pdinfo, char *texfile, 
+int texprint (MODEL *pmod, const DATAINFO *pdinfo, char *fname, 
+	      gretlopt opt);
+
+int rtfprint (MODEL *pmod, const DATAINFO *pdinfo, char *fname, 
 	      gretlopt opt);
 
 char *tex_escape (char *targ, const char *src);
