@@ -32,6 +32,8 @@ enum {
     INVERSE = NC + 1
 };
 
+static char *get_mangled_name_by_id (int v);
+
 static int check_vals (const double *x, const double *y, int n)
 {
     int ret = VARS_IDENTICAL;
@@ -70,6 +72,7 @@ make_transform_varname (char *vname, const char *orig, int ci,
 {
     *vname = '\0';
 
+
     if (ci == DIFF) {
 	strcpy(vname, "d_");
 	strncat(vname, orig, len - 2);
@@ -79,6 +82,9 @@ make_transform_varname (char *vname, const char *orig, int ci,
     } else if (ci == SDIFF) {
 	strcpy(vname, "sd_");
 	strncat(vname, orig, len - 3);
+    } else if (ci == ORTHDEV) {
+	strcpy(vname, "o_");
+	strncat(vname, orig, len - 2);
     } else if (ci == LOGS) {
 	strcpy(vname, "l_");
 	strncat(vname, orig, len - 2);
@@ -109,6 +115,12 @@ make_transform_varname (char *vname, const char *orig, int ci,
 	strncat(vname, orig, len - 2);
     }
 
+#if TRDEBUG
+    fprintf(stderr, "make_transform_varname:\n"
+	    "orig='%s', ci=%d, len=%d, vname='%s'\n", 
+	    orig, ci, len, vname);
+#endif
+
     return 0;
 }
 
@@ -145,15 +157,16 @@ make_transform_label (char *label, const char *parent,
 
 int is_standard_lag (int v, const DATAINFO *pdinfo, int *parent)
 {
-    const char *test = VARLABEL(pdinfo, v);
-    char pm, vname[VNAMELEN];
-    int lag, ret = 0;
+    int pv = 0, ret = 0;
 
-    if (sscanf(test, "= %15[^(](t %c %d)", vname, &pm, &lag) == 3) {
+    if (pdinfo->varinfo[v]->transform == LAGS) {
+	pv = varindex(pdinfo, pdinfo->varinfo[v]->parent);
+	pv = (pv < pdinfo->v)? pv : 0;
+    }
+
+    if (pv > 0) {
 	if (parent != NULL) {
-	    int pv = varindex(pdinfo, vname);
-
-	    *parent = (pv < pdinfo->v)? pv : 0;
+	    *parent = pv;
 	}
 	ret = 1;
     }
@@ -163,15 +176,11 @@ int is_standard_lag (int v, const DATAINFO *pdinfo, int *parent)
 
 int is_dummy_child (int v, const DATAINFO *pdinfo, int *parent)
 {
-    const char *test = VARLABEL(pdinfo, v);
-    char vname[VNAMELEN];
-    double val;
     int pv = pdinfo->v;
     int i = 0, ret = 0;
 
-    if (sscanf(test, _("dummy for %s = %lf"), vname, &val) == 2 ||
-	sscanf(test, "dummy for %s = %lf", vname, &val) == 2) {
-	pv = varindex(pdinfo, vname);
+    if (pdinfo->varinfo[v]->transform == DUMMIFY) {
+	pv = varindex(pdinfo, pdinfo->varinfo[v]->parent);
     } else if (!strncmp(pdinfo->varname[v], "dt_", 3)) {
 	if (sscanf(pdinfo->varname[v] + 3, "%d", &i) && i > 1) {
 	    pv = varindex(pdinfo, "dt_1");
@@ -192,16 +201,26 @@ int is_dummy_child (int v, const DATAINFO *pdinfo, int *parent)
     return ret;
 }
 
-static void make_xp_varname (char *vname, const char *v1, 
-			     const char *v2, int len)
+static void make_xp_varname (char *vname, int v1, int v2,
+			     const DATAINFO *pdinfo,
+			     int len)
 {
-    int v2len = len / 2;
-    int v1len = (len % 2)? v2len : v2len - 1;
+    int n1 = strlen(pdinfo->varname[v1]);
+    int n2 = strlen(pdinfo->varname[v2]);
+    int cut = n1 + n2 + 1 - len;
+    int cut1 = 0, cut2 = 0;
+
+    if (cut > 0) {
+	cut1 = cut2 = cut/2;
+	if (cut % 2) {
+	    cut2++;
+	}
+    }
 
     *vname = '\0';
-    strncat(vname, v1, v1len);
+    strncat(vname, pdinfo->varname[v1], n1 - cut1);
     strcat(vname, "_");
-    strncat(vname, v2, v2len);
+    strncat(vname, pdinfo->varname[v2], n2 - cut2);    
 }
 
 /* Array into which to write a generated variable, prior to testing
@@ -255,7 +274,7 @@ static int get_lag (int v, int lag, double *lagvec,
 {
     int t1 = (lag > 0)? lag : 0;
     int t2 = pdinfo->n - 1;
-    int t, lt;
+    int t, lt, miss;
 
     for (t=0; t<pdinfo->n; t++) {
 	lagvec[t] = NADBL;
@@ -264,8 +283,10 @@ static int get_lag (int v, int lag, double *lagvec,
     if (dated_daily_data(pdinfo)) {
 	for (t=t1; t<=t2; t++) {
 	    lt = t - lag;
-	    while (lt >= 0 && na(Z[v][lt])) {
+	    miss = 0;
+	    while (na(Z[v][lt]) && lt > 0 && miss < 6) {
 		lt--;
+		miss++;
 	    }
 	    lagvec[t] = Z[v][lt];
 	}
@@ -352,6 +373,14 @@ static int get_diff (int v, double *diffvec, int ci,
     return 0;
 }
 
+/* orthogonal deviations */
+
+static int get_orthdev (int v, double *xvec, const double **Z, 
+			const DATAINFO *pdinfo)
+{
+    return orthdev_series(Z[v], xvec, pdinfo);
+}
+
 /* write square or cross-product into xvec */
 
 static int get_xpx (int vi, int vj, double *xvec, const double **Z, 
@@ -420,10 +449,53 @@ enum transform_results {
     VARNAME_DUPLICATE
 };
 
+/* Note: new behavior as of 2008-02-11 */
+
+#define TR_OVERWRITE 1
+
+static int transform_handle_duplicate (int v, const double *x,
+				       const char *label,
+				       DATAINFO *pdinfo, double **Z,
+				       int origv)
+{
+    int ret = VARNAME_DUPLICATE;
+    int t, ok = 0;
+
+    if (!strcmp(label, VARLABEL(pdinfo, v))) {
+	/* labels identical, so OK? */
+	ok = 1;
+	
+    }
+
+#if TR_OVERWRITE
+    if (!ok && v < origv) {
+	ok = 1;
+    }
+#endif
+
+    if (ok) {
+#if TRDEBUG
+	fprintf(stderr, "check_add_transform: updating var %d (%s)\n",
+		v, pdinfo->varname[v]);
+#endif
+	for (t=0; t<pdinfo->n; t++) {
+	    Z[v][t] = x[t];
+	}
+	if (*label != '\0') {
+	    strcpy(VARLABEL(pdinfo, v), label);
+	}
+	pdinfo->varinfo[v]->flags = 0;
+	ret = VAR_EXISTS_OK;
+    }	
+
+    return ret;
+}
+
 static int 
 check_add_transform (int vnum, const double *x,
 		     const char *vname, const char *label,
-		     DATAINFO *pdinfo, double ***pZ)
+		     DATAINFO *pdinfo, double ***pZ,
+		     int origv)
 {
     int t, ret = VAR_ADDED_OK;
 
@@ -450,20 +522,8 @@ check_add_transform (int vnum, const double *x,
 	    }
 	    ret = VAR_EXISTS_OK;
 	} else {
-	    if (!strcmp(label, VARLABEL(pdinfo, vnum))) {
-		/* labels match: update the values */
-#if TRDEBUG
-		fprintf(stderr, "check_add_transform: updating var %d (%s)\n",
-			vnum, vname);
-#endif
-		for (t=0; t<pdinfo->n; t++) {
-		    (*pZ)[vnum][t] = x[t];
-		}
-		ret = VAR_EXISTS_OK;
-	    } else {
-		/* labels do not match: problem */
-		ret = VARNAME_DUPLICATE;
-	    }
+	    ret = transform_handle_duplicate(vnum, x, label, pdinfo,
+					     *pZ, origv);
 	}
     } else {
 	/* no var of this name, working from scratch */
@@ -481,22 +541,42 @@ check_add_transform (int vnum, const double *x,
     return ret;
 }
 
+static int get_lag_ID (int srcv, int lag, const DATAINFO *pdinfo)
+{
+    const char *vname = pdinfo->varname[srcv];
+    VARINFO *vinfo;
+    int i;
+
+    for (i=1; i<pdinfo->v; i++) {
+	vinfo = pdinfo->varinfo[i];
+	if (vinfo->lag == lag && !strcmp(vinfo->parent, vname)) {
+	    return i;
+	}
+    }
+
+    return -1;
+}
+
 /* get_transform: create specified transformation of variable v if
    this variable does not already exist.
 
    The dummies resulting from DUMMIFY are automatically marked as
    discrete.
 
+   origv is the number of variables in the dataset prior to the
+   current round of adding.
+
    Return the ID number of the transformed var, or -1 on error.
 */
 
 static int get_transform (int ci, int v, int aux, double x, 
 			  double ***pZ, DATAINFO *pdinfo,
-			  int startlen)
+			  int startlen, int origv)
 {
-    char vname[VNAMELEN];
-    char label[MAXLABEL];
+    char vname[VNAMELEN] = {0};
+    char label[MAXLABEL] = {0};
     int len, vno = -1, err = 0;
+    const char *srcname;
     double *vx;
 
     vx = testvec(pdinfo->n);
@@ -511,6 +591,8 @@ static int get_transform (int ci, int v, int aux, double x,
 	err = get_log(v, vx, (const double **) *pZ, pdinfo);
     } else if (ci == DIFF || ci == LDIFF || ci == SDIFF) {
 	err = get_diff(v, vx, ci, (const double **) *pZ, pdinfo);
+    } else if (ci == ORTHDEV) {
+	err = get_orthdev(v, vx, (const double **) *pZ, pdinfo);
     } else if (ci == SQUARE) {
 	/* "aux" = second variable number */
 	err = get_xpx(v, aux, vx, (const double **) *pZ, pdinfo);
@@ -525,6 +607,15 @@ static int get_transform (int ci, int v, int aux, double x,
 	return -1;
     }
 
+    if (ci == LAGS && (vno = get_lag_ID(v, aux, pdinfo)) > 0) {
+	/* special case: pre-existing lag */
+	err = check_add_transform(vno, vx, vname, label, pdinfo, pZ, origv);
+	if (err != VAR_EXISTS_OK) {
+	    vno = -1;
+	}
+	return vno;
+    }
+
     if (ci == SQUARE && v != aux) {
 	sprintf(label, _("= %s times %s"), pdinfo->varname[v], 
 		pdinfo->varname[aux]);
@@ -533,22 +624,24 @@ static int get_transform (int ci, int v, int aux, double x,
     } else {
 	make_transform_label(label, pdinfo->varname[v], ci, aux);
     }
-    
-    for (len=startlen; len<=VNAMELEN; len++) {
 
+    srcname = get_mangled_name_by_id(v);
+    if (srcname == NULL) {
+	srcname = pdinfo->varname[v];
+    }
+
+    for (len=startlen; len<=VNAMELEN; len++) {
 	if (len == VNAMELEN) {
 	    /* last resort: hack the name */
 	    make_varname_unique(vname, 0, pdinfo);
 	} else if (ci == SQUARE && v != aux) {
-	    make_xp_varname(vname, pdinfo->varname[v],
-			    pdinfo->varname[aux], len);
+	    make_xp_varname(vname, v, aux, pdinfo, len);
 	} else {
-	    make_transform_varname(vname, pdinfo->varname[v], ci, 
-				   aux, len);
+	    make_transform_varname(vname, srcname, ci, aux, len);
 	}
 	vno = varindex(pdinfo, vname);
 
-	err = check_add_transform(vno, vx, vname, label, pdinfo, pZ);
+	err = check_add_transform(vno, vx, vname, label, pdinfo, pZ, origv);
 	if (err != VAR_EXISTS_OK && err != VAR_ADDED_OK) {
 	    vno = -1;
 	}
@@ -558,8 +651,18 @@ static int get_transform (int ci, int v, int aux, double x,
 	}
     }
 
-    if (!err && ci == DUMMIFY && vno > 0) {
-	set_var_discrete(pdinfo, vno, 1);
+    if (!err && vno > 0) {
+	if (ci == DUMMIFY) {
+	    strcpy(pdinfo->varinfo[vno]->parent, pdinfo->varname[v]);
+	    pdinfo->varinfo[vno]->transform = DUMMIFY;
+	    set_var_discrete(pdinfo, vno, 1);
+	} else if (ci == LAGS || ci == DIFF) {
+	    strcpy(pdinfo->varinfo[vno]->parent, pdinfo->varname[v]);
+	    pdinfo->varinfo[vno]->transform = ci;
+	    if (ci == LAGS) {
+		pdinfo->varinfo[vno]->lag = aux;
+	    }
+	}
     }
 
     return vno;
@@ -582,17 +685,83 @@ int laggenr (int v, int lag, double ***pZ, DATAINFO *pdinfo)
 {
     int lno;
 
-    if (var_is_scalar(pdinfo, v) || lag > pdinfo->n) {
+    if (var_is_scalar(pdinfo, v)) {
+	sprintf(gretl_errmsg, _("variable %s is a scalar"), pdinfo->varname[v]);
+	lno = -1;
+    } else if (lag > pdinfo->n || -lag > pdinfo->n) {
+	sprintf(gretl_errmsg, _("Invalid lag order %d"), lag);
 	lno = -1;
     } else if (lag == 0) {
 	lno = v;
     } else {
-	lno = get_transform(LAGS, v, lag, 0.0, pZ, pdinfo, VNAMELEN - 3);
+	lno = get_transform(LAGS, v, lag, 0.0, pZ, pdinfo, 
+			    VNAMELEN - 3, pdinfo->v);
     }
 
     return lno;
 }
 
+/**
+ * laggenr_from_to: 
+ * @v: ID number in dataset of source variable.
+ * @minlag: minimum lag order.
+ * @maxlag: maximum lag order.
+ * @pZ: pointer to data array.
+ * @pdinfo: information on dataset.
+ * @err: location to receive error code.
+ *
+ * Creates the specified lags of variable @v if they do not
+ * already exist.
+ *
+ * Returns: list of lag variables, or %NULL or on error.
+ */
+
+int *laggenr_from_to (int v, int minlag, int maxlag, double ***pZ, 
+		      DATAINFO *pdinfo, int *err)
+{
+    int *llist;
+    int i, lv, nlags = -1;
+    int p, reverse;
+
+    if (maxlag < 0) {
+	nlags = -maxlag + minlag + 1;
+    } else if (maxlag > 0) {
+	nlags = maxlag - minlag + 1;
+    } 
+
+    if (nlags <= 0) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    llist = gretl_list_new(nlags);
+    if (llist == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    reverse = (maxlag < minlag);
+    p = minlag;
+
+    for (i=0; i<nlags; i++) {
+	lv = laggenr(v, p, pZ, pdinfo);
+	if (lv < 0) {
+	    *err = E_DATA;
+	    free(llist);
+	    llist = NULL;
+	    break;
+	}
+	llist[i+1] = lv;
+	if (reverse) {
+	    p--;
+	} else {
+	    p++;
+	}
+    }
+
+    return llist;
+}
+
 /**
  * loggenr: 
  * @v: ID number in dataset of source variable.
@@ -607,7 +776,8 @@ int laggenr (int v, int lag, double ***pZ, DATAINFO *pdinfo)
 
 int loggenr (int v, double ***pZ, DATAINFO *pdinfo)
 {
-    return get_transform(LOGS, v, 0, 0.0, pZ, pdinfo, VNAMELEN - 3);
+    return get_transform(LOGS, v, 0, 0.0, pZ, pdinfo, 
+			 VNAMELEN - 3, pdinfo->v);
 }
 
 /**
@@ -624,7 +794,8 @@ int loggenr (int v, double ***pZ, DATAINFO *pdinfo)
 
 int invgenr (int v, double ***pZ, DATAINFO *pdinfo)
 {
-    return get_transform(INVERSE, v, 0, 0.0, pZ, pdinfo, VNAMELEN - 3);
+    return get_transform(INVERSE, v, 0, 0.0, pZ, pdinfo, 
+			 VNAMELEN - 3, pdinfo->v);
 }
 
 /**
@@ -656,7 +827,8 @@ int diffgenr (int v, int ci, double ***pZ, DATAINFO *pdinfo)
 	return -1;
     }
 
-    return get_transform(ci, v, 0, 0.0, pZ, pdinfo, VNAMELEN - 3);
+    return get_transform(ci, v, 0, 0.0, pZ, pdinfo, 
+			 VNAMELEN - 3, pdinfo->v);
 }
 
 /**
@@ -681,16 +853,23 @@ int xpxgenr (int vi, int vj, double ***pZ, DATAINFO *pdinfo)
 	}
     }
 
-    return get_transform(SQUARE, vi, vj, 0.0, pZ, pdinfo, VNAMELEN - 3);
+    return get_transform(SQUARE, vi, vj, 0.0, pZ, pdinfo, 
+			 VNAMELEN - 3, pdinfo->v);
 }
 
 static int 
 get_starting_length (const int *list, DATAINFO *pdinfo, int trim)
 {
     int width = VNAMELEN - 3 - trim;
+    const char *vni, *vnj;
     int len, maxlen = 0;
+    int conflict = 0;
     int i, j;
 
+    if (list[0] == 1) {
+	return VNAMELEN - 1;
+    }
+
     for (i=1; i<=list[0]; i++) {
 	len = strlen(pdinfo->varname[list[i]]);
 	if (len > maxlen) {
@@ -704,20 +883,15 @@ get_starting_length (const int *list, DATAINFO *pdinfo, int trim)
     }
 
     for (len=width; len<=maxlen; len++) {
-	int conflict = 0;
-
-	for (i=1; i<=list[0]; i++) {
-	    for (j=i+1; j<=list[0]; j++) {
-		if (!strncmp(pdinfo->varname[list[i]],
-			     pdinfo->varname[list[j]],
-			     len)) {
+	conflict = 0;
+	for (i=1; i<=list[0] && !conflict; i++) {
+	    vni = pdinfo->varname[list[i]];
+	    for (j=i+1; j<=list[0] && !conflict; j++) {
+		vnj = pdinfo->varname[list[j]];
+		if (!strncmp(vni, vnj, len)) {
 		    conflict = 1;
-		    break;
 		}
 	    }
-	    if (conflict) {
-		break;
-	    }
 	}
 	if (!conflict) {
 	    break;
@@ -735,6 +909,167 @@ get_starting_length (const int *list, DATAINFO *pdinfo, int trim)
     return len;
 }
 
+struct mangled_name {
+    int v;
+    char *s;
+};
+
+static struct mangled_name *mnames;
+static int n_mnames;
+
+static void destroy_mangled_names (void)
+{
+    int i;
+
+    for (i=0; i<n_mnames; i++) {
+	free(mnames[i].s);
+    }
+    free(mnames);
+    mnames = NULL;
+    n_mnames = 0;
+}
+
+static int push_mangled_name (int v, const char *s)
+{
+    static struct mangled_name *tmp;
+
+    tmp = realloc(mnames, (n_mnames + 1) * sizeof *tmp);
+
+    if (tmp == NULL) {
+	destroy_mangled_names();
+	return E_ALLOC;
+    }
+
+    mnames = tmp;
+    mnames[n_mnames].v = v;
+    mnames[n_mnames].s = gretl_strdup(s);
+
+    n_mnames++;
+
+    return 0;
+}
+
+static char *get_mangled_name_by_id (int v)
+{
+    int i;
+
+    for (i=0; i<n_mnames; i++) {
+	if (mnames[i].v == v) {
+	    return mnames[i].s;
+	}
+    }
+
+    return NULL;
+}
+
+static int make_mangled_name (int v, const char *s, int nc)
+{
+    char tmp[16], sfx[16];
+    int n, err;
+
+    if (get_mangled_name_by_id(v)) {
+	return 0;
+    }
+
+    *tmp = *sfx = '\0';
+    /* we know the bit out here must be distinct? */
+    strcat(sfx, s + nc);
+    n = strlen(sfx);
+    strncat(tmp, s, nc - n);
+    strcat(tmp, sfx);
+
+#if TRDEBUG
+    fprintf(stderr, "mangled name: '%s' -> '%s'\n", s, tmp);
+#endif
+
+    err = push_mangled_name(v, tmp);
+
+    return err;
+}
+
+static int 
+transform_preprocess_list (int *list, double **Z, const DATAINFO *pdinfo,
+			   int f)
+{
+    int maxc = VNAMELEN - 3;
+    int longnames = 0;
+    char **S = NULL;
+    int i, v, ok;
+    int err = 0;
+
+    if (f == SQUARE || f == LDIFF || f == SDIFF) {
+	/* 3-character prefixes */
+	maxc--;
+    } else if (f == DUMMIFY) {
+	/* "D" plus suffix */
+	maxc--;
+    }
+
+    for (i=1; i<=list[0]; i++) {
+	v = list[i];
+	ok = 1;
+
+	if (f == LOGS || f == SQUARE) {
+	    if (v == 0 || var_is_scalar(pdinfo, v)) {
+		ok = 1; 
+	    }
+	    if (gretl_isdummy(pdinfo->t1, pdinfo->t2, Z[v])) {
+		ok = 0;
+	    }
+	} else if (f == LAGS) {
+	    if (v == 0 || var_is_scalar(pdinfo, v)) {
+		ok = 0;
+	    }
+	} else if (f == DIFF || f == LDIFF || f == SDIFF || 
+		   f == ORTHDEV) {
+	    if (var_is_scalar(pdinfo, v)) {
+		ok = 0;
+	    }
+	} else if (f == DUMMIFY) {
+	    ok = 0; /* reverse burden of proof */
+	    if (v > 0 && var_is_series(pdinfo, v)) {
+		if (var_is_discrete(pdinfo, v)) {
+		    /* pre-approved */
+		    ok = 1;
+		} else if (gretl_isdiscrete(0, pdinfo->n - 1, Z[v])) {
+		    ok = 1;
+		}
+	    }
+	}
+
+	if (!ok) {
+	    gretl_list_delete_at_pos(list, i--);
+	} else if (strlen(pdinfo->varname[v]) > maxc) {
+	    strings_array_add(&S, &longnames, pdinfo->varname[v]);
+	}
+    }
+
+    if (list[0] == 0) {
+	err = E_TYPES;
+    }
+
+    if (longnames > 0) {
+	if (!err) {
+	    int j, herr = 0;
+
+	    for (i=0; i<longnames && !herr; i++) {
+		for (j=i+1; j<longnames && !herr; j++) {
+		    if (!strncmp(S[i], S[j], maxc)) {
+			v = varindex(pdinfo, S[i]);
+			herr = make_mangled_name(v, S[i], maxc);
+			v = varindex(pdinfo, S[j]);
+			herr += make_mangled_name(v, S[j], maxc);
+		    }
+		}
+	    }
+	}
+
+	free_strings_array(S, longnames);
+    }
+
+    return err;
+}
+
 /**
  * list_loggenr:
  * @list: on entry, list of variables to process; on exit,
@@ -750,31 +1085,35 @@ get_starting_length (const int *list, DATAINFO *pdinfo, int trim)
 
 int list_loggenr (int *list, double ***pZ, DATAINFO *pdinfo)
 {
-    int tnum, i, v;
+    int origv = pdinfo->v;
+    int tnum, i, j, v;
     int startlen;
-    int n_ok = 0;
+    int l0 = 0;
+    int err;
+
+    err = transform_preprocess_list(list, *pZ, pdinfo, LOGS);
+    if (err) {
+	return err;
+    }
 
     startlen = get_starting_length(list, pdinfo, 2);
 
+    j = 1;
     for (i=1; i<=list[0]; i++) {
 	v = list[i];
-
-	if (v == 0 || var_is_scalar(pdinfo, v)) {
-	    continue; 
-	}
-	if (gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[v])) {
-	    continue;
-	}
-
-	tnum = get_transform(LOGS, v, 0, 0.0, pZ, pdinfo, startlen);
-
+	tnum = get_transform(LOGS, v, 0, 0.0, pZ, pdinfo, startlen,
+			     origv);
 	if (tnum > 0) {
-	    n_ok++;
-	    list[i] = tnum;
+	    list[j++] = tnum;
+	    l0++;
 	}
     }
 
-    return (n_ok > 0)? 0 : E_LOGS;
+    list[0] = l0;
+
+    destroy_mangled_names();
+
+    return (l0 > 0)? 0 : E_LOGS;
 }
 
 static int *make_lags_list (int *list, int order, DATAINFO *pdinfo)
@@ -807,12 +1146,15 @@ static int *make_lags_list (int *list, int order, DATAINFO *pdinfo)
 
 int list_laggenr (int **plist, int order, double ***pZ, DATAINFO *pdinfo)
 {
+    int origv = pdinfo->v;
     int *list = *plist;
     int *laglist = NULL;
-    int l, i, j;
-    int startlen;
+    int l, i, j, v, lv;
+    int startlen, l0 = 0;
+    int err;
 
     if (order < 0) {
+	sprintf(gretl_errmsg, _("Invalid lag order %d"), order);
 	return E_DATA;
     }
 
@@ -820,39 +1162,43 @@ int list_laggenr (int **plist, int order, double ***pZ, DATAINFO *pdinfo)
 	order = default_lag_order(pdinfo);
     } 
 
+    err = transform_preprocess_list(list, *pZ, pdinfo, LAGS);
+    if (err) {
+	return err;
+    }
+
     laglist = make_lags_list(list, order, pdinfo);
     if (laglist == NULL) {
+	destroy_mangled_names();
 	return E_ALLOC;
     }
 
     startlen = get_starting_length(list, pdinfo, (order > 9)? 3 : 2);
 
     j = 1;
-    
     for (i=1; i<=list[0]; i++) {
-	int lv, v = list[i];
-
-	if (v == 0 || var_is_scalar(pdinfo, v)) {
-	    continue;
-	}
-
+	v = list[i];
 	for (l=1; l<=order; l++) {
-	    lv = get_transform(LAGS, v, l, 0.0, pZ, pdinfo, startlen);
+	    lv = get_transform(LAGS, v, l, 0.0, pZ, pdinfo, startlen, origv);
 #if TRDEBUG > 1
 	    fprintf(stderr, "base var '%s', lag %d: lv = %d\n",
 		    pdinfo->varname[v], l, lv);
 #endif
-	    if (lv < 0) {
-		return 1;
-	    }
+	    if (lv > 0) {
 #if TRDEBUG > 1
-	    fprintf(stderr, "lag var name '%s', label '%s'\n",
-		    pdinfo->varname[lv], VARLABEL(pdinfo, lv));
+		fprintf(stderr, "lag var name '%s', label '%s'\n",
+			pdinfo->varname[lv], VARLABEL(pdinfo, lv));
 #endif
-	    laglist[j++] = lv;
+		laglist[j++] = lv;
+		l0++;
+	    }
 	}
     }
 
+    destroy_mangled_names();
+
+    laglist[0] = l0;
+
     free(*plist);
     *plist = laglist;
 
@@ -897,8 +1243,14 @@ int default_lag_order (const DATAINFO *pdinfo)
 
 int list_diffgenr (int *list, int ci, double ***pZ, DATAINFO *pdinfo)
 {
+    int origv = pdinfo->v;
     int i, v, startlen;
-    int tnum, err = 0;
+    int tnum, l0 = 0;
+    int err;
+
+    if (list[0] == 0) {
+	return 0;
+    }
 
     if (ci != DIFF && ci != LDIFF && ci != SDIFF) {
 	return 1;
@@ -906,20 +1258,82 @@ int list_diffgenr (int *list, int ci, double ***pZ, DATAINFO *pdinfo)
 
     if (ci == SDIFF && !dataset_is_seasonal(pdinfo)) {
 	return E_PDWRONG;
-    }    
+    } 
+
+    err = transform_preprocess_list(list, *pZ, pdinfo, ci);
+    if (err) {
+	return err;
+    }
 
     startlen = get_starting_length(list, pdinfo, (ci == DIFF)? 2 : 3);
     
     for (i=1; i<=list[0] && !err; i++) {
 	v = list[i];
-	tnum = get_transform(ci, v, 0, 0.0, pZ, pdinfo, startlen);
+	tnum = get_transform(ci, v, 0, 0.0, pZ, pdinfo, startlen, origv);
 	if (tnum < 0) {
 	    err = 1;
 	} else {
 	    list[i] = tnum;
+	    l0++;
 	}
     }
 
+    list[0] = l0;
+
+    destroy_mangled_names();
+
+    return err;
+}
+
+/**
+ * list_orthdev:
+ * @list: list of variables to process.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: data information struct.
+ *
+ * Generate orthogonal deviations of the variables in @list, and add
+ * them to the data set.
+ *
+ * Returns: 0 on success, error code on error.
+ */
+
+int list_orthdev (int *list, double ***pZ, DATAINFO *pdinfo)
+{
+    int origv = pdinfo->v;
+    int i, v, startlen;
+    int tnum, l0 = 0;
+    int err;
+
+    if (list[0] == 0) {
+	return 0;
+    }
+
+    if (!dataset_is_panel(pdinfo)) {
+	return E_PDWRONG;
+    } 
+
+    err = transform_preprocess_list(list, *pZ, pdinfo, ORTHDEV);
+    if (err) {
+	return err;
+    }
+
+    startlen = get_starting_length(list, pdinfo, 2);
+    
+    for (i=1; i<=list[0] && !err; i++) {
+	v = list[i];
+	tnum = get_transform(ORTHDEV, v, 0, 0.0, pZ, pdinfo, startlen, origv);
+	if (tnum < 0) {
+	    err = 1;
+	} else {
+	    list[i] = tnum;
+	    l0++;
+	}
+    }
+
+    list[0] = l0;
+
+    destroy_mangled_names();
+
     return err;
 }
 
@@ -943,17 +1357,26 @@ int list_diffgenr (int *list, int ci, double ***pZ, DATAINFO *pdinfo)
 int list_xpxgenr (int **plist, double ***pZ, DATAINFO *pdinfo, 
 		  gretlopt opt)
 {
+    int origv = pdinfo->v;
     int *list = *plist;
-    int l0 = list[0];
     int *xpxlist = NULL;
     int tnum, i, j, k, vi, vj;
-    int startlen;
+    int startlen, l0;
+    int err;
+
+    err = transform_preprocess_list(list, *pZ, pdinfo, SQUARE);
+    if (err) {
+	return err;
+    }
+
+    l0 = list[0];
 
     if (opt & OPT_O) {
 	int maxterms = (l0 * l0 + l0) / 2;
     
 	xpxlist = gretl_list_new(maxterms);
 	if (xpxlist == NULL) {
+	    destroy_mangled_names();
 	    return E_ALLOC;
 	}
     } else {
@@ -966,20 +1389,12 @@ int list_xpxgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
     k = 1;
     for (i=1; i<=l0; i++) {
 	vi = list[i];
-
-	if (vi == 0 || var_is_scalar(pdinfo, vi)) {
-	    continue; 
-	}
-	if (gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[vi])) {
-	    continue;
-	}
-
-	tnum = get_transform(SQUARE, vi, vi, 0.0, pZ, pdinfo, startlen);
+	tnum = get_transform(SQUARE, vi, vi, 0.0, pZ, pdinfo, startlen,
+			     origv);
 	if (tnum > 0) {
 	    xpxlist[k++] = tnum;
 	    xpxlist[0] += 1;
 	}
-
 	if (opt & OPT_O) {
 	    for (j=i+1; j<=l0; j++) {
 		vj = list[j];
@@ -992,6 +1407,8 @@ int list_xpxgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
+    destroy_mangled_names();
+
     if (opt & OPT_O) {
 	free(*plist);
 	*plist = xpxlist;
@@ -1002,24 +1419,6 @@ int list_xpxgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
 
 #define DUMDEBUG 0
 
-static int dummify_candidate (int v, double **Z, DATAINFO *pdinfo)
-{
-    if (var_is_discrete(pdinfo, v)) {
-	/* pre-approved */
-	return 1;
-    }
-
-    if (v == 0 || var_is_scalar(pdinfo, v)) {
-	return 0;
-    }  
-
-    if (gretl_isdiscrete(0, pdinfo->n - 1, Z[v])) {
-	return 1;
-    }
-
-    return 0;
-}
-
 /**
  * list_dumgenr:
  * @plist: pointer to list of variables to process; on exit
@@ -1044,25 +1443,32 @@ static int dummify_candidate (int v, double **Z, DATAINFO *pdinfo)
 int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
 		  gretlopt opt)
 {
+    int origv = pdinfo->v;
     int *list = *plist;
     int *tmplist = NULL;
     double *x = NULL;
     int i, j, t, n;
     int startlen;
-    int err = 0;
+    int err;
+
+    err = transform_preprocess_list(list, *pZ, pdinfo, DUMMIFY);
+    if (err) {
+	return err;
+    }
 
     tmplist = gretl_null_list();
     if (tmplist == NULL) {
-	return E_ALLOC;
+	err = E_ALLOC;
+	goto bailout;
     }
     
     x = malloc(pdinfo->n * sizeof *x);
     if (x == NULL) {
-	free(tmplist);
-	return E_ALLOC;
+	err = E_ALLOC;
+	goto bailout;
     }
 
-    startlen = get_starting_length(list, pdinfo, 3); /* ?? */
+     startlen = get_starting_length(list, pdinfo, 3);
 
     for (i=1; i<=list[0] && !err; i++) {
 	int vi = list[i];
@@ -1070,10 +1476,6 @@ int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
 	int jmin, jmax;
 	double xt;
 
-	if (!dummify_candidate(vi, *pZ, pdinfo)) {
-	    continue;
-	}
-
 	n = 0;
 	for (t=0; t<pdinfo->n; t++) {
 	    xt = (*pZ)[vi][t];
@@ -1106,7 +1508,8 @@ int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
 #endif
 
 	for (j=jmin; j<jmax && !err; j++) {
-	    tnum = get_transform(DUMMIFY, vi, j+1, x[j], pZ, pdinfo, startlen);
+	    tnum = get_transform(DUMMIFY, vi, j+1, x[j], pZ, pdinfo, 
+				 startlen, origv);
 #if DUMDEBUG   
 	    fprintf(stderr, "VALUE = %g, tnum = %d\n", x[j], tnum);
 #endif
@@ -1128,6 +1531,8 @@ int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
 
     free(x);
 
+ bailout:
+
     if (!err) {
 	free(*plist);
 	*plist = tmplist;
@@ -1138,6 +1543,8 @@ int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
 	free(tmplist);
     }
 
+    destroy_mangled_names();
+
     return err;
 }
 
@@ -1200,3 +1607,5 @@ int gettrend (double ***pZ, DATAINFO *pdinfo, int square)
 	    
     return idx;
 }
+
+
