@@ -387,7 +387,8 @@ tsls_make_hatlist (const int *reglist, int *instlist, int *hatlist)
 	}
 	if (endog) {
 	    if (reglist[i] == 0) {
-		/* found const in reglist but not instlist: needs fixing */
+		/* found const in reglist but not instlist: 
+		   needs fixing -- or is this debatable? */
 		addconst = 1;
 	    } else {
 		hatlist[++k] = reglist[i];
@@ -463,10 +464,10 @@ tsls_sargan_test (MODEL *tsls_model, int Orank, int *instlist,
 	double OTest = smod.rsq * smod.nobs;
 
 	if (test != NULL) {
-	    model_test_set_teststat(test, GRETL_STAT_TR2);
+	    model_test_set_teststat(test, GRETL_STAT_LM);
 	    model_test_set_dfn(test, Orank);
 	    model_test_set_value(test, OTest);
-	    model_test_set_pvalue(test, chisq_cdf_comp(OTest, Orank));
+	    model_test_set_pvalue(test, chisq_cdf_comp(Orank, OTest));
 	    maybe_add_test_to_model(tsls_model, test);
 	}	
     }
@@ -528,7 +529,7 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 	    model_test_set_teststat(test, GRETL_STAT_WALD_CHISQ);
 	    model_test_set_dfn(test, df);
 	    model_test_set_value(test, HTest);
-	    model_test_set_pvalue(test, chisq_cdf_comp(HTest, df));
+	    model_test_set_pvalue(test, chisq_cdf_comp(df, HTest));
 	    maybe_add_test_to_model(tmod, test);
 	}
     }
@@ -1022,7 +1023,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     ninst = list[0] - pos;
 
     if (rlen < 2 || ninst <= 0) {
-	tsls.errcode = E_DATA;
+	tsls.errcode = E_ARGS;
 	return tsls;
     }
 
