@@ -24,6 +24,7 @@
 #include "libset.h"
 #include "usermat.h"
 #include "gretl_xml.h"
+#include "genparse.h"
 
 #define MDEBUG 0
 #define LEVEL_AUTO -1
@@ -32,6 +33,7 @@ struct user_matrix_ {
     gretl_matrix *M;
     int level;
     char name[VNAMELEN];
+    char **colnames;
 };
 
 static user_matrix **matrices;
@@ -83,6 +85,7 @@ static user_matrix *user_matrix_new (gretl_matrix *M, const char *name)
     u->level = gretl_function_depth();
     *u->name = '\0';
     strncat(u->name, name, VNAMELEN - 1);
+    u->colnames = NULL;
 
     return u;
 }
@@ -119,6 +122,10 @@ static int real_user_matrix_add (gretl_matrix *M, const char *name,
 	return 0;
     }
 
+    if (gretl_matrix_xna_check(M)) {
+	return E_NAN;
+    }
+
 #if 0
     if (check_varname(name)) {
 	return E_DATA;
@@ -249,11 +256,19 @@ int user_matrix_replace_matrix (user_matrix *u, gretl_matrix *M)
 	return E_UNKVAR;
     }
 
+    if (gretl_matrix_xna_check(M)) {
+	return E_NAN;
+    }
+
     if (M != u->M) {
 #if MDEBUG
 	fprintf(stderr, " freeing u->M at %p, replacing with "
 		"matrix at %p\n", u->M, M);
 #endif
+	if (u->colnames != NULL && M->cols != u->M->cols) {
+	    free_strings_array(u->colnames, u->M->cols);
+	    u->colnames = NULL;
+	}
 	gretl_matrix_free(u->M);
 	u->M = M;
     }
@@ -539,7 +554,7 @@ static int *mspec_to_list (int type, union msel *sel, int n,
 
     for (i=1; i<=slice[0] && !*err; i++) {
 	if (slice[i] < 1 || slice[i] > n) {
-	    fprintf(stderr, "index value %d is out of bounds\n", 
+	    sprintf(gretl_errmsg, _("Index value %d is out of bounds"), 
 		    slice[i]);
 	    *err = 1;
 	}
@@ -784,8 +799,13 @@ static void destroy_user_matrix (user_matrix *u)
     fprintf(stderr, "destroy_user_matrix: freeing matrix at %p...", 
 	    (void *) u->M);
 #endif
+
+    if (u->colnames != NULL) {
+	free_strings_array(u->colnames, u->M->cols);
+    }
     gretl_matrix_free(u->M);
     free(u);
+
 #if MDEBUG
     fprintf(stderr, " done\n");
 #endif
@@ -956,7 +976,7 @@ int user_matrix_destroy_by_name (const char *name, PRN *prn)
 
     err = user_matrix_destroy(u);
 
-    if (!err && prn != NULL) {
+    if (!err && prn != NULL && gretl_messages_on()) {
 	pprintf(prn, _("Deleted matrix %s"), name);
 	pputc(prn, '\n');
     }
@@ -964,20 +984,119 @@ int user_matrix_destroy_by_name (const char *name, PRN *prn)
     return err;
 }
 
-static double 
-real_user_matrix_get_determinant (const gretl_matrix *m, int log, int *err)
+int umatrix_set_colnames_from_string (const gretl_matrix *M, 
+				      const char *s)
 {
+    user_matrix *u = get_user_matrix_by_data(M);
+    int n, err = 0;
+
+    if (u == NULL) {
+	return E_UNKVAR;
+    }
+
+    n = M->cols;
+
+    if (s == NULL || *s == '\0') {
+	if (u->colnames != NULL) {
+	    free_strings_array(u->colnames, n);
+	    u->colnames = NULL;
+	}
+    } else {
+	char **S;
+	int ns;
+
+	S = gretl_string_split(s, &ns);
+	if (S == NULL) {
+	    err = E_ALLOC;
+	} else if (ns != n) {
+	    err = E_NONCONF;
+	    free_strings_array(S, ns);
+	} else {
+	    if (u->colnames != NULL) {
+		free_strings_array(u->colnames, n);
+	    }
+	    u->colnames = S;
+	}
+    }
+
+    return err;
+}
+
+int umatrix_set_colnames_from_list (const gretl_matrix *M, 
+				    const int *list,
+				    const DATAINFO *pdinfo)
+{
+    user_matrix *u = get_user_matrix_by_data(M);
+    int i, n, err = 0;
+
+    if (u == NULL) {
+	return E_UNKVAR;
+    }
+
+    n = M->cols;
+
+    if (list == NULL || list[0] == 0) {
+	if (u->colnames != NULL) {
+	    free_strings_array(u->colnames, n);
+	    u->colnames = NULL;
+	}
+    } else if (list[0] != n) {
+	err = E_NONCONF;
+    } else {
+	char **S = strings_array_new(n);
+
+	if (S == NULL) {
+	    err = E_ALLOC;
+	}
+
+	for (i=0; i<n && !err; i++) {
+	    S[i] = gretl_strndup(pdinfo->varname[list[i+1]], 12);
+	    if (S[i] == NULL) {
+		err = E_ALLOC;
+	    }
+	}
+
+	if (err) {
+	    free_strings_array(S, n);
+	} else {
+	    if (u->colnames != NULL) {
+		free_strings_array(u->colnames, n);
+	    }
+	    u->colnames = S;
+	}
+    }
+
+    return err;
+}
+
+const char **user_matrix_get_column_names (const gretl_matrix *M)
+{
+    user_matrix *u = get_user_matrix_by_data(M);
+
+    return (u != NULL)? (const char **) u->colnames : NULL;
+}
+
+double 
+user_matrix_get_determinant (gretl_matrix *m, int f, int *err)
+{
+    gretl_matrix *tmp = NULL;
     double d = NADBL;
 
-    if (!gretl_is_null_matrix(m)) {
-	gretl_matrix *tmp = gretl_matrix_copy(m);
+    if (gretl_is_null_matrix(m)) {
+	return d;
+    } else if (!matrix_is_user_matrix(m)) {
+	tmp = m;
+    } else {
+	tmp = gretl_matrix_copy(m);
+    }
 
-	if (tmp != NULL) {
-	    if (log) {
-		d = gretl_matrix_log_determinant(tmp, err);
-	    } else {
-		d = gretl_matrix_determinant(tmp, err);
-	    }
+    if (tmp != NULL) {
+	if (f == F_LDET) {
+	    d = gretl_matrix_log_determinant(tmp, err);
+	} else {
+	    d = gretl_matrix_determinant(tmp, err);
+	}
+	if (tmp != m) {
 	    gretl_matrix_free(tmp);
 	}
     }
@@ -985,53 +1104,49 @@ real_user_matrix_get_determinant (const gretl_matrix *m, int log, int *err)
     return d;
 }
 
-double user_matrix_get_determinant (const gretl_matrix *m, int *err)
-{
-    return real_user_matrix_get_determinant(m, 0, err);
-}
-
-double user_matrix_get_log_determinant (const gretl_matrix *m, int *err)
-{
-    return real_user_matrix_get_determinant(m, 1, err);
-}
-
-static gretl_matrix *
-real_user_matrix_get_inverse (const gretl_matrix *m, int moore, int *err)
+gretl_matrix *user_matrix_matrix_func (gretl_matrix *m, int f, 
+				       int *err)
 {
     gretl_matrix *R = NULL;
 
     if (gretl_is_null_matrix(m)) {
 	*err = E_DATA;
+    } else if (!matrix_is_user_matrix(m)) {
+	R = m;
     } else {
 	R = gretl_matrix_copy(m);
 	if (R == NULL) {
 	    *err = E_ALLOC;
+	}
+    }
+
+    if (R != NULL) {
+	if (f == F_CDEMEAN) {
+	    gretl_matrix_demean_by_column(R);
+	} else if (f == F_CHOL) {
+	    *err = gretl_matrix_cholesky_decomp(R);
+	} else if (f == F_INVPD) {
+	    *err = gretl_invpd(R);
+	} else if (f == F_GINV) {
+	    *err = gretl_matrix_moore_penrose(R);
+	} else if (f == F_INV) {
+	    *err = gretl_invert_matrix(R);
+	} else if (f == F_UPPER) {
+	    *err = gretl_matrix_zero_upper(R);
+	} else if (f == F_LOWER) {
+	    *err = gretl_matrix_zero_lower(R);
 	} else {
-	    if (moore) {
-		*err = gretl_matrix_moore_penrose(R);
-	    } else {
-		*err = gretl_invert_matrix(R);
-	    } 
-	    if (*err) {
-		gretl_matrix_free(R);
-		R = NULL;
-	    }
-	} 
+	    *err = E_DATA;
+	}
+	if (*err && R != m) {
+	    gretl_matrix_free(R);
+	    R = NULL;
+	}
     } 
    
     return R;
 }
 
-gretl_matrix *user_matrix_get_inverse (const gretl_matrix *m, int *err)
-{
-    return real_user_matrix_get_inverse(m, 0, err);
-}
-
-gretl_matrix *user_matrix_moore_penrose (const gretl_matrix *m, int *err)
-{
-    return real_user_matrix_get_inverse(m, 1, err);
-}
-
 static void matrix_cannibalize (gretl_matrix *targ, gretl_matrix *src)
 {
     targ->rows = src->rows;
@@ -1121,47 +1236,6 @@ int matrix_XTX_in_place (gretl_matrix *m)
     return err;
 }
 
-gretl_matrix *user_matrix_cholesky_decomp (const gretl_matrix *m,
-					   int *err)
-{
-    gretl_matrix *R = NULL;
-
-    if (gretl_is_null_matrix(m)) {
-	*err = E_DATA;
-    } else {
-	R = gretl_matrix_copy(m);
-	if (R == NULL) {
-	    *err = E_ALLOC;
-	} else {
-	    *err = gretl_matrix_cholesky_decomp(R);
-	    if (*err) {
-		gretl_matrix_free(R);
-		R = NULL;
-	    }
-	} 
-    }
-
-    return R;
-}
-
-gretl_matrix *user_matrix_column_demean (const gretl_matrix *m, int *err)
-{
-    gretl_matrix *R = NULL;
-
-    if (gretl_is_null_matrix(m)) {
-	*err = E_DATA;
-    } else {
-	R = gretl_matrix_copy(m);
-	if (R == NULL) {
-	    *err = E_ALLOC;
-	} else {
-	    gretl_matrix_demean_by_column(R);
-	} 
-    }
-
-    return R;
-}
-
 gretl_matrix *user_matrix_vec (const gretl_matrix *m, int *err)
 {
     gretl_matrix *R = NULL;
@@ -1231,22 +1305,6 @@ gretl_matrix *user_matrix_unvech (const gretl_matrix *m, int *err)
     return R;
 }
 
-gretl_matrix *user_matrix_upper (const gretl_matrix *m, int *err)
-{
-    gretl_matrix *R = gretl_matrix_copy(m);
-
-    *err = gretl_matrix_zero_lower(R);
-    return R;
-}
-
-gretl_matrix *user_matrix_lower (const gretl_matrix *m, int *err)
-{
-    gretl_matrix *R = gretl_matrix_copy(m);
-
-    *err = gretl_matrix_zero_upper(R);
-    return R;
-}
-
 static int 
 real_user_matrix_QR_decomp (const gretl_matrix *m, gretl_matrix **Q, 
 			    gretl_matrix **R)
@@ -1471,6 +1529,24 @@ gretl_matrix *user_matrix_ols (const gretl_matrix *Y,
     return B;
 }
 
+static void maybe_eigen_trim (gretl_matrix *E)
+{
+    double x;
+    int i, allreal = 1;
+
+    for (i=0; i<E->rows; i++) {
+	x = gretl_matrix_get(E, i, 1);
+	if (x != 0.0) {
+	    allreal = 0;
+	    break;
+	}
+    }
+
+    if (allreal) {
+	gretl_matrix_reuse(E, -1, 1);
+    }
+}
+
 gretl_matrix *
 user_matrix_eigen_analysis (const gretl_matrix *m, const char *rname, int symm,
 			    int *err)
@@ -1484,6 +1560,11 @@ user_matrix_eigen_analysis (const gretl_matrix *m, const char *rname, int symm,
 	return NULL;
     }
 
+    if (gretl_matrix_xna_check(m)) {
+	*err = E_NAN;
+	return NULL;
+    }
+
     if (rname != NULL && strcmp(rname, "null")) {
 	vecs = 1;
 	if (get_matrix_by_name(rname) == NULL) {
@@ -1502,6 +1583,9 @@ user_matrix_eigen_analysis (const gretl_matrix *m, const char *rname, int symm,
 	    E = gretl_symmetric_matrix_eigenvals(C, vecs, err);
 	} else {
 	    E = gretl_general_matrix_eigenvals(C, vecs, err);
+	    if (E != NULL && E->cols == 2) {
+		maybe_eigen_trim(E);
+	    }
 	}
     }
 
@@ -1516,6 +1600,40 @@ user_matrix_eigen_analysis (const gretl_matrix *m, const char *rname, int symm,
     return E;
 }
 
+static void xml_put_user_matrix (user_matrix *u, FILE *fp)
+{
+    gretl_matrix *M;
+    int i, j;
+
+    if (u == NULL || u->M == NULL) {
+	return;
+    }
+
+    M = u->M;
+
+    fprintf(fp, "<gretl-matrix name=\"%s\" rows=\"%d\" cols=\"%d\"", 
+	    u->name, M->rows, M->cols);
+
+    if (u->colnames != NULL) {
+	fputs(" colnames=\"", fp);
+	for (j=0; j<M->cols; j++) {
+	    fputs(u->colnames[j], fp);
+	    fputc((j < M->cols - 1)? ' ' : '"', fp);
+	}
+    } 
+
+    fputs(">\n", fp);
+
+    for (i=0; i<M->rows; i++) {
+	for (j=0; j<M->cols; j++) {
+	    fprintf(fp, "%.15g ", gretl_matrix_get(M, i, j));
+	}
+	fputc('\n', fp);
+    }
+
+    fputs("</gretl-matrix>\n", fp); 
+}
+
 void write_matrices_to_file (FILE *fp)
 {
     int i;
@@ -1527,7 +1645,7 @@ void write_matrices_to_file (FILE *fp)
 
     for (i=0; i<n_matrices; i++) {
 	if (matrices[i]->M != NULL) {
-	    gretl_xml_put_matrix(matrices[i]->M, matrices[i]->name, fp);
+	    xml_put_user_matrix(matrices[i], fp);
 	}
     }
 
