@@ -89,23 +89,24 @@ int copy_matrix_as (const gretl_matrix *m, const char *new);
 int user_matrix_set_name_and_level (const gretl_matrix *M, char *name, 
 				    int level);
 
+int umatrix_set_colnames_from_string (const gretl_matrix *M, 
+				      const char *s);
+
+int umatrix_set_colnames_from_list (const gretl_matrix *M, 
+				    const int *list,
+				    const DATAINFO *pdinfo);
+
+const char **user_matrix_get_column_names (const gretl_matrix *M);
+
 void destroy_user_matrices (void);
 
 int destroy_user_matrices_at_level (int level);
 
 int destroy_private_matrices (void);
 
-double user_matrix_get_determinant (const gretl_matrix *m, int *err);
+double user_matrix_get_determinant (gretl_matrix *m, int f, int *err);
 
-double user_matrix_get_log_determinant (const gretl_matrix *m, int *err);
-
-gretl_matrix *user_matrix_get_inverse (const gretl_matrix *m, int *err);
-
-gretl_matrix *user_matrix_moore_penrose (const gretl_matrix *m, int *err);
-
-gretl_matrix *user_matrix_cholesky_decomp (const gretl_matrix *m, int *err);
-
-gretl_matrix *user_matrix_column_demean (const gretl_matrix *m, int *err);
+gretl_matrix *user_matrix_matrix_func (gretl_matrix *m, int f, int *err);
 
 gretl_matrix *user_matrix_vec (const gretl_matrix *m, int *err);
 
@@ -113,10 +114,6 @@ gretl_matrix *user_matrix_vech (const gretl_matrix *m, int *err);
 
 gretl_matrix *user_matrix_unvech (const gretl_matrix *m, int *err);
 
-gretl_matrix *user_matrix_upper (const gretl_matrix *m, int *err);
-
-gretl_matrix *user_matrix_lower (const gretl_matrix *m, int *err);
-
 gretl_matrix *
 user_matrix_QR_decomp (const gretl_matrix *m, const char *rname, 
 		       int *err);
