@@ -21,6 +21,7 @@
 
 #include "libgretl.h" 
 #include "var.h"  
+#include "johansen.h"
 #include "varprint.h"
 #include "vartest.h"
 #include "libset.h"
@@ -39,6 +40,9 @@ enum {
 static JohansenInfo *
 johansen_info_new (GRETL_VAR *var, int rank, gretlopt opt);
 
+static gretl_matrix *
+gretl_VAR_get_fcast_se (GRETL_VAR *var, int periods);
+
 static int VAR_add_models (GRETL_VAR *var, const DATAINFO *pdinfo)
 {
     int n = var->neqns;
@@ -127,6 +131,24 @@ static int VAR_add_residuals_matrix (GRETL_VAR *var)
     return err;
 }
 
+static int VAR_add_vcv_matrix (GRETL_VAR *var)
+{
+    int k, err = 0;
+
+    if (var->vcv != NULL) {
+	return 0;
+    }      
+
+    k = var->ncoeff * var->neqns;
+
+    var->vcv = gretl_matrix_alloc(k, k);
+    if (var->vcv == NULL) {
+	err = E_ALLOC;
+    } 
+
+    return err;
+}
+
 int nrestr (const GRETL_VAR *v) 
 {
     int n = 0;
@@ -169,6 +191,7 @@ void gretl_VAR_clear (GRETL_VAR *var)
     var->E = NULL;
     var->C = NULL;
     var->S = NULL;
+    var->vcv = NULL;
     var->F = NULL;
 
     var->models = NULL;
@@ -335,6 +358,11 @@ static int VAR_make_lists (GRETL_VAR *v, const int *list,
 	}
     }
 
+    if (!err && (v->ylist == NULL || v->ylist[0] < 2)) {
+	/* first test for at least 2 endog vars */
+	err = E_ARGS;
+    }
+
     if (!err && v->ci == VECM) {
 	if (v->xlist != NULL && gretl_list_has_separator(v->xlist)) {
 	    int *tmp = NULL;
@@ -357,6 +385,11 @@ static int VAR_make_lists (GRETL_VAR *v, const int *list,
 	}	
     }
 
+    if (!err && (v->ylist == NULL || v->ylist[0] < 2)) {
+	/* re-test after (possibly) losing const */
+	err = E_ARGS;
+    }
+
 #if VDEBUG
     printlist(v->ylist, "v->ylist");
     printlist(v->xlist, "v->xlist");
@@ -569,7 +602,7 @@ static GRETL_VAR *gretl_VAR_new (int code, int order, int rank,
     ci = (code >= VECM_ESTIMATE)? VECM : VAR;
 
     if ((ci == VAR && order < 1) || (ci == VECM && order < 0)) {
-	sprintf(gretl_errmsg, "VAR: invalid lag order %d", order);
+	sprintf(gretl_errmsg, _("Invalid lag order %d"), order);
 	*errp = E_DATA;
 	return NULL;
     }
@@ -592,12 +625,7 @@ static GRETL_VAR *gretl_VAR_new (int code, int order, int rank,
 
     err = VAR_make_lists(var, list, Z, pdinfo);
 
-    if (!err && var->ylist[0] < 2) {
-	strcpy(gretl_errmsg, "VAR: needs at least two variables");
-	err = E_DATA;
-    }
-
-    if (rank > var->ylist[0]) {
+    if (!err && rank > var->ylist[0]) {
 	sprintf(gretl_errmsg, _("vecm: rank %d is out of bounds"), rank);
 	err = E_DATA;
     }
@@ -631,6 +659,10 @@ static GRETL_VAR *gretl_VAR_new (int code, int order, int rank,
 	err = VAR_add_cholesky_matrix(var);
     }
 
+    if (!err && var->ci == VAR) {
+	err = VAR_add_vcv_matrix(var);
+    }
+
     if (!err && code != VAR_LAGSEL) {
 	err = VAR_add_models(var, pdinfo);
     }
@@ -722,6 +754,7 @@ void gretl_VAR_free (GRETL_VAR *var)
     gretl_matrix_free(var->X);
     gretl_matrix_free(var->B);
     gretl_matrix_free(var->XTX);
+    gretl_matrix_free(var->vcv);
 
     gretl_matrix_free(var->A);
     gretl_matrix_free(var->L);
@@ -751,32 +784,56 @@ void gretl_VAR_free (GRETL_VAR *var)
 }
 
 static int VAR_add_fcast_variance (GRETL_VAR *var, gretl_matrix *F,
-				   int nf, int pre_obs)
+				   int n_static)
 {
-    double vti;
-    int i, s;
+    gretl_matrix *se = NULL;
+    double ftj, vti;
+    int k, n = var->neqns;
+    int i, j, s;
     int err = 0;
 
-    for (i=0; i<var->neqns; i++) {
-	gretl_matrix *vd;
-	int totcol;
+    if (n_static < 0) {
+	fprintf(stderr, "n_static = %d\n", n_static);
+	n_static = 0;
+    }
 
-	vd = gretl_VAR_get_fcast_decomp(var, i, nf - pre_obs);
-	if (vd != NULL) {
-	    totcol = gretl_matrix_cols(vd) - 1;
-	    for (s=0; s<nf; s++) {
-		if (s < pre_obs) {
-		    gretl_matrix_set(F, s, var->neqns + i, NADBL);
-		} else {
-		    vti = gretl_matrix_get(vd, s - pre_obs, totcol);
-		    gretl_matrix_set(F, s, var->neqns + i, vti);
-		}
-	    }
-	    gretl_matrix_free(vd);
-	} else {
+    k = F->rows - n_static;
+
+    if (k > 0) {
+	se = gretl_VAR_get_fcast_se(var, k);
+	if (se == NULL) {
 	    err = E_ALLOC;
-	    for (s=0; s<nf; s++) {
-		gretl_matrix_set(F, s, var->neqns + i, NADBL);
+	    goto bailout;
+	}
+    }
+
+    for (j=0; j<n; j++) {
+	for (s=0; s<F->rows; s++) {
+	    ftj = gretl_matrix_get(F, s, j);
+	    if (na(ftj)) {
+		gretl_matrix_set(F, s, n + j, NADBL);
+	    } else {
+		i = s - n_static;
+		if (i < 0) {
+		    vti = sqrt(gretl_matrix_get(var->S, j, j));
+		} else {
+		    vti = gretl_matrix_get(se, i, j);
+		}
+		gretl_matrix_set(F, s, n + j, vti);
+	    }
+	}
+    }
+
+    if (se != NULL) {
+	gretl_matrix_free(se);
+    }
+
+ bailout:
+
+    if (err) {
+	for (i=0; i<n; i++) {
+	    for (s=0; s<F->rows; s++) {
+		gretl_matrix_set(F, s, n + i, NADBL);
 	    }
 	}
     }
@@ -784,37 +841,62 @@ static int VAR_add_fcast_variance (GRETL_VAR *var, gretl_matrix *F,
     return err;
 }
 
+/* determine start of dynamic portion of forecast */
+
+static int VAR_get_tdyn (GRETL_VAR *var, int t1, int t2, gretlopt opt)
+{
+    int td;
+
+    if (opt & OPT_D) {
+	/* force dynamic */
+	td = t1;
+    } else if (opt & OPT_S) {
+	/* force static */
+	td = t2 + 1;
+    } else {
+	/* out of sample */
+	td = var->t2 + 1;
+    }
+
+    return td;
+}
+
 static int
-VAR_add_forecast (GRETL_VAR *var, int t0, int t1, int t2,
+VAR_add_forecast (GRETL_VAR *var, int t1, int t2,
 		  const double **Z, const DATAINFO *pdinfo, 
 		  gretlopt opt)
 {
     const MODEL *pmod;
-    gretl_matrix *F;
-    double fti, xti;
-    int nf = t2 - t0 + 1;
-    int staticfc = (opt & OPT_S);
+    double fti, xti, xtid;
+    int tdyn, nf = t2 - t1 + 1;
     int i, j, k, s, t;
     int lag, vj, m = 0;
     int fcols;
 
-    fcols = (staticfc)? var->neqns : 2 * var->neqns;
+    fcols = 2 * var->neqns;
 
     /* rows = number of forecast periods; cols = 1 to hold forecast
        for each variable, plus 1 to hold variance for each variable
        if forecast is dynamic.
     */
 
-    F = gretl_zero_matrix_new(nf, fcols);
-    if (F == NULL) {
+    var->F = gretl_zero_matrix_new(nf, fcols);
+    if (var->F == NULL) {
 	return E_ALLOC;
     }
 
     if (var->detflags & DET_SEAS) {
-	m = get_subperiod(t0, pdinfo, NULL);
+	m = get_subperiod(t1, pdinfo, NULL);
     }
 
-    for (t=t0, s=0; t<=t2; t++, s++) {
+    /* start of dynamic portion of forecast? */
+    tdyn = VAR_get_tdyn(var, t1, t2, opt);
+
+#if 0
+    fprintf(stderr, "var fcast: t1=%d, tdyn=%d, t2=%d\n", t1, tdyn, t2);
+#endif
+
+    for (t=t1, s=0; t<=t2; t++, s++) {
 	int miss = 0;
 
 	for (i=0; i<var->neqns; i++) {
@@ -831,7 +913,8 @@ VAR_add_forecast (GRETL_VAR *var, int t0, int t1, int t2,
 	    for (j=0; j<var->neqns; j++) {
 		vj = var->ylist[j+1];
 		for (lag=1; lag<=var->order; lag++) {
-		    if (t < t1 || staticfc || s - lag < 0) {
+		    xtid = NADBL;
+		    if (t < tdyn || s - lag < 0) {
 			/* pre-forecast value */
 			if (t - lag < 0) {
 			    xti = NADBL;
@@ -839,14 +922,19 @@ VAR_add_forecast (GRETL_VAR *var, int t0, int t1, int t2,
 			    xti = Z[vj][t-lag];
 			}
 		    } else {
-			/* prior forecast value */
-			xti = gretl_matrix_get(F, s - lag, j); /* ?? */
+			/* prior forecast value preferred */
+			if (t - lag >= 0) {
+			    xtid = Z[vj][t-lag];
+			}
+			xti = gretl_matrix_get(var->F, s-lag, j);
 		    }
-		    if (na(xti)) {
-			miss = 1;
-		    } else {
+		    if (!na(xti)) {
 			fti += pmod->coeff[k] * xti;
-		    }
+		    } else if (!na(xtid)) {
+			fti += pmod->coeff[k] * xtid;
+		    } else {
+			miss = 1;
+		    } 
 		    k++;
 		}
 	    }
@@ -882,7 +970,7 @@ VAR_add_forecast (GRETL_VAR *var, int t0, int t1, int t2,
 		fti = NADBL;
 	    }
 
-	    gretl_matrix_set(F, s, i, fti);
+	    gretl_matrix_set(var->F, s, i, fti);
 	}
 
 	if (var->detflags & DET_SEAS) {
@@ -890,64 +978,60 @@ VAR_add_forecast (GRETL_VAR *var, int t0, int t1, int t2,
 	}
     }
 
-    /* now get variances, if not static */
-    if (!staticfc) {
-	VAR_add_fcast_variance(var, F, nf, t1 - t0);
-    }
+    VAR_add_fcast_variance(var, var->F, tdyn - t1);
 
-    gretl_matrix_set_t1(F, t0);
-    gretl_matrix_set_t2(F, t2);
-
-    var->F = F;
+    gretl_matrix_set_t1(var->F, t1);
+    gretl_matrix_set_t2(var->F, t2);
 
 #if VDEBUG
-    gretl_matrix_print(F, "var->F");
+    gretl_matrix_print(var->F, "var->F");
 #endif
 
     return 0;
 }
 
 static int
-VECM_add_forecast (GRETL_VAR *var, int t0, int t1, int t2,
-		   const double **Z, DATAINFO *pdinfo, 
+VECM_add_forecast (GRETL_VAR *var, int t1, int t2,
+		   const double **Z, const DATAINFO *pdinfo, 
 		   gretlopt opt)
 {
-    gretl_matrix *F = NULL;
     gretl_matrix *B = NULL;
-
     double s0 = 0, s1 = 1;
     int order = effective_order(var);
     int nexo = (var->xlist != NULL)? var->xlist[0] : 0;
     int nseas = var->jinfo->seasonals;
-    int nf = t2 - t0 + 1;
-    int staticfc = (opt & OPT_S);
+    int tdyn, nf = t2 - t1 + 1;
     int i, j, k, vj, s, t;
     int fcols, m = 0;
 
-    fcols = (staticfc)? var->neqns : 2 * var->neqns;
+    fcols = 2 * var->neqns;
 
-    F = gretl_zero_matrix_new(nf, fcols);
-    if (F == NULL) {
+    var->F = gretl_zero_matrix_new(nf, fcols);
+    if (var->F == NULL) {
 	return E_ALLOC;
     }
 
     B = VAR_coeff_matrix_from_VECM(var);
     if (B == NULL) {
-	gretl_matrix_free(F);
+	gretl_matrix_free(var->F);
+	var->F = NULL;
 	return E_ALLOC;
     }
 
+    /* start of dynamic portion of forecast? */
+    tdyn = VAR_get_tdyn(var, t1, t2, opt);
+
     if (nseas > 0) {
-	m = get_subperiod(t0, pdinfo, NULL);
+	m = get_subperiod(t1, pdinfo, NULL);
 	s1 -= 1.0 / pdinfo->pd;
 	s0 = s1 - 1;
     }
 
-    for (t=t0, s=0; t<=t2; t++, s++) {
+    for (t=t1, s=0; t<=t2; t++, s++) {
 
 	for (i=0; i<var->neqns; i++) {
-	    double bij, xtj, fti = 0.0;
-	    int ft, col = 0;
+	    double bij, xtj, xtjd, fti = 0.0;
+	    int col = 0;
 
 	    /* unrestricted constant, if present */
 	    if (var->ifc) {
@@ -964,19 +1048,23 @@ VECM_add_forecast (GRETL_VAR *var, int t0, int t1, int t2,
 			break;
 		    }			
 		    bij = gretl_matrix_get(B, i, col++);
-		    ft = s - k;
-		    if (t >= t1 && ft >= 0 && !staticfc) {
-			/* use prior forecast if available */
-			xtj = gretl_matrix_get(F, ft, j);
-		    } else {
+		    xtjd = NADBL;
+		    if (t < tdyn || s - k < 0) {
+			/* pre-forecast value */
 			xtj = Z[vj][t-k];
+		    } else {
+			/* prior forecast value preferred */
+			xtjd = Z[vj][t-k];
+			xtj = gretl_matrix_get(var->F, s-k, j);
 		    }
-		    if (na(xtj)) {
+		    if (!na(xtj)) {
+			fti += bij * xtj;
+		    } else if (!na(xtjd)) {
+			fti += bij * xtjd;
+		    } else {
 			fti = NADBL;
 			break;
-		    } else {
-			fti += bij * xtj;
-		    }
+		    } 
 		}
 		if (na(fti)) {
 		    break;
@@ -1035,7 +1123,7 @@ VECM_add_forecast (GRETL_VAR *var, int t0, int t1, int t2,
 	    
 	set_fcast:
 
-	    gretl_matrix_set(F, s, i, fti);
+	    gretl_matrix_set(var->F, s, i, fti);
 	}
 
 	if (nseas > 0) {
@@ -1045,49 +1133,33 @@ VECM_add_forecast (GRETL_VAR *var, int t0, int t1, int t2,
 
     gretl_matrix_free(B);
 
-    /* now get variances, if not static */
-    if (!staticfc) {
-	VAR_add_fcast_variance(var, F, nf, t1 - t0);
-    }
+    VAR_add_fcast_variance(var, var->F, tdyn - t1);
 
-    gretl_matrix_set_t1(F, t0);
-    gretl_matrix_set_t2(F, t2);
-
-    var->F = F;
+    gretl_matrix_set_t1(var->F, t1);
+    gretl_matrix_set_t2(var->F, t2);
 
 #if VDEBUG
-    gretl_matrix_print(F, "var->F");
+    gretl_matrix_print(var->F, "var->F");
 #endif
 
     return 0;
 }
 
 const gretl_matrix *
-gretl_VAR_get_forecast_matrix (GRETL_VAR *var, int t0, int t1, int t2,
+gretl_VAR_get_forecast_matrix (GRETL_VAR *var, int t1, int t2,
 			       const double **Z, DATAINFO *pdinfo, 
-			       gretlopt opt)
+			       gretlopt opt, int *err)
 {
     if (var->F != NULL) {
-	int ncols, nf = t2 - t0 + 1;
-	int ft1 = gretl_matrix_get_t1(var->F);
-
-	ncols = (opt & OPT_S)? var->neqns: 2 * var->neqns;
-
-	if (nf == gretl_matrix_rows(var->F) && t1 == ft1 && 
-	    ncols == gretl_matrix_cols(var->F)) {
-	    ; /* already done, fine */
-	} else {
-	    gretl_matrix_free(var->F);
-	    var->F = NULL;
-	}
+	/* There's a forecast attached, but it may not be what we want */
+	gretl_matrix_free(var->F);
+	var->F = NULL;
     }
 
-    if (var->F == NULL) {
-	if (var->ci == VECM) {
-	    VECM_add_forecast(var, t0, t1, t2, Z, pdinfo, opt);
-	} else {
-	    VAR_add_forecast(var, t0, t1, t2, Z, pdinfo, opt);
-	}
+    if (var->ci == VECM) {
+	*err = VECM_add_forecast(var, t1, t2, Z, pdinfo, opt);
+    } else {
+	*err = VAR_add_forecast(var, t1, t2, Z, pdinfo, opt);
     }
 
     return var->F;
@@ -1209,6 +1281,11 @@ int gretl_VAR_do_error_decomp (const gretl_matrix *S,
 const gretl_matrix *
 gretl_VAR_get_residual_matrix (const GRETL_VAR *var)
 {
+    if (var->E != NULL) {
+	var->E->t1 = var->t1;
+	var->E->t2 = var->t2;
+    }
+
     return var->E;
 }
 
@@ -1402,85 +1479,143 @@ gretl_VAR_get_impulse_response (GRETL_VAR *var,
     return ret;
 }
 
-gretl_matrix *
-gretl_VAR_get_fcast_decomp (GRETL_VAR *var, int targ, int periods) 
+static gretl_matrix *
+gretl_VAR_get_fcast_se (GRETL_VAR *var, int periods)
 {
-    int rows = var->neqns * effective_order(var);
-    gretl_matrix *ctmp = NULL, *idx = NULL, *vtmp = NULL;
-    gretl_matrix *cic = NULL, *vt = NULL;
-    gretl_matrix *vd = NULL;
-    int i, t, err = 0;
-
-    if (targ >= var->neqns) {
-	fprintf(stderr, "Target variable out of bounds\n");
-	return NULL;
-    } 
+    int k = var->neqns * effective_order(var);
+    gretl_matrix *vtmp = NULL;
+    gretl_matrix *cc = NULL, *vt = NULL;
+    gretl_matrix *se = NULL;
+    double vti;
+    int i, t;
 
     if (periods <= 0) {
 	fprintf(stderr, "Invalid number of periods\n");
 	return NULL;
     }
 
-    vd = gretl_matrix_alloc(periods, var->neqns + 1);
-    ctmp = gretl_matrix_alloc(var->neqns, rows);
-    idx = gretl_matrix_alloc(var->neqns, var->neqns); 
-    cic = gretl_matrix_alloc(rows, rows);
-    vt = gretl_matrix_alloc(rows, rows);
-    vtmp = gretl_matrix_alloc(rows, rows);
+    se = gretl_zero_matrix_new(periods, var->neqns);
+    vt = gretl_matrix_alloc(k, k);
+    cc = gretl_zero_matrix_new(k, k);
+    vtmp = gretl_matrix_alloc(k, k);
 
-    if (vd == NULL || ctmp == NULL || idx == NULL ||
-	cic == NULL || vt == NULL || vtmp == NULL) {
-	gretl_matrix_free(vd);
-	gretl_matrix_free(ctmp);
-	gretl_matrix_free(idx);
-	gretl_matrix_free(cic);
+    if (se == NULL || cc == NULL || 
+	vt == NULL || vtmp == NULL) {
+	gretl_matrix_free(se);
+	gretl_matrix_free(cc);
 	gretl_matrix_free(vt);
 	gretl_matrix_free(vtmp);
 	return NULL;
     }
 
-    for (i=0; i<var->neqns; i++) {
+    for (t=0; t<periods; t++) {
+	if (t == 0) {
+	    /* initial variance */
+	    gretl_matrix_inscribe_matrix(cc, var->S, 0, 0, GRETL_MOD_NONE);
+	    gretl_matrix_copy_values(vt, cc);
+	} else {
+	    /* calculate further variances */
+	    gretl_matrix_copy_values(vtmp, vt);
+	    gretl_matrix_qform(var->A, GRETL_MOD_NONE,
+			       vtmp, vt, GRETL_MOD_NONE);
+	    gretl_matrix_add_to(vt, cc);
+	}
+
+	for (i=0; i<var->neqns; i++) {
+	    vti = gretl_matrix_get(vt, i, i);
+	    gretl_matrix_set(se, t, i, sqrt(vti));
+	}
+    }
+
+    gretl_matrix_free(cc);
+    gretl_matrix_free(vt);
+    gretl_matrix_free(vtmp);
+
+    return se;
+}
+
+gretl_matrix *
+gretl_VAR_get_fcast_decomp (GRETL_VAR *var, int targ, int periods,
+			    int *errp)
+{
+    int n = var->neqns;
+    int k = n * effective_order(var);
+    gretl_matrix *idx = NULL, *vtmp = NULL;
+    gretl_matrix *cic = NULL, *vt = NULL;
+    gretl_matrix *vd = NULL;
+    int i, t, err = 0;
+
+    *errp = 0;
+
+    if (targ >= n) {
+	fprintf(stderr, "Target variable out of bounds\n");
+	*errp = E_DATA;
+	return NULL;
+    } 
+
+    if (periods <= 0) {
+	fprintf(stderr, "Invalid number of periods\n");
+	*errp = E_DATA;
+	return NULL;
+    }
+
+    vd = gretl_zero_matrix_new(periods, n + 1);
+    idx = gretl_zero_matrix_new(n, n); 
+    cic = gretl_matrix_alloc(k, k);
+    vt = gretl_matrix_alloc(k, k);
+    vtmp = gretl_matrix_alloc(k, k);
+
+    if (vd == NULL || idx == NULL || cic == NULL || 
+	vt == NULL || vtmp == NULL) {
+	gretl_matrix_free(vd);
+	gretl_matrix_free(idx);
+	gretl_matrix_free(cic);
+	gretl_matrix_free(vt);
+	gretl_matrix_free(vtmp);
+	*errp = E_ALLOC;
+	return NULL;
+    }
+
+    for (i=0; i<n && !err; i++) {
 	double vti;
 
-	/* make appropriate index matrix */
-	gretl_matrix_zero(idx);
+	/* adjust index matrix */
+	if (i > 0) {
+	    gretl_matrix_set(idx, i-1, i-1, 0.0);
+	}
 	gretl_matrix_set(idx, i, i, 1.0);
 
 	for (t=0; t<periods && !err; t++) {
 
 	    if (t == 0) {
 		/* calculate initial variances */
-		err = gretl_matrix_multiply_mod(idx, GRETL_MOD_NONE,
-						var->C, GRETL_MOD_TRANSPOSE,
-						ctmp, GRETL_MOD_NONE);
-		err = gretl_matrix_multiply(var->C, ctmp, cic);
+		err = gretl_matrix_qform(var->C, GRETL_MOD_NONE,
+					 idx, cic, GRETL_MOD_NONE);
 		gretl_matrix_copy_values(vt, cic);
 	    } else {
 		/* calculate further variances */
-		err = gretl_matrix_multiply_mod(vt, GRETL_MOD_NONE,
-						var->A, GRETL_MOD_TRANSPOSE,
-						vtmp, GRETL_MOD_NONE);
-		err = gretl_matrix_multiply(var->A, vtmp, vt);
+		gretl_matrix_copy_values(vtmp, vt);
+		err = gretl_matrix_qform(var->A, GRETL_MOD_NONE,
+					 vtmp, vt, GRETL_MOD_NONE);
 		gretl_matrix_add_to(vt, cic);
 	    }
 
-	    if (err) break;
-
-	    vti = gretl_matrix_get(vt, targ, targ);
-	    gretl_matrix_set(vd, t, i, vti);
+	    if (!err) {
+		vti = gretl_matrix_get(vt, targ, targ);
+		gretl_matrix_set(vd, t, i, vti);
+	    }
 	}
     }
 
-    /* normalize variance contributions as percentage shares */
     for (t=0; t<periods && !err; t++) {
-	double vtot = 0.0;
-	double vi;
+	double vi, vtot = 0.0;
 
-	for (i=0; i<var->neqns; i++) {
+	for (i=0; i<n; i++) {
 	    vtot += gretl_matrix_get(vd, t, i);
 	}
 
-	for (i=0; i<var->neqns; i++) {
+	/* normalize variance contributions as % shares */
+	for (i=0; i<n; i++) {
 	    vi = gretl_matrix_get(vd, t, i);
 	    gretl_matrix_set(vd, t, i, 100.0 * vi / vtot);
 	}
@@ -1488,12 +1623,17 @@ gretl_VAR_get_fcast_decomp (GRETL_VAR *var, int targ, int periods)
 	gretl_matrix_set(vd, t, var->neqns, sqrt(vtot));
     }
 
-    gretl_matrix_free(ctmp);
     gretl_matrix_free(idx);
     gretl_matrix_free(cic);
     gretl_matrix_free(vt);
     gretl_matrix_free(vtmp);
 
+    if (err) {
+	*errp = err;
+	gretl_matrix_free(vd);
+	vd = NULL;
+    }
+
     return vd;
 }
 
@@ -1539,7 +1679,8 @@ static int VAR_add_roots (GRETL_VAR *var)
     int err = 0;
 
     if (var->A == NULL) {
-	return 1;
+	fprintf(stderr, "VAR_add_roots: var->A is missing\n");
+	return E_DATA;
     }
 
     var->L = NULL;
@@ -1568,11 +1709,17 @@ static int VAR_add_roots (GRETL_VAR *var)
     return err;
 }
 
-const gretl_matrix *gretl_VAR_get_roots (GRETL_VAR *var)
+const gretl_matrix *gretl_VAR_get_roots (GRETL_VAR *var, int *err)
 {
+    if (var == NULL) {
+	fprintf(stderr, "gretl_VAR_get_roots: VAR is NULL\n");
+	*err = E_DATA;
+	return NULL;
+    } 
+
     if (var->L == NULL) {
 	/* roots not computed yet */
-	VAR_add_roots(var);
+	*err = VAR_add_roots(var);
     }
 
     return var->L;
@@ -1728,6 +1875,13 @@ void VAR_write_A_matrix (GRETL_VAR *v)
     }
 }
 
+static void VAR_write_vcv_matrix (GRETL_VAR *v)
+{
+    if (v->S != NULL && v->XTX != NULL && v->vcv != NULL) {
+	gretl_matrix_kronecker_product(v->S, v->XTX, v->vcv);
+    }
+}
+
 static int VAR_finalize (GRETL_VAR *var)
 {
     int err = 0;
@@ -1739,9 +1893,12 @@ static int VAR_finalize (GRETL_VAR *var)
     if (!err && var->order > 1) {
 	err = last_lag_LR_prep(var, var->ifc);
     }
+
     if (!err) {
 	err = VAR_add_stats(var);
+	VAR_write_vcv_matrix(var);
     }
+
     if (!err) {
 	err = gretl_VAR_do_error_decomp(var->S, var->C);
     }
@@ -2349,19 +2506,18 @@ johansen_wrapper (int code, int order, int rank,
 		  const int *list, 
 		  gretl_restriction *rset, 
 		  const double **Z, const DATAINFO *pdinfo, 
-		  gretlopt opt, PRN *prn)
+		  gretlopt opt, PRN *prn, int *err)
 {
     GRETL_VAR *jvar;
-    int err = 0;
 
     jvar = gretl_VAR_new(code, order - 1, rank, list, Z, pdinfo,
-			 opt, &err);
+			 opt, err);
     if (jvar == NULL) {
 	return NULL;
     }    
 
     if (jvar != NULL && !jvar->err) {
-	jvar->err = johansen_driver(jvar, rset, Z, pdinfo, opt, prn);
+	*err = jvar->err = johansen_driver(jvar, rset, Z, pdinfo, opt, prn);
     }
 
     return jvar;
@@ -2390,8 +2546,10 @@ GRETL_VAR *johansen_test (int order, const int *list,
 			  const double **Z, const DATAINFO *pdinfo,
 			  gretlopt opt, PRN *prn)
 {
+    int err = 0;
+
     return johansen_wrapper(VECM_CTEST, order, 0, list, NULL, 
-			    Z, pdinfo, opt, prn);
+			    Z, pdinfo, opt, prn, &err);
 }
 
 /**
@@ -2418,16 +2576,11 @@ int johansen_test_simple (int order, const int *list,
 			  const double **Z, const DATAINFO *pdinfo, 
 			  gretlopt opt, PRN *prn)
 {
-    GRETL_VAR *jvar;
-    int err;
+    GRETL_VAR *jvar = NULL;
+    int err = 0;
 
     jvar = johansen_wrapper(VECM_CTEST, order, 0, list, NULL, 
-			    Z, pdinfo, opt, prn);
-    if (jvar == NULL) {
-	err = E_ALLOC;
-    } else {
-	err = jvar->err;
-    }
+			    Z, pdinfo, opt, prn, &err);
 
     if (jvar != NULL) {
 	gretl_VAR_free(jvar);
@@ -2464,17 +2617,11 @@ GRETL_VAR *gretl_VECM (int order, int rank, int *list,
     }
 
     jvar = johansen_wrapper(VECM_ESTIMATE, order, rank, list, NULL, 
-			    Z, pdinfo, opt, prn);
+			    Z, pdinfo, opt, prn, err);
     
-    if (jvar != NULL) {
-	if (!jvar->err) {
-	    gretl_VAR_print(jvar, pdinfo, opt, prn);
-	} else {
-	    *err = jvar->err;
-	}
-    } else {
-	*err = 1;
-    }
+    if (jvar != NULL && !jvar->err) {
+	gretl_VAR_print(jvar, pdinfo, opt, prn);
+    } 
 
     return jvar;
 }
@@ -2571,13 +2718,9 @@ real_gretl_restricted_vecm (GRETL_VAR *orig,
     jvar = johansen_wrapper(VECM_ESTIMATE, orig->order + 1, 
 			    orig->jinfo->rank, list,
 			    rset, Z, pdinfo, 
-			    jopt, prn);
+			    jopt, prn, err);
 
-    if (jvar == NULL) {
-	*err = 1;
-    } else if (jvar->err) {
-	*err = jvar->err;
-    } else {
+    if (jvar != NULL && !jvar->err) {
 	gretlopt ropt, prnopt = OPT_NONE;
 	int df;
 
@@ -2585,7 +2728,7 @@ real_gretl_restricted_vecm (GRETL_VAR *orig,
 
 	if (df > 0) {
 	    double x = 2 * (orig->ll - jvar->ll);
-	    double pv = chisq_cdf_comp(x, df);
+	    double pv = chisq_cdf_comp(df, x);
 
 	    rset_add_results(rset, x, pv, jvar->ll);
 	    rset_record_LR_result(rset);
@@ -2930,13 +3073,15 @@ gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx,
     }
 
     if (idx == M_UHAT) {
-	src = var->E;
+	src = gretl_VAR_get_residual_matrix(var);
     } else if (idx == M_COMPAN) {
 	src = var->A;
     } else if (idx == M_COEFF) {
 	M = VAR_get_coeff_matrix(var, err);
 	copy = 0;
     } else if (idx == M_VCV) {
+	src = var->vcv;
+    } else if (idx == M_SIGMA) {
 	src = var->S;
     } else if (vecm_matrix(idx)) {
 	if (var->jinfo != NULL) {
@@ -3234,6 +3379,7 @@ static int rebuild_VAR_matrices (GRETL_VAR *var)
     MODEL *pmod;
     double x;
     int gotA = (var->A != NULL);
+    int gotV = (var->vcv != NULL);
     int j, i;
     int err = 0;
 
@@ -3245,6 +3391,10 @@ static int rebuild_VAR_matrices (GRETL_VAR *var)
 	err = VAR_add_companion_matrix(var);
     }
 
+    if (!err && var->vcv == NULL) {
+	err = VAR_add_vcv_matrix(var);
+    } 
+
     if (!err && var->C == NULL) {
 	err = VAR_add_cholesky_matrix(var);
     } 
@@ -3276,6 +3426,10 @@ static int rebuild_VAR_matrices (GRETL_VAR *var)
 	VAR_write_A_matrix(var);
     }
 
+    if (!err && !gotV) {
+	VAR_write_vcv_matrix(var);
+    }
+
     return err;
 }
 
