@@ -19,6 +19,7 @@
 
 #include "libgretl.h" 
 #include "var.h"  
+#include "johansen.h"
 #include "vartest.h"
 #include "matrix_extra.h"
 #include "libset.h"
@@ -40,14 +41,32 @@ int gretl_VAR_autocorrelation_test (GRETL_VAR *var, int order,
 				    double ***pZ, DATAINFO *pdinfo,
 				    PRN *prn)
 {
+    MODEL *pmod;
+    double lb;
     int i, err = 0;
 
+    if (order == 0) {
+	order = pdinfo->pd;
+    }
+
     for (i=0; i<var->neqns && !err; i++) {
+	pmod = var->models[i];
 	pprintf(prn, "%s %d:\n", _("Equation"), i + 1);
-	err = autocorr_test(var->models[i], order, pZ, pdinfo,
-			    OPT_Q | OPT_S, prn);
-	gretl_model_test_print(var->models[i], 0, prn);
-	gretl_model_destroy_tests(var->models[i]);
+	if (pmod->list[0] == 1) {
+	    /* VECM */
+	    lb = ljung_box(order, pmod->t1, pmod->t2, pmod->uhat, &err);
+	    if (!err) {
+		pprintf(prn, "Ljung-Box Q' = %g %s = P(%s(%d) > %g) = %.3g\n", 
+			lb, _("with p-value"), _("Chi-square"), order,
+			lb, chisq_cdf_comp(order, lb));
+		pputc(prn, '\n');
+	    }
+	} else {
+	    err = autocorr_test(pmod, order, pZ, pdinfo,
+				OPT_Q | OPT_S, prn);
+	    gretl_model_test_print(var->models[i], 0, prn);
+	    gretl_model_destroy_tests(var->models[i]);
+	}
     }
 
     return err;
@@ -364,7 +383,7 @@ int VAR_do_lagsel (GRETL_VAR *var, const double **Z,
 		gretl_matrix_set(lltab, m, 1, 0);
 	    } else {
 		LRtest = 2.0 * (ll - gretl_matrix_get(lltab, m-1, 0));
-		gretl_matrix_set(lltab, m, 1, chisq_cdf_comp(LRtest, n * n));
+		gretl_matrix_set(lltab, m, 1, chisq_cdf_comp(n * n, LRtest));
 	    }	
 	    
 	    for (c=0; c<N_IVALS; c++) {
@@ -382,7 +401,7 @@ int VAR_do_lagsel (GRETL_VAR *var, const double **Z,
     if (!err) {
 	gretl_matrix_set(lltab, m, 0, var->ll);
 	LRtest = 2.0 * (var->ll - gretl_matrix_get(lltab, m - 1, 0));
-	gretl_matrix_set(lltab, m, 1, chisq_cdf_comp(LRtest, n * n));
+	gretl_matrix_set(lltab, m, 1, chisq_cdf_comp(n * n, LRtest));
 	gretl_matrix_set(crittab, m, 0, var->AIC);
 	gretl_matrix_set(crittab, m, 1, var->BIC);
 	gretl_matrix_set(crittab, m, 2, var->HQC);
@@ -625,7 +644,7 @@ static int gretl_VAR_real_omit_test (const GRETL_VAR *orig,
 
     LR = orig->T * (new->ldet - orig->ldet);
     df = orig->neqns * omitlist[0];
-    pval = chisq_cdf_comp(LR, df);
+    pval = chisq_cdf_comp(df, LR);
     
     pputs(prn, _("\n  Null hypothesis: the regression parameters are "
 		 "zero for the variables\n\n"));
