@@ -1,20 +1,20 @@
-/* gretl - The Gnu Regression, Econometrics and Time-series Library
- * Copyright (C) 1999-2006 Allin Cottrell and Riccardo "Jack" Lucchetti
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "libgretl.h"
@@ -38,6 +38,7 @@ struct diag_info {
 struct arbond_ {
     gretlopt opt;         /* option flags */
     int doZX;             /* calculation flag */
+    int doZy;             /* calculation flag */
     int step;             /* what step are we on? */
     int yno;              /* ID number of dependent var */
     int p;                /* lag order for dependent variable */
@@ -205,8 +206,7 @@ static int maybe_add_const_to_ilist (arbond *ab)
     return err;
 }
 
-static int arbond_make_lists (arbond *ab, const int *list, int xpos,
-			      const DATAINFO *pdinfo)
+static int arbond_make_lists (arbond *ab, const int *list, int xpos)
 {
     int i, nz = 0, spos = 0;
     int err = 0;
@@ -354,13 +354,14 @@ arbond_init (arbond *ab, const int *list, const DATAINFO *pdinfo,
     ab->opt = opt;
     ab->step = 1;
     ab->doZX = 1;
+    ab->doZy = 1;
     ab->nx = 0;
     ab->nz = 0;
     ab->t1min = 0;
     ab->ndum = 0;
 
     if (list[0] >= xpos) {
-	err = arbond_make_lists(ab, list, xpos, pdinfo);
+	err = arbond_make_lists(ab, list, xpos);
 	if (err) {
 	    return err;
 	}
@@ -404,29 +405,41 @@ arbond_init (arbond *ab, const int *list, const DATAINFO *pdinfo,
     return err;
 }
 
-/* see if there are NAs for the dependent variable, or
-   for any of the independent variables, at a given
-   observation */
+/* See if we have valid values for the dependent variable (in
+   differenced or deviations form) plus p lags of same, and all of
+   the independent variables, at the given observation, s.
+*/
 
-static int anymiss (arbond *ab, const double **Z, int s)
+static int obs_is_usable (arbond *ab, const double **Z, int s)
 {
+    int imax = ab->p + 1;
     int i;
 
-    for (i=0; i<=ab->p+1; i++) {
+    /* FIXME orthgonal deviations: we can compute such a
+       deviation for time-slot s if we have a value for
+       y at s-1 (given that we shift these forward) plus
+       one or more valid observations at s, s+1, ...
+       If I'm thinking correctly we don't necessarily 
+       require a valid value at s, or in other words
+       the first block below is too conservative.
+    */
+
+    for (i=0; i<=imax; i++) {
 	if (na(Z[ab->yno][s-i])) {
-	    return 1;
+	    return 0;
 	}
     }
 
     if (ab->xlist != NULL) {
+	/* check the independent vars */
 	for (i=1; i<=ab->xlist[0]; i++) {
 	    if (na(Z[ab->xlist[i]][s])) {
-		return 1;
+		return 0;
 	    }
 	}
     }
 
-    return 0;
+    return 1;
 }
 
 static int bzcols (arbond *ab, int i)
@@ -445,12 +458,11 @@ static int bzcols (arbond *ab, int i)
     return nc;
 }
 
-static int 
-arbond_sample_check (arbond *ab, const int *list, 
-		     const double **Z, const DATAINFO *pdinfo)
+static int arbond_sample_check (arbond *ab, const double **Z)
 {
     const double *y = Z[ab->yno];
     char *mask = NULL;
+    int tmin, tmax;
     int t1min = ab->T - 1;
     int t1imin = ab->T - 1;
     int t2max = 0;
@@ -486,6 +498,9 @@ arbond_sample_check (arbond *ab, const int *list,
 	ab->qmax = ab->T;
     }
 
+    tmin = ab->p + 1;
+    tmax = ab->T;
+
     for (i=0; i<ab->N; i++) {
 	int t1i = ab->T - 1, t2i = 0; 
 	int Ti = 0, usable = 0;
@@ -498,21 +513,21 @@ arbond_sample_check (arbond *ab, const int *list,
 	    mask[t] = 0;
 	}
 
-	/* identify the observations at which we can form the required
+	/* Identify the observations at which we can form the required
 	   Delta y terms, have the requisite independent variables,
 	   and can construct at least one orthogonality condition
-	   using a lagged level of y
+	   using a lagged level of y.
 	*/
 
 	s = data_index(ab, i);
-	for (t=ab->p+1; t<ab->T; t++) {
-	    if (anymiss(ab, Z, s + t)) {
-		mask[t] = 1;
-	    } else {
+	for (t=tmin; t<tmax; t++) {
+	    if (obs_is_usable(ab, Z, s + t)) {
 		usable++;
 		if (t < t1i) t1i = t;
 		if (t > t2i) t2i = t;
-	    } 
+	    } else {
+		mask[t] = 1;
+	    }
 	}
 
 	if (usable == 0) {
@@ -520,7 +535,7 @@ arbond_sample_check (arbond *ab, const int *list,
 	    ab->ui[i].t1 = -1;
 	    ab->ui[i].t2 = -1;
 #if ADEBUG
-	    fprintf(stderr, "not usable\n");
+	    fprintf(stderr, "unit not usable\n");
 #endif
 	    continue;
 	}
@@ -584,7 +599,7 @@ arbond_sample_check (arbond *ab, const int *list,
     } else {
 	/* compute the number of columns in Zi */
 	int tau = t2max - t1min + 1;
-	int nblocks = tau - ab->p - 1; /* was tau - 2 */
+	int nblocks = tau - ab->p - 1;
 	int bcols, cols = 0;
 
 #if ADEBUG
@@ -811,7 +826,12 @@ static int sargan_test (arbond *ab)
 					  &err);
 
     if (ab->step == 1) {
-	ab->sargan *= 2.0 / ab->s2; /* allow for scale factor in H */
+	/* allow for scale factor in H matrix */
+	if (ab->opt & OPT_H) {
+	    ab->sargan /= ab->s2;
+	} else {
+	    ab->sargan *= 2.0 / ab->s2; 
+	}
     }
 
 #if ADEBUG
@@ -1303,7 +1323,10 @@ static int arbond_variance (arbond *ab)
 #endif
 
     /* while we're at it... */
-    ar_test(ab, C);
+    if (!(ab->opt & OPT_H)) {
+	/* FIXME */
+	ar_test(ab, C);
+    }
     sargan_test(ab);
     arbond_wald_test(ab);
 
@@ -1390,7 +1413,7 @@ static int arbond_prepare_model (MODEL *pmod, arbond *ab,
 				 const double **X, const DATAINFO *pdinfo)
 {
     const double *y = X[ab->yno];
-    double x;
+    char prefix;
     int i, j;
     int err = 0;
 
@@ -1426,10 +1449,12 @@ static int arbond_prepare_model (MODEL *pmod, arbond *ab,
 	return pmod->errcode;
     }
 
+    prefix = (ab->opt & OPT_H)? 'O' : 'D';
+
     j = 0;
     for (i=0; i<ab->p; i++) {
 	/* FIXME possible varname overflow */
-	sprintf(pmod->params[j++], "D%.10s(-%d)", pdinfo->varname[ab->yno], i+1);
+	sprintf(pmod->params[j++], "%c%.10s(-%d)", prefix, pdinfo->varname[ab->yno], i+1);
     }
 
     for (i=0; i<ab->nx; i++) {
@@ -1442,21 +1467,11 @@ static int arbond_prepare_model (MODEL *pmod, arbond *ab,
 
     err = gretl_model_allocate_storage(pmod);
 
-    if (!err) {
-	gretl_model_new_vcv(pmod, NULL);
-    }
-
     if (!err) {
 	for (i=0; i<ab->k; i++) {
 	    pmod->coeff[i] = ab->beta->val[i];
-	    for (j=0; j<=i; j++) {
-		x = gretl_matrix_get(ab->vbeta, i, j);
-		pmod->vcv[ijton(i, j, ab->k)] = x;
-		if (i == j) {
-		    pmod->sderr[i] = sqrt(x);
-		}
-	    }
 	}
+	err = gretl_model_write_vcv(pmod, ab->vbeta);
     }
 
     if (!err) {
@@ -1795,13 +1810,24 @@ static int arbond_calculate (arbond *ab)
 	ab->doZX = 0;
     }
 
+#if ADEBUG
+    gretl_matrix_print(ab->ZX, "Z'X (arbond_calculate)");
+#endif
+
     /* find X'Z A */
     gretl_matrix_multiply_mod(ab->ZX, GRETL_MOD_TRANSPOSE,
 			      ab->A, GRETL_MOD_NONE,
 			      ab->XZA, GRETL_MOD_NONE);
 
     /* calculate "numerator", X'ZAZ'y */
-    gretl_matrix_multiply(ab->ZT, ab->dy, ab->R1);
+    if (ab->doZy) {
+	gretl_matrix_multiply(ab->ZT, ab->dy, ab->R1);
+    } else {
+	ab->doZy = 1;
+    }
+#if ADEBUG
+    gretl_matrix_print(ab->R1, "Z'y (arbond_calculate)");
+#endif
     gretl_matrix_multiply(ab->XZA, ab->R1, ab->beta);
 
     /* calculate "denominator", X'ZAZ'X */
@@ -1867,9 +1893,41 @@ static int arbond_step_2 (arbond *ab, PRN *prn)
     return err;
 }
 
-static void arbond_make_dy_and_X (arbond *ab, const double *y,
-				  const double **Z)
+static double odev_at_lag (const double *x, int t, int lag, int pd)
 {
+    double ret, xbar = 0.0;
+    int s, Tt, n = 0;
+
+    t -= lag + 1;
+
+    if (t < 0 || na(x[t])) {
+	return NADBL;
+    }
+
+    Tt = pd - (t % pd) - (lag + 1);
+
+    for (s=1; s<=Tt; s++) {
+	if (!na(x[t+s]) && !na(x[t+s+lag])) {
+	    xbar += x[t+s];
+	    n++;
+	}
+    }
+
+    if (n > 0) {
+	xbar /= n;
+	ret = sqrt(n / (n + 1.0)) * (x[t] - xbar);
+    } else {
+	ret = NADBL;
+    }
+
+    return ret;
+}
+
+static int arbond_make_dy_and_X (arbond *ab, const double *y,
+				 const double **Z, 
+				 const DATAINFO *pdinfo)
+{
+    int odev = (ab->opt & OPT_H);
     int i, j, s, t, k = 0;
     double x;
 
@@ -1883,10 +1941,20 @@ static void arbond_make_dy_and_X (arbond *ab, const double *y,
 	    }
 	    s = data_index(ab, i) + t;
 	    /* current difference of dependent var */
-	    gretl_vector_set(ab->dy, k, y[s] - y[s-1]);
+	    if (odev) {
+		x = odev_at_lag(y, s, 0, pdinfo->pd);
+		gretl_vector_set(ab->dy, k, x);
+	    } else {
+		gretl_vector_set(ab->dy, k, y[s] - y[s-1]);
+	    }
 	    for (j=0; j<ab->p; j++) {
 		/* lagged difference of dependent var */
-		gretl_matrix_set(ab->dX, k, j, y[s-j-1] - y[s-j-2]);
+		if (odev) {
+		    x = odev_at_lag(y, s, j+1, pdinfo->pd);
+		    gretl_matrix_set(ab->dX, k, j, x);
+		} else {
+		    gretl_matrix_set(ab->dX, k, j, y[s-j-1] - y[s-j-2]);
+		}
 	    }
 	    for (j=0; j<ab->nx; j++) {
 		/* independent vars */
@@ -1906,11 +1974,19 @@ static void arbond_make_dy_and_X (arbond *ab, const double *y,
     gretl_matrix_print(ab->dy, "dy");
     gretl_matrix_print(ab->dX, "X");
 #endif
+
+    return 0;
 }
 
+#define TRY_CUM 0
+
 static int arbond_make_Z_and_A (arbond *ab, const double *y,
 				const double **Z)
 {
+#if TRY_CUM
+    gretl_matrix *dXi = NULL, *dyi = NULL;
+    int cum_row = 0;
+#endif
     int i, j, k, s, t, c = 0;
     int zi, zj, zk;
     double x;
@@ -1920,7 +1996,14 @@ static int arbond_make_Z_and_A (arbond *ab, const double *y,
 #endif
     int err = 0;
 
+#if TRY_CUM
+    dXi = gretl_matrix_alloc(ab->Zi->rows, ab->dX->cols);
+    dyi = gretl_matrix_alloc(ab->Zi->rows, 1);
+#endif
+
     gretl_matrix_zero(ab->A);
+    gretl_matrix_zero(ab->ZX);
+    gretl_matrix_zero(ab->R1);
 
     for (i=0; i<ab->N && !err; i++) {
 	int ycols = ab->p; /* intial y block width */
@@ -1967,7 +2050,7 @@ static int arbond_make_Z_and_A (arbond *ab, const double *y,
 	    }
 	    
 	    /* additional block-diagonal columns, if required --
-	       needs for checking for the unbalanced case 
+	       needs checking for the unbalanced case 
 	    */
 	    zj = 0;
 	    for (zi=0; zi<ab->nzb; zi++) {
@@ -2013,20 +2096,67 @@ static int arbond_make_Z_and_A (arbond *ab, const double *y,
 	gretl_matrix_print(ab->Zi, zstr);
 #endif
 
-	err = make_first_diff_matrix(ab, i);
+#if TRY_CUM
+	/* Try cumulating Z'X and Z'y here: we need to extract the
+	   relevant Ti-related slices of dX and dy.
+	*/
+	int merr = 0;
+
+	gretl_matrix_reuse(dXi, Ti, -1);
+	gretl_matrix_reuse(dyi, Ti, -1);
+	
+	merr += gretl_matrix_extract_matrix(dXi, ab->dX, cum_row, 0,
+					    GRETL_MOD_NONE);
+	gretl_matrix_print(dXi, "dXi");
+
+	merr += gretl_matrix_extract_matrix(dyi, ab->dy, cum_row, 0,
+					    GRETL_MOD_NONE);
+	gretl_matrix_print(dyi, "dyi");
+
+	merr += gretl_matrix_multiply_mod(ab->Zi, GRETL_MOD_TRANSPOSE,
+					  dXi, GRETL_MOD_NONE,
+					  ab->ZX, GRETL_MOD_CUMULATE);
+
+	merr += gretl_matrix_multiply_mod(ab->Zi, GRETL_MOD_TRANSPOSE,
+					  dyi, GRETL_MOD_NONE,
+					  ab->R1, GRETL_MOD_CUMULATE);
+
+	fprintf(stderr, "merr = %d\n", merr);
+	cum_row += Ti;
+#if ADEBUG
+	gretl_matrix_print(ab->ZX, "Z'X");
+	gretl_matrix_print(ab->R1, "Z'y");
+#endif
+#endif
 
 	/* Cumulate Z_i' H Z_i into A_N */
-	gretl_matrix_qform(ab->Zi, GRETL_MOD_TRANSPOSE,
-			   ab->H, ab->A, GRETL_MOD_CUMULATE); 
+	if (ab->opt & OPT_H) {
+	    /* orthogonal deviations: "H" is identity matrix */
+	    gretl_matrix_multiply_mod(ab->Zi, GRETL_MOD_TRANSPOSE,
+				      ab->Zi, GRETL_MOD_NONE,
+				      ab->A, GRETL_MOD_CUMULATE);
+	} else {
+	    err = make_first_diff_matrix(ab, i);
+	    gretl_matrix_qform(ab->Zi, GRETL_MOD_TRANSPOSE,
+			       ab->H, ab->A, GRETL_MOD_CUMULATE);
+	}
 
 	/* Write Zi into ZT at offset 0, c */
 	gretl_matrix_inscribe_matrix(ab->ZT, ab->Zi, 0, c, GRETL_MOD_TRANSPOSE);
 	c += Ti;
     }
 
-    /* clean up */
-    make_first_diff_matrix(NULL, 0);
+    if (!(ab->opt & OPT_H)) {
+	/* clean up */
+	make_first_diff_matrix(NULL, 0);
+    }
 
+#if TRY_CUM
+    ab->doZX = 0;
+    ab->doZy = 0;
+    gretl_matrix_free(dXi);
+    gretl_matrix_free(dyi);
+#else
     if (!err) {
 	/* mask zero rows of ZT, if required */
 	zmask = zero_row_mask(ab->ZT, &err);
@@ -2035,6 +2165,11 @@ static int arbond_make_Z_and_A (arbond *ab, const double *y,
 	    free(zmask);
 	}
     } 
+#endif
+
+#if ADEBUG
+    gretl_matrix_print(ab->A, "\\sum Z_i' H Z_i");
+#endif
 
     if (!err) {
 	gretl_matrix_divide_by_scalar(ab->A, ab->N);
@@ -2162,7 +2297,7 @@ arbond_estimate (const int *list, const char *istr, const double **X,
 	return mod;
     }
 
-    mod.errcode = arbond_sample_check(&ab, list, X, pdinfo);
+    mod.errcode = arbond_sample_check(&ab, X);
     if (mod.errcode) {
 	fprintf(stderr, "Error %d in arbond_sample_check\n", mod.errcode);
 	goto bailout;
@@ -2181,19 +2316,23 @@ arbond_estimate (const int *list, const char *istr, const double **X,
     /* build the differenced vector \bar{y} plus the
        X matrix
     */
-    arbond_make_dy_and_X(&ab, X[ab.yno], X);
+    err = arbond_make_dy_and_X(&ab, X[ab.yno], X, pdinfo);
 
     /* build instrument matrix blocks, Z_i, and insert into
        big Z' matrix; cumulate first-stage A_N as we go */
-    err = arbond_make_Z_and_A(&ab, X[ab.yno], X);
+    if (!err) {
+	err = arbond_make_Z_and_A(&ab, X[ab.yno], X);
+    }
 
     /* invert A_N: we back up ab.A in case inversion fails */
-    gretl_matrix_copy_values(ab.Acpy, ab.A);
-    err = gretl_invert_symmetric_matrix(ab.A);
-    if (err) {
-	fprintf(stderr, "inverting ab.A failed on first pass\n");
-	/* failed: try again, reducing A based on its rank */
-	err = try_alt_inverse(&ab);
+    if (!err) {
+	gretl_matrix_copy_values(ab.Acpy, ab.A);
+	err = gretl_invert_symmetric_matrix(ab.A);
+	if (err) {
+	    fprintf(stderr, "inverting ab.A failed on first pass\n");
+	    /* failed: try again, reducing A based on its rank */
+	    err = try_alt_inverse(&ab);
+	}
     }
 
 #if ADEBUG
