@@ -22,13 +22,12 @@
    to Allin Cottrell; thanks also to Stephen Moshier for cephes.
 */
 
-#include "../cephes/polrt.c"
-
 #include "libgretl.h"
 #include "bhhh_max.h"
 #include "libset.h"
 #include "kalman.h"
 #include "matrix_extra.h"
+#include "../cephes/libprob.h"
 
 #define ARMA_DEBUG 0
 #define AINIT_DEBUG 0
@@ -610,7 +609,32 @@ static gretl_matrix *vQ;
 static double *ac;
 static double *mc;
 
-static struct arma_info *kainfo;    
+static struct arma_info *kainfo;   
+
+static void allocate_kalman_matrices (int r, int r2,
+				      int k, int T)
+{
+    S = gretl_column_vector_alloc(r);
+    P = gretl_matrix_alloc(r, r);
+    F = gretl_matrix_alloc(r, r);
+    A = gretl_column_vector_alloc(k);
+    H = gretl_column_vector_alloc(r);
+    E = gretl_column_vector_alloc(T);
+    Q = gretl_matrix_alloc(r, r);
+    Svar = gretl_matrix_alloc(r2, r2);
+}
+
+static void free_kalman_matrices (void)
+{
+    gretl_matrix_free(S);
+    gretl_matrix_free(P);
+    gretl_matrix_free(F);
+    gretl_matrix_free(A);
+    gretl_matrix_free(H);
+    gretl_matrix_free(E);
+    gretl_matrix_free(Q);
+    gretl_matrix_free(Svar);
+}
 
 static int ainfo_get_r (struct arma_info *ainfo)
 {
@@ -945,20 +969,22 @@ kalman_arma_score_callback (const double *b, int i, void *data)
 */
 
 static int arma_OPG_vcv (MODEL *pmod, kalman *K, double *b, 
-			 double s2, int k, int T)
+			 double s2, int k, int T,
+			 PRN *prn)
 {
     gretl_matrix *G = NULL;
     gretl_matrix *V = NULL;
-    double x;
-    int i, j, s, t;
-    int idx, err = 0;
+    double rcond;
+    int s, t;
+    int err = 0;
 
     s = 0;
     for (t=pmod->t1; t<=pmod->t2; t++) {
 	pmod->uhat[t] = gretl_vector_get(E, s++);
     }
 
-    G = build_OPG_matrix(b, k, T, kalman_arma_score_callback, (void *) K, &err);
+    G = build_OPG_matrix(b, k, T, kalman_arma_score_callback, 
+			 (void *) K, &err);
     if (err) {
 	goto bailout;
     }
@@ -973,97 +999,29 @@ static int arma_OPG_vcv (MODEL *pmod, kalman *K, double *b,
 			      G, GRETL_MOD_TRANSPOSE,
 			      V, GRETL_MOD_NONE);
 
-    err = gretl_invert_symmetric_matrix(V);
+    rcond = gretl_symmetric_matrix_rcond(V, &err);
+    if (!err && rcond < 1.0E-10) {
+	pprintf(prn, "OPG: rcond = %g; will try Hessian\n", rcond);
+	err = 1;
+    }
 
     if (!err) {
-	for (i=0; i<k; i++) {
-	    for (j=0; j<=i; j++) {
-		idx = ijton(i, j, k);
-		x = s2 * gretl_matrix_get(V, i, j);
-		pmod->vcv[idx] = x;
-		if (i == j) {
-		    pmod->sderr[i] = sqrt(x);
-		}
-	    }
-	}
+	err = gretl_invert_symmetric_matrix(V);
+    }
+
+    if (!err) {
+	gretl_matrix_multiply_by_scalar(V, s2);
+	err = gretl_model_write_vcv(pmod, V);
     }
 
  bailout:
 
-    pmod->errcode = err;
-
     gretl_matrix_free(G);
     gretl_matrix_free(V);
     
     return err;
 }
 
-/* in Kalman case the basic model struct is empty, so we have
-   to allocate for coefficients, residuals and so on */
-
-static int kalman_arma_model_allocate (MODEL *pmod, int k, int T,
-				       double *vcv)
-{
-    int err = 0;
-
-    pmod->ncoeff = k;
-    pmod->full_n = T;
-
-    err = gretl_model_allocate_storage(pmod);
-
-    if (!err && vcv == NULL) {
-	err = gretl_model_new_vcv(pmod, NULL);
-    }
-
-    return err;
-}
-
-static int kalman_arma_finish (MODEL *pmod, const int *alist,
-			       struct arma_info *ainfo,
-			       const double **Z, const DATAINFO *pdinfo, 
-			       kalman *K, double *b, double *vcv,
-			       int k, int T)
-{
-    double s2;
-    int i;
-
-    pmod->t1 = ainfo->t1;
-    pmod->t2 = ainfo->t2;
-    pmod->nobs = T;
-
-    pmod->errcode = kalman_arma_model_allocate(pmod, k, ainfo->T, vcv);
-    if (pmod->errcode) {
-	return pmod->errcode;
-    }
-
-    for (i=0; i<k; i++) {
-	pmod->coeff[i] = b[i];
-    }
-
-    s2 = kalman_get_arma_variance(K);
-    pmod->sigma = sqrt(s2);
-
-    pmod->lnL = kalman_get_loglik(K);
-
-#if ARMA_DEBUG
-    fprintf(stderr, "kalman_arma_finish: doing VCV, method %s\n",
-	    (vcv != NULL)? "Hessian" : "OPG");
-#endif
-
-    if (vcv != NULL) {
-	arma_hessian_vcv(pmod, vcv, k);
-    } else {
-	arma_OPG_vcv(pmod, K, b, s2, k, T);
-    }
-
-    write_arma_model_stats(pmod, alist, ainfo, Z, pdinfo);
-    arma_model_add_roots(pmod, ainfo, b);
-
-    gretl_model_set_int(pmod, "arma_flags", ARMA_EXACT);
-
-    return pmod->errcode;
-}
-
 #if ARMA_DEBUG
 
 static void debug_print_theta (const double *theta,
@@ -1120,6 +1078,110 @@ static double kalman_arma_ll (const double *b, void *p)
     return ll;
 }
 
+static void maybe_rescale_hessian (int kopt, double *vcv, 
+				   int k, int T)
+{
+    if (kopt & KALMAN_AVG_LL) {
+	int i, k2 = k * (k + 1) / 2;
+
+	for (i=0; i<k2; i++) {
+	    vcv[i] /= T;
+	}
+    }
+}
+
+static int arma_use_hessian (gretlopt opt)
+{
+    int ret = 1;
+
+    if (opt & OPT_G) {
+	ret = 0;
+    } else if (libset_get_int(ARMA_VCV) == VCV_OP) {
+	ret = 0;
+    }
+
+    return ret;
+}
+
+static int kalman_arma_finish (MODEL *pmod, const int *alist,
+			       struct arma_info *ainfo,
+			       const double **Z, const DATAINFO *pdinfo, 
+			       kalman *K, double *b, int k, int T,
+			       gretlopt opt, PRN *prn)
+{
+    int do_hess = arma_use_hessian(opt);
+    double s2;
+    int kopt, i;
+    int err;
+
+    pmod->t1 = ainfo->t1;
+    pmod->t2 = ainfo->t2;
+    pmod->nobs = T;
+    pmod->ncoeff = k;
+    pmod->full_n = ainfo->T;
+    
+    /* in the Kalman case the basic model struct is empty, so we 
+       have to allocate for coefficients, residuals and so on
+    */
+
+    err = gretl_model_allocate_storage(pmod);
+    if (err) {
+	return err;
+    }
+
+    for (i=0; i<k; i++) {
+	pmod->coeff[i] = b[i];
+    }
+
+    s2 = kalman_get_arma_variance(K);
+    pmod->sigma = sqrt(s2);
+
+    pmod->lnL = kalman_get_loglik(K);
+    kopt = kalman_get_options(K);
+
+    /* rescale if we're using average loglikelihood */
+    if (kopt & KALMAN_AVG_LL) {
+	pmod->lnL *= T;
+    }
+
+#if ARMA_DEBUG
+    fprintf(stderr, "kalman_arma_finish: doing VCV, method %s\n",
+	    (do_hess)? "Hessian" : "OPG");
+#endif
+
+    if (!do_hess) {
+	/* try OPG first, hessian as fallback */
+	err = arma_OPG_vcv(pmod, K, b, s2, k, T, prn);
+	if (err) {
+	    err = 0;
+	    do_hess = 1;
+	} else {
+	    gretl_model_set_int(pmod, "ml_vcv", VCV_OP);
+	}
+    }	
+
+    if (do_hess) { 
+	double *vcv;
+
+	kalman_do_ma_check = 0;
+	vcv = numerical_hessian(b, ainfo->nc, kalman_arma_ll, K, &err);
+	kalman_do_ma_check = 1;
+	if (!err) {
+	    maybe_rescale_hessian(kopt, vcv, k, T);
+	    arma_hessian_vcv(pmod, vcv, k);
+	    gretl_model_set_int(pmod, "ml_vcv", VCV_HESSIAN);
+	}
+    }
+
+    if (!err) {
+	write_arma_model_stats(pmod, alist, ainfo, Z, pdinfo);
+	arma_model_add_roots(pmod, ainfo, b);
+	gretl_model_set_int(pmod, "arma_flags", ARMA_EXACT);
+    }
+
+    return err;
+}
+
 static gretl_matrix *form_arma_y_vector (const int *alist, 
 					 const double **Z,
 					 struct arma_info *ainfo,
@@ -1236,7 +1298,7 @@ static void transform_arma_const (double *b, struct arma_info *ainfo)
     int i, k = 0;
 
 #if AINIT_DEBUG
-    fprintf(stderr, "transform_arma_const\n");
+    fprintf(stderr, "transform_arma_const: initially = %g\n", b[0]);
 #endif
 
     for (i=0; i<ainfo->p; i++) {
@@ -1252,6 +1314,34 @@ static void transform_arma_const (double *b, struct arma_info *ainfo)
     b[0] /= (narfac * sarfac);
 }
 
+static int kalman_undo_y_scaling (struct arma_info *ainfo,
+				  gretl_matrix *y, double *b, 
+				  kalman *K)
+{
+    double *beta = b + 1 + ainfo->np + ainfo->P +
+	ainfo->nq + ainfo->Q;
+    int i, t, T = ainfo->t2 - ainfo->t1 + 1;
+    int err = 0;
+
+    b[0] *= ainfo->dyscale;
+
+    for (i=0; i<ainfo->nexo; i++) {
+	beta[i] *= ainfo->dyscale;
+    }
+
+    i = ainfo->t1;
+    for (t=0; t<T; t++) {
+	y->val[t] *= ainfo->dyscale;
+	ainfo->dy[i++] *= ainfo->dyscale;
+    }
+
+    if (na(kalman_arma_ll(b, K))) {
+	err = 1;
+    }
+
+    return err;
+}
+
 static int kalman_arma (const int *alist, double *coeff, 
 			const double **Z, const DATAINFO *pdinfo,
 			struct arma_info *ainfo, MODEL *pmod,
@@ -1285,7 +1375,7 @@ static int kalman_arma (const int *alist, double *coeff,
 #if AINIT_DEBUG
     fputs("initial coefficients:\n", stderr);
     for (i=0; i<ainfo->nc; i++) {
-	fprintf(stderr, " b[%d] = %g\n", i, b[i]);
+	fprintf(stderr, " b[%d] = % .10E\n", i, b[i]);
     }
 #endif
 
@@ -1323,15 +1413,7 @@ static int kalman_arma (const int *alist, double *coeff,
 
     clear_gretl_matrix_err();
 
-    S = gretl_column_vector_alloc(r);
-    P = gretl_matrix_alloc(r, r);
-    F = gretl_matrix_alloc(r, r);
-    A = gretl_column_vector_alloc(k);
-    H = gretl_column_vector_alloc(r);
-    E = gretl_column_vector_alloc(T);
-    Q = gretl_matrix_alloc(r, r);
-
-    Svar = gretl_matrix_alloc(r2, r2);
+    allocate_kalman_matrices(r, r2, k, T);
 
     if (arma_using_vech(ainfo)) {
 	vQ = gretl_column_vector_alloc(m);
@@ -1373,7 +1455,13 @@ static int kalman_arma (const int *alist, double *coeff,
 	} else {
 	    kalman_set_nonshift(K, r);
 	}
-	kalman_use_ARMA_ll(K);
+
+	if (T > 3072 || getenv("KALMAN_AVG_LL") != NULL) {
+	    kalman_set_options(K, KALMAN_ARMA_LL | KALMAN_AVG_LL);
+	} else {
+	    kalman_set_options(K, KALMAN_ARMA_LL);
+	}
+
 	err = BFGS_max(b, ainfo->nc, maxit, reltol, 
 		       &fncount, &grcount, kalman_arma_ll, C_LOGLIK,
 		       NULL, K, opt, prn);
@@ -1382,38 +1470,30 @@ static int kalman_arma (const int *alist, double *coeff,
 	} 
     }
 
-    if (err) {
-	pmod->errcode = err;
-    } else {
-	double *hess = NULL;
+    if (!err && ainfo->dyscale != 1.0) {
+	kalman_undo_y_scaling(ainfo, y, b, K);
+    }
 
+    if (!err) {
 	gretl_model_set_int(pmod, "fncount", fncount);
 	gretl_model_set_int(pmod, "grcount", grcount);
-	kalman_do_ma_check = 0;
-	hess = numerical_hessian(b, ainfo->nc, kalman_arma_ll, K, &err);
-	kalman_do_ma_check = 1;
-	if (err) {
-	    /* fall back to OPG */
-	    err = 0;
-	}
-	kalman_arma_finish(pmod, alist, ainfo, Z, pdinfo, 
-			   K, b, hess, ainfo->nc, T);
+	err = kalman_arma_finish(pmod, alist, ainfo, 
+				 Z, pdinfo, K, b, 
+				 ainfo->nc, T, opt,
+				 prn);
     } 
 
+    if (err) {
+	pmod->errcode = err;
+    }
+
     kalman_free(K);
 
-    gretl_matrix_free(S);
-    gretl_matrix_free(P);
-    gretl_matrix_free(F);
-    gretl_matrix_free(A);
-    gretl_matrix_free(H);
-    gretl_matrix_free(E);
-    gretl_matrix_free(Q);
+    free_kalman_matrices();
 
     gretl_matrix_free(y);
     gretl_matrix_free(x);
 
-    gretl_matrix_free(Svar);
     gretl_matrix_free(Svar2);
     gretl_matrix_free(vQ);
 
@@ -1559,15 +1639,11 @@ static void nls_kickstart (int b0, int by1,
 }
 
 static int arma_get_nls_model (MODEL *amod, struct arma_info *ainfo,
-			       int narmax, double ***pZ, DATAINFO *pdinfo) 
+			       int narmax, const double *coeff,
+			       double ***pZ, DATAINFO *pdinfo,
+			       PRN *prn) 
 {
-#if AINIT_DEBUG
-    PRN *prn = gretl_print_new(GRETL_PRINT_STDERR);
-    gretlopt nlsopt = OPT_A | OPT_V;
-#else
-    PRN *prn = NULL;
-    gretlopt nlsopt = OPT_A | OPT_C;
-#endif
+    gretlopt nlsopt = OPT_A;
     char fnstr[MAXLINE];
     char term[32];
     nlspec *spec;
@@ -1582,6 +1658,20 @@ static int arma_get_nls_model (MODEL *amod, struct arma_info *ainfo,
 	return E_ALLOC;
     }
 
+    if (arma_least_squares(ainfo)) {
+	/* respect verbose option */
+	if (prn != NULL) {
+	    nlsopt |= OPT_V;
+	}
+    } else {
+#if AINIT_DEBUG
+	nlsopt |= OPT_V;
+#else
+	/* don't bother with standard errors */
+	nlsopt |= OPT_C;
+#endif
+    }
+
     nlspec_set_t1_t2(spec, 0, ainfo->t2 - ainfo->t1); /* ?? */
 
     nparam = ainfo->ifc + ainfo->np + ainfo->P + ainfo->nexo;
@@ -1604,7 +1694,11 @@ static int arma_get_nls_model (MODEL *amod, struct arma_info *ainfo,
     k = 1;
 
     if (ainfo->ifc) {
-	(*pZ)[v][0] = gretl_mean(0, pdinfo->n - 1, (*pZ)[1]);
+	if (coeff != NULL) {
+	    (*pZ)[v][0] = coeff[0];
+	} else {
+	    (*pZ)[v][0] = gretl_mean(0, pdinfo->n - 1, (*pZ)[1]);
+	}
 	strcpy(pdinfo->varname[v], "b0");
 	b0 = v;
 	plist[k++] = v++;
@@ -1614,7 +1708,12 @@ static int arma_get_nls_model (MODEL *amod, struct arma_info *ainfo,
 	if (AR_included(ainfo, i)) {
 	    if (by1 == 0) {
 		by1 = v;
-		(*pZ)[v][0] = 0.1;
+		if (coeff == NULL) {
+		    (*pZ)[v][0] = 0.1;
+		}
+	    }
+	    if (coeff != NULL) {
+		(*pZ)[v][0] = coeff[k-1];
 	    }
 	    sprintf(pdinfo->varname[v], "phi%d", i+1);
 	    plist[k++] = v++;
@@ -1624,13 +1723,21 @@ static int arma_get_nls_model (MODEL *amod, struct arma_info *ainfo,
     for (i=0; i<ainfo->P; i++) {
 	if (by1 == 0) {
 	    by1 = v;
-	    (*pZ)[v][0] = 0.1;
+	    if (coeff == NULL) {
+		(*pZ)[v][0] = 0.1;
+	    }
+	}
+	if (coeff != NULL) {
+	    (*pZ)[v][0] = coeff[k-1];
 	}
 	sprintf(pdinfo->varname[v], "Phi%d", i+1);
 	plist[k++] = v++;
     }
 
     for (i=0; i<ainfo->nexo; i++) {
+	if (coeff != NULL) {
+	    (*pZ)[v][0] = coeff[k-1];
+	}
 	sprintf(pdinfo->varname[v], "b%d", i+1);
 	plist[k++] = v++;
     }
@@ -1679,7 +1786,9 @@ static int arma_get_nls_model (MODEL *amod, struct arma_info *ainfo,
     }
 
     if (!err) {
-	nls_kickstart(b0, by1, amod, pZ, pdinfo);
+	if (coeff == NULL) {
+	    nls_kickstart(b0, by1, amod, pZ, pdinfo);
+	}
 
 #if AINIT_DEBUG
 	fprintf(stderr, "initting using NLS spec:\n %s\n", fnstr);
@@ -1705,7 +1814,6 @@ static int arma_get_nls_model (MODEL *amod, struct arma_info *ainfo,
 	if (!err) {
 	    printmodel(amod, pdinfo, OPT_NONE, prn);
 	}
-	gretl_print_destroy(prn);
 #endif
     }
 
@@ -1969,18 +2077,37 @@ static void arma_init_transcribe_coeffs (struct arma_info *ainfo,
     }	
 }
 
+static void maybe_rescale_dy (struct arma_info *ainfo)
+{
+    double ybar = gretl_mean(0, ainfo->T - 1, ainfo->dy);
+
+    if (fabs(ybar) > 250) {
+	int t;
+
+	for (t=0; t<ainfo->T; t++) {
+	    if (!na(ainfo->dy[t])) {
+		ainfo->dy[t] /= ybar;
+	    }
+	}
+	ainfo->dyscale = ybar;
+    }
+}
+
 /* Run a least squares model to get initial values for the AR
    coefficients, either OLS or NLS.  We use NLS if there is
    nonlinearity due to either (a) the presence of both a seasonal and
    a non-seasonal AR component or (b) the presence of exogenous
    variables in the context of a non-zero AR order, where estimation
-   will be via exact ML.  In this initialization any MA coefficients
-   are simply set to near-zero.
+   will be via exact ML.  
+
+   In this initialization any MA coefficients are simply set to
+   near-zero.
 */
 
 static int ar_arma_init (const int *list, double *coeff, 
 			 const double **Z, const DATAINFO *pdinfo,
-			 struct arma_info *ainfo, PRN *prn)
+			 struct arma_info *ainfo, 
+			 MODEL *pmod, PRN *prn)
 {
     int an = pdinfo->t2 - ainfo->t1 + 1;
     int nmixed = ainfo->np * ainfo->P;
@@ -2014,9 +2141,13 @@ static int ar_arma_init (const int *list, double *coeff,
     if (narmax > 0) {
 	/* ARMAX-induced lags of exog vars */
 	av += ainfo->nexo * ptotal;
-    }    
+    } 
 
-    adinfo = create_new_dataset(&aZ, av, an, 0);
+    if (arma_exact_ml(ainfo) && ainfo->ifc && ainfo->dy != NULL) {
+	maybe_rescale_dy(ainfo);
+    }
+
+    adinfo = create_auxiliary_dataset(&aZ, av, an);
     if (adinfo == NULL) {
 	return E_ALLOC;
     }
@@ -2039,10 +2170,14 @@ static int ar_arma_init (const int *list, double *coeff,
 			    Z, aZ, adinfo);
 
     if (nonlin) {
+	PRN *dprn = NULL;
+
 #if AINIT_DEBUG
 	fprintf(stderr, "arma:_init_by_ls: doing NLS\n");
+	dprn = prn;
 #endif
-	err = arma_get_nls_model(&armod, ainfo, narmax, &aZ, adinfo);
+	err = arma_get_nls_model(&armod, ainfo, narmax, NULL, &aZ, adinfo,
+				 dprn);
     } else {
 #if AINIT_DEBUG
 	printlist(alist, "'alist' in ar_arma_init (OLS)");
@@ -2051,19 +2186,6 @@ static int ar_arma_init (const int *list, double *coeff,
 	err = armod.errcode;
     }
 
-    if (!err) {
-	arma_init_transcribe_coeffs(ainfo, &armod, coeff);
-    }
-
-    if (!err && arma_exact_ml(ainfo) && ainfo->ifc && 
-	(!nonlin || ainfo->nexo == 0)) {
-	/* handle the case where we need to translate from an
-	   estimate of the regression constant to the
-	   unconditional mean of y_t
-	*/
-	transform_arma_const(coeff, ainfo);
-    }
-
 #if AINIT_DEBUG
     if (!err) {
 	fprintf(stderr, "LS init: ncoeff = %d, nobs = %d\n", 
@@ -2076,6 +2198,19 @@ static int ar_arma_init (const int *list, double *coeff,
     }
 #endif
 
+    if (!err) {
+	arma_init_transcribe_coeffs(ainfo, &armod, coeff);
+    }
+
+    /* handle the case where we need to translate from an
+       estimate of the regression constant to the
+       unconditional mean of y_t
+    */
+    if (!err && arma_exact_ml(ainfo) && ainfo->ifc && 
+	(!nonlin || ainfo->nexo == 0)) {
+	transform_arma_const(coeff, ainfo);
+    }
+
     if (!err && prn != NULL) {
 	if (nonlin) {
 	    pputs(prn, "\narma initialization: using nonlinear AR model\n\n");
@@ -2085,13 +2220,98 @@ static int ar_arma_init (const int *list, double *coeff,
     }
 
     /* clean up */
+    clear_model(&armod);
     free(alist);
     destroy_dataset(aZ, adinfo);
-    clear_model(&armod);
 
     return err;
 }
 
+static int arma_by_ls (const int *list, double *coeff, 
+		       const double **Z, const DATAINFO *pdinfo,
+		       struct arma_info *ainfo, 
+		       MODEL *pmod, PRN *prn)
+{
+    int an = pdinfo->t2 - ainfo->t1 + 1;
+    int nmixed = ainfo->np * ainfo->P;
+    int ptotal = ainfo->np + ainfo->P + nmixed;
+    int av = ptotal + ainfo->nexo + 2;
+    double **aZ = NULL;
+    DATAINFO *adinfo = NULL;
+    int *alist = NULL;
+    int nonlin = 0;
+
+    adinfo = create_auxiliary_dataset(&aZ, av, an);
+    if (adinfo == NULL) {
+	return E_ALLOC;
+    }
+
+    if (ptotal > 0 && nmixed > 0) {
+	/* we'll have to use NLS */
+	nonlin = 1;
+    } else {
+	/* OLS: need regression list */
+	alist = make_ar_ols_list(ainfo, av);
+    }
+
+    /* add variable names to auxiliary dataset */
+    arma_init_add_varnames(ainfo, ptotal, 0, adinfo);
+
+    /* build temporary dataset */
+    arma_init_build_dataset(ainfo, ptotal, 0, list,
+			    Z, aZ, adinfo);
+
+    if (nonlin) {
+	pmod->errcode = arma_get_nls_model(pmod, ainfo, 0, coeff, &aZ, adinfo,
+					   prn);
+    } else {
+	*pmod = lsq(alist, &aZ, adinfo, OLS, OPT_A | OPT_Z);
+    }
+
+    /* clean up */
+    free(alist);
+    destroy_dataset(aZ, adinfo);
+
+    if (!pmod->errcode && pmod->full_n < pdinfo->n) {
+	/* the model series are short */
+	double *uhat, *yhat;
+	int s, t;
+
+	uhat = malloc(pdinfo->n * sizeof *uhat);
+	yhat = malloc(pdinfo->n * sizeof *yhat);
+	if (uhat == NULL || yhat == NULL) {
+	    free(uhat);
+	    free(yhat);
+	    pmod->errcode = E_ALLOC;
+	} else {
+	    for (t=0; t<pdinfo->n; t++) {
+		uhat[t] = yhat[t] = NADBL;
+	    }
+	    t = ainfo->t1;
+	    for (s=0; s<pmod->full_n; s++, t++) {
+		uhat[t] = pmod->uhat[s];
+		yhat[t] = pmod->yhat[s];
+	    }
+	    free(pmod->uhat);
+	    pmod->uhat = uhat;
+	    free(pmod->yhat);
+	    pmod->yhat = yhat;
+	}
+    }
+
+    if (!pmod->errcode) {
+	pmod->t1 = ainfo->t1;
+	pmod->t2 = ainfo->t2;
+	pmod->full_n = pdinfo->n;
+
+	write_arma_model_stats(pmod, list, ainfo, Z, pdinfo);
+	arma_model_add_roots(pmod, ainfo, pmod->coeff);
+	gretl_model_set_int(pmod, "arma_flags", ARMA_LS);
+    }
+
+    return pmod->errcode;
+}
+
 #define MINLAGS 16
 
 static int hr_init_check (const DATAINFO *pdinfo, struct arma_info *ainfo)
@@ -2221,7 +2441,7 @@ static int hr_arma_init (const int *list, double *coeff,
 	y = Z[ainfo->yno];
     }
 
-    adinfo = create_new_dataset(&aZ, pass1v + qtotal, an, 0);
+    adinfo = create_auxiliary_dataset(&aZ, pass1v + qtotal, an);
     if (adinfo == NULL) {
 	return E_ALLOC;
     }
@@ -2377,7 +2597,7 @@ static int hr_arma_init (const int *list, double *coeff,
 	err = armod.errcode;
     } else {
 #if AINIT_DEBUG
-	PRN *modprn = gretl_print_new(GRETL_PRINT_STDERR);
+	PRN *modprn = gretl_print_new(GRETL_PRINT_STDERR, NULL);
 
 	printmodel(&armod, adinfo, OPT_S, modprn);
 	gretl_print_destroy(modprn);
@@ -2428,8 +2648,12 @@ static int user_arma_init (double *coeff, struct arma_info *ainfo,
 	return E_DATA;
     }
 
-    if (!(flags & ARMA_EXACT)) {
-	/* in EXACT case, this is handled within BFGS */
+    if (flags & ARMA_EXACT) {
+	/* initialization is handled within BFGS */
+	for (i=0; i<ainfo->nc; i++) {
+	    coeff[i] = 0.0;
+	}	
+    } else {
 	const gretl_matrix *m = get_init_vals();
 
 	pputs(prn, "\narma initialization: at user-specified values\n\n");
@@ -2539,52 +2763,6 @@ static int bhhh_arma (const int *alist, double *coeff,
     return pmod->errcode;
 }
 
-#if 0 /* not yet */
-
-static int bhhh_arma_init (const int *alist, double *coeff, 
-			   const double **Z, const DATAINFO *pdinfo,
-			   struct arma_info *ainfo)
-{
-    model_info *minfo = NULL;
-    const double **X = NULL;
-    int err = 0;
-
-    /* construct virtual dataset for dep var, real regressors */
-    X = make_armax_X(alist, ainfo, Z);
-    if (X == NULL) {
-	return E_ALLOC;
-    }
-
-    /* create model_info struct to feed to bhhh_max() */
-    minfo = set_up_arma_model_info(ainfo);
-    if (minfo == NULL) {
-	free(X);
-	return E_ALLOC;
-    }
-
-    /* call BHHH conditional ML function (OPG regression) */
-    err = bhhh_max(arma_ll, X, coeff, minfo, OPT_NONE, NULL);
-    
-    if (err) {
-	fprintf(stderr, "bhhh_arma_init: bhhh_max returned %d\n", err);
-	err = E_NOCONV;
-    } else {
-	double *theta = model_info_get_theta(minfo);
-	int i;
-	
-	for (i=0; i<ainfo->nc; i++) {
-	    coeff[i] = theta[i];
-	}
-    }
-
-    free(X);
-    model_info_free(minfo);
-
-    return err;
-}
-
-#endif /* not yet */
-
 /* Should we try Hannan-Rissanen initialization of ARMA
    coefficients? */
 
@@ -2605,14 +2783,12 @@ static int prefer_hr_init (struct arma_info *ainfo)
 	    ret = 0;
 	}
 
-#if 1 
 	/* not sure about this: HR catches the MA terms, but NLS
 	   handles better the AR interactions
 	*/
 	if (ainfo->p > 0 && ainfo->P > 0) {
 	    ret = 0;
 	}
-#endif
 
 	/* overlapping orders screw things up */
 	if ((ainfo->P > 0 && ainfo->p >= ainfo->pd) ||
@@ -2657,7 +2833,6 @@ MODEL arma_model (const int *list, const char *pqspec,
 
     arma_info_init(&ainfo, flags, pqspec, pdinfo);
     gretl_model_init(&armod); 
-    gretl_model_smpl_init(&armod, pdinfo);
 
     alist = gretl_list_copy(list);
     if (alist == NULL) {
@@ -2703,7 +2878,15 @@ MODEL arma_model (const int *list, const char *pqspec,
 	goto bailout;
     }
 
-    /* second pass: try Hannan-Rissanen? */
+    if (!(flags & ARMA_EXACT) && ainfo.q == 0 && ainfo.Q == 0) {
+	/* pure AR model can be estimated via least squares */
+	ainfo.flags |= ARMA_LS;
+	arma_by_ls(alist, (init_done)? coeff : NULL, Z, pdinfo, 
+		   &ainfo, &armod, vprn);
+	goto bailout;
+    }
+
+    /* second pass: try Hannan-Rissanen, if suitable */
     if (!init_done && prefer_hr_init(&ainfo)) {
 	err = hr_init_check(pdinfo, &ainfo);
 	if (!err) {
@@ -2723,7 +2906,8 @@ MODEL arma_model (const int *list, const char *pqspec,
 
     /* third pass: estimate pure AR model by OLS or NLS */
     if (!init_done) {
-	err = ar_arma_init(alist, coeff, Z, pdinfo, &ainfo, vprn);
+	err = ar_arma_init(alist, coeff, Z, pdinfo, &ainfo, 
+			   &armod, vprn);
     }
 
     if (err) {
@@ -2732,9 +2916,6 @@ MODEL arma_model (const int *list, const char *pqspec,
     }
 
     if (flags & ARMA_EXACT) {
-#if 0 /* not yet */
-	bhhh_arma_init(alist, coeff, Z, pdinfo, &ainfo);
-#endif
 	kalman_arma(alist, coeff, Z, pdinfo, &ainfo, &armod, opt, vprn);
     } else {
 	bhhh_arma(alist, coeff, Z, pdinfo, &ainfo, &armod, opt, vprn);
@@ -2742,6 +2923,10 @@ MODEL arma_model (const int *list, const char *pqspec,
 
  bailout:
 
+    if (!armod.errcode) {
+	gretl_model_smpl_init(&armod, pdinfo);
+    }
+
     free(alist);
     free(coeff);
     arma_info_cleanup(&ainfo);
@@ -2752,6 +2937,7 @@ MODEL arma_model (const int *list, const char *pqspec,
     if (vprn != prn) {
 	close_down_verbose_printer(vprn);
     }
+
     vprn = NULL;
 
     return armod;
