@@ -25,6 +25,7 @@ struct arma_info {
     int pd;          /* periodicity of data */
     int T;           /* full length of data series */
     double *dy;      /* differenced dependent variable */
+    double dyscale;  /* scale factor for dy */
     const char *pqspec; /* input string with specific AR, MA lags */
 };
 
@@ -33,11 +34,13 @@ struct arma_info {
 #define arma_by_x12a(a)        ((a)->flags & ARMA_X12A)
 #define arma_exact_ml(a)       ((a)->flags & ARMA_EXACT)
 #define arma_using_vech(a)     ((a)->flags & ARMA_VECH)
+#define arma_least_squares(a)  ((a)->flags & ARMA_LS)
 
 #define set_arma_has_seasonal(a)  ((a)->flags |= ARMA_SEAS)
 #define set_arma_is_arima(a)      ((a)->flags |= ARMA_DSPEC)
 #define unset_arma_is_arima(a)    ((a)->flags &= ~ARMA_DSPEC)
 #define set_arma_use_vech(a)      ((a)->flags |= ARMA_VECH)
+#define set_arma_least_squares(a) ((a)->flags |= ARMA_LS)
 
 #define AR_included(a,i) (a->pmask == NULL || a->pmask[i] == '1')
 #define MA_included(a,i) (a->qmask == NULL || a->qmask[i] == '1')
@@ -73,6 +76,7 @@ arma_info_init (struct arma_info *ainfo, char flags,
     ainfo->T = pdinfo->n;
 
     ainfo->dy = NULL;
+    ainfo->dyscale = 1.0;
     ainfo->pqspec = pqspec;
 }
 
@@ -191,7 +195,15 @@ static gretl_matrix *get_arma_pq_vec (struct arma_info *ainfo,
 	if (*s == '{') {
 	    m = matrix_from_spec(s + 1, tmp, err);
 	} else {
-	    m = get_matrix_by_name(s);
+	    char *p, mname[VNAMELEN];
+
+	    *mname = '\0';
+	    strncat(mname, s, VNAMELEN - 1);
+	    p = strchr(mname, ',');
+	    if (p != NULL) {
+		*p = '\0';
+	    }
+	    m = get_matrix_by_name(mname);
 	    if (m == NULL) {
 		*err = E_UNKVAR;
 	    }
@@ -209,7 +221,7 @@ arma_make_masks (struct arma_info *ainfo, int *list)
 
     if (ainfo->p > 0) {
 	ainfo->np = ainfo->p;
-	if (ainfo->pqspec != NULL) {
+	if (ainfo->pqspec != NULL && *ainfo->pqspec != '\0') {
 	    m = get_arma_pq_vec(ainfo, AR_MASK, &tmp, &err);
 	    if (m != NULL) {
 		ainfo->pmask = mask_from_vec(m, ainfo, AR_MASK, &err);
@@ -222,7 +234,7 @@ arma_make_masks (struct arma_info *ainfo, int *list)
 
     if (ainfo->q > 0 && !err) {
 	ainfo->nq = ainfo->q;
-	if (ainfo->pqspec != NULL) {
+	if (ainfo->pqspec != NULL && *ainfo->pqspec != '\0') {
 	    m = get_arma_pq_vec(ainfo, MA_MASK, &tmp, &err);
 	    if (m != NULL) {
 		ainfo->qmask = mask_from_vec(m, ainfo, MA_MASK, &err);
@@ -276,33 +288,26 @@ static int arima_integrate (double *dx, const double *x,
 	if (d > 0) {
 	    ix[t] += x[t-1];
 	} 
-	if (d > 1) {
-	    ix[t] += x[t-1];
-	    ix[t] -= x[t-2];
+	if (d == 2) {
+	    ix[t] += x[t-1] - x[t-2];
 	}
 	if (D > 0) {
 	    ix[t] += x[t-s];
 	    if (d > 0) {
 		ix[t] -= x[t-s-1];
 	    }
-	    if (d > 1) {
-		ix[t] -= x[t-s-1];
-		ix[t] += x[t-2*s];
+	    if (d == 2) {
+		ix[t] -= x[t-s-1] - x[t-s-2];
 	    }	    
 	} 
-	if (D > 1) {
-	    ix[t] += x[t-s];
-	    ix[t] -= x[t-2*s];
+	if (D == 2) {
+	    ix[t] += x[t-s] - x[t-2*s];
 	    if (d > 0) {
-		ix[t] += x[t-s];
-		ix[t] -= x[t-s-1];
-		ix[t] += x[t-2*s-1];
+		ix[t] -= x[t-s-1] - x[t-2*s-1];
 	    }
-	    if (d > 1) {
-		ix[t] -= 2 * x[t-s-1];
-		ix[t] += 2 * x[t-s-2];
-		ix[t] += x[t-2*s-1];
-		ix[t] -= x[t-2*s-2];
+	    if (d == 2) {
+		ix[t] -= x[t-s-1] - x[t-2*s-1];
+		ix[t] += x[t-s-2] - x[t-2*s-2];
 	    }
 	}
     }
@@ -386,8 +391,10 @@ static void write_arma_model_stats (MODEL *pmod, const int *list,
 	y = Z[ainfo->yno];
     }
 
-    pmod->ybar = gretl_mean(pmod->t1, pmod->t2, y);
-    pmod->sdy = gretl_stddev(pmod->t1, pmod->t2, y);
+    if (!arma_least_squares(ainfo)) {
+	pmod->ybar = gretl_mean(pmod->t1, pmod->t2, y);
+	pmod->sdy = gretl_stddev(pmod->t1, pmod->t2, y);
+    }
 
     mean_error = pmod->ess = 0.0;
 
@@ -415,7 +422,7 @@ static void write_arma_model_stats (MODEL *pmod, const int *list,
     pmod->rsq = pmod->adjrsq = pmod->fstt = NADBL;
     pmod->tss = NADBL;
 
-    if (!arma_by_x12a(ainfo)) {
+    if (!arma_by_x12a(ainfo) && !arma_least_squares(ainfo)) {
 	mle_criteria(pmod, 1);
     }
 
@@ -429,7 +436,7 @@ static void write_arma_model_stats (MODEL *pmod, const int *list,
 static void calc_max_lag (struct arma_info *ainfo)
 {
     int pmax = ainfo->p;
-    int dmax = ainfo->d; /* FIXME what about seasonal D? */
+    int dmax = ainfo->d;
 
     if (arma_has_seasonal(ainfo)) {
 	pmax += ainfo->P * ainfo->pd;
@@ -449,49 +456,59 @@ static int
 arma_adjust_sample (const DATAINFO *pdinfo, const double **Z, const int *list,
 		    struct arma_info *ainfo)
 {
+    int vstart = arma_list_y_position(ainfo);
     int t1 = pdinfo->t1, t2 = pdinfo->t2;
     int an, i, v, t, t1min;
-    int vstart, pmax, anymiss;
+    int anymiss;
 
 #if SAMPLE_DEBUG
-    fprintf(stderr, "arma_adjust_sample: at start, t1=%d, t2=%d\n",
-	    t1, t2);
+    fprintf(stderr, "arma_adjust_sample: at start, t1=%d, t2=%d, maxlag = %d\n",
+	    t1, t2, ainfo->maxlag);
 #endif
 
-    vstart = arma_list_y_position(ainfo);
-
-    pmax = ainfo->p + ainfo->P * ainfo->pd;
-
-    /* determine starting point for valid data, t1min */
+    /* determine the starting point for valid data, t1min */
     t1min = 0;
     for (t=0; t<=pdinfo->t2; t++) {
 	anymiss = 0;
 	for (i=vstart; i<=list[0]; i++) {
-	    v = list[i];
-	    if (na(Z[v][t])) {
+	    if (na(Z[list[i]][t])) {
 		anymiss = 1;
+		t1min++;
 		break;
 	    }
 	}
-	if (anymiss) {
-	    t1min++;
-        } else {
+	if (!anymiss) {
 	    break;
 	}
     }
 
-    if (arma_by_x12a(ainfo) || arma_exact_ml(ainfo)) {
-	/* FIXME x12a in conditional mode? */
-	;
-    } else {
-	t1min += ainfo->maxlag;
-    }
+#if SAMPLE_DEBUG
+    fprintf(stderr, " phase 1: t1min = %d\n", t1min);
+#endif
 
-    /* if the notional starting point is before the start of
-       valid data, advance it */
+    /* if the notional starting point, t1, is before the start of
+       valid data, t1min, advance t1 */
     if (t1 < t1min) {
 	t1 = t1min;
     }
+    
+    if (!arma_exact_ml(ainfo)) {
+	/* conditional ML: ensure that the sample start allows for
+	   the required lags of y */
+	int t0;
+
+	if (t1 < ainfo->maxlag) {
+	    t1 = ainfo->maxlag;
+	}
+	t0 = t1 - ainfo->maxlag;
+	t1min = t1;
+	v = list[vstart];
+	for (t=t0; t<t1min; t++) {
+	    if (na(Z[v][t])) {
+		t1 = t + ainfo->maxlag + 1;
+	    }
+	}
+    }
 
     /* trim any missing obs from the end of the specified sample
        range 
@@ -499,42 +516,24 @@ arma_adjust_sample (const DATAINFO *pdinfo, const double **Z, const int *list,
     for (t=pdinfo->t2; t>=t1; t--) {
 	anymiss = 0;
 	for (i=vstart; i<=list[0]; i++) {
-	    v = list[i];
-	    if (na(Z[v][t])) {
+	    if (na(Z[list[i]][t])) {
 		anymiss = 1;
+		t2--;
 		break;
 	    }
 	}
-	if (anymiss) {
-	    t2--;
-        } else {
+	if (!anymiss) {
 	    break;
 	}
     }
 
-    if (arma_by_x12a(ainfo) || arma_exact_ml(ainfo)) {
-	/* FIXME x12a in conditional mode? */
-	t1min = t1;
-    } else {    
-	t1min = t1 - pmax; /* wrong? */
-	if (t1min < 0) {
-	    t1min = 0;
-	}
-    }
-
     /* check for missing obs within the sample range */
-    for (t=t1min; t<t2; t++) {
+    for (t=t1; t<t2; t++) {
 	for (i=vstart; i<=list[0]; i++) {
-	    if (t < t1 && i > vstart) {
-		continue;
-	    }
 	    v = list[i];
 	    if (na(Z[v][t])) {
-		char msg[64];
-
-		sprintf(msg, _("Missing value encountered for "
-			       "variable %d, obs %d"), v, t + 1);
-		gretl_errmsg_set(msg);
+		gretl_errmsg_sprintf(_("Missing value encountered for "
+				       "variable %d, obs %d"), v, t + 1);
 		return 1;
 	    }
 	}
@@ -542,7 +541,8 @@ arma_adjust_sample (const DATAINFO *pdinfo, const double **Z, const int *list,
 
     an = t2 - t1 + 1;
     if (an <= ainfo->nc) {
-	return 1; 
+	/* insufficient observations */
+	return E_DF; 
     }
 
 #if SAMPLE_DEBUG
@@ -881,37 +881,36 @@ arima_difference (const double *y, struct arma_info *ainfo)
 	if (ainfo->d > 0) {
 	    dy[t] -= y[t-1];
 	} 
-	if (ainfo->d > 1) {
-	    dy[t] -= y[t-1];
-	    dy[t] += y[t-2];
+	if (ainfo->d == 2) {
+	    dy[t] -= y[t-1] - y[t-2];
 	}
 	if (ainfo->D > 0) {
 	    dy[t] -= y[t-s];
 	    if (ainfo->d > 0) {
 		dy[t] += y[t-s-1];
 	    }
-	    if (ainfo->d > 1) {
-		dy[t] += y[t-s-1];
-		dy[t] -= y[t-2*s];
+	    if (ainfo->d == 2) {
+		dy[t] += y[t-s-1] - y[t-s-2];
 	    }	    
 	} 
-	if (ainfo->D > 1) {
-	    dy[t] -= y[t-s];
-	    dy[t] += y[t-2*s];
+	if (ainfo->D == 2) {
+	    dy[t] -= y[t-s] - y[t-2*s];
 	    if (ainfo->d > 0) {
-		dy[t] -= y[t-s];
-		dy[t] += y[t-s-1];
-		dy[t] -= y[t-2*s-1];
+		dy[t] += y[t-s-1] - y[t-2*s-1];
 	    }
-	    if (ainfo->d > 1) {
-		dy[t] += 2 * y[t-s-1];
-		dy[t] -= 2 * y[t-s-2];
-		dy[t] -= y[t-2*s-1];
-		dy[t] += y[t-2*s-2];
+	    if (ainfo->d == 2) {
+		dy[t] += y[t-s-1] - y[t-2*s-1];
+		dy[t] -= y[t-s-2] - y[t-2*s-2];
 	    }
 	}
     }
 
+#if ARMA_DEBUG > 1
+    for (t=0; t<ainfo->T; t++) {
+	fprintf(stderr, "dy[%d] = % 12.5f\n", t, dy[t]);
+    }
+#endif    
+
     ainfo->dy = dy;
 
     return 0;
