@@ -275,8 +275,9 @@ static int check_file_type (FILE *fp)
     return err;
 }
 
-int wf1_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
-		  PRN *prn)
+int wf1_get_data (const char *fname, 
+		  double ***pZ, DATAINFO *pdinfo,
+		  gretlopt opt, PRN *prn)
 {
     FILE *fp;
     double **newZ = NULL;
@@ -333,7 +334,7 @@ int wf1_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 	    pputs(prn, _("warning: some variable names were duplicated\n"));
 	}
 
-	err = merge_or_replace_data(pZ, ppdinfo, &newZ, &newinfo, prn);
+	err = merge_or_replace_data(pZ, pdinfo, &newZ, &newinfo, opt, prn);
     }
 
     fclose(fp);
