@@ -17,9 +17,7 @@
  * 
  */
 
-/*
-  Based on the Gnumeric excel plugin by Michael Meeks.
-*/
+/* Originally based on the Gnumeric excel plugin by Michael Meeks */
 
 #include <gtk/gtk.h>
 
@@ -27,9 +25,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <errno.h>
 
 #include "libgretl.h"
 #include "version.h"
+#include "csvdata.h"
 
 #include "importer.h"
 #include "biff.h"
@@ -201,17 +201,24 @@ static double biff_get_rk (const unsigned char *ptr)
 static char *convert8to7 (const char *s, int count) 
 {
     char *dest;
+    int n = strspn(s, " \t");
 
-    if (count > VNAMELEN - 1) {
-	count = VNAMELEN - 1;
+    count -= n;
+
+    if (count <= 0) {
+	dest = g_strdup("");
+    } else {
+	if (count > VNAMELEN - 1) {
+	    count = VNAMELEN - 1;
+	}
+	dest = malloc(VNAMELEN);
+	*dest = '\0';
+	s += n;
+	strncat(dest, s, count);
+	iso_to_ascii(dest);
+	tailstrip(dest);
     }
 
-    dest = malloc(VNAMELEN);
-    *dest = '\0';
-    strncat(dest, s, count);
-    iso_to_ascii(dest);
-    top_n_tail(dest);
-
     dprintf("convert8to7: returning '%s'\n", dest);
 
     return dest;
@@ -301,6 +308,7 @@ copy_unicode_string (unsigned char *src, int remlen,
 	/* let's not mess with excessive strings */
 	return g_strdup("bigstr");
     } else if (csize == 1) {
+	dprintf("original string = '%s'\n", src + this_skip);
 	return convert8to7((char *) src + this_skip, count);
     } else { 
 	return convert16to7(src + this_skip, count);
@@ -341,30 +349,48 @@ static int row_col_err (int row, int col, PRN *prn)
     return err;
 }
 
+/* This function is called on LABELSST records: we check for possible
+   NAs and also for numeric values that may have strayed into the
+   string table, and that shouldn't really be treated as quoted
+   strings.  We don't just use strtod at the outset, because some
+   XLS files (put out by agencies that should know better!) contain
+   numerical values that are "nicely formatted" as text using spaces
+   or commas for thousands separators -- we try stripping this
+   junk out first before doing the numeric-value test.
+*/
+
 static int check_copy_string (struct sheetrow *prow, int row, int col, 
 			      int idx, const char *s)
 {
+    dprintf("inspecting sst[%d] = '%s'\n", idx, s);
+    
     if (row > 0 && col > 0) {
+	const char *numok = "0123456789 -,.";
 	int i, len = strlen(s);
-	int commas = 0;
+	int commas = 0, digits = 0;
 	static int warned = 0;
 
 	if (len == 0) {
-	    dprintf("converting sst[%d] '%s' to NA\n", idx, s);
-	    prow->cells[col] = g_strdup("NA");
+	    dprintf(" converting to NA\n");
+	    prow->cells[col] = g_strdup("-999");
 	    return 0;
-	}	    
+	}
 
 	for (i=0; i<len; i++) {
-	    if (!isdigit(s[i]) && s[i] != ' ' && s[i] != '-' &&
-		s[i] != ',' && s[i] != '.') {
+	    if (strchr(numok, s[i]) == NULL) {
+		/* does not look promising for numerical value */
 		len = 0;
 		break;
 	    }
-	    if (s[i] == ',') commas++;
+	    if (isdigit(s[i])) {
+		digits++;
+	    } else if (s[i] == ',') {
+		commas++;
+	    }
 	}
 
-	if (len > 0) {
+	if (len > 0 && digits > 0) {
+	    /* may be numerical? */
 	    char *p, *q = malloc(len + 1);
 
 	    if (q == NULL) return 1;
@@ -381,19 +407,27 @@ static int check_copy_string (struct sheetrow *prow, int row, int col,
 			warned = 1;
 		    }
 		    if (len - i != 4) {
-			/* probably decimal? */
+			/* comma is probably decimal separator? */
 			*p++ = '.';
 		    }
 		}
 	    }
 	    *p = '\0';
-	    dprintf("converting sst[%d] '%s' to numeric as %s\n", idx, s, q);
-	    prow->cells[col] = q;
-	    return 0;
+
+	    /* If we don't do a rigorous check on q, as below, we
+	       may end up with zeros where there should be NAs.
+	    */ 
+	    if (numeric_string(q)) {
+		dprintf(" taking '%s' to be numeric string: %s\n", s, q);
+		prow->cells[col] = q;
+		return 0;
+	    } else {
+		free(q);
+	    }
 	} 
     }
 
-    dprintf("copying sst[%d] '%s' into place\n", idx, s);
+    dprintf(" copying '%s' into place as string\n", s);
     prow->cells[col] = g_strdup_printf("\"%s", s);
 
     return 0;
@@ -444,7 +478,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	if (row_col_err(i, j, prn)) {
 	    return 1;
 	}
-	if (q->ls_op == BIFF_NUMBER || q->ls_op == BIFF_RK) {
+	if (q->ls_op == BIFF_NUMBER || q->ls_op == BIFF_RK || q->ls_op == BIFF_MULRK) {
 	    guint16 xfref = EX_GETXF(q);
 	    int fmt = wbook_find_format(book, xfref);
 
@@ -633,11 +667,11 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 		    prow->cells[j] = g_strdup((ptr[2])? "1" : "0");
 		} else if (fcode == 0x2 || fcode == 0x3) {
 		    /* error code or empty */
-		    prow->cells[j] = g_strdup("-999.0");
+		    prow->cells[j] = g_strdup("-999");
 		} else {
 		    fprintf(stderr, "Bad formula code 0x%u\n", 
 			    (unsigned) fcode);
-		    prow->cells[j] = g_strdup("-999.0");
+		    prow->cells[j] = g_strdup("-999");
 		}
 	    } else {
 		/* floating-point */
@@ -645,7 +679,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 		dprintf(" floating-point value = %g\n", val);
 		if (isnan(val)) {
 		    fprintf(stderr, "Got a NaN\n");
-		    prow->cells[j] = g_strdup("-999.0");
+		    prow->cells[j] = g_strdup("-999");
 		} else {
 		    prow->cells[j] = g_strdup_printf("%.15g", val);
 		}
@@ -957,12 +991,15 @@ static int first_col_strings (wbook *book)
     int startrow = book->row_offset + 1;
     int ret = 1;
 
+    dprintf("checking for first column strings...\n");
+
     for (i=startrow; i<nrows; i++) {
 	dprintf("book->row_offset=%d, i=%d\n", book->row_offset, i);
 	dprintf("rows = %p\n", (void *) rows);
 	if (rows == NULL || rows[i].cells == NULL || 
 	    rows[i].cells[j] == NULL ||
 	    !IS_STRING(rows[i].cells[j])) {
+	    dprintf("no: not a string at row %d\n", i);
 	    ret = 0;
 	    break;
 	}
@@ -1006,7 +1043,7 @@ check_all_varnames (wbook *book, int totcols, const char *blank_col)
     int vnames = 0;
     int ret = VARNAMES_NONE;
 
-    if (book_obs_labels(book)) {
+    if (book_obs_labels(book) || book_numeric_dates(book)) {
 	startcol++;
 	gotcols = 1;
     }
@@ -1071,23 +1108,9 @@ check_all_varnames (wbook *book, int totcols, const char *blank_col)
 
 static int missval_string (const char *s)
 {
-    if (s + 1 == 0) {
-	return 1;
-    } else {
-	char test[6] = {0};
+    s++;
 
-	strncat(test, s + 1, 4);
-	tailstrip(test);
-	lower(test);
-	if (!strcmp(test, "na") || 
-	    !strcmp(test, "n.a.") ||
-	    !strcmp(test, "..") ||
-	    !strcmp(test, "?")) {
-	    return 1;
-	}
-    }
-
-    return 0;
+    return (*s == '\0' || import_na_string(s));
 }
 
 struct string_err {
@@ -1106,7 +1129,7 @@ check_data_block (wbook *book, int totcols, const char *blank_col,
     int startrow = book->row_offset + 1;
     int j, i, ret = 0;
 
-    if (book_obs_labels(book)) {
+    if (book_obs_labels(book) || book_numeric_dates(book)) {
 	startcol++;
     }
 
@@ -1128,12 +1151,12 @@ check_data_block (wbook *book, int totcols, const char *blank_col,
 		ret = -1;
 	    } else if (rows[i].cells[j] == NULL) {
 		dprintf("data_block: rows[%d].cells[%d] = NULL\n", i, j);
-		rows[i].cells[j] = g_strdup("-999.0");
+		rows[i].cells[j] = g_strdup("-999");
 		ret = -1;
 	    } else if (IS_STRING(rows[i].cells[j])) {
 		if (missval_string(rows[i].cells[j])) {
 		    free(rows[i].cells[j]);
-		    rows[i].cells[j] = g_strdup("-999.0");
+		    rows[i].cells[j] = g_strdup("-999");
 		    ret = -1;
 		} else {
 		    err->row = i + 1;
@@ -1161,6 +1184,7 @@ n_vars_from_col (wbook *book, int totcols, char *blank_col)
     int i, nv = 1;
 
     if (book_time_series(book) || book_obs_labels(book)) {
+	/* got a first non-data column */
 	offset++;
     }
 
@@ -1180,12 +1204,12 @@ static int transcribe_data (wbook *book, double **Z, DATAINFO *pdinfo,
     int roff = book->row_offset;
     int i, t, j = 1;
 
-    if (book_time_series(book) || book_obs_labels(book)) {
+    if (book_obs_labels(book) || book_time_series(book)) {
 	startcol++;
     } 
 
-    for (i=startcol; i<=totcols; i++) {
-	int ts, missing = 0;
+    for (i=startcol; i<totcols; i++) { /* was i<=totcols */
+	int ts;
 
 	if (blank_col[i]) {
 	    continue;
@@ -1209,15 +1233,7 @@ static int transcribe_data (wbook *book, double **Z, DATAINFO *pdinfo,
 	dprintf("set varname[%d] = '%s'\n", j, pdinfo->varname[j]);
 
 	for (t=0; t<pdinfo->n; t++) {
-	    if (book->missmask != NULL) {
-		while (book->missmask[t]) {
-		    Z[j][t++] = NADBL;
-		    missing++;
-		}
-	    }
-
-	    ts = t + 1 + roff - missing;
-
+	    ts = t + 1 + roff;
 	    if (rows[ts].cells == NULL || i >= rows[ts].end ||
 		rows[ts].cells[i] == NULL) {
 		continue;
@@ -1229,7 +1245,7 @@ static int transcribe_data (wbook *book, double **Z, DATAINFO *pdinfo,
 		    "= '%s'\n", j, t, i, ts, rows[ts].cells[i]);
 
 	    Z[j][t] = atof(rows[ts].cells[i]);
-	    if (Z[j][t] == -999.0) {
+	    if (Z[j][t] == -999 || Z[j][t] == -9999) {
 		Z[j][t] = NADBL;
 	    }
 	}
@@ -1241,8 +1257,8 @@ static int transcribe_data (wbook *book, double **Z, DATAINFO *pdinfo,
 }
 
 static int 
-get_sheet_dimensions (int *totcols, int *datacols, char **blank_col,
-		      PRN *prn)
+get_sheet_dimensions (wbook *book, int *totcols, int *datacols, 
+		      char **blank_col, PRN *prn)
 {
     char *blanks = NULL;
     int i, j;
@@ -1300,8 +1316,12 @@ get_sheet_dimensions (int *totcols, int *datacols, char **blank_col,
 	}
     }
 
-    printf("rows=%d, data cols=%d total cols=%d\n", nrows, 
-	   *datacols, *totcols);
+    if (book_numeric_dates(book)) {
+	*datacols -= 1;
+    }
+
+    printf("rows=%d, total cols=%d, data cols=%d\n", nrows, 
+	   *totcols, *datacols);
 
     if (*datacols < 1) {
 	pputs(prn, _("No data found.\n"));
@@ -1314,20 +1334,40 @@ get_sheet_dimensions (int *totcols, int *datacols, char **blank_col,
     return 0;
 }
 
-static void book_time_series_setup (wbook *book, DATAINFO *newinfo, int pd)
+static int col0_is_numeric (int nrows, int row_offset, int col_offset)
 {
-    const char *s = rows[1 + book->row_offset].cells[book->col_offset];
+    int t, tstart = 1 + row_offset;
+    int nx = 0;
+    char *test;
 
-    if (book_numeric_dates(book)) {
-	int d0 = atoi(s);
+    fprintf(stderr, "testing for all numerical values in col %d\n", 
+	    col_offset);
 
-	MS_excel_date_string(newinfo->stobs, d0, pd, book_base_1904(book));
-    } else {
-	if (*s == '"' || *s == '\'') s++;
-	strcpy(newinfo->stobs, s);
-	colonize_obs(newinfo->stobs);
+    for (t=tstart; t<nrows; t++) {
+	test = rows[t].cells[col_offset];
+	if (!numeric_string(test)) {
+	    fprintf(stderr, " no: non-numeric cell at row %d\n", t + 1);
+	    return 0;
+	} else if (test != NULL && *test != '\0') {
+	    nx++;
+	}
     }
 
+    return nx > 0;
+}
+
+static int alpha_cell (const char *s)
+{
+    if (s != NULL) {
+	if (*s == '"' || *s == '\'') s++;
+	return isalpha(*s);
+    }
+
+    return 0;
+}
+
+static void book_time_series_setup (wbook *book, DATAINFO *newinfo, int pd)
+{
     newinfo->pd = pd;
     newinfo->structure = TIME_SERIES;
 
@@ -1339,10 +1379,11 @@ static void book_time_series_setup (wbook *book, DATAINFO *newinfo, int pd)
     book_unset_obs_labels(book);
 }
 
-static int 
-real_excel_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
-		     int gui, PRN *prn)
+int xls_get_data (const char *fname, int *list, char *sheetname,
+		  double ***pZ, DATAINFO *pdinfo,
+		  gretlopt opt, PRN *prn)
 {
+    int gui = (opt & OPT_G);
     wbook xbook;
     wbook *book = &xbook;
     double **newZ = NULL;
@@ -1350,6 +1391,9 @@ real_excel_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
     int datacols, totcols;
     struct string_err strerr;
     char *blank_col = NULL;
+    int ts_markers = 0;
+    char **ts_S = NULL;
+    int r0, c0;
     int i, t, pd = 0;
     int err = 0;
 
@@ -1359,9 +1403,13 @@ real_excel_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 	return 1;
     }
 
+    if (sheetname != NULL) {
+	fprintf(stderr, "xls_get_data: sheetname='%s'\n", sheetname);
+    }
+
     gretl_push_c_numeric_locale();
 
-    wbook_init(book);
+    wbook_init(book, list, sheetname);
 
     if (excel_book_get_info(fname, book)) {
 	pputs(prn, _("Failed to get workbook info"));
@@ -1373,11 +1421,18 @@ real_excel_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 	wbook_print_info(book);
     }
 
-    if (!err && gui) {
-	wsheet_menu(book, book->nsheets > 1);
-	if (book_debugging(book)) {
-	    debug_print = 1;
-	    print_version();
+    if (!err) {
+	if (gui) {
+	    wsheet_menu(book, book->nsheets > 1);
+	    if (book_debugging(book)) {
+		debug_print = 1;
+		print_version();
+	    }
+	} else {
+	    err = wbook_check_params(book);
+	    if (err) {
+		gretl_errmsg_set(_("Invalid argument for worksheet import"));
+	    }
 	}
     }
 
@@ -1406,7 +1461,7 @@ real_excel_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
     }
 
     /* get sizes and locate any blank columns */
-    err = get_sheet_dimensions(&totcols, &datacols, &blank_col, prn);
+    err = get_sheet_dimensions(book, &totcols, &datacols, &blank_col, prn);
 
     if (err) goto getout;
 
@@ -1423,6 +1478,8 @@ real_excel_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 
     if (first_col_strings(book)) {
 	puts("found label strings in first imported column");
+    } else if (book_numeric_dates(book)) {
+	puts("found calendar dates in first imported column");
     } else {
 	puts("check for label strings in first imported column: not found");
     }
@@ -1457,28 +1514,29 @@ real_excel_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
     } else if (err == -1) {
 	pputs(prn, _("Warning: there were missing values\n"));
 	err = 0;
-    }	    
+    }
 
-    /* do we have a first column containing dates? */
-    if (book_numeric_dates(book)) {
-	pd = pd_from_numeric_dates(nrows, book->row_offset, book->col_offset, 
-				   NULL, book);
-    } else if (!book_auto_varnames(book)) {
-	int r0 = book->row_offset;
-	int c0 = book->col_offset;
-	
-	if (obs_column_heading(rows[r0].cells[c0])) {
-	    pd = consistent_date_labels(nrows, r0, c0, NULL);
+    r0 = book->row_offset;
+    c0 = book->col_offset;
+    newinfo->n = nrows - 1 - book->row_offset;
+
+    if (book_numeric_dates(book) || 
+	(!book_auto_varnames(book) && import_obs_label(rows[r0].cells[c0]))) {
+	pd = importer_dates_check(book->row_offset + 1, book->col_offset, 
+				  book->flags, NULL, newinfo, prn, &err);
+	if (pd > 0) {
+	    /* got time-series info from dates/labels */
+	    book_time_series_setup(book, newinfo, pd);
+	    ts_markers = newinfo->markers;
+	    ts_S = newinfo->S;
+	} else if (!book_numeric_dates(book) && alpha_cell(rows[r0].cells[c0]) && 
+	    col0_is_numeric(nrows, r0, c0)) {
+	    book_unset_obs_labels(book);
 	}
     }
 
-    if (pd) {
-	book_time_series_setup(book, newinfo, pd);
-    }    
-
-    /* number of variables and observations for import dataset */
+    /* dimensions of the dataset */
     newinfo->v = n_vars_from_col(book, totcols, blank_col);
-    newinfo->n = nrows - 1 - book->row_offset + book->totmiss;
     fprintf(stderr, "newinfo->v = %d, newinfo->n = %d\n",
 	    newinfo->v, newinfo->n);
 
@@ -1489,7 +1547,8 @@ real_excel_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
     }
 
     if (book_time_series(book)) {
-	ntodate_full(newinfo->endobs, newinfo->n - 1, newinfo);
+	newinfo->markers = ts_markers;
+	newinfo->S = ts_S;
     } else {
 	dataset_obs_info_default(newinfo);
     } 
@@ -1513,7 +1572,11 @@ real_excel_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 	}
     }
 
-    err = merge_or_replace_data(pZ, ppdinfo, &newZ, &newinfo, prn);
+    err = merge_or_replace_data(pZ, pdinfo, &newZ, &newinfo, opt, prn);
+
+    if (!err && gui) {
+	wbook_record_params(book, list);
+    }
 
  getout:
     
@@ -1536,16 +1599,5 @@ real_excel_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
     return err;
 }  
 
-int excel_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
-		    PRN *prn)
-{
-    return real_excel_get_data(fname, pZ, ppdinfo, 1, prn);
-}
-
-int cli_get_xls (const char *fname, double ***pZ, DATAINFO **ppdinfo,
-		 PRN *prn)
-{
-    return real_excel_get_data(fname, pZ, ppdinfo, 0, prn);
-}
 
 
