@@ -22,6 +22,7 @@
 #include "libgretl.h"
 #include "gretl_xml.h"
 #include "importer.h"
+#include "csvdata.h"
 
 #undef IDEBUG
 
@@ -41,6 +42,7 @@ typedef enum {
 
 static void wsheet_init (wsheet *sheet)
 {
+    sheet->col_offset = sheet->row_offset = 0;
     sheet->maxcol = sheet->maxrow = 0;
     sheet->text_cols = sheet->text_rows = 0;
     sheet->colheads = 0;
@@ -122,58 +124,32 @@ static void wsheet_print_info (wsheet *sheet)
 
 static int wsheet_allocate (wsheet *sheet, int cols, int rows)
 {
-    int i, j, t;
+    int i, t;
 
 #if 1
     fprintf(stderr, "wsheet_allocate: allocating %d variables, each %d obs\n",
 	    cols, rows);
 #endif
 
-    sheet->Z = malloc(cols * sizeof *sheet->Z);
-    if (sheet->Z == NULL) return 1;
+    sheet->Z = doubles_array_new(cols, rows);
+    if (sheet->Z == NULL) {
+	return 1;
+    }
 
     for (i=0; i<cols; i++) {
-	sheet->Z[i] = malloc(rows * sizeof **sheet->Z);
-	if (sheet->Z[i] == NULL) {
-	    for (j=0; j<i; j++) {
-		free(sheet->Z[j]);
-		sheet->Z[j] = NULL;
-	    }
-	    return 1;
-	}
 	for (t=0; t<rows; t++) {
 	    sheet->Z[i][t] = NADBL;
 	}
     }
 
-    sheet->varname = malloc(cols * sizeof *sheet->varname);
-    if (sheet->varname == NULL) return 1;
-
-    for (i=0; i<cols; i++) {
-	sheet->varname[i] = malloc(VNAMELEN * sizeof **sheet->varname);
-	if (sheet->varname[i] == NULL) {
-	    for (j=0; j<i; j++) {
-		free(sheet->varname[j]);
-		sheet->varname[j] = NULL;
-	    }	    
-	    return 1;
-	}
-	sheet->varname[i][0] = '\0';
+    sheet->varname = strings_array_new_with_length(cols, VNAMELEN);
+    if (sheet->varname == NULL) {
+	return 1;
     }
 
-    sheet->label = malloc(rows * sizeof *sheet->label);
-    if (sheet->label == NULL) return 1;
-
-    for (t=0; t<rows; t++) {
-	sheet->label[t] = malloc(VNAMELEN * sizeof *sheet->label[t]);
-	if (sheet->label[t] == NULL) {
-	    for (j=0; j<i; j++) {
-		free(sheet->label[j]);
-		sheet->label[j] = NULL;
-	    }	   
-	    return 1;
-	}
-	sheet->label[t][0] = '\0';
+    sheet->label = strings_array_new_with_length(rows, VNAMELEN);
+    if (sheet->label == NULL) {
+	return 1;
     }
 
     return 0;
@@ -187,7 +163,7 @@ static int wsheet_get_real_size (xmlNodePtr node, wsheet *sheet)
     sheet->maxrow = 0;
     sheet->maxcol = 0;
 
-    while (p) {
+    while (p != NULL) {
 	if (!xmlStrcmp(p->name, (XUC) "Cell")) {
 	    int i, j;
 
@@ -230,16 +206,33 @@ static void check_for_date_format (wsheet *sheet, const char *fmt)
     }
 }
 
+static int stray_numeric (int vtype, char *tmp, double *x)
+{
+    if (vtype == VALUE_STRING) {
+	if (string_is_blank(tmp)) {
+	    *x = NADBL;
+	    return 1;
+	} else if (import_na_string(tmp)) {
+	    *x = NADBL;
+	    return 1;
+	} else if (numeric_string(tmp)) {
+	    *x = atof(tmp);
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
 static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
 {
     xmlNodePtr p = node->xmlChildrenNode;
     char *tmp;
     double x;
-    int i, t, vtype = 0;
-    char *toprows, *leftcols;
+    int vtype = 0;
     int gotlabels = 0;
     int cols, rows;
-    int colmin, rowmin;
+    int i, t, r, c;
     int err = 0;
 
     cols = sheet->maxcol + 1 - sheet->col_offset;
@@ -259,89 +252,102 @@ static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
 	return 1;
     }
 
-    leftcols = calloc(cols, 1);
-    toprows = calloc(rows, 1);
-
-    if (toprows == NULL || leftcols == NULL) {
-	wsheet_free(sheet);
-	return 1;
-    }
-
-    colmin = sheet->col_offset;
-    rowmin = sheet->row_offset;
-
     sheet->colheads = 0;
 
-    while (p && !err) {
+    while (p != NULL && !err) {
 	if (!xmlStrcmp(p->name, (XUC) "Cell")) {
-	    int i_real = 0, t_real = 0;
-
 	    x = NADBL;
-	    i = 0; t = 0;
+	    c = r = 0;
+	    i = t = -1;
 
+	    /* what column are we in? */
 	    tmp = (char *) xmlGetProp(p, (XUC) "Col");
 	    if (tmp) {
-		i = atoi(tmp);
-		i_real = i - colmin;
+		c = atoi(tmp);
+		i = c - sheet->col_offset;
 		free(tmp);
 	    }
+
+	    /* what row are we on? */
 	    tmp = (char *) xmlGetProp(p, (XUC) "Row");
 	    if (tmp) {
-		t = atoi(tmp);
-		t_real = t - rowmin;
+		r = atoi(tmp);
+		t = r - sheet->row_offset;
 		free(tmp);
 	    }
-	    if (i_real >= 0 && t_real >= 0) {
-		tmp = (char *) xmlGetProp(p, (XUC) "ValueType");
 
-		if (tmp) {
-		    vtype = atoi(tmp);
-		    free(tmp);
-		} else { 
-		    /* a formula perhaps? */
-		    pprintf(prn, _("Couldn't get value for col %d, row %d.\n"
-				   "Maybe there's a formula in the sheet?"),
-			    i, t);
-		    err = 1;
+	    if (i < 0 || t < 0) {
+		/* we're not in the requested reading area */
+		p = p->next;
+		continue;
+	    }
+
+	    /* check that we have a value of some type available */
+	    tmp = (char *) xmlGetProp(p, (XUC) "ValueType");
+	    if (tmp) {
+		vtype = atoi(tmp);
+		free(tmp);
+	    } else { 
+		/* a formula perhaps? */
+		pprintf(prn, _("Couldn't get value for col %d, row %d.\n"
+			       "Maybe there's a formula in the sheet?"),
+			c, r);
+		err = 1;
+		break;
+	    }
+
+	    /* get and process the actual cell content */
+	    tmp = (char *) xmlNodeGetContent(p);
+	    if (tmp != NULL) {
+		if (VTYPE_IS_NUMERIC(vtype) || vtype == VALUE_STRING) {
+		    if (i == 0) {
+			/* first column: write content to labels */
+			strncat(sheet->label[t], tmp, OBSLEN - 1);
+		    }
 		}
 
-		if (!err && (tmp = (char *) xmlNodeGetContent(p))) {
-		    if (VTYPE_IS_NUMERIC(vtype) || vtype == VALUE_STRING) {
-			if (i_real == 0) {
-			    strncat(sheet->label[t_real], tmp, OBSLEN - 1);
-			}
+		if (i == 0 && t == 1 && VTYPE_IS_NUMERIC(vtype)) {
+		    char *fmt = (char *) xmlGetProp(p, (XUC) "ValueFormat");
+
+		    if (fmt) {
+			check_for_date_format(sheet, fmt);
+			free(fmt);
 		    }
+		}
 
-		    if (i_real == 0 && t_real == 1 && VTYPE_IS_NUMERIC(vtype)) {
-			char *fmt = (char *) xmlGetProp(p, (XUC) "ValueFormat");
-
-			if (fmt) {
-			    check_for_date_format(sheet, fmt);
-			    free(fmt);
-			}
-		    }
-
-		    if (VTYPE_IS_NUMERIC(vtype)) {
-			x = atof(tmp);
-			sheet->Z[i_real][t_real] = x;
-			toprows[t_real] = leftcols[i_real] = 0;
-		    } else if (vtype == VALUE_STRING) {
-			if (t_real == 0) {
-			    strncat(sheet->varname[i_real], tmp, VNAMELEN - 1);
-			    sheet->colheads += 1;
-			    if (i_real == 0 && !strcmp(tmp, "obs")) {
-				; /* keep going */
-			    } else if (check_varname(sheet->varname[i_real])) {
+		if (VTYPE_IS_NUMERIC(vtype)) {
+		    x = atof(tmp);
+		    sheet->Z[i][t] = x;
+		} else if (i > 0 && stray_numeric(vtype, tmp, &x)) {
+		    sheet->Z[i][t] = x;
+		} else if (vtype == VALUE_STRING) {
+		    if (t == 0) {
+			/* first row: look for varnames */
+			strncat(sheet->varname[i], tmp, VNAMELEN - 1);
+			sheet->colheads += 1;
+			if (i == 0 && import_obs_label(tmp)) {
+			    ; /* keep going */
+			} else {
+			    charsub(sheet->varname[i], ' ', '_');
+			    if (check_varname(sheet->varname[i])) {
 				invalid_varname(prn);
 				err = 1;
 			    }
-			} else if (i_real == 0) {
+			}
+		    } else if (i == 0) {
+			/* first column, not first row */
+			if (!gotlabels) {
 			    gotlabels = 1;
 			}
-			toprows[t_real] = leftcols[i_real] = 1;
+			sheet->text_cols = 1;
+		    } else {
+			pprintf(prn, _("Expected numeric data, found string:\n"
+				       "'%s' at row %d, column %d\n"), 
+				tmp, r, c);
+			err = 1;
 		    }
-		    free(tmp);
 		}
+		free(tmp);
 	    }
 	}
 	p = p->next;
@@ -349,39 +355,12 @@ static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
 
     if (gotlabels && sheet->colheads == 1) {
 	/* rough notion here: if there's only one heading, it's
-	   probably not really a variable name */
+	   probably not really a variable name, but rather
+	   a first observation label 
+	*/
 	sheet->colheads = 0;
     }
 
-    if (!err) {
-	for (i=0; i<cols; i++) {
-	    if (leftcols[i]) {
-		sheet->text_cols += 1;
-	    }
-	}
-
-	for (t=0; t<rows; t++) {
-	    if (toprows[t]) {
-		sheet->text_rows += 1;
-	    }
-	}
-
-	if (sheet->text_rows > 1) {
-	    pputs(prn, _("Found an extraneous row of text"));
-	    pputc(prn, '\n');
-	    err = 1;
-	}
-
-	if (sheet->text_cols > 1) {
-	    pputs(prn, _("Found an extraneous column of text"));
-	    pputc(prn, '\n');
-	    err = 1;
-	}
-    }
-
-    free(toprows);
-    free(leftcols);
-
     return err;
 }
 
@@ -431,6 +410,7 @@ static int wsheet_get_data (const char *fname, wsheet *sheet, PRN *prn)
 			    sheetcount++;
 			    tmp = (char *) xmlNodeGetContent(snode);
 			    if (tmp) {
+				tailstrip(tmp);
 				if (!strcmp(tmp, sheet->name) &&
 				    sheetcount == sheet->ID + 1) {
 				    got_sheet = 1;
@@ -466,6 +446,7 @@ static int wsheet_get_data (const char *fname, wsheet *sheet, PRN *prn)
     xmlCleanupParser();
 
     if (!got_sheet) {
+	fprintf(stderr, "'%s': couldn't find the requested sheet\n", sheet->name);
 	err = 1;
     }
 
@@ -485,11 +466,14 @@ static int wbook_record_name (char *name, wbook *book)
     book->sheetnames = sheetnames;
     book->nsheets = ns;
     book->sheetnames[ns - 1] = name;
+    tailstrip(name);
 
     return 0;
 }
 
-static int wbook_get_info (const char *fname, wbook *book, PRN *prn) 
+static int wbook_get_info (const char *fname, const int *list,
+			   char *sheetname, wbook *book, 
+			   PRN *prn) 
 {
     xmlDocPtr doc;
     xmlNodePtr cur, sub;
@@ -499,7 +483,7 @@ static int wbook_get_info (const char *fname, wbook *book, PRN *prn)
     LIBXML_TEST_VERSION 
 	xmlKeepBlanksDefault(0);
 
-    wbook_init(book);
+    wbook_init(book, list, sheetname);
 
     doc = xmlParseFile(fname);
     if (doc == NULL) {
@@ -551,13 +535,11 @@ static int wbook_get_info (const char *fname, wbook *book, PRN *prn)
 static int wsheet_setup (wsheet *sheet, wbook *book, int n)
 {
     sheet->name = gretl_strdup(book->sheetnames[n]);
-
     if (sheet->name == NULL) {
 	return 1;
     }
 
     sheet->ID = n;
-
     sheet->col_offset = book->col_offset;
     sheet->row_offset = book->row_offset;    
     
@@ -580,51 +562,9 @@ static int wsheet_labels_complete (wsheet *sheet)
     return ret;
 }
 
-static int rigorous_dates_check (wsheet *sheet, DATAINFO *pdinfo)
-{
-    int t, rows = sheet->maxrow + 1 - sheet->row_offset;
-    int startrow = 1 + sheet->row_offset;
-    int n, nbak = 0, err = 0;
-
-    fprintf(stderr, "Doing rigorous dates check: pd = %d, stobs = '%s'\n", 
-	    pdinfo->pd, pdinfo->stobs);
-
-    for (t=startrow; t<rows; t++) {
-	n = dateton(sheet->label[t], pdinfo);
-	if (t > startrow && n != nbak + 1) {
-	    fprintf(stderr, "problem: date[%d]='%s' (%d) but date[%d]='%s' (%d)\n",
-		    t - startrow + 1, sheet->label[t], n,
-		    t - startrow, sheet->label[t-1], nbak);
-	    err = 1;
-	    break;
-	}
-	nbak = n;
-    }    
-
-    return err;
-}
-
 static void 
 sheet_time_series_setup (wsheet *sheet, wbook *book, DATAINFO *newinfo, int pd)
 {
-    const char *s;
-
-    if (sheet->colheads == 0) {
-	s = sheet->label[0];
-    } else {
-	s = sheet->label[1];
-    }
-
-    if (book_numeric_dates(book)) {
-	int d0 = atoi(s);
-
-	MS_excel_date_string(newinfo->stobs, d0, pd, book_base_1904(book));
-    } else {
-	if (*s == '"' || *s == '\'') s++;
-	strcpy(newinfo->stobs, s);
-	colonize_obs(newinfo->stobs);
-    }
-
     newinfo->pd = pd;
     newinfo->structure = TIME_SERIES;
 
@@ -633,15 +573,45 @@ sheet_time_series_setup (wsheet *sheet, wbook *book, DATAINFO *newinfo, int pd)
     fprintf(stderr, "sd0=%g\n", newinfo->sd0);
 
     sheet->text_cols = 1;
-
     book_set_time_series(book);
     book_unset_obs_labels(book);
 }
 
-static int 
-real_wbook_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
-		     int gui, PRN *prn)
+/* check that a given column doesn't contain all NAs */
+
+static int column_is_blank (wsheet *sheet, int k, int n)
 {
+    int t, s = (sheet->colheads)? 1 : 0;
+
+    for (t=0; t<n; t++) {
+	if (!na(sheet->Z[k][s++])) {
+	    return 0;
+	}
+    }
+
+    return 1;
+}
+
+/* first column labels: are they all numeric strings? */
+
+static int labels_numeric (char **S, int n)
+{
+    int t;
+
+    for (t=0; t<n; t++) {
+	if (!numeric_string(S[t])) {
+	    return 0;
+	}
+    }  
+
+    return 1;
+}
+
+int gnumeric_get_data (const char *fname, int *list, char *sheetname,
+		       double ***pZ, DATAINFO *pdinfo,
+		       gretlopt opt, PRN *prn)
+{
+    int gui = (opt & OPT_G);
     wbook gbook;
     wbook *book = &gbook;
     wsheet gsheet;
@@ -661,7 +631,7 @@ real_wbook_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 
     gretl_push_c_numeric_locale();
 
-    if (wbook_get_info(fname, book, prn)) {
+    if (wbook_get_info(fname, list, sheetname, book, prn)) {
 	pputs(prn, _("Failed to get workbook info"));
 	err = 1;
 	goto getout;
@@ -683,6 +653,15 @@ real_wbook_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 	    wsheet_menu(book, 0);
 	    sheetnum = 0;
 	}
+    } else {
+	err = wbook_check_params(book);
+	if (err) {
+	    gretl_errmsg_set(_("Invalid argument for worksheet import"));
+	} else if (book->selected >= 0) {
+	    sheetnum = book->selected;
+	} else {
+	    sheetnum = 0;
+	}
     }
 
     if (book->selected == -1) {
@@ -707,10 +686,12 @@ real_wbook_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
     if (err) {
 	goto getout;
     } else {
-	int nrows = sheet->maxrow + 1 - sheet->row_offset;
-	int r0 = sheet->row_offset;
-	int i, j, t, i_sheet;
+	int r0 = 1;
+	int i, j, t;
+	int ts_markers = 0;
+	char **ts_S = NULL;
 	int blank_cols = 0;
+	int missvals = 0;
 	int pd = 0;
 
 	if (sheet->text_cols > 0) {
@@ -719,32 +700,42 @@ real_wbook_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 
 	if (sheet->colheads == 0) {
 	    book_set_auto_varnames(book);
-	    if (sheet->row_offset > 0) {
-		r0--;
-	    }
+	    r0 = 0;
 	}
 
-	if (book != NULL && book_numeric_dates(book)) {
-	    pd = pd_from_numeric_dates(nrows, r0, 0, sheet->label, book);
-	} else if (sheet->colheads > 0) {
-	    if (obs_column_heading(sheet->label[0])) {
-		pd = consistent_date_labels(nrows, r0, 0, sheet->label);
-	    }
+	if (book_numeric_dates(book)) {
+	    puts("found calendar dates in first imported column");
+	} else if (sheet->text_cols > 0) {
+	    puts("found label strings in first imported column");
+	} else {
+	    puts("check for label strings in first imported column: not found");
 	}
 
-	if (pd) {
-	    sheet_time_series_setup(sheet, book, newinfo, pd);
-	    if (!book_numeric_dates(book)) {
-		rigorous_dates_check(sheet, newinfo);
-	    }
-	}	
+	newinfo->n = sheet->maxrow - sheet->row_offset;
 
-	newinfo->v = sheet->maxcol + 2 - sheet->col_offset - sheet->text_cols;
-	newinfo->n = sheet->maxrow - sheet->row_offset + book->totmiss;
-	if (sheet->colheads == 0) {
+	if (!sheet->colheads) {
+	    pputs(prn, _("it seems there are no variable names\n"));
 	    newinfo->n += 1;
 	}
 
+	if (book_numeric_dates(book) || 
+	    (sheet->colheads > 0 && import_obs_label(sheet->label[0]))) {
+	    pd = importer_dates_check(r0, 0, book->flags, sheet->label,
+				      newinfo, prn, &err);
+	    if (pd > 0) {
+		/* got time-series info from dates/labels */
+		sheet_time_series_setup(sheet, book, newinfo, pd);
+		ts_markers = newinfo->markers;
+		ts_S = newinfo->S;
+	    } else if (!book_numeric_dates(book)) {
+		if (labels_numeric(sheet->label, newinfo->n)) {
+		    sheet->text_cols = 0;
+		    book_unset_obs_labels(book);
+		}
+	    }
+	}
+
+	newinfo->v = sheet->maxcol + 2 - sheet->col_offset - sheet->text_cols;
 	fprintf(stderr, "newinfo->v = %d, newinfo->n = %d\n",
 		newinfo->v, newinfo->n);
 
@@ -754,40 +745,40 @@ real_wbook_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 	}
 
 	if (book_time_series(book)) {
-	    ntodate_full(newinfo->endobs, newinfo->n - 1, newinfo);
-	    fprintf(stderr, "endobs='%s'\n", newinfo->endobs);
+	    newinfo->markers = ts_markers;
+	    newinfo->S = ts_S;
 	} else {
 	    dataset_obs_info_default(newinfo);
 	} 
 
 	j = 1;
-
 	for (i=1; i<newinfo->v; i++) {
-	    i_sheet = i - 1 + sheet->text_cols;
-	    if (sheet->colheads && *sheet->varname[i_sheet] == '\0') {
+	    int s = (sheet->colheads)? 1 : 0;
+	    int k = i - 1 + sheet->text_cols;
+	    double zkt;
+
+	    if (column_is_blank(sheet, k, newinfo->n)) {
 		blank_cols++;
+		continue;
+	    } 
+
+	    if (sheet->colheads && *sheet->varname[k] != '\0') {
+		strcpy(newinfo->varname[j], sheet->varname[k]);
 	    } else {
-		int s = (sheet->colheads)? 1 : 0;
-
-		if (sheet->colheads) {
-		    strcpy(newinfo->varname[j], sheet->varname[i_sheet]);
-		} else {
-		    sprintf(newinfo->varname[j], "v%d", j);
-		}
-		for (t=0; t<newinfo->n; t++) {
-		    if (book->missmask != NULL) {
-			while (book->missmask[t]) {
-			    newZ[j][t++] = NADBL;
-			}
-		    }
-		    newZ[j][t] = sheet->Z[i_sheet][s++];
-		}
-		j++;
+		sprintf(newinfo->varname[j], "v%d", j);
 	    }
-	}
-
-	if (fix_varname_duplicates(newinfo)) {
-	    pputs(prn, _("warning: some variable names were duplicated\n"));
+	    for (t=0; t<newinfo->n; t++) {
+		zkt = sheet->Z[k][s++];
+		if (zkt == -999 || zkt == -9999) {
+		    newZ[j][t] = NADBL;
+		} else {
+		    newZ[j][t] = zkt;
+		}
+		if (na(newZ[j][t])) {
+		    missvals = 1;
+		}
+	    }
+	    j++;
 	}
 
 	if (blank_cols > 0) {
@@ -796,6 +787,14 @@ real_wbook_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 	    dataset_drop_last_variables(blank_cols, &newZ, newinfo);
 	}
 
+	if (missvals) {
+	    pputs(prn, _("Warning: there were missing values\n"));
+	}
+
+	if (fix_varname_duplicates(newinfo)) {
+	    pputs(prn, _("warning: some variable names were duplicated\n"));
+	}
+
 	if (book_obs_labels(book) && wsheet_labels_complete(sheet)) {
 	    int offset = (sheet->colheads)? 1 : 0;
 
@@ -807,7 +806,11 @@ real_wbook_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 	    }
 	}
 
-	err = merge_or_replace_data(pZ, ppdinfo, &newZ, &newinfo, prn);
+	err = merge_or_replace_data(pZ, pdinfo, &newZ, &newinfo, opt, prn);
+
+	if (!err && gui) {
+	    wbook_record_params(book, list);
+	}
     } 
 
  getout:
@@ -824,15 +827,3 @@ real_wbook_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
     return err;
 }
 
-int wbook_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
-		    PRN *prn)
-{
-    return real_wbook_get_data(fname, pZ, ppdinfo, 1, prn);
-}
-
-int cli_get_gnumeric (const char *fname, double ***pZ, DATAINFO **ppdinfo,
-		      PRN *prn)
-{
-    return real_wbook_get_data(fname, pZ, ppdinfo, 0, prn);
-}
-
