@@ -53,7 +53,7 @@ struct h_container_ {
     gretl_vector *gama;		/* selection eq. parameters */
     double sigma;
     double rho;
-    double lambda;		/* rho*sigma by definiton */
+    double lambda;		/* rho*sigma by definition */
 
     gretl_matrix *vcv;		/* Variance-covariance matrix */
     gretl_matrix *VProbit;	/* 1st stage probit covariance matrix */
@@ -284,14 +284,21 @@ static int h_container_fill (h_container *HC, const int *Xl,
     HC->ksel = probmod->ncoeff;
 
     HC->beta = gretl_column_vector_alloc(HC->kmain);
-    HC->gama = gretl_coeff_vector_from_model(probmod, NULL);
-    if (HC->beta == NULL || HC->gama == NULL) {
+    if (HC->beta == NULL) {
 	return E_ALLOC;
     }
 
+    HC->gama = gretl_coeff_vector_from_model(probmod, NULL, &err);
+    if (err) {
+	gretl_matrix_free(HC->beta);
+	HC->beta = NULL;
+	return err;
+    }
+
     for (i=0; i<HC->kmain; i++) {
 	gretl_vector_set(HC->beta, i, olsmod->coeff[i]);
     }
+
     HC->lambda = olsmod->coeff[HC->kmain];
 
     /*
@@ -417,13 +424,15 @@ static int h_container_fill (h_container *HC, const int *Xl,
     }
 
     if (!err) {
-	HC->fitted = gretl_matrix_alloc(HC->nunc, 1);
-	HC->u = gretl_matrix_alloc(HC->nunc, 1);
-	HC->ndx = gretl_matrix_alloc(HC->ntot, 1);
-	HC->VProbit = gretl_vcv_matrix_from_model(probmod, NULL);
+	HC->VProbit = gretl_vcv_matrix_from_model(probmod, NULL, &err);
+    }
 
-	if (HC->fitted == NULL || HC->u == NULL || 
-	    HC->ndx == NULL || HC->VProbit == NULL) {
+    if (!err) {
+	HC->fitted = gretl_matrix_alloc(HC->nunc, 1);
+	HC->u      = gretl_matrix_alloc(HC->nunc, 1);
+	HC->ndx    = gretl_matrix_alloc(HC->ntot, 1);
+
+	if (HC->fitted == NULL || HC->u == NULL || HC->ndx == NULL) {
 	    err = E_ALLOC;
 	}
     }
@@ -946,7 +955,7 @@ static MODEL heckit_init (int *list, double ***pZ, DATAINFO *pdinfo,
 			  h_container *HC) 
 {
 #if HDEBUG
-    PRN *prn = gretl_print_new(GRETL_PRINT_STDOUT);
+    PRN *prn = gretl_print_new(GRETL_PRINT_STDOUT, NULL);
 #endif
     MODEL hm;
     MODEL probmod;
@@ -964,6 +973,11 @@ static MODEL heckit_init (int *list, double ***pZ, DATAINFO *pdinfo,
 	goto bailout;
     } 
 
+    if (Xlist == NULL || Zlist == NULL) {
+	err = E_ARGS;
+	goto bailout;
+    }
+
     err = make_heckit_NA_mask(HC, Xlist, Zlist, (const double **) *pZ, 
 			      pdinfo);
     if (err) {
