@@ -297,8 +297,9 @@ static int read_jmulti_data (FILE *fp, char *line, int len,
     return err;
 }
 
-int jmulti_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
-		     PRN *prn)
+int jmulti_get_data (const char *fname, 
+		     double ***pZ, DATAINFO *pdinfo,
+		     gretlopt opt, PRN *prn)
 {
     int maxlen = 0;
     int nvar = 0, nobs = 0;
@@ -383,7 +384,7 @@ int jmulti_get_data (const char *fname, double ***pZ, DATAINFO **ppdinfo,
 	newinfo->descrip = descrip;
 	descrip = NULL;
 
-	err = merge_or_replace_data(pZ, ppdinfo, &newZ, &newinfo, prn);
+	err = merge_or_replace_data(pZ, pdinfo, &newZ, &newinfo, opt, prn);
     }
 
  bailout:
