@@ -358,6 +358,9 @@ adjust_sample_for_missing (int *sample, int n, const MODEL *pmod)
 }
 
 #define ITERS 500
+#define RESAMPLE_RESIDUALS 0
+
+#if RESAMPLE_RESIDUALS
 
 /* populate dependent var using resampled residuals */
 
@@ -394,6 +397,31 @@ make_data_arrays (MODEL *pmod, double **Z,
     }
 }
 
+#else
+
+/* populate both y and X using resampled data rows */
+
+static void
+make_data_arrays (MODEL *pmod, double **Z,
+		  double *a, double *b,
+		  const int *sample,
+		  int nrows, int k, int m)
+{
+    int i, j, v, t;
+
+    for (i=0; i<m; i++) {
+	t = sample[i];
+	for (j=0; j<k; j++) {
+	    v = pmod->list[j+2];
+	    a[i + j * nrows] = Z[v][t];
+	}
+	v = pmod->list[1];
+	b[i] = a[i + k * nrows] = Z[v][t];
+    }
+}
+
+#endif
+
 /* obtain bootstrap estimates of LAD covariance matrix */
 
 static int bootstrap_vcv (MODEL *pmod, double **Z,
@@ -526,40 +554,18 @@ static int bootstrap_vcv (MODEL *pmod, double **Z,
     return 0;
 }
 
-static int record_y_median (MODEL *pmod, const double *y)
+static void lad_loglik (MODEL *pmod)
 {
-    int T = pmod->t2 - pmod->t1 + 1;
-    double *sy, m;
-    int t, n, n2p;
+    double tau = 0.5, R = 0.0;
+    int n = pmod->nobs;
+    int t;
 
-    sy = malloc(T * sizeof *sy);
-
-    if (sy == NULL) {
-	return E_ALLOC;
-    }
-
-    n = 0;
     for (t=pmod->t1; t<=pmod->t2; t++) {
-	if (!model_missing(pmod, t)) {
-	    sy[n++] = y[t];
-	}
+	R += pmod->uhat[t] * (tau - (pmod->uhat[t] < 0));
     }
 
-    if (n == 0) {
-	free(sy);
-	return E_DATA;
-    }
-
-    qsort(sy, n, sizeof *sy, gretl_compare_doubles); 
-
-    n2p = (T = n / 2) + 1;
-    m = (n % 2)? sy[n2p - 1] : 0.5 * (sy[T - 1] + sy[n2p - 1]);
-
-    gretl_model_set_double(pmod, "ymedian", m);
-
-    free(sy);
-
-    return 0;
+    pmod->lnL = n * (log(tau * (1-tau)) - 1 - log(R/n));
+    mle_criteria(pmod, 0);
 }
 
 int lad_driver (MODEL *pmod, double **Z, DATAINFO *pdinfo)
@@ -626,11 +632,12 @@ int lad_driver (MODEL *pmod, double **Z, DATAINFO *pdinfo)
     ladcode = (int) a[m + 1 + n * nrows];
     if (ladcode == 2) {
 	pmod->errcode = E_SINGULAR;
-    } else {
-	gretl_model_set_int(pmod, "ladcode", ladcode);
+    } else if (ladcode == 0) {
+	gretl_model_set_int(pmod, "nonunique", 1);
     }
 
     if (pmod->errcode == 0) {
+	double SAR = a[m + n * nrows];
 
 	for (i=0; i<n; i++) {
 	    pmod->coeff[i] = x[i];
@@ -645,10 +652,10 @@ int lad_driver (MODEL *pmod, double **Z, DATAINFO *pdinfo)
 	}
 
 	/* sum of absolute residuals */
-	gretl_model_set_double(pmod, "ladsum", a[m + n * nrows]);
+	gretl_model_set_double(pmod, "ladsum", SAR);
 
 	/* median of dependent variable */
-	record_y_median(pmod, Z[yno]);
+	gretl_model_add_y_median(pmod, Z[yno]);
 
 	/* set ess-based stats to missing value */
 	pmod->rsq = NADBL;
@@ -657,12 +664,13 @@ int lad_driver (MODEL *pmod, double **Z, DATAINFO *pdinfo)
 
 	/* LaPlace errors: equivalent of standard error is sum of
 	   absolute residuals over nobs */
-	pmod->sigma = pmod->rho / pmod->nobs; 
+	pmod->sigma = SAR / pmod->nobs; 
+
+	lad_loglik(pmod);
 
 	if (bootstrap_vcv(pmod, Z, a, b, e, x, m, n, dim)) {
 	    pmod->errcode = E_ALLOC;
 	}
-
     }
 
     pmod->ci = LAD;
