@@ -1,20 +1,20 @@
-/* gretl - The Gnu Regression, Econometrics and Time-series Library
- * Copyright (C) 1999-2006 Allin Cottrell and Riccardo "Jack" Lucchetti
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License 
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License 
- * along with this software; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 /*
@@ -25,7 +25,7 @@
 
 #include "libgretl.h"
 #include "libset.h"    /* unused */
-#include "mod_garch.h"
+#include "garch.h"
 
 #define GDEBUG 0
 
@@ -43,7 +43,7 @@ enum {
 typedef struct garch_container_ garch_container;
 
 struct garch_container_ {
-    double *y;           /* dependent variable */
+    const double *y;     /* dependent variable */
     const double **X;    /* regressors (constant excluded) */
     int t1;              /* beginning of sample */
     int t2;              /* end of sample */
@@ -63,6 +63,8 @@ struct garch_container_ {
     double **blockglue;  /* derivatives of the loglik wrt residuals and variances */
     double **G;          /* score matrix */
     double *tot_score;   /* score vector (sum of G) */
+    double scale;        /* scale factor for dependent var */
+    int boundcheck;      /* enable bounds check */
 };
 
 #if 0
@@ -110,11 +112,11 @@ static int allocate_eh_derivs (garch_container *DH)
 }
 
 static garch_container *
-garch_container_new (double *y, const double **X, 
-		     int t1, int t2, int nobs, int nx,
+garch_container_new (const double *y, const double **X, 
+		     int t1, int t2, int nobs, int nc,
 		     int p, int q, int init_method, 
-		     double *res, double *h, 
-		     int analytical)
+		     double *e, double *e2, double *h, 
+		     double scale, int analytical)
 {
     garch_container *DH = malloc(sizeof *DH);
 
@@ -122,25 +124,22 @@ garch_container_new (double *y, const double **X,
 	return NULL;
     }
 
-    DH->e2 = malloc(nobs * sizeof *DH->e2);
-    if (DH->e2 == NULL) {
-	free(DH);
-	return NULL;
-    }
-
     DH->y = y;
     DH->X = X;
     DH->t1 = t1;
     DH->t2 = t2;
     DH->nobs = nobs;
-    DH->ncm = nx;
+    DH->ncm = nc - 1;
     DH->p = p;
     DH->q = q;
     DH->init = init_method;
-    DH->e = res;
+    DH->e = e;
+    DH->e2 = e2;
     DH->h = h;
     DH->ascore = analytical;
-    DH->k = 1 + nx + 1 + p + q;
+    DH->k = nc + 1 + p + q;
+    DH->boundcheck = 1;
+    DH->scale = scale;
 
     DH->score_e = NULL;
     DH->score_h = NULL;
@@ -149,7 +148,6 @@ garch_container_new (double *y, const double **X,
 
     if (DH->ascore) {
 	if (allocate_eh_derivs(DH)) {
-	    free(DH->e2);
 	    free(DH);
 	    DH = NULL;
 	}
@@ -163,7 +161,6 @@ static void garch_container_destroy (garch_container *DH)
     if (DH->ascore) {
 	free_eh_derivs(DH);
     }
-    free(DH->e2);
     free(DH);
 }
 
@@ -202,27 +199,50 @@ static void normal_score (const garch_container *DH)
 
 /* Compute the GARCH quantities */
 
+static int check_nonnegative (const double *par, int ncm, int k)
+{
+    int nonzero = (par[ncm+1] >= 0.0);
+    double sum = 0.0;
+    int i;
+
+    for (i=ncm+2; i<k; i++) {
+	nonzero &= (par[i] >= 1.0e-12);
+	sum += par[i];
+	if (!nonzero) {
+	    break;
+	}
+    }
+
+    nonzero &= (sum <= 1.0);
+
+    return nonzero;
+}
+
 static int garch_etht (const double *par, void *ptr)
 {
     garch_container *DH = (garch_container *) ptr;
-
+    double **dedq = DH->score_e;
+    double **dhdq = DH->score_h;
     int t1 = DH->t1;
     int t2 = DH->t2;
     int p = DH->p;
     int q = DH->q;
-
     int maxlag = (p > q)? p : q;
-
     int i, j, k, ret = 0;
     int ncm = DH->ncm;
-
-    double **dedq = DH->score_e;
-    double **dhdq = DH->score_h;
-
     int t, T = t2 - t1 + 1;
     double et, ht, tmp, h0 = 0.0;
     double u_var = 0.0;
 
+    /* check for nonnegative params */
+
+    if (DH->boundcheck) {
+	ret = !check_nonnegative(par, ncm, DH->k);
+	if (ret) {
+	    return E_DATA;
+	}
+    }
+
     /* compute residuals */
 
     tmp = 0.0;
@@ -232,8 +252,8 @@ static int garch_etht (const double *par, void *ptr)
 	} else {
 	    et = DH->y[t] - par[0];
 	    if (DH->X != NULL) {
-		for (i=0; i<ncm; i++) {
-		    et -= DH->X[i][t]*par[i+1];
+		for (i=1; i<=ncm; i++) {
+		    et -= DH->X[i][t]*par[i];
 		}
 	    }
 	    DH->e[t] = et;
@@ -298,7 +318,7 @@ static int garch_etht (const double *par, void *ptr)
 	    for (i=0; i<ncm; i++) {
 		dh0 = 0.0;
 		for (t=t1; t<=t2; t++) {
-		    dh0 -= DH->e[t] * DH->X[i][t];
+		    dh0 -= DH->e[t] * DH->X[i+1][t];
 		}
 		for (t=t1-maxlag; t<t1; t++) {
 		    dhdq[i+1][t] = dh0  * 2.0 / T;
@@ -339,12 +359,12 @@ static int garch_etht (const double *par, void *ptr)
     for (t=t1; t<=t2; t++) {
 	ht = par[ncm+1];
 
-	for (i=1; i<=p; i++) {
+	for (i=1; i<=q; i++) {
 	    ht += DH->e2[t-i] * par[ncm+i+1];
 	}
 
-	for (i=1; i<=q; i++) {
-	    ht += DH->h[t-i] * par[ncm+i+p+1];
+	for (i=1; i<=p; i++) {
+	    ht += DH->h[t-i] * par[ncm+i+q+1];
 	}
 	
 	DH->h[t] = ht;
@@ -355,8 +375,8 @@ static int garch_etht (const double *par, void *ptr)
 	    dedq[0][t] = -1.0;
 	    k = ncm+1;
 	    dhdq[0][t] = 0.0;
-	    for (i=1; i<=p; i++) {
-		if (t - p < t1 && DH->init == INIT_VAR_RESID) {
+	    for (i=1; i<=q; i++) {
+		if (t - q < t1 && DH->init == INIT_VAR_RESID) {
 		    dhdq[0][t] += par[k+i] * dhdq[0][t1-1];
 		} else {	
 		    dhdq[0][t] += 2.0 * par[k+i] * DH->e[t-i] * dedq[0][t-i];
@@ -365,11 +385,11 @@ static int garch_etht (const double *par, void *ptr)
 	    
 	    /* regressors */
 	    for (i=1; i<=ncm; i++) {
-		dedq[i][t] = -(DH->X[i-1][t]);
+		dedq[i][t] = -(DH->X[i][t]);
 		k = ncm+1;
 		dhdq[i][t] = 0.0;
-		for (j=1; j<=p; j++) {
-		    if (t - p < t1 && DH->init == INIT_VAR_RESID) { 
+		for (j=1; j<=q; j++) {
+		    if (t - q < t1 && DH->init == INIT_VAR_RESID) { 
 			// add INIT_THEO here
 			dhdq[i][t] += par[k+j] * dhdq[i][t1-1];
 		    } else {	
@@ -389,7 +409,7 @@ static int garch_etht (const double *par, void *ptr)
 	    
 	    /* garch params: alphas */
 	    k = ncm + 2;
-	    for (i=1; i<=p; i++) {
+	    for (i=1; i<=q; i++) {
 		dedq[k][t] = 0.0;
 		dhdq[k][t] = DH->e2[t-i];
 		if (t - p < t1 && DH->init == INIT_VAR_THEO) {
@@ -402,7 +422,7 @@ static int garch_etht (const double *par, void *ptr)
 	    
 	    /* garch params: betas */
 	    k = ncm + p + 2;
-	    for (i=1; i<=q; i++) {
+	    for (i=1; i<=p; i++) {
 		dedq[k][t] = 0.0;
 		dhdq[k][t] = DH->h[t-i];
 		if (t - p < t1 && DH->init == INIT_VAR_THEO) {
@@ -416,7 +436,7 @@ static int garch_etht (const double *par, void *ptr)
 	    /* "real" recursive part */
 	    for (i=0; i<DH->k; i++) {
 		k = ncm + p + 2;
-		for (j=1; j<=q; j++) {
+		for (j=1; j<=p; j++) {
 		    dhdq[i][t] += par[k++] * dhdq[i][t-j];
 		}
 	    }
@@ -508,19 +528,24 @@ static int anal_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll,
     return err;
 }
 
-static int garch_iinfo (garch_container *DH, gretl_matrix *info)
+static gretl_matrix *garch_iinfo (garch_container *DH, int *err)
 {
+    gretl_matrix *info;
     double **tmp_info;
     double tmpi, tmpj, tmpx1, tmpx2, x;
     int i, j, t;
 
+    info = gretl_matrix_alloc(DH->k, DH->k);
     if (info == NULL) {
-	return E_ALLOC;
+	*err = E_ALLOC;
+	return NULL;
     }
 
     tmp_info = doubles_array_new(DH->k, DH->k);
     if (tmp_info == NULL) {
-	return E_ALLOC;
+	*err = E_ALLOC;
+	gretl_matrix_free(info);
+	return NULL;
     }
 
     for (i=0; i<DH->k; i++) {
@@ -532,11 +557,11 @@ static int garch_iinfo (garch_container *DH, gretl_matrix *info)
     for (t=DH->t1; t<=DH->t2; t++) {
 	for (i=0; i<=DH->ncm; i++) {
 	    tmpi = DH->score_h[i][t] / DH->h[t];
-	    tmpx1 = (i==0)? 2.0 : 2.0*(DH->X[i-1][t]);
+	    tmpx1 = (i==0)? 2.0 : 2.0*(DH->X[i][t]);
 	    tmpx1 /= DH->h[t];
 	    for (j=0; j<=i; j++) {
 		tmpj = DH->score_h[j][t] / DH->h[t];
-		tmpx2 = (j==0)? 1.0 : 1.0*(DH->X[j-1][t]);
+		tmpx2 = (j==0)? 1.0 : 1.0*(DH->X[j][t]);
 		x = tmpx1 * tmpx2 + tmpi * tmpj;
 		tmp_info[i][j] += x;
 	    }
@@ -573,22 +598,27 @@ static int garch_iinfo (garch_container *DH, gretl_matrix *info)
     gretl_matrix_print(info, "Information matrix (inverse)");
 #endif
 
-    return 0;
+    return info;
 }
 
-static int garch_opg (garch_container *DH, gretl_matrix *GG)
+static gretl_matrix *garch_opg (garch_container *DH, int *err)
 {
+    gretl_matrix *GG;
     double **tmp_GG;
     double tmpi, x;
     int t, i, j;
 
+    GG = gretl_matrix_alloc(DH->k, DH->k);
     if (GG == NULL) {
-	return E_ALLOC;
+	*err = E_ALLOC;
+	return NULL;
     }
 
     tmp_GG = doubles_array_new(DH->k, DH->k);
     if (tmp_GG == NULL) {
-	return E_ALLOC;
+	*err = E_ALLOC;
+	gretl_matrix_free(GG);
+	return NULL;
     }
 
     for (i=0; i<DH->k; i++) {
@@ -622,79 +652,93 @@ static int garch_opg (garch_container *DH, gretl_matrix *GG)
     gretl_matrix_print(GG, "OPG matrix");
 #endif
 
-    return 0;
+    return GG;
 }
 
-static int garch_ihess (garch_container *DH, double *theta, gretl_matrix *invH)
+#define ANALYTICAL_HESSIAN 1
+
+#if ANALYTICAL_HESSIAN
+
+static gretl_matrix * 
+garch_ihess (garch_container *DH, double *theta, int *err)
 {
+    gretl_matrix *H;
+
+    H = garch_analytical_hessian(DH->y, DH->X, DH->t1, DH->t2, DH->nobs, 
+				 DH->ncm + 1, DH->p, DH->q, theta, DH->e, 
+				 DH->e2, DH->h, DH->scale, err);
+
+    return H;
+}
+
+#else
+
+static gretl_matrix *
+garch_ihess (garch_container *DH, double *theta, int *err)
+{
+    gretl_matrix *iH;
     double vij, *V;
     int i, j, k, npar = DH->k;
-    int err = 0;
+    int tmp = DH->boundcheck;
 
-    if (invH == NULL) {
-	return E_ALLOC;
+    iH = gretl_matrix_alloc(npar, npar);
+    if (iH == NULL) {
+	*err = E_ALLOC;
+	return NULL;
     }
 
-    V = numerical_hessian(theta, npar, loglik, DH, &err);
-    if (V == NULL) {
-	return err;
+    DH->boundcheck = 0;
+    V = numerical_hessian(theta, npar, loglik, DH, err);
+    DH->boundcheck = tmp;
+
+    if (*err) {
+	return NULL;
     }
 
     k = 0;
     for (i=0; i<npar; i++) {
 	for (j=i; j<npar; j++) {
 	    vij = V[k++];
-	    gretl_matrix_set(invH, i, j, vij);
+	    gretl_matrix_set(iH, i, j, vij);
 	    if (i != j) {
-		gretl_matrix_set(invH, j, i, vij);
+		gretl_matrix_set(iH, j, i, vij);
 	    }
 	}
     }
 
-#if GDEBUG
-    gretl_matrix_print(invH, "Hessian (inverse)");
-#endif
+# if GDEBUG
+    gretl_matrix_print(iH, "Hessian (inverse)");
+# endif
 
     free(V);
 
-    return 0;
+    return iH;
 }
 
+#endif
+
 static int
 garch_covariance_matrix (int vopt, double *theta, garch_container *DH,
-			 double *vcv) 
+			 gretl_matrix *V) 
 {
-    gretl_matrix *V = NULL;
     gretl_matrix *GG = NULL;
     gretl_matrix *iinfo = NULL;
     gretl_matrix *invhess = NULL;
-    int npar = DH->k;
     int err = 0;
 
-    if (vopt == VCV_BW || vopt == VCV_QML) {
-	/* need a distinct matrix for workspace */
-	V = gretl_matrix_alloc(npar, npar);
-	if (V == NULL) {
-	    return E_ALLOC;
-	}
-    }
-
     if (vopt == VCV_OP || vopt == VCV_QML || vopt == VCV_BW) { 
 	/* GG' needed */
-	GG = gretl_matrix_alloc(npar, npar);
-	err = garch_opg(DH, GG);
+	GG = garch_opg(DH, &err);
     }
 
     if (vopt == VCV_IM || vopt == VCV_BW) { 
 	/* information matrix needed */
-	iinfo = gretl_matrix_alloc(npar, npar);
-	err = garch_iinfo(DH, iinfo);
+	iinfo = garch_iinfo(DH, &err);
     }
 
     if (vopt == VCV_QML || vopt == VCV_HESSIAN) { 
 	/* Hessian matrix needed */
-	invhess = gretl_matrix_alloc(npar, npar);
-	err = garch_ihess(DH, theta, invhess);
+	invhess = garch_ihess(DH, theta, &err);
     }
 
     if (err) {
@@ -703,16 +747,13 @@ garch_covariance_matrix (int vopt, double *theta, garch_container *DH,
 
     switch (vopt) {
     case VCV_HESSIAN:
-	V = invhess;
-	invhess = NULL;
+	gretl_matrix_copy_values(V, invhess);
 	break;
     case VCV_IM:
-	V = iinfo;
-	iinfo = NULL;
+	gretl_matrix_copy_values(V, iinfo);
 	break;
     case VCV_OP:
-	V = GG;
-	GG = NULL;
+	gretl_matrix_copy_values(V, GG);
 	err = gretl_invert_symmetric_matrix(V);
 	break;
     case VCV_BW:
@@ -731,27 +772,11 @@ garch_covariance_matrix (int vopt, double *theta, garch_container *DH,
     gretl_matrix_print(V, "Variance-covariance matrix");
 #endif
 
-    if (!err) {
-	double vij;
-	int i, j;
-
-	for (i=0; i<npar; i++) {
-	    for (j=i; j<npar; j++) {
-		vij = gretl_matrix_get(V, i, j);
-		vcv[(i*npar) + j] = vij;
-		if (i != j) {
-		    vcv[(j*npar) + i] = vij;
-		}
-	    }
-	}
-    }
-
  bailout:
 
     gretl_matrix_free(GG);
     gretl_matrix_free(iinfo);
     gretl_matrix_free(invhess);
-    gretl_matrix_free(V);
 
     return err;
 }
@@ -786,74 +811,56 @@ static void test_score (garch_container *DH, double *theta)
 /*
    Parameters to garch_estimate_mod()
 
-   t1:    beginning of sample in auxiliary database
-   t2:    end of sample in auxiliary database
-   nobs:  total number of observations in auxiliary database
-   X:     data matrix for auxiliary database (regressors, not needed on
-          output)
-   nx:    number of columns of X
-   coeff: vector of coefficient for the conditional mean, normally
-          initialised by OLS on input (not needed on output) -- does NOT
-	  include the constant
-   nc:    number of elements in coeff
-   vcv:   n^2 vector (0 on input) to store covariance matrix of coeff
-   res2:  vector of 0's on input, squared resids on output (not needed)
-   e:     vector of 0's on input, resids on output
-   h:     null pointer on input, conditional variances on output
-   y:     on input, vector with dep. var., not needed on output
-   amax:  vector; element 0 holds the garch intercept; 1 and 2 the
-          arch & garch orders; from 3 onwards, the arch & garch 
-          parameters
-   b:     0 on input, holds vector of coefficient for the conditional
-          mean on output
-   scale: double used to scale dep. var.
-   fncount: 0 on input, holds number of function evaluations on output
-   fncount: 0 on input, holds number of gradient evaluations on output
-   prn:   print handle for info on iterations and other diagnostic output
+   y:     the dependent variable
+   X:     the independent regressors (including the constant)
+   t1:    beginning of sample relative to the arrays y, X
+   t2:    end of sample
+   nobs:  total number of observations in y, X
+   nc:    number of columns in X
+   p:     number of 'beta' variance params
+   q:     number of 'alpha' variance params (excluding the constant)
+   theta: full parameter vector, pre-initialized; on output, the
+          estimates at convergence.
+   V:     covariance matrix of parameters (all 0 on input)
+   e:     vector of 0's on input, residuals on output
+   e2:    storage for squared residuals on output
+   h:     storage for conditional variances on output
+   scale: factor used to scale the dependent variable
+   pll:   location to receive log-likelihood on output
+   fncount: 0 on input, number of function evaluations on output
+   fncount: 0 on input, number of gradient evaluations on output
+   vopt:  code indicating which version of the covariance
+          matrix to compute in V
+   prn:   print handle for info on iterations etc.
 
 */
 
-int garch_estimate_mod (int t1, int t2, int nobs, 
-			const double **X, int nx, double *coeff, int nc, 
-			double *vcv, double *res2, double *res, double *h,
-			double *y, double *amax, double *b, 
-			double scale, int *fncount, int *grcount,
-			PRN *prn, int vopt)
+int garch_estimate_mod (const double *y, const double **X,
+			int t1, int t2, int nobs, int nc,
+			int p, int q, double *theta,  gretl_matrix *V,
+			double *e, double *e2, double *h,
+			double scale, double *pll, 
+			int *fncount, int *grcount,
+			int vopt, PRN *prn)
 {
     garch_container *DH;
-    int p = amax[1];
-    int q = amax[2];
-    int npar = 1 + nx + 1 + p + q;
+    int npar = nc + 1 + p + q; 
     int analytical = 1;
-    double *theta = NULL;
-    int i, j, err = 0;
+    int err = 0;
 
     /* BFGS apparatus */
     int maxit = 10000;
     double reltol = 1.0e-13;
 
-    DH = garch_container_new(y, X, t1, t2, nobs, nx, p, q, 
-			     INIT_VAR_RESID, res, h, 
-			     analytical);
+    DH = garch_container_new(y, X, t1, t2, nobs, nc, p, q, 
+			     INIT_VAR_RESID, e, e2, h, 
+			     scale, analytical);
 
     if (DH == NULL) {
 	return E_ALLOC;
     }
 
-    theta = malloc(npar * sizeof *theta);
-    if (theta == NULL) {
-	garch_container_destroy(DH);
-	return E_ALLOC;
-    }
-
     theta[0] = gretl_mean(t1, t2, y);
-    for (i=1; i<=nx; i++) {
-	theta[i] = coeff[i];
-    }
-    theta[nx+1] = amax[0];
-    for (i=nx+2, j=3; i<npar; i++) {
-	theta[i] = amax[j++];
-    }
 
 #if 0
     if (DH->ascore) {
@@ -869,32 +876,14 @@ int garch_estimate_mod (int t1, int t2, int nobs,
 		   DH, (prn != NULL)? OPT_V : OPT_NONE, 
 		   prn);
     
-    amax[0] = loglik(theta, DH) - (t2 - t1 + 1) * log(scale);
+    *pll = loglik(theta, DH) - (t2 - t1 + 1) * log(scale);
     
 #if GDEBUG
     test_score(DH, theta);
 #endif
 
-    err = garch_covariance_matrix(vopt, theta, DH, vcv);
+    err = garch_covariance_matrix(vopt, theta, DH, V);
 
-    if (!err) {
-	double sderr;
-
-	/* transcribe coefficients and standard errors 
-	   note: slightly different from fcp */
-	for (i=0, j=0; i<npar; i++) {
-	    if (vcv[j] > 0.0) {
-		sderr = sqrt(vcv[j]);
-	    } else {
-		sderr = 0.0;
-	    }
-	    j += npar + 1;
-	    amax[i+1] = theta[i];
-	    amax[i+1+npar] = sderr;
-	}
-    }
-
-    free(theta);
     garch_container_destroy(DH);
 
     return err;
