@@ -108,9 +108,9 @@ make_sys_X_block (gretl_matrix *X, const MODEL *pmod,
 static int
 gls_sigma_from_uhat (equation_system *sys, gretl_matrix *sigma)
 {
-    const gretl_matrix *e = sys->uhat;
-    int m = sys->n_equations;
-    int T = sys->n_obs;
+    const gretl_matrix *e = sys->E;
+    int m = sys->neqns;
+    int T = sys->T;
     int geomean = system_vcv_geomean(sys);
     int i, j, t;
     double xx;
@@ -169,7 +169,7 @@ sys_resids (equation_system *sys, int eq, const double **Z)
 	pmod->yhat[t] = yh;
 	pmod->uhat[t] = Z[pmod->list[1]][t] - yh;
 	/* for cross-equation vcv */
-	gretl_matrix_set(sys->uhat, t - pmod->t1, pmod->ID, pmod->uhat[t]);
+	gretl_matrix_set(sys->E, t - pmod->t1, pmod->ID, pmod->uhat[t]);
 	pmod->ess += pmod->uhat[t] * pmod->uhat[t];
     }
 
@@ -189,7 +189,7 @@ liml_scale_vcv (equation_system *sys, gretl_matrix *vcv)
     int vi, vj;
     int i, j, k;
 
-    for (i=0; i<sys->n_equations; i++) {
+    for (i=0; i<sys->neqns; i++) {
 	s2 = sys->models[i]->sigma * sys->models[i]->sigma;
 	for (j=0; j<sys->models[i]->ncoeff; j++) {
 	    for (k=j; k<sys->models[i]->ncoeff; k++) {
@@ -223,7 +223,7 @@ single_eq_common_sigma_scale_vcv (equation_system *sys, gretl_matrix *vcv)
 
     /* is this right? */
 
-    for (i=0; i<sys->n_equations; i++) {
+    for (i=0; i<sys->neqns; i++) {
 	ess += sys->models[i]->ess;
 	den += sys->models[i]->nobs;
 	if (dfc) {
@@ -250,7 +250,7 @@ single_eq_scale_vcv (equation_system *sys, gretl_matrix *vcv)
     int ioff = 0, joff = 0;
     int i, k, ii, jj;
 
-    for (i=0; i<sys->n_equations; i++) {
+    for (i=0; i<sys->neqns; i++) {
 	pmod = sys->models[i];
 	s2 = pmod->sigma * pmod->sigma;
 	k = pmod->ncoeff;
@@ -315,7 +315,7 @@ calculate_sys_coeffs (equation_system *sys,
 #endif
 
     j0 = 0;
-    for (i=0; i<sys->n_equations; i++) {
+    for (i=0; i<sys->neqns; i++) {
 	for (j=0; j<sys->models[i]->ncoeff; j++) {
 	    k = j0 + j;
 	    bij = gretl_vector_get(y, k);
@@ -352,7 +352,7 @@ calculate_sys_coeffs (equation_system *sys,
 
     /* now set the model standard errors */
     j0 = 0;
-    for (i=0; i<sys->n_equations; i++) {
+    for (i=0; i<sys->neqns; i++) {
 	for (j=0; j<sys->models[i]->ncoeff; j++) {
 	    k = j0 + j;
 	    sys->models[i]->sderr[j] = sqrt(gretl_matrix_get(vcv, k, k));
@@ -426,8 +426,8 @@ static int hansen_sargan_test (equation_system *sys,
 {
     const int *exlist = system_get_instr_vars(sys);
     int nx = exlist[0];
-    int m = sys->n_equations;
-    int T = sys->n_obs;
+    int m = sys->neqns;
+    int T = sys->T;
     int df = system_get_overid_df(sys);
 
     const double *Wi, *Wj;
@@ -457,7 +457,7 @@ static int hansen_sargan_test (equation_system *sys,
 	for (j=i; j<nx; j++) {
 	    Wj = Z[exlist[j+1]] + sys->t1;
 	    x = 0.0;
-	    for (t=0; t<sys->n_obs; t++) {
+	    for (t=0; t<sys->T; t++) {
 		x += Wi[t] * Wj[t];
 	    }
 	    gretl_matrix_set(WTW, i, j, x);
@@ -484,7 +484,7 @@ static int hansen_sargan_test (equation_system *sys,
 	    Wj = Z[exlist[j+1]] + sys->t1;
 	    x = 0.0;
 	    for (t=0; t<T; t++) {
-		x += gretl_matrix_get(sys->uhat, t, i) * Wj[t];
+		x += gretl_matrix_get(sys->E, t, i) * Wj[t];
 	    }
 	    gretl_matrix_set(eW, i, j, x);
 	}
@@ -511,13 +511,13 @@ static int hansen_sargan_test (equation_system *sys,
 		x += gretl_matrix_get(tmp, i, t) * 
 		    gretl_matrix_get(eW, j, t); /* transposed */
 	    }
-	    X2 += gretl_matrix_get(sys->sigma, i, j) * x;
+	    X2 += gretl_matrix_get(sys->S, i, j) * x;
 	}
     }
 
 #if SDEBUG
     fprintf(stderr, "Hansen-Sargan: Chi-square(%d) = %g (p-value %g)\n", 
-	    df, X2, chisq_cdf_comp(X2, df));
+	    df, X2, chisq_cdf_comp(df, X2));
 #endif
     sys->X2 = X2;
 
@@ -535,20 +535,20 @@ static int basic_system_allocate (equation_system *sys,
 				  gretl_matrix **X,
 				  gretl_matrix **y)
 {
-    int m = sys->n_equations;
-    int T = sys->n_obs;
+    int m = sys->neqns;
+    int T = sys->T;
     int ldx = mk + nr;
 
     /* allocate a model for each stochastic equation */
     sys->models = gretl_model_array_new(m);
 
-    sys->uhat = gretl_matrix_alloc(T, m);
-    if (sys->uhat == NULL) {
+    sys->E = gretl_matrix_alloc(T, m);
+    if (sys->E == NULL) {
 	return E_ALLOC;
     }
 
-    sys->sigma = gretl_matrix_alloc(m, m);
-    if (sys->sigma == NULL) {
+    sys->S = gretl_matrix_alloc(m, m);
+    if (sys->S == NULL) {
 	return E_ALLOC;
     }
 
@@ -569,8 +569,8 @@ static int basic_system_allocate (equation_system *sys,
 
 double sur_ll (equation_system *sys)
 {
-    int m = sys->n_equations;
-    int T = sys->n_obs;
+    int m = sys->neqns;
+    int T = sys->T;
     gretl_matrix *sigtmp;
     double ldet;
 
@@ -657,9 +657,9 @@ augment_y_with_restrictions (gretl_matrix *y, int mk, int nr,
    three-stage least squares
 */
 
-static int converged (equation_system *sys, 
-		      double *llbak, int *err,
-		      PRN *prn)
+static int sys_converged (equation_system *sys, 
+			  double *llbak, int *err,
+			  gretlopt opt, PRN *prn)
 {
     double crit, tol = 0.0;
     int met = 0;
@@ -671,9 +671,9 @@ static int converged (equation_system *sys,
 	tol = SYS_LL_TOL;
 	crit = ll - *llbak;
 
-#if SDEBUG
-	printf("SUR iteration %d, ll = %.8g\n", sys->iters, ll);
-#endif
+	if (opt & OPT_V) {
+	    pprintf(prn, "Iteration %3d, ll = %#.8g\n", sys->iters, ll);
+	}
 	if (crit <= tol) {
 	    met = 1;
 	} else if (sys->iters < SYS_MAX_ITER) {
@@ -682,17 +682,20 @@ static int converged (equation_system *sys,
     } else if (sys->method == SYS_METHOD_3SLS) {
 	tol = SYS_BDIFF_TOL;
 	crit = sys->bdiff;
-
-#if SDEBUG
-	printf("3SLS iteration %d, crit = %.8g\n", sys->iters, crit);
-#endif
+	if (opt & OPT_V) {
+	    pprintf(prn, "Iteration %3d, criterion = %g\n", sys->iters, crit);
+	}
 	if (crit <= tol) {
 	    met = 1;
 	} 
     }
 
+    if (met && tol > 0 && (opt & OPT_V)) {
+	pprintf(prn, "Tolerance of %g is met\n", tol);
+    }
+
     if (!met && sys->iters >= SYS_MAX_ITER) {
-	pprintf(prn, "reached %d iterations without meeting "
+	pprintf(prn, "Reached %d iterations without meeting "
 		"tolerance of %g\n", sys->iters, tol);
 	*err = E_NOCONV;
     }	
@@ -705,7 +708,7 @@ static void clean_up_models (equation_system *sys)
     double ess = 0.0;
     int i;
 
-    for (i=0; i<sys->n_equations; i++) {
+    for (i=0; i<sys->neqns; i++) {
 	ess += sys->models[i]->ess;
 	if (sys->method == SYS_METHOD_3SLS || 
 	    sys->method == SYS_METHOD_FIML || 
@@ -728,9 +731,9 @@ static int drop_redundant_instruments (equation_system *sys,
     int j, k, pos, err = 0;
 
     for (j=1; j<=droplist[0]; j++) {
-	pos = gretl_list_position(droplist[j], sys->instr_vars);
+	pos = gretl_list_position(droplist[j], sys->ilist);
 	if (pos > 0) {
-	    gretl_list_delete_at_pos(sys->instr_vars, pos);
+	    gretl_list_delete_at_pos(sys->ilist, pos);
 	} else {
 	    err = 1;
 	}
@@ -803,7 +806,7 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     } 
 
     /* number of equations */
-    m = sys->n_equations;
+    m = sys->neqns;
 
     /* max indep vars per equation */
     k = system_max_indep_vars(sys);
@@ -812,7 +815,7 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     mk = system_n_indep_vars(sys);
 
     /* number of observations per series */
-    T = sys->n_obs;
+    T = sys->T;
 
     /* allocate models etc */
     err = basic_system_allocate(sys, mk, nr, &X, &y);
@@ -883,13 +886,13 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 	}
 
 	for (t=0; t<T; t++) {
-	    gretl_matrix_set(sys->uhat, t, i, pmod->uhat[t + sys->t1]);
+	    gretl_matrix_set(sys->E, t, i, pmod->uhat[t + sys->t1]);
 	}
     }
 
     if (err) {
 	fprintf(stderr, "system_estimate: after single-equation "
-		"estimation, err = %d", err);
+		"estimation, err = %d\n", err);
 	goto cleanup;
     }
 
@@ -906,19 +909,19 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     */
  iteration_start:
 
-    gls_sigma_from_uhat(sys, sys->sigma);
+    gls_sigma_from_uhat(sys, sys->S);
 
 #if SDEBUG > 1
-    gretl_matrix_print(sys->sigma, "gls_sigma_from_uhat");
+    gretl_matrix_print(sys->S, "gls_sigma_from_uhat");
 #endif
 
     if (method == SYS_METHOD_WLS) {
 	gretl_matrix_zero(X);
-	err = gretl_invert_diagonal_matrix(sys->sigma);
+	err = gretl_invert_diagonal_matrix(sys->S);
     } else if (single_equation || rtsls) {
 	gretl_matrix_zero(X);
     } else {
-	err = gretl_invert_symmetric_matrix(sys->sigma);    
+	err = gretl_invert_symmetric_matrix(sys->S);    
     }
 
 #if SDEBUG
@@ -983,7 +986,7 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 	    if (rtsls || (single_equation && method != SYS_METHOD_WLS)) {
 		sij = 1.0;
 	    } else {
-		sij = gretl_matrix_get(sys->sigma, i, j);
+		sij = gretl_matrix_get(sys->S, i, j);
 	    }
 
 	    kronecker_place(X, M, krow, kcol, sij);
@@ -1051,7 +1054,7 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 		if (rtsls || (single_equation && method != SYS_METHOD_WLS)) {
 		    sil = 1.0;
 		} else {
-		    sil = gretl_matrix_get(sys->sigma, i, l);
+		    sil = gretl_matrix_get(sys->S, i, l);
 		}
 
 		yv += xx * sil;
@@ -1084,7 +1087,7 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (do_iteration) {
-	if (!converged(sys, &llbak, &err, prn)) {
+	if (!sys_converged(sys, &llbak, &err, opt, prn)) {
 	    if (err) {
 		goto cleanup;
 	    } else {
@@ -1100,7 +1103,7 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* refresh sigma (non-inverted) */
-    gls_sigma_from_uhat(sys, sys->sigma);
+    gls_sigma_from_uhat(sys, sys->S);
 
     if (method == SYS_METHOD_FIML) {
 	/* compute FIML estimates */
