@@ -19,7 +19,7 @@
 
 #include "libgretl.h"
 
-#include "f2c.h"
+#include "gretl_f2c.h"
 #include "clapack_double.h"
 
 static double packed_matrix_norm (const double *x, int k)
@@ -234,8 +234,8 @@ model_vif_vector (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     return vif;
 }
 
-#define xtx_ok(c) (c == OLS || c == CORC || c == HILU || \
-		   c == WLS || c == PWE || c == HCCM || \
+#define xtx_ok(c) (c == OLS || c == AR1 || \
+		   c == WLS || c == HCCM || \
 		   c == HSK) 
 
 int print_vifs (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
