@@ -30,11 +30,12 @@ print "; -- gretl.iss --\n";
 print "\n[Setup]\n";
 print "AppName=gretl\n";
 print "AppVerName=gretl version $verstr\n";
+print "AppVersion=$verstr\n";
 print "AppPublisherURL=http://gretl.sourceforge.net/\n";
 print "AppSupportURL=http://gretl.sourceforge.net/\n";
-print "AppCopyright=Copyright C 1999-2007 Allin Cottrell and Riccardo Lucchetti\n";
 print "DefaultDirName={pf}\\gretl\n";
 print "DefaultGroupName=gretl\n";
+print "PrivilegesRequired=poweruser\n";
 print "UninstallDisplayIcon={app}\\gretlw32.exe\n";
 print "ChangesAssociations=yes\n";
 print "DirExistsWarning=no\n";
@@ -42,9 +43,6 @@ print "DirExistsWarning=no\n";
 print "\n[InstallDelete]\n";
 print "Type: files; Name: \"{app}\\*.dll\"\n";
 
-print "\n[Dirs]\n";
-print "Name: {code:GetDataDir}; Flags: uninsneveruninstall\n";
-
 print "\n[Files]\n";
 
 while ($line = <STDIN>) {
@@ -57,13 +55,9 @@ while ($line = <STDIN>) {
     chomp($line);
     @pathbits = split(/\\/, $line);
     print "Source: \"$line\"; "; 
-    if ($line =~ /README/) {
-        print "Destdir: \"{code:GetDataDir}";
-    } else {
-	print "Destdir: \"{app}";
-	for ($i = 1; $i < @pathbits - 1; $i++) {
-	    print "\\$pathbits[$i]";
-	}
+    print "Destdir: \"{app}";
+    for ($i = 1; $i < @pathbits - 1; $i++) {
+        print "\\$pathbits[$i]";
     }
     print "\"\n";
 }
@@ -88,8 +82,8 @@ print "Root: HKLM; Subkey: \"Software\\gretl\"; Flags: uninsdeletekey\n";
 # specific entries
 print "Root: HKLM; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
 print "\"gretldir\"; ValueData: \"{app}\"\n";
-print "Root: HKCU; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
-print "\"userdir\"; ValueData: \"{code:GetDataDir}\"\n";
+# print "Root: HKCU; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
+# print "\"userdir\"; ValueData: \"{code:GetDataDir}\"\n";
 print "Root: HKLM; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
 print "\"gnuplot\"; ValueData: \"{app}\\wgnuplot.exe\"\n";
 print "Root: HKLM; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
@@ -123,9 +117,9 @@ print "\"Png_font\"; ValueData: \"verdana 8\"\n";
 print "Root: HKCU; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
 print "\"Gp_colors\"; ValueData: \"\"\n";
 print "Root: HKCU; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
-print "\"DataPage\"; ValueData: \"Ramanathan\"\n";
+print "\"DataPage\"; ValueData: \"Gretl\"\n";
 print "Root: HKCU; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
-print "\"ScriptPage\"; ValueData: \"Ramanathan\"\n";
+print "\"ScriptPage\"; ValueData: \"Gretl\"\n";
 print "Root: HKCU; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
 print "\"manpref\"; ValueData: \"0\"\n";
 print "Root: HKCU; Subkey: \"Software\\gretl\"; ValueType: string; ValueName: ";
@@ -172,44 +166,4 @@ print "Root: HKCR; Subkey: \"GretlScriptFile\\shell\\open\\command\"; ";
 print "ValueType: string; ValueName: \"\"; ValueData: ";
 print "\"\"\"{app}\\gretlw32.exe\"\" -r \"\"%1\"\"\"\n";
 
-print "\n[Code]\n";
-print "var\n";
-print "  DataDirPage: TInputDirWizardPage;\n";
-print "\n";
-print "procedure InitializeWizard;\n";
-print "begin\n";
-print "  { Create the pages }\n";
-print "   DataDirPage := CreateInputDirPage(wpSelectDir,\n";
-print "                  'Select Personal Data Directory', \n";
-print "                  'Where should personal data files be kept?',\n";
-print "                  'Select the folder for storing personal data files, then click Next.',\n";
-print "                   False, '');\n";
-print "  DataDirPage.Add('');\n";
-print "  DataDirPage.Values[0] := GetPreviousData('DataDir', '');\n";
-print "  if DataDirPage.Values[0] = '' then\n";
-print "     DataDirPage.Values[0] := ExpandConstant('{userappdata}\\gretl');\n";
-print "end;\n";
-print "\n";
-print "procedure RegisterPreviousData(PreviousDataKey: Integer);\n";
-print "begin\n";
-print "  { Store the settings so we can restore them next time }\n";
-print "  SetPreviousData(PreviousDataKey, 'DataDir', DataDirPage.Values[0]);\n";
-print "end;\n";
-print "\n";
-print "function UpdateReadyMemo(Space, NewLine, MemoUserInfoInfo, MemoDirInfo, MemoTypeInfo,\n";
-print "  MemoComponentsInfo, MemoGroupInfo, MemoTasksInfo: String): String;\n";
-print "var\n";
-print "  S: String;\n";
-print "begin\n";
-print "  { Fill the 'Ready Memo' with the normal settings and the custom settings }\n";
-print "  S := '';\n";
-print "  S := S + MemoDirInfo + NewLine;\n";
-print "  S := S + Space + DataDirPage.Values[0] + ' (personal data files)' + NewLine;\n";
-print "  Result := S;\n";
-print "end;\n\n";
-print "function GetDataDir(Param: String): String;\n";
-print "begin\n";
-print "  { Return the selected DataDir }\n";
-print "  Result := DataDirPage.Values[0];\n";
-print "end;\n";
 
